// This file is generated by TypeBuilder_cpp.template.

// Copyright 2016 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include "content/browser/devtools/protocol/bluetooth_emulation.h"

#include "content/browser/devtools/protocol/protocol.h"

#include "third_party/inspector_protocol/crdtp/cbor.h"
#include "third_party/inspector_protocol/crdtp/find_by_first.h"
#include "third_party/inspector_protocol/crdtp/span.h"

namespace content {
namespace protocol {
namespace BluetoothEmulation {

using crdtp::DeserializerState;
using crdtp::ProtocolTypeTraits;

// ------------- Enum values from types.

const char Metainfo::domainName[] = "BluetoothEmulation";
const char Metainfo::commandPrefix[] = "BluetoothEmulation.";
const char Metainfo::version[] = "1.3";

namespace CentralStateEnum {
const char Absent[] = "absent";
const char PoweredOff[] = "powered-off";
const char PoweredOn[] = "powered-on";
} // namespace CentralStateEnum


CRDTP_BEGIN_DESERIALIZER(ManufacturerData)
    CRDTP_DESERIALIZE_FIELD("data", m_data),
    CRDTP_DESERIALIZE_FIELD("key", m_key),
CRDTP_END_DESERIALIZER()

CRDTP_BEGIN_SERIALIZER(ManufacturerData)
    CRDTP_SERIALIZE_FIELD("key", m_key);
    CRDTP_SERIALIZE_FIELD("data", m_data);
CRDTP_END_SERIALIZER();


CRDTP_BEGIN_DESERIALIZER(ScanRecord)
    CRDTP_DESERIALIZE_FIELD_OPT("appearance", m_appearance),
    CRDTP_DESERIALIZE_FIELD_OPT("manufacturerData", m_manufacturerData),
    CRDTP_DESERIALIZE_FIELD_OPT("name", m_name),
    CRDTP_DESERIALIZE_FIELD_OPT("txPower", m_txPower),
    CRDTP_DESERIALIZE_FIELD_OPT("uuids", m_uuids),
CRDTP_END_DESERIALIZER()

CRDTP_BEGIN_SERIALIZER(ScanRecord)
    CRDTP_SERIALIZE_FIELD("name", m_name);
    CRDTP_SERIALIZE_FIELD("uuids", m_uuids);
    CRDTP_SERIALIZE_FIELD("appearance", m_appearance);
    CRDTP_SERIALIZE_FIELD("txPower", m_txPower);
    CRDTP_SERIALIZE_FIELD("manufacturerData", m_manufacturerData);
CRDTP_END_SERIALIZER();


CRDTP_BEGIN_DESERIALIZER(ScanEntry)
    CRDTP_DESERIALIZE_FIELD("deviceAddress", m_deviceAddress),
    CRDTP_DESERIALIZE_FIELD("rssi", m_rssi),
    CRDTP_DESERIALIZE_FIELD("scanRecord", m_scanRecord),
CRDTP_END_DESERIALIZER()

CRDTP_BEGIN_SERIALIZER(ScanEntry)
    CRDTP_SERIALIZE_FIELD("deviceAddress", m_deviceAddress);
    CRDTP_SERIALIZE_FIELD("rssi", m_rssi);
    CRDTP_SERIALIZE_FIELD("scanRecord", m_scanRecord);
CRDTP_END_SERIALIZER();


// ------------- Enum values from params.


// ------------- Frontend notifications.

void Frontend::flush()
{
    frontend_channel_->FlushProtocolNotifications();
}

void Frontend::sendRawNotification(std::unique_ptr<Serializable> notification)
{
    frontend_channel_->SendProtocolNotification(std::move(notification));
}

// --------------------- Dispatcher.

class DomainDispatcherImpl : public protocol::DomainDispatcher {
public:
    DomainDispatcherImpl(FrontendChannel* frontendChannel, Backend* backend)
        : DomainDispatcher(frontendChannel)
        , m_backend(backend) {}
    ~DomainDispatcherImpl() override { }

    using CallHandler = void (DomainDispatcherImpl::*)(const crdtp::Dispatchable& dispatchable);

    std::function<void(const crdtp::Dispatchable&)> Dispatch(crdtp::span<uint8_t> command_name) override;

    void enable(const crdtp::Dispatchable& dispatchable);
    void disable(const crdtp::Dispatchable& dispatchable);
    void simulatePreconnectedPeripheral(const crdtp::Dispatchable& dispatchable);
    void simulateAdvertisement(const crdtp::Dispatchable& dispatchable);
 protected:
    Backend* m_backend;
};

namespace {
// This helper method with a static map of command methods (instance methods
// of DomainDispatcherImpl declared just above) by their name is used immediately below,
// in the DomainDispatcherImpl::Dispatch method.
DomainDispatcherImpl::CallHandler CommandByName(crdtp::span<uint8_t> command_name) {
  static auto* commands = [](){
    auto* commands = new std::vector<std::pair<crdtp::span<uint8_t>,
                              DomainDispatcherImpl::CallHandler>>{
    {
          crdtp::SpanFrom("disable"),
          &DomainDispatcherImpl::disable
    },
    {
          crdtp::SpanFrom("enable"),
          &DomainDispatcherImpl::enable
    },
    {
          crdtp::SpanFrom("simulateAdvertisement"),
          &DomainDispatcherImpl::simulateAdvertisement
    },
    {
          crdtp::SpanFrom("simulatePreconnectedPeripheral"),
          &DomainDispatcherImpl::simulatePreconnectedPeripheral
    },
    };
    return commands;
  }();
  return crdtp::FindByFirst<DomainDispatcherImpl::CallHandler>(*commands, command_name, nullptr);
}
}  // namespace

std::function<void(const crdtp::Dispatchable&)> DomainDispatcherImpl::Dispatch(crdtp::span<uint8_t> command_name) {
  CallHandler handler = CommandByName(command_name);
  if (!handler) return nullptr;

  return [this, handler](const crdtp::Dispatchable& dispatchable) {
    (this->*handler)(dispatchable);
  };
}


namespace {

struct enableParams : public crdtp::DeserializableProtocolObject<enableParams> {
    String state;
    DECLARE_DESERIALIZATION_SUPPORT();
};

CRDTP_BEGIN_DESERIALIZER(enableParams)
    CRDTP_DESERIALIZE_FIELD("state", state),
CRDTP_END_DESERIALIZER()

}  // namespace

void DomainDispatcherImpl::enable(const crdtp::Dispatchable& dispatchable)
{
    // Prepare input parameters.
    auto deserializer = crdtp::DeferredMessage::FromSpan(dispatchable.Params())->MakeDeserializer();
    enableParams params;
    if (!enableParams::Deserialize(&deserializer, &params)) {
      ReportInvalidParams(dispatchable, deserializer);
      return;
    }

    std::unique_ptr<DomainDispatcher::WeakPtr> weak = weakPtr();
    DispatchResponse response = m_backend->Enable(params.state);
    if (response.IsFallThrough()) {
        channel()->FallThrough(dispatchable.CallId(), crdtp::SpanFrom("BluetoothEmulation.enable"), dispatchable.Serialized());
        return;
    }
    if (weak->get())
        weak->get()->sendResponse(dispatchable.CallId(), response);
    return;
}

namespace {


}  // namespace

void DomainDispatcherImpl::disable(const crdtp::Dispatchable& dispatchable)
{
    // Prepare input parameters.

    std::unique_ptr<DomainDispatcher::WeakPtr> weak = weakPtr();
    DispatchResponse response = m_backend->Disable();
    if (response.IsFallThrough()) {
        channel()->FallThrough(dispatchable.CallId(), crdtp::SpanFrom("BluetoothEmulation.disable"), dispatchable.Serialized());
        return;
    }
    if (weak->get())
        weak->get()->sendResponse(dispatchable.CallId(), response);
    return;
}

class SimulatePreconnectedPeripheralCallbackImpl : public Backend::SimulatePreconnectedPeripheralCallback, public DomainDispatcher::Callback {
public:
    SimulatePreconnectedPeripheralCallbackImpl(std::unique_ptr<DomainDispatcher::WeakPtr> backendImpl, int callId, crdtp::span<uint8_t> message)
        : DomainDispatcher::Callback(std::move(backendImpl), callId,
crdtp::SpanFrom("BluetoothEmulation.simulatePreconnectedPeripheral"), message) { }

    void sendSuccess() override
    {
        crdtp::ObjectSerializer serializer;
        sendIfActive(serializer.Finish(), DispatchResponse::Success());
    }

    void fallThrough() override
    {
        fallThroughIfActive();
    }

    void sendFailure(const DispatchResponse& response) override
    {
        DCHECK(response.IsError());
        sendIfActive(nullptr, response);
    }
};

namespace {

struct simulatePreconnectedPeripheralParams : public crdtp::DeserializableProtocolObject<simulatePreconnectedPeripheralParams> {
    String address;
    String name;
    std::unique_ptr<protocol::Array<protocol::BluetoothEmulation::ManufacturerData>> manufacturerData;
    std::unique_ptr<protocol::Array<String>> knownServiceUuids;
    DECLARE_DESERIALIZATION_SUPPORT();
};

CRDTP_BEGIN_DESERIALIZER(simulatePreconnectedPeripheralParams)
    CRDTP_DESERIALIZE_FIELD("address", address),
    CRDTP_DESERIALIZE_FIELD("knownServiceUuids", knownServiceUuids),
    CRDTP_DESERIALIZE_FIELD("manufacturerData", manufacturerData),
    CRDTP_DESERIALIZE_FIELD("name", name),
CRDTP_END_DESERIALIZER()

}  // namespace

void DomainDispatcherImpl::simulatePreconnectedPeripheral(const crdtp::Dispatchable& dispatchable)
{
    // Prepare input parameters.
    auto deserializer = crdtp::DeferredMessage::FromSpan(dispatchable.Params())->MakeDeserializer();
    simulatePreconnectedPeripheralParams params;
    if (!simulatePreconnectedPeripheralParams::Deserialize(&deserializer, &params)) {
      ReportInvalidParams(dispatchable, deserializer);
      return;
    }

    m_backend->SimulatePreconnectedPeripheral(params.address, params.name, std::move(params.manufacturerData), std::move(params.knownServiceUuids), std::make_unique<SimulatePreconnectedPeripheralCallbackImpl>(weakPtr(), dispatchable.CallId(), dispatchable.Serialized()));
}

class SimulateAdvertisementCallbackImpl : public Backend::SimulateAdvertisementCallback, public DomainDispatcher::Callback {
public:
    SimulateAdvertisementCallbackImpl(std::unique_ptr<DomainDispatcher::WeakPtr> backendImpl, int callId, crdtp::span<uint8_t> message)
        : DomainDispatcher::Callback(std::move(backendImpl), callId,
crdtp::SpanFrom("BluetoothEmulation.simulateAdvertisement"), message) { }

    void sendSuccess() override
    {
        crdtp::ObjectSerializer serializer;
        sendIfActive(serializer.Finish(), DispatchResponse::Success());
    }

    void fallThrough() override
    {
        fallThroughIfActive();
    }

    void sendFailure(const DispatchResponse& response) override
    {
        DCHECK(response.IsError());
        sendIfActive(nullptr, response);
    }
};

namespace {

struct simulateAdvertisementParams : public crdtp::DeserializableProtocolObject<simulateAdvertisementParams> {
    std::unique_ptr<protocol::BluetoothEmulation::ScanEntry> entry;
    DECLARE_DESERIALIZATION_SUPPORT();
};

CRDTP_BEGIN_DESERIALIZER(simulateAdvertisementParams)
    CRDTP_DESERIALIZE_FIELD("entry", entry),
CRDTP_END_DESERIALIZER()

}  // namespace

void DomainDispatcherImpl::simulateAdvertisement(const crdtp::Dispatchable& dispatchable)
{
    // Prepare input parameters.
    auto deserializer = crdtp::DeferredMessage::FromSpan(dispatchable.Params())->MakeDeserializer();
    simulateAdvertisementParams params;
    if (!simulateAdvertisementParams::Deserialize(&deserializer, &params)) {
      ReportInvalidParams(dispatchable, deserializer);
      return;
    }

    m_backend->SimulateAdvertisement(std::move(params.entry), std::make_unique<SimulateAdvertisementCallbackImpl>(weakPtr(), dispatchable.CallId(), dispatchable.Serialized()));
}

namespace {
// This helper method (with a static map of redirects) is used from Dispatcher::wire
// immediately below.
const std::vector<std::pair<crdtp::span<uint8_t>, crdtp::span<uint8_t>>>& SortedRedirects() {
  static auto* redirects = [](){
    auto* redirects = new std::vector<std::pair<crdtp::span<uint8_t>, crdtp::span<uint8_t>>>{
    };
    return redirects;
  }();
  return *redirects;
}
}  // namespace

// static
void Dispatcher::wire(UberDispatcher* uber, Backend* backend)
{
    auto dispatcher = std::make_unique<DomainDispatcherImpl>(uber->channel(), backend);
    uber->WireBackend(crdtp::SpanFrom("BluetoothEmulation"), SortedRedirects(), std::move(dispatcher));
}

} // BluetoothEmulation
} // namespace content
} // namespace protocol
