// Copyright 2014 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_

#include <stddef.h>

#include "base/check_op.h"
#include "third_party/blink/public/mojom/use_counter/metrics/css_property_id.mojom-blink-forward.h"
#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/platform/wtf/allocator/allocator.h"
#include "third_party/blink/renderer/platform/wtf/hash_functions.h"
#include "third_party/blink/renderer/platform/wtf/hash_traits.h"

namespace WTF {
class AtomicString;
class String;
}  // namespace WTF

namespace blink {

class ExecutionContext;

enum class CSSPropertyID {
    kInvalid = 0,
    kVariable = 1,
    kColorScheme = 2,
    kForcedColorAdjust = 3,
    kMaskImage = 4,
    kMathDepth = 5,
    kPosition = 6,
    kPositionAnchor = 7,
    kTextSizeAdjust = 8,
    kAppearance = 9,
    kColor = 10,
    kDirection = 11,
    kFontFamily = 12,
    kFontFeatureSettings = 13,
    kFontKerning = 14,
    kFontOpticalSizing = 15,
    kFontPalette = 16,
    kFontSize = 17,
    kFontSizeAdjust = 18,
    kFontStretch = 19,
    kFontStyle = 20,
    kFontSynthesisSmallCaps = 21,
    kFontSynthesisStyle = 22,
    kFontSynthesisWeight = 23,
    kFontVariantAlternates = 24,
    kFontVariantCaps = 25,
    kFontVariantEastAsian = 26,
    kFontVariantEmoji = 27,
    kFontVariantLigatures = 28,
    kFontVariantNumeric = 29,
    kFontVariantPosition = 30,
    kFontVariationSettings = 31,
    kFontWeight = 32,
    kInternalVisitedColor = 33,
    kPositionArea = 34,
    kTextOrientation = 35,
    kTextRendering = 36,
    kTextSpacingTrim = 37,
    kWebkitFontSmoothing = 38,
    kWebkitLocale = 39,
    kWebkitTextOrientation = 40,
    kWebkitWritingMode = 41,
    kWritingMode = 42,
    kZoom = 43,
    kAccentColor = 44,
    kAdditiveSymbols = 45,
    kAlignContent = 46,
    kAlignItems = 47,
    kAlignSelf = 48,
    kAlignmentBaseline = 49,
    kAll = 50,
    kAnchorName = 51,
    kAnchorScope = 52,
    kAnimationComposition = 53,
    kAnimationDelay = 54,
    kAnimationDirection = 55,
    kAnimationDuration = 56,
    kAnimationFillMode = 57,
    kAnimationIterationCount = 58,
    kAnimationName = 59,
    kAnimationPlayState = 60,
    kAnimationRangeEnd = 61,
    kAnimationRangeStart = 62,
    kAnimationTimeline = 63,
    kAnimationTimingFunction = 64,
    kAnimationTriggerExitRangeEnd = 65,
    kAnimationTriggerExitRangeStart = 66,
    kAnimationTriggerRangeEnd = 67,
    kAnimationTriggerRangeStart = 68,
    kAnimationTriggerTimeline = 69,
    kAnimationTriggerType = 70,
    kAppRegion = 71,
    kAscentOverride = 72,
    kAspectRatio = 73,
    kBackdropFilter = 74,
    kBackfaceVisibility = 75,
    kBackgroundAttachment = 76,
    kBackgroundBlendMode = 77,
    kBackgroundClip = 78,
    kBackgroundColor = 79,
    kBackgroundImage = 80,
    kBackgroundOrigin = 81,
    kBackgroundPositionX = 82,
    kBackgroundPositionY = 83,
    kBackgroundRepeat = 84,
    kBackgroundSize = 85,
    kBasePalette = 86,
    kBaselineShift = 87,
    kBaselineSource = 88,
    kBlockSize = 89,
    kBorderBlockEndColor = 90,
    kBorderBlockEndStyle = 91,
    kBorderBlockEndWidth = 92,
    kBorderBlockStartColor = 93,
    kBorderBlockStartStyle = 94,
    kBorderBlockStartWidth = 95,
    kBorderBottomColor = 96,
    kBorderBottomLeftRadius = 97,
    kBorderBottomRightRadius = 98,
    kBorderBottomStyle = 99,
    kBorderBottomWidth = 100,
    kBorderCollapse = 101,
    kBorderEndEndRadius = 102,
    kBorderEndStartRadius = 103,
    kBorderImageOutset = 104,
    kBorderImageRepeat = 105,
    kBorderImageSlice = 106,
    kBorderImageSource = 107,
    kBorderImageWidth = 108,
    kBorderInlineEndColor = 109,
    kBorderInlineEndStyle = 110,
    kBorderInlineEndWidth = 111,
    kBorderInlineStartColor = 112,
    kBorderInlineStartStyle = 113,
    kBorderInlineStartWidth = 114,
    kBorderLeftColor = 115,
    kBorderLeftStyle = 116,
    kBorderLeftWidth = 117,
    kBorderRightColor = 118,
    kBorderRightStyle = 119,
    kBorderRightWidth = 120,
    kBorderStartEndRadius = 121,
    kBorderStartStartRadius = 122,
    kBorderTopColor = 123,
    kBorderTopLeftRadius = 124,
    kBorderTopRightRadius = 125,
    kBorderTopStyle = 126,
    kBorderTopWidth = 127,
    kBottom = 128,
    kBoxDecorationBreak = 129,
    kBoxShadow = 130,
    kBoxSizing = 131,
    kBreakAfter = 132,
    kBreakBefore = 133,
    kBreakInside = 134,
    kBufferedRendering = 135,
    kCaptionSide = 136,
    kCaretAnimation = 137,
    kCaretColor = 138,
    kClear = 139,
    kClip = 140,
    kClipPath = 141,
    kClipRule = 142,
    kColorInterpolation = 143,
    kColorInterpolationFilters = 144,
    kColorRendering = 145,
    kColumnCount = 146,
    kColumnFill = 147,
    kColumnGap = 148,
    kColumnRuleBreak = 149,
    kColumnRuleColor = 150,
    kColumnRuleStyle = 151,
    kColumnRuleWidth = 152,
    kColumnSpan = 153,
    kColumnWidth = 154,
    kContain = 155,
    kContainIntrinsicBlockSize = 156,
    kContainIntrinsicHeight = 157,
    kContainIntrinsicInlineSize = 158,
    kContainIntrinsicWidth = 159,
    kContainerName = 160,
    kContainerType = 161,
    kContent = 162,
    kContentVisibility = 163,
    kCounterIncrement = 164,
    kCounterReset = 165,
    kCounterSet = 166,
    kCursor = 167,
    kCx = 168,
    kCy = 169,
    kD = 170,
    kDescentOverride = 171,
    kDisplay = 172,
    kDominantBaseline = 173,
    kDynamicRangeLimit = 174,
    kEmptyCells = 175,
    kFallback = 176,
    kFieldSizing = 177,
    kFill = 178,
    kFillOpacity = 179,
    kFillRule = 180,
    kFilter = 181,
    kFlexBasis = 182,
    kFlexDirection = 183,
    kFlexGrow = 184,
    kFlexShrink = 185,
    kFlexWrap = 186,
    kFloat = 187,
    kFloodColor = 188,
    kFloodOpacity = 189,
    kFontDisplay = 190,
    kGridAutoColumns = 191,
    kGridAutoFlow = 192,
    kGridAutoRows = 193,
    kGridColumnEnd = 194,
    kGridColumnStart = 195,
    kGridRowEnd = 196,
    kGridRowStart = 197,
    kGridTemplateAreas = 198,
    kGridTemplateColumns = 199,
    kGridTemplateRows = 200,
    kHeight = 201,
    kHyphenateCharacter = 202,
    kHyphenateLimitChars = 203,
    kHyphens = 204,
    kImageOrientation = 205,
    kImageRendering = 206,
    kInherits = 207,
    kInitialLetter = 208,
    kInitialValue = 209,
    kInlineSize = 210,
    kInsetBlockEnd = 211,
    kInsetBlockStart = 212,
    kInsetInlineEnd = 213,
    kInsetInlineStart = 214,
    kInteractivity = 215,
    kInterestTargetHideDelay = 216,
    kInterestTargetShowDelay = 217,
    kInternalAlignContentBlock = 218,
    kInternalEmptyLineHeight = 219,
    kInternalFontSizeDelta = 220,
    kInternalForcedBackgroundColor = 221,
    kInternalForcedBorderColor = 222,
    kInternalForcedColor = 223,
    kInternalForcedOutlineColor = 224,
    kInternalForcedVisitedColor = 225,
    kInternalOverflowBlock = 226,
    kInternalOverflowInline = 227,
    kInternalVisitedBackgroundColor = 228,
    kInternalVisitedBorderBlockEndColor = 229,
    kInternalVisitedBorderBlockStartColor = 230,
    kInternalVisitedBorderBottomColor = 231,
    kInternalVisitedBorderInlineEndColor = 232,
    kInternalVisitedBorderInlineStartColor = 233,
    kInternalVisitedBorderLeftColor = 234,
    kInternalVisitedBorderRightColor = 235,
    kInternalVisitedBorderTopColor = 236,
    kInternalVisitedCaretColor = 237,
    kInternalVisitedColumnRuleColor = 238,
    kInternalVisitedFill = 239,
    kInternalVisitedOutlineColor = 240,
    kInternalVisitedStroke = 241,
    kInternalVisitedTextDecorationColor = 242,
    kInternalVisitedTextEmphasisColor = 243,
    kInternalVisitedTextFillColor = 244,
    kInternalVisitedTextStrokeColor = 245,
    kInterpolateSize = 246,
    kIsolation = 247,
    kJustifyContent = 248,
    kJustifyItems = 249,
    kJustifySelf = 250,
    kLeft = 251,
    kLetterSpacing = 252,
    kLightingColor = 253,
    kLineBreak = 254,
    kLineClamp = 255,
    kLineGapOverride = 256,
    kLineHeight = 257,
    kListStyleImage = 258,
    kListStylePosition = 259,
    kListStyleType = 260,
    kMarginBlockEnd = 261,
    kMarginBlockStart = 262,
    kMarginBottom = 263,
    kMarginInlineEnd = 264,
    kMarginInlineStart = 265,
    kMarginLeft = 266,
    kMarginRight = 267,
    kMarginTop = 268,
    kMarkerEnd = 269,
    kMarkerMid = 270,
    kMarkerStart = 271,
    kMaskClip = 272,
    kMaskComposite = 273,
    kMaskMode = 274,
    kMaskOrigin = 275,
    kMaskRepeat = 276,
    kMaskSize = 277,
    kMaskType = 278,
    kMasonryAutoTracks = 279,
    kMasonryDirection = 280,
    kMasonryFill = 281,
    kMasonrySlack = 282,
    kMasonryTemplateTracks = 283,
    kMasonryTrackEnd = 284,
    kMasonryTrackStart = 285,
    kMathShift = 286,
    kMathStyle = 287,
    kMaxBlockSize = 288,
    kMaxHeight = 289,
    kMaxInlineSize = 290,
    kMaxWidth = 291,
    kMinBlockSize = 292,
    kMinHeight = 293,
    kMinInlineSize = 294,
    kMinWidth = 295,
    kMixBlendMode = 296,
    kNavigation = 297,
    kNegative = 298,
    kObjectFit = 299,
    kObjectPosition = 300,
    kObjectViewBox = 301,
    kOffsetAnchor = 302,
    kOffsetDistance = 303,
    kOffsetPath = 304,
    kOffsetPosition = 305,
    kOffsetRotate = 306,
    kOpacity = 307,
    kOrder = 308,
    kOriginTrialTestProperty = 309,
    kOrphans = 310,
    kOutlineColor = 311,
    kOutlineOffset = 312,
    kOutlineStyle = 313,
    kOutlineWidth = 314,
    kOverflowAnchor = 315,
    kOverflowBlock = 316,
    kOverflowClipMargin = 317,
    kOverflowInline = 318,
    kOverflowWrap = 319,
    kOverflowX = 320,
    kOverflowY = 321,
    kOverlay = 322,
    kOverrideColors = 323,
    kOverscrollBehaviorBlock = 324,
    kOverscrollBehaviorInline = 325,
    kOverscrollBehaviorX = 326,
    kOverscrollBehaviorY = 327,
    kPad = 328,
    kPaddingBlockEnd = 329,
    kPaddingBlockStart = 330,
    kPaddingBottom = 331,
    kPaddingInlineEnd = 332,
    kPaddingInlineStart = 333,
    kPaddingLeft = 334,
    kPaddingRight = 335,
    kPaddingTop = 336,
    kPage = 337,
    kPageOrientation = 338,
    kPaintOrder = 339,
    kPerspective = 340,
    kPerspectiveOrigin = 341,
    kPointerEvents = 342,
    kPositionTryFallbacks = 343,
    kPositionTryOrder = 344,
    kPositionVisibility = 345,
    kPrefix = 346,
    kQuotes = 347,
    kR = 348,
    kRange = 349,
    kReadingFlow = 350,
    kResize = 351,
    kResult = 352,
    kRight = 353,
    kRotate = 354,
    kRowGap = 355,
    kRubyAlign = 356,
    kRubyPosition = 357,
    kRx = 358,
    kRy = 359,
    kScale = 360,
    kScrollBehavior = 361,
    kScrollInitialTarget = 362,
    kScrollMarginBlockEnd = 363,
    kScrollMarginBlockStart = 364,
    kScrollMarginBottom = 365,
    kScrollMarginInlineEnd = 366,
    kScrollMarginInlineStart = 367,
    kScrollMarginLeft = 368,
    kScrollMarginRight = 369,
    kScrollMarginTop = 370,
    kScrollMarkerGroup = 371,
    kScrollPaddingBlockEnd = 372,
    kScrollPaddingBlockStart = 373,
    kScrollPaddingBottom = 374,
    kScrollPaddingInlineEnd = 375,
    kScrollPaddingInlineStart = 376,
    kScrollPaddingLeft = 377,
    kScrollPaddingRight = 378,
    kScrollPaddingTop = 379,
    kScrollSnapAlign = 380,
    kScrollSnapStop = 381,
    kScrollSnapType = 382,
    kScrollStartBlock = 383,
    kScrollStartInline = 384,
    kScrollStartX = 385,
    kScrollStartY = 386,
    kScrollTimelineAxis = 387,
    kScrollTimelineName = 388,
    kScrollbarColor = 389,
    kScrollbarGutter = 390,
    kScrollbarWidth = 391,
    kShapeImageThreshold = 392,
    kShapeMargin = 393,
    kShapeOutside = 394,
    kShapeRendering = 395,
    kSize = 396,
    kSizeAdjust = 397,
    kSpeak = 398,
    kSpeakAs = 399,
    kSrc = 400,
    kStopColor = 401,
    kStopOpacity = 402,
    kStroke = 403,
    kStrokeDasharray = 404,
    kStrokeDashoffset = 405,
    kStrokeLinecap = 406,
    kStrokeLinejoin = 407,
    kStrokeMiterlimit = 408,
    kStrokeOpacity = 409,
    kStrokeWidth = 410,
    kSuffix = 411,
    kSymbols = 412,
    kSyntax = 413,
    kSystem = 414,
    kTabSize = 415,
    kTableLayout = 416,
    kTextAlign = 417,
    kTextAlignLast = 418,
    kTextAnchor = 419,
    kTextAutospace = 420,
    kTextBoxEdge = 421,
    kTextBoxTrim = 422,
    kTextCombineUpright = 423,
    kTextDecorationColor = 424,
    kTextDecorationLine = 425,
    kTextDecorationSkipInk = 426,
    kTextDecorationStyle = 427,
    kTextDecorationThickness = 428,
    kTextEmphasisColor = 429,
    kTextEmphasisPosition = 430,
    kTextEmphasisStyle = 431,
    kTextIndent = 432,
    kTextOverflow = 433,
    kTextShadow = 434,
    kTextTransform = 435,
    kTextUnderlineOffset = 436,
    kTextUnderlinePosition = 437,
    kTextWrapMode = 438,
    kTextWrapStyle = 439,
    kTimelineScope = 440,
    kTop = 441,
    kTouchAction = 442,
    kTransform = 443,
    kTransformBox = 444,
    kTransformOrigin = 445,
    kTransformStyle = 446,
    kTransitionBehavior = 447,
    kTransitionDelay = 448,
    kTransitionDuration = 449,
    kTransitionProperty = 450,
    kTransitionTimingFunction = 451,
    kTranslate = 452,
    kTypes = 453,
    kUnicodeBidi = 454,
    kUnicodeRange = 455,
    kUserSelect = 456,
    kVectorEffect = 457,
    kVerticalAlign = 458,
    kViewTimelineAxis = 459,
    kViewTimelineInset = 460,
    kViewTimelineName = 461,
    kViewTransitionCaptureMode = 462,
    kViewTransitionClass = 463,
    kViewTransitionGroup = 464,
    kViewTransitionName = 465,
    kVisibility = 466,
    kWebkitBorderHorizontalSpacing = 467,
    kWebkitBorderImage = 468,
    kWebkitBorderVerticalSpacing = 469,
    kWebkitBoxAlign = 470,
    kWebkitBoxDecorationBreak = 471,
    kWebkitBoxDirection = 472,
    kWebkitBoxFlex = 473,
    kWebkitBoxOrdinalGroup = 474,
    kWebkitBoxOrient = 475,
    kWebkitBoxPack = 476,
    kWebkitBoxReflect = 477,
    kWebkitLineBreak = 478,
    kWebkitLineClamp = 479,
    kWebkitMaskBoxImageOutset = 480,
    kWebkitMaskBoxImageRepeat = 481,
    kWebkitMaskBoxImageSlice = 482,
    kWebkitMaskBoxImageSource = 483,
    kWebkitMaskBoxImageWidth = 484,
    kWebkitMaskPositionX = 485,
    kWebkitMaskPositionY = 486,
    kWebkitPerspectiveOriginX = 487,
    kWebkitPerspectiveOriginY = 488,
    kWebkitPrintColorAdjust = 489,
    kWebkitRtlOrdering = 490,
    kWebkitRubyPosition = 491,
    kWebkitTapHighlightColor = 492,
    kWebkitTextCombine = 493,
    kWebkitTextDecorationsInEffect = 494,
    kWebkitTextFillColor = 495,
    kWebkitTextSecurity = 496,
    kWebkitTextStrokeColor = 497,
    kWebkitTextStrokeWidth = 498,
    kWebkitTransformOriginX = 499,
    kWebkitTransformOriginY = 500,
    kWebkitTransformOriginZ = 501,
    kWebkitUserDrag = 502,
    kWebkitUserModify = 503,
    kWhiteSpaceCollapse = 504,
    kWidows = 505,
    kWidth = 506,
    kWillChange = 507,
    kWordBreak = 508,
    kWordSpacing = 509,
    kX = 510,
    kY = 511,
    kZIndex = 512,
    kAnimation = 513,
    kAnimationRange = 514,
    kAnimationTrigger = 515,
    kAnimationTriggerExitRange = 516,
    kAnimationTriggerRange = 517,
    kBackground = 518,
    kBackgroundPosition = 519,
    kBorder = 520,
    kBorderBlock = 521,
    kBorderBlockColor = 522,
    kBorderBlockEnd = 523,
    kBorderBlockStart = 524,
    kBorderBlockStyle = 525,
    kBorderBlockWidth = 526,
    kBorderBottom = 527,
    kBorderColor = 528,
    kBorderImage = 529,
    kBorderInline = 530,
    kBorderInlineColor = 531,
    kBorderInlineEnd = 532,
    kBorderInlineStart = 533,
    kBorderInlineStyle = 534,
    kBorderInlineWidth = 535,
    kBorderLeft = 536,
    kBorderRadius = 537,
    kBorderRight = 538,
    kBorderSpacing = 539,
    kBorderStyle = 540,
    kBorderTop = 541,
    kBorderWidth = 542,
    kColumnRule = 543,
    kColumns = 544,
    kContainIntrinsicSize = 545,
    kContainer = 546,
    kFlex = 547,
    kFlexFlow = 548,
    kFont = 549,
    kFontSynthesis = 550,
    kFontVariant = 551,
    kGap = 552,
    kGrid = 553,
    kGridArea = 554,
    kGridColumn = 555,
    kGridRow = 556,
    kGridTemplate = 557,
    kInset = 558,
    kInsetBlock = 559,
    kInsetInline = 560,
    kInterestTargetDelay = 561,
    kListStyle = 562,
    kMargin = 563,
    kMarginBlock = 564,
    kMarginInline = 565,
    kMarker = 566,
    kMask = 567,
    kMaskPosition = 568,
    kMasonryFlow = 569,
    kMasonryTrack = 570,
    kOffset = 571,
    kOutline = 572,
    kOverflow = 573,
    kOverscrollBehavior = 574,
    kPadding = 575,
    kPaddingBlock = 576,
    kPaddingInline = 577,
    kPageBreakAfter = 578,
    kPageBreakBefore = 579,
    kPageBreakInside = 580,
    kPlaceContent = 581,
    kPlaceItems = 582,
    kPlaceSelf = 583,
    kPositionTry = 584,
    kScrollMargin = 585,
    kScrollMarginBlock = 586,
    kScrollMarginInline = 587,
    kScrollPadding = 588,
    kScrollPaddingBlock = 589,
    kScrollPaddingInline = 590,
    kScrollStart = 591,
    kScrollTimeline = 592,
    kTextBox = 593,
    kTextDecoration = 594,
    kTextEmphasis = 595,
    kTextSpacing = 596,
    kTextWrap = 597,
    kTransition = 598,
    kViewTimeline = 599,
    kWebkitColumnBreakAfter = 600,
    kWebkitColumnBreakBefore = 601,
    kWebkitColumnBreakInside = 602,
    kWebkitMaskBoxImage = 603,
    kWebkitTextStroke = 604,
    kWhiteSpace = 605,
    kAliasWebkitAppearance = 606,
    kAliasWebkitAppRegion = 607,
    kAliasWebkitMaskClip = 608,
    kAliasWebkitMaskComposite = 609,
    kAliasWebkitMaskImage = 610,
    kAliasWebkitMaskOrigin = 611,
    kAliasWebkitMaskRepeat = 612,
    kAliasWebkitMaskSize = 613,
    kAliasWebkitBorderEndColor = 614,
    kAliasWebkitBorderEndStyle = 615,
    kAliasWebkitBorderEndWidth = 616,
    kAliasWebkitBorderStartColor = 617,
    kAliasWebkitBorderStartStyle = 618,
    kAliasWebkitBorderStartWidth = 619,
    kAliasWebkitBorderBeforeColor = 620,
    kAliasWebkitBorderBeforeStyle = 621,
    kAliasWebkitBorderBeforeWidth = 622,
    kAliasWebkitBorderAfterColor = 623,
    kAliasWebkitBorderAfterStyle = 624,
    kAliasWebkitBorderAfterWidth = 625,
    kAliasWebkitMarginEnd = 626,
    kAliasWebkitMarginStart = 627,
    kAliasWebkitMarginBefore = 628,
    kAliasWebkitMarginAfter = 629,
    kAliasWebkitPaddingEnd = 630,
    kAliasWebkitPaddingStart = 631,
    kAliasWebkitPaddingBefore = 632,
    kAliasWebkitPaddingAfter = 633,
    kAliasWebkitLogicalWidth = 634,
    kAliasWebkitLogicalHeight = 635,
    kAliasWebkitMinLogicalWidth = 636,
    kAliasWebkitMinLogicalHeight = 637,
    kAliasWebkitMaxLogicalWidth = 638,
    kAliasWebkitMaxLogicalHeight = 639,
    kAliasWebkitBorderAfter = 640,
    kAliasWebkitBorderBefore = 641,
    kAliasWebkitBorderEnd = 642,
    kAliasWebkitBorderStart = 643,
    kAliasWebkitMask = 644,
    kAliasWebkitMaskPosition = 645,
    kAliasEpubCaptionSide = 646,
    kAliasEpubTextCombine = 647,
    kAliasEpubTextEmphasis = 648,
    kAliasEpubTextEmphasisColor = 649,
    kAliasEpubTextEmphasisStyle = 650,
    kAliasEpubTextOrientation = 651,
    kAliasEpubTextTransform = 652,
    kAliasEpubWordBreak = 653,
    kAliasEpubWritingMode = 654,
    kAliasWebkitAlignContent = 655,
    kAliasWebkitAlignItems = 656,
    kAliasWebkitAlignSelf = 657,
    kAliasWebkitAnimation = 658,
    kAliasWebkitAnimationDelay = 659,
    kAliasWebkitAnimationDirection = 660,
    kAliasWebkitAnimationDuration = 661,
    kAliasWebkitAnimationFillMode = 662,
    kAliasWebkitAnimationIterationCount = 663,
    kAliasWebkitAnimationName = 664,
    kAliasWebkitAnimationPlayState = 665,
    kAliasWebkitAnimationTimingFunction = 666,
    kAliasWebkitBackfaceVisibility = 667,
    kAliasWebkitBackgroundClip = 668,
    kAliasWebkitBackgroundOrigin = 669,
    kAliasWebkitBackgroundSize = 670,
    kAliasWebkitBorderBottomLeftRadius = 671,
    kAliasWebkitBorderBottomRightRadius = 672,
    kAliasWebkitBorderRadius = 673,
    kAliasWebkitBorderTopLeftRadius = 674,
    kAliasWebkitBorderTopRightRadius = 675,
    kAliasWebkitBoxShadow = 676,
    kAliasWebkitBoxSizing = 677,
    kAliasWebkitClipPath = 678,
    kAliasWebkitColumnCount = 679,
    kAliasWebkitColumnGap = 680,
    kAliasWebkitColumnRule = 681,
    kAliasWebkitColumnRuleColor = 682,
    kAliasWebkitColumnRuleStyle = 683,
    kAliasWebkitColumnRuleWidth = 684,
    kAliasWebkitColumnSpan = 685,
    kAliasWebkitColumnWidth = 686,
    kAliasWebkitColumns = 687,
    kAliasWebkitFilter = 688,
    kAliasWebkitFlex = 689,
    kAliasWebkitFlexBasis = 690,
    kAliasWebkitFlexDirection = 691,
    kAliasWebkitFlexFlow = 692,
    kAliasWebkitFlexGrow = 693,
    kAliasWebkitFlexShrink = 694,
    kAliasWebkitFlexWrap = 695,
    kAliasWebkitFontFeatureSettings = 696,
    kAliasWebkitHyphenateCharacter = 697,
    kAliasWebkitJustifyContent = 698,
    kAliasWebkitOpacity = 699,
    kAliasWebkitOrder = 700,
    kAliasWebkitPerspective = 701,
    kAliasWebkitPerspectiveOrigin = 702,
    kAliasWebkitShapeImageThreshold = 703,
    kAliasWebkitShapeMargin = 704,
    kAliasWebkitShapeOutside = 705,
    kAliasWebkitTextEmphasis = 706,
    kAliasWebkitTextEmphasisColor = 707,
    kAliasWebkitTextEmphasisPosition = 708,
    kAliasWebkitTextEmphasisStyle = 709,
    kAliasWebkitTextSizeAdjust = 710,
    kAliasWebkitTransform = 711,
    kAliasWebkitTransformOrigin = 712,
    kAliasWebkitTransformStyle = 713,
    kAliasWebkitTransition = 714,
    kAliasWebkitTransitionDelay = 715,
    kAliasWebkitTransitionDuration = 716,
    kAliasWebkitTransitionProperty = 717,
    kAliasWebkitTransitionTimingFunction = 718,
    kAliasWebkitUserSelect = 719,
    kAliasWordWrap = 720,
    kAliasGridColumnGap = 721,
    kAliasGridRowGap = 722,
    kAliasGridGap = 723,
};

const CSSPropertyID kCSSPropertyAliasList[] = {
    CSSPropertyID::kAliasWebkitAppearance,
    CSSPropertyID::kAliasWebkitAppRegion,
    CSSPropertyID::kAliasWebkitMaskClip,
    CSSPropertyID::kAliasWebkitMaskComposite,
    CSSPropertyID::kAliasWebkitMaskImage,
    CSSPropertyID::kAliasWebkitMaskOrigin,
    CSSPropertyID::kAliasWebkitMaskRepeat,
    CSSPropertyID::kAliasWebkitMaskSize,
    CSSPropertyID::kAliasWebkitBorderEndColor,
    CSSPropertyID::kAliasWebkitBorderEndStyle,
    CSSPropertyID::kAliasWebkitBorderEndWidth,
    CSSPropertyID::kAliasWebkitBorderStartColor,
    CSSPropertyID::kAliasWebkitBorderStartStyle,
    CSSPropertyID::kAliasWebkitBorderStartWidth,
    CSSPropertyID::kAliasWebkitBorderBeforeColor,
    CSSPropertyID::kAliasWebkitBorderBeforeStyle,
    CSSPropertyID::kAliasWebkitBorderBeforeWidth,
    CSSPropertyID::kAliasWebkitBorderAfterColor,
    CSSPropertyID::kAliasWebkitBorderAfterStyle,
    CSSPropertyID::kAliasWebkitBorderAfterWidth,
    CSSPropertyID::kAliasWebkitMarginEnd,
    CSSPropertyID::kAliasWebkitMarginStart,
    CSSPropertyID::kAliasWebkitMarginBefore,
    CSSPropertyID::kAliasWebkitMarginAfter,
    CSSPropertyID::kAliasWebkitPaddingEnd,
    CSSPropertyID::kAliasWebkitPaddingStart,
    CSSPropertyID::kAliasWebkitPaddingBefore,
    CSSPropertyID::kAliasWebkitPaddingAfter,
    CSSPropertyID::kAliasWebkitLogicalWidth,
    CSSPropertyID::kAliasWebkitLogicalHeight,
    CSSPropertyID::kAliasWebkitMinLogicalWidth,
    CSSPropertyID::kAliasWebkitMinLogicalHeight,
    CSSPropertyID::kAliasWebkitMaxLogicalWidth,
    CSSPropertyID::kAliasWebkitMaxLogicalHeight,
    CSSPropertyID::kAliasWebkitBorderAfter,
    CSSPropertyID::kAliasWebkitBorderBefore,
    CSSPropertyID::kAliasWebkitBorderEnd,
    CSSPropertyID::kAliasWebkitBorderStart,
    CSSPropertyID::kAliasWebkitMask,
    CSSPropertyID::kAliasWebkitMaskPosition,
    CSSPropertyID::kAliasEpubCaptionSide,
    CSSPropertyID::kAliasEpubTextCombine,
    CSSPropertyID::kAliasEpubTextEmphasis,
    CSSPropertyID::kAliasEpubTextEmphasisColor,
    CSSPropertyID::kAliasEpubTextEmphasisStyle,
    CSSPropertyID::kAliasEpubTextOrientation,
    CSSPropertyID::kAliasEpubTextTransform,
    CSSPropertyID::kAliasEpubWordBreak,
    CSSPropertyID::kAliasEpubWritingMode,
    CSSPropertyID::kAliasWebkitAlignContent,
    CSSPropertyID::kAliasWebkitAlignItems,
    CSSPropertyID::kAliasWebkitAlignSelf,
    CSSPropertyID::kAliasWebkitAnimation,
    CSSPropertyID::kAliasWebkitAnimationDelay,
    CSSPropertyID::kAliasWebkitAnimationDirection,
    CSSPropertyID::kAliasWebkitAnimationDuration,
    CSSPropertyID::kAliasWebkitAnimationFillMode,
    CSSPropertyID::kAliasWebkitAnimationIterationCount,
    CSSPropertyID::kAliasWebkitAnimationName,
    CSSPropertyID::kAliasWebkitAnimationPlayState,
    CSSPropertyID::kAliasWebkitAnimationTimingFunction,
    CSSPropertyID::kAliasWebkitBackfaceVisibility,
    CSSPropertyID::kAliasWebkitBackgroundClip,
    CSSPropertyID::kAliasWebkitBackgroundOrigin,
    CSSPropertyID::kAliasWebkitBackgroundSize,
    CSSPropertyID::kAliasWebkitBorderBottomLeftRadius,
    CSSPropertyID::kAliasWebkitBorderBottomRightRadius,
    CSSPropertyID::kAliasWebkitBorderRadius,
    CSSPropertyID::kAliasWebkitBorderTopLeftRadius,
    CSSPropertyID::kAliasWebkitBorderTopRightRadius,
    CSSPropertyID::kAliasWebkitBoxShadow,
    CSSPropertyID::kAliasWebkitBoxSizing,
    CSSPropertyID::kAliasWebkitClipPath,
    CSSPropertyID::kAliasWebkitColumnCount,
    CSSPropertyID::kAliasWebkitColumnGap,
    CSSPropertyID::kAliasWebkitColumnRule,
    CSSPropertyID::kAliasWebkitColumnRuleColor,
    CSSPropertyID::kAliasWebkitColumnRuleStyle,
    CSSPropertyID::kAliasWebkitColumnRuleWidth,
    CSSPropertyID::kAliasWebkitColumnSpan,
    CSSPropertyID::kAliasWebkitColumnWidth,
    CSSPropertyID::kAliasWebkitColumns,
    CSSPropertyID::kAliasWebkitFilter,
    CSSPropertyID::kAliasWebkitFlex,
    CSSPropertyID::kAliasWebkitFlexBasis,
    CSSPropertyID::kAliasWebkitFlexDirection,
    CSSPropertyID::kAliasWebkitFlexFlow,
    CSSPropertyID::kAliasWebkitFlexGrow,
    CSSPropertyID::kAliasWebkitFlexShrink,
    CSSPropertyID::kAliasWebkitFlexWrap,
    CSSPropertyID::kAliasWebkitFontFeatureSettings,
    CSSPropertyID::kAliasWebkitHyphenateCharacter,
    CSSPropertyID::kAliasWebkitJustifyContent,
    CSSPropertyID::kAliasWebkitOpacity,
    CSSPropertyID::kAliasWebkitOrder,
    CSSPropertyID::kAliasWebkitPerspective,
    CSSPropertyID::kAliasWebkitPerspectiveOrigin,
    CSSPropertyID::kAliasWebkitShapeImageThreshold,
    CSSPropertyID::kAliasWebkitShapeMargin,
    CSSPropertyID::kAliasWebkitShapeOutside,
    CSSPropertyID::kAliasWebkitTextEmphasis,
    CSSPropertyID::kAliasWebkitTextEmphasisColor,
    CSSPropertyID::kAliasWebkitTextEmphasisPosition,
    CSSPropertyID::kAliasWebkitTextEmphasisStyle,
    CSSPropertyID::kAliasWebkitTextSizeAdjust,
    CSSPropertyID::kAliasWebkitTransform,
    CSSPropertyID::kAliasWebkitTransformOrigin,
    CSSPropertyID::kAliasWebkitTransformStyle,
    CSSPropertyID::kAliasWebkitTransition,
    CSSPropertyID::kAliasWebkitTransitionDelay,
    CSSPropertyID::kAliasWebkitTransitionDuration,
    CSSPropertyID::kAliasWebkitTransitionProperty,
    CSSPropertyID::kAliasWebkitTransitionTimingFunction,
    CSSPropertyID::kAliasWebkitUserSelect,
    CSSPropertyID::kAliasWordWrap,
    CSSPropertyID::kAliasGridColumnGap,
    CSSPropertyID::kAliasGridRowGap,
    CSSPropertyID::kAliasGridGap,
};

const CSSPropertyID kCSSComputableProperties[] = {
    CSSPropertyID::kAccentColor,
    CSSPropertyID::kAlignContent,
    CSSPropertyID::kAlignItems,
    CSSPropertyID::kAlignSelf,
    CSSPropertyID::kAlignmentBaseline,
    CSSPropertyID::kAnchorName,
    CSSPropertyID::kAnchorScope,
    CSSPropertyID::kAnimationComposition,
    CSSPropertyID::kAnimationDelay,
    CSSPropertyID::kAnimationDirection,
    CSSPropertyID::kAnimationDuration,
    CSSPropertyID::kAnimationFillMode,
    CSSPropertyID::kAnimationIterationCount,
    CSSPropertyID::kAnimationName,
    CSSPropertyID::kAnimationPlayState,
    CSSPropertyID::kAnimationRangeEnd,
    CSSPropertyID::kAnimationRangeStart,
    CSSPropertyID::kAnimationTimeline,
    CSSPropertyID::kAnimationTimingFunction,
    CSSPropertyID::kAnimationTriggerExitRangeEnd,
    CSSPropertyID::kAnimationTriggerExitRangeStart,
    CSSPropertyID::kAnimationTriggerRangeEnd,
    CSSPropertyID::kAnimationTriggerRangeStart,
    CSSPropertyID::kAnimationTriggerTimeline,
    CSSPropertyID::kAnimationTriggerType,
    CSSPropertyID::kAppRegion,
    CSSPropertyID::kAppearance,
    CSSPropertyID::kBackdropFilter,
    CSSPropertyID::kBackfaceVisibility,
    CSSPropertyID::kBackgroundAttachment,
    CSSPropertyID::kBackgroundBlendMode,
    CSSPropertyID::kBackgroundClip,
    CSSPropertyID::kBackgroundColor,
    CSSPropertyID::kBackgroundImage,
    CSSPropertyID::kBackgroundOrigin,
    CSSPropertyID::kBackgroundPosition,
    CSSPropertyID::kBackgroundRepeat,
    CSSPropertyID::kBackgroundSize,
    CSSPropertyID::kBaselineShift,
    CSSPropertyID::kBaselineSource,
    CSSPropertyID::kBlockSize,
    CSSPropertyID::kBorderBlockEndColor,
    CSSPropertyID::kBorderBlockEndStyle,
    CSSPropertyID::kBorderBlockEndWidth,
    CSSPropertyID::kBorderBlockStartColor,
    CSSPropertyID::kBorderBlockStartStyle,
    CSSPropertyID::kBorderBlockStartWidth,
    CSSPropertyID::kBorderBottomColor,
    CSSPropertyID::kBorderBottomLeftRadius,
    CSSPropertyID::kBorderBottomRightRadius,
    CSSPropertyID::kBorderBottomStyle,
    CSSPropertyID::kBorderBottomWidth,
    CSSPropertyID::kBorderCollapse,
    CSSPropertyID::kBorderEndEndRadius,
    CSSPropertyID::kBorderEndStartRadius,
    CSSPropertyID::kBorderImageOutset,
    CSSPropertyID::kBorderImageRepeat,
    CSSPropertyID::kBorderImageSlice,
    CSSPropertyID::kBorderImageSource,
    CSSPropertyID::kBorderImageWidth,
    CSSPropertyID::kBorderInlineEndColor,
    CSSPropertyID::kBorderInlineEndStyle,
    CSSPropertyID::kBorderInlineEndWidth,
    CSSPropertyID::kBorderInlineStartColor,
    CSSPropertyID::kBorderInlineStartStyle,
    CSSPropertyID::kBorderInlineStartWidth,
    CSSPropertyID::kBorderLeftColor,
    CSSPropertyID::kBorderLeftStyle,
    CSSPropertyID::kBorderLeftWidth,
    CSSPropertyID::kBorderRightColor,
    CSSPropertyID::kBorderRightStyle,
    CSSPropertyID::kBorderRightWidth,
    CSSPropertyID::kBorderStartEndRadius,
    CSSPropertyID::kBorderStartStartRadius,
    CSSPropertyID::kBorderTopColor,
    CSSPropertyID::kBorderTopLeftRadius,
    CSSPropertyID::kBorderTopRightRadius,
    CSSPropertyID::kBorderTopStyle,
    CSSPropertyID::kBorderTopWidth,
    CSSPropertyID::kBottom,
    CSSPropertyID::kBoxDecorationBreak,
    CSSPropertyID::kBoxShadow,
    CSSPropertyID::kBoxSizing,
    CSSPropertyID::kBreakAfter,
    CSSPropertyID::kBreakBefore,
    CSSPropertyID::kBreakInside,
    CSSPropertyID::kBufferedRendering,
    CSSPropertyID::kCaptionSide,
    CSSPropertyID::kCaretAnimation,
    CSSPropertyID::kCaretColor,
    CSSPropertyID::kClear,
    CSSPropertyID::kClip,
    CSSPropertyID::kClipPath,
    CSSPropertyID::kClipRule,
    CSSPropertyID::kColor,
    CSSPropertyID::kColorInterpolation,
    CSSPropertyID::kColorInterpolationFilters,
    CSSPropertyID::kColorRendering,
    CSSPropertyID::kColumnCount,
    CSSPropertyID::kColumnGap,
    CSSPropertyID::kColumnRuleBreak,
    CSSPropertyID::kColumnRuleColor,
    CSSPropertyID::kColumnRuleStyle,
    CSSPropertyID::kColumnRuleWidth,
    CSSPropertyID::kColumnSpan,
    CSSPropertyID::kColumnWidth,
    CSSPropertyID::kContainIntrinsicBlockSize,
    CSSPropertyID::kContainIntrinsicHeight,
    CSSPropertyID::kContainIntrinsicInlineSize,
    CSSPropertyID::kContainIntrinsicSize,
    CSSPropertyID::kContainIntrinsicWidth,
    CSSPropertyID::kContainerName,
    CSSPropertyID::kContainerType,
    CSSPropertyID::kContent,
    CSSPropertyID::kCursor,
    CSSPropertyID::kCx,
    CSSPropertyID::kCy,
    CSSPropertyID::kD,
    CSSPropertyID::kDirection,
    CSSPropertyID::kDisplay,
    CSSPropertyID::kDominantBaseline,
    CSSPropertyID::kDynamicRangeLimit,
    CSSPropertyID::kEmptyCells,
    CSSPropertyID::kFieldSizing,
    CSSPropertyID::kFill,
    CSSPropertyID::kFillOpacity,
    CSSPropertyID::kFillRule,
    CSSPropertyID::kFilter,
    CSSPropertyID::kFlexBasis,
    CSSPropertyID::kFlexDirection,
    CSSPropertyID::kFlexGrow,
    CSSPropertyID::kFlexShrink,
    CSSPropertyID::kFlexWrap,
    CSSPropertyID::kFloat,
    CSSPropertyID::kFloodColor,
    CSSPropertyID::kFloodOpacity,
    CSSPropertyID::kFontFamily,
    CSSPropertyID::kFontKerning,
    CSSPropertyID::kFontOpticalSizing,
    CSSPropertyID::kFontPalette,
    CSSPropertyID::kFontSize,
    CSSPropertyID::kFontSizeAdjust,
    CSSPropertyID::kFontStretch,
    CSSPropertyID::kFontStyle,
    CSSPropertyID::kFontSynthesisSmallCaps,
    CSSPropertyID::kFontSynthesisStyle,
    CSSPropertyID::kFontSynthesisWeight,
    CSSPropertyID::kFontVariant,
    CSSPropertyID::kFontVariantAlternates,
    CSSPropertyID::kFontVariantCaps,
    CSSPropertyID::kFontVariantEastAsian,
    CSSPropertyID::kFontVariantEmoji,
    CSSPropertyID::kFontVariantLigatures,
    CSSPropertyID::kFontVariantNumeric,
    CSSPropertyID::kFontVariantPosition,
    CSSPropertyID::kFontWeight,
    CSSPropertyID::kGridAutoColumns,
    CSSPropertyID::kGridAutoFlow,
    CSSPropertyID::kGridAutoRows,
    CSSPropertyID::kGridColumnEnd,
    CSSPropertyID::kGridColumnStart,
    CSSPropertyID::kGridRowEnd,
    CSSPropertyID::kGridRowStart,
    CSSPropertyID::kGridTemplateAreas,
    CSSPropertyID::kGridTemplateColumns,
    CSSPropertyID::kGridTemplateRows,
    CSSPropertyID::kHeight,
    CSSPropertyID::kHyphenateCharacter,
    CSSPropertyID::kHyphenateLimitChars,
    CSSPropertyID::kHyphens,
    CSSPropertyID::kImageOrientation,
    CSSPropertyID::kImageRendering,
    CSSPropertyID::kInitialLetter,
    CSSPropertyID::kInlineSize,
    CSSPropertyID::kInsetBlockEnd,
    CSSPropertyID::kInsetBlockStart,
    CSSPropertyID::kInsetInlineEnd,
    CSSPropertyID::kInsetInlineStart,
    CSSPropertyID::kInteractivity,
    CSSPropertyID::kInterestTargetHideDelay,
    CSSPropertyID::kInterestTargetShowDelay,
    CSSPropertyID::kInterpolateSize,
    CSSPropertyID::kIsolation,
    CSSPropertyID::kJustifyContent,
    CSSPropertyID::kJustifyItems,
    CSSPropertyID::kJustifySelf,
    CSSPropertyID::kLeft,
    CSSPropertyID::kLetterSpacing,
    CSSPropertyID::kLightingColor,
    CSSPropertyID::kLineBreak,
    CSSPropertyID::kLineClamp,
    CSSPropertyID::kLineHeight,
    CSSPropertyID::kListStyleImage,
    CSSPropertyID::kListStylePosition,
    CSSPropertyID::kListStyleType,
    CSSPropertyID::kMarginBlockEnd,
    CSSPropertyID::kMarginBlockStart,
    CSSPropertyID::kMarginBottom,
    CSSPropertyID::kMarginInlineEnd,
    CSSPropertyID::kMarginInlineStart,
    CSSPropertyID::kMarginLeft,
    CSSPropertyID::kMarginRight,
    CSSPropertyID::kMarginTop,
    CSSPropertyID::kMarkerEnd,
    CSSPropertyID::kMarkerMid,
    CSSPropertyID::kMarkerStart,
    CSSPropertyID::kMaskClip,
    CSSPropertyID::kMaskComposite,
    CSSPropertyID::kMaskImage,
    CSSPropertyID::kMaskMode,
    CSSPropertyID::kMaskOrigin,
    CSSPropertyID::kMaskPosition,
    CSSPropertyID::kMaskRepeat,
    CSSPropertyID::kMaskSize,
    CSSPropertyID::kMaskType,
    CSSPropertyID::kMasonryAutoTracks,
    CSSPropertyID::kMasonryDirection,
    CSSPropertyID::kMasonryFill,
    CSSPropertyID::kMasonrySlack,
    CSSPropertyID::kMasonryTemplateTracks,
    CSSPropertyID::kMasonryTrackEnd,
    CSSPropertyID::kMasonryTrackStart,
    CSSPropertyID::kMathDepth,
    CSSPropertyID::kMathShift,
    CSSPropertyID::kMathStyle,
    CSSPropertyID::kMaxBlockSize,
    CSSPropertyID::kMaxHeight,
    CSSPropertyID::kMaxInlineSize,
    CSSPropertyID::kMaxWidth,
    CSSPropertyID::kMinBlockSize,
    CSSPropertyID::kMinHeight,
    CSSPropertyID::kMinInlineSize,
    CSSPropertyID::kMinWidth,
    CSSPropertyID::kMixBlendMode,
    CSSPropertyID::kObjectFit,
    CSSPropertyID::kObjectPosition,
    CSSPropertyID::kObjectViewBox,
    CSSPropertyID::kOffsetAnchor,
    CSSPropertyID::kOffsetDistance,
    CSSPropertyID::kOffsetPath,
    CSSPropertyID::kOffsetPosition,
    CSSPropertyID::kOffsetRotate,
    CSSPropertyID::kOpacity,
    CSSPropertyID::kOrder,
    CSSPropertyID::kOrphans,
    CSSPropertyID::kOutlineColor,
    CSSPropertyID::kOutlineOffset,
    CSSPropertyID::kOutlineStyle,
    CSSPropertyID::kOutlineWidth,
    CSSPropertyID::kOverflowAnchor,
    CSSPropertyID::kOverflowBlock,
    CSSPropertyID::kOverflowClipMargin,
    CSSPropertyID::kOverflowInline,
    CSSPropertyID::kOverflowWrap,
    CSSPropertyID::kOverflowX,
    CSSPropertyID::kOverflowY,
    CSSPropertyID::kOverlay,
    CSSPropertyID::kOverscrollBehaviorBlock,
    CSSPropertyID::kOverscrollBehaviorInline,
    CSSPropertyID::kPaddingBlockEnd,
    CSSPropertyID::kPaddingBlockStart,
    CSSPropertyID::kPaddingBottom,
    CSSPropertyID::kPaddingInlineEnd,
    CSSPropertyID::kPaddingInlineStart,
    CSSPropertyID::kPaddingLeft,
    CSSPropertyID::kPaddingRight,
    CSSPropertyID::kPaddingTop,
    CSSPropertyID::kPaintOrder,
    CSSPropertyID::kPerspective,
    CSSPropertyID::kPerspectiveOrigin,
    CSSPropertyID::kPointerEvents,
    CSSPropertyID::kPosition,
    CSSPropertyID::kPositionAnchor,
    CSSPropertyID::kPositionArea,
    CSSPropertyID::kPositionTryFallbacks,
    CSSPropertyID::kPositionTryOrder,
    CSSPropertyID::kPositionVisibility,
    CSSPropertyID::kR,
    CSSPropertyID::kReadingFlow,
    CSSPropertyID::kResize,
    CSSPropertyID::kRight,
    CSSPropertyID::kRotate,
    CSSPropertyID::kRowGap,
    CSSPropertyID::kRubyAlign,
    CSSPropertyID::kRubyPosition,
    CSSPropertyID::kRx,
    CSSPropertyID::kRy,
    CSSPropertyID::kScale,
    CSSPropertyID::kScrollBehavior,
    CSSPropertyID::kScrollInitialTarget,
    CSSPropertyID::kScrollMarginBlockEnd,
    CSSPropertyID::kScrollMarginBlockStart,
    CSSPropertyID::kScrollMarginInlineEnd,
    CSSPropertyID::kScrollMarginInlineStart,
    CSSPropertyID::kScrollMarkerGroup,
    CSSPropertyID::kScrollPaddingBlockEnd,
    CSSPropertyID::kScrollPaddingBlockStart,
    CSSPropertyID::kScrollPaddingInlineEnd,
    CSSPropertyID::kScrollPaddingInlineStart,
    CSSPropertyID::kScrollStartBlock,
    CSSPropertyID::kScrollStartInline,
    CSSPropertyID::kScrollStartX,
    CSSPropertyID::kScrollStartY,
    CSSPropertyID::kScrollTimelineAxis,
    CSSPropertyID::kScrollTimelineName,
    CSSPropertyID::kScrollbarColor,
    CSSPropertyID::kScrollbarGutter,
    CSSPropertyID::kScrollbarWidth,
    CSSPropertyID::kShapeImageThreshold,
    CSSPropertyID::kShapeMargin,
    CSSPropertyID::kShapeOutside,
    CSSPropertyID::kShapeRendering,
    CSSPropertyID::kSpeak,
    CSSPropertyID::kStopColor,
    CSSPropertyID::kStopOpacity,
    CSSPropertyID::kStroke,
    CSSPropertyID::kStrokeDasharray,
    CSSPropertyID::kStrokeDashoffset,
    CSSPropertyID::kStrokeLinecap,
    CSSPropertyID::kStrokeLinejoin,
    CSSPropertyID::kStrokeMiterlimit,
    CSSPropertyID::kStrokeOpacity,
    CSSPropertyID::kStrokeWidth,
    CSSPropertyID::kTabSize,
    CSSPropertyID::kTableLayout,
    CSSPropertyID::kTextAlign,
    CSSPropertyID::kTextAlignLast,
    CSSPropertyID::kTextAnchor,
    CSSPropertyID::kTextAutospace,
    CSSPropertyID::kTextBoxEdge,
    CSSPropertyID::kTextBoxTrim,
    CSSPropertyID::kTextDecoration,
    CSSPropertyID::kTextDecorationColor,
    CSSPropertyID::kTextDecorationLine,
    CSSPropertyID::kTextDecorationSkipInk,
    CSSPropertyID::kTextDecorationStyle,
    CSSPropertyID::kTextEmphasisColor,
    CSSPropertyID::kTextEmphasisPosition,
    CSSPropertyID::kTextEmphasisStyle,
    CSSPropertyID::kTextIndent,
    CSSPropertyID::kTextOverflow,
    CSSPropertyID::kTextRendering,
    CSSPropertyID::kTextShadow,
    CSSPropertyID::kTextSizeAdjust,
    CSSPropertyID::kTextSpacingTrim,
    CSSPropertyID::kTextTransform,
    CSSPropertyID::kTextUnderlinePosition,
    CSSPropertyID::kTextWrapMode,
    CSSPropertyID::kTextWrapStyle,
    CSSPropertyID::kTimelineScope,
    CSSPropertyID::kTop,
    CSSPropertyID::kTouchAction,
    CSSPropertyID::kTransform,
    CSSPropertyID::kTransformOrigin,
    CSSPropertyID::kTransformStyle,
    CSSPropertyID::kTransitionBehavior,
    CSSPropertyID::kTransitionDelay,
    CSSPropertyID::kTransitionDuration,
    CSSPropertyID::kTransitionProperty,
    CSSPropertyID::kTransitionTimingFunction,
    CSSPropertyID::kTranslate,
    CSSPropertyID::kUnicodeBidi,
    CSSPropertyID::kUserSelect,
    CSSPropertyID::kVectorEffect,
    CSSPropertyID::kVerticalAlign,
    CSSPropertyID::kViewTimelineAxis,
    CSSPropertyID::kViewTimelineInset,
    CSSPropertyID::kViewTimelineName,
    CSSPropertyID::kViewTransitionCaptureMode,
    CSSPropertyID::kViewTransitionClass,
    CSSPropertyID::kViewTransitionGroup,
    CSSPropertyID::kViewTransitionName,
    CSSPropertyID::kVisibility,
    CSSPropertyID::kWhiteSpaceCollapse,
    CSSPropertyID::kWidows,
    CSSPropertyID::kWidth,
    CSSPropertyID::kWillChange,
    CSSPropertyID::kWordBreak,
    CSSPropertyID::kWordSpacing,
    CSSPropertyID::kWritingMode,
    CSSPropertyID::kX,
    CSSPropertyID::kY,
    CSSPropertyID::kZIndex,
    CSSPropertyID::kZoom,
    CSSPropertyID::kWebkitBorderHorizontalSpacing,
    CSSPropertyID::kWebkitBorderImage,
    CSSPropertyID::kWebkitBorderVerticalSpacing,
    CSSPropertyID::kWebkitBoxAlign,
    CSSPropertyID::kWebkitBoxDecorationBreak,
    CSSPropertyID::kWebkitBoxDirection,
    CSSPropertyID::kWebkitBoxFlex,
    CSSPropertyID::kWebkitBoxOrdinalGroup,
    CSSPropertyID::kWebkitBoxOrient,
    CSSPropertyID::kWebkitBoxPack,
    CSSPropertyID::kWebkitBoxReflect,
    CSSPropertyID::kWebkitFontSmoothing,
    CSSPropertyID::kWebkitLineBreak,
    CSSPropertyID::kWebkitLineClamp,
    CSSPropertyID::kWebkitLocale,
    CSSPropertyID::kWebkitMaskBoxImage,
    CSSPropertyID::kWebkitMaskBoxImageOutset,
    CSSPropertyID::kWebkitMaskBoxImageRepeat,
    CSSPropertyID::kWebkitMaskBoxImageSlice,
    CSSPropertyID::kWebkitMaskBoxImageSource,
    CSSPropertyID::kWebkitMaskBoxImageWidth,
    CSSPropertyID::kWebkitPrintColorAdjust,
    CSSPropertyID::kWebkitRtlOrdering,
    CSSPropertyID::kWebkitTapHighlightColor,
    CSSPropertyID::kWebkitTextCombine,
    CSSPropertyID::kWebkitTextDecorationsInEffect,
    CSSPropertyID::kWebkitTextFillColor,
    CSSPropertyID::kWebkitTextOrientation,
    CSSPropertyID::kWebkitTextSecurity,
    CSSPropertyID::kWebkitTextStrokeColor,
    CSSPropertyID::kWebkitTextStrokeWidth,
    CSSPropertyID::kWebkitUserDrag,
    CSSPropertyID::kWebkitUserModify,
    CSSPropertyID::kWebkitWritingMode,
};

// The lowest CSSPropertyID excluding kInvalid and kVariable.
const int kIntFirstCSSProperty = 2;
const CSSPropertyID kFirstCSSProperty =
    static_cast<CSSPropertyID>(kIntFirstCSSProperty);

// The number of unresolved CSS properties excluding kInvalid and kVariable.
const int kNumCSSProperties = 722;

// The highest resolved CSSPropertyID.
const int kIntLastCSSProperty = 605;
const CSSPropertyID kLastCSSProperty =
    static_cast<CSSPropertyID>(kIntLastCSSProperty);

// The highest unresolved CSSPropertyID.
const CSSPropertyID kLastUnresolvedCSSProperty =
    static_cast<CSSPropertyID>(723);

const CSSPropertyID kFirstHighPriorityCSSProperty = kFirstCSSProperty;
const CSSPropertyID kLastHighPriorityCSSProperty = CSSPropertyID::kZoom;

// 1 + <The highest unresolved CSSPropertyID>.
const int kNumCSSPropertyIDs = static_cast<int>(kLastUnresolvedCSSProperty) + 1;

const size_t kMaxCSSPropertyNameLength = 48;
constexpr size_t kCSSPropertyIDBitLength = 10;
constexpr size_t kMaxShorthandExpansion = 18;

static_assert((static_cast<size_t>(1) << kCSSPropertyIDBitLength) >
                  static_cast<size_t>(kLastUnresolvedCSSProperty),
              "kCSSPropertyIDBitLength has enough bits");

// These are basically just change-detector tests, so that we do not
// accidentally add new high-priority properties or break the code generator.
static_assert(CSSPropertyID::kColorScheme == kFirstHighPriorityCSSProperty);
static_assert(CSSPropertyID::kZoom == kLastHighPriorityCSSProperty);
static_assert((static_cast<int>(kLastHighPriorityCSSProperty) -
               static_cast<int>(kFirstHighPriorityCSSProperty)) == 41,
              "There should a low number of high-priority properties");

inline int GetCSSPropertyIDIndex(CSSPropertyID id) {
    DCHECK_GE(id, kFirstCSSProperty);
    DCHECK_LE(id, kLastCSSProperty);
    return static_cast<int>(id) - kIntFirstCSSProperty;
}

constexpr bool IsHighPriority(CSSPropertyID id) {
  return id >= kFirstHighPriorityCSSProperty &&
      id <= kLastHighPriorityCSSProperty;
}

inline bool IsCSSPropertyIDWithName(CSSPropertyID id)
{
    return id >= kFirstCSSProperty && id <= kLastUnresolvedCSSProperty;
}

inline bool IsValidCSSPropertyID(CSSPropertyID id)
{
    return id != CSSPropertyID::kInvalid;
}

inline CSSPropertyID ConvertToCSSPropertyID(int value)
{
    DCHECK_GE(value, static_cast<int>(CSSPropertyID::kInvalid));
    DCHECK_LE(value, kIntLastCSSProperty);
    return static_cast<CSSPropertyID>(value);
}

int CORE_EXPORT ResolveCSSPropertyAlias(int value);

inline bool IsPropertyAlias(CSSPropertyID id) {
  return static_cast<int>(id) >= 606;
}

inline CSSPropertyID ResolveCSSPropertyID(CSSPropertyID id)
{
  int int_id = static_cast<int>(id);
  if (IsPropertyAlias(id))
    int_id = ResolveCSSPropertyAlias(int_id);
  return ConvertToCSSPropertyID(int_id);
}

CSSPropertyID CORE_EXPORT CssPropertyID(const ExecutionContext*,
                                        const WTF::String&);

class CSSPropertyIDList {
  STACK_ALLOCATED();

 public:
  class Iterator {
    STACK_ALLOCATED();
   public:
    Iterator(int id) : id_(id) {}
    CSSPropertyID operator*() const { return ConvertToCSSPropertyID(id_); }
    Iterator& operator++() {
      id_++;
      return *this;
    }
    bool operator!=(const Iterator& i) const { return id_ != i.id_; }

   private:
    int id_;
  };
  Iterator begin() const { return Iterator(kIntFirstCSSProperty); }
  Iterator end() const { return Iterator(kIntLastCSSProperty + 1); }
};

mojom::blink::CSSSampleId CORE_EXPORT GetCSSSampleId(CSSPropertyID id);

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
