// Copyright 2014 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_

#include <string.h>
#include <stdint.h>

#include <string_view>

#include "base/containers/span.h"
#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/core/css/parser/css_parser_mode.h"

namespace blink {

enum class CSSValueID {
  kInvalid = 0,
  kInherit = 1,
  kInitial = 2,
  kNone = 3,
  kHidden = 4,
  kInset = 5,
  kGroove = 6,
  kOutset = 7,
  kRidge = 8,
  kDotted = 9,
  kDashed = 10,
  kSolid = 11,
  kDouble = 12,
  kCaption = 13,
  kIcon = 14,
  kMenu = 15,
  kMessageBox = 16,
  kSmallCaption = 17,
  kWebkitMiniControl = 18,
  kWebkitSmallControl = 19,
  kWebkitControl = 20,
  kStatusBar = 21,
  kItalic = 22,
  kOblique = 23,
  kAll = 24,
  kCommonLigatures = 25,
  kNoCommonLigatures = 26,
  kDiscretionaryLigatures = 27,
  kNoDiscretionaryLigatures = 28,
  kHistoricalLigatures = 29,
  kNoHistoricalLigatures = 30,
  kContextual = 31,
  kNoContextual = 32,
  kSmallCaps = 33,
  kAllSmallCaps = 34,
  kPetiteCaps = 35,
  kAllPetiteCaps = 36,
  kUnicase = 37,
  kTitlingCaps = 38,
  kJis78 = 39,
  kJis83 = 40,
  kJis90 = 41,
  kJis04 = 42,
  kSimplified = 43,
  kTraditional = 44,
  kFullWidth = 45,
  kProportionalWidth = 46,
  kRuby = 47,
  kLiningNums = 48,
  kOldstyleNums = 49,
  kProportionalNums = 50,
  kTabularNums = 51,
  kDiagonalFractions = 52,
  kStackedFractions = 53,
  kOrdinal = 54,
  kSlashedZero = 55,
  kStylistic = 56,
  kHistoricalForms = 57,
  kStyleset = 58,
  kCharacterVariant = 59,
  kSwash = 60,
  kOrnaments = 61,
  kAnnotation = 62,
  kNormal = 63,
  kBold = 64,
  kBolder = 65,
  kLighter = 66,
  kWeight = 67,
  kUltraCondensed = 68,
  kExtraCondensed = 69,
  kCondensed = 70,
  kSemiCondensed = 71,
  kSemiExpanded = 72,
  kExpanded = 73,
  kExtraExpanded = 74,
  kUltraExpanded = 75,
  kXxSmall = 76,
  kXSmall = 77,
  kSmall = 78,
  kMedium = 79,
  kLarge = 80,
  kXLarge = 81,
  kXxLarge = 82,
  kXxxLarge = 83,
  kSmaller = 84,
  kLarger = 85,
  kWebkitXxxLarge = 86,
  kExHeight = 87,
  kCapHeight = 88,
  kChWidth = 89,
  kIcWidth = 90,
  kIcHeight = 91,
  kSerif = 92,
  kSansSerif = 93,
  kCursive = 94,
  kFantasy = 95,
  kMonospace = 96,
  kSystemUi = 97,
  kWebkitBody = 98,
  kMath = 99,
  kSwap = 100,
  kFallback = 101,
  kOptional = 102,
  kFontTech = 103,
  kFontFormat = 104,
  kEmoji = 105,
  kUnicode = 106,
  kPaletteMix = 107,
  kAqua = 108,
  kBlack = 109,
  kBlue = 110,
  kFuchsia = 111,
  kGray = 112,
  kGreen = 113,
  kLime = 114,
  kMaroon = 115,
  kNavy = 116,
  kOlive = 117,
  kOrange = 118,
  kPurple = 119,
  kRed = 120,
  kSilver = 121,
  kTeal = 122,
  kWhite = 123,
  kYellow = 124,
  kTransparent = 125,
  kWebkitLink = 126,
  kWebkitActivelink = 127,
  kAccentcolor = 128,
  kAccentcolortext = 129,
  kActiveborder = 130,
  kActivecaption = 131,
  kActivetext = 132,
  kAppworkspace = 133,
  kBackground = 134,
  kButtonborder = 135,
  kButtonface = 136,
  kButtonhighlight = 137,
  kButtonshadow = 138,
  kButtontext = 139,
  kCanvas = 140,
  kCanvastext = 141,
  kCaptiontext = 142,
  kField = 143,
  kFieldtext = 144,
  kGraytext = 145,
  kHighlight = 146,
  kHighlighttext = 147,
  kInactiveborder = 148,
  kInactivecaption = 149,
  kInactivecaptiontext = 150,
  kInfobackground = 151,
  kInfotext = 152,
  kLinktext = 153,
  kMark = 154,
  kMarktext = 155,
  kMenutext = 156,
  kSelecteditem = 157,
  kSelecteditemtext = 158,
  kScrollbar = 159,
  kThreeddarkshadow = 160,
  kThreedface = 161,
  kThreedhighlight = 162,
  kThreedlightshadow = 163,
  kThreedshadow = 164,
  kVisitedtext = 165,
  kWindow = 166,
  kWindowframe = 167,
  kWindowtext = 168,
  kInternalActiveListBoxSelection = 169,
  kInternalActiveListBoxSelectionText = 170,
  kInternalInactiveListBoxSelection = 171,
  kInternalInactiveListBoxSelectionText = 172,
  kWebkitFocusRingColor = 173,
  kCurrentcolor = 174,
  kGrey = 175,
  kInternalQuirkInherit = 176,
  kInternalSpellingErrorColor = 177,
  kInternalGrammarErrorColor = 178,
  kInternalSearchColor = 179,
  kInternalSearchTextColor = 180,
  kInternalCurrentSearchColor = 181,
  kInternalCurrentSearchTextColor = 182,
  kAliceblue = 183,
  kAntiquewhite = 184,
  kAquamarine = 185,
  kAzure = 186,
  kBeige = 187,
  kBisque = 188,
  kBlanchedalmond = 189,
  kBlueviolet = 190,
  kBrown = 191,
  kBurlywood = 192,
  kCadetblue = 193,
  kChartreuse = 194,
  kChocolate = 195,
  kCoral = 196,
  kCornflowerblue = 197,
  kCornsilk = 198,
  kCrimson = 199,
  kCyan = 200,
  kDarkblue = 201,
  kDarkcyan = 202,
  kDarkgoldenrod = 203,
  kDarkgray = 204,
  kDarkgreen = 205,
  kDarkgrey = 206,
  kDarkkhaki = 207,
  kDarkmagenta = 208,
  kDarkolivegreen = 209,
  kDarkorange = 210,
  kDarkorchid = 211,
  kDarkred = 212,
  kDarksalmon = 213,
  kDarkseagreen = 214,
  kDarkslateblue = 215,
  kDarkslategray = 216,
  kDarkslategrey = 217,
  kDarkturquoise = 218,
  kDarkviolet = 219,
  kDeeppink = 220,
  kDeepskyblue = 221,
  kDimgray = 222,
  kDimgrey = 223,
  kDodgerblue = 224,
  kFirebrick = 225,
  kFloralwhite = 226,
  kForestgreen = 227,
  kGainsboro = 228,
  kGhostwhite = 229,
  kGold = 230,
  kGoldenrod = 231,
  kGreenyellow = 232,
  kHoneydew = 233,
  kHotpink = 234,
  kIndianred = 235,
  kIndigo = 236,
  kIvory = 237,
  kKhaki = 238,
  kLavender = 239,
  kLavenderblush = 240,
  kLawngreen = 241,
  kLemonchiffon = 242,
  kLightblue = 243,
  kLightcoral = 244,
  kLightcyan = 245,
  kLightgoldenrodyellow = 246,
  kLightgray = 247,
  kLightgreen = 248,
  kLightgrey = 249,
  kLightpink = 250,
  kLightsalmon = 251,
  kLightseagreen = 252,
  kLightskyblue = 253,
  kLightslategray = 254,
  kLightslategrey = 255,
  kLightsteelblue = 256,
  kLightyellow = 257,
  kLimegreen = 258,
  kLinen = 259,
  kMagenta = 260,
  kMediumaquamarine = 261,
  kMediumblue = 262,
  kMediumorchid = 263,
  kMediumpurple = 264,
  kMediumseagreen = 265,
  kMediumslateblue = 266,
  kMediumspringgreen = 267,
  kMediumturquoise = 268,
  kMediumvioletred = 269,
  kMidnightblue = 270,
  kMintcream = 271,
  kMistyrose = 272,
  kMoccasin = 273,
  kNavajowhite = 274,
  kOldlace = 275,
  kOlivedrab = 276,
  kOrangered = 277,
  kOrchid = 278,
  kPalegoldenrod = 279,
  kPalegreen = 280,
  kPaleturquoise = 281,
  kPalevioletred = 282,
  kPapayawhip = 283,
  kPeachpuff = 284,
  kPeru = 285,
  kPink = 286,
  kPlum = 287,
  kPowderblue = 288,
  kRebeccapurple = 289,
  kRosybrown = 290,
  kRoyalblue = 291,
  kSaddlebrown = 292,
  kSalmon = 293,
  kSandybrown = 294,
  kSeagreen = 295,
  kSeashell = 296,
  kSienna = 297,
  kSkyblue = 298,
  kSlateblue = 299,
  kSlategray = 300,
  kSlategrey = 301,
  kSnow = 302,
  kSpringgreen = 303,
  kSteelblue = 304,
  kTan = 305,
  kThistle = 306,
  kTomato = 307,
  kTurquoise = 308,
  kViolet = 309,
  kWheat = 310,
  kWhitesmoke = 311,
  kYellowgreen = 312,
  kRepeat = 313,
  kRepeatX = 314,
  kRepeatY = 315,
  kNoRepeat = 316,
  kClear = 317,
  kCopy = 318,
  kSourceOver = 319,
  kSourceIn = 320,
  kSourceOut = 321,
  kSourceAtop = 322,
  kDestinationOver = 323,
  kDestinationIn = 324,
  kDestinationOut = 325,
  kDestinationAtop = 326,
  kXor = 327,
  kPlusLighter = 328,
  kSubtract = 329,
  kIntersect = 330,
  kExclude = 331,
  kBaseline = 332,
  kMiddle = 333,
  kSub = 334,
  kSuper = 335,
  kTextTop = 336,
  kTextBottom = 337,
  kTop = 338,
  kBottom = 339,
  kWebkitBaselineMiddle = 340,
  kWebkitAuto = 341,
  kLeft = 342,
  kRight = 343,
  kCenter = 344,
  kJustify = 345,
  kWebkitLeft = 346,
  kWebkitRight = 347,
  kWebkitCenter = 348,
  kWebkitMatchParent = 349,
  kInternalCenter = 350,
  kInlineStart = 351,
  kInlineEnd = 352,
  kOutside = 353,
  kInside = 354,
  kDisc = 355,
  kCircle = 356,
  kSquare = 357,
  kDisclosureOpen = 358,
  kDisclosureClosed = 359,
  kDecimal = 360,
  kInline = 361,
  kBlock = 362,
  kFlowRoot = 363,
  kFlow = 364,
  kTable = 365,
  kFlex = 366,
  kGrid = 367,
  kMasonry = 368,
  kContents = 369,
  kTableRowGroup = 370,
  kTableHeaderGroup = 371,
  kTableFooterGroup = 372,
  kTableRow = 373,
  kTableColumnGroup = 374,
  kTableColumn = 375,
  kTableCell = 376,
  kTableCaption = 377,
  kRubyText = 378,
  kInlineBlock = 379,
  kInlineTable = 380,
  kInlineFlex = 381,
  kInlineGrid = 382,
  kInlineMasonry = 383,
  kWebkitBox = 384,
  kWebkitInlineBox = 385,
  kWebkitFlex = 386,
  kWebkitInlineFlex = 387,
  kLayout = 388,
  kInlineLayout = 389,
  kListItem = 390,
  kAuto = 391,
  kCrosshair = 392,
  kDefault = 393,
  kPointer = 394,
  kMove = 395,
  kVerticalText = 396,
  kCell = 397,
  kContextMenu = 398,
  kAlias = 399,
  kProgress = 400,
  kNoDrop = 401,
  kNotAllowed = 402,
  kZoomIn = 403,
  kZoomOut = 404,
  kEResize = 405,
  kNeResize = 406,
  kNwResize = 407,
  kNResize = 408,
  kSeResize = 409,
  kSwResize = 410,
  kSResize = 411,
  kWResize = 412,
  kEwResize = 413,
  kNsResize = 414,
  kNeswResize = 415,
  kNwseResize = 416,
  kColResize = 417,
  kRowResize = 418,
  kText = 419,
  kWait = 420,
  kHelp = 421,
  kAllScroll = 422,
  kGrab = 423,
  kGrabbing = 424,
  kWebkitGrab = 425,
  kWebkitGrabbing = 426,
  kWebkitZoomIn = 427,
  kWebkitZoomOut = 428,
  kLtr = 429,
  kRtl = 430,
  kCapitalize = 431,
  kUppercase = 432,
  kLowercase = 433,
  kMathAuto = 434,
  kVisible = 435,
  kCollapse = 436,
  kPreserve = 437,
  kPreserveBreaks = 438,
  kPretty = 439,
  kA3 = 440,
  kA4 = 441,
  kA5 = 442,
  kAbove = 443,
  kAbsolute = 444,
  kAlways = 445,
  kAvoid = 446,
  kB4 = 447,
  kB5 = 448,
  kBelow = 449,
  kBidiOverride = 450,
  kBlink = 451,
  kBoth = 452,
  kBreakSpaces = 453,
  kCloseQuote = 454,
  kEmbed = 455,
  kFixed = 456,
  kHand = 457,
  kHide = 458,
  kIsolate = 459,
  kIsolateOverride = 460,
  kPlaintext = 461,
  kWebkitIsolate = 462,
  kWebkitIsolateOverride = 463,
  kWebkitPlaintext = 464,
  kJisB5 = 465,
  kJisB4 = 466,
  kLandscape = 467,
  kLedger = 468,
  kLegal = 469,
  kLetter = 470,
  kLineThrough = 471,
  kLocal = 472,
  kNoCloseQuote = 473,
  kNoOpenQuote = 474,
  kNowrap = 475,
  kOpenQuote = 476,
  kOverlay = 477,
  kOverline = 478,
  kPortrait = 479,
  kPre = 480,
  kPreLine = 481,
  kPreWrap = 482,
  kRelative = 483,
  kScroll = 484,
  kSeparate = 485,
  kShow = 486,
  kStatic = 487,
  kThick = 488,
  kThin = 489,
  kUnderline = 490,
  kView = 491,
  kWavy = 492,
  kCompact = 493,
  kStretch = 494,
  kStart = 495,
  kEnd = 496,
  kClone = 497,
  kSlice = 498,
  kReverse = 499,
  kHorizontal = 500,
  kVertical = 501,
  kInlineAxis = 502,
  kBlockAxis = 503,
  kFlexStart = 504,
  kFlexEnd = 505,
  kSpaceBetween = 506,
  kSpaceAround = 507,
  kSpaceEvenly = 508,
  kUnsafe = 509,
  kSafe = 510,
  kAnchorCenter = 511,
  kRow = 512,
  kRowReverse = 513,
  kColumn = 514,
  kColumnReverse = 515,
  kWrap = 516,
  kWrapReverse = 517,
  kAutoFlow = 518,
  kDense = 519,
  kReadOnly = 520,
  kReadWrite = 521,
  kReadWritePlaintextOnly = 522,
  kElement = 523,
  kWebkitMinContent = 524,
  kWebkitMaxContent = 525,
  kWebkitFillAvailable = 526,
  kWebkitFitContent = 527,
  kMinContent = 528,
  kMaxContent = 529,
  kFitContent = 530,
  kNoAutospace = 531,
  kCap = 532,
  kEx = 533,
  kTrimBoth = 534,
  kTrimEnd = 535,
  kTrimStart = 536,
  kClip = 537,
  kEllipsis = 538,
  kSpellingError = 539,
  kGrammarError = 540,
  kFromFont = 541,
  kSpaceAll = 542,
  kSpaceFirst = 543,
  kBreakAll = 544,
  kKeepAll = 545,
  kAutoPhrase = 546,
  kBreakWord = 547,
  kSpace = 548,
  kLoose = 549,
  kStrict = 550,
  kAfterWhiteSpace = 551,
  kAnywhere = 552,
  kManual = 553,
  kCheckbox = 554,
  kRadio = 555,
  kButton = 556,
  kListbox = 557,
  kInternalMediaControl = 558,
  kMenulist = 559,
  kMenulistButton = 560,
  kMeter = 561,
  kProgressBar = 562,
  kSearchfield = 563,
  kTextfield = 564,
  kTextarea = 565,
  kSliderVertical = 566,
  kRound = 567,
  kBaseSelect = 568,
  kInternalAutoBase = 569,
  kBorder = 570,
  kBorderBox = 571,
  kContent = 572,
  kContentBox = 573,
  kPadding = 574,
  kPaddingBox = 575,
  kMarginBox = 576,
  kNoClip = 577,
  kContain = 578,
  kCover = 579,
  kLogical = 580,
  kVisual = 581,
  kReplace = 582,
  kAccumulate = 583,
  kAlternate = 584,
  kAlternateReverse = 585,
  kForwards = 586,
  kBackwards = 587,
  kInfinite = 588,
  kRunning = 589,
  kPaused = 590,
  kFlat = 591,
  kPreserve3d = 592,
  kFillBox = 593,
  kViewBox = 594,
  kEase = 595,
  kLinear = 596,
  kEaseIn = 597,
  kEaseOut = 598,
  kEaseInOut = 599,
  kJumpBoth = 600,
  kJumpEnd = 601,
  kJumpNone = 602,
  kJumpStart = 603,
  kStepStart = 604,
  kStepEnd = 605,
  kSteps = 606,
  kFrames = 607,
  kCubicBezier = 608,
  kDocument = 609,
  kReset = 610,
  kZoom = 611,
  kVisiblepainted = 612,
  kVisiblefill = 613,
  kVisiblestroke = 614,
  kPainted = 615,
  kFill = 616,
  kStroke = 617,
  kBoundingBox = 618,
  kSpellOut = 619,
  kDigits = 620,
  kLiteralPunctuation = 621,
  kNoPunctuation = 622,
  kAntialiased = 623,
  kSubpixelAntialiased = 624,
  kOptimizespeed = 625,
  kOptimizelegibility = 626,
  kGeometricprecision = 627,
  kCrispedges = 628,
  kEconomy = 629,
  kExact = 630,
  kLr = 631,
  kRl = 632,
  kTb = 633,
  kLrTb = 634,
  kRlTb = 635,
  kTbRl = 636,
  kHorizontalTb = 637,
  kVerticalRl = 638,
  kVerticalLr = 639,
  kSidewaysRl = 640,
  kSidewaysLr = 641,
  kAfter = 642,
  kBefore = 643,
  kInterCharacter = 644,
  kOver = 645,
  kUnder = 646,
  kFilled = 647,
  kOpen = 648,
  kDot = 649,
  kDoubleCircle = 650,
  kTriangle = 651,
  kSesame = 652,
  kEllipse = 653,
  kClosestSide = 654,
  kClosestCorner = 655,
  kFarthestSide = 656,
  kFarthestCorner = 657,
  kMixed = 658,
  kSideways = 659,
  kSidewaysRight = 660,
  kUpright = 661,
  kVerticalRight = 662,
  kOn = 663,
  kOff = 664,
  kOptimizequality = 665,
  kPixelated = 666,
  kWebkitOptimizeContrast = 667,
  kFromImage = 668,
  kRotateLeft = 669,
  kRotateRight = 670,
  kNonzero = 671,
  kEvenodd = 672,
  kAt = 673,
  kAlphabetic = 674,
  kBorderless = 675,
  kFullscreen = 676,
  kStandalone = 677,
  kMinimalUi = 678,
  kBrowser = 679,
  kWindowControlsOverlay = 680,
  kTabbed = 681,
  kPictureInPicture = 682,
  kMinimized = 683,
  kMaximized = 684,
  kPaged = 685,
  kSlow = 686,
  kFast = 687,
  kSticky = 688,
  kCoarse = 689,
  kFine = 690,
  kOnDemand = 691,
  kHover = 692,
  kMultiply = 693,
  kScreen = 694,
  kDarken = 695,
  kLighten = 696,
  kColorDodge = 697,
  kColorBurn = 698,
  kHardLight = 699,
  kSoftLight = 700,
  kDifference = 701,
  kExclusion = 702,
  kHue = 703,
  kSaturation = 704,
  kColor = 705,
  kLuminosity = 706,
  kScaleDown = 707,
  kBalance = 708,
  kDrag = 709,
  kNoDrag = 710,
  kSpan = 711,
  kMinmax = 712,
  kSubgrid = 713,
  kProgressive = 714,
  kInterlace = 715,
  kMarkers = 716,
  kAlpha = 717,
  kLuminance = 718,
  kMatchSource = 719,
  kSRGB = 720,
  kLinearrgb = 721,
  kButt = 722,
  kMiter = 723,
  kBevel = 724,
  kBeforeEdge = 725,
  kAfterEdge = 726,
  kCentral = 727,
  kTextBeforeEdge = 728,
  kTextAfterEdge = 729,
  kIdeographic = 730,
  kHanging = 731,
  kMathematical = 732,
  kIntersection = 733,
  kSpanningItem = 734,
  kUseScript = 735,
  kNoChange = 736,
  kResetSize = 737,
  kDynamic = 738,
  kNonScalingStroke = 739,
  kInternalExtendToZoom = 740,
  kPanX = 741,
  kPanY = 742,
  kPanLeft = 743,
  kPanRight = 744,
  kPanUp = 745,
  kPanDown = 746,
  kManipulation = 747,
  kPinchZoom = 748,
  kLastBaseline = 749,
  kFirstBaseline = 750,
  kFirst = 751,
  kLast = 752,
  kSelfStart = 753,
  kSelfEnd = 754,
  kLegacy = 755,
  kSmooth = 756,
  kScrollPosition = 757,
  kRevert = 758,
  kRevertLayer = 759,
  kUnset = 760,
  kLinearGradient = 761,
  kRadialGradient = 762,
  kConicGradient = 763,
  kRepeatingLinearGradient = 764,
  kRepeatingRadialGradient = 765,
  kRepeatingConicGradient = 766,
  kPaint = 767,
  kCrossFade = 768,
  kWebkitCrossFade = 769,
  kWebkitGradient = 770,
  kWebkitLinearGradient = 771,
  kWebkitRadialGradient = 772,
  kWebkitRepeatingLinearGradient = 773,
  kWebkitRepeatingRadialGradient = 774,
  kWebkitImageSet = 775,
  kImageSet = 776,
  kType = 777,
  kTo = 778,
  kColorStop = 779,
  kRadial = 780,
  kAttr = 781,
  kCounter = 782,
  kCounters = 783,
  kRect = 784,
  kPolygon = 785,
  kShape = 786,
  kBy = 787,
  kLine = 788,
  kClose = 789,
  kHline = 790,
  kVline = 791,
  kArc = 792,
  kCw = 793,
  kCcw = 794,
  kCurve = 795,
  kOrigin = 796,
  kWith = 797,
  kFormat = 798,
  kCollection = 799,
  kEmbeddedOpentype = 800,
  kOpentype = 801,
  kSVG = 802,
  kTruetype = 803,
  kWoff = 804,
  kWoff2 = 805,
  kTech = 806,
  kFeaturesOpentype = 807,
  kFeaturesAat = 808,
  kFeaturesGraphite = 809,
  kColorCOLRv0 = 810,
  kColorCOLRv1 = 811,
  kColorSVG = 812,
  kColorSbix = 813,
  kColorCBDT = 814,
  kVariations = 815,
  kPalettes = 816,
  kIncremental = 817,
  kInvert = 818,
  kGrayscale = 819,
  kSepia = 820,
  kSaturate = 821,
  kHueRotate = 822,
  kOpacity = 823,
  kBrightness = 824,
  kContrast = 825,
  kBlur = 826,
  kDropShadow = 827,
  kUrl = 828,
  kRgb = 829,
  kRgba = 830,
  kHsl = 831,
  kHsla = 832,
  kHwb = 833,
  kLab = 834,
  kOklab = 835,
  kLch = 836,
  kOklch = 837,
  kLightDark = 838,
  kSRGBLinear = 839,
  kDisplayP3 = 840,
  kA98Rgb = 841,
  kProphotoRgb = 842,
  kXyz = 843,
  kXyzD50 = 844,
  kXyzD65 = 845,
  kShorter = 846,
  kLonger = 847,
  kDecreasing = 848,
  kIncreasing = 849,
  kIn = 850,
  kColorMix = 851,
  kFrom = 852,
  kR = 853,
  kG = 854,
  kB = 855,
  kH = 856,
  kS = 857,
  kL = 858,
  kW = 859,
  kA = 860,
  kC = 861,
  kMatrix = 862,
  kMatrix3d = 863,
  kPerspective = 864,
  kRotate = 865,
  kRotateX = 866,
  kRotateY = 867,
  kRotateZ = 868,
  kRotate3d = 869,
  kScale = 870,
  kScaleX = 871,
  kScaleY = 872,
  kScaleZ = 873,
  kScale3d = 874,
  kSkew = 875,
  kSkewX = 876,
  kSkewY = 877,
  kTranslate = 878,
  kTranslateX = 879,
  kTranslateY = 880,
  kTranslateZ = 881,
  kTranslate3d = 882,
  kX = 883,
  kY = 884,
  kZ = 885,
  kPath = 886,
  kRay = 887,
  kSides = 888,
  kStrokeBox = 889,
  kCalc = 890,
  kWebkitCalc = 891,
  kMin = 892,
  kMax = 893,
  kClamp = 894,
  kCalcSize = 895,
  kAny = 896,
  kSin = 897,
  kCos = 898,
  kAsin = 899,
  kAtan = 900,
  kAtan2 = 901,
  kAcos = 902,
  kMod = 903,
  kRem = 904,
  kUp = 905,
  kDown = 906,
  kToZero = 907,
  kSign = 908,
  kAbs = 909,
  kPow = 910,
  kSqrt = 911,
  kHypot = 912,
  kLog = 913,
  kExp = 914,
  kInfinity = 915,
  kNegativeInfinity = 916,
  kNan = 917,
  kPi = 918,
  kE = 919,
  kMandatory = 920,
  kProximity = 921,
  kStyle = 922,
  kSize = 923,
  kBlockSize = 924,
  kInlineSize = 925,
  kScrollState = 926,
  kBlockStart = 927,
  kBlockEnd = 928,
  kAutoFill = 929,
  kAutoFit = 930,
  kVar = 931,
  kInternalVariableValue = 932,
  kEnv = 933,
  kArg = 934,
  kAvoidPage = 935,
  kPage = 936,
  kRecto = 937,
  kVerso = 938,
  kAvoidColumn = 939,
  kP3 = 940,
  kRec2020 = 941,
  kAdd = 942,
  kAutoAdd = 943,
  kTrue = 944,
  kFalse = 945,
  kNoPreference = 946,
  kDark = 947,
  kLight = 948,
  kOnly = 949,
  kReduce = 950,
  kActive = 951,
  kPreserveParentColor = 952,
  kBackButton = 953,
  kFabricated = 954,
  kSelector = 955,
  kAtRule = 956,
  kContinuous = 957,
  kFolded = 958,
  kStable = 959,
  kBothEdges = 960,
  kMore = 961,
  kLess = 962,
  kCustom = 963,
  kCyclic = 964,
  kSymbolic = 965,
  kNumeric = 966,
  kAdditive = 967,
  kExtends = 968,
  kInternalSimpChineseInformal = 969,
  kInternalSimpChineseFormal = 970,
  kInternalTradChineseInformal = 971,
  kInternalTradChineseFormal = 972,
  kInternalKoreanHangulFormal = 973,
  kInternalKoreanHanjaInformal = 974,
  kInternalKoreanHanjaFormal = 975,
  kInternalHebrew = 976,
  kInternalLowerArmenian = 977,
  kInternalUpperArmenian = 978,
  kInternalEthiopicNumeric = 979,
  kBullets = 980,
  kNumbers = 981,
  kWords = 982,
  kStandard = 983,
  kHigh = 984,
  kConstrainedHigh = 985,
  kDynamicRangeLimitMix = 986,
  kLayer = 987,
  kSupports = 988,
  kScope = 989,
  kColorContrast = 990,
  kVs = 991,
  kAA = 992,
  kAALarge = 993,
  kAAA = 994,
  kAAALarge = 995,
  kDrop = 996,
  kRaise = 997,
  kXywh = 998,
  kAnchor = 999,
  kAnchorSize = 1000,
  kWidth = 1001,
  kHeight = 1002,
  kSelfBlock = 1003,
  kSelfInline = 1004,
  kEntry = 1005,
  kExit = 1006,
  kEntryCrossing = 1007,
  kExitCrossing = 1008,
  kRoot = 1009,
  kNearest = 1010,
  kSelf = 1011,
  kAllowDiscrete = 1012,
  kInverted = 1013,
  kLayered = 1014,
  kMatchElement = 1015,
  kEnabled = 1016,
  kInitialOnly = 1017,
  kSpanLeft = 1018,
  kSpanRight = 1019,
  kXStart = 1020,
  kXEnd = 1021,
  kSpanXStart = 1022,
  kSpanXEnd = 1023,
  kXSelfStart = 1024,
  kXSelfEnd = 1025,
  kSpanXSelfStart = 1026,
  kSpanXSelfEnd = 1027,
  kSpanAll = 1028,
  kSpanTop = 1029,
  kSpanBottom = 1030,
  kYStart = 1031,
  kYEnd = 1032,
  kSpanYStart = 1033,
  kSpanYEnd = 1034,
  kYSelfStart = 1035,
  kYSelfEnd = 1036,
  kSpanYSelfStart = 1037,
  kSpanYSelfEnd = 1038,
  kSpanBlockStart = 1039,
  kSpanBlockEnd = 1040,
  kSelfBlockStart = 1041,
  kSelfBlockEnd = 1042,
  kSpanSelfBlockStart = 1043,
  kSpanSelfBlockEnd = 1044,
  kSpanInlineStart = 1045,
  kSpanInlineEnd = 1046,
  kSelfInlineStart = 1047,
  kSelfInlineEnd = 1048,
  kSpanSelfInlineStart = 1049,
  kSpanSelfInlineEnd = 1050,
  kSpanStart = 1051,
  kSpanEnd = 1052,
  kSpanSelfStart = 1053,
  kSpanSelfEnd = 1054,
  kPositionArea = 1055,
  kInternalTextareaAuto = 1056,
  kMostWidth = 1057,
  kMostHeight = 1058,
  kMostBlockSize = 1059,
  kMostInlineSize = 1060,
  kFlipBlock = 1061,
  kFlipInline = 1062,
  kFlipStart = 1063,
  kAnchorsVisible = 1064,
  kNoOverflow = 1065,
  kFlexVisual = 1066,
  kFlexFlow = 1067,
  kGridRows = 1068,
  kGridColumns = 1069,
  kGridOrder = 1070,
  kContextFill = 1071,
  kContextStroke = 1072,
  kMediaProgress = 1073,
  kContainerProgress = 1074,
  kOf = 1075,
  kBlinkFeature = 1076,
  kNumericOnly = 1077,
  kAllowKeywords = 1078,
  kInert = 1079,
  kSiblingCount = 1080,
  kSiblingIndex = 1081,
  kReturns = 1082,
  kOnce = 1083,
  kState = 1084,
  kIf = 1085,
  kElse = 1086,
};

constexpr int kNumCSSValueKeywords = 1087;
constexpr size_t kMaxCSSValueKeywordLength = 42;

inline bool IsValidCSSValueID(CSSValueID id)
{
    return id != CSSValueID::kInvalid;
}

CORE_EXPORT std::string_view GetCSSValueName(CSSValueID);

template <typename StringType>
inline StringType GetCSSValueNameAs(CSSValueID value_id) {
  return StringType(base::as_byte_span(GetCSSValueName(value_id)));
}

bool IsValueAllowedInMode(CSSValueID id, CSSParserMode mode);

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
