// Copyright 2021 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_ANIMATION_CSS_INTRINSIC_LENGTH_INTERPOLATION_TYPE_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_ANIMATION_CSS_INTRINSIC_LENGTH_INTERPOLATION_TYPE_H_

#include <optional>

#include "third_party/blink/renderer/core/animation/css_interpolation_type.h"
#include "third_party/blink/renderer/core/css_value_keywords.h"

namespace blink {

class StyleIntrinsicLength;

class CSSIntrinsicLengthInterpolationType : public CSSInterpolationType {
 public:
  explicit CSSIntrinsicLengthInterpolationType(PropertyHandle property)
      : CSSInterpolationType(property) {
    DCHECK(property.GetCSSProperty().PropertyID() ==
               CSSPropertyID::kContainIntrinsicWidth ||
           property.GetCSSProperty().PropertyID() ==
               CSSPropertyID::kContainIntrinsicHeight);
  }

  InterpolationValue MaybeConvertStandardPropertyUnderlyingValue(
      const ComputedStyle&) const final;
  PairwiseInterpolationValue MaybeMergeSingles(
      InterpolationValue&& start,
      InterpolationValue&& end) const final;
  void ApplyStandardPropertyValue(const InterpolableValue&,
                                  const NonInterpolableValue*,
                                  StyleResolverState&) const final;
  void Composite(UnderlyingValueOwner&,
                 double underlying_fraction,
                 const InterpolationValue&,
                 double interpolation_fraction) const final;

  static InterpolableValue* CreateInterpolableIntrinsicDimension(
      const StyleIntrinsicLength&);

 private:
  StyleIntrinsicLength GetIntrinsicDimension(const ComputedStyle&) const;
  void SetIntrinsicDimension(ComputedStyleBuilder&,
                             const StyleIntrinsicLength& dimension) const;

  InterpolationValue MaybeConvertNeutral(const InterpolationValue& underlying,
                                         ConversionCheckers&) const final;
  InterpolationValue MaybeConvertInitial(const StyleResolverState&,
                                         ConversionCheckers&) const final;
  InterpolationValue MaybeConvertInherit(const StyleResolverState&,
                                         ConversionCheckers&) const final;
  InterpolationValue MaybeConvertValue(const CSSValue&,
                                       const StyleResolverState*,
                                       ConversionCheckers&) const final;
};

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_ANIMATION_CSS_INTRINSIC_LENGTH_INTERPOLATION_TYPE_H_
