// Copyright 2021 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include "third_party/blink/renderer/core/css/style_recalc_context.h"

#include "third_party/blink/renderer/core/display_lock/display_lock_context.h"
#include "third_party/blink/renderer/core/dom/flat_tree_traversal.h"
#include "third_party/blink/renderer/core/style/computed_style.h"

namespace blink {

StyleRecalcContext StyleRecalcContext::FromInclusiveAncestors(
    Element& start_element) {
  StyleRecalcContext result;
  bool have_container_result = false;
  for (auto* element = &start_element; element;
       element = FlatTreeTraversal::ParentElement(*element)) {
    const ComputedStyle* style = element->GetComputedStyle();
    if (!have_container_result) {
      if (!style) {
        // TODO(crbug.com/40250356): Eliminate all invalid calls to
        // StyleRecalcContext::From[Inclusive]Ancestors, then either turn
        // if (!style) into CHECK(style) or simplify into checking:
        // element->GetComputedStyle()->IsContainerForSizeContainerQueries()
        //
        // This used to use base::debug::DumpWithoutCrashing() but generated too
        // many failures in the wild to keep around (would upload too many crash
        // reports). Consider adding UMA stats back if we want to track this or
        // land a strategy to figure it out and fix what's going on.
        have_container_result = true;
      } else if (style->IsContainerForSizeContainerQueries()) {
        result.container = element;
        have_container_result = true;
      }
    }

    if (auto* display_lock_context = element->GetDisplayLockContext()) {
      if (display_lock_context->IsAuto() && display_lock_context->IsLocked()) {
        result.has_content_visibility_auto_locked_ancestor = true;
      }
    }

    if (have_container_result &&
        result.has_content_visibility_auto_locked_ancestor) {
      break;
    }
  }
  return result;
}

StyleRecalcContext StyleRecalcContext::FromAncestors(Element& element) {
  // TODO(crbug.com/1145970): Avoid this work if we're not inside a container
  if (Element* parent = FlatTreeTraversal::ParentElement(element)) {
    return FromInclusiveAncestors(*parent);
  }
  return StyleRecalcContext();
}

}  // namespace blink
