// Copyright 2022 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include "chrome/common/extensions/api/generated_schemas.h"

#include <algorithm>
#include <iterator>
#include <string_view>

#include "base/containers/fixed_flat_map.h"

namespace {
constexpr char kCookies[] = R"R({"namespace":"cookies","types":[{"id":"cookies.SameSiteStatus","type":"string","enum":["no_restriction","lax","strict","unspecified"]},{"id":"cookies.CookiePartitionKey","type":"object","properties":{"topLevelSite":{"type":"string","optional":true},"hasCrossSiteAncestor":{"type":"boolean","optional":true}}},{"id":"cookies.Cookie","type":"object","properties":{"name":{"type":"string"},"value":{"type":"string"},"domain":{"type":"string"},"hostOnly":{"type":"boolean"},"path":{"type":"string"},"secure":{"type":"boolean"},"httpOnly":{"type":"boolean"},"sameSite":{"$ref":"cookies.SameSiteStatus"},"session":{"type":"boolean"},"expirationDate":{"type":"number","optional":true},"storeId":{"type":"string"},"partitionKey":{"$ref":"cookies.CookiePartitionKey","optional":true}}},{"id":"cookies.CookieStore","type":"object","properties":{"id":{"type":"string"},"tabIds":{"type":"array","items":{"type":"integer"}}}},{"id":"cookies.OnChangedCause","type":"string","enum":["evicted","expired","explicit","expired_overwrite","overwrite"]},{"id":"cookies.CookieDetails","type":"object","properties":{"url":{"type":"string"},"name":{"type":"string"},"storeId":{"type":"string","optional":true},"partitionKey":{"$ref":"cookies.CookiePartitionKey","optional":true}}},{"id":"cookies.FrameDetails","type":"object","properties":{"tabId":{"type":"integer","optional":true},"frameId":{"type":"integer","optional":true},"documentId":{"type":"string","optional":true}}}],"functions":[{"name":"get","type":"function","parameters":[{"name":"details","$ref":"cookies.CookieDetails"}],"returns_async":{"name":"callback","parameters":[{"name":"cookie","$ref":"cookies.Cookie","optional":true}]}},{"name":"getAll","type":"function","parameters":[{"type":"object","name":"details","properties":{"url":{"type":"string","optional":true},"name":{"type":"string","optional":true},"domain":{"type":"string","optional":true},"path":{"type":"string","optional":true},"secure":{"type":"boolean","optional":true},"session":{"type":"boolean","optional":true},"storeId":{"type":"string","optional":true},"partitionKey":{"$ref":"cookies.CookiePartitionKey","optional":true}}}],"returns_async":{"name":"callback","parameters":[{"name":"cookies","type":"array","items":{"$ref":"cookies.Cookie"}}]}},{"name":"set","type":"function","parameters":[{"type":"object","name":"details","properties":{"url":{"type":"string"},"name":{"type":"string","optional":true},"value":{"type":"string","optional":true},"domain":{"type":"string","optional":true},"path":{"type":"string","optional":true},"secure":{"type":"boolean","optional":true},"httpOnly":{"type":"boolean","optional":true},"sameSite":{"$ref":"cookies.SameSiteStatus","optional":true},"expirationDate":{"type":"number","optional":true},"storeId":{"type":"string","optional":true},"partitionKey":{"$ref":"cookies.CookiePartitionKey","optional":true}}}],"returns_async":{"name":"callback","optional":true,"min_version":"11.0.674.0","parameters":[{"name":"cookie","$ref":"cookies.Cookie","optional":true}]}},{"name":"remove","type":"function","parameters":[{"name":"details","$ref":"cookies.CookieDetails"}],"returns_async":{"name":"callback","optional":true,"min_version":"11.0.674.0","parameters":[{"name":"details","type":"object","optional":true,"properties":{"url":{"type":"string"},"name":{"type":"string"},"storeId":{"type":"string"},"partitionKey":{"$ref":"cookies.CookiePartitionKey","optional":true}}}]}},{"name":"getAllCookieStores","type":"function","parameters":[],"returns_async":{"name":"callback","parameters":[{"name":"cookieStores","type":"array","items":{"$ref":"cookies.CookieStore"}}]}},{"name":"getPartitionKey","type":"function","parameters":[{"name":"details","$ref":"cookies.FrameDetails"}],"returns_async":{"name":"callback","optional":true,"parameters":[{"name":"details","type":"object","properties":{"partitionKey":{"$ref":"cookies.CookiePartitionKey"}}}]}}],"events":[{"name":"onChanged","type":"function","parameters":[{"type":"object","name":"changeInfo","properties":{"removed":{"type":"boolean"},"cookie":{"$ref":"cookies.Cookie"},"cause":{"min_version":"12.0.707.0","$ref":"cookies.OnChangedCause"}}}]}]})R";
constexpr char kNotifications[] = R"R({"namespace":"notifications","types":[{"id":"notifications.TemplateType","type":"string","enum":[{"name":"basic"},{"name":"image"},{"name":"list"},{"name":"progress"}]},{"id":"notifications.PermissionLevel","type":"string","enum":[{"name":"granted"},{"name":"denied"}]},{"id":"notifications.NotificationItem","properties":{"title":{"name":"title","type":"string"},"message":{"name":"message","type":"string"}},"type":"object"},{"id":"notifications.NotificationBitmap","properties":{"width":{"name":"width","type":"integer"},"height":{"name":"height","type":"integer"},"data":{"optional":true,"name":"data","type":"binary","isInstanceOf":"ArrayBuffer"}},"type":"object"},{"id":"notifications.NotificationButton","properties":{"title":{"name":"title","type":"string"},"iconUrl":{"deprecated":"Button icons not visible for Mac OS X users.","optional":true,"name":"iconUrl","type":"string"},"iconBitmap":{"optional":true,"name":"iconBitmap","$ref":"notifications.NotificationBitmap"}},"type":"object"},{"id":"notifications.NotificationOptions","properties":{"type":{"optional":true,"name":"type","$ref":"notifications.TemplateType"},"iconUrl":{"optional":true,"name":"iconUrl","type":"string"},"iconBitmap":{"optional":true,"name":"iconBitmap","$ref":"notifications.NotificationBitmap"},"appIconMaskUrl":{"deprecated":"The app icon mask is not visible for Mac OS X users.","optional":true,"name":"appIconMaskUrl","type":"string"},"appIconMaskBitmap":{"optional":true,"name":"appIconMaskBitmap","$ref":"notifications.NotificationBitmap"},"title":{"optional":true,"name":"title","type":"string"},"message":{"optional":true,"name":"message","type":"string"},"contextMessage":{"optional":true,"name":"contextMessage","type":"string"},"priority":{"optional":true,"name":"priority","type":"integer"},"eventTime":{"optional":true,"name":"eventTime","type":"number"},"buttons":{"optional":true,"name":"buttons","type":"array","items":{"$ref":"notifications.NotificationButton"}},"expandedMessage":{"optional":true,"name":"expandedMessage","type":"string"},"imageUrl":{"deprecated":"The image is not visible for Mac OS X users.","optional":true,"name":"imageUrl","type":"string"},"imageBitmap":{"optional":true,"name":"imageBitmap","$ref":"notifications.NotificationBitmap"},"items":{"optional":true,"name":"items","type":"array","items":{"$ref":"notifications.NotificationItem"}},"progress":{"optional":true,"name":"progress","type":"integer"},"isClickable":{"deprecated":"This UI hint is ignored as of Chrome 67","optional":true,"name":"isClickable","type":"boolean"},"requireInteraction":{"optional":true,"name":"requireInteraction","type":"boolean"},"silent":{"optional":true,"name":"silent","type":"boolean"}},"type":"object"}],"functions":[{"parameters":[{"name":"notificationId","optional":true,"type":"string"},{"name":"options","$ref":"notifications.NotificationOptions"}],"returns_async":{"name":"callback","optional":true,"parameters":[{"name":"notificationId","type":"string"}]},"name":"create","type":"function"},{"parameters":[{"name":"notificationId","type":"string"},{"name":"options","$ref":"notifications.NotificationOptions"}],"returns_async":{"name":"callback","optional":true,"parameters":[{"name":"wasUpdated","type":"boolean"}]},"name":"update","type":"function"},{"parameters":[{"name":"notificationId","type":"string"}],"returns_async":{"name":"callback","optional":true,"parameters":[{"name":"wasCleared","type":"boolean"}]},"name":"clear","type":"function"},{"parameters":[],"returns_async":{"name":"callback","parameters":[{"name":"notifications","type":"object","additionalProperties":{"type":"any"}}]},"name":"getAll","type":"function"},{"parameters":[],"returns_async":{"name":"callback","parameters":[{"name":"level","$ref":"notifications.PermissionLevel"}]},"name":"getPermissionLevel","type":"function"}],"properties":{},"events":[{"parameters":[{"name":"notificationId","type":"string"},{"name":"byUser","type":"boolean"}],"name":"onClosed","type":"function"},{"parameters":[{"name":"notificationId","type":"string"}],"name":"onClicked","type":"function"},{"parameters":[{"name":"notificationId","type":"string"},{"name":"buttonIndex","type":"integer"}],"name":"onButtonClicked","type":"function"},{"parameters":[{"name":"level","$ref":"notifications.PermissionLevel"}],"name":"onPermissionLevelChanged","type":"function"},{"deprecated":"Custom notification settings button is no longer supported.","parameters":[],"name":"onShowSettings","type":"function"}],"platforms":null,"deprecated":null})R";
constexpr char kBrowsingData[] = R"R({"namespace":"browsingData","types":[{"id":"browsingData.RemovalOptions","type":"object","properties":{"since":{"type":"number","optional":true},"originTypes":{"type":"object","optional":true,"properties":{"unprotectedWeb":{"type":"boolean","optional":true},"protectedWeb":{"type":"boolean","optional":true},"extension":{"type":"boolean","optional":true}}},"origins":{"type":"array","items":{"type":"string"},"optional":true,"minItems":1},"excludeOrigins":{"type":"array","items":{"type":"string"},"optional":true}}},{"id":"browsingData.DataTypeSet","type":"object","properties":{"appcache":{"type":"boolean","optional":true},"cache":{"type":"boolean","optional":true},"cacheStorage":{"type":"boolean","optional":true},"cookies":{"type":"boolean","optional":true},"downloads":{"type":"boolean","optional":true},"fileSystems":{"type":"boolean","optional":true},"formData":{"type":"boolean","optional":true},"history":{"type":"boolean","optional":true},"indexedDB":{"type":"boolean","optional":true},"localStorage":{"type":"boolean","optional":true},"serverBoundCertificates":{"type":"boolean","optional":true,"deprecated":"Support for server-bound certificates has been removed. This data type will be ignored."},"passwords":{"type":"boolean","optional":true},"pluginData":{"type":"boolean","optional":true,"deprecated":"Support for Flash has been removed. This data type will be ignored."},"serviceWorkers":{"type":"boolean","optional":true},"webSQL":{"type":"boolean","optional":true}}}],"functions":[{"name":"settings","type":"function","parameters":[],"returns_async":{"name":"callback","parameters":[{"name":"result","type":"object","properties":{"options":{"$ref":"browsingData.RemovalOptions"},"dataToRemove":{"$ref":"browsingData.DataTypeSet"},"dataRemovalPermitted":{"$ref":"browsingData.DataTypeSet"}}}]}},{"name":"remove","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"},{"name":"dataToRemove","$ref":"browsingData.DataTypeSet"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removeAppcache","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removeCache","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removeCacheStorage","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removeCookies","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removeDownloads","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removeFileSystems","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removeFormData","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removeHistory","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removeIndexedDB","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removeLocalStorage","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removePluginData","deprecated":"Support for Flash has been removed. This function has no effect.","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removePasswords","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removeServiceWorkers","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"removeWebSQL","type":"function","parameters":[{"$ref":"browsingData.RemovalOptions","name":"options"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}}]})R";
constexpr char kExtension[] = R"R({"namespace":"extension","properties":{"lastError":{"type":"object","optional":true,"deprecated":"Please use $(ref:runtime.lastError).","properties":{"message":{"type":"string"}}},"inIncognitoContext":{"type":"boolean","optional":true}},"types":[{"id":"extension.ViewType","type":"string","enum":["tab","popup"]}],"functions":[{"name":"sendRequest","deprecated":"Please use $(ref:runtime.sendMessage).","type":"function","parameters":[{"type":"string","name":"extensionId","optional":true},{"type":"any","name":"request"}],"returns_async":{"name":"callback","optional":true,"parameters":[{"name":"response","type":"any"}]}},{"name":"getURL","deprecated":"Please use $(ref:runtime.getURL).","type":"function","parameters":[{"type":"string","name":"path"}],"returns":{"type":"string"}},{"name":"getViews","type":"function","parameters":[{"type":"object","name":"fetchProperties","optional":true,"properties":{"type":{"$ref":"extension.ViewType","optional":true},"windowId":{"type":"integer","optional":true},"tabId":{"type":"integer","optional":true}}}],"returns":{"type":"array","items":{"name":"viewGlobals","type":"object","isInstanceOf":"Window","additionalProperties":{"type":"any"}}}},{"name":"getBackgroundPage","type":"function","parameters":[],"returns":{"type":"object","optional":true,"name":"backgroundPageGlobal","isInstanceOf":"Window","additionalProperties":{"type":"any"}}},{"name":"getExtensionTabs","deprecated":"Please use $(ref:extension.getViews) <code>{type: \"tab\"}</code>.","type":"function","maximumManifestVersion":1,"parameters":[{"type":"integer","name":"windowId","optional":true}],"returns":{"type":"array","items":{"type":"object","name":"tabGlobals","isInstanceOf":"Window","additionalProperties":{"type":"any"}}}},{"name":"isAllowedIncognitoAccess","type":"function","min_version":"12.0.706.0","parameters":[],"returns_async":{"name":"callback","parameters":[{"name":"isAllowedAccess","type":"boolean"}]}},{"name":"isAllowedFileSchemeAccess","type":"function","min_version":"12.0.706.0","parameters":[],"returns_async":{"name":"callback","parameters":[{"name":"isAllowedAccess","type":"boolean"}]}},{"name":"setUpdateUrlData","type":"function","parameters":[{"type":"string","name":"data","maxLength":1024}]}],"events":[{"name":"onRequest","deprecated":"Please use $(ref:runtime.onMessage).","type":"function","options":{"unmanaged":true},"parameters":[{"name":"request","type":"any","optional":true},{"name":"sender","$ref":"runtime.MessageSender"},{"name":"sendResponse","type":"function"}]},{"name":"onRequestExternal","deprecated":"Please use $(ref:runtime.onMessageExternal).","type":"function","parameters":[{"name":"request","type":"any","optional":true},{"name":"sender","$ref":"runtime.MessageSender"},{"name":"sendResponse","type":"function"}]}]})R";
constexpr char kApp[] = R"R({"namespace":"app","types":[{"id":"app.Details","type":"object","properties":{},"additionalProperties":{"type":"any"}},{"id":"app.DOMWindow","type":"object","properties":{},"additionalProperties":{"type":"any"}},{"id":"app.InstallState","type":"string","enum":["not_installed","installed","disabled"]},{"id":"app.RunningState","type":"string","enum":["running","cannot_run","ready_to_run"]}],"functions":[{"name":"getIsInstalled","type":"function","parameters":[],"returns":{"name":"isInstalled","type":"boolean"}},{"name":"installState","type":"function","parameters":[],"returns_async":{"name":"callback","parameters":[{"$ref":"app.InstallState","name":"state"}]}},{"name":"runningState","type":"function","parameters":[],"returns":{"$ref":"app.RunningState"}},{"name":"getDetails","type":"function","parameters":[],"returns":{"$ref":"app.Details","optional":true}}]})R";
constexpr char kCommands[] = R"R({"namespace":"commands","types":[{"id":"commands.Command","type":"object","properties":{"name":{"type":"string","optional":true},"description":{"type":"string","optional":true},"shortcut":{"type":"string","optional":true}}}],"events":[{"name":"onCommand","type":"function","parameters":[{"name":"command","type":"string"},{"$ref":"tabs.Tab","name":"tab","optional":true}]}],"functions":[{"name":"getAll","type":"function","parameters":[],"returns_async":{"name":"callback","optional":true,"parameters":[{"name":"commands","type":"array","items":{"$ref":"commands.Command"}}]}}]})R";
constexpr char kDeclarativeContent[] = R"R({"namespace":"declarativeContent","types":[{"id":"declarativeContent.ImageDataType","type":"binary","isInstanceOf":"ImageData","additionalProperties":{"type":"any"}},{"id":"declarativeContent.PageStateMatcherInstanceType","type":"string","enum":["declarativeContent.PageStateMatcher"]},{"id":"declarativeContent.ShowPageActionInstanceType","type":"string","enum":["declarativeContent.ShowPageAction"]},{"id":"declarativeContent.ShowActionInstanceType","type":"string","enum":["declarativeContent.ShowAction"]},{"id":"declarativeContent.SetIconInstanceType","type":"string","enum":["declarativeContent.SetIcon"]},{"id":"declarativeContent.RequestContentScriptInstanceType","type":"string","enum":["declarativeContent.RequestContentScript"]},{"id":"declarativeContent.PageStateMatcher","type":"object","properties":{"pageUrl":{"$ref":"events.UrlFilter","optional":true},"css":{"type":"array","optional":true,"items":{"type":"string"}},"isBookmarked":{"type":"boolean","optional":true},"instanceType":{"$ref":"declarativeContent.PageStateMatcherInstanceType"}}},{"id":"declarativeContent.ShowPageAction","deprecated":"Please use $(ref:declarativeContent.ShowAction).","type":"object","properties":{"instanceType":{"$ref":"declarativeContent.ShowPageActionInstanceType"}}},{"id":"declarativeContent.ShowAction","type":"object","properties":{"instanceType":{"$ref":"declarativeContent.ShowActionInstanceType"}}},{"id":"declarativeContent.SetIcon","type":"object","properties":{"instanceType":{"$ref":"declarativeContent.SetIconInstanceType"},"imageData":{"choices":[{"$ref":"declarativeContent.ImageDataType"},{"type":"object","additionalProperties":{"type":"any"}}],"optional":true}}},{"id":"declarativeContent.RequestContentScript","type":"object","properties":{"css":{"type":"array","optional":true,"items":{"type":"string"}},"js":{"type":"array","optional":true,"items":{"type":"string"}},"allFrames":{"type":"boolean","optional":true},"matchAboutBlank":{"type":"boolean","optional":true},"instanceType":{"$ref":"declarativeContent.RequestContentScriptInstanceType"}}}],"functions":[],"events":[{"name":"onPageChanged","options":{"supportsListeners":false,"supportsRules":true,"conditions":["declarativeContent.PageStateMatcher"],"actions":["declarativeContent.RequestContentScript","declarativeContent.SetIcon","declarativeContent.ShowPageAction","declarativeContent.ShowAction"]}}]})R";
constexpr char kDom[] = R"R({"namespace":"dom","functions":[{"name":"openOrClosedShadowRoot","type":"function","parameters":[{"name":"element","type":"object","isInstanceOf":"HTMLElement","additionalProperties":{"type":"any"}}],"returns":{"name":"shadowRoot","type":"object","additionalProperties":{"type":"any"}}}]})R";
constexpr char kPrivacy[] = R"R({"namespace":"privacy","types":[{"id":"privacy.IPHandlingPolicy","type":"string","enum":["default","default_public_and_private_interfaces","default_public_interface_only","disable_non_proxied_udp"]}],"properties":{"network":{"type":"object","value":{},"properties":{"networkPredictionEnabled":{"$ref":"types.ChromeSetting","value":["networkPredictionEnabled",{"type":"boolean"}]},"webRTCIPHandlingPolicy":{"$ref":"types.ChromeSetting","value":["webRTCIPHandlingPolicy",{"$ref":"privacy.IPHandlingPolicy"}]}}},"services":{"type":"object","value":{},"properties":{"alternateErrorPagesEnabled":{"$ref":"types.ChromeSetting","value":["alternateErrorPagesEnabled",{"type":"boolean"}]},"autofillEnabled":{"$ref":"types.ChromeSetting","value":["autofillEnabled",{"type":"boolean"}],"deprecated":"Please use privacy.services.autofillAddressEnabled and privacy.services.autofillCreditCardEnabled. This remains for backward compatibility in this release and will be removed in the future."},"autofillAddressEnabled":{"$ref":"types.ChromeSetting","value":["autofillAddressEnabled",{"type":"boolean"}]},"autofillCreditCardEnabled":{"$ref":"types.ChromeSetting","value":["autofillCreditCardEnabled",{"type":"boolean"}]},"passwordSavingEnabled":{"$ref":"types.ChromeSetting","value":["passwordSavingEnabled",{"type":"boolean"}]},"safeBrowsingEnabled":{"$ref":"types.ChromeSetting","value":["safeBrowsingEnabled",{"type":"boolean"}]},"safeBrowsingExtendedReportingEnabled":{"$ref":"types.ChromeSetting","value":["safeBrowsingExtendedReportingEnabled",{"type":"boolean"}]},"searchSuggestEnabled":{"$ref":"types.ChromeSetting","value":["searchSuggestEnabled",{"type":"boolean"}]},"spellingServiceEnabled":{"$ref":"types.ChromeSetting","value":["spellingServiceEnabled",{"type":"boolean"}]},"translationServiceEnabled":{"$ref":"types.ChromeSetting","value":["translationServiceEnabled",{"type":"boolean"}]}}},"websites":{"type":"object","value":{},"properties":{"thirdPartyCookiesAllowed":{"$ref":"types.ChromeSetting","value":["thirdPartyCookiesAllowed",{"type":"boolean"}]},"topicsEnabled":{"$ref":"types.ChromeSetting","value":["topicsEnabled",{"type":"boolean"}]},"fledgeEnabled":{"$ref":"types.ChromeSetting","value":["fledgeEnabled",{"type":"boolean"}]},"adMeasurementEnabled":{"$ref":"types.ChromeSetting","value":["adMeasurementEnabled",{"type":"boolean"}]},"hyperlinkAuditingEnabled":{"$ref":"types.ChromeSetting","value":["hyperlinkAuditingEnabled",{"type":"boolean"}]},"referrersEnabled":{"$ref":"types.ChromeSetting","value":["referrersEnabled",{"type":"boolean"}]},"doNotTrackEnabled":{"$ref":"types.ChromeSetting","value":["doNotTrackEnabled",{"type":"boolean"}]},"protectedContentEnabled":{"$ref":"types.ChromeSetting","value":["protectedContentEnabled",{"type":"boolean"}],"platforms":["win","chromeos","lacros"]},"relatedWebsiteSetsEnabled":{"$ref":"types.ChromeSetting","value":["relatedWebsiteSetsEnabled",{"type":"boolean"}]}}}}})R";
constexpr char kProxy[] = R"R({"namespace":"proxy","types":[{"id":"proxy.Scheme","type":"string","enum":["http","https","quic","socks4","socks5"]},{"id":"proxy.Mode","type":"string","enum":["direct","auto_detect","pac_script","fixed_servers","system"]},{"id":"proxy.ProxyServer","type":"object","properties":{"scheme":{"$ref":"proxy.Scheme","optional":true},"host":{"type":"string"},"port":{"type":"integer","optional":true}}},{"id":"proxy.ProxyRules","type":"object","properties":{"singleProxy":{"$ref":"proxy.ProxyServer","optional":true},"proxyForHttp":{"$ref":"proxy.ProxyServer","optional":true},"proxyForHttps":{"$ref":"proxy.ProxyServer","optional":true},"proxyForFtp":{"$ref":"proxy.ProxyServer","optional":true},"fallbackProxy":{"$ref":"proxy.ProxyServer","optional":true},"bypassList":{"type":"array","items":{"type":"string"},"optional":true}}},{"id":"proxy.PacScript","type":"object","properties":{"url":{"type":"string","optional":true},"data":{"type":"string","optional":true},"mandatory":{"type":"boolean","optional":true}}},{"id":"proxy.ProxyConfig","type":"object","properties":{"rules":{"$ref":"proxy.ProxyRules","optional":true},"pacScript":{"$ref":"proxy.PacScript","optional":true},"mode":{"$ref":"proxy.Mode"}}}],"properties":{"settings":{"$ref":"types.ChromeSetting","value":["proxy",{"$ref":"proxy.ProxyConfig"}]}},"events":[{"name":"onProxyError","type":"function","parameters":[{"type":"object","name":"details","properties":{"fatal":{"type":"boolean"},"error":{"type":"string"},"details":{"type":"string"}}}]}]})R";
constexpr char kTts[] = R"R({"namespace":"tts","types":[{"id":"tts.EventType","type":"string","enum":["start","end","word","sentence","marker","interrupted","cancelled","error","pause","resume"]},{"id":"tts.VoiceGender","type":"string","deprecated":"Gender is deprecated and is ignored.","enum":["male","female"]},{"id":"tts.TtsOptions","type":"object","properties":{"enqueue":{"type":"boolean","optional":true},"voiceName":{"type":"string","optional":true},"extensionId":{"type":"string","optional":true},"lang":{"type":"string","optional":true},"gender":{"$ref":"tts.VoiceGender","optional":true,"deprecated":"Gender is deprecated and will be ignored."},"rate":{"type":"number","optional":true,"minimum":0.1,"maximum":10},"pitch":{"type":"number","optional":true,"minimum":0,"maximum":2},"volume":{"type":"number","optional":true,"minimum":0,"maximum":1},"requiredEventTypes":{"type":"array","items":{"type":"string"},"optional":true},"desiredEventTypes":{"type":"array","items":{"type":"string"},"optional":true},"onEvent":{"type":"function","optional":true,"parameters":[{"name":"event","$ref":"tts.TtsEvent"}]}}},{"id":"tts.TtsEvent","type":"object","properties":{"type":{"$ref":"tts.EventType"},"charIndex":{"type":"integer","optional":true},"errorMessage":{"type":"string","optional":true},"srcId":{"type":"number","optional":true},"isFinalEvent":{"type":"boolean","optional":true},"length":{"type":"integer","optional":true}}},{"id":"tts.TtsVoice","type":"object","properties":{"voiceName":{"type":"string","optional":true},"lang":{"type":"string","optional":true},"gender":{"$ref":"tts.VoiceGender","optional":true,"deprecated":"Gender is deprecated and will be ignored."},"remote":{"type":"boolean","optional":true},"extensionId":{"type":"string","optional":true},"eventTypes":{"type":"array","items":{"$ref":"tts.EventType"},"optional":true}}}],"functions":[{"name":"speak","type":"function","parameters":[{"type":"string","name":"utterance"},{"$ref":"tts.TtsOptions","name":"options","optional":true}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"stop","type":"function","parameters":[]},{"name":"pause","type":"function","parameters":[]},{"name":"resume","type":"function","parameters":[]},{"name":"isSpeaking","type":"function","parameters":[],"returns_async":{"name":"callback","optional":true,"parameters":[{"name":"speaking","type":"boolean"}]}},{"name":"getVoices","type":"function","parameters":[],"returns_async":{"name":"callback","optional":true,"parameters":[{"type":"array","name":"voices","items":{"$ref":"tts.TtsVoice"}}]}}],"events":[{"name":"onEvent","type":"function","parameters":[{"name":"event","$ref":"tts.TtsEvent"}]},{"name":"onVoicesChanged","type":"function"}]})R";
constexpr char kTtsEngine[] = R"R({"namespace":"ttsEngine","types":[{"id":"ttsEngine.TtsClientSource","type":"string","enum":["chromefeature","extension"]},{"id":"ttsEngine.TtsClient","type":"object","properties":{"id":{"type":"string"},"source":{"$ref":"ttsEngine.TtsClientSource"}}},{"id":"ttsEngine.VoiceGender","type":"string","deprecated":"Gender is deprecated and will be ignored.","enum":["male","female"]},{"id":"ttsEngine.LanguageUninstallOptions","type":"object","properties":{"uninstallImmediately":{"type":"boolean"}}},{"id":"ttsEngine.LanguageInstallStatus","type":"string","enum":["notInstalled","installing","installed","failed"]},{"id":"ttsEngine.LanguageStatus","type":"object","properties":{"lang":{"type":"string"},"installStatus":{"$ref":"ttsEngine.LanguageInstallStatus"},"error":{"type":"string","optional":true}}},{"id":"ttsEngine.SpeakOptions","type":"object","properties":{"voiceName":{"type":"string","optional":true},"lang":{"type":"string","optional":true},"gender":{"$ref":"ttsEngine.VoiceGender","optional":true,"deprecated":"Gender is deprecated and will be ignored."},"rate":{"type":"number","optional":true,"minimum":0.1,"maximum":10.0},"pitch":{"type":"number","optional":true,"minimum":0,"maximum":2},"volume":{"type":"number","optional":true,"minimum":0,"maximum":1}}},{"id":"ttsEngine.AudioStreamOptions","type":"object","properties":{"sampleRate":{"type":"integer"},"bufferSize":{"type":"integer"}}},{"id":"ttsEngine.AudioBuffer","type":"object","properties":{"audioBuffer":{"type":"binary"},"charIndex":{"type":"integer","optional":true},"isLastBuffer":{"type":"boolean","optional":true}}}],"functions":[{"name":"updateVoices","type":"function","parameters":[{"type":"array","name":"voices","items":{"$ref":"tts.TtsVoice"}}]},{"name":"sendTtsEvent","type":"function","parameters":[{"type":"integer","name":"requestId"},{"name":"event","$ref":"tts.TtsEvent"}]},{"name":"sendTtsAudio","type":"function","parameters":[{"type":"integer","name":"requestId"},{"name":"audio","$ref":"ttsEngine.AudioBuffer"}],"platforms":["chromeos"]},{"name":"updateLanguage","type":"function","parameters":[{"name":"status","$ref":"ttsEngine.LanguageStatus"}]}],"events":[{"name":"onSpeak","type":"function","parameters":[{"type":"string","name":"utterance"},{"$ref":"ttsEngine.SpeakOptions","name":"options"},{"name":"sendTtsEvent","type":"function","parameters":[{"name":"event","$ref":"tts.TtsEvent"}]}]},{"name":"onSpeakWithAudioStream","type":"function","parameters":[{"type":"string","name":"utterance"},{"$ref":"ttsEngine.SpeakOptions","name":"options"},{"$ref":"ttsEngine.AudioStreamOptions","name":"audioStreamOptions"},{"name":"sendTtsAudio","type":"function","parameters":[{"$ref":"ttsEngine.AudioBuffer","name":"audioBufferParams"}]},{"name":"sendError","type":"function","parameters":[{"type":"string","name":"errorMessage","optional":true}]}],"platforms":["chromeos"]},{"name":"onStop","type":"function"},{"name":"onPause","type":"function"},{"name":"onResume","type":"function"},{"name":"onInstallLanguageRequest","type":"function","parameters":[{"name":"requestor","$ref":"ttsEngine.TtsClient"},{"name":"lang","type":"string"}]},{"name":"onUninstallLanguageRequest","type":"function","parameters":[{"name":"requestor","$ref":"ttsEngine.TtsClient"},{"name":"lang","type":"string"},{"name":"uninstallOptions","$ref":"ttsEngine.LanguageUninstallOptions"}]},{"name":"onLanguageStatusRequest","type":"function","parameters":[{"name":"requestor","$ref":"ttsEngine.TtsClient"},{"name":"lang","type":"string"}]}]})R";
constexpr char kInputMethodPrivate[] = R"R({"namespace":"inputMethodPrivate","types":[{"id":"inputMethodPrivate.MenuItemStyle","type":"string","enum":["check","radio","separator"]},{"id":"inputMethodPrivate.MenuItem","type":"object","properties":{"id":{"type":"string"},"label":{"type":"string","optional":true},"style":{"$ref":"inputMethodPrivate.MenuItemStyle","optional":true},"visible":{"type":"boolean","optional":true},"checked":{"type":"boolean","optional":true},"enabled":{"type":"boolean","optional":true}}},{"id":"inputMethodPrivate.UnderlineStyle","type":"string","enum":["underline","doubleUnderline","noUnderline"]},{"id":"inputMethodPrivate.FocusReason","type":"string","enum":["mouse","touch","pen","other"]},{"id":"inputMethodPrivate.InputModeType","type":"string","enum":["noKeyboard","text","tel","url","email","numeric","decimal","search"]},{"id":"inputMethodPrivate.InputContextType","type":"string","enum":["text","search","tel","url","email","number","password","null"]},{"id":"inputMethodPrivate.AutoCapitalizeType","type":"string","enum":["off","characters","words","sentences"]},{"id":"inputMethodPrivate.LanguagePackStatus","type":"string","enum":["unknown","notInstalled","inProgress","installed","errorOther","errorNeedsReboot"]},{"id":"inputMethodPrivate.LanguagePackStatusChange","type":"object","properties":{"engineIds":{"type":"array","items":{"type":"string"}},"status":{"$ref":"inputMethodPrivate.LanguagePackStatus"}}},{"id":"inputMethodPrivate.InputContext","type":"object","properties":{"contextID":{"type":"integer"},"type":{"$ref":"inputMethodPrivate.InputContextType"},"mode":{"$ref":"inputMethodPrivate.InputModeType"},"autoCorrect":{"type":"boolean"},"autoComplete":{"type":"boolean"},"autoCapitalize":{"$ref":"inputMethodPrivate.AutoCapitalizeType"},"spellCheck":{"type":"boolean"},"shouldDoLearning":{"type":"boolean"},"focusReason":{"$ref":"inputMethodPrivate.FocusReason"},"appKey":{"type":"string","optional":true}}},{"id":"inputMethodPrivate.InputMethodSettings","type":"object","properties":{"enableCompletion":{"type":"boolean","optional":true},"enableDoubleSpacePeriod":{"type":"boolean","optional":true},"enableGestureTyping":{"type":"boolean","optional":true},"enablePrediction":{"type":"boolean","optional":true},"enableSoundOnKeypress":{"type":"boolean","optional":true},"physicalKeyboardAutoCorrectionEnabledByDefault":{"type":"boolean","optional":true},"physicalKeyboardAutoCorrectionLevel":{"type":"integer","optional":true},"physicalKeyboardEnableCapitalization":{"type":"boolean","optional":true},"physicalKeyboardEnableDiacriticsOnLongpress":{"type":"boolean","optional":true},"physicalKeyboardEnablePredictiveWriting":{"type":"boolean","optional":true},"virtualKeyboardAutoCorrectionLevel":{"type":"integer","optional":true},"virtualKeyboardEnableCapitalization":{"type":"boolean","optional":true},"xkbLayout":{"type":"string","optional":true},"koreanEnableSyllableInput":{"type":"boolean","optional":true},"koreanKeyboardLayout":{"type":"string","optional":true},"koreanShowHangulCandidate":{"type":"boolean","optional":true},"pinyinChinesePunctuation":{"type":"boolean","optional":true},"pinyinDefaultChinese":{"type":"boolean","optional":true},"pinyinEnableFuzzy":{"type":"boolean","optional":true},"pinyinEnableLowerPaging":{"type":"boolean","optional":true},"pinyinEnableUpperPaging":{"type":"boolean","optional":true},"pinyinFullWidthCharacter":{"type":"boolean","optional":true},"pinyinFuzzyConfig":{"type":"object","optional":true,"properties":{"an_ang":{"type":"boolean","optional":true},"c_ch":{"type":"boolean","optional":true},"en_eng":{"type":"boolean","optional":true},"f_h":{"type":"boolean","optional":true},"ian_iang":{"type":"boolean","optional":true},"in_ing":{"type":"boolean","optional":true},"k_g":{"type":"boolean","optional":true},"l_n":{"type":"boolean","optional":true},"r_l":{"type":"boolean","optional":true},"s_sh":{"type":"boolean","optional":true},"uan_uang":{"type":"boolean","optional":true},"z_zh":{"type":"boolean","optional":true}}},"zhuyinKeyboardLayout":{"type":"string","optional":true},"zhuyinPageSize":{"type":"integer","optional":true},"zhuyinSelectKeys":{"type":"string","optional":true},"vietnameseVniAllowFlexibleDiacritics":{"type":"boolean","optional":true},"vietnameseVniNewStyleToneMarkPlacement":{"type":"boolean","optional":true},"vietnameseVniInsertDoubleHornOnUo":{"type":"boolean","optional":true},"vietnameseVniShowUnderline":{"type":"boolean","optional":true},"vietnameseTelexAllowFlexibleDiacritics":{"type":"boolean","optional":true},"vietnameseTelexNewStyleToneMarkPlacement":{"type":"boolean","optional":true},"vietnameseTelexInsertDoubleHornOnUo":{"type":"boolean","optional":true},"vietnameseTelexInsertUHornOnW":{"type":"boolean","optional":true},"vietnameseTelexShowUnderline":{"type":"boolean","optional":true}}}],"functions":[{"name":"getInputMethodConfig","type":"function","parameters":[],"returns_async":{"name":"callback","parameters":[{"name":"config","type":"object","properties":{"isPhysicalKeyboardAutocorrectEnabled":{"type":"boolean"},"isImeMenuActivated":{"type":"boolean"}}}]}},{"name":"getInputMethods","type":"function","parameters":[],"returns_async":{"name":"callback","parameters":[{"name":"inputMethods","type":"array","items":{"type":"object","properties":{"id":{"type":"string"},"name":{"type":"string"},"indicator":{"type":"string"}}}}]}},{"name":"getCurrentInputMethod","type":"function","parameters":[],"returns_async":{"name":"callback","parameters":[{"name":"inputMethodId","type":"string"}]}},{"name":"setCurrentInputMethod","type":"function","parameters":[{"name":"inputMethodId","type":"string"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"switchToLastUsedInputMethod","type":"function","parameters":[],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"fetchAllDictionaryWords","type":"function","parameters":[],"returns_async":{"name":"callback","parameters":[{"name":"words","type":"array","items":{"type":"string"}}]}},{"name":"addWordToDictionary","type":"function","parameters":[{"name":"word","type":"string"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"setXkbLayout","type":"function","parameters":[{"name":"xkb_name","type":"string"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"finishComposingText","type":"function","parameters":[{"name":"parameters","type":"object","properties":{"contextID":{"type":"integer"}}}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"showInputView","type":"function","parameters":[],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"hideInputView","type":"function","parameters":[],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"openOptionsPage","type":"function","parameters":[{"name":"inputMethodId","type":"string"}]},{"name":"getSurroundingText","type":"function","parameters":[{"name":"beforeLength","type":"integer"},{"name":"afterLength","type":"integer"}],"returns_async":{"name":"callback","parameters":[{"name":"surroundingInfo","type":"object","properties":{"before":{"type":"string"},"selected":{"type":"string"},"after":{"type":"string"}}}]}},{"name":"getSettings","type":"function","parameters":[{"name":"engineID","type":"string"}],"returns_async":{"name":"callback","parameters":[{"name":"settings","$ref":"inputMethodPrivate.InputMethodSettings","optional":true}]}},{"name":"setSettings","type":"function","parameters":[{"name":"engineID","type":"string"},{"name":"settings","$ref":"inputMethodPrivate.InputMethodSettings"}],"returns_async":{"name":"callback","optional":true,"parameters":[]}},{"name":"setCompositionRange","type":"function","parameters":[{"name":"parameters","type":"object","properties":{"contextID":{"type":"integer"},"selectionBefore":{"type":"integer"},"selectionAfter":{"type":"integer"},"segments":{"type":"array","optional":true,"items":{"type":"object","properties":{"start":{"type":"integer"},"end":{"type":"integer"},"style":{"$ref":"inputMethodPrivate.UnderlineStyle"}}}}}}],"returns_async":{"name":"callback","optional":true,"parameters":[{"name":"success","type":"boolea)R" R"R(n"}]}},{"name":"reset","type":"function","parameters":[]},{"name":"onAutocorrect","type":"function","platforms":["chromeos"],"parameters":[{"name":"parameters","type":"object","properties":{"contextID":{"type":"integer"},"typedWord":{"type":"string"},"correctedWord":{"type":"string"},"startIndex":{"type":"integer"}}}]},{"name":"notifyInputMethodReadyForTesting","type":"function","parameters":[]},{"name":"getLanguagePackStatus","type":"function","parameters":[{"name":"inputMethodId","type":"string"}],"returns_async":{"name":"callback","parameters":[{"name":"status","$ref":"inputMethodPrivate.LanguagePackStatus"}]}}],"events":[{"name":"onCaretBoundsChanged","type":"function","parameters":[{"name":"caretBounds","type":"object","properties":{"x":{"type":"integer"},"y":{"type":"integer"},"w":{"type":"integer"},"h":{"type":"integer"}}}]},{"name":"onChanged","type":"function","parameters":[{"name":"newInputMethodId","type":"string"}]},{"name":"onDictionaryLoaded","type":"function","parameters":[]},{"name":"onDictionaryChanged","type":"function","parameters":[{"name":"added","type":"array","items":{"type":"string"}},{"name":"removed","type":"array","items":{"type":"string"}}]},{"name":"onImeMenuActivationChanged","type":"function","parameters":[{"name":"activation","type":"boolean"}]},{"name":"onImeMenuListChanged","type":"function","parameters":[]},{"name":"onImeMenuItemsChanged","type":"function","parameters":[{"name":"engineID","type":"string"},{"name":"items","type":"array","items":{"$ref":"inputMethodPrivate.MenuItem"}}]},{"name":"onFocus","type":"function","parameters":[{"$ref":"inputMethodPrivate.InputContext","name":"context"}]},{"name":"onSettingsChanged","type":"function","parameters":[{"name":"engineID","type":"string"},{"name":"settings","$ref":"inputMethodPrivate.InputMethodSettings"}]},{"name":"onScreenProjectionChanged","type":"function","parameters":[{"name":"isProjected","type":"boolean"}]},{"name":"onSuggestionsChanged","type":"function","parameters":[{"name":"suggestions","type":"array","items":{"type":"string"}}]},{"name":"onInputMethodOptionsChanged","type":"function","parameters":[{"name":"engineID","type":"string"}]},{"name":"onLanguagePackStatusChanged","type":"function","parameters":[{"name":"change","$ref":"inputMethodPrivate.LanguagePackStatusChange"}]}]})R";
}  // namespace

namespace extensions {
namespace api {

// static
bool ChromeGeneratedSchemas::IsGenerated(std::string_view name) {
  return !Get(name).empty();
}

// static
std::string_view ChromeGeneratedSchemas::Get(std::string_view name) {
  static constexpr auto kSchemas = base::MakeFixedFlatMap<std::string_view, std::string_view>({
    {"app", kApp},
    {"browsingData", kBrowsingData},
    {"commands", kCommands},
    {"cookies", kCookies},
    {"declarativeContent", kDeclarativeContent},
    {"dom", kDom},
    {"extension", kExtension},
    {"inputMethodPrivate", kInputMethodPrivate},
    {"notifications", kNotifications},
    {"privacy", kPrivacy},
    {"proxy", kProxy},
    {"tts", kTts},
    {"ttsEngine", kTtsEngine},
  });
  auto it = kSchemas.find(name);
  return it != kSchemas.end() ? it->second : std::string_view();
}

}  // namespace api
}  // namespace extensions
