#include "src/ast/ast.h"
#include "src/builtins/builtins-array-gen.h"
#include "src/builtins/builtins-bigint-gen.h"
#include "src/builtins/builtins-call-gen.h"
#include "src/builtins/builtins-collections-gen.h"
#include "src/builtins/builtins-constructor-gen.h"
#include "src/builtins/builtins-data-view-gen.h"
#include "src/builtins/builtins-iterator-gen.h"
#include "src/builtins/builtins-object-gen.h"
#include "src/builtins/builtins-promise-gen.h"
#include "src/builtins/builtins-promise.h"
#include "src/builtins/builtins-proxy-gen.h"
#include "src/builtins/builtins-regexp-gen.h"
#include "src/builtins/builtins-string-gen.h"
#include "src/builtins/builtins-typed-array-gen.h"
#include "src/builtins/builtins-utils-gen.h"
#include "src/builtins/builtins-wasm-gen.h"
#include "src/builtins/builtins.h"
#include "src/codegen/code-factory.h"
#include "src/debug/debug-wasm-objects.h"
#include "src/heap/factory-inl.h"
#include "src/ic/binary-op-assembler.h"
#include "src/ic/handler-configuration-inl.h"
#include "src/objects/arguments.h"
#include "src/objects/bigint.h"
#include "src/objects/call-site-info.h"
#include "src/objects/elements-kind.h"
#include "src/objects/free-space.h"
#include "src/objects/intl-objects.h"
#include "src/objects/js-atomics-synchronization.h"
#include "src/objects/js-break-iterator.h"
#include "src/objects/js-collator.h"
#include "src/objects/js-date-time-format.h"
#include "src/objects/js-display-names.h"
#include "src/objects/js-disposable-stack.h"
#include "src/objects/js-duration-format.h"
#include "src/objects/js-function.h"
#include "src/objects/js-generator.h"
#include "src/objects/js-iterator-helpers.h"
#include "src/objects/js-list-format.h"
#include "src/objects/js-locale.h"
#include "src/objects/js-number-format.h"
#include "src/objects/js-objects.h"
#include "src/objects/js-plural-rules.h"
#include "src/objects/js-promise.h"
#include "src/objects/js-raw-json.h"
#include "src/objects/js-regexp-string-iterator.h"
#include "src/objects/js-relative-time-format.h"
#include "src/objects/js-segment-iterator-inl.h"
#include "src/objects/js-segmenter.h"
#include "src/objects/js-segments.h"
#include "src/objects/js-shadow-realm.h"
#include "src/objects/js-shared-array.h"
#include "src/objects/js-struct.h"
#include "src/objects/js-temporal-objects.h"
#include "src/objects/js-weak-refs.h"
#include "src/objects/objects.h"
#include "src/objects/ordered-hash-table.h"
#include "src/objects/property-array.h"
#include "src/objects/property-descriptor-object.h"
#include "src/objects/source-text-module.h"
#include "src/objects/swiss-hash-table-helpers.h"
#include "src/objects/swiss-name-dictionary.h"
#include "src/objects/synthetic-module.h"
#include "src/objects/template-objects.h"
#include "src/objects/torque-defined-classes.h"
#include "src/objects/turbofan-types.h"
#include "src/objects/turboshaft-types.h"
#include "src/torque/runtime-support.h"
#include "src/wasm/value-type.h"
#include "src/wasm/wasm-linkage.h"
#include "src/wasm/wasm-module.h"
#include "src/codegen/code-stub-assembler-inl.h"
// Required Builtins:
#include "torque-generated/src/builtins/js-to-wasm-tq-csa.h"
#include "torque-generated/src/builtins/array-findindex-tq-csa.h"
#include "torque-generated/src/builtins/array-flat-tq-csa.h"
#include "torque-generated/src/builtins/array-join-tq-csa.h"
#include "torque-generated/src/builtins/array-splice-tq-csa.h"
#include "torque-generated/src/builtins/base-tq-csa.h"
#include "torque-generated/src/builtins/builtins-bigint-tq-csa.h"
#include "torque-generated/src/builtins/builtins-string-tq-csa.h"
#include "torque-generated/src/builtins/cast-tq-csa.h"
#include "torque-generated/src/builtins/convert-tq-csa.h"
#include "torque-generated/src/builtins/frames-tq-csa.h"
#include "torque-generated/src/builtins/frame-arguments-tq-csa.h"
#include "torque-generated/src/builtins/torque-internal-tq-csa.h"
#include "torque-generated/src/objects/contexts-tq-csa.h"
#include "torque-generated/src/objects/fixed-array-tq-csa.h"
#include "torque-generated/src/builtins/js-to-js-tq-csa.h"
#include "torque-generated/src/builtins/js-to-wasm-tq-csa.h"
#include "torque-generated/src/builtins/wasm-tq-csa.h"
#include "torque-generated/src/builtins/wasm-to-js-tq-csa.h"
#include "torque-generated/src/wasm/wasm-objects-tq-csa.h"

namespace v8 {
namespace internal {

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=70&c=1
TNode<Int32T> FromConstexpr_ValueKind_constexpr_kRef_0(compiler::CodeAssemblerState* state_, wasm::ValueKind p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 70);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=70&c=1
TNode<Int32T> FromConstexpr_ValueKind_constexpr_kRefNull_0(compiler::CodeAssemblerState* state_, wasm::ValueKind p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 70);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=76&c=1
TNode<Int32T> FromConstexpr_Promise_constexpr_kPromise_0(compiler::CodeAssemblerState* state_, wasm::Promise p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 76);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=76&c=1
TNode<Int32T> FromConstexpr_Promise_constexpr_kNoPromise_0(compiler::CodeAssemblerState* state_, wasm::Promise p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 76);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=76&c=1
TNode<Int32T> FromConstexpr_Promise_constexpr_kStressSwitch_0(compiler::CodeAssemblerState* state_, wasm::Promise p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 76);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=82&c=1
TNode<Int32T> FromConstexpr_HeapType_constexpr_kExtern_0(compiler::CodeAssemblerState* state_, wasm::HeapType::Representation p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 82);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=82&c=1
TNode<Int32T> FromConstexpr_HeapType_constexpr_kNoExtern_0(compiler::CodeAssemblerState* state_, wasm::HeapType::Representation p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 82);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=82&c=1
TNode<Int32T> FromConstexpr_HeapType_constexpr_kString_0(compiler::CodeAssemblerState* state_, wasm::HeapType::Representation p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 82);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=82&c=1
TNode<Int32T> FromConstexpr_HeapType_constexpr_kEq_0(compiler::CodeAssemblerState* state_, wasm::HeapType::Representation p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 82);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=82&c=1
TNode<Int32T> FromConstexpr_HeapType_constexpr_kI31_0(compiler::CodeAssemblerState* state_, wasm::HeapType::Representation p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 82);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=82&c=1
TNode<Int32T> FromConstexpr_HeapType_constexpr_kStruct_0(compiler::CodeAssemblerState* state_, wasm::HeapType::Representation p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 82);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=82&c=1
TNode<Int32T> FromConstexpr_HeapType_constexpr_kArray_0(compiler::CodeAssemblerState* state_, wasm::HeapType::Representation p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 82);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=82&c=1
TNode<Int32T> FromConstexpr_HeapType_constexpr_kAny_0(compiler::CodeAssemblerState* state_, wasm::HeapType::Representation p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 82);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=82&c=1
TNode<Int32T> FromConstexpr_HeapType_constexpr_kNone_0(compiler::CodeAssemblerState* state_, wasm::HeapType::Representation p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 82);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=82&c=1
TNode<Int32T> FromConstexpr_HeapType_constexpr_kFunc_0(compiler::CodeAssemblerState* state_, wasm::HeapType::Representation p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 82);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=82&c=1
TNode<Int32T> FromConstexpr_HeapType_constexpr_kNoFunc_0(compiler::CodeAssemblerState* state_, wasm::HeapType::Representation p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 82);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=82&c=1
TNode<Int32T> FromConstexpr_HeapType_constexpr_kExn_0(compiler::CodeAssemblerState* state_, wasm::HeapType::Representation p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 82);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=82&c=1
TNode<Int32T> FromConstexpr_HeapType_constexpr_kNoExn_0(compiler::CodeAssemblerState* state_, wasm::HeapType::Representation p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 82);
    tmp0 = ca_.Int32Constant(CastToUnderlyingTypeIfEnum(p_o));
    tmp1 = (TNode<Int32T>{tmp0});
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Int32T>{tmp1};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=150&c=1
TNode<Uint32T> Bitcast_WasmCodePointer_float32_0(compiler::CodeAssemblerState* state_, TNode<Float32T> p_v) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Uint32T> tmp0;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 151);
    tmp0 = CodeStubAssembler(state_).BitcastFloat32ToInt32(TNode<Float32T>{p_v});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 150);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TNode<Uint32T>{tmp0};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=159&c=1
TNode<IntPtrT> TruncateBigIntToI64_0(compiler::CodeAssemblerState* state_, TNode<Context> p_context, TNode<Object> p_input) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block6(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block7(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block8(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block9(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT> block1(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT> block10(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<BigInt> tmp0;
  TNode<IntPtrT> tmp1;
  TNode<IntPtrT> tmp2;
  TNode<BoolT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 162);
    tmp0 = CodeStubAssembler(state_).ToBigInt(TNode<Context>{p_context}, TNode<Object>{p_input});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 164);
    tmp1 = BigIntBuiltinsAssembler(state_).ReadBigIntLength(TNode<BigInt>{tmp0});
    tmp2 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp3 = CodeStubAssembler(state_).WordEqual(TNode<IntPtrT>{tmp1}, TNode<IntPtrT>{tmp2});
    ca_.Branch(tmp3, &block6, std::vector<compiler::Node*>{}, &block7, std::vector<compiler::Node*>{});
  }

  TNode<IntPtrT> tmp4;
  if (block6.is_used()) {
    ca_.Bind(&block6);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 165);
    tmp4 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.Goto(&block1, tmp4);
  }

  TNode<IntPtrT> tmp5;
  TNode<UintPtrT> tmp6;
  TNode<Uint32T> tmp7;
  TNode<Uint32T> tmp8;
  TNode<BoolT> tmp9;
  if (block7.is_used()) {
    ca_.Bind(&block7);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 168);
    tmp5 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp6 = CodeStubAssembler(state_).LoadBigIntDigit(TNode<BigInt>{tmp0}, TNode<IntPtrT>{tmp5});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 169);
    tmp7 = BigIntBuiltinsAssembler(state_).ReadBigIntSign(TNode<BigInt>{tmp0});
    tmp8 = kPositiveSign_0(state_);
    tmp9 = CodeStubAssembler(state_).Word32Equal(TNode<Uint32T>{tmp7}, TNode<Uint32T>{tmp8});
    ca_.Branch(tmp9, &block8, std::vector<compiler::Node*>{}, &block9, std::vector<compiler::Node*>{});
  }

  TNode<IntPtrT> tmp10;
  if (block8.is_used()) {
    ca_.Bind(&block8);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 173);
    tmp10 = CodeStubAssembler(state_).Signed(TNode<UintPtrT>{tmp6});
    ca_.Goto(&block1, tmp10);
  }

  TNode<IntPtrT> tmp11;
  TNode<IntPtrT> tmp12;
  TNode<IntPtrT> tmp13;
  if (block9.is_used()) {
    ca_.Bind(&block9);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 175);
    tmp11 = CodeStubAssembler(state_).Signed(TNode<UintPtrT>{tmp6});
    tmp12 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp13 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{tmp12}, TNode<IntPtrT>{tmp11});
    ca_.Goto(&block1, tmp13);
  }

  TNode<IntPtrT> phi_bb1_2;
  if (block1.is_used()) {
    ca_.Bind(&block1, &phi_bb1_2);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 159);
    ca_.Goto(&block10, phi_bb1_2);
  }

  TNode<IntPtrT> phi_bb10_2;
    ca_.Bind(&block10, &phi_bb10_2);
  return TNode<IntPtrT>{phi_bb10_2};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=273&c=1
TorqueStructReturnSlotAllocator_0 NewReturnSlotAllocator_0(compiler::CodeAssemblerState* state_) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<IntPtrT> tmp0;
  TNode<IntPtrT> tmp1;
  TNode<IntPtrT> tmp2;
  TNode<BoolT> tmp3;
  TNode<BoolT> tmp4;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 275);
    tmp0 = FromConstexpr_intptr_constexpr_intptr_0(state_, arraysize(wasm::kGpReturnRegisters));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 276);
    tmp1 = FromConstexpr_intptr_constexpr_intptr_0(state_, arraysize(wasm::kFpReturnRegisters));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 277);
    tmp2 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 278);
    tmp3 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 279);
    tmp4 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 273);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReturnSlotAllocator_0{TNode<IntPtrT>{tmp0}, TNode<IntPtrT>{tmp1}, TNode<BoolT>{tmp3}, TNode<BoolT>{tmp4}, TNode<IntPtrT>{tmp2}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=406&c=1
TorqueStructLocationAllocator_0 LocationAllocatorForParams_0(compiler::CodeAssemblerState* state_, TorqueStructReference_intptr_0 p_paramBuffer, TNode<IntPtrT> p_paramBufferSize) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block3(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT> block4(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block9(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block10(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT> block11(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block12(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<IntPtrT> tmp0;
  TNode<IntPtrT> tmp1;
  TNode<IntPtrT> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 410);
    tmp0 = CodeStubAssembler(state_).UniqueIntPtrConstant(arraysize(wasm::kGpParamRegisters) - 1);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 411);
    tmp1 = CodeStubAssembler(state_).UniqueIntPtrConstant(arraysize(wasm::kFpParamRegisters));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 413);
    tmp2 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp3 = CodeStubAssembler(state_).IntPtrMul(TNode<IntPtrT>{tmp0}, TNode<IntPtrT>{tmp2});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 414);
    if (((CodeStubAssembler(state_).ConstexprBoolNot((CodeStubAssembler(state_).Is64()))))) {
      ca_.Goto(&block2);
    } else {
      ca_.Goto(&block3);
    }
  }

  TNode<IntPtrT> tmp4;
  TNode<IntPtrT> tmp5;
  TNode<IntPtrT> tmp6;
  if (block2.is_used()) {
    ca_.Bind(&block2);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 416);
    tmp4 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp5 = CodeStubAssembler(state_).WordAnd(TNode<IntPtrT>{tmp3}, TNode<IntPtrT>{tmp4});
    tmp6 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp3}, TNode<IntPtrT>{tmp5});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 414);
    ca_.Goto(&block4, tmp6);
  }

  if (block3.is_used()) {
    ca_.Bind(&block3);
    ca_.Goto(&block4, tmp3);
  }

  TNode<IntPtrT> phi_bb4_7;
  TNode<IntPtrT> tmp7;
  TNode<IntPtrT> tmp8;
  TNode<IntPtrT> tmp9;
  TNode<IntPtrT> tmp10;
  TNode<IntPtrT> tmp11;
  TNode<BoolT> tmp12;
  TNode<IntPtrT> tmp13;
  TNode<BoolT> tmp14;
  if (block4.is_used()) {
    ca_.Bind(&block4, &phi_bb4_7);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 419);
    tmp7 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb4_7}, TNode<IntPtrT>{p_paramBuffer.offset});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 421);
    tmp8 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_float64_0(state_)));
    tmp9 = CodeStubAssembler(state_).IntPtrMul(TNode<IntPtrT>{tmp1}, TNode<IntPtrT>{tmp8});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 420);
    tmp10 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp7}, TNode<IntPtrT>{tmp9});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 423);
    tmp11 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 424);
    tmp12 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 425);
    tmp13 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp14 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{p_paramBufferSize}, TNode<IntPtrT>{tmp13});
    ca_.Branch(tmp14, &block9, std::vector<compiler::Node*>{}, &block10, std::vector<compiler::Node*>{});
  }

  TNode<IntPtrT> tmp15;
  if (block9.is_used()) {
    ca_.Bind(&block9);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 426);
    tmp15 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{p_paramBuffer.offset}, TNode<IntPtrT>{p_paramBufferSize});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 425);
    ca_.Goto(&block11, tmp15);
  }

  TNode<IntPtrT> tmp16;
  if (block10.is_used()) {
    ca_.Bind(&block10);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 428);
    tmp16 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 425);
    ca_.Goto(&block11, tmp16);
  }

  TNode<IntPtrT> phi_bb11_10;
  if (block11.is_used()) {
    ca_.Bind(&block11, &phi_bb11_10);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 406);
    ca_.Goto(&block12);
  }

    ca_.Bind(&block12);
  return TorqueStructLocationAllocator_0{TNode<Object>{p_paramBuffer.object}, TNode<IntPtrT>{tmp0}, TNode<IntPtrT>{tmp1}, TNode<IntPtrT>{p_paramBuffer.offset}, TNode<IntPtrT>{tmp7}, TNode<IntPtrT>{tmp10}, TNode<IntPtrT>{tmp10}, TNode<IntPtrT>{phi_bb11_10}, TNode<IntPtrT>{tmp11}, TNode<BoolT>{tmp12}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=433&c=1
TorqueStructLocationAllocator_0 LocationAllocatorForReturns_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_gpRegs, TNode<RawPtrT> p_fpRegs, TNode<RawPtrT> p_stack) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<TaggedIndex> tmp0;
  TNode<IntPtrT> tmp1;
  TNode<IntPtrT> tmp2;
  TNode<IntPtrT> tmp3;
  TNode<IntPtrT> tmp4;
  TNode<IntPtrT> tmp5;
  TNode<IntPtrT> tmp6;
  TNode<IntPtrT> tmp7;
  TNode<IntPtrT> tmp8;
  TNode<IntPtrT> tmp9;
  TNode<IntPtrT> tmp10;
  TNode<IntPtrT> tmp11;
  TNode<IntPtrT> tmp12;
  TNode<BoolT> tmp13;
  TNode<IntPtrT> tmp14;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 436);
    tmp0 = kZeroBitPattern_0(state_);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 437);
    tmp1 = FromConstexpr_intptr_constexpr_intptr_0(state_, arraysize(wasm::kGpReturnRegisters));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 438);
    tmp2 = FromConstexpr_intptr_constexpr_intptr_0(state_, arraysize(wasm::kFpReturnRegisters));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 439);
    tmp3 = Convert_intptr_RawPtr_0(state_, TNode<RawPtrT>{p_gpRegs});
    tmp4 = FromConstexpr_intptr_constexpr_int31_0(state_, kHeapObjectTag);
    tmp5 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp3}, TNode<IntPtrT>{tmp4});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 440);
    tmp6 = Convert_intptr_RawPtr_0(state_, TNode<RawPtrT>{p_fpRegs});
    tmp7 = FromConstexpr_intptr_constexpr_int31_0(state_, kHeapObjectTag);
    tmp8 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp6}, TNode<IntPtrT>{tmp7});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 441);
    tmp9 = Convert_intptr_RawPtr_0(state_, TNode<RawPtrT>{p_stack});
    tmp10 = FromConstexpr_intptr_constexpr_int31_0(state_, kHeapObjectTag);
    tmp11 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp9}, TNode<IntPtrT>{tmp10});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 443);
    tmp12 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 444);
    tmp13 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 446);
    tmp14 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 433);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructLocationAllocator_0{TNode<Object>{tmp0}, TNode<IntPtrT>{tmp1}, TNode<IntPtrT>{tmp2}, TNode<IntPtrT>{tmp5}, TNode<IntPtrT>{tmp8}, TNode<IntPtrT>{tmp11}, TNode<IntPtrT>{tmp11}, TNode<IntPtrT>{tmp14}, TNode<IntPtrT>{tmp12}, TNode<BoolT>{tmp13}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=450&c=1
TNode<Object> JSToWasmObject_0(compiler::CodeAssemblerState* state_, TNode<NativeContext> p_context, TNode<HeapObject> p_trustedInstanceDataOrUndefined, TNode<Int32T> p_targetType, TNode<Object> p_value) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block6(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block7(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block8(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block4(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block5(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block3(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block9(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block11(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block12(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block13(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block14(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block15(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block17(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block18(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block21(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block20(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block16(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block10(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object> block1(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object> block22(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  TNode<Int32T> tmp2;
  TNode<Int32T> tmp3;
  TNode<Int32T> tmp4;
  TNode<Int32T> tmp5;
  TNode<Int32T> tmp6;
  TNode<BoolT> tmp7;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 454);
    tmp0 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueType::kKindBits);
    tmp1 = CodeStubAssembler(state_).Word32Sar(TNode<Int32T>{p_targetType}, TNode<Int32T>{tmp0});
    tmp2 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmHeapTypeBitsMask);
    tmp3 = CodeStubAssembler(state_).Word32And(TNode<Int32T>{tmp1}, TNode<Int32T>{tmp2});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 455);
    tmp4 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmValueKindBitsMask);
    tmp5 = CodeStubAssembler(state_).Word32And(TNode<Int32T>{p_targetType}, TNode<Int32T>{tmp4});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 456);
    tmp6 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kExtern);
    tmp7 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp3}, TNode<Int32T>{tmp6});
    ca_.Branch(tmp7, &block2, std::vector<compiler::Node*>{}, &block3, std::vector<compiler::Node*>{});
  }

  TNode<Int32T> tmp8;
  TNode<BoolT> tmp9;
  if (block2.is_used()) {
    ca_.Bind(&block2);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 457);
    tmp8 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRef);
    tmp9 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp8});
    ca_.Branch(tmp9, &block6, std::vector<compiler::Node*>{}, &block7, std::vector<compiler::Node*>{});
  }

  TNode<Null> tmp10;
  TNode<BoolT> tmp11;
  if (block6.is_used()) {
    ca_.Bind(&block6);
    tmp10 = Null_0(state_);
    tmp11 = CodeStubAssembler(state_).TaggedEqual(TNode<Object>{p_value}, TNode<HeapObject>{tmp10});
    ca_.Goto(&block8, tmp11);
  }

  TNode<BoolT> tmp12;
  if (block7.is_used()) {
    ca_.Bind(&block7);
    tmp12 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.Goto(&block8, tmp12);
  }

  TNode<BoolT> phi_bb8_7;
  if (block8.is_used()) {
    ca_.Bind(&block8, &phi_bb8_7);
    ca_.Branch(phi_bb8_7, &block4, std::vector<compiler::Node*>{}, &block5, std::vector<compiler::Node*>{});
  }

  if (block4.is_used()) {
    ca_.Bind(&block4);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 458);
    CodeStubAssembler(state_).ThrowTypeError(TNode<Context>{p_context}, MessageTemplate::kWasmTrapJSTypeError);
  }

  if (block5.is_used()) {
    ca_.Bind(&block5);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 460);
    ca_.Goto(&block1, p_value);
  }

  TNode<Int32T> tmp13;
  TNode<BoolT> tmp14;
  if (block3.is_used()) {
    ca_.Bind(&block3);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 462);
    tmp13 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kString);
    tmp14 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp3}, TNode<Int32T>{tmp13});
    ca_.Branch(tmp14, &block9, std::vector<compiler::Node*>{}, &block10, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp15;
  if (block9.is_used()) {
    ca_.Bind(&block9);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 463);
    tmp15 = CodeStubAssembler(state_).TaggedIsSmi(TNode<Object>{p_value});
    ca_.Branch(tmp15, &block11, std::vector<compiler::Node*>{}, &block12, std::vector<compiler::Node*>{});
  }

  if (block11.is_used()) {
    ca_.Bind(&block11);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 464);
    CodeStubAssembler(state_).ThrowTypeError(TNode<Context>{p_context}, MessageTemplate::kWasmTrapJSTypeError);
  }

  TNode<HeapObject> tmp16;
  TNode<BoolT> tmp17;
  if (block12.is_used()) {
    ca_.Bind(&block12);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 466);
    tmp16 = UnsafeCast_HeapObject_0(state_, TNode<Context>{p_context}, TNode<Object>{p_value});
    tmp17 = CodeStubAssembler(state_).IsString(TNode<HeapObject>{tmp16});
    ca_.Branch(tmp17, &block13, std::vector<compiler::Node*>{}, &block14, std::vector<compiler::Node*>{});
  }

  if (block13.is_used()) {
    ca_.Bind(&block13);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 467);
    ca_.Goto(&block1, p_value);
  }

  TNode<Null> tmp18;
  TNode<BoolT> tmp19;
  if (block14.is_used()) {
    ca_.Bind(&block14);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 469);
    tmp18 = Null_0(state_);
    tmp19 = CodeStubAssembler(state_).TaggedEqual(TNode<Object>{p_value}, TNode<HeapObject>{tmp18});
    ca_.Branch(tmp19, &block15, std::vector<compiler::Node*>{}, &block16, std::vector<compiler::Node*>{});
  }

  TNode<Int32T> tmp20;
  TNode<BoolT> tmp21;
  if (block15.is_used()) {
    ca_.Bind(&block15);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 470);
    tmp20 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRef);
    tmp21 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp20});
    ca_.Branch(tmp21, &block17, std::vector<compiler::Node*>{}, &block18, std::vector<compiler::Node*>{});
  }

  if (block17.is_used()) {
    ca_.Bind(&block17);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 471);
    CodeStubAssembler(state_).ThrowTypeError(TNode<Context>{p_context}, MessageTemplate::kWasmTrapJSTypeError);
  }

  TNode<Int32T> tmp22;
  TNode<BoolT> tmp23;
  if (block18.is_used()) {
    ca_.Bind(&block18);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 473);
    tmp22 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRefNull);
    tmp23 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp22});
    ca_.Branch(tmp23, &block20, std::vector<compiler::Node*>{}, &block21, std::vector<compiler::Node*>{});
  }

  if (block21.is_used()) {
    ca_.Bind(&block21);
    {
      auto pos_stack = ca_.GetMacroSourcePositionStack();
      pos_stack.push_back({"src/builtins/js-to-wasm.tq", 473});
      CodeStubAssembler(state_).FailAssert("Torque assert 'kind == ValueKind::kRefNull' failed", pos_stack);
    }
  }

  TNode<WasmNull> tmp24;
  if (block20.is_used()) {
    ca_.Bind(&block20);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 474);
    tmp24 = kWasmNull_0(state_);
    ca_.Goto(&block1, tmp24);
  }

  if (block16.is_used()) {
    ca_.Bind(&block16);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 478);
    CodeStubAssembler(state_).ThrowTypeError(TNode<Context>{p_context}, MessageTemplate::kWasmTrapJSTypeError);
  }

  TNode<Smi> tmp25;
  TNode<Object> tmp26;
  if (block10.is_used()) {
    ca_.Bind(&block10);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 481);
    tmp25 = Convert_Smi_int32_0(state_, TNode<Int32T>{p_targetType});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 480);
    tmp26 = CodeStubAssembler(state_).CallRuntime(Runtime::kWasmGenericJSToWasmObject, p_context, p_trustedInstanceDataOrUndefined, p_value, tmp25); 
    ca_.Goto(&block1, tmp26);
  }

  TNode<Object> phi_bb1_4;
  if (block1.is_used()) {
    ca_.Bind(&block1, &phi_bb1_4);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 450);
    ca_.Goto(&block22, phi_bb1_4);
  }

  TNode<Object> phi_bb22_4;
    ca_.Bind(&block22, &phi_bb22_4);
  return TNode<Object>{phi_bb22_4};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=485&c=1
void HandleF32Params_0(compiler::CodeAssemblerState* state_, TNode<NativeContext> p_context, TorqueStructLocationAllocator_0 p_locationAllocator, TorqueStructReference_intptr_0 p_toRef, TNode<Object> p_param) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block5(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block6(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block8(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block3(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block9(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block10(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block12(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block15(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block16(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block18(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block13(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block14(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block11(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block4(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block19(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 488);
    if ((wasm::kIsFpAlwaysDouble)) {
      ca_.Goto(&block2);
    } else {
      ca_.Goto(&block3);
    }
  }

  TNode<IntPtrT> tmp0;
  TNode<BoolT> tmp1;
  if (block2.is_used()) {
    ca_.Bind(&block2);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 489);
    tmp0 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp1 = CodeStubAssembler(state_).IntPtrGreaterThanOrEqual(TNode<IntPtrT>{p_locationAllocator.remainingFPRegs}, TNode<IntPtrT>{tmp0});
    ca_.Branch(tmp1, &block5, std::vector<compiler::Node*>{}, &block6, std::vector<compiler::Node*>{});
  }

  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  TNode<Float32T> tmp4;
  TNode<Float64T> tmp5;
  TNode<Float64T> tmp6;
  if (block5.is_used()) {
    ca_.Bind(&block5);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 490);
    std::tie(tmp2, tmp3) = RefCast_float64_0(state_, TorqueStructReference_intptr_0{TNode<Object>{p_toRef.object}, TNode<IntPtrT>{p_toRef.offset}, TorqueStructUnsafe_0{}}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 491);
    tmp4 = ca_.CallBuiltin<Float32T>(Builtin::kWasmTaggedToFloat32, p_context, p_param);
    tmp5 = CodeStubAssembler(state_).ChangeFloat32ToFloat64(TNode<Float32T>{tmp4});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 490);
    tmp6 = CodeStubAssembler(state_).Float64SilenceNaN(TNode<Float64T>{tmp5});
    CodeStubAssembler(state_).StoreReference<Float64T>(CodeStubAssembler::Reference{tmp2, tmp3}, tmp6);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 489);
    ca_.Goto(&block8);
  }

  TNode<Object> tmp7;
  TNode<IntPtrT> tmp8;
  TNode<Float32T> tmp9;
  if (block6.is_used()) {
    ca_.Bind(&block6);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 493);
    std::tie(tmp7, tmp8) = RefCast_float32_0(state_, TorqueStructReference_intptr_0{TNode<Object>{p_toRef.object}, TNode<IntPtrT>{p_toRef.offset}, TorqueStructUnsafe_0{}}).Flatten();
    tmp9 = ca_.CallBuiltin<Float32T>(Builtin::kWasmTaggedToFloat32, p_context, p_param);
    CodeStubAssembler(state_).StoreReference<Float32T>(CodeStubAssembler::Reference{tmp7, tmp8}, tmp9);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 489);
    ca_.Goto(&block8);
  }

  if (block8.is_used()) {
    ca_.Bind(&block8);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 488);
    ca_.Goto(&block4);
  }

  if (block3.is_used()) {
    ca_.Bind(&block3);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 495);
    if ((wasm::kIsBigEndian)) {
      ca_.Goto(&block9);
    } else {
      ca_.Goto(&block10);
    }
  }

  TNode<Float32T> tmp10;
  TNode<Uint32T> tmp11;
  TNode<IntPtrT> tmp12;
  TNode<IntPtrT> tmp13;
  TNode<IntPtrT> tmp14;
  if (block9.is_used()) {
    ca_.Bind(&block9);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 496);
    tmp10 = ca_.CallBuiltin<Float32T>(Builtin::kWasmTaggedToFloat32, p_context, p_param);
    tmp11 = Bitcast_WasmCodePointer_float32_0(state_, TNode<Float32T>{tmp10});
    tmp12 = Convert_intptr_WasmCodePointer_0(state_, TNode<Uint32T>{tmp11});
    tmp13 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x20ull));
    tmp14 = CodeStubAssembler(state_).WordShl(TNode<IntPtrT>{tmp12}, TNode<IntPtrT>{tmp13});
    CodeStubAssembler(state_).StoreReference<IntPtrT>(CodeStubAssembler::Reference{p_toRef.object, p_toRef.offset}, tmp14);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 495);
    ca_.Goto(&block11);
  }

  if (block10.is_used()) {
    ca_.Bind(&block10);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 497);
    if ((wasm::kIsBigEndianOnSim)) {
      ca_.Goto(&block12);
    } else {
      ca_.Goto(&block13);
    }
  }

  TNode<IntPtrT> tmp15;
  TNode<BoolT> tmp16;
  if (block12.is_used()) {
    ca_.Bind(&block12);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 498);
    tmp15 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp16 = CodeStubAssembler(state_).IntPtrGreaterThanOrEqual(TNode<IntPtrT>{p_locationAllocator.remainingFPRegs}, TNode<IntPtrT>{tmp15});
    ca_.Branch(tmp16, &block15, std::vector<compiler::Node*>{}, &block16, std::vector<compiler::Node*>{});
  }

  TNode<Float32T> tmp17;
  TNode<Uint32T> tmp18;
  TNode<IntPtrT> tmp19;
  TNode<IntPtrT> tmp20;
  TNode<IntPtrT> tmp21;
  if (block15.is_used()) {
    ca_.Bind(&block15);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 499);
    tmp17 = ca_.CallBuiltin<Float32T>(Builtin::kWasmTaggedToFloat32, p_context, p_param);
    tmp18 = Bitcast_WasmCodePointer_float32_0(state_, TNode<Float32T>{tmp17});
    tmp19 = Convert_intptr_WasmCodePointer_0(state_, TNode<Uint32T>{tmp18});
    tmp20 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x20ull));
    tmp21 = CodeStubAssembler(state_).WordShl(TNode<IntPtrT>{tmp19}, TNode<IntPtrT>{tmp20});
    CodeStubAssembler(state_).StoreReference<IntPtrT>(CodeStubAssembler::Reference{p_toRef.object, p_toRef.offset}, tmp21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 498);
    ca_.Goto(&block18);
  }

  TNode<Float32T> tmp22;
  TNode<Uint32T> tmp23;
  TNode<IntPtrT> tmp24;
  if (block16.is_used()) {
    ca_.Bind(&block16);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 502);
    tmp22 = ca_.CallBuiltin<Float32T>(Builtin::kWasmTaggedToFloat32, p_context, p_param);
    tmp23 = Bitcast_WasmCodePointer_float32_0(state_, TNode<Float32T>{tmp22});
    tmp24 = Convert_intptr_WasmCodePointer_0(state_, TNode<Uint32T>{tmp23});
    CodeStubAssembler(state_).StoreReference<IntPtrT>(CodeStubAssembler::Reference{p_toRef.object, p_toRef.offset}, tmp24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 498);
    ca_.Goto(&block18);
  }

  if (block18.is_used()) {
    ca_.Bind(&block18);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 497);
    ca_.Goto(&block14);
  }

  if (block13.is_used()) {
    ca_.Bind(&block13);
    ca_.Goto(&block14);
  }

  if (block14.is_used()) {
    ca_.Bind(&block14);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 495);
    ca_.Goto(&block11);
  }

  if (block11.is_used()) {
    ca_.Bind(&block11);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 488);
    ca_.Goto(&block4);
  }

  if (block4.is_used()) {
    ca_.Bind(&block4);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 485);
    ca_.Goto(&block19);
  }

    ca_.Bind(&block19);
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=507&c=1
TNode<Object> JSToWasmWrapperHelper_0(compiler::CodeAssemblerState* state_, TNode<NativeContext> p_context, TNode<Object> p__receiver, TNode<JSFunction> p_target, TorqueStructArguments p_arguments, wasm::Promise p_promise) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block5(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block6(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block3(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block4(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block7(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block12(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block11(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, IntPtrT> block16(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, IntPtrT> block14(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block25(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block29(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block30(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block32(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block33(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block35(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block36(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block37(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block34(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block28(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block26(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block38(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block42(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block43(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block45(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block46(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block48(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block49(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block50(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block47(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block41(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block51(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block55(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block56(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block58(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block59(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block61(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block62(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block63(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block60(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block54(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block52(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block53(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block39(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block64(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block68(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block69(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block71(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block72(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block74(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block75(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block76(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block73(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block67(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block65(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block77(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block81(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block82(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block83(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block84(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block85(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block80(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block78(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block88(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block89(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, BoolT> block90(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block87(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block86(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block79(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block66(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block40(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block27(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, IntPtrT> block15(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, IntPtrT> block91(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, IntPtrT> block94(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, IntPtrT> block95(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, IntPtrT> block99(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, IntPtrT> block97(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block110(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block111(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, BoolT> block112(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block108(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block114(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block115(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block117(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block118(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block120(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block121(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block122(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block119(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block113(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT> block109(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, IntPtrT> block98(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, IntPtrT, IntPtrT> block92(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, IntPtrT, IntPtrT> block124(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, IntPtrT, IntPtrT> block125(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT, IntPtrT, IntPtrT, BoolT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block123(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT> block8(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT> block130(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT> block129(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT> block131(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT> block132(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, Object, IntPtrT, IntPtrT> block133(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT> block144(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT> block145(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT> block146(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT> block147(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT> block148(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block151(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block153(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block149(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block154(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block155(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T, Int32T> block156(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block157(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block158(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block162(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block163(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block160(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block166(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block167(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block169(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block170(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block172(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block173(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block168(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block165(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block178(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block177(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block176(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT> block179(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT> block180(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT> block181(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT> block182(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block174(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block161(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block185(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block186(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block183(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block189(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block190(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block192(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block193(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block195(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block196(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block191(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block188(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block197(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, Object, IntPtrT> block200(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block198(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT> block201(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT> block202(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT> block203(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block199(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block184(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block206(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block207(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block204(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block210(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block211(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block212(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block216(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block217(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block219(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block220(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block215(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block213(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block209(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT> block221(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block222(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block223(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block205(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block226(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block227(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block224(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block229(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block233(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block234(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block236(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block237(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block239(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block240(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block235(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block232(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block241(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block230(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block243(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block244(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block246(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block247(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block249(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block250(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block245(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block242(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block252(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block253(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block255(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block256(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block258(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT> block259(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT> block254(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT> block251(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT> block260(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT> block261(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT, Object, IntPtrT> block262(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object, IntPtrT, Object, IntPtrT, Object, IntPtrT> block263(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block231(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block225(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block264(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block265(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block268(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block269(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block270(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block273(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block271(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block274(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block275(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, BoolT> block272(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block267(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block266(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block276(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, Object, Int32T, Object> block279(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, Object, Int32T> block277(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, Object, Int32T> block280(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, Object, Int32T> block281(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, Object, Int32T> block278(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, Int32T> block282(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, Int32T> block283(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, Int32T, Int32T> block284(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, Int32T> block289(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, Int32T> block290(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, Int32T> block293(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block228(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block208(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block187(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block164(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block294(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block295(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block150(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block296(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block299(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block302(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block300(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block303(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block304(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block301(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block297(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object> block298(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block305(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block307(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block308(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block310(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block311(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block314(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T, Int32T> block315(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block312(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T, Int32T> block316(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block318(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block319(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block322(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block323(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block324(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block327(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block328(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block325(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block329(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T, BoolT> block326(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block320(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T, Int32T, Int32T> block330(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T, Int32T> block331(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T, Int32T> block332(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block333(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block321(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T, Object> block334(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block335(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T, Int32T, Int32T> block336(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T, Int32T> block337(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block338(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block343(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block344(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block348(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block349(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block351(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block352(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block354(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T> block355(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Object, IntPtrT> block350(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Object, IntPtrT> block347(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Object, IntPtrT, Object, IntPtrT> block356(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Object, IntPtrT, Int32T, Int32T> block357(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Object, IntPtrT, Int32T> block358(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Int32T, Int32T> block313(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block306(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, IntPtrT> block361(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, IntPtrT> block362(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block366(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block367(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, BoolT> block368(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block365(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block364(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block369(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block370(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block371(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block372(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block373(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block374(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block375(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block376(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block377(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block378(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block379(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block380(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block381(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block382(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block383(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block384(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block385(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block386(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object> block389(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block387(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block392(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block390(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object> block394(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block391(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block397(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block395(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object> block399(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T> block396(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object> block393(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object> block388(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object> block402(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object> block403(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object> block400(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object> block405(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT, Object, Int32T, Object> block401(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT, Object, HeapObject> block143(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT> block406(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object, IntPtrT, BoolT> block407(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object> block409(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<IntPtrT> tmp0;
  TNode<SharedFunctionInfo> tmp1;
  TNode<WasmExportedFunctionData> tmp2;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 510);
    tmp0 = FromConstexpr_intptr_constexpr_int31_0(state_, 16);
    tmp1 = CodeStubAssembler(state_).LoadReference<SharedFunctionInfo>(CodeStubAssembler::Reference{p_target, tmp0});
    tmp2 = CodeStubAssembler(state_).LoadSharedFunctionInfoWasmExportedFunctionData(TNode<SharedFunctionInfo>{tmp1});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 515);
    if (((CodeStubAssembler(state_).ConstexprBoolNot(((CodeStubAssembler(state_).ConstexprInt32Equal(p_promise, wasm::Promise::kPromise)) || (CodeStubAssembler(state_).ConstexprInt32Equal(p_promise, wasm::Promise::kStressSwitch))))))) {
      ca_.Goto(&block2);
    } else {
      ca_.Goto(&block3);
    }
  }

  TNode<IntPtrT> tmp3;
  TNode<Cell> tmp4;
  TNode<IntPtrT> tmp5;
  TNode<Object> tmp6;
  TNode<Smi> tmp7;
  TNode<Smi> tmp8;
  TNode<Smi> tmp9;
  TNode<IntPtrT> tmp10;
  TNode<Cell> tmp11;
  TNode<IntPtrT> tmp12;
  TNode<Smi> tmp13;
  TNode<BoolT> tmp14;
  if (block2.is_used()) {
    ca_.Bind(&block2);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 517);
    tmp3 = FromConstexpr_intptr_constexpr_int31_0(state_, 32);
    tmp4 = CodeStubAssembler(state_).LoadReference<Cell>(CodeStubAssembler::Reference{tmp2, tmp3});
    tmp5 = FromConstexpr_intptr_constexpr_int31_0(state_, 4);
    tmp6 = CodeStubAssembler(state_).LoadReference<Object>(CodeStubAssembler::Reference{tmp4, tmp5});
    tmp7 = UnsafeCast_Smi_0(state_, TNode<Context>{p_context}, TNode<Object>{tmp6});
    tmp8 = SmiConstant_0(state_, IntegerLiteral(false, 0x1ull));
    tmp9 = CodeStubAssembler(state_).SmiSub(TNode<Smi>{tmp7}, TNode<Smi>{tmp8});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 518);
    tmp10 = FromConstexpr_intptr_constexpr_int31_0(state_, 32);
    tmp11 = CodeStubAssembler(state_).LoadReference<Cell>(CodeStubAssembler::Reference{tmp2, tmp10});
    tmp12 = FromConstexpr_intptr_constexpr_int31_0(state_, 4);
    CodeStubAssembler(state_).StoreReference<Object>(CodeStubAssembler::Reference{tmp11, tmp12}, tmp9);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 519);
    tmp13 = SmiConstant_0(state_, IntegerLiteral(false, 0x0ull));
    tmp14 = CodeStubAssembler(state_).SmiEqual(TNode<Smi>{tmp9}, TNode<Smi>{tmp13});
    ca_.Branch(tmp14, &block5, std::vector<compiler::Node*>{}, &block6, std::vector<compiler::Node*>{});
  }

  TNode<Smi> tmp15;
  TNode<Object> tmp16;
  if (block5.is_used()) {
    ca_.Bind(&block5);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 520);
    tmp15 = kNoContext_0(state_);
    tmp16 = CodeStubAssembler(state_).CallRuntime(Runtime::kTierUpJSToWasmWrapper, tmp15, tmp2); 
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 519);
    ca_.Goto(&block6);
  }

  if (block6.is_used()) {
    ca_.Bind(&block6);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 515);
    ca_.Goto(&block4);
  }

  if (block3.is_used()) {
    ca_.Bind(&block3);
    ca_.Goto(&block4);
  }

  TNode<IntPtrT> tmp17;
  TNode<RawPtrT> tmp18;
  TNode<WasmInternalFunction> tmp19;
  TNode<TrustedObject> tmp20;
  TNode<WasmTrustedInstanceData> tmp21;
  TNode<IntPtrT> tmp22;
  TNode<Object> tmp23;
  TNode<IntPtrT> tmp24;
  TNode<IntPtrT> tmp25;
  TNode<Int32T> tmp26;
  TNode<IntPtrT> tmp27;
  TNode<Object> tmp28;
  TNode<IntPtrT> tmp29;
  TNode<IntPtrT> tmp30;
  TNode<Int32T> tmp31;
  TNode<IntPtrT> tmp32;
  TNode<Object> tmp33;
  TNode<IntPtrT> tmp34;
  TNode<RawPtrT> tmp35;
  TNode<RawPtrT> tmp36;
  TNode<Int32T> tmp37;
  TNode<IntPtrT> tmp38;
  TNode<Object> tmp39;
  TNode<IntPtrT> tmp40;
  TNode<IntPtrT> tmp41;
  TNode<Undefined> tmp42;
  TNode<IntPtrT> tmp43;
  TNode<BoolT> tmp44;
  TNode<Int32T> tmp45;
  TNode<BoolT> tmp46;
  if (block4.is_used()) {
    ca_.Bind(&block4);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 524);
    tmp17 = FromConstexpr_intptr_constexpr_int31_0(state_, 48);
    tmp18 = CodeStubAssembler(state_).LoadReference<RawPtrT>(CodeStubAssembler::Reference{tmp2, tmp17});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 526);
    tmp19 = CodeStubAssembler(state_).LoadWasmInternalFunctionFromFunctionData(TNode<WasmFunctionData>{tmp2});
    tmp20 = CodeStubAssembler(state_).LoadImplicitArgFromWasmInternalFunction(TNode<WasmInternalFunction>{tmp19});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 528);
    tmp21 = CodeStubAssembler(state_).LoadWasmTrustedInstanceDataFromWasmExportedFunctionData(TNode<WasmExportedFunctionData>{tmp2});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 531);
    tmp22 = FromConstexpr_intptr_constexpr_intptr_0(state_, wasm::FunctionSig::kParameterCountOffset);
    std::tie(tmp23, tmp24) = GetRefAt_intptr_RawFunctionSigPtr_0(state_, TNode<RawPtrT>{tmp18}, TNode<IntPtrT>{tmp22}).Flatten();
    tmp25 = CodeStubAssembler(state_).LoadReference<IntPtrT>(CodeStubAssembler::Reference{tmp23, tmp24});
    tmp26 = CodeStubAssembler(state_).TruncateIntPtrToInt32(TNode<IntPtrT>{tmp25});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 534);
    tmp27 = FromConstexpr_intptr_constexpr_intptr_0(state_, wasm::FunctionSig::kReturnCountOffset);
    std::tie(tmp28, tmp29) = GetRefAt_intptr_RawFunctionSigPtr_0(state_, TNode<RawPtrT>{tmp18}, TNode<IntPtrT>{tmp27}).Flatten();
    tmp30 = CodeStubAssembler(state_).LoadReference<IntPtrT>(CodeStubAssembler::Reference{tmp28, tmp29});
    tmp31 = CodeStubAssembler(state_).TruncateIntPtrToInt32(TNode<IntPtrT>{tmp30});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 536);
    tmp32 = FromConstexpr_intptr_constexpr_intptr_0(state_, wasm::FunctionSig::kRepsOffset);
    std::tie(tmp33, tmp34) = GetRefAt_RawPtr_RawFunctionSigPtr_0(state_, TNode<RawPtrT>{tmp18}, TNode<IntPtrT>{tmp32}).Flatten();
    tmp35 = CodeStubAssembler(state_).LoadReference<RawPtrT>(CodeStubAssembler::Reference{tmp33, tmp34});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 539);
    tmp36 = (TNode<RawPtrT>{tmp35});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 540);
    tmp37 = CodeStubAssembler(state_).Int32Add(TNode<Int32T>{tmp26}, TNode<Int32T>{tmp31});
    tmp38 = Convert_intptr_int32_0(state_, TNode<Int32T>{tmp37});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 538);
    std::tie(tmp39, tmp40, tmp41) = NewOffHeapConstSlice_int32_0(state_, TNode<RawPtrT>{tmp36}, TNode<IntPtrT>{tmp38}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 547);
    tmp42 = Undefined_0(state_);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 548);
    tmp43 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 549);
    tmp44 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 550);
    tmp45 = FromConstexpr_int32_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x1ull));
    tmp46 = CodeStubAssembler(state_).Int32GreaterThan(TNode<Int32T>{tmp31}, TNode<Int32T>{tmp45});
    ca_.Branch(tmp46, &block7, std::vector<compiler::Node*>{}, &block8, std::vector<compiler::Node*>{tmp42, tmp43, tmp44});
  }

  TNode<Smi> tmp47;
  TNode<JSArray> tmp48;
  TNode<IntPtrT> tmp49;
  TNode<IntPtrT> tmp50;
  TNode<Object> tmp51;
  TNode<IntPtrT> tmp52;
  TNode<IntPtrT> tmp53;
  if (block7.is_used()) {
    ca_.Bind(&block7);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 551);
    tmp47 = Convert_Smi_int32_0(state_, TNode<Int32T>{tmp31});
    tmp48 = ca_.CallBuiltin<JSArray>(Builtin::kWasmAllocateJSArray, p_context, tmp47);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 556);
    tmp49 = Convert_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp50 = Convert_intptr_int32_0(state_, TNode<Int32T>{tmp31});
    compiler::CodeAssemblerLabel label54(&ca_);
    std::tie(tmp51, tmp52, tmp53) = Subslice_int32_0(state_, TorqueStructSlice_int32_ConstReference_int32_0{TNode<Object>{tmp39}, TNode<IntPtrT>{tmp40}, TNode<IntPtrT>{tmp41}, TorqueStructUnsafe_0{}}, TNode<IntPtrT>{tmp49}, TNode<IntPtrT>{tmp50}, &label54).Flatten();
    ca_.Goto(&block11);
    if (label54.is_used()) {
      ca_.Bind(&label54);
      ca_.Goto(&block12);
    }
  }

  if (block12.is_used()) {
    ca_.Bind(&block12);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 557);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<IntPtrT> tmp55;
  TNode<IntPtrT> tmp56;
  TNode<BoolT> tmp57;
  TNode<BoolT> tmp58;
  TNode<IntPtrT> tmp59;
  TNode<IntPtrT> tmp60;
  TNode<IntPtrT> tmp61;
  if (block11.is_used()) {
    ca_.Bind(&block11);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 558);
    std::tie(tmp55, tmp56, tmp57, tmp58, tmp59) = NewReturnSlotAllocator_0(state_).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 140);
    tmp60 = TimesSizeOf_int32_0(state_, TNode<IntPtrT>{tmp53});
    tmp61 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp52}, TNode<IntPtrT>{tmp60});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 561);
    ca_.Goto(&block16, tmp44, tmp55, tmp56, tmp57, tmp58, tmp59, tmp52);
  }

  TNode<BoolT> phi_bb16_19;
  TNode<IntPtrT> phi_bb16_23;
  TNode<IntPtrT> phi_bb16_24;
  TNode<BoolT> phi_bb16_25;
  TNode<BoolT> phi_bb16_26;
  TNode<IntPtrT> phi_bb16_27;
  TNode<IntPtrT> phi_bb16_29;
  TNode<BoolT> tmp62;
  TNode<BoolT> tmp63;
  if (block16.is_used()) {
    ca_.Bind(&block16, &phi_bb16_19, &phi_bb16_23, &phi_bb16_24, &phi_bb16_25, &phi_bb16_26, &phi_bb16_27, &phi_bb16_29);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 223);
    tmp62 = CodeStubAssembler(state_).WordEqual(TNode<IntPtrT>{phi_bb16_29}, TNode<IntPtrT>{tmp61});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 561);
    tmp63 = CodeStubAssembler(state_).Word32BinaryNot(TNode<BoolT>{tmp62});
    ca_.Branch(tmp63, &block14, std::vector<compiler::Node*>{phi_bb16_19, phi_bb16_23, phi_bb16_24, phi_bb16_25, phi_bb16_26, phi_bb16_27, phi_bb16_29}, &block15, std::vector<compiler::Node*>{phi_bb16_19, phi_bb16_23, phi_bb16_24, phi_bb16_25, phi_bb16_26, phi_bb16_27, phi_bb16_29});
  }

  TNode<BoolT> phi_bb14_19;
  TNode<IntPtrT> phi_bb14_23;
  TNode<IntPtrT> phi_bb14_24;
  TNode<BoolT> phi_bb14_25;
  TNode<BoolT> phi_bb14_26;
  TNode<IntPtrT> phi_bb14_27;
  TNode<IntPtrT> phi_bb14_29;
  TNode<Object> tmp64;
  TNode<IntPtrT> tmp65;
  TNode<IntPtrT> tmp66;
  TNode<IntPtrT> tmp67;
  TNode<Int32T> tmp68;
  TNode<Int32T> tmp69;
  TNode<BoolT> tmp70;
  if (block14.is_used()) {
    ca_.Bind(&block14, &phi_bb14_19, &phi_bb14_23, &phi_bb14_24, &phi_bb14_25, &phi_bb14_26, &phi_bb14_27, &phi_bb14_29);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 246);
    std::tie(tmp64, tmp65) = NewReference_int32_0(state_, TNode<Object>{tmp51}, TNode<IntPtrT>{phi_bb14_29}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 247);
    tmp66 = FromConstexpr_intptr_constexpr_int31_0(state_, kInt32Size);
    tmp67 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb14_29}, TNode<IntPtrT>{tmp66});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 231);
    tmp68 = CodeStubAssembler(state_).LoadReference<Int32T>(CodeStubAssembler::Reference{tmp64, tmp65});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 563);
    tmp69 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmI32.raw_bit_field());
    tmp70 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp68}, TNode<Int32T>{tmp69});
    ca_.Branch(tmp70, &block25, std::vector<compiler::Node*>{phi_bb14_19, phi_bb14_23, phi_bb14_24, phi_bb14_25, phi_bb14_26, phi_bb14_27}, &block26, std::vector<compiler::Node*>{phi_bb14_19, phi_bb14_23, phi_bb14_24, phi_bb14_25, phi_bb14_26, phi_bb14_27});
  }

  TNode<BoolT> phi_bb25_19;
  TNode<IntPtrT> phi_bb25_23;
  TNode<IntPtrT> phi_bb25_24;
  TNode<BoolT> phi_bb25_25;
  TNode<BoolT> phi_bb25_26;
  TNode<IntPtrT> phi_bb25_27;
  TNode<IntPtrT> tmp71;
  TNode<BoolT> tmp72;
  if (block25.is_used()) {
    ca_.Bind(&block25, &phi_bb25_19, &phi_bb25_23, &phi_bb25_24, &phi_bb25_25, &phi_bb25_26, &phi_bb25_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 213);
    tmp71 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp72 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb25_23}, TNode<IntPtrT>{tmp71});
    ca_.Branch(tmp72, &block29, std::vector<compiler::Node*>{phi_bb25_19, phi_bb25_23, phi_bb25_24, phi_bb25_25, phi_bb25_26, phi_bb25_27}, &block30, std::vector<compiler::Node*>{phi_bb25_19, phi_bb25_23, phi_bb25_24, phi_bb25_25, phi_bb25_26, phi_bb25_27});
  }

  TNode<BoolT> phi_bb29_19;
  TNode<IntPtrT> phi_bb29_23;
  TNode<IntPtrT> phi_bb29_24;
  TNode<BoolT> phi_bb29_25;
  TNode<BoolT> phi_bb29_26;
  TNode<IntPtrT> phi_bb29_27;
  TNode<IntPtrT> tmp73;
  TNode<IntPtrT> tmp74;
  if (block29.is_used()) {
    ca_.Bind(&block29, &phi_bb29_19, &phi_bb29_23, &phi_bb29_24, &phi_bb29_25, &phi_bb29_26, &phi_bb29_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 214);
    tmp73 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp74 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb29_23}, TNode<IntPtrT>{tmp73});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 215);
    ca_.Goto(&block28, phi_bb29_19, tmp74, phi_bb29_24, phi_bb29_25, phi_bb29_26, phi_bb29_27);
  }

  TNode<BoolT> phi_bb30_19;
  TNode<IntPtrT> phi_bb30_23;
  TNode<IntPtrT> phi_bb30_24;
  TNode<BoolT> phi_bb30_25;
  TNode<BoolT> phi_bb30_26;
  TNode<IntPtrT> phi_bb30_27;
  if (block30.is_used()) {
    ca_.Bind(&block30, &phi_bb30_19, &phi_bb30_23, &phi_bb30_24, &phi_bb30_25, &phi_bb30_26, &phi_bb30_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block32, phi_bb30_19, phi_bb30_23, phi_bb30_24, phi_bb30_25, phi_bb30_26, phi_bb30_27);
    } else {
      ca_.Goto(&block33, phi_bb30_19, phi_bb30_23, phi_bb30_24, phi_bb30_25, phi_bb30_26, phi_bb30_27);
    }
  }

  TNode<BoolT> phi_bb32_19;
  TNode<IntPtrT> phi_bb32_23;
  TNode<IntPtrT> phi_bb32_24;
  TNode<BoolT> phi_bb32_25;
  TNode<BoolT> phi_bb32_26;
  TNode<IntPtrT> phi_bb32_27;
  TNode<IntPtrT> tmp75;
  TNode<IntPtrT> tmp76;
  if (block32.is_used()) {
    ca_.Bind(&block32, &phi_bb32_19, &phi_bb32_23, &phi_bb32_24, &phi_bb32_25, &phi_bb32_26, &phi_bb32_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 198);
    tmp75 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp76 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb32_27}, TNode<IntPtrT>{tmp75});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    ca_.Goto(&block34, phi_bb32_19, phi_bb32_23, phi_bb32_24, phi_bb32_25, phi_bb32_26, tmp76);
  }

  TNode<BoolT> phi_bb33_19;
  TNode<IntPtrT> phi_bb33_23;
  TNode<IntPtrT> phi_bb33_24;
  TNode<BoolT> phi_bb33_25;
  TNode<BoolT> phi_bb33_26;
  TNode<IntPtrT> phi_bb33_27;
  if (block33.is_used()) {
    ca_.Bind(&block33, &phi_bb33_19, &phi_bb33_23, &phi_bb33_24, &phi_bb33_25, &phi_bb33_26, &phi_bb33_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Branch(phi_bb33_25, &block35, std::vector<compiler::Node*>{phi_bb33_19, phi_bb33_23, phi_bb33_24, phi_bb33_25, phi_bb33_26, phi_bb33_27}, &block36, std::vector<compiler::Node*>{phi_bb33_19, phi_bb33_23, phi_bb33_24, phi_bb33_25, phi_bb33_26, phi_bb33_27});
  }

  TNode<BoolT> phi_bb35_19;
  TNode<IntPtrT> phi_bb35_23;
  TNode<IntPtrT> phi_bb35_24;
  TNode<BoolT> phi_bb35_25;
  TNode<BoolT> phi_bb35_26;
  TNode<IntPtrT> phi_bb35_27;
  TNode<BoolT> tmp77;
  TNode<BoolT> tmp78;
  if (block35.is_used()) {
    ca_.Bind(&block35, &phi_bb35_19, &phi_bb35_23, &phi_bb35_24, &phi_bb35_25, &phi_bb35_26, &phi_bb35_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 201);
    tmp77 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 202);
    tmp78 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Goto(&block37, phi_bb35_19, phi_bb35_23, phi_bb35_24, tmp77, tmp78, phi_bb35_27);
  }

  TNode<BoolT> phi_bb36_19;
  TNode<IntPtrT> phi_bb36_23;
  TNode<IntPtrT> phi_bb36_24;
  TNode<BoolT> phi_bb36_25;
  TNode<BoolT> phi_bb36_26;
  TNode<IntPtrT> phi_bb36_27;
  TNode<IntPtrT> tmp79;
  TNode<IntPtrT> tmp80;
  TNode<BoolT> tmp81;
  TNode<BoolT> tmp82;
  if (block36.is_used()) {
    ca_.Bind(&block36, &phi_bb36_19, &phi_bb36_23, &phi_bb36_24, &phi_bb36_25, &phi_bb36_26, &phi_bb36_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 204);
    tmp79 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x2ull));
    tmp80 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb36_27}, TNode<IntPtrT>{tmp79});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 205);
    tmp81 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 206);
    tmp82 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Goto(&block37, phi_bb36_19, phi_bb36_23, phi_bb36_24, tmp81, tmp82, tmp80);
  }

  TNode<BoolT> phi_bb37_19;
  TNode<IntPtrT> phi_bb37_23;
  TNode<IntPtrT> phi_bb37_24;
  TNode<BoolT> phi_bb37_25;
  TNode<BoolT> phi_bb37_26;
  TNode<IntPtrT> phi_bb37_27;
  if (block37.is_used()) {
    ca_.Bind(&block37, &phi_bb37_19, &phi_bb37_23, &phi_bb37_24, &phi_bb37_25, &phi_bb37_26, &phi_bb37_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    ca_.Goto(&block34, phi_bb37_19, phi_bb37_23, phi_bb37_24, phi_bb37_25, phi_bb37_26, phi_bb37_27);
  }

  TNode<BoolT> phi_bb34_19;
  TNode<IntPtrT> phi_bb34_23;
  TNode<IntPtrT> phi_bb34_24;
  TNode<BoolT> phi_bb34_25;
  TNode<BoolT> phi_bb34_26;
  TNode<IntPtrT> phi_bb34_27;
  if (block34.is_used()) {
    ca_.Bind(&block34, &phi_bb34_19, &phi_bb34_23, &phi_bb34_24, &phi_bb34_25, &phi_bb34_26, &phi_bb34_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 564);
    ca_.Goto(&block28, phi_bb34_19, phi_bb34_23, phi_bb34_24, phi_bb34_25, phi_bb34_26, phi_bb34_27);
  }

  TNode<BoolT> phi_bb28_19;
  TNode<IntPtrT> phi_bb28_23;
  TNode<IntPtrT> phi_bb28_24;
  TNode<BoolT> phi_bb28_25;
  TNode<BoolT> phi_bb28_26;
  TNode<IntPtrT> phi_bb28_27;
  if (block28.is_used()) {
    ca_.Bind(&block28, &phi_bb28_19, &phi_bb28_23, &phi_bb28_24, &phi_bb28_25, &phi_bb28_26, &phi_bb28_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 563);
    ca_.Goto(&block27, phi_bb28_19, phi_bb28_23, phi_bb28_24, phi_bb28_25, phi_bb28_26, phi_bb28_27);
  }

  TNode<BoolT> phi_bb26_19;
  TNode<IntPtrT> phi_bb26_23;
  TNode<IntPtrT> phi_bb26_24;
  TNode<BoolT> phi_bb26_25;
  TNode<BoolT> phi_bb26_26;
  TNode<IntPtrT> phi_bb26_27;
  TNode<Int32T> tmp83;
  TNode<BoolT> tmp84;
  if (block26.is_used()) {
    ca_.Bind(&block26, &phi_bb26_19, &phi_bb26_23, &phi_bb26_24, &phi_bb26_25, &phi_bb26_26, &phi_bb26_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 565);
    tmp83 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmI64.raw_bit_field());
    tmp84 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp68}, TNode<Int32T>{tmp83});
    ca_.Branch(tmp84, &block38, std::vector<compiler::Node*>{phi_bb26_19, phi_bb26_23, phi_bb26_24, phi_bb26_25, phi_bb26_26, phi_bb26_27}, &block39, std::vector<compiler::Node*>{phi_bb26_19, phi_bb26_23, phi_bb26_24, phi_bb26_25, phi_bb26_26, phi_bb26_27});
  }

  TNode<BoolT> phi_bb38_19;
  TNode<IntPtrT> phi_bb38_23;
  TNode<IntPtrT> phi_bb38_24;
  TNode<BoolT> phi_bb38_25;
  TNode<BoolT> phi_bb38_26;
  TNode<IntPtrT> phi_bb38_27;
  TNode<IntPtrT> tmp85;
  TNode<BoolT> tmp86;
  if (block38.is_used()) {
    ca_.Bind(&block38, &phi_bb38_19, &phi_bb38_23, &phi_bb38_24, &phi_bb38_25, &phi_bb38_26, &phi_bb38_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 213);
    tmp85 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp86 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb38_23}, TNode<IntPtrT>{tmp85});
    ca_.Branch(tmp86, &block42, std::vector<compiler::Node*>{phi_bb38_19, phi_bb38_23, phi_bb38_24, phi_bb38_25, phi_bb38_26, phi_bb38_27}, &block43, std::vector<compiler::Node*>{phi_bb38_19, phi_bb38_23, phi_bb38_24, phi_bb38_25, phi_bb38_26, phi_bb38_27});
  }

  TNode<BoolT> phi_bb42_19;
  TNode<IntPtrT> phi_bb42_23;
  TNode<IntPtrT> phi_bb42_24;
  TNode<BoolT> phi_bb42_25;
  TNode<BoolT> phi_bb42_26;
  TNode<IntPtrT> phi_bb42_27;
  TNode<IntPtrT> tmp87;
  TNode<IntPtrT> tmp88;
  if (block42.is_used()) {
    ca_.Bind(&block42, &phi_bb42_19, &phi_bb42_23, &phi_bb42_24, &phi_bb42_25, &phi_bb42_26, &phi_bb42_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 214);
    tmp87 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp88 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb42_23}, TNode<IntPtrT>{tmp87});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 215);
    ca_.Goto(&block41, phi_bb42_19, tmp88, phi_bb42_24, phi_bb42_25, phi_bb42_26, phi_bb42_27);
  }

  TNode<BoolT> phi_bb43_19;
  TNode<IntPtrT> phi_bb43_23;
  TNode<IntPtrT> phi_bb43_24;
  TNode<BoolT> phi_bb43_25;
  TNode<BoolT> phi_bb43_26;
  TNode<IntPtrT> phi_bb43_27;
  if (block43.is_used()) {
    ca_.Bind(&block43, &phi_bb43_19, &phi_bb43_23, &phi_bb43_24, &phi_bb43_25, &phi_bb43_26, &phi_bb43_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block45, phi_bb43_19, phi_bb43_23, phi_bb43_24, phi_bb43_25, phi_bb43_26, phi_bb43_27);
    } else {
      ca_.Goto(&block46, phi_bb43_19, phi_bb43_23, phi_bb43_24, phi_bb43_25, phi_bb43_26, phi_bb43_27);
    }
  }

  TNode<BoolT> phi_bb45_19;
  TNode<IntPtrT> phi_bb45_23;
  TNode<IntPtrT> phi_bb45_24;
  TNode<BoolT> phi_bb45_25;
  TNode<BoolT> phi_bb45_26;
  TNode<IntPtrT> phi_bb45_27;
  TNode<IntPtrT> tmp89;
  TNode<IntPtrT> tmp90;
  if (block45.is_used()) {
    ca_.Bind(&block45, &phi_bb45_19, &phi_bb45_23, &phi_bb45_24, &phi_bb45_25, &phi_bb45_26, &phi_bb45_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 198);
    tmp89 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp90 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb45_27}, TNode<IntPtrT>{tmp89});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    ca_.Goto(&block47, phi_bb45_19, phi_bb45_23, phi_bb45_24, phi_bb45_25, phi_bb45_26, tmp90);
  }

  TNode<BoolT> phi_bb46_19;
  TNode<IntPtrT> phi_bb46_23;
  TNode<IntPtrT> phi_bb46_24;
  TNode<BoolT> phi_bb46_25;
  TNode<BoolT> phi_bb46_26;
  TNode<IntPtrT> phi_bb46_27;
  if (block46.is_used()) {
    ca_.Bind(&block46, &phi_bb46_19, &phi_bb46_23, &phi_bb46_24, &phi_bb46_25, &phi_bb46_26, &phi_bb46_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Branch(phi_bb46_25, &block48, std::vector<compiler::Node*>{phi_bb46_19, phi_bb46_23, phi_bb46_24, phi_bb46_25, phi_bb46_26, phi_bb46_27}, &block49, std::vector<compiler::Node*>{phi_bb46_19, phi_bb46_23, phi_bb46_24, phi_bb46_25, phi_bb46_26, phi_bb46_27});
  }

  TNode<BoolT> phi_bb48_19;
  TNode<IntPtrT> phi_bb48_23;
  TNode<IntPtrT> phi_bb48_24;
  TNode<BoolT> phi_bb48_25;
  TNode<BoolT> phi_bb48_26;
  TNode<IntPtrT> phi_bb48_27;
  TNode<BoolT> tmp91;
  TNode<BoolT> tmp92;
  if (block48.is_used()) {
    ca_.Bind(&block48, &phi_bb48_19, &phi_bb48_23, &phi_bb48_24, &phi_bb48_25, &phi_bb48_26, &phi_bb48_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 201);
    tmp91 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 202);
    tmp92 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Goto(&block50, phi_bb48_19, phi_bb48_23, phi_bb48_24, tmp91, tmp92, phi_bb48_27);
  }

  TNode<BoolT> phi_bb49_19;
  TNode<IntPtrT> phi_bb49_23;
  TNode<IntPtrT> phi_bb49_24;
  TNode<BoolT> phi_bb49_25;
  TNode<BoolT> phi_bb49_26;
  TNode<IntPtrT> phi_bb49_27;
  TNode<IntPtrT> tmp93;
  TNode<IntPtrT> tmp94;
  TNode<BoolT> tmp95;
  TNode<BoolT> tmp96;
  if (block49.is_used()) {
    ca_.Bind(&block49, &phi_bb49_19, &phi_bb49_23, &phi_bb49_24, &phi_bb49_25, &phi_bb49_26, &phi_bb49_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 204);
    tmp93 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x2ull));
    tmp94 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb49_27}, TNode<IntPtrT>{tmp93});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 205);
    tmp95 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 206);
    tmp96 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Goto(&block50, phi_bb49_19, phi_bb49_23, phi_bb49_24, tmp95, tmp96, tmp94);
  }

  TNode<BoolT> phi_bb50_19;
  TNode<IntPtrT> phi_bb50_23;
  TNode<IntPtrT> phi_bb50_24;
  TNode<BoolT> phi_bb50_25;
  TNode<BoolT> phi_bb50_26;
  TNode<IntPtrT> phi_bb50_27;
  if (block50.is_used()) {
    ca_.Bind(&block50, &phi_bb50_19, &phi_bb50_23, &phi_bb50_24, &phi_bb50_25, &phi_bb50_26, &phi_bb50_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    ca_.Goto(&block47, phi_bb50_19, phi_bb50_23, phi_bb50_24, phi_bb50_25, phi_bb50_26, phi_bb50_27);
  }

  TNode<BoolT> phi_bb47_19;
  TNode<IntPtrT> phi_bb47_23;
  TNode<IntPtrT> phi_bb47_24;
  TNode<BoolT> phi_bb47_25;
  TNode<BoolT> phi_bb47_26;
  TNode<IntPtrT> phi_bb47_27;
  if (block47.is_used()) {
    ca_.Bind(&block47, &phi_bb47_19, &phi_bb47_23, &phi_bb47_24, &phi_bb47_25, &phi_bb47_26, &phi_bb47_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 566);
    ca_.Goto(&block41, phi_bb47_19, phi_bb47_23, phi_bb47_24, phi_bb47_25, phi_bb47_26, phi_bb47_27);
  }

  TNode<BoolT> phi_bb41_19;
  TNode<IntPtrT> phi_bb41_23;
  TNode<IntPtrT> phi_bb41_24;
  TNode<BoolT> phi_bb41_25;
  TNode<BoolT> phi_bb41_26;
  TNode<IntPtrT> phi_bb41_27;
  if (block41.is_used()) {
    ca_.Bind(&block41, &phi_bb41_19, &phi_bb41_23, &phi_bb41_24, &phi_bb41_25, &phi_bb41_26, &phi_bb41_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 567);
    if (((CodeStubAssembler(state_).ConstexprBoolNot((CodeStubAssembler(state_).Is64()))))) {
      ca_.Goto(&block51, phi_bb41_19, phi_bb41_23, phi_bb41_24, phi_bb41_25, phi_bb41_26, phi_bb41_27);
    } else {
      ca_.Goto(&block52, phi_bb41_19, phi_bb41_23, phi_bb41_24, phi_bb41_25, phi_bb41_26, phi_bb41_27);
    }
  }

  TNode<BoolT> phi_bb51_19;
  TNode<IntPtrT> phi_bb51_23;
  TNode<IntPtrT> phi_bb51_24;
  TNode<BoolT> phi_bb51_25;
  TNode<BoolT> phi_bb51_26;
  TNode<IntPtrT> phi_bb51_27;
  TNode<IntPtrT> tmp97;
  TNode<BoolT> tmp98;
  if (block51.is_used()) {
    ca_.Bind(&block51, &phi_bb51_19, &phi_bb51_23, &phi_bb51_24, &phi_bb51_25, &phi_bb51_26, &phi_bb51_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 213);
    tmp97 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp98 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb51_23}, TNode<IntPtrT>{tmp97});
    ca_.Branch(tmp98, &block55, std::vector<compiler::Node*>{phi_bb51_19, phi_bb51_23, phi_bb51_24, phi_bb51_25, phi_bb51_26, phi_bb51_27}, &block56, std::vector<compiler::Node*>{phi_bb51_19, phi_bb51_23, phi_bb51_24, phi_bb51_25, phi_bb51_26, phi_bb51_27});
  }

  TNode<BoolT> phi_bb55_19;
  TNode<IntPtrT> phi_bb55_23;
  TNode<IntPtrT> phi_bb55_24;
  TNode<BoolT> phi_bb55_25;
  TNode<BoolT> phi_bb55_26;
  TNode<IntPtrT> phi_bb55_27;
  TNode<IntPtrT> tmp99;
  TNode<IntPtrT> tmp100;
  if (block55.is_used()) {
    ca_.Bind(&block55, &phi_bb55_19, &phi_bb55_23, &phi_bb55_24, &phi_bb55_25, &phi_bb55_26, &phi_bb55_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 214);
    tmp99 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp100 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb55_23}, TNode<IntPtrT>{tmp99});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 215);
    ca_.Goto(&block54, phi_bb55_19, tmp100, phi_bb55_24, phi_bb55_25, phi_bb55_26, phi_bb55_27);
  }

  TNode<BoolT> phi_bb56_19;
  TNode<IntPtrT> phi_bb56_23;
  TNode<IntPtrT> phi_bb56_24;
  TNode<BoolT> phi_bb56_25;
  TNode<BoolT> phi_bb56_26;
  TNode<IntPtrT> phi_bb56_27;
  if (block56.is_used()) {
    ca_.Bind(&block56, &phi_bb56_19, &phi_bb56_23, &phi_bb56_24, &phi_bb56_25, &phi_bb56_26, &phi_bb56_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block58, phi_bb56_19, phi_bb56_23, phi_bb56_24, phi_bb56_25, phi_bb56_26, phi_bb56_27);
    } else {
      ca_.Goto(&block59, phi_bb56_19, phi_bb56_23, phi_bb56_24, phi_bb56_25, phi_bb56_26, phi_bb56_27);
    }
  }

  TNode<BoolT> phi_bb58_19;
  TNode<IntPtrT> phi_bb58_23;
  TNode<IntPtrT> phi_bb58_24;
  TNode<BoolT> phi_bb58_25;
  TNode<BoolT> phi_bb58_26;
  TNode<IntPtrT> phi_bb58_27;
  TNode<IntPtrT> tmp101;
  TNode<IntPtrT> tmp102;
  if (block58.is_used()) {
    ca_.Bind(&block58, &phi_bb58_19, &phi_bb58_23, &phi_bb58_24, &phi_bb58_25, &phi_bb58_26, &phi_bb58_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 198);
    tmp101 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp102 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb58_27}, TNode<IntPtrT>{tmp101});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    ca_.Goto(&block60, phi_bb58_19, phi_bb58_23, phi_bb58_24, phi_bb58_25, phi_bb58_26, tmp102);
  }

  TNode<BoolT> phi_bb59_19;
  TNode<IntPtrT> phi_bb59_23;
  TNode<IntPtrT> phi_bb59_24;
  TNode<BoolT> phi_bb59_25;
  TNode<BoolT> phi_bb59_26;
  TNode<IntPtrT> phi_bb59_27;
  if (block59.is_used()) {
    ca_.Bind(&block59, &phi_bb59_19, &phi_bb59_23, &phi_bb59_24, &phi_bb59_25, &phi_bb59_26, &phi_bb59_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Branch(phi_bb59_25, &block61, std::vector<compiler::Node*>{phi_bb59_19, phi_bb59_23, phi_bb59_24, phi_bb59_25, phi_bb59_26, phi_bb59_27}, &block62, std::vector<compiler::Node*>{phi_bb59_19, phi_bb59_23, phi_bb59_24, phi_bb59_25, phi_bb59_26, phi_bb59_27});
  }

  TNode<BoolT> phi_bb61_19;
  TNode<IntPtrT> phi_bb61_23;
  TNode<IntPtrT> phi_bb61_24;
  TNode<BoolT> phi_bb61_25;
  TNode<BoolT> phi_bb61_26;
  TNode<IntPtrT> phi_bb61_27;
  TNode<BoolT> tmp103;
  TNode<BoolT> tmp104;
  if (block61.is_used()) {
    ca_.Bind(&block61, &phi_bb61_19, &phi_bb61_23, &phi_bb61_24, &phi_bb61_25, &phi_bb61_26, &phi_bb61_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 201);
    tmp103 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 202);
    tmp104 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Goto(&block63, phi_bb61_19, phi_bb61_23, phi_bb61_24, tmp103, tmp104, phi_bb61_27);
  }

  TNode<BoolT> phi_bb62_19;
  TNode<IntPtrT> phi_bb62_23;
  TNode<IntPtrT> phi_bb62_24;
  TNode<BoolT> phi_bb62_25;
  TNode<BoolT> phi_bb62_26;
  TNode<IntPtrT> phi_bb62_27;
  TNode<IntPtrT> tmp105;
  TNode<IntPtrT> tmp106;
  TNode<BoolT> tmp107;
  TNode<BoolT> tmp108;
  if (block62.is_used()) {
    ca_.Bind(&block62, &phi_bb62_19, &phi_bb62_23, &phi_bb62_24, &phi_bb62_25, &phi_bb62_26, &phi_bb62_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 204);
    tmp105 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x2ull));
    tmp106 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb62_27}, TNode<IntPtrT>{tmp105});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 205);
    tmp107 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 206);
    tmp108 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Goto(&block63, phi_bb62_19, phi_bb62_23, phi_bb62_24, tmp107, tmp108, tmp106);
  }

  TNode<BoolT> phi_bb63_19;
  TNode<IntPtrT> phi_bb63_23;
  TNode<IntPtrT> phi_bb63_24;
  TNode<BoolT> phi_bb63_25;
  TNode<BoolT> phi_bb63_26;
  TNode<IntPtrT> phi_bb63_27;
  if (block63.is_used()) {
    ca_.Bind(&block63, &phi_bb63_19, &phi_bb63_23, &phi_bb63_24, &phi_bb63_25, &phi_bb63_26, &phi_bb63_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    ca_.Goto(&block60, phi_bb63_19, phi_bb63_23, phi_bb63_24, phi_bb63_25, phi_bb63_26, phi_bb63_27);
  }

  TNode<BoolT> phi_bb60_19;
  TNode<IntPtrT> phi_bb60_23;
  TNode<IntPtrT> phi_bb60_24;
  TNode<BoolT> phi_bb60_25;
  TNode<BoolT> phi_bb60_26;
  TNode<IntPtrT> phi_bb60_27;
  if (block60.is_used()) {
    ca_.Bind(&block60, &phi_bb60_19, &phi_bb60_23, &phi_bb60_24, &phi_bb60_25, &phi_bb60_26, &phi_bb60_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 569);
    ca_.Goto(&block54, phi_bb60_19, phi_bb60_23, phi_bb60_24, phi_bb60_25, phi_bb60_26, phi_bb60_27);
  }

  TNode<BoolT> phi_bb54_19;
  TNode<IntPtrT> phi_bb54_23;
  TNode<IntPtrT> phi_bb54_24;
  TNode<BoolT> phi_bb54_25;
  TNode<BoolT> phi_bb54_26;
  TNode<IntPtrT> phi_bb54_27;
  if (block54.is_used()) {
    ca_.Bind(&block54, &phi_bb54_19, &phi_bb54_23, &phi_bb54_24, &phi_bb54_25, &phi_bb54_26, &phi_bb54_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 567);
    ca_.Goto(&block53, phi_bb54_19, phi_bb54_23, phi_bb54_24, phi_bb54_25, phi_bb54_26, phi_bb54_27);
  }

  TNode<BoolT> phi_bb52_19;
  TNode<IntPtrT> phi_bb52_23;
  TNode<IntPtrT> phi_bb52_24;
  TNode<BoolT> phi_bb52_25;
  TNode<BoolT> phi_bb52_26;
  TNode<IntPtrT> phi_bb52_27;
  if (block52.is_used()) {
    ca_.Bind(&block52, &phi_bb52_19, &phi_bb52_23, &phi_bb52_24, &phi_bb52_25, &phi_bb52_26, &phi_bb52_27);
    ca_.Goto(&block53, phi_bb52_19, phi_bb52_23, phi_bb52_24, phi_bb52_25, phi_bb52_26, phi_bb52_27);
  }

  TNode<BoolT> phi_bb53_19;
  TNode<IntPtrT> phi_bb53_23;
  TNode<IntPtrT> phi_bb53_24;
  TNode<BoolT> phi_bb53_25;
  TNode<BoolT> phi_bb53_26;
  TNode<IntPtrT> phi_bb53_27;
  if (block53.is_used()) {
    ca_.Bind(&block53, &phi_bb53_19, &phi_bb53_23, &phi_bb53_24, &phi_bb53_25, &phi_bb53_26, &phi_bb53_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 565);
    ca_.Goto(&block40, phi_bb53_19, phi_bb53_23, phi_bb53_24, phi_bb53_25, phi_bb53_26, phi_bb53_27);
  }

  TNode<BoolT> phi_bb39_19;
  TNode<IntPtrT> phi_bb39_23;
  TNode<IntPtrT> phi_bb39_24;
  TNode<BoolT> phi_bb39_25;
  TNode<BoolT> phi_bb39_26;
  TNode<IntPtrT> phi_bb39_27;
  TNode<Int32T> tmp109;
  TNode<BoolT> tmp110;
  if (block39.is_used()) {
    ca_.Bind(&block39, &phi_bb39_19, &phi_bb39_23, &phi_bb39_24, &phi_bb39_25, &phi_bb39_26, &phi_bb39_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 571);
    tmp109 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmF32.raw_bit_field());
    tmp110 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp68}, TNode<Int32T>{tmp109});
    ca_.Branch(tmp110, &block64, std::vector<compiler::Node*>{phi_bb39_19, phi_bb39_23, phi_bb39_24, phi_bb39_25, phi_bb39_26, phi_bb39_27}, &block65, std::vector<compiler::Node*>{phi_bb39_19, phi_bb39_23, phi_bb39_24, phi_bb39_25, phi_bb39_26, phi_bb39_27});
  }

  TNode<BoolT> phi_bb64_19;
  TNode<IntPtrT> phi_bb64_23;
  TNode<IntPtrT> phi_bb64_24;
  TNode<BoolT> phi_bb64_25;
  TNode<BoolT> phi_bb64_26;
  TNode<IntPtrT> phi_bb64_27;
  TNode<IntPtrT> tmp111;
  TNode<BoolT> tmp112;
  if (block64.is_used()) {
    ca_.Bind(&block64, &phi_bb64_19, &phi_bb64_23, &phi_bb64_24, &phi_bb64_25, &phi_bb64_26, &phi_bb64_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 221);
    tmp111 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp112 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb64_24}, TNode<IntPtrT>{tmp111});
    ca_.Branch(tmp112, &block68, std::vector<compiler::Node*>{phi_bb64_19, phi_bb64_23, phi_bb64_24, phi_bb64_25, phi_bb64_26, phi_bb64_27}, &block69, std::vector<compiler::Node*>{phi_bb64_19, phi_bb64_23, phi_bb64_24, phi_bb64_25, phi_bb64_26, phi_bb64_27});
  }

  TNode<BoolT> phi_bb68_19;
  TNode<IntPtrT> phi_bb68_23;
  TNode<IntPtrT> phi_bb68_24;
  TNode<BoolT> phi_bb68_25;
  TNode<BoolT> phi_bb68_26;
  TNode<IntPtrT> phi_bb68_27;
  TNode<IntPtrT> tmp113;
  TNode<IntPtrT> tmp114;
  if (block68.is_used()) {
    ca_.Bind(&block68, &phi_bb68_19, &phi_bb68_23, &phi_bb68_24, &phi_bb68_25, &phi_bb68_26, &phi_bb68_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 222);
    tmp113 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp114 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb68_24}, TNode<IntPtrT>{tmp113});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 223);
    ca_.Goto(&block67, phi_bb68_19, phi_bb68_23, tmp114, phi_bb68_25, phi_bb68_26, phi_bb68_27);
  }

  TNode<BoolT> phi_bb69_19;
  TNode<IntPtrT> phi_bb69_23;
  TNode<IntPtrT> phi_bb69_24;
  TNode<BoolT> phi_bb69_25;
  TNode<BoolT> phi_bb69_26;
  TNode<IntPtrT> phi_bb69_27;
  if (block69.is_used()) {
    ca_.Bind(&block69, &phi_bb69_19, &phi_bb69_23, &phi_bb69_24, &phi_bb69_25, &phi_bb69_26, &phi_bb69_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block71, phi_bb69_19, phi_bb69_23, phi_bb69_24, phi_bb69_25, phi_bb69_26, phi_bb69_27);
    } else {
      ca_.Goto(&block72, phi_bb69_19, phi_bb69_23, phi_bb69_24, phi_bb69_25, phi_bb69_26, phi_bb69_27);
    }
  }

  TNode<BoolT> phi_bb71_19;
  TNode<IntPtrT> phi_bb71_23;
  TNode<IntPtrT> phi_bb71_24;
  TNode<BoolT> phi_bb71_25;
  TNode<BoolT> phi_bb71_26;
  TNode<IntPtrT> phi_bb71_27;
  TNode<IntPtrT> tmp115;
  TNode<IntPtrT> tmp116;
  if (block71.is_used()) {
    ca_.Bind(&block71, &phi_bb71_19, &phi_bb71_23, &phi_bb71_24, &phi_bb71_25, &phi_bb71_26, &phi_bb71_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 198);
    tmp115 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp116 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb71_27}, TNode<IntPtrT>{tmp115});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    ca_.Goto(&block73, phi_bb71_19, phi_bb71_23, phi_bb71_24, phi_bb71_25, phi_bb71_26, tmp116);
  }

  TNode<BoolT> phi_bb72_19;
  TNode<IntPtrT> phi_bb72_23;
  TNode<IntPtrT> phi_bb72_24;
  TNode<BoolT> phi_bb72_25;
  TNode<BoolT> phi_bb72_26;
  TNode<IntPtrT> phi_bb72_27;
  if (block72.is_used()) {
    ca_.Bind(&block72, &phi_bb72_19, &phi_bb72_23, &phi_bb72_24, &phi_bb72_25, &phi_bb72_26, &phi_bb72_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Branch(phi_bb72_25, &block74, std::vector<compiler::Node*>{phi_bb72_19, phi_bb72_23, phi_bb72_24, phi_bb72_25, phi_bb72_26, phi_bb72_27}, &block75, std::vector<compiler::Node*>{phi_bb72_19, phi_bb72_23, phi_bb72_24, phi_bb72_25, phi_bb72_26, phi_bb72_27});
  }

  TNode<BoolT> phi_bb74_19;
  TNode<IntPtrT> phi_bb74_23;
  TNode<IntPtrT> phi_bb74_24;
  TNode<BoolT> phi_bb74_25;
  TNode<BoolT> phi_bb74_26;
  TNode<IntPtrT> phi_bb74_27;
  TNode<BoolT> tmp117;
  TNode<BoolT> tmp118;
  if (block74.is_used()) {
    ca_.Bind(&block74, &phi_bb74_19, &phi_bb74_23, &phi_bb74_24, &phi_bb74_25, &phi_bb74_26, &phi_bb74_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 201);
    tmp117 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 202);
    tmp118 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Goto(&block76, phi_bb74_19, phi_bb74_23, phi_bb74_24, tmp117, tmp118, phi_bb74_27);
  }

  TNode<BoolT> phi_bb75_19;
  TNode<IntPtrT> phi_bb75_23;
  TNode<IntPtrT> phi_bb75_24;
  TNode<BoolT> phi_bb75_25;
  TNode<BoolT> phi_bb75_26;
  TNode<IntPtrT> phi_bb75_27;
  TNode<IntPtrT> tmp119;
  TNode<IntPtrT> tmp120;
  TNode<BoolT> tmp121;
  TNode<BoolT> tmp122;
  if (block75.is_used()) {
    ca_.Bind(&block75, &phi_bb75_19, &phi_bb75_23, &phi_bb75_24, &phi_bb75_25, &phi_bb75_26, &phi_bb75_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 204);
    tmp119 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x2ull));
    tmp120 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb75_27}, TNode<IntPtrT>{tmp119});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 205);
    tmp121 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 206);
    tmp122 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Goto(&block76, phi_bb75_19, phi_bb75_23, phi_bb75_24, tmp121, tmp122, tmp120);
  }

  TNode<BoolT> phi_bb76_19;
  TNode<IntPtrT> phi_bb76_23;
  TNode<IntPtrT> phi_bb76_24;
  TNode<BoolT> phi_bb76_25;
  TNode<BoolT> phi_bb76_26;
  TNode<IntPtrT> phi_bb76_27;
  if (block76.is_used()) {
    ca_.Bind(&block76, &phi_bb76_19, &phi_bb76_23, &phi_bb76_24, &phi_bb76_25, &phi_bb76_26, &phi_bb76_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    ca_.Goto(&block73, phi_bb76_19, phi_bb76_23, phi_bb76_24, phi_bb76_25, phi_bb76_26, phi_bb76_27);
  }

  TNode<BoolT> phi_bb73_19;
  TNode<IntPtrT> phi_bb73_23;
  TNode<IntPtrT> phi_bb73_24;
  TNode<BoolT> phi_bb73_25;
  TNode<BoolT> phi_bb73_26;
  TNode<IntPtrT> phi_bb73_27;
  if (block73.is_used()) {
    ca_.Bind(&block73, &phi_bb73_19, &phi_bb73_23, &phi_bb73_24, &phi_bb73_25, &phi_bb73_26, &phi_bb73_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 572);
    ca_.Goto(&block67, phi_bb73_19, phi_bb73_23, phi_bb73_24, phi_bb73_25, phi_bb73_26, phi_bb73_27);
  }

  TNode<BoolT> phi_bb67_19;
  TNode<IntPtrT> phi_bb67_23;
  TNode<IntPtrT> phi_bb67_24;
  TNode<BoolT> phi_bb67_25;
  TNode<BoolT> phi_bb67_26;
  TNode<IntPtrT> phi_bb67_27;
  if (block67.is_used()) {
    ca_.Bind(&block67, &phi_bb67_19, &phi_bb67_23, &phi_bb67_24, &phi_bb67_25, &phi_bb67_26, &phi_bb67_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 571);
    ca_.Goto(&block66, phi_bb67_19, phi_bb67_23, phi_bb67_24, phi_bb67_25, phi_bb67_26, phi_bb67_27);
  }

  TNode<BoolT> phi_bb65_19;
  TNode<IntPtrT> phi_bb65_23;
  TNode<IntPtrT> phi_bb65_24;
  TNode<BoolT> phi_bb65_25;
  TNode<BoolT> phi_bb65_26;
  TNode<IntPtrT> phi_bb65_27;
  TNode<Int32T> tmp123;
  TNode<BoolT> tmp124;
  if (block65.is_used()) {
    ca_.Bind(&block65, &phi_bb65_19, &phi_bb65_23, &phi_bb65_24, &phi_bb65_25, &phi_bb65_26, &phi_bb65_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 573);
    tmp123 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmF64.raw_bit_field());
    tmp124 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp68}, TNode<Int32T>{tmp123});
    ca_.Branch(tmp124, &block77, std::vector<compiler::Node*>{phi_bb65_19, phi_bb65_23, phi_bb65_24, phi_bb65_25, phi_bb65_26, phi_bb65_27}, &block78, std::vector<compiler::Node*>{phi_bb65_19, phi_bb65_23, phi_bb65_24, phi_bb65_25, phi_bb65_26, phi_bb65_27});
  }

  TNode<BoolT> phi_bb77_19;
  TNode<IntPtrT> phi_bb77_23;
  TNode<IntPtrT> phi_bb77_24;
  TNode<BoolT> phi_bb77_25;
  TNode<BoolT> phi_bb77_26;
  TNode<IntPtrT> phi_bb77_27;
  TNode<IntPtrT> tmp125;
  TNode<BoolT> tmp126;
  if (block77.is_used()) {
    ca_.Bind(&block77, &phi_bb77_19, &phi_bb77_23, &phi_bb77_24, &phi_bb77_25, &phi_bb77_26, &phi_bb77_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 229);
    tmp125 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp126 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb77_24}, TNode<IntPtrT>{tmp125});
    ca_.Branch(tmp126, &block81, std::vector<compiler::Node*>{phi_bb77_19, phi_bb77_23, phi_bb77_24, phi_bb77_25, phi_bb77_26, phi_bb77_27}, &block82, std::vector<compiler::Node*>{phi_bb77_19, phi_bb77_23, phi_bb77_24, phi_bb77_25, phi_bb77_26, phi_bb77_27});
  }

  TNode<BoolT> phi_bb81_19;
  TNode<IntPtrT> phi_bb81_23;
  TNode<IntPtrT> phi_bb81_24;
  TNode<BoolT> phi_bb81_25;
  TNode<BoolT> phi_bb81_26;
  TNode<IntPtrT> phi_bb81_27;
  TNode<IntPtrT> tmp127;
  TNode<IntPtrT> tmp128;
  if (block81.is_used()) {
    ca_.Bind(&block81, &phi_bb81_19, &phi_bb81_23, &phi_bb81_24, &phi_bb81_25, &phi_bb81_26, &phi_bb81_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 230);
    tmp127 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp128 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb81_24}, TNode<IntPtrT>{tmp127});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 231);
    ca_.Goto(&block80, phi_bb81_19, phi_bb81_23, tmp128, phi_bb81_25, phi_bb81_26, phi_bb81_27);
  }

  TNode<BoolT> phi_bb82_19;
  TNode<IntPtrT> phi_bb82_23;
  TNode<IntPtrT> phi_bb82_24;
  TNode<BoolT> phi_bb82_25;
  TNode<BoolT> phi_bb82_26;
  TNode<IntPtrT> phi_bb82_27;
  if (block82.is_used()) {
    ca_.Bind(&block82, &phi_bb82_19, &phi_bb82_23, &phi_bb82_24, &phi_bb82_25, &phi_bb82_26, &phi_bb82_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 233);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block83, phi_bb82_19, phi_bb82_23, phi_bb82_24, phi_bb82_25, phi_bb82_26, phi_bb82_27);
    } else {
      ca_.Goto(&block84, phi_bb82_19, phi_bb82_23, phi_bb82_24, phi_bb82_25, phi_bb82_26, phi_bb82_27);
    }
  }

  TNode<BoolT> phi_bb83_19;
  TNode<IntPtrT> phi_bb83_23;
  TNode<IntPtrT> phi_bb83_24;
  TNode<BoolT> phi_bb83_25;
  TNode<BoolT> phi_bb83_26;
  TNode<IntPtrT> phi_bb83_27;
  TNode<IntPtrT> tmp129;
  TNode<IntPtrT> tmp130;
  if (block83.is_used()) {
    ca_.Bind(&block83, &phi_bb83_19, &phi_bb83_23, &phi_bb83_24, &phi_bb83_25, &phi_bb83_26, &phi_bb83_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 234);
    tmp129 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp130 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb83_27}, TNode<IntPtrT>{tmp129});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 233);
    ca_.Goto(&block85, phi_bb83_19, phi_bb83_23, phi_bb83_24, phi_bb83_25, phi_bb83_26, tmp130);
  }

  TNode<BoolT> phi_bb84_19;
  TNode<IntPtrT> phi_bb84_23;
  TNode<IntPtrT> phi_bb84_24;
  TNode<BoolT> phi_bb84_25;
  TNode<BoolT> phi_bb84_26;
  TNode<IntPtrT> phi_bb84_27;
  TNode<IntPtrT> tmp131;
  TNode<IntPtrT> tmp132;
  TNode<BoolT> tmp133;
  if (block84.is_used()) {
    ca_.Bind(&block84, &phi_bb84_19, &phi_bb84_23, &phi_bb84_24, &phi_bb84_25, &phi_bb84_26, &phi_bb84_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 236);
    tmp131 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x2ull));
    tmp132 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb84_27}, TNode<IntPtrT>{tmp131});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 237);
    tmp133 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 233);
    ca_.Goto(&block85, phi_bb84_19, phi_bb84_23, phi_bb84_24, phi_bb84_25, tmp133, tmp132);
  }

  TNode<BoolT> phi_bb85_19;
  TNode<IntPtrT> phi_bb85_23;
  TNode<IntPtrT> phi_bb85_24;
  TNode<BoolT> phi_bb85_25;
  TNode<BoolT> phi_bb85_26;
  TNode<IntPtrT> phi_bb85_27;
  if (block85.is_used()) {
    ca_.Bind(&block85, &phi_bb85_19, &phi_bb85_23, &phi_bb85_24, &phi_bb85_25, &phi_bb85_26, &phi_bb85_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 574);
    ca_.Goto(&block80, phi_bb85_19, phi_bb85_23, phi_bb85_24, phi_bb85_25, phi_bb85_26, phi_bb85_27);
  }

  TNode<BoolT> phi_bb80_19;
  TNode<IntPtrT> phi_bb80_23;
  TNode<IntPtrT> phi_bb80_24;
  TNode<BoolT> phi_bb80_25;
  TNode<BoolT> phi_bb80_26;
  TNode<IntPtrT> phi_bb80_27;
  if (block80.is_used()) {
    ca_.Bind(&block80, &phi_bb80_19, &phi_bb80_23, &phi_bb80_24, &phi_bb80_25, &phi_bb80_26, &phi_bb80_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 573);
    ca_.Goto(&block79, phi_bb80_19, phi_bb80_23, phi_bb80_24, phi_bb80_25, phi_bb80_26, phi_bb80_27);
  }

  TNode<BoolT> phi_bb78_19;
  TNode<IntPtrT> phi_bb78_23;
  TNode<IntPtrT> phi_bb78_24;
  TNode<BoolT> phi_bb78_25;
  TNode<BoolT> phi_bb78_26;
  TNode<IntPtrT> phi_bb78_27;
  TNode<Int32T> tmp134;
  TNode<Int32T> tmp135;
  TNode<Int32T> tmp136;
  TNode<BoolT> tmp137;
  if (block78.is_used()) {
    ca_.Bind(&block78, &phi_bb78_19, &phi_bb78_23, &phi_bb78_24, &phi_bb78_25, &phi_bb78_26, &phi_bb78_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 576);
    tmp134 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmValueKindBitsMask);
    tmp135 = CodeStubAssembler(state_).Word32And(TNode<Int32T>{tmp68}, TNode<Int32T>{tmp134});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 577);
    tmp136 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRef);
    tmp137 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp135}, TNode<Int32T>{tmp136});
    ca_.Branch(tmp137, &block88, std::vector<compiler::Node*>{phi_bb78_19, phi_bb78_23, phi_bb78_24, phi_bb78_25, phi_bb78_26, phi_bb78_27}, &block89, std::vector<compiler::Node*>{phi_bb78_19, phi_bb78_23, phi_bb78_24, phi_bb78_25, phi_bb78_26, phi_bb78_27});
  }

  TNode<BoolT> phi_bb88_19;
  TNode<IntPtrT> phi_bb88_23;
  TNode<IntPtrT> phi_bb88_24;
  TNode<BoolT> phi_bb88_25;
  TNode<BoolT> phi_bb88_26;
  TNode<IntPtrT> phi_bb88_27;
  TNode<BoolT> tmp138;
  if (block88.is_used()) {
    ca_.Bind(&block88, &phi_bb88_19, &phi_bb88_23, &phi_bb88_24, &phi_bb88_25, &phi_bb88_26, &phi_bb88_27);
    tmp138 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block90, phi_bb88_19, phi_bb88_23, phi_bb88_24, phi_bb88_25, phi_bb88_26, phi_bb88_27, tmp138);
  }

  TNode<BoolT> phi_bb89_19;
  TNode<IntPtrT> phi_bb89_23;
  TNode<IntPtrT> phi_bb89_24;
  TNode<BoolT> phi_bb89_25;
  TNode<BoolT> phi_bb89_26;
  TNode<IntPtrT> phi_bb89_27;
  TNode<Int32T> tmp139;
  TNode<BoolT> tmp140;
  if (block89.is_used()) {
    ca_.Bind(&block89, &phi_bb89_19, &phi_bb89_23, &phi_bb89_24, &phi_bb89_25, &phi_bb89_26, &phi_bb89_27);
    tmp139 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRefNull);
    tmp140 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp135}, TNode<Int32T>{tmp139});
    ca_.Goto(&block90, phi_bb89_19, phi_bb89_23, phi_bb89_24, phi_bb89_25, phi_bb89_26, phi_bb89_27, tmp140);
  }

  TNode<BoolT> phi_bb90_19;
  TNode<IntPtrT> phi_bb90_23;
  TNode<IntPtrT> phi_bb90_24;
  TNode<BoolT> phi_bb90_25;
  TNode<BoolT> phi_bb90_26;
  TNode<IntPtrT> phi_bb90_27;
  TNode<BoolT> phi_bb90_34;
  if (block90.is_used()) {
    ca_.Bind(&block90, &phi_bb90_19, &phi_bb90_23, &phi_bb90_24, &phi_bb90_25, &phi_bb90_26, &phi_bb90_27, &phi_bb90_34);
    ca_.Branch(phi_bb90_34, &block86, std::vector<compiler::Node*>{phi_bb90_19, phi_bb90_23, phi_bb90_24, phi_bb90_25, phi_bb90_26, phi_bb90_27}, &block87, std::vector<compiler::Node*>{phi_bb90_19, phi_bb90_23, phi_bb90_24, phi_bb90_25, phi_bb90_26, phi_bb90_27});
  }

  TNode<BoolT> phi_bb87_19;
  TNode<IntPtrT> phi_bb87_23;
  TNode<IntPtrT> phi_bb87_24;
  TNode<BoolT> phi_bb87_25;
  TNode<BoolT> phi_bb87_26;
  TNode<IntPtrT> phi_bb87_27;
  if (block87.is_used()) {
    ca_.Bind(&block87, &phi_bb87_19, &phi_bb87_23, &phi_bb87_24, &phi_bb87_25, &phi_bb87_26, &phi_bb87_27);
    {
      auto pos_stack = ca_.GetMacroSourcePositionStack();
      pos_stack.push_back({"src/builtins/js-to-wasm.tq", 577});
      CodeStubAssembler(state_).FailAssert("Torque assert 'retKind == ValueKind::kRef || retKind == ValueKind::kRefNull' failed", pos_stack);
    }
  }

  TNode<BoolT> phi_bb86_19;
  TNode<IntPtrT> phi_bb86_23;
  TNode<IntPtrT> phi_bb86_24;
  TNode<BoolT> phi_bb86_25;
  TNode<BoolT> phi_bb86_26;
  TNode<IntPtrT> phi_bb86_27;
  TNode<BoolT> tmp141;
  if (block86.is_used()) {
    ca_.Bind(&block86, &phi_bb86_19, &phi_bb86_23, &phi_bb86_24, &phi_bb86_25, &phi_bb86_26, &phi_bb86_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 579);
    tmp141 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 573);
    ca_.Goto(&block79, tmp141, phi_bb86_23, phi_bb86_24, phi_bb86_25, phi_bb86_26, phi_bb86_27);
  }

  TNode<BoolT> phi_bb79_19;
  TNode<IntPtrT> phi_bb79_23;
  TNode<IntPtrT> phi_bb79_24;
  TNode<BoolT> phi_bb79_25;
  TNode<BoolT> phi_bb79_26;
  TNode<IntPtrT> phi_bb79_27;
  if (block79.is_used()) {
    ca_.Bind(&block79, &phi_bb79_19, &phi_bb79_23, &phi_bb79_24, &phi_bb79_25, &phi_bb79_26, &phi_bb79_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 571);
    ca_.Goto(&block66, phi_bb79_19, phi_bb79_23, phi_bb79_24, phi_bb79_25, phi_bb79_26, phi_bb79_27);
  }

  TNode<BoolT> phi_bb66_19;
  TNode<IntPtrT> phi_bb66_23;
  TNode<IntPtrT> phi_bb66_24;
  TNode<BoolT> phi_bb66_25;
  TNode<BoolT> phi_bb66_26;
  TNode<IntPtrT> phi_bb66_27;
  if (block66.is_used()) {
    ca_.Bind(&block66, &phi_bb66_19, &phi_bb66_23, &phi_bb66_24, &phi_bb66_25, &phi_bb66_26, &phi_bb66_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 565);
    ca_.Goto(&block40, phi_bb66_19, phi_bb66_23, phi_bb66_24, phi_bb66_25, phi_bb66_26, phi_bb66_27);
  }

  TNode<BoolT> phi_bb40_19;
  TNode<IntPtrT> phi_bb40_23;
  TNode<IntPtrT> phi_bb40_24;
  TNode<BoolT> phi_bb40_25;
  TNode<BoolT> phi_bb40_26;
  TNode<IntPtrT> phi_bb40_27;
  if (block40.is_used()) {
    ca_.Bind(&block40, &phi_bb40_19, &phi_bb40_23, &phi_bb40_24, &phi_bb40_25, &phi_bb40_26, &phi_bb40_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 563);
    ca_.Goto(&block27, phi_bb40_19, phi_bb40_23, phi_bb40_24, phi_bb40_25, phi_bb40_26, phi_bb40_27);
  }

  TNode<BoolT> phi_bb27_19;
  TNode<IntPtrT> phi_bb27_23;
  TNode<IntPtrT> phi_bb27_24;
  TNode<BoolT> phi_bb27_25;
  TNode<BoolT> phi_bb27_26;
  TNode<IntPtrT> phi_bb27_27;
  if (block27.is_used()) {
    ca_.Bind(&block27, &phi_bb27_19, &phi_bb27_23, &phi_bb27_24, &phi_bb27_25, &phi_bb27_26, &phi_bb27_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 561);
    ca_.Goto(&block16, phi_bb27_19, phi_bb27_23, phi_bb27_24, phi_bb27_25, phi_bb27_26, phi_bb27_27, tmp67);
  }

  TNode<BoolT> phi_bb15_19;
  TNode<IntPtrT> phi_bb15_23;
  TNode<IntPtrT> phi_bb15_24;
  TNode<BoolT> phi_bb15_25;
  TNode<BoolT> phi_bb15_26;
  TNode<IntPtrT> phi_bb15_27;
  TNode<IntPtrT> phi_bb15_29;
  if (block15.is_used()) {
    ca_.Bind(&block15, &phi_bb15_19, &phi_bb15_23, &phi_bb15_24, &phi_bb15_25, &phi_bb15_26, &phi_bb15_27, &phi_bb15_29);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 583);
    ca_.Branch(phi_bb15_19, &block91, std::vector<compiler::Node*>{phi_bb15_19, phi_bb15_23, phi_bb15_24, phi_bb15_25, phi_bb15_26, phi_bb15_27, phi_bb15_29}, &block92, std::vector<compiler::Node*>{phi_bb15_19, phi_bb15_23, phi_bb15_24, phi_bb15_25, phi_bb15_26, phi_bb15_27, phi_bb15_29, tmp61});
  }

  TNode<BoolT> phi_bb91_19;
  TNode<IntPtrT> phi_bb91_23;
  TNode<IntPtrT> phi_bb91_24;
  TNode<BoolT> phi_bb91_25;
  TNode<BoolT> phi_bb91_26;
  TNode<IntPtrT> phi_bb91_27;
  TNode<IntPtrT> phi_bb91_29;
  TNode<BoolT> tmp142;
  if (block91.is_used()) {
    ca_.Bind(&block91, &phi_bb91_19, &phi_bb91_23, &phi_bb91_24, &phi_bb91_25, &phi_bb91_26, &phi_bb91_27, &phi_bb91_29);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 252);
    tmp142 = CodeStubAssembler(state_).Word32BinaryNot(TNode<BoolT>{phi_bb91_26});
    ca_.Branch(tmp142, &block94, std::vector<compiler::Node*>{phi_bb91_19, phi_bb91_23, phi_bb91_24, phi_bb91_25, phi_bb91_26, phi_bb91_27, phi_bb91_29}, &block95, std::vector<compiler::Node*>{phi_bb91_19, phi_bb91_23, phi_bb91_24, phi_bb91_25, phi_bb91_26, phi_bb91_27, phi_bb91_29});
  }

  TNode<BoolT> phi_bb94_19;
  TNode<IntPtrT> phi_bb94_23;
  TNode<IntPtrT> phi_bb94_24;
  TNode<BoolT> phi_bb94_25;
  TNode<BoolT> phi_bb94_26;
  TNode<IntPtrT> phi_bb94_27;
  TNode<IntPtrT> phi_bb94_29;
  TNode<BoolT> tmp143;
  if (block94.is_used()) {
    ca_.Bind(&block94, &phi_bb94_19, &phi_bb94_23, &phi_bb94_24, &phi_bb94_25, &phi_bb94_26, &phi_bb94_27, &phi_bb94_29);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 253);
    tmp143 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 252);
    ca_.Goto(&block95, phi_bb94_19, phi_bb94_23, phi_bb94_24, tmp143, phi_bb94_26, phi_bb94_27, phi_bb94_29);
  }

  TNode<BoolT> phi_bb95_19;
  TNode<IntPtrT> phi_bb95_23;
  TNode<IntPtrT> phi_bb95_24;
  TNode<BoolT> phi_bb95_25;
  TNode<BoolT> phi_bb95_26;
  TNode<IntPtrT> phi_bb95_27;
  TNode<IntPtrT> phi_bb95_29;
  TNode<IntPtrT> tmp144;
  TNode<IntPtrT> tmp145;
  if (block95.is_used()) {
    ca_.Bind(&block95, &phi_bb95_19, &phi_bb95_23, &phi_bb95_24, &phi_bb95_25, &phi_bb95_26, &phi_bb95_27, &phi_bb95_29);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 140);
    tmp144 = TimesSizeOf_int32_0(state_, TNode<IntPtrT>{tmp53});
    tmp145 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp52}, TNode<IntPtrT>{tmp144});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 586);
    ca_.Goto(&block99, phi_bb95_19, phi_bb95_23, phi_bb95_24, phi_bb95_25, phi_bb95_26, phi_bb95_27, tmp52);
  }

  TNode<BoolT> phi_bb99_19;
  TNode<IntPtrT> phi_bb99_23;
  TNode<IntPtrT> phi_bb99_24;
  TNode<BoolT> phi_bb99_25;
  TNode<BoolT> phi_bb99_26;
  TNode<IntPtrT> phi_bb99_27;
  TNode<IntPtrT> phi_bb99_29;
  TNode<BoolT> tmp146;
  TNode<BoolT> tmp147;
  if (block99.is_used()) {
    ca_.Bind(&block99, &phi_bb99_19, &phi_bb99_23, &phi_bb99_24, &phi_bb99_25, &phi_bb99_26, &phi_bb99_27, &phi_bb99_29);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 223);
    tmp146 = CodeStubAssembler(state_).WordEqual(TNode<IntPtrT>{phi_bb99_29}, TNode<IntPtrT>{tmp145});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 586);
    tmp147 = CodeStubAssembler(state_).Word32BinaryNot(TNode<BoolT>{tmp146});
    ca_.Branch(tmp147, &block97, std::vector<compiler::Node*>{phi_bb99_19, phi_bb99_23, phi_bb99_24, phi_bb99_25, phi_bb99_26, phi_bb99_27, phi_bb99_29}, &block98, std::vector<compiler::Node*>{phi_bb99_19, phi_bb99_23, phi_bb99_24, phi_bb99_25, phi_bb99_26, phi_bb99_27, phi_bb99_29});
  }

  TNode<BoolT> phi_bb97_19;
  TNode<IntPtrT> phi_bb97_23;
  TNode<IntPtrT> phi_bb97_24;
  TNode<BoolT> phi_bb97_25;
  TNode<BoolT> phi_bb97_26;
  TNode<IntPtrT> phi_bb97_27;
  TNode<IntPtrT> phi_bb97_29;
  TNode<Object> tmp148;
  TNode<IntPtrT> tmp149;
  TNode<IntPtrT> tmp150;
  TNode<IntPtrT> tmp151;
  TNode<Int32T> tmp152;
  TNode<Int32T> tmp153;
  TNode<Int32T> tmp154;
  TNode<Int32T> tmp155;
  TNode<BoolT> tmp156;
  if (block97.is_used()) {
    ca_.Bind(&block97, &phi_bb97_19, &phi_bb97_23, &phi_bb97_24, &phi_bb97_25, &phi_bb97_26, &phi_bb97_27, &phi_bb97_29);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 246);
    std::tie(tmp148, tmp149) = NewReference_int32_0(state_, TNode<Object>{tmp51}, TNode<IntPtrT>{phi_bb97_29}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 247);
    tmp150 = FromConstexpr_intptr_constexpr_int31_0(state_, kInt32Size);
    tmp151 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb97_29}, TNode<IntPtrT>{tmp150});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 231);
    tmp152 = CodeStubAssembler(state_).LoadReference<Int32T>(CodeStubAssembler::Reference{tmp148, tmp149});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 588);
    tmp153 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmValueKindBitsMask);
    tmp154 = CodeStubAssembler(state_).Word32And(TNode<Int32T>{tmp152}, TNode<Int32T>{tmp153});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 589);
    tmp155 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRef);
    tmp156 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp154}, TNode<Int32T>{tmp155});
    ca_.Branch(tmp156, &block110, std::vector<compiler::Node*>{phi_bb97_19, phi_bb97_23, phi_bb97_24, phi_bb97_25, phi_bb97_26, phi_bb97_27}, &block111, std::vector<compiler::Node*>{phi_bb97_19, phi_bb97_23, phi_bb97_24, phi_bb97_25, phi_bb97_26, phi_bb97_27});
  }

  TNode<BoolT> phi_bb110_19;
  TNode<IntPtrT> phi_bb110_23;
  TNode<IntPtrT> phi_bb110_24;
  TNode<BoolT> phi_bb110_25;
  TNode<BoolT> phi_bb110_26;
  TNode<IntPtrT> phi_bb110_27;
  TNode<BoolT> tmp157;
  if (block110.is_used()) {
    ca_.Bind(&block110, &phi_bb110_19, &phi_bb110_23, &phi_bb110_24, &phi_bb110_25, &phi_bb110_26, &phi_bb110_27);
    tmp157 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block112, phi_bb110_19, phi_bb110_23, phi_bb110_24, phi_bb110_25, phi_bb110_26, phi_bb110_27, tmp157);
  }

  TNode<BoolT> phi_bb111_19;
  TNode<IntPtrT> phi_bb111_23;
  TNode<IntPtrT> phi_bb111_24;
  TNode<BoolT> phi_bb111_25;
  TNode<BoolT> phi_bb111_26;
  TNode<IntPtrT> phi_bb111_27;
  TNode<Int32T> tmp158;
  TNode<BoolT> tmp159;
  if (block111.is_used()) {
    ca_.Bind(&block111, &phi_bb111_19, &phi_bb111_23, &phi_bb111_24, &phi_bb111_25, &phi_bb111_26, &phi_bb111_27);
    tmp158 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRefNull);
    tmp159 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp154}, TNode<Int32T>{tmp158});
    ca_.Goto(&block112, phi_bb111_19, phi_bb111_23, phi_bb111_24, phi_bb111_25, phi_bb111_26, phi_bb111_27, tmp159);
  }

  TNode<BoolT> phi_bb112_19;
  TNode<IntPtrT> phi_bb112_23;
  TNode<IntPtrT> phi_bb112_24;
  TNode<BoolT> phi_bb112_25;
  TNode<BoolT> phi_bb112_26;
  TNode<IntPtrT> phi_bb112_27;
  TNode<BoolT> phi_bb112_34;
  if (block112.is_used()) {
    ca_.Bind(&block112, &phi_bb112_19, &phi_bb112_23, &phi_bb112_24, &phi_bb112_25, &phi_bb112_26, &phi_bb112_27, &phi_bb112_34);
    ca_.Branch(phi_bb112_34, &block108, std::vector<compiler::Node*>{phi_bb112_19, phi_bb112_23, phi_bb112_24, phi_bb112_25, phi_bb112_26, phi_bb112_27}, &block109, std::vector<compiler::Node*>{phi_bb112_19, phi_bb112_23, phi_bb112_24, phi_bb112_25, phi_bb112_26, phi_bb112_27});
  }

  TNode<BoolT> phi_bb108_19;
  TNode<IntPtrT> phi_bb108_23;
  TNode<IntPtrT> phi_bb108_24;
  TNode<BoolT> phi_bb108_25;
  TNode<BoolT> phi_bb108_26;
  TNode<IntPtrT> phi_bb108_27;
  TNode<IntPtrT> tmp160;
  TNode<BoolT> tmp161;
  if (block108.is_used()) {
    ca_.Bind(&block108, &phi_bb108_19, &phi_bb108_23, &phi_bb108_24, &phi_bb108_25, &phi_bb108_26, &phi_bb108_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 213);
    tmp160 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp161 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb108_23}, TNode<IntPtrT>{tmp160});
    ca_.Branch(tmp161, &block114, std::vector<compiler::Node*>{phi_bb108_19, phi_bb108_23, phi_bb108_24, phi_bb108_25, phi_bb108_26, phi_bb108_27}, &block115, std::vector<compiler::Node*>{phi_bb108_19, phi_bb108_23, phi_bb108_24, phi_bb108_25, phi_bb108_26, phi_bb108_27});
  }

  TNode<BoolT> phi_bb114_19;
  TNode<IntPtrT> phi_bb114_23;
  TNode<IntPtrT> phi_bb114_24;
  TNode<BoolT> phi_bb114_25;
  TNode<BoolT> phi_bb114_26;
  TNode<IntPtrT> phi_bb114_27;
  TNode<IntPtrT> tmp162;
  TNode<IntPtrT> tmp163;
  if (block114.is_used()) {
    ca_.Bind(&block114, &phi_bb114_19, &phi_bb114_23, &phi_bb114_24, &phi_bb114_25, &phi_bb114_26, &phi_bb114_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 214);
    tmp162 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp163 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb114_23}, TNode<IntPtrT>{tmp162});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 215);
    ca_.Goto(&block113, phi_bb114_19, tmp163, phi_bb114_24, phi_bb114_25, phi_bb114_26, phi_bb114_27);
  }

  TNode<BoolT> phi_bb115_19;
  TNode<IntPtrT> phi_bb115_23;
  TNode<IntPtrT> phi_bb115_24;
  TNode<BoolT> phi_bb115_25;
  TNode<BoolT> phi_bb115_26;
  TNode<IntPtrT> phi_bb115_27;
  if (block115.is_used()) {
    ca_.Bind(&block115, &phi_bb115_19, &phi_bb115_23, &phi_bb115_24, &phi_bb115_25, &phi_bb115_26, &phi_bb115_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block117, phi_bb115_19, phi_bb115_23, phi_bb115_24, phi_bb115_25, phi_bb115_26, phi_bb115_27);
    } else {
      ca_.Goto(&block118, phi_bb115_19, phi_bb115_23, phi_bb115_24, phi_bb115_25, phi_bb115_26, phi_bb115_27);
    }
  }

  TNode<BoolT> phi_bb117_19;
  TNode<IntPtrT> phi_bb117_23;
  TNode<IntPtrT> phi_bb117_24;
  TNode<BoolT> phi_bb117_25;
  TNode<BoolT> phi_bb117_26;
  TNode<IntPtrT> phi_bb117_27;
  TNode<IntPtrT> tmp164;
  TNode<IntPtrT> tmp165;
  if (block117.is_used()) {
    ca_.Bind(&block117, &phi_bb117_19, &phi_bb117_23, &phi_bb117_24, &phi_bb117_25, &phi_bb117_26, &phi_bb117_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 198);
    tmp164 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp165 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb117_27}, TNode<IntPtrT>{tmp164});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    ca_.Goto(&block119, phi_bb117_19, phi_bb117_23, phi_bb117_24, phi_bb117_25, phi_bb117_26, tmp165);
  }

  TNode<BoolT> phi_bb118_19;
  TNode<IntPtrT> phi_bb118_23;
  TNode<IntPtrT> phi_bb118_24;
  TNode<BoolT> phi_bb118_25;
  TNode<BoolT> phi_bb118_26;
  TNode<IntPtrT> phi_bb118_27;
  if (block118.is_used()) {
    ca_.Bind(&block118, &phi_bb118_19, &phi_bb118_23, &phi_bb118_24, &phi_bb118_25, &phi_bb118_26, &phi_bb118_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Branch(phi_bb118_25, &block120, std::vector<compiler::Node*>{phi_bb118_19, phi_bb118_23, phi_bb118_24, phi_bb118_25, phi_bb118_26, phi_bb118_27}, &block121, std::vector<compiler::Node*>{phi_bb118_19, phi_bb118_23, phi_bb118_24, phi_bb118_25, phi_bb118_26, phi_bb118_27});
  }

  TNode<BoolT> phi_bb120_19;
  TNode<IntPtrT> phi_bb120_23;
  TNode<IntPtrT> phi_bb120_24;
  TNode<BoolT> phi_bb120_25;
  TNode<BoolT> phi_bb120_26;
  TNode<IntPtrT> phi_bb120_27;
  TNode<BoolT> tmp166;
  TNode<BoolT> tmp167;
  if (block120.is_used()) {
    ca_.Bind(&block120, &phi_bb120_19, &phi_bb120_23, &phi_bb120_24, &phi_bb120_25, &phi_bb120_26, &phi_bb120_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 201);
    tmp166 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 202);
    tmp167 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Goto(&block122, phi_bb120_19, phi_bb120_23, phi_bb120_24, tmp166, tmp167, phi_bb120_27);
  }

  TNode<BoolT> phi_bb121_19;
  TNode<IntPtrT> phi_bb121_23;
  TNode<IntPtrT> phi_bb121_24;
  TNode<BoolT> phi_bb121_25;
  TNode<BoolT> phi_bb121_26;
  TNode<IntPtrT> phi_bb121_27;
  TNode<IntPtrT> tmp168;
  TNode<IntPtrT> tmp169;
  TNode<BoolT> tmp170;
  TNode<BoolT> tmp171;
  if (block121.is_used()) {
    ca_.Bind(&block121, &phi_bb121_19, &phi_bb121_23, &phi_bb121_24, &phi_bb121_25, &phi_bb121_26, &phi_bb121_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 204);
    tmp168 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x2ull));
    tmp169 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb121_27}, TNode<IntPtrT>{tmp168});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 205);
    tmp170 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 206);
    tmp171 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 200);
    ca_.Goto(&block122, phi_bb121_19, phi_bb121_23, phi_bb121_24, tmp170, tmp171, tmp169);
  }

  TNode<BoolT> phi_bb122_19;
  TNode<IntPtrT> phi_bb122_23;
  TNode<IntPtrT> phi_bb122_24;
  TNode<BoolT> phi_bb122_25;
  TNode<BoolT> phi_bb122_26;
  TNode<IntPtrT> phi_bb122_27;
  if (block122.is_used()) {
    ca_.Bind(&block122, &phi_bb122_19, &phi_bb122_23, &phi_bb122_24, &phi_bb122_25, &phi_bb122_26, &phi_bb122_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 197);
    ca_.Goto(&block119, phi_bb122_19, phi_bb122_23, phi_bb122_24, phi_bb122_25, phi_bb122_26, phi_bb122_27);
  }

  TNode<BoolT> phi_bb119_19;
  TNode<IntPtrT> phi_bb119_23;
  TNode<IntPtrT> phi_bb119_24;
  TNode<BoolT> phi_bb119_25;
  TNode<BoolT> phi_bb119_26;
  TNode<IntPtrT> phi_bb119_27;
  if (block119.is_used()) {
    ca_.Bind(&block119, &phi_bb119_19, &phi_bb119_23, &phi_bb119_24, &phi_bb119_25, &phi_bb119_26, &phi_bb119_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 590);
    ca_.Goto(&block113, phi_bb119_19, phi_bb119_23, phi_bb119_24, phi_bb119_25, phi_bb119_26, phi_bb119_27);
  }

  TNode<BoolT> phi_bb113_19;
  TNode<IntPtrT> phi_bb113_23;
  TNode<IntPtrT> phi_bb113_24;
  TNode<BoolT> phi_bb113_25;
  TNode<BoolT> phi_bb113_26;
  TNode<IntPtrT> phi_bb113_27;
  if (block113.is_used()) {
    ca_.Bind(&block113, &phi_bb113_19, &phi_bb113_23, &phi_bb113_24, &phi_bb113_25, &phi_bb113_26, &phi_bb113_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 589);
    ca_.Goto(&block109, phi_bb113_19, phi_bb113_23, phi_bb113_24, phi_bb113_25, phi_bb113_26, phi_bb113_27);
  }

  TNode<BoolT> phi_bb109_19;
  TNode<IntPtrT> phi_bb109_23;
  TNode<IntPtrT> phi_bb109_24;
  TNode<BoolT> phi_bb109_25;
  TNode<BoolT> phi_bb109_26;
  TNode<IntPtrT> phi_bb109_27;
  if (block109.is_used()) {
    ca_.Bind(&block109, &phi_bb109_19, &phi_bb109_23, &phi_bb109_24, &phi_bb109_25, &phi_bb109_26, &phi_bb109_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 586);
    ca_.Goto(&block99, phi_bb109_19, phi_bb109_23, phi_bb109_24, phi_bb109_25, phi_bb109_26, phi_bb109_27, tmp151);
  }

  TNode<BoolT> phi_bb98_19;
  TNode<IntPtrT> phi_bb98_23;
  TNode<IntPtrT> phi_bb98_24;
  TNode<BoolT> phi_bb98_25;
  TNode<BoolT> phi_bb98_26;
  TNode<IntPtrT> phi_bb98_27;
  TNode<IntPtrT> phi_bb98_29;
  if (block98.is_used()) {
    ca_.Bind(&block98, &phi_bb98_19, &phi_bb98_23, &phi_bb98_24, &phi_bb98_25, &phi_bb98_26, &phi_bb98_27, &phi_bb98_29);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 583);
    ca_.Goto(&block92, phi_bb98_19, phi_bb98_23, phi_bb98_24, phi_bb98_25, phi_bb98_26, phi_bb98_27, phi_bb98_29, tmp145);
  }

  TNode<BoolT> phi_bb92_19;
  TNode<IntPtrT> phi_bb92_23;
  TNode<IntPtrT> phi_bb92_24;
  TNode<BoolT> phi_bb92_25;
  TNode<BoolT> phi_bb92_26;
  TNode<IntPtrT> phi_bb92_27;
  TNode<IntPtrT> phi_bb92_29;
  TNode<IntPtrT> phi_bb92_30;
  if (block92.is_used()) {
    ca_.Bind(&block92, &phi_bb92_19, &phi_bb92_23, &phi_bb92_24, &phi_bb92_25, &phi_bb92_26, &phi_bb92_27, &phi_bb92_29, &phi_bb92_30);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 242);
    ca_.Branch(phi_bb92_26, &block124, std::vector<compiler::Node*>{phi_bb92_19, phi_bb92_23, phi_bb92_24, phi_bb92_25, phi_bb92_26, phi_bb92_27, phi_bb92_29, phi_bb92_30}, &block125, std::vector<compiler::Node*>{phi_bb92_19, phi_bb92_23, phi_bb92_24, phi_bb92_25, phi_bb92_26, phi_bb92_27, phi_bb92_29, phi_bb92_30});
  }

  TNode<BoolT> phi_bb124_19;
  TNode<IntPtrT> phi_bb124_23;
  TNode<IntPtrT> phi_bb124_24;
  TNode<BoolT> phi_bb124_25;
  TNode<BoolT> phi_bb124_26;
  TNode<IntPtrT> phi_bb124_27;
  TNode<IntPtrT> phi_bb124_29;
  TNode<IntPtrT> phi_bb124_30;
  TNode<IntPtrT> tmp172;
  TNode<IntPtrT> tmp173;
  if (block124.is_used()) {
    ca_.Bind(&block124, &phi_bb124_19, &phi_bb124_23, &phi_bb124_24, &phi_bb124_25, &phi_bb124_26, &phi_bb124_27, &phi_bb124_29, &phi_bb124_30);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 243);
    tmp172 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x1ull));
    tmp173 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb124_27}, TNode<IntPtrT>{tmp172});
    ca_.Goto(&block123, phi_bb124_19, phi_bb124_23, phi_bb124_24, phi_bb124_25, phi_bb124_26, phi_bb124_27, phi_bb124_29, phi_bb124_30, tmp173);
  }

  TNode<BoolT> phi_bb125_19;
  TNode<IntPtrT> phi_bb125_23;
  TNode<IntPtrT> phi_bb125_24;
  TNode<BoolT> phi_bb125_25;
  TNode<BoolT> phi_bb125_26;
  TNode<IntPtrT> phi_bb125_27;
  TNode<IntPtrT> phi_bb125_29;
  TNode<IntPtrT> phi_bb125_30;
  if (block125.is_used()) {
    ca_.Bind(&block125, &phi_bb125_19, &phi_bb125_23, &phi_bb125_24, &phi_bb125_25, &phi_bb125_26, &phi_bb125_27, &phi_bb125_29, &phi_bb125_30);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 245);
    ca_.Goto(&block123, phi_bb125_19, phi_bb125_23, phi_bb125_24, phi_bb125_25, phi_bb125_26, phi_bb125_27, phi_bb125_29, phi_bb125_30, phi_bb125_27);
  }

  TNode<BoolT> phi_bb123_19;
  TNode<IntPtrT> phi_bb123_23;
  TNode<IntPtrT> phi_bb123_24;
  TNode<BoolT> phi_bb123_25;
  TNode<BoolT> phi_bb123_26;
  TNode<IntPtrT> phi_bb123_27;
  TNode<IntPtrT> phi_bb123_29;
  TNode<IntPtrT> phi_bb123_30;
  TNode<IntPtrT> phi_bb123_31;
  if (block123.is_used()) {
    ca_.Bind(&block123, &phi_bb123_19, &phi_bb123_23, &phi_bb123_24, &phi_bb123_25, &phi_bb123_26, &phi_bb123_27, &phi_bb123_29, &phi_bb123_30, &phi_bb123_31);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 550);
    ca_.Goto(&block8, tmp48, phi_bb123_31, phi_bb123_19);
  }

  TNode<Object> phi_bb8_17;
  TNode<IntPtrT> phi_bb8_18;
  TNode<BoolT> phi_bb8_19;
  TNode<IntPtrT> tmp174;
  TNode<IntPtrT> tmp175;
  TNode<Object> tmp176;
  TNode<IntPtrT> tmp177;
  TNode<IntPtrT> tmp178;
  if (block8.is_used()) {
    ca_.Bind(&block8, &phi_bb8_17, &phi_bb8_18, &phi_bb8_19);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 598);
    tmp174 = Convert_intptr_int32_0(state_, TNode<Int32T>{tmp31});
    tmp175 = Convert_intptr_int32_0(state_, TNode<Int32T>{tmp26});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 597);
    compiler::CodeAssemblerLabel label179(&ca_);
    std::tie(tmp176, tmp177, tmp178) = Subslice_int32_0(state_, TorqueStructSlice_int32_ConstReference_int32_0{TNode<Object>{tmp39}, TNode<IntPtrT>{tmp40}, TNode<IntPtrT>{tmp41}, TorqueStructUnsafe_0{}}, TNode<IntPtrT>{tmp174}, TNode<IntPtrT>{tmp175}, &label179).Flatten();
    ca_.Goto(&block129, phi_bb8_17, phi_bb8_18, phi_bb8_19);
    if (label179.is_used()) {
      ca_.Bind(&label179);
      ca_.Goto(&block130, phi_bb8_17, phi_bb8_18, phi_bb8_19);
    }
  }

  TNode<Object> phi_bb130_17;
  TNode<IntPtrT> phi_bb130_18;
  TNode<BoolT> phi_bb130_19;
  if (block130.is_used()) {
    ca_.Bind(&block130, &phi_bb130_17, &phi_bb130_18, &phi_bb130_19);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 599);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<Object> phi_bb129_17;
  TNode<IntPtrT> phi_bb129_18;
  TNode<BoolT> phi_bb129_19;
  TNode<Int32T> tmp180;
  TNode<BoolT> tmp181;
  if (block129.is_used()) {
    ca_.Bind(&block129, &phi_bb129_17, &phi_bb129_18, &phi_bb129_19);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 610);
    tmp180 = FromConstexpr_int32_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0xaull));
    tmp181 = CodeStubAssembler(state_).Int32LessThanOrEqual(TNode<Int32T>{tmp26}, TNode<Int32T>{tmp180});
    ca_.Branch(tmp181, &block131, std::vector<compiler::Node*>{phi_bb129_17, phi_bb129_18, phi_bb129_19}, &block132, std::vector<compiler::Node*>{phi_bb129_17, phi_bb129_18, phi_bb129_19});
  }

  TNode<Object> phi_bb131_17;
  TNode<IntPtrT> phi_bb131_18;
  TNode<BoolT> phi_bb131_19;
  TNode<RawPtrT> tmp182;
  TNode<RawPtrT> tmp183;
  TNode<IntPtrT> tmp184;
  TNode<Object> tmp185;
  TNode<IntPtrT> tmp186;
  if (block131.is_used()) {
    ca_.Bind(&block131, &phi_bb131_17, &phi_bb131_18, &phi_bb131_19);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 632);
    tmp182 = CodeStubAssembler(state_).StackSlotPtr((CodeStubAssembler(state_).ConstexprInt31Mul((FromConstexpr_constexpr_int31_constexpr_IntegerLiteral_0(state_, (CodeStubAssembler(state_).ConstexprIntegerLiteralAdd((CodeStubAssembler(state_).ConstexprIntegerLiteralAdd(IntegerLiteral(false, 0xaull), IntegerLiteral(false, 0x8ull))), IntegerLiteral(false, 0x2ull))))), (SizeOf_float64_0(state_)))), (SizeOf_float64_0(state_)));
    tmp183 = (TNode<RawPtrT>{tmp182});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 633);
    tmp184 = Convert_intptr_constexpr_int31_0(state_, (CodeStubAssembler(state_).ConstexprInt31Mul((FromConstexpr_constexpr_int31_constexpr_IntegerLiteral_0(state_, (CodeStubAssembler(state_).ConstexprIntegerLiteralAdd((CodeStubAssembler(state_).ConstexprIntegerLiteralAdd(IntegerLiteral(false, 0xaull), IntegerLiteral(false, 0x8ull))), IntegerLiteral(false, 0x2ull))))), (SizeOf_float64_0(state_)))));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 634);
    std::tie(tmp185, tmp186) = NewOffHeapReference_intptr_0(state_, TNode<RawPtrT>{tmp183}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 610);
    ca_.Goto(&block133, phi_bb131_17, phi_bb131_18, phi_bb131_19, tmp185, tmp186, tmp184);
  }

  TNode<Object> phi_bb132_17;
  TNode<IntPtrT> phi_bb132_18;
  TNode<BoolT> phi_bb132_19;
  TNode<IntPtrT> tmp187;
  TNode<IntPtrT> tmp188;
  TNode<IntPtrT> tmp189;
  TNode<IntPtrT> tmp190;
  TNode<IntPtrT> tmp191;
  TNode<IntPtrT> tmp192;
  TNode<IntPtrT> tmp193;
  TNode<IntPtrT> tmp194;
  TNode<RawPtrT> tmp195;
  TNode<Object> tmp196;
  TNode<IntPtrT> tmp197;
  if (block132.is_used()) {
    ca_.Bind(&block132, &phi_bb132_17, &phi_bb132_18, &phi_bb132_19);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 651);
    tmp187 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_float64_0(state_)));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 655);
    tmp188 = CodeStubAssembler(state_).UniqueIntPtrConstant(arraysize(wasm::kFpParamRegisters));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 656);
    tmp189 = Convert_intptr_int32_0(state_, TNode<Int32T>{tmp26});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 655);
    tmp190 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp188}, TNode<IntPtrT>{tmp189});
    tmp191 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x2ull));
    tmp192 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp190}, TNode<IntPtrT>{tmp191});
    tmp193 = CodeStubAssembler(state_).IntPtrMul(TNode<IntPtrT>{tmp192}, TNode<IntPtrT>{tmp187});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 654);
    tmp194 = Convert_intptr_intptr_0(state_, TNode<IntPtrT>{tmp193});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 658);
    tmp195 = CodeStubAssembler(state_).AllocateBuffer(TNode<IntPtrT>{tmp194});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 659);
    std::tie(tmp196, tmp197) = NewOffHeapReference_intptr_0(state_, TNode<RawPtrT>{tmp195}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 610);
    ca_.Goto(&block133, phi_bb132_17, phi_bb132_18, phi_bb132_19, tmp196, tmp197, tmp194);
  }

  TNode<Object> phi_bb133_17;
  TNode<IntPtrT> phi_bb133_18;
  TNode<BoolT> phi_bb133_19;
  TNode<Object> phi_bb133_23;
  TNode<IntPtrT> phi_bb133_24;
  TNode<IntPtrT> phi_bb133_25;
  TNode<Object> tmp198;
  TNode<IntPtrT> tmp199;
  TNode<IntPtrT> tmp200;
  TNode<IntPtrT> tmp201;
  TNode<IntPtrT> tmp202;
  TNode<IntPtrT> tmp203;
  TNode<IntPtrT> tmp204;
  TNode<IntPtrT> tmp205;
  TNode<IntPtrT> tmp206;
  TNode<BoolT> tmp207;
      TNode<Object> tmp209;
  TNode<BoolT> tmp210;
      TNode<Object> tmp212;
  TNode<Smi> tmp213;
      TNode<Object> tmp215;
  TNode<Int32T> tmp216;
      TNode<Object> tmp218;
  TNode<Int32T> tmp219;
      TNode<Object> tmp221;
  if (block133.is_used()) {
    ca_.Bind(&block133, &phi_bb133_17, &phi_bb133_18, &phi_bb133_19, &phi_bb133_23, &phi_bb133_24, &phi_bb133_25);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 664);
    compiler::CodeAssemblerExceptionHandlerLabel catch208__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch208__label);
    std::tie(tmp198, tmp199, tmp200, tmp201, tmp202, tmp203, tmp204, tmp205, tmp206, tmp207) = LocationAllocatorForParams_0(state_, TorqueStructReference_intptr_0{TNode<Object>{phi_bb133_23}, TNode<IntPtrT>{phi_bb133_24}, TorqueStructUnsafe_0{}}, TNode<IntPtrT>{phi_bb133_25}).Flatten();
    }
    if (catch208__label.is_used()) {
      compiler::CodeAssemblerLabel catch208_skip(&ca_);
      ca_.Goto(&catch208_skip);
      ca_.Bind(&catch208__label, &tmp209);
      ca_.Goto(&block144, phi_bb133_17, phi_bb133_18, phi_bb133_19);
      ca_.Bind(&catch208_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 665);
    compiler::CodeAssemblerExceptionHandlerLabel catch211__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch211__label);
    tmp210 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    }
    if (catch211__label.is_used()) {
      compiler::CodeAssemblerLabel catch211_skip(&ca_);
      ca_.Goto(&catch211_skip);
      ca_.Bind(&catch211__label, &tmp212);
      ca_.Goto(&block145, phi_bb133_17, phi_bb133_18, phi_bb133_19);
      ca_.Bind(&catch211_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 674);
    compiler::CodeAssemblerExceptionHandlerLabel catch214__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch214__label);
    tmp213 = Convert_Smi_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    }
    if (catch214__label.is_used()) {
      compiler::CodeAssemblerLabel catch214_skip(&ca_);
      ca_.Goto(&catch214_skip);
      ca_.Bind(&catch214__label, &tmp215);
      ca_.Goto(&block146, phi_bb133_17, phi_bb133_18, phi_bb133_19);
      ca_.Bind(&catch214_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 676);
    compiler::CodeAssemblerExceptionHandlerLabel catch217__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch217__label);
    tmp216 = FromConstexpr_int32_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    }
    if (catch217__label.is_used()) {
      compiler::CodeAssemblerLabel catch217_skip(&ca_);
      ca_.Goto(&catch217_skip);
      ca_.Bind(&catch217__label, &tmp218);
      ca_.Goto(&block147, phi_bb133_17, phi_bb133_18, phi_bb133_19);
      ca_.Bind(&catch217_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 677);
    compiler::CodeAssemblerExceptionHandlerLabel catch220__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch220__label);
    tmp219 = FromConstexpr_int32_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    }
    if (catch220__label.is_used()) {
      compiler::CodeAssemblerLabel catch220_skip(&ca_);
      ca_.Goto(&catch220_skip);
      ca_.Bind(&catch220__label, &tmp221);
      ca_.Goto(&block148, phi_bb133_17, phi_bb133_18, phi_bb133_19);
      ca_.Bind(&catch220_skip);
    }
    ca_.Goto(&block151, phi_bb133_17, phi_bb133_18, phi_bb133_19, tmp199, tmp200, tmp201, tmp202, tmp203, tmp206, tmp207, tmp210, tmp213, tmp216, tmp219);
  }

  TNode<Object> phi_bb144_17;
  TNode<IntPtrT> phi_bb144_18;
  TNode<BoolT> phi_bb144_19;
  TNode<HeapObject> tmp222;
  if (block144.is_used()) {
    ca_.Bind(&block144, &phi_bb144_17, &phi_bb144_18, &phi_bb144_19);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 664);
    tmp222 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb144_17, phi_bb144_18, phi_bb144_19, tmp209, tmp222);
  }

  TNode<Object> phi_bb145_17;
  TNode<IntPtrT> phi_bb145_18;
  TNode<BoolT> phi_bb145_19;
  TNode<HeapObject> tmp223;
  if (block145.is_used()) {
    ca_.Bind(&block145, &phi_bb145_17, &phi_bb145_18, &phi_bb145_19);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 665);
    tmp223 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb145_17, phi_bb145_18, phi_bb145_19, tmp212, tmp223);
  }

  TNode<Object> phi_bb146_17;
  TNode<IntPtrT> phi_bb146_18;
  TNode<BoolT> phi_bb146_19;
  TNode<HeapObject> tmp224;
  if (block146.is_used()) {
    ca_.Bind(&block146, &phi_bb146_17, &phi_bb146_18, &phi_bb146_19);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 674);
    tmp224 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb146_17, phi_bb146_18, phi_bb146_19, tmp215, tmp224);
  }

  TNode<Object> phi_bb147_17;
  TNode<IntPtrT> phi_bb147_18;
  TNode<BoolT> phi_bb147_19;
  TNode<HeapObject> tmp225;
  if (block147.is_used()) {
    ca_.Bind(&block147, &phi_bb147_17, &phi_bb147_18, &phi_bb147_19);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 676);
    tmp225 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb147_17, phi_bb147_18, phi_bb147_19, tmp218, tmp225);
  }

  TNode<Object> phi_bb148_17;
  TNode<IntPtrT> phi_bb148_18;
  TNode<BoolT> phi_bb148_19;
  TNode<HeapObject> tmp226;
  if (block148.is_used()) {
    ca_.Bind(&block148, &phi_bb148_17, &phi_bb148_18, &phi_bb148_19);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 677);
    tmp226 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb148_17, phi_bb148_18, phi_bb148_19, tmp221, tmp226);
  }

  TNode<Object> phi_bb151_17;
  TNode<IntPtrT> phi_bb151_18;
  TNode<BoolT> phi_bb151_19;
  TNode<IntPtrT> phi_bb151_27;
  TNode<IntPtrT> phi_bb151_28;
  TNode<IntPtrT> phi_bb151_29;
  TNode<IntPtrT> phi_bb151_30;
  TNode<IntPtrT> phi_bb151_31;
  TNode<IntPtrT> phi_bb151_34;
  TNode<BoolT> phi_bb151_35;
  TNode<BoolT> phi_bb151_36;
  TNode<Object> phi_bb151_37;
  TNode<Int32T> phi_bb151_38;
  TNode<Int32T> phi_bb151_39;
  TNode<BoolT> tmp227;
      TNode<Object> tmp229;
  if (block151.is_used()) {
    ca_.Bind(&block151, &phi_bb151_17, &phi_bb151_18, &phi_bb151_19, &phi_bb151_27, &phi_bb151_28, &phi_bb151_29, &phi_bb151_30, &phi_bb151_31, &phi_bb151_34, &phi_bb151_35, &phi_bb151_36, &phi_bb151_37, &phi_bb151_38, &phi_bb151_39);
    compiler::CodeAssemblerExceptionHandlerLabel catch228__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch228__label);
    tmp227 = CodeStubAssembler(state_).Int32LessThan(TNode<Int32T>{phi_bb151_38}, TNode<Int32T>{tmp26});
    }
    if (catch228__label.is_used()) {
      compiler::CodeAssemblerLabel catch228_skip(&ca_);
      ca_.Goto(&catch228_skip);
      ca_.Bind(&catch228__label, &tmp229);
      ca_.Goto(&block153, phi_bb151_17, phi_bb151_18, phi_bb151_19, phi_bb151_27, phi_bb151_28, phi_bb151_29, phi_bb151_30, phi_bb151_31, phi_bb151_34, phi_bb151_35, phi_bb151_36, phi_bb151_37, phi_bb151_38, phi_bb151_39, phi_bb151_38);
      ca_.Bind(&catch228_skip);
    }
    ca_.Branch(tmp227, &block149, std::vector<compiler::Node*>{phi_bb151_17, phi_bb151_18, phi_bb151_19, phi_bb151_27, phi_bb151_28, phi_bb151_29, phi_bb151_30, phi_bb151_31, phi_bb151_34, phi_bb151_35, phi_bb151_36, phi_bb151_37, phi_bb151_38, phi_bb151_39}, &block150, std::vector<compiler::Node*>{phi_bb151_17, phi_bb151_18, phi_bb151_19, phi_bb151_27, phi_bb151_28, phi_bb151_29, phi_bb151_30, phi_bb151_31, phi_bb151_34, phi_bb151_35, phi_bb151_36, phi_bb151_37, phi_bb151_38, phi_bb151_39});
  }

  TNode<Object> phi_bb153_17;
  TNode<IntPtrT> phi_bb153_18;
  TNode<BoolT> phi_bb153_19;
  TNode<IntPtrT> phi_bb153_27;
  TNode<IntPtrT> phi_bb153_28;
  TNode<IntPtrT> phi_bb153_29;
  TNode<IntPtrT> phi_bb153_30;
  TNode<IntPtrT> phi_bb153_31;
  TNode<IntPtrT> phi_bb153_34;
  TNode<BoolT> phi_bb153_35;
  TNode<BoolT> phi_bb153_36;
  TNode<Object> phi_bb153_37;
  TNode<Int32T> phi_bb153_38;
  TNode<Int32T> phi_bb153_39;
  TNode<Int32T> phi_bb153_40;
  TNode<HeapObject> tmp230;
  if (block153.is_used()) {
    ca_.Bind(&block153, &phi_bb153_17, &phi_bb153_18, &phi_bb153_19, &phi_bb153_27, &phi_bb153_28, &phi_bb153_29, &phi_bb153_30, &phi_bb153_31, &phi_bb153_34, &phi_bb153_35, &phi_bb153_36, &phi_bb153_37, &phi_bb153_38, &phi_bb153_39, &phi_bb153_40);
    tmp230 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb153_17, phi_bb153_18, phi_bb153_19, tmp229, tmp230);
  }

  TNode<Object> phi_bb149_17;
  TNode<IntPtrT> phi_bb149_18;
  TNode<BoolT> phi_bb149_19;
  TNode<IntPtrT> phi_bb149_27;
  TNode<IntPtrT> phi_bb149_28;
  TNode<IntPtrT> phi_bb149_29;
  TNode<IntPtrT> phi_bb149_30;
  TNode<IntPtrT> phi_bb149_31;
  TNode<IntPtrT> phi_bb149_34;
  TNode<BoolT> phi_bb149_35;
  TNode<BoolT> phi_bb149_36;
  TNode<Object> phi_bb149_37;
  TNode<Int32T> phi_bb149_38;
  TNode<Int32T> phi_bb149_39;
  TNode<IntPtrT> tmp231;
      TNode<Object> tmp233;
  TNode<Object> tmp234;
      TNode<Object> tmp236;
  TNode<Int32T> tmp237;
      TNode<Object> tmp239;
  TNode<Int32T> tmp240;
      TNode<Object> tmp242;
  TNode<IntPtrT> tmp243;
      TNode<Object> tmp245;
  TNode<IntPtrT> tmp246;
  TNode<IntPtrT> tmp247;
  TNode<Object> tmp248;
  TNode<IntPtrT> tmp249;
  TNode<Int32T> tmp250;
  TNode<Int32T> tmp251;
      TNode<Object> tmp253;
  TNode<BoolT> tmp254;
      TNode<Object> tmp256;
  if (block149.is_used()) {
    ca_.Bind(&block149, &phi_bb149_17, &phi_bb149_18, &phi_bb149_19, &phi_bb149_27, &phi_bb149_28, &phi_bb149_29, &phi_bb149_30, &phi_bb149_31, &phi_bb149_34, &phi_bb149_35, &phi_bb149_36, &phi_bb149_37, &phi_bb149_38, &phi_bb149_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 678);
    compiler::CodeAssemblerExceptionHandlerLabel catch232__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch232__label);
    tmp231 = Convert_intptr_int32_0(state_, TNode<Int32T>{phi_bb149_39});
    }
    if (catch232__label.is_used()) {
      compiler::CodeAssemblerLabel catch232_skip(&ca_);
      ca_.Goto(&catch232_skip);
      ca_.Bind(&catch232__label, &tmp233);
      ca_.Goto(&block154, phi_bb149_17, phi_bb149_18, phi_bb149_19, phi_bb149_27, phi_bb149_28, phi_bb149_29, phi_bb149_30, phi_bb149_31, phi_bb149_34, phi_bb149_35, phi_bb149_36, phi_bb149_37, phi_bb149_38, phi_bb149_39, phi_bb149_39);
      ca_.Bind(&catch232_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch235__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch235__label);
    tmp234 = CodeStubAssembler(state_).GetArgumentValue(TorqueStructArguments{TNode<RawPtrT>{p_arguments.frame}, TNode<RawPtrT>{p_arguments.base}, TNode<IntPtrT>{p_arguments.length}, TNode<IntPtrT>{p_arguments.actual_count}}, TNode<IntPtrT>{tmp231});
    }
    if (catch235__label.is_used()) {
      compiler::CodeAssemblerLabel catch235_skip(&ca_);
      ca_.Goto(&catch235_skip);
      ca_.Bind(&catch235__label, &tmp236);
      ca_.Goto(&block155, phi_bb149_17, phi_bb149_18, phi_bb149_19, phi_bb149_27, phi_bb149_28, phi_bb149_29, phi_bb149_30, phi_bb149_31, phi_bb149_34, phi_bb149_35, phi_bb149_36, phi_bb149_37, phi_bb149_38, phi_bb149_39);
      ca_.Bind(&catch235_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 680);
    compiler::CodeAssemblerExceptionHandlerLabel catch238__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch238__label);
    tmp237 = FromConstexpr_int32_constexpr_int31_0(state_, 1);
    }
    if (catch238__label.is_used()) {
      compiler::CodeAssemblerLabel catch238_skip(&ca_);
      ca_.Goto(&catch238_skip);
      ca_.Bind(&catch238__label, &tmp239);
      ca_.Goto(&block156, phi_bb149_17, phi_bb149_18, phi_bb149_19, phi_bb149_27, phi_bb149_28, phi_bb149_29, phi_bb149_30, phi_bb149_31, phi_bb149_34, phi_bb149_35, phi_bb149_36, phi_bb149_37, phi_bb149_38, phi_bb149_39, phi_bb149_38, phi_bb149_38);
      ca_.Bind(&catch238_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch241__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch241__label);
    tmp240 = CodeStubAssembler(state_).Int32Add(TNode<Int32T>{phi_bb149_38}, TNode<Int32T>{tmp237});
    }
    if (catch241__label.is_used()) {
      compiler::CodeAssemblerLabel catch241_skip(&ca_);
      ca_.Goto(&catch241_skip);
      ca_.Bind(&catch241__label, &tmp242);
      ca_.Goto(&block157, phi_bb149_17, phi_bb149_18, phi_bb149_19, phi_bb149_27, phi_bb149_28, phi_bb149_29, phi_bb149_30, phi_bb149_31, phi_bb149_34, phi_bb149_35, phi_bb149_36, phi_bb149_37, phi_bb149_38, phi_bb149_39, phi_bb149_38);
      ca_.Bind(&catch241_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch244__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch244__label);
    tmp243 = Convert_intptr_int32_0(state_, TNode<Int32T>{phi_bb149_38});
    }
    if (catch244__label.is_used()) {
      compiler::CodeAssemblerLabel catch244_skip(&ca_);
      ca_.Goto(&catch244_skip);
      ca_.Bind(&catch244__label, &tmp245);
      ca_.Goto(&block158, phi_bb149_17, phi_bb149_18, phi_bb149_19, phi_bb149_27, phi_bb149_28, phi_bb149_29, phi_bb149_30, phi_bb149_31, phi_bb149_34, phi_bb149_35, phi_bb149_36, phi_bb149_37, phi_bb149_39, phi_bb149_38);
      ca_.Bind(&catch244_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp246 = TimesSizeOf_int32_0(state_, TNode<IntPtrT>{tmp243});
    tmp247 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp177}, TNode<IntPtrT>{tmp246});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp248, tmp249) = NewReference_int32_0(state_, TNode<Object>{tmp176}, TNode<IntPtrT>{tmp247}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 679);
    tmp250 = CodeStubAssembler(state_).LoadReference<Int32T>(CodeStubAssembler::Reference{tmp248, tmp249});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 681);
    compiler::CodeAssemblerExceptionHandlerLabel catch252__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch252__label);
    tmp251 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmI32.raw_bit_field());
    }
    if (catch252__label.is_used()) {
      compiler::CodeAssemblerLabel catch252_skip(&ca_);
      ca_.Goto(&catch252_skip);
      ca_.Bind(&catch252__label, &tmp253);
      ca_.Goto(&block162, phi_bb149_17, phi_bb149_18, phi_bb149_19, phi_bb149_27, phi_bb149_28, phi_bb149_29, phi_bb149_30, phi_bb149_31, phi_bb149_34, phi_bb149_35, phi_bb149_36, phi_bb149_37, phi_bb149_39);
      ca_.Bind(&catch252_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch255__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch255__label);
    tmp254 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp250}, TNode<Int32T>{tmp251});
    }
    if (catch255__label.is_used()) {
      compiler::CodeAssemblerLabel catch255_skip(&ca_);
      ca_.Goto(&catch255_skip);
      ca_.Bind(&catch255__label, &tmp256);
      ca_.Goto(&block163, phi_bb149_17, phi_bb149_18, phi_bb149_19, phi_bb149_27, phi_bb149_28, phi_bb149_29, phi_bb149_30, phi_bb149_31, phi_bb149_34, phi_bb149_35, phi_bb149_36, phi_bb149_37, phi_bb149_39);
      ca_.Bind(&catch255_skip);
    }
    ca_.Branch(tmp254, &block160, std::vector<compiler::Node*>{phi_bb149_17, phi_bb149_18, phi_bb149_19, phi_bb149_27, phi_bb149_28, phi_bb149_29, phi_bb149_30, phi_bb149_31, phi_bb149_34, phi_bb149_35, phi_bb149_36, phi_bb149_37, phi_bb149_39}, &block161, std::vector<compiler::Node*>{phi_bb149_17, phi_bb149_18, phi_bb149_19, phi_bb149_27, phi_bb149_28, phi_bb149_29, phi_bb149_30, phi_bb149_31, phi_bb149_34, phi_bb149_35, phi_bb149_36, phi_bb149_37, phi_bb149_39});
  }

  TNode<Object> phi_bb154_17;
  TNode<IntPtrT> phi_bb154_18;
  TNode<BoolT> phi_bb154_19;
  TNode<IntPtrT> phi_bb154_27;
  TNode<IntPtrT> phi_bb154_28;
  TNode<IntPtrT> phi_bb154_29;
  TNode<IntPtrT> phi_bb154_30;
  TNode<IntPtrT> phi_bb154_31;
  TNode<IntPtrT> phi_bb154_34;
  TNode<BoolT> phi_bb154_35;
  TNode<BoolT> phi_bb154_36;
  TNode<Object> phi_bb154_37;
  TNode<Int32T> phi_bb154_38;
  TNode<Int32T> phi_bb154_39;
  TNode<Int32T> phi_bb154_40;
  TNode<HeapObject> tmp257;
  if (block154.is_used()) {
    ca_.Bind(&block154, &phi_bb154_17, &phi_bb154_18, &phi_bb154_19, &phi_bb154_27, &phi_bb154_28, &phi_bb154_29, &phi_bb154_30, &phi_bb154_31, &phi_bb154_34, &phi_bb154_35, &phi_bb154_36, &phi_bb154_37, &phi_bb154_38, &phi_bb154_39, &phi_bb154_40);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 678);
    tmp257 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb154_17, phi_bb154_18, phi_bb154_19, tmp233, tmp257);
  }

  TNode<Object> phi_bb155_17;
  TNode<IntPtrT> phi_bb155_18;
  TNode<BoolT> phi_bb155_19;
  TNode<IntPtrT> phi_bb155_27;
  TNode<IntPtrT> phi_bb155_28;
  TNode<IntPtrT> phi_bb155_29;
  TNode<IntPtrT> phi_bb155_30;
  TNode<IntPtrT> phi_bb155_31;
  TNode<IntPtrT> phi_bb155_34;
  TNode<BoolT> phi_bb155_35;
  TNode<BoolT> phi_bb155_36;
  TNode<Object> phi_bb155_37;
  TNode<Int32T> phi_bb155_38;
  TNode<Int32T> phi_bb155_39;
  TNode<HeapObject> tmp258;
  if (block155.is_used()) {
    ca_.Bind(&block155, &phi_bb155_17, &phi_bb155_18, &phi_bb155_19, &phi_bb155_27, &phi_bb155_28, &phi_bb155_29, &phi_bb155_30, &phi_bb155_31, &phi_bb155_34, &phi_bb155_35, &phi_bb155_36, &phi_bb155_37, &phi_bb155_38, &phi_bb155_39);
    tmp258 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb155_17, phi_bb155_18, phi_bb155_19, tmp236, tmp258);
  }

  TNode<Object> phi_bb156_17;
  TNode<IntPtrT> phi_bb156_18;
  TNode<BoolT> phi_bb156_19;
  TNode<IntPtrT> phi_bb156_27;
  TNode<IntPtrT> phi_bb156_28;
  TNode<IntPtrT> phi_bb156_29;
  TNode<IntPtrT> phi_bb156_30;
  TNode<IntPtrT> phi_bb156_31;
  TNode<IntPtrT> phi_bb156_34;
  TNode<BoolT> phi_bb156_35;
  TNode<BoolT> phi_bb156_36;
  TNode<Object> phi_bb156_37;
  TNode<Int32T> phi_bb156_38;
  TNode<Int32T> phi_bb156_39;
  TNode<Int32T> phi_bb156_44;
  TNode<Int32T> phi_bb156_45;
  TNode<HeapObject> tmp259;
  if (block156.is_used()) {
    ca_.Bind(&block156, &phi_bb156_17, &phi_bb156_18, &phi_bb156_19, &phi_bb156_27, &phi_bb156_28, &phi_bb156_29, &phi_bb156_30, &phi_bb156_31, &phi_bb156_34, &phi_bb156_35, &phi_bb156_36, &phi_bb156_37, &phi_bb156_38, &phi_bb156_39, &phi_bb156_44, &phi_bb156_45);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 680);
    tmp259 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb156_17, phi_bb156_18, phi_bb156_19, tmp239, tmp259);
  }

  TNode<Object> phi_bb157_17;
  TNode<IntPtrT> phi_bb157_18;
  TNode<BoolT> phi_bb157_19;
  TNode<IntPtrT> phi_bb157_27;
  TNode<IntPtrT> phi_bb157_28;
  TNode<IntPtrT> phi_bb157_29;
  TNode<IntPtrT> phi_bb157_30;
  TNode<IntPtrT> phi_bb157_31;
  TNode<IntPtrT> phi_bb157_34;
  TNode<BoolT> phi_bb157_35;
  TNode<BoolT> phi_bb157_36;
  TNode<Object> phi_bb157_37;
  TNode<Int32T> phi_bb157_38;
  TNode<Int32T> phi_bb157_39;
  TNode<Int32T> phi_bb157_44;
  TNode<HeapObject> tmp260;
  if (block157.is_used()) {
    ca_.Bind(&block157, &phi_bb157_17, &phi_bb157_18, &phi_bb157_19, &phi_bb157_27, &phi_bb157_28, &phi_bb157_29, &phi_bb157_30, &phi_bb157_31, &phi_bb157_34, &phi_bb157_35, &phi_bb157_36, &phi_bb157_37, &phi_bb157_38, &phi_bb157_39, &phi_bb157_44);
    tmp260 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb157_17, phi_bb157_18, phi_bb157_19, tmp242, tmp260);
  }

  TNode<Object> phi_bb158_17;
  TNode<IntPtrT> phi_bb158_18;
  TNode<BoolT> phi_bb158_19;
  TNode<IntPtrT> phi_bb158_27;
  TNode<IntPtrT> phi_bb158_28;
  TNode<IntPtrT> phi_bb158_29;
  TNode<IntPtrT> phi_bb158_30;
  TNode<IntPtrT> phi_bb158_31;
  TNode<IntPtrT> phi_bb158_34;
  TNode<BoolT> phi_bb158_35;
  TNode<BoolT> phi_bb158_36;
  TNode<Object> phi_bb158_37;
  TNode<Int32T> phi_bb158_39;
  TNode<Int32T> phi_bb158_44;
  TNode<HeapObject> tmp261;
  if (block158.is_used()) {
    ca_.Bind(&block158, &phi_bb158_17, &phi_bb158_18, &phi_bb158_19, &phi_bb158_27, &phi_bb158_28, &phi_bb158_29, &phi_bb158_30, &phi_bb158_31, &phi_bb158_34, &phi_bb158_35, &phi_bb158_36, &phi_bb158_37, &phi_bb158_39, &phi_bb158_44);
    tmp261 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb158_17, phi_bb158_18, phi_bb158_19, tmp245, tmp261);
  }

  TNode<Object> phi_bb162_17;
  TNode<IntPtrT> phi_bb162_18;
  TNode<BoolT> phi_bb162_19;
  TNode<IntPtrT> phi_bb162_27;
  TNode<IntPtrT> phi_bb162_28;
  TNode<IntPtrT> phi_bb162_29;
  TNode<IntPtrT> phi_bb162_30;
  TNode<IntPtrT> phi_bb162_31;
  TNode<IntPtrT> phi_bb162_34;
  TNode<BoolT> phi_bb162_35;
  TNode<BoolT> phi_bb162_36;
  TNode<Object> phi_bb162_37;
  TNode<Int32T> phi_bb162_39;
  TNode<HeapObject> tmp262;
  if (block162.is_used()) {
    ca_.Bind(&block162, &phi_bb162_17, &phi_bb162_18, &phi_bb162_19, &phi_bb162_27, &phi_bb162_28, &phi_bb162_29, &phi_bb162_30, &phi_bb162_31, &phi_bb162_34, &phi_bb162_35, &phi_bb162_36, &phi_bb162_37, &phi_bb162_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 681);
    tmp262 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb162_17, phi_bb162_18, phi_bb162_19, tmp253, tmp262);
  }

  TNode<Object> phi_bb163_17;
  TNode<IntPtrT> phi_bb163_18;
  TNode<BoolT> phi_bb163_19;
  TNode<IntPtrT> phi_bb163_27;
  TNode<IntPtrT> phi_bb163_28;
  TNode<IntPtrT> phi_bb163_29;
  TNode<IntPtrT> phi_bb163_30;
  TNode<IntPtrT> phi_bb163_31;
  TNode<IntPtrT> phi_bb163_34;
  TNode<BoolT> phi_bb163_35;
  TNode<BoolT> phi_bb163_36;
  TNode<Object> phi_bb163_37;
  TNode<Int32T> phi_bb163_39;
  TNode<HeapObject> tmp263;
  if (block163.is_used()) {
    ca_.Bind(&block163, &phi_bb163_17, &phi_bb163_18, &phi_bb163_19, &phi_bb163_27, &phi_bb163_28, &phi_bb163_29, &phi_bb163_30, &phi_bb163_31, &phi_bb163_34, &phi_bb163_35, &phi_bb163_36, &phi_bb163_37, &phi_bb163_39);
    tmp263 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb163_17, phi_bb163_18, phi_bb163_19, tmp256, tmp263);
  }

  TNode<Object> phi_bb160_17;
  TNode<IntPtrT> phi_bb160_18;
  TNode<BoolT> phi_bb160_19;
  TNode<IntPtrT> phi_bb160_27;
  TNode<IntPtrT> phi_bb160_28;
  TNode<IntPtrT> phi_bb160_29;
  TNode<IntPtrT> phi_bb160_30;
  TNode<IntPtrT> phi_bb160_31;
  TNode<IntPtrT> phi_bb160_34;
  TNode<BoolT> phi_bb160_35;
  TNode<BoolT> phi_bb160_36;
  TNode<Object> phi_bb160_37;
  TNode<Int32T> phi_bb160_39;
  TNode<IntPtrT> tmp264;
  TNode<IntPtrT> tmp265;
  TNode<IntPtrT> tmp266;
  TNode<BoolT> tmp267;
  if (block160.is_used()) {
    ca_.Bind(&block160, &phi_bb160_17, &phi_bb160_18, &phi_bb160_19, &phi_bb160_27, &phi_bb160_28, &phi_bb160_29, &phi_bb160_30, &phi_bb160_31, &phi_bb160_34, &phi_bb160_35, &phi_bb160_36, &phi_bb160_37, &phi_bb160_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 308);
    tmp264 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp265 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb160_27}, TNode<IntPtrT>{tmp264});
    tmp266 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp267 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb160_27}, TNode<IntPtrT>{tmp266});
    ca_.Branch(tmp267, &block166, std::vector<compiler::Node*>{phi_bb160_17, phi_bb160_18, phi_bb160_19, phi_bb160_28, phi_bb160_29, phi_bb160_30, phi_bb160_31, phi_bb160_34, phi_bb160_35, phi_bb160_36, phi_bb160_37, phi_bb160_39}, &block167, std::vector<compiler::Node*>{phi_bb160_17, phi_bb160_18, phi_bb160_19, phi_bb160_28, phi_bb160_29, phi_bb160_30, phi_bb160_31, phi_bb160_34, phi_bb160_35, phi_bb160_36, phi_bb160_37, phi_bb160_39});
  }

  TNode<Object> phi_bb166_17;
  TNode<IntPtrT> phi_bb166_18;
  TNode<BoolT> phi_bb166_19;
  TNode<IntPtrT> phi_bb166_28;
  TNode<IntPtrT> phi_bb166_29;
  TNode<IntPtrT> phi_bb166_30;
  TNode<IntPtrT> phi_bb166_31;
  TNode<IntPtrT> phi_bb166_34;
  TNode<BoolT> phi_bb166_35;
  TNode<BoolT> phi_bb166_36;
  TNode<Object> phi_bb166_37;
  TNode<Int32T> phi_bb166_39;
  TNode<Object> tmp268;
  TNode<IntPtrT> tmp269;
  TNode<IntPtrT> tmp270;
  TNode<IntPtrT> tmp271;
  if (block166.is_used()) {
    ca_.Bind(&block166, &phi_bb166_17, &phi_bb166_18, &phi_bb166_19, &phi_bb166_28, &phi_bb166_29, &phi_bb166_30, &phi_bb166_31, &phi_bb166_34, &phi_bb166_35, &phi_bb166_36, &phi_bb166_37, &phi_bb166_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    std::tie(tmp268, tmp269) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb166_29}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 311);
    tmp270 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp271 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb166_29}, TNode<IntPtrT>{tmp270});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 312);
    ca_.Goto(&block165, phi_bb166_17, phi_bb166_18, phi_bb166_19, phi_bb166_28, tmp271, phi_bb166_30, phi_bb166_31, phi_bb166_34, phi_bb166_35, phi_bb166_36, phi_bb166_37, phi_bb166_39, tmp268, tmp269);
  }

  TNode<Object> phi_bb167_17;
  TNode<IntPtrT> phi_bb167_18;
  TNode<BoolT> phi_bb167_19;
  TNode<IntPtrT> phi_bb167_28;
  TNode<IntPtrT> phi_bb167_29;
  TNode<IntPtrT> phi_bb167_30;
  TNode<IntPtrT> phi_bb167_31;
  TNode<IntPtrT> phi_bb167_34;
  TNode<BoolT> phi_bb167_35;
  TNode<BoolT> phi_bb167_36;
  TNode<Object> phi_bb167_37;
  TNode<Int32T> phi_bb167_39;
  if (block167.is_used()) {
    ca_.Bind(&block167, &phi_bb167_17, &phi_bb167_18, &phi_bb167_19, &phi_bb167_28, &phi_bb167_29, &phi_bb167_30, &phi_bb167_31, &phi_bb167_34, &phi_bb167_35, &phi_bb167_36, &phi_bb167_37, &phi_bb167_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block169, phi_bb167_17, phi_bb167_18, phi_bb167_19, phi_bb167_28, phi_bb167_29, phi_bb167_30, phi_bb167_31, phi_bb167_34, phi_bb167_35, phi_bb167_36, phi_bb167_37, phi_bb167_39);
    } else {
      ca_.Goto(&block170, phi_bb167_17, phi_bb167_18, phi_bb167_19, phi_bb167_28, phi_bb167_29, phi_bb167_30, phi_bb167_31, phi_bb167_34, phi_bb167_35, phi_bb167_36, phi_bb167_37, phi_bb167_39);
    }
  }

  TNode<Object> phi_bb169_17;
  TNode<IntPtrT> phi_bb169_18;
  TNode<BoolT> phi_bb169_19;
  TNode<IntPtrT> phi_bb169_28;
  TNode<IntPtrT> phi_bb169_29;
  TNode<IntPtrT> phi_bb169_30;
  TNode<IntPtrT> phi_bb169_31;
  TNode<IntPtrT> phi_bb169_34;
  TNode<BoolT> phi_bb169_35;
  TNode<BoolT> phi_bb169_36;
  TNode<Object> phi_bb169_37;
  TNode<Int32T> phi_bb169_39;
  TNode<Object> tmp272;
  TNode<IntPtrT> tmp273;
  TNode<IntPtrT> tmp274;
  TNode<IntPtrT> tmp275;
  if (block169.is_used()) {
    ca_.Bind(&block169, &phi_bb169_17, &phi_bb169_18, &phi_bb169_19, &phi_bb169_28, &phi_bb169_29, &phi_bb169_30, &phi_bb169_31, &phi_bb169_34, &phi_bb169_35, &phi_bb169_36, &phi_bb169_37, &phi_bb169_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp272, tmp273) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb169_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp274 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp275 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb169_31}, TNode<IntPtrT>{tmp274});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block168, phi_bb169_17, phi_bb169_18, phi_bb169_19, phi_bb169_28, phi_bb169_29, phi_bb169_30, tmp275, phi_bb169_34, phi_bb169_35, phi_bb169_36, phi_bb169_37, phi_bb169_39, tmp272, tmp273);
  }

  TNode<Object> phi_bb170_17;
  TNode<IntPtrT> phi_bb170_18;
  TNode<BoolT> phi_bb170_19;
  TNode<IntPtrT> phi_bb170_28;
  TNode<IntPtrT> phi_bb170_29;
  TNode<IntPtrT> phi_bb170_30;
  TNode<IntPtrT> phi_bb170_31;
  TNode<IntPtrT> phi_bb170_34;
  TNode<BoolT> phi_bb170_35;
  TNode<BoolT> phi_bb170_36;
  TNode<Object> phi_bb170_37;
  TNode<Int32T> phi_bb170_39;
  TNode<IntPtrT> tmp276;
  TNode<BoolT> tmp277;
  if (block170.is_used()) {
    ca_.Bind(&block170, &phi_bb170_17, &phi_bb170_18, &phi_bb170_19, &phi_bb170_28, &phi_bb170_29, &phi_bb170_30, &phi_bb170_31, &phi_bb170_34, &phi_bb170_35, &phi_bb170_36, &phi_bb170_37, &phi_bb170_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp276 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp277 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb170_34}, TNode<IntPtrT>{tmp276});
    ca_.Branch(tmp277, &block172, std::vector<compiler::Node*>{phi_bb170_17, phi_bb170_18, phi_bb170_19, phi_bb170_28, phi_bb170_29, phi_bb170_30, phi_bb170_31, phi_bb170_34, phi_bb170_35, phi_bb170_36, phi_bb170_37, phi_bb170_39}, &block173, std::vector<compiler::Node*>{phi_bb170_17, phi_bb170_18, phi_bb170_19, phi_bb170_28, phi_bb170_29, phi_bb170_30, phi_bb170_31, phi_bb170_34, phi_bb170_35, phi_bb170_36, phi_bb170_37, phi_bb170_39});
  }

  TNode<Object> phi_bb172_17;
  TNode<IntPtrT> phi_bb172_18;
  TNode<BoolT> phi_bb172_19;
  TNode<IntPtrT> phi_bb172_28;
  TNode<IntPtrT> phi_bb172_29;
  TNode<IntPtrT> phi_bb172_30;
  TNode<IntPtrT> phi_bb172_31;
  TNode<IntPtrT> phi_bb172_34;
  TNode<BoolT> phi_bb172_35;
  TNode<BoolT> phi_bb172_36;
  TNode<Object> phi_bb172_37;
  TNode<Int32T> phi_bb172_39;
  TNode<Object> tmp278;
  TNode<IntPtrT> tmp279;
  TNode<IntPtrT> tmp280;
  TNode<BoolT> tmp281;
  if (block172.is_used()) {
    ca_.Bind(&block172, &phi_bb172_17, &phi_bb172_18, &phi_bb172_19, &phi_bb172_28, &phi_bb172_29, &phi_bb172_30, &phi_bb172_31, &phi_bb172_34, &phi_bb172_35, &phi_bb172_36, &phi_bb172_37, &phi_bb172_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp278, tmp279) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb172_34}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp280 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp281 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block168, phi_bb172_17, phi_bb172_18, phi_bb172_19, phi_bb172_28, phi_bb172_29, phi_bb172_30, phi_bb172_31, tmp280, tmp281, phi_bb172_36, phi_bb172_37, phi_bb172_39, tmp278, tmp279);
  }

  TNode<Object> phi_bb173_17;
  TNode<IntPtrT> phi_bb173_18;
  TNode<BoolT> phi_bb173_19;
  TNode<IntPtrT> phi_bb173_28;
  TNode<IntPtrT> phi_bb173_29;
  TNode<IntPtrT> phi_bb173_30;
  TNode<IntPtrT> phi_bb173_31;
  TNode<IntPtrT> phi_bb173_34;
  TNode<BoolT> phi_bb173_35;
  TNode<BoolT> phi_bb173_36;
  TNode<Object> phi_bb173_37;
  TNode<Int32T> phi_bb173_39;
  TNode<Object> tmp282;
  TNode<IntPtrT> tmp283;
  TNode<IntPtrT> tmp284;
  TNode<IntPtrT> tmp285;
  TNode<IntPtrT> tmp286;
  TNode<IntPtrT> tmp287;
  TNode<BoolT> tmp288;
  if (block173.is_used()) {
    ca_.Bind(&block173, &phi_bb173_17, &phi_bb173_18, &phi_bb173_19, &phi_bb173_28, &phi_bb173_29, &phi_bb173_30, &phi_bb173_31, &phi_bb173_34, &phi_bb173_35, &phi_bb173_36, &phi_bb173_37, &phi_bb173_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp282, tmp283) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb173_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp284 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp285 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb173_31}, TNode<IntPtrT>{tmp284});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp286 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp287 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp285}, TNode<IntPtrT>{tmp286});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp288 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block168, phi_bb173_17, phi_bb173_18, phi_bb173_19, phi_bb173_28, phi_bb173_29, phi_bb173_30, tmp287, tmp285, tmp288, phi_bb173_36, phi_bb173_37, phi_bb173_39, tmp282, tmp283);
  }

  TNode<Object> phi_bb168_17;
  TNode<IntPtrT> phi_bb168_18;
  TNode<BoolT> phi_bb168_19;
  TNode<IntPtrT> phi_bb168_28;
  TNode<IntPtrT> phi_bb168_29;
  TNode<IntPtrT> phi_bb168_30;
  TNode<IntPtrT> phi_bb168_31;
  TNode<IntPtrT> phi_bb168_34;
  TNode<BoolT> phi_bb168_35;
  TNode<BoolT> phi_bb168_36;
  TNode<Object> phi_bb168_37;
  TNode<Int32T> phi_bb168_39;
  TNode<Object> phi_bb168_42;
  TNode<IntPtrT> phi_bb168_43;
  if (block168.is_used()) {
    ca_.Bind(&block168, &phi_bb168_17, &phi_bb168_18, &phi_bb168_19, &phi_bb168_28, &phi_bb168_29, &phi_bb168_30, &phi_bb168_31, &phi_bb168_34, &phi_bb168_35, &phi_bb168_36, &phi_bb168_37, &phi_bb168_39, &phi_bb168_42, &phi_bb168_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 314);
    ca_.Goto(&block165, phi_bb168_17, phi_bb168_18, phi_bb168_19, phi_bb168_28, phi_bb168_29, phi_bb168_30, phi_bb168_31, phi_bb168_34, phi_bb168_35, phi_bb168_36, phi_bb168_37, phi_bb168_39, phi_bb168_42, phi_bb168_43);
  }

  TNode<Object> phi_bb165_17;
  TNode<IntPtrT> phi_bb165_18;
  TNode<BoolT> phi_bb165_19;
  TNode<IntPtrT> phi_bb165_28;
  TNode<IntPtrT> phi_bb165_29;
  TNode<IntPtrT> phi_bb165_30;
  TNode<IntPtrT> phi_bb165_31;
  TNode<IntPtrT> phi_bb165_34;
  TNode<BoolT> phi_bb165_35;
  TNode<BoolT> phi_bb165_36;
  TNode<Object> phi_bb165_37;
  TNode<Int32T> phi_bb165_39;
  TNode<Object> phi_bb165_42;
  TNode<IntPtrT> phi_bb165_43;
  TNode<Smi> tmp289;
      TNode<Object> tmp292;
  if (block165.is_used()) {
    ca_.Bind(&block165, &phi_bb165_17, &phi_bb165_18, &phi_bb165_19, &phi_bb165_28, &phi_bb165_29, &phi_bb165_30, &phi_bb165_31, &phi_bb165_34, &phi_bb165_35, &phi_bb165_36, &phi_bb165_37, &phi_bb165_39, &phi_bb165_42, &phi_bb165_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 684);
    compiler::CodeAssemblerLabel label290(&ca_);
    compiler::CodeAssemblerExceptionHandlerLabel catch291__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch291__label);
    tmp289 = Cast_Smi_0(state_, TNode<Object>{tmp234}, &label290);
    }
    if (catch291__label.is_used()) {
      compiler::CodeAssemblerLabel catch291_skip(&ca_);
      ca_.Goto(&catch291_skip);
      ca_.Bind(&catch291__label, &tmp292);
      ca_.Goto(&block178, phi_bb165_17, phi_bb165_18, phi_bb165_19, phi_bb165_28, phi_bb165_29, phi_bb165_30, phi_bb165_31, phi_bb165_34, phi_bb165_35, phi_bb165_36, phi_bb165_37, phi_bb165_39, phi_bb165_42, phi_bb165_43);
      ca_.Bind(&catch291_skip);
    }
    ca_.Goto(&block176, phi_bb165_17, phi_bb165_18, phi_bb165_19, phi_bb165_28, phi_bb165_29, phi_bb165_30, phi_bb165_31, phi_bb165_34, phi_bb165_35, phi_bb165_36, phi_bb165_37, phi_bb165_39, phi_bb165_42, phi_bb165_43);
    if (label290.is_used()) {
      ca_.Bind(&label290);
      ca_.Goto(&block177, phi_bb165_17, phi_bb165_18, phi_bb165_19, phi_bb165_28, phi_bb165_29, phi_bb165_30, phi_bb165_31, phi_bb165_34, phi_bb165_35, phi_bb165_36, phi_bb165_37, phi_bb165_39, phi_bb165_42, phi_bb165_43);
    }
  }

  TNode<Object> phi_bb178_17;
  TNode<IntPtrT> phi_bb178_18;
  TNode<BoolT> phi_bb178_19;
  TNode<IntPtrT> phi_bb178_28;
  TNode<IntPtrT> phi_bb178_29;
  TNode<IntPtrT> phi_bb178_30;
  TNode<IntPtrT> phi_bb178_31;
  TNode<IntPtrT> phi_bb178_34;
  TNode<BoolT> phi_bb178_35;
  TNode<BoolT> phi_bb178_36;
  TNode<Object> phi_bb178_37;
  TNode<Int32T> phi_bb178_39;
  TNode<Object> phi_bb178_42;
  TNode<IntPtrT> phi_bb178_43;
  TNode<HeapObject> tmp293;
  if (block178.is_used()) {
    ca_.Bind(&block178, &phi_bb178_17, &phi_bb178_18, &phi_bb178_19, &phi_bb178_28, &phi_bb178_29, &phi_bb178_30, &phi_bb178_31, &phi_bb178_34, &phi_bb178_35, &phi_bb178_36, &phi_bb178_37, &phi_bb178_39, &phi_bb178_42, &phi_bb178_43);
    tmp293 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb178_17, phi_bb178_18, phi_bb178_19, tmp292, tmp293);
  }

  TNode<Object> phi_bb177_17;
  TNode<IntPtrT> phi_bb177_18;
  TNode<BoolT> phi_bb177_19;
  TNode<IntPtrT> phi_bb177_28;
  TNode<IntPtrT> phi_bb177_29;
  TNode<IntPtrT> phi_bb177_30;
  TNode<IntPtrT> phi_bb177_31;
  TNode<IntPtrT> phi_bb177_34;
  TNode<BoolT> phi_bb177_35;
  TNode<BoolT> phi_bb177_36;
  TNode<Object> phi_bb177_37;
  TNode<Int32T> phi_bb177_39;
  TNode<Object> phi_bb177_42;
  TNode<IntPtrT> phi_bb177_43;
  TNode<Int32T> tmp294;
      TNode<Object> tmp296;
  TNode<IntPtrT> tmp297;
      TNode<Object> tmp299;
  if (block177.is_used()) {
    ca_.Bind(&block177, &phi_bb177_17, &phi_bb177_18, &phi_bb177_19, &phi_bb177_28, &phi_bb177_29, &phi_bb177_30, &phi_bb177_31, &phi_bb177_34, &phi_bb177_35, &phi_bb177_36, &phi_bb177_37, &phi_bb177_39, &phi_bb177_42, &phi_bb177_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 688);
    compiler::CodeAssemblerExceptionHandlerLabel catch295__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch295__label);
    tmp294 = ca_.CallBuiltin<Int32T>(Builtin::kWasmTaggedNonSmiToInt32, p_context, ca_.UncheckedCast<HeapObject>(tmp234));
    }
    if (catch295__label.is_used()) {
      compiler::CodeAssemblerLabel catch295_skip(&ca_);
      ca_.Goto(&catch295_skip);
      ca_.Bind(&catch295__label, &tmp296);
      ca_.Goto(&block181, phi_bb177_17, phi_bb177_18, phi_bb177_19, phi_bb177_28, phi_bb177_29, phi_bb177_30, phi_bb177_31, phi_bb177_34, phi_bb177_35, phi_bb177_36, phi_bb177_37, phi_bb177_39, phi_bb177_42, phi_bb177_43, phi_bb177_42, phi_bb177_43);
      ca_.Bind(&catch295_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch298__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch298__label);
    tmp297 = Convert_intptr_int32_0(state_, TNode<Int32T>{tmp294});
    }
    if (catch298__label.is_used()) {
      compiler::CodeAssemblerLabel catch298_skip(&ca_);
      ca_.Goto(&catch298_skip);
      ca_.Bind(&catch298__label, &tmp299);
      ca_.Goto(&block182, phi_bb177_17, phi_bb177_18, phi_bb177_19, phi_bb177_28, phi_bb177_29, phi_bb177_30, phi_bb177_31, phi_bb177_34, phi_bb177_35, phi_bb177_36, phi_bb177_37, phi_bb177_39, phi_bb177_42, phi_bb177_43, phi_bb177_42, phi_bb177_43);
      ca_.Bind(&catch298_skip);
    }
    CodeStubAssembler(state_).StoreReference<IntPtrT>(CodeStubAssembler::Reference{phi_bb177_42, phi_bb177_43}, tmp297);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 684);
    ca_.Goto(&block174, phi_bb177_17, phi_bb177_18, phi_bb177_19, phi_bb177_28, phi_bb177_29, phi_bb177_30, phi_bb177_31, phi_bb177_34, phi_bb177_35, phi_bb177_36, phi_bb177_37, phi_bb177_39, phi_bb177_42, phi_bb177_43);
  }

  TNode<Object> phi_bb176_17;
  TNode<IntPtrT> phi_bb176_18;
  TNode<BoolT> phi_bb176_19;
  TNode<IntPtrT> phi_bb176_28;
  TNode<IntPtrT> phi_bb176_29;
  TNode<IntPtrT> phi_bb176_30;
  TNode<IntPtrT> phi_bb176_31;
  TNode<IntPtrT> phi_bb176_34;
  TNode<BoolT> phi_bb176_35;
  TNode<BoolT> phi_bb176_36;
  TNode<Object> phi_bb176_37;
  TNode<Int32T> phi_bb176_39;
  TNode<Object> phi_bb176_42;
  TNode<IntPtrT> phi_bb176_43;
  TNode<Int32T> tmp300;
      TNode<Object> tmp302;
  TNode<IntPtrT> tmp303;
      TNode<Object> tmp305;
  if (block176.is_used()) {
    ca_.Bind(&block176, &phi_bb176_17, &phi_bb176_18, &phi_bb176_19, &phi_bb176_28, &phi_bb176_29, &phi_bb176_30, &phi_bb176_31, &phi_bb176_34, &phi_bb176_35, &phi_bb176_36, &phi_bb176_37, &phi_bb176_39, &phi_bb176_42, &phi_bb176_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 685);
    compiler::CodeAssemblerExceptionHandlerLabel catch301__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch301__label);
    tmp300 = CodeStubAssembler(state_).SmiToInt32(TNode<Smi>{tmp289});
    }
    if (catch301__label.is_used()) {
      compiler::CodeAssemblerLabel catch301_skip(&ca_);
      ca_.Goto(&catch301_skip);
      ca_.Bind(&catch301__label, &tmp302);
      ca_.Goto(&block179, phi_bb176_17, phi_bb176_18, phi_bb176_19, phi_bb176_28, phi_bb176_29, phi_bb176_30, phi_bb176_31, phi_bb176_34, phi_bb176_35, phi_bb176_36, phi_bb176_37, phi_bb176_39, phi_bb176_42, phi_bb176_43, phi_bb176_42, phi_bb176_43);
      ca_.Bind(&catch301_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch304__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch304__label);
    tmp303 = Convert_intptr_int32_0(state_, TNode<Int32T>{tmp300});
    }
    if (catch304__label.is_used()) {
      compiler::CodeAssemblerLabel catch304_skip(&ca_);
      ca_.Goto(&catch304_skip);
      ca_.Bind(&catch304__label, &tmp305);
      ca_.Goto(&block180, phi_bb176_17, phi_bb176_18, phi_bb176_19, phi_bb176_28, phi_bb176_29, phi_bb176_30, phi_bb176_31, phi_bb176_34, phi_bb176_35, phi_bb176_36, phi_bb176_37, phi_bb176_39, phi_bb176_42, phi_bb176_43, phi_bb176_42, phi_bb176_43);
      ca_.Bind(&catch304_skip);
    }
    CodeStubAssembler(state_).StoreReference<IntPtrT>(CodeStubAssembler::Reference{phi_bb176_42, phi_bb176_43}, tmp303);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 684);
    ca_.Goto(&block174, phi_bb176_17, phi_bb176_18, phi_bb176_19, phi_bb176_28, phi_bb176_29, phi_bb176_30, phi_bb176_31, phi_bb176_34, phi_bb176_35, phi_bb176_36, phi_bb176_37, phi_bb176_39, phi_bb176_42, phi_bb176_43);
  }

  TNode<Object> phi_bb179_17;
  TNode<IntPtrT> phi_bb179_18;
  TNode<BoolT> phi_bb179_19;
  TNode<IntPtrT> phi_bb179_28;
  TNode<IntPtrT> phi_bb179_29;
  TNode<IntPtrT> phi_bb179_30;
  TNode<IntPtrT> phi_bb179_31;
  TNode<IntPtrT> phi_bb179_34;
  TNode<BoolT> phi_bb179_35;
  TNode<BoolT> phi_bb179_36;
  TNode<Object> phi_bb179_37;
  TNode<Int32T> phi_bb179_39;
  TNode<Object> phi_bb179_42;
  TNode<IntPtrT> phi_bb179_43;
  TNode<Object> phi_bb179_46;
  TNode<IntPtrT> phi_bb179_47;
  TNode<HeapObject> tmp306;
  if (block179.is_used()) {
    ca_.Bind(&block179, &phi_bb179_17, &phi_bb179_18, &phi_bb179_19, &phi_bb179_28, &phi_bb179_29, &phi_bb179_30, &phi_bb179_31, &phi_bb179_34, &phi_bb179_35, &phi_bb179_36, &phi_bb179_37, &phi_bb179_39, &phi_bb179_42, &phi_bb179_43, &phi_bb179_46, &phi_bb179_47);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 685);
    tmp306 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb179_17, phi_bb179_18, phi_bb179_19, tmp302, tmp306);
  }

  TNode<Object> phi_bb180_17;
  TNode<IntPtrT> phi_bb180_18;
  TNode<BoolT> phi_bb180_19;
  TNode<IntPtrT> phi_bb180_28;
  TNode<IntPtrT> phi_bb180_29;
  TNode<IntPtrT> phi_bb180_30;
  TNode<IntPtrT> phi_bb180_31;
  TNode<IntPtrT> phi_bb180_34;
  TNode<BoolT> phi_bb180_35;
  TNode<BoolT> phi_bb180_36;
  TNode<Object> phi_bb180_37;
  TNode<Int32T> phi_bb180_39;
  TNode<Object> phi_bb180_42;
  TNode<IntPtrT> phi_bb180_43;
  TNode<Object> phi_bb180_46;
  TNode<IntPtrT> phi_bb180_47;
  TNode<HeapObject> tmp307;
  if (block180.is_used()) {
    ca_.Bind(&block180, &phi_bb180_17, &phi_bb180_18, &phi_bb180_19, &phi_bb180_28, &phi_bb180_29, &phi_bb180_30, &phi_bb180_31, &phi_bb180_34, &phi_bb180_35, &phi_bb180_36, &phi_bb180_37, &phi_bb180_39, &phi_bb180_42, &phi_bb180_43, &phi_bb180_46, &phi_bb180_47);
    tmp307 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb180_17, phi_bb180_18, phi_bb180_19, tmp305, tmp307);
  }

  TNode<Object> phi_bb181_17;
  TNode<IntPtrT> phi_bb181_18;
  TNode<BoolT> phi_bb181_19;
  TNode<IntPtrT> phi_bb181_28;
  TNode<IntPtrT> phi_bb181_29;
  TNode<IntPtrT> phi_bb181_30;
  TNode<IntPtrT> phi_bb181_31;
  TNode<IntPtrT> phi_bb181_34;
  TNode<BoolT> phi_bb181_35;
  TNode<BoolT> phi_bb181_36;
  TNode<Object> phi_bb181_37;
  TNode<Int32T> phi_bb181_39;
  TNode<Object> phi_bb181_42;
  TNode<IntPtrT> phi_bb181_43;
  TNode<Object> phi_bb181_46;
  TNode<IntPtrT> phi_bb181_47;
  TNode<HeapObject> tmp308;
  if (block181.is_used()) {
    ca_.Bind(&block181, &phi_bb181_17, &phi_bb181_18, &phi_bb181_19, &phi_bb181_28, &phi_bb181_29, &phi_bb181_30, &phi_bb181_31, &phi_bb181_34, &phi_bb181_35, &phi_bb181_36, &phi_bb181_37, &phi_bb181_39, &phi_bb181_42, &phi_bb181_43, &phi_bb181_46, &phi_bb181_47);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 688);
    tmp308 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb181_17, phi_bb181_18, phi_bb181_19, tmp296, tmp308);
  }

  TNode<Object> phi_bb182_17;
  TNode<IntPtrT> phi_bb182_18;
  TNode<BoolT> phi_bb182_19;
  TNode<IntPtrT> phi_bb182_28;
  TNode<IntPtrT> phi_bb182_29;
  TNode<IntPtrT> phi_bb182_30;
  TNode<IntPtrT> phi_bb182_31;
  TNode<IntPtrT> phi_bb182_34;
  TNode<BoolT> phi_bb182_35;
  TNode<BoolT> phi_bb182_36;
  TNode<Object> phi_bb182_37;
  TNode<Int32T> phi_bb182_39;
  TNode<Object> phi_bb182_42;
  TNode<IntPtrT> phi_bb182_43;
  TNode<Object> phi_bb182_46;
  TNode<IntPtrT> phi_bb182_47;
  TNode<HeapObject> tmp309;
  if (block182.is_used()) {
    ca_.Bind(&block182, &phi_bb182_17, &phi_bb182_18, &phi_bb182_19, &phi_bb182_28, &phi_bb182_29, &phi_bb182_30, &phi_bb182_31, &phi_bb182_34, &phi_bb182_35, &phi_bb182_36, &phi_bb182_37, &phi_bb182_39, &phi_bb182_42, &phi_bb182_43, &phi_bb182_46, &phi_bb182_47);
    tmp309 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb182_17, phi_bb182_18, phi_bb182_19, tmp299, tmp309);
  }

  TNode<Object> phi_bb174_17;
  TNode<IntPtrT> phi_bb174_18;
  TNode<BoolT> phi_bb174_19;
  TNode<IntPtrT> phi_bb174_28;
  TNode<IntPtrT> phi_bb174_29;
  TNode<IntPtrT> phi_bb174_30;
  TNode<IntPtrT> phi_bb174_31;
  TNode<IntPtrT> phi_bb174_34;
  TNode<BoolT> phi_bb174_35;
  TNode<BoolT> phi_bb174_36;
  TNode<Object> phi_bb174_37;
  TNode<Int32T> phi_bb174_39;
  TNode<Object> phi_bb174_42;
  TNode<IntPtrT> phi_bb174_43;
  if (block174.is_used()) {
    ca_.Bind(&block174, &phi_bb174_17, &phi_bb174_18, &phi_bb174_19, &phi_bb174_28, &phi_bb174_29, &phi_bb174_30, &phi_bb174_31, &phi_bb174_34, &phi_bb174_35, &phi_bb174_36, &phi_bb174_37, &phi_bb174_39, &phi_bb174_42, &phi_bb174_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 681);
    ca_.Goto(&block164, phi_bb174_17, phi_bb174_18, phi_bb174_19, tmp265, phi_bb174_28, phi_bb174_29, phi_bb174_30, phi_bb174_31, phi_bb174_34, phi_bb174_35, phi_bb174_36, phi_bb174_37, phi_bb174_39);
  }

  TNode<Object> phi_bb161_17;
  TNode<IntPtrT> phi_bb161_18;
  TNode<BoolT> phi_bb161_19;
  TNode<IntPtrT> phi_bb161_27;
  TNode<IntPtrT> phi_bb161_28;
  TNode<IntPtrT> phi_bb161_29;
  TNode<IntPtrT> phi_bb161_30;
  TNode<IntPtrT> phi_bb161_31;
  TNode<IntPtrT> phi_bb161_34;
  TNode<BoolT> phi_bb161_35;
  TNode<BoolT> phi_bb161_36;
  TNode<Object> phi_bb161_37;
  TNode<Int32T> phi_bb161_39;
  TNode<Int32T> tmp310;
      TNode<Object> tmp312;
  TNode<BoolT> tmp313;
      TNode<Object> tmp315;
  if (block161.is_used()) {
    ca_.Bind(&block161, &phi_bb161_17, &phi_bb161_18, &phi_bb161_19, &phi_bb161_27, &phi_bb161_28, &phi_bb161_29, &phi_bb161_30, &phi_bb161_31, &phi_bb161_34, &phi_bb161_35, &phi_bb161_36, &phi_bb161_37, &phi_bb161_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 691);
    compiler::CodeAssemblerExceptionHandlerLabel catch311__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch311__label);
    tmp310 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmF32.raw_bit_field());
    }
    if (catch311__label.is_used()) {
      compiler::CodeAssemblerLabel catch311_skip(&ca_);
      ca_.Goto(&catch311_skip);
      ca_.Bind(&catch311__label, &tmp312);
      ca_.Goto(&block185, phi_bb161_17, phi_bb161_18, phi_bb161_19, phi_bb161_27, phi_bb161_28, phi_bb161_29, phi_bb161_30, phi_bb161_31, phi_bb161_34, phi_bb161_35, phi_bb161_36, phi_bb161_37, phi_bb161_39);
      ca_.Bind(&catch311_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch314__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch314__label);
    tmp313 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp250}, TNode<Int32T>{tmp310});
    }
    if (catch314__label.is_used()) {
      compiler::CodeAssemblerLabel catch314_skip(&ca_);
      ca_.Goto(&catch314_skip);
      ca_.Bind(&catch314__label, &tmp315);
      ca_.Goto(&block186, phi_bb161_17, phi_bb161_18, phi_bb161_19, phi_bb161_27, phi_bb161_28, phi_bb161_29, phi_bb161_30, phi_bb161_31, phi_bb161_34, phi_bb161_35, phi_bb161_36, phi_bb161_37, phi_bb161_39);
      ca_.Bind(&catch314_skip);
    }
    ca_.Branch(tmp313, &block183, std::vector<compiler::Node*>{phi_bb161_17, phi_bb161_18, phi_bb161_19, phi_bb161_27, phi_bb161_28, phi_bb161_29, phi_bb161_30, phi_bb161_31, phi_bb161_34, phi_bb161_35, phi_bb161_36, phi_bb161_37, phi_bb161_39}, &block184, std::vector<compiler::Node*>{phi_bb161_17, phi_bb161_18, phi_bb161_19, phi_bb161_27, phi_bb161_28, phi_bb161_29, phi_bb161_30, phi_bb161_31, phi_bb161_34, phi_bb161_35, phi_bb161_36, phi_bb161_37, phi_bb161_39});
  }

  TNode<Object> phi_bb185_17;
  TNode<IntPtrT> phi_bb185_18;
  TNode<BoolT> phi_bb185_19;
  TNode<IntPtrT> phi_bb185_27;
  TNode<IntPtrT> phi_bb185_28;
  TNode<IntPtrT> phi_bb185_29;
  TNode<IntPtrT> phi_bb185_30;
  TNode<IntPtrT> phi_bb185_31;
  TNode<IntPtrT> phi_bb185_34;
  TNode<BoolT> phi_bb185_35;
  TNode<BoolT> phi_bb185_36;
  TNode<Object> phi_bb185_37;
  TNode<Int32T> phi_bb185_39;
  TNode<HeapObject> tmp316;
  if (block185.is_used()) {
    ca_.Bind(&block185, &phi_bb185_17, &phi_bb185_18, &phi_bb185_19, &phi_bb185_27, &phi_bb185_28, &phi_bb185_29, &phi_bb185_30, &phi_bb185_31, &phi_bb185_34, &phi_bb185_35, &phi_bb185_36, &phi_bb185_37, &phi_bb185_39);
    tmp316 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb185_17, phi_bb185_18, phi_bb185_19, tmp312, tmp316);
  }

  TNode<Object> phi_bb186_17;
  TNode<IntPtrT> phi_bb186_18;
  TNode<BoolT> phi_bb186_19;
  TNode<IntPtrT> phi_bb186_27;
  TNode<IntPtrT> phi_bb186_28;
  TNode<IntPtrT> phi_bb186_29;
  TNode<IntPtrT> phi_bb186_30;
  TNode<IntPtrT> phi_bb186_31;
  TNode<IntPtrT> phi_bb186_34;
  TNode<BoolT> phi_bb186_35;
  TNode<BoolT> phi_bb186_36;
  TNode<Object> phi_bb186_37;
  TNode<Int32T> phi_bb186_39;
  TNode<HeapObject> tmp317;
  if (block186.is_used()) {
    ca_.Bind(&block186, &phi_bb186_17, &phi_bb186_18, &phi_bb186_19, &phi_bb186_27, &phi_bb186_28, &phi_bb186_29, &phi_bb186_30, &phi_bb186_31, &phi_bb186_34, &phi_bb186_35, &phi_bb186_36, &phi_bb186_37, &phi_bb186_39);
    tmp317 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb186_17, phi_bb186_18, phi_bb186_19, tmp315, tmp317);
  }

  TNode<Object> phi_bb183_17;
  TNode<IntPtrT> phi_bb183_18;
  TNode<BoolT> phi_bb183_19;
  TNode<IntPtrT> phi_bb183_27;
  TNode<IntPtrT> phi_bb183_28;
  TNode<IntPtrT> phi_bb183_29;
  TNode<IntPtrT> phi_bb183_30;
  TNode<IntPtrT> phi_bb183_31;
  TNode<IntPtrT> phi_bb183_34;
  TNode<BoolT> phi_bb183_35;
  TNode<BoolT> phi_bb183_36;
  TNode<Object> phi_bb183_37;
  TNode<Int32T> phi_bb183_39;
  TNode<IntPtrT> tmp318;
  TNode<IntPtrT> tmp319;
  TNode<IntPtrT> tmp320;
  TNode<BoolT> tmp321;
  if (block183.is_used()) {
    ca_.Bind(&block183, &phi_bb183_17, &phi_bb183_18, &phi_bb183_19, &phi_bb183_27, &phi_bb183_28, &phi_bb183_29, &phi_bb183_30, &phi_bb183_31, &phi_bb183_34, &phi_bb183_35, &phi_bb183_36, &phi_bb183_37, &phi_bb183_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 318);
    tmp318 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp319 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb183_28}, TNode<IntPtrT>{tmp318});
    tmp320 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp321 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb183_28}, TNode<IntPtrT>{tmp320});
    ca_.Branch(tmp321, &block189, std::vector<compiler::Node*>{phi_bb183_17, phi_bb183_18, phi_bb183_19, phi_bb183_27, phi_bb183_29, phi_bb183_30, phi_bb183_31, phi_bb183_34, phi_bb183_35, phi_bb183_36, phi_bb183_37, phi_bb183_39}, &block190, std::vector<compiler::Node*>{phi_bb183_17, phi_bb183_18, phi_bb183_19, phi_bb183_27, phi_bb183_29, phi_bb183_30, phi_bb183_31, phi_bb183_34, phi_bb183_35, phi_bb183_36, phi_bb183_37, phi_bb183_39});
  }

  TNode<Object> phi_bb189_17;
  TNode<IntPtrT> phi_bb189_18;
  TNode<BoolT> phi_bb189_19;
  TNode<IntPtrT> phi_bb189_27;
  TNode<IntPtrT> phi_bb189_29;
  TNode<IntPtrT> phi_bb189_30;
  TNode<IntPtrT> phi_bb189_31;
  TNode<IntPtrT> phi_bb189_34;
  TNode<BoolT> phi_bb189_35;
  TNode<BoolT> phi_bb189_36;
  TNode<Object> phi_bb189_37;
  TNode<Int32T> phi_bb189_39;
  TNode<Object> tmp322;
  TNode<IntPtrT> tmp323;
  TNode<IntPtrT> tmp324;
  TNode<IntPtrT> tmp325;
  if (block189.is_used()) {
    ca_.Bind(&block189, &phi_bb189_17, &phi_bb189_18, &phi_bb189_19, &phi_bb189_27, &phi_bb189_29, &phi_bb189_30, &phi_bb189_31, &phi_bb189_34, &phi_bb189_35, &phi_bb189_36, &phi_bb189_37, &phi_bb189_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 319);
    std::tie(tmp322, tmp323) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb189_30}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 321);
    tmp324 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_float64_0(state_)));
    tmp325 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb189_30}, TNode<IntPtrT>{tmp324});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 322);
    ca_.Goto(&block188, phi_bb189_17, phi_bb189_18, phi_bb189_19, phi_bb189_27, phi_bb189_29, tmp325, phi_bb189_31, phi_bb189_34, phi_bb189_35, phi_bb189_36, phi_bb189_37, phi_bb189_39, tmp322, tmp323);
  }

  TNode<Object> phi_bb190_17;
  TNode<IntPtrT> phi_bb190_18;
  TNode<BoolT> phi_bb190_19;
  TNode<IntPtrT> phi_bb190_27;
  TNode<IntPtrT> phi_bb190_29;
  TNode<IntPtrT> phi_bb190_30;
  TNode<IntPtrT> phi_bb190_31;
  TNode<IntPtrT> phi_bb190_34;
  TNode<BoolT> phi_bb190_35;
  TNode<BoolT> phi_bb190_36;
  TNode<Object> phi_bb190_37;
  TNode<Int32T> phi_bb190_39;
  if (block190.is_used()) {
    ca_.Bind(&block190, &phi_bb190_17, &phi_bb190_18, &phi_bb190_19, &phi_bb190_27, &phi_bb190_29, &phi_bb190_30, &phi_bb190_31, &phi_bb190_34, &phi_bb190_35, &phi_bb190_36, &phi_bb190_37, &phi_bb190_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block192, phi_bb190_17, phi_bb190_18, phi_bb190_19, phi_bb190_27, phi_bb190_29, phi_bb190_30, phi_bb190_31, phi_bb190_34, phi_bb190_35, phi_bb190_36, phi_bb190_37, phi_bb190_39);
    } else {
      ca_.Goto(&block193, phi_bb190_17, phi_bb190_18, phi_bb190_19, phi_bb190_27, phi_bb190_29, phi_bb190_30, phi_bb190_31, phi_bb190_34, phi_bb190_35, phi_bb190_36, phi_bb190_37, phi_bb190_39);
    }
  }

  TNode<Object> phi_bb192_17;
  TNode<IntPtrT> phi_bb192_18;
  TNode<BoolT> phi_bb192_19;
  TNode<IntPtrT> phi_bb192_27;
  TNode<IntPtrT> phi_bb192_29;
  TNode<IntPtrT> phi_bb192_30;
  TNode<IntPtrT> phi_bb192_31;
  TNode<IntPtrT> phi_bb192_34;
  TNode<BoolT> phi_bb192_35;
  TNode<BoolT> phi_bb192_36;
  TNode<Object> phi_bb192_37;
  TNode<Int32T> phi_bb192_39;
  TNode<Object> tmp326;
  TNode<IntPtrT> tmp327;
  TNode<IntPtrT> tmp328;
  TNode<IntPtrT> tmp329;
  if (block192.is_used()) {
    ca_.Bind(&block192, &phi_bb192_17, &phi_bb192_18, &phi_bb192_19, &phi_bb192_27, &phi_bb192_29, &phi_bb192_30, &phi_bb192_31, &phi_bb192_34, &phi_bb192_35, &phi_bb192_36, &phi_bb192_37, &phi_bb192_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp326, tmp327) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb192_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp328 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp329 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb192_31}, TNode<IntPtrT>{tmp328});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block191, phi_bb192_17, phi_bb192_18, phi_bb192_19, phi_bb192_27, phi_bb192_29, phi_bb192_30, tmp329, phi_bb192_34, phi_bb192_35, phi_bb192_36, phi_bb192_37, phi_bb192_39, tmp326, tmp327);
  }

  TNode<Object> phi_bb193_17;
  TNode<IntPtrT> phi_bb193_18;
  TNode<BoolT> phi_bb193_19;
  TNode<IntPtrT> phi_bb193_27;
  TNode<IntPtrT> phi_bb193_29;
  TNode<IntPtrT> phi_bb193_30;
  TNode<IntPtrT> phi_bb193_31;
  TNode<IntPtrT> phi_bb193_34;
  TNode<BoolT> phi_bb193_35;
  TNode<BoolT> phi_bb193_36;
  TNode<Object> phi_bb193_37;
  TNode<Int32T> phi_bb193_39;
  TNode<IntPtrT> tmp330;
  TNode<BoolT> tmp331;
  if (block193.is_used()) {
    ca_.Bind(&block193, &phi_bb193_17, &phi_bb193_18, &phi_bb193_19, &phi_bb193_27, &phi_bb193_29, &phi_bb193_30, &phi_bb193_31, &phi_bb193_34, &phi_bb193_35, &phi_bb193_36, &phi_bb193_37, &phi_bb193_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp330 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp331 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb193_34}, TNode<IntPtrT>{tmp330});
    ca_.Branch(tmp331, &block195, std::vector<compiler::Node*>{phi_bb193_17, phi_bb193_18, phi_bb193_19, phi_bb193_27, phi_bb193_29, phi_bb193_30, phi_bb193_31, phi_bb193_34, phi_bb193_35, phi_bb193_36, phi_bb193_37, phi_bb193_39}, &block196, std::vector<compiler::Node*>{phi_bb193_17, phi_bb193_18, phi_bb193_19, phi_bb193_27, phi_bb193_29, phi_bb193_30, phi_bb193_31, phi_bb193_34, phi_bb193_35, phi_bb193_36, phi_bb193_37, phi_bb193_39});
  }

  TNode<Object> phi_bb195_17;
  TNode<IntPtrT> phi_bb195_18;
  TNode<BoolT> phi_bb195_19;
  TNode<IntPtrT> phi_bb195_27;
  TNode<IntPtrT> phi_bb195_29;
  TNode<IntPtrT> phi_bb195_30;
  TNode<IntPtrT> phi_bb195_31;
  TNode<IntPtrT> phi_bb195_34;
  TNode<BoolT> phi_bb195_35;
  TNode<BoolT> phi_bb195_36;
  TNode<Object> phi_bb195_37;
  TNode<Int32T> phi_bb195_39;
  TNode<Object> tmp332;
  TNode<IntPtrT> tmp333;
  TNode<IntPtrT> tmp334;
  TNode<BoolT> tmp335;
  if (block195.is_used()) {
    ca_.Bind(&block195, &phi_bb195_17, &phi_bb195_18, &phi_bb195_19, &phi_bb195_27, &phi_bb195_29, &phi_bb195_30, &phi_bb195_31, &phi_bb195_34, &phi_bb195_35, &phi_bb195_36, &phi_bb195_37, &phi_bb195_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp332, tmp333) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb195_34}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp334 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp335 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block191, phi_bb195_17, phi_bb195_18, phi_bb195_19, phi_bb195_27, phi_bb195_29, phi_bb195_30, phi_bb195_31, tmp334, tmp335, phi_bb195_36, phi_bb195_37, phi_bb195_39, tmp332, tmp333);
  }

  TNode<Object> phi_bb196_17;
  TNode<IntPtrT> phi_bb196_18;
  TNode<BoolT> phi_bb196_19;
  TNode<IntPtrT> phi_bb196_27;
  TNode<IntPtrT> phi_bb196_29;
  TNode<IntPtrT> phi_bb196_30;
  TNode<IntPtrT> phi_bb196_31;
  TNode<IntPtrT> phi_bb196_34;
  TNode<BoolT> phi_bb196_35;
  TNode<BoolT> phi_bb196_36;
  TNode<Object> phi_bb196_37;
  TNode<Int32T> phi_bb196_39;
  TNode<Object> tmp336;
  TNode<IntPtrT> tmp337;
  TNode<IntPtrT> tmp338;
  TNode<IntPtrT> tmp339;
  TNode<IntPtrT> tmp340;
  TNode<IntPtrT> tmp341;
  TNode<BoolT> tmp342;
  if (block196.is_used()) {
    ca_.Bind(&block196, &phi_bb196_17, &phi_bb196_18, &phi_bb196_19, &phi_bb196_27, &phi_bb196_29, &phi_bb196_30, &phi_bb196_31, &phi_bb196_34, &phi_bb196_35, &phi_bb196_36, &phi_bb196_37, &phi_bb196_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp336, tmp337) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb196_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp338 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp339 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb196_31}, TNode<IntPtrT>{tmp338});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp340 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp341 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp339}, TNode<IntPtrT>{tmp340});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp342 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block191, phi_bb196_17, phi_bb196_18, phi_bb196_19, phi_bb196_27, phi_bb196_29, phi_bb196_30, tmp341, tmp339, tmp342, phi_bb196_36, phi_bb196_37, phi_bb196_39, tmp336, tmp337);
  }

  TNode<Object> phi_bb191_17;
  TNode<IntPtrT> phi_bb191_18;
  TNode<BoolT> phi_bb191_19;
  TNode<IntPtrT> phi_bb191_27;
  TNode<IntPtrT> phi_bb191_29;
  TNode<IntPtrT> phi_bb191_30;
  TNode<IntPtrT> phi_bb191_31;
  TNode<IntPtrT> phi_bb191_34;
  TNode<BoolT> phi_bb191_35;
  TNode<BoolT> phi_bb191_36;
  TNode<Object> phi_bb191_37;
  TNode<Int32T> phi_bb191_39;
  TNode<Object> phi_bb191_42;
  TNode<IntPtrT> phi_bb191_43;
  if (block191.is_used()) {
    ca_.Bind(&block191, &phi_bb191_17, &phi_bb191_18, &phi_bb191_19, &phi_bb191_27, &phi_bb191_29, &phi_bb191_30, &phi_bb191_31, &phi_bb191_34, &phi_bb191_35, &phi_bb191_36, &phi_bb191_37, &phi_bb191_39, &phi_bb191_42, &phi_bb191_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 324);
    ca_.Goto(&block188, phi_bb191_17, phi_bb191_18, phi_bb191_19, phi_bb191_27, phi_bb191_29, phi_bb191_30, phi_bb191_31, phi_bb191_34, phi_bb191_35, phi_bb191_36, phi_bb191_37, phi_bb191_39, phi_bb191_42, phi_bb191_43);
  }

  TNode<Object> phi_bb188_17;
  TNode<IntPtrT> phi_bb188_18;
  TNode<BoolT> phi_bb188_19;
  TNode<IntPtrT> phi_bb188_27;
  TNode<IntPtrT> phi_bb188_29;
  TNode<IntPtrT> phi_bb188_30;
  TNode<IntPtrT> phi_bb188_31;
  TNode<IntPtrT> phi_bb188_34;
  TNode<BoolT> phi_bb188_35;
  TNode<BoolT> phi_bb188_36;
  TNode<Object> phi_bb188_37;
  TNode<Int32T> phi_bb188_39;
  TNode<Object> phi_bb188_42;
  TNode<IntPtrT> phi_bb188_43;
  if (block188.is_used()) {
    ca_.Bind(&block188, &phi_bb188_17, &phi_bb188_18, &phi_bb188_19, &phi_bb188_27, &phi_bb188_29, &phi_bb188_30, &phi_bb188_31, &phi_bb188_34, &phi_bb188_35, &phi_bb188_36, &phi_bb188_37, &phi_bb188_39, &phi_bb188_42, &phi_bb188_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 693);
    if ((((wasm::kIsFpAlwaysDouble || wasm::kIsBigEndian) || wasm::kIsBigEndianOnSim))) {
      ca_.Goto(&block197, phi_bb188_17, phi_bb188_18, phi_bb188_19, phi_bb188_27, phi_bb188_29, phi_bb188_30, phi_bb188_31, phi_bb188_34, phi_bb188_35, phi_bb188_36, phi_bb188_37, phi_bb188_39, phi_bb188_42, phi_bb188_43);
    } else {
      ca_.Goto(&block198, phi_bb188_17, phi_bb188_18, phi_bb188_19, phi_bb188_27, phi_bb188_29, phi_bb188_30, phi_bb188_31, phi_bb188_34, phi_bb188_35, phi_bb188_36, phi_bb188_37, phi_bb188_39, phi_bb188_42, phi_bb188_43);
    }
  }

  TNode<Object> phi_bb197_17;
  TNode<IntPtrT> phi_bb197_18;
  TNode<BoolT> phi_bb197_19;
  TNode<IntPtrT> phi_bb197_27;
  TNode<IntPtrT> phi_bb197_29;
  TNode<IntPtrT> phi_bb197_30;
  TNode<IntPtrT> phi_bb197_31;
  TNode<IntPtrT> phi_bb197_34;
  TNode<BoolT> phi_bb197_35;
  TNode<BoolT> phi_bb197_36;
  TNode<Object> phi_bb197_37;
  TNode<Int32T> phi_bb197_39;
  TNode<Object> phi_bb197_42;
  TNode<IntPtrT> phi_bb197_43;
      TNode<Object> tmp344;
  if (block197.is_used()) {
    ca_.Bind(&block197, &phi_bb197_17, &phi_bb197_18, &phi_bb197_19, &phi_bb197_27, &phi_bb197_29, &phi_bb197_30, &phi_bb197_31, &phi_bb197_34, &phi_bb197_35, &phi_bb197_36, &phi_bb197_37, &phi_bb197_39, &phi_bb197_42, &phi_bb197_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 694);
    compiler::CodeAssemblerExceptionHandlerLabel catch343__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch343__label);
    HandleF32Params_0(state_, TNode<NativeContext>{p_context}, TorqueStructLocationAllocator_0{TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb197_27}, TNode<IntPtrT>{tmp319}, TNode<IntPtrT>{phi_bb197_29}, TNode<IntPtrT>{phi_bb197_30}, TNode<IntPtrT>{phi_bb197_31}, TNode<IntPtrT>{tmp204}, TNode<IntPtrT>{tmp205}, TNode<IntPtrT>{phi_bb197_34}, TNode<BoolT>{phi_bb197_35}}, TorqueStructReference_intptr_0{TNode<Object>{phi_bb197_42}, TNode<IntPtrT>{phi_bb197_43}, TorqueStructUnsafe_0{}}, TNode<Object>{tmp234});
    }
    if (catch343__label.is_used()) {
      compiler::CodeAssemblerLabel catch343_skip(&ca_);
      ca_.Goto(&catch343_skip);
      ca_.Bind(&catch343__label, &tmp344);
      ca_.Goto(&block200, phi_bb197_17, phi_bb197_18, phi_bb197_19, phi_bb197_27, phi_bb197_29, phi_bb197_30, phi_bb197_31, phi_bb197_34, phi_bb197_35, phi_bb197_36, phi_bb197_37, phi_bb197_39, phi_bb197_42, phi_bb197_43, phi_bb197_27, phi_bb197_29, phi_bb197_30, phi_bb197_31, phi_bb197_34, phi_bb197_35, phi_bb197_42, phi_bb197_43);
      ca_.Bind(&catch343_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 693);
    ca_.Goto(&block199, phi_bb197_17, phi_bb197_18, phi_bb197_19, phi_bb197_27, phi_bb197_29, phi_bb197_30, phi_bb197_31, phi_bb197_34, phi_bb197_35, phi_bb197_36, phi_bb197_37, phi_bb197_39, phi_bb197_42, phi_bb197_43);
  }

  TNode<Object> phi_bb200_17;
  TNode<IntPtrT> phi_bb200_18;
  TNode<BoolT> phi_bb200_19;
  TNode<IntPtrT> phi_bb200_27;
  TNode<IntPtrT> phi_bb200_29;
  TNode<IntPtrT> phi_bb200_30;
  TNode<IntPtrT> phi_bb200_31;
  TNode<IntPtrT> phi_bb200_34;
  TNode<BoolT> phi_bb200_35;
  TNode<BoolT> phi_bb200_36;
  TNode<Object> phi_bb200_37;
  TNode<Int32T> phi_bb200_39;
  TNode<Object> phi_bb200_42;
  TNode<IntPtrT> phi_bb200_43;
  TNode<IntPtrT> phi_bb200_46;
  TNode<IntPtrT> phi_bb200_48;
  TNode<IntPtrT> phi_bb200_49;
  TNode<IntPtrT> phi_bb200_50;
  TNode<IntPtrT> phi_bb200_53;
  TNode<BoolT> phi_bb200_54;
  TNode<Object> phi_bb200_55;
  TNode<IntPtrT> phi_bb200_56;
  TNode<HeapObject> tmp345;
  if (block200.is_used()) {
    ca_.Bind(&block200, &phi_bb200_17, &phi_bb200_18, &phi_bb200_19, &phi_bb200_27, &phi_bb200_29, &phi_bb200_30, &phi_bb200_31, &phi_bb200_34, &phi_bb200_35, &phi_bb200_36, &phi_bb200_37, &phi_bb200_39, &phi_bb200_42, &phi_bb200_43, &phi_bb200_46, &phi_bb200_48, &phi_bb200_49, &phi_bb200_50, &phi_bb200_53, &phi_bb200_54, &phi_bb200_55, &phi_bb200_56);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 694);
    tmp345 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb200_17, phi_bb200_18, phi_bb200_19, tmp344, tmp345);
  }

  TNode<Object> phi_bb198_17;
  TNode<IntPtrT> phi_bb198_18;
  TNode<BoolT> phi_bb198_19;
  TNode<IntPtrT> phi_bb198_27;
  TNode<IntPtrT> phi_bb198_29;
  TNode<IntPtrT> phi_bb198_30;
  TNode<IntPtrT> phi_bb198_31;
  TNode<IntPtrT> phi_bb198_34;
  TNode<BoolT> phi_bb198_35;
  TNode<BoolT> phi_bb198_36;
  TNode<Object> phi_bb198_37;
  TNode<Int32T> phi_bb198_39;
  TNode<Object> phi_bb198_42;
  TNode<IntPtrT> phi_bb198_43;
  TNode<Float32T> tmp346;
      TNode<Object> tmp348;
  TNode<Uint32T> tmp349;
      TNode<Object> tmp351;
  TNode<IntPtrT> tmp352;
      TNode<Object> tmp354;
  if (block198.is_used()) {
    ca_.Bind(&block198, &phi_bb198_17, &phi_bb198_18, &phi_bb198_19, &phi_bb198_27, &phi_bb198_29, &phi_bb198_30, &phi_bb198_31, &phi_bb198_34, &phi_bb198_35, &phi_bb198_36, &phi_bb198_37, &phi_bb198_39, &phi_bb198_42, &phi_bb198_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 696);
    compiler::CodeAssemblerExceptionHandlerLabel catch347__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch347__label);
    tmp346 = ca_.CallBuiltin<Float32T>(Builtin::kWasmTaggedToFloat32, p_context, tmp234);
    }
    if (catch347__label.is_used()) {
      compiler::CodeAssemblerLabel catch347_skip(&ca_);
      ca_.Goto(&catch347_skip);
      ca_.Bind(&catch347__label, &tmp348);
      ca_.Goto(&block201, phi_bb198_17, phi_bb198_18, phi_bb198_19, phi_bb198_27, phi_bb198_29, phi_bb198_30, phi_bb198_31, phi_bb198_34, phi_bb198_35, phi_bb198_36, phi_bb198_37, phi_bb198_39, phi_bb198_42, phi_bb198_43, phi_bb198_42, phi_bb198_43);
      ca_.Bind(&catch347_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch350__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch350__label);
    tmp349 = Bitcast_WasmCodePointer_float32_0(state_, TNode<Float32T>{tmp346});
    }
    if (catch350__label.is_used()) {
      compiler::CodeAssemblerLabel catch350_skip(&ca_);
      ca_.Goto(&catch350_skip);
      ca_.Bind(&catch350__label, &tmp351);
      ca_.Goto(&block202, phi_bb198_17, phi_bb198_18, phi_bb198_19, phi_bb198_27, phi_bb198_29, phi_bb198_30, phi_bb198_31, phi_bb198_34, phi_bb198_35, phi_bb198_36, phi_bb198_37, phi_bb198_39, phi_bb198_42, phi_bb198_43, phi_bb198_42, phi_bb198_43);
      ca_.Bind(&catch350_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch353__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch353__label);
    tmp352 = Convert_intptr_WasmCodePointer_0(state_, TNode<Uint32T>{tmp349});
    }
    if (catch353__label.is_used()) {
      compiler::CodeAssemblerLabel catch353_skip(&ca_);
      ca_.Goto(&catch353_skip);
      ca_.Bind(&catch353__label, &tmp354);
      ca_.Goto(&block203, phi_bb198_17, phi_bb198_18, phi_bb198_19, phi_bb198_27, phi_bb198_29, phi_bb198_30, phi_bb198_31, phi_bb198_34, phi_bb198_35, phi_bb198_36, phi_bb198_37, phi_bb198_39, phi_bb198_42, phi_bb198_43, phi_bb198_42, phi_bb198_43);
      ca_.Bind(&catch353_skip);
    }
    CodeStubAssembler(state_).StoreReference<IntPtrT>(CodeStubAssembler::Reference{phi_bb198_42, phi_bb198_43}, tmp352);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 693);
    ca_.Goto(&block199, phi_bb198_17, phi_bb198_18, phi_bb198_19, phi_bb198_27, phi_bb198_29, phi_bb198_30, phi_bb198_31, phi_bb198_34, phi_bb198_35, phi_bb198_36, phi_bb198_37, phi_bb198_39, phi_bb198_42, phi_bb198_43);
  }

  TNode<Object> phi_bb201_17;
  TNode<IntPtrT> phi_bb201_18;
  TNode<BoolT> phi_bb201_19;
  TNode<IntPtrT> phi_bb201_27;
  TNode<IntPtrT> phi_bb201_29;
  TNode<IntPtrT> phi_bb201_30;
  TNode<IntPtrT> phi_bb201_31;
  TNode<IntPtrT> phi_bb201_34;
  TNode<BoolT> phi_bb201_35;
  TNode<BoolT> phi_bb201_36;
  TNode<Object> phi_bb201_37;
  TNode<Int32T> phi_bb201_39;
  TNode<Object> phi_bb201_42;
  TNode<IntPtrT> phi_bb201_43;
  TNode<Object> phi_bb201_44;
  TNode<IntPtrT> phi_bb201_45;
  TNode<HeapObject> tmp355;
  if (block201.is_used()) {
    ca_.Bind(&block201, &phi_bb201_17, &phi_bb201_18, &phi_bb201_19, &phi_bb201_27, &phi_bb201_29, &phi_bb201_30, &phi_bb201_31, &phi_bb201_34, &phi_bb201_35, &phi_bb201_36, &phi_bb201_37, &phi_bb201_39, &phi_bb201_42, &phi_bb201_43, &phi_bb201_44, &phi_bb201_45);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 696);
    tmp355 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb201_17, phi_bb201_18, phi_bb201_19, tmp348, tmp355);
  }

  TNode<Object> phi_bb202_17;
  TNode<IntPtrT> phi_bb202_18;
  TNode<BoolT> phi_bb202_19;
  TNode<IntPtrT> phi_bb202_27;
  TNode<IntPtrT> phi_bb202_29;
  TNode<IntPtrT> phi_bb202_30;
  TNode<IntPtrT> phi_bb202_31;
  TNode<IntPtrT> phi_bb202_34;
  TNode<BoolT> phi_bb202_35;
  TNode<BoolT> phi_bb202_36;
  TNode<Object> phi_bb202_37;
  TNode<Int32T> phi_bb202_39;
  TNode<Object> phi_bb202_42;
  TNode<IntPtrT> phi_bb202_43;
  TNode<Object> phi_bb202_44;
  TNode<IntPtrT> phi_bb202_45;
  TNode<HeapObject> tmp356;
  if (block202.is_used()) {
    ca_.Bind(&block202, &phi_bb202_17, &phi_bb202_18, &phi_bb202_19, &phi_bb202_27, &phi_bb202_29, &phi_bb202_30, &phi_bb202_31, &phi_bb202_34, &phi_bb202_35, &phi_bb202_36, &phi_bb202_37, &phi_bb202_39, &phi_bb202_42, &phi_bb202_43, &phi_bb202_44, &phi_bb202_45);
    tmp356 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb202_17, phi_bb202_18, phi_bb202_19, tmp351, tmp356);
  }

  TNode<Object> phi_bb203_17;
  TNode<IntPtrT> phi_bb203_18;
  TNode<BoolT> phi_bb203_19;
  TNode<IntPtrT> phi_bb203_27;
  TNode<IntPtrT> phi_bb203_29;
  TNode<IntPtrT> phi_bb203_30;
  TNode<IntPtrT> phi_bb203_31;
  TNode<IntPtrT> phi_bb203_34;
  TNode<BoolT> phi_bb203_35;
  TNode<BoolT> phi_bb203_36;
  TNode<Object> phi_bb203_37;
  TNode<Int32T> phi_bb203_39;
  TNode<Object> phi_bb203_42;
  TNode<IntPtrT> phi_bb203_43;
  TNode<Object> phi_bb203_44;
  TNode<IntPtrT> phi_bb203_45;
  TNode<HeapObject> tmp357;
  if (block203.is_used()) {
    ca_.Bind(&block203, &phi_bb203_17, &phi_bb203_18, &phi_bb203_19, &phi_bb203_27, &phi_bb203_29, &phi_bb203_30, &phi_bb203_31, &phi_bb203_34, &phi_bb203_35, &phi_bb203_36, &phi_bb203_37, &phi_bb203_39, &phi_bb203_42, &phi_bb203_43, &phi_bb203_44, &phi_bb203_45);
    tmp357 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb203_17, phi_bb203_18, phi_bb203_19, tmp354, tmp357);
  }

  TNode<Object> phi_bb199_17;
  TNode<IntPtrT> phi_bb199_18;
  TNode<BoolT> phi_bb199_19;
  TNode<IntPtrT> phi_bb199_27;
  TNode<IntPtrT> phi_bb199_29;
  TNode<IntPtrT> phi_bb199_30;
  TNode<IntPtrT> phi_bb199_31;
  TNode<IntPtrT> phi_bb199_34;
  TNode<BoolT> phi_bb199_35;
  TNode<BoolT> phi_bb199_36;
  TNode<Object> phi_bb199_37;
  TNode<Int32T> phi_bb199_39;
  TNode<Object> phi_bb199_42;
  TNode<IntPtrT> phi_bb199_43;
  if (block199.is_used()) {
    ca_.Bind(&block199, &phi_bb199_17, &phi_bb199_18, &phi_bb199_19, &phi_bb199_27, &phi_bb199_29, &phi_bb199_30, &phi_bb199_31, &phi_bb199_34, &phi_bb199_35, &phi_bb199_36, &phi_bb199_37, &phi_bb199_39, &phi_bb199_42, &phi_bb199_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 691);
    ca_.Goto(&block187, phi_bb199_17, phi_bb199_18, phi_bb199_19, phi_bb199_27, tmp319, phi_bb199_29, phi_bb199_30, phi_bb199_31, phi_bb199_34, phi_bb199_35, phi_bb199_36, phi_bb199_37, phi_bb199_39);
  }

  TNode<Object> phi_bb184_17;
  TNode<IntPtrT> phi_bb184_18;
  TNode<BoolT> phi_bb184_19;
  TNode<IntPtrT> phi_bb184_27;
  TNode<IntPtrT> phi_bb184_28;
  TNode<IntPtrT> phi_bb184_29;
  TNode<IntPtrT> phi_bb184_30;
  TNode<IntPtrT> phi_bb184_31;
  TNode<IntPtrT> phi_bb184_34;
  TNode<BoolT> phi_bb184_35;
  TNode<BoolT> phi_bb184_36;
  TNode<Object> phi_bb184_37;
  TNode<Int32T> phi_bb184_39;
  TNode<Int32T> tmp358;
      TNode<Object> tmp360;
  TNode<BoolT> tmp361;
      TNode<Object> tmp363;
  if (block184.is_used()) {
    ca_.Bind(&block184, &phi_bb184_17, &phi_bb184_18, &phi_bb184_19, &phi_bb184_27, &phi_bb184_28, &phi_bb184_29, &phi_bb184_30, &phi_bb184_31, &phi_bb184_34, &phi_bb184_35, &phi_bb184_36, &phi_bb184_37, &phi_bb184_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 698);
    compiler::CodeAssemblerExceptionHandlerLabel catch359__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch359__label);
    tmp358 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmF64.raw_bit_field());
    }
    if (catch359__label.is_used()) {
      compiler::CodeAssemblerLabel catch359_skip(&ca_);
      ca_.Goto(&catch359_skip);
      ca_.Bind(&catch359__label, &tmp360);
      ca_.Goto(&block206, phi_bb184_17, phi_bb184_18, phi_bb184_19, phi_bb184_27, phi_bb184_28, phi_bb184_29, phi_bb184_30, phi_bb184_31, phi_bb184_34, phi_bb184_35, phi_bb184_36, phi_bb184_37, phi_bb184_39);
      ca_.Bind(&catch359_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch362__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch362__label);
    tmp361 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp250}, TNode<Int32T>{tmp358});
    }
    if (catch362__label.is_used()) {
      compiler::CodeAssemblerLabel catch362_skip(&ca_);
      ca_.Goto(&catch362_skip);
      ca_.Bind(&catch362__label, &tmp363);
      ca_.Goto(&block207, phi_bb184_17, phi_bb184_18, phi_bb184_19, phi_bb184_27, phi_bb184_28, phi_bb184_29, phi_bb184_30, phi_bb184_31, phi_bb184_34, phi_bb184_35, phi_bb184_36, phi_bb184_37, phi_bb184_39);
      ca_.Bind(&catch362_skip);
    }
    ca_.Branch(tmp361, &block204, std::vector<compiler::Node*>{phi_bb184_17, phi_bb184_18, phi_bb184_19, phi_bb184_27, phi_bb184_28, phi_bb184_29, phi_bb184_30, phi_bb184_31, phi_bb184_34, phi_bb184_35, phi_bb184_36, phi_bb184_37, phi_bb184_39}, &block205, std::vector<compiler::Node*>{phi_bb184_17, phi_bb184_18, phi_bb184_19, phi_bb184_27, phi_bb184_28, phi_bb184_29, phi_bb184_30, phi_bb184_31, phi_bb184_34, phi_bb184_35, phi_bb184_36, phi_bb184_37, phi_bb184_39});
  }

  TNode<Object> phi_bb206_17;
  TNode<IntPtrT> phi_bb206_18;
  TNode<BoolT> phi_bb206_19;
  TNode<IntPtrT> phi_bb206_27;
  TNode<IntPtrT> phi_bb206_28;
  TNode<IntPtrT> phi_bb206_29;
  TNode<IntPtrT> phi_bb206_30;
  TNode<IntPtrT> phi_bb206_31;
  TNode<IntPtrT> phi_bb206_34;
  TNode<BoolT> phi_bb206_35;
  TNode<BoolT> phi_bb206_36;
  TNode<Object> phi_bb206_37;
  TNode<Int32T> phi_bb206_39;
  TNode<HeapObject> tmp364;
  if (block206.is_used()) {
    ca_.Bind(&block206, &phi_bb206_17, &phi_bb206_18, &phi_bb206_19, &phi_bb206_27, &phi_bb206_28, &phi_bb206_29, &phi_bb206_30, &phi_bb206_31, &phi_bb206_34, &phi_bb206_35, &phi_bb206_36, &phi_bb206_37, &phi_bb206_39);
    tmp364 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb206_17, phi_bb206_18, phi_bb206_19, tmp360, tmp364);
  }

  TNode<Object> phi_bb207_17;
  TNode<IntPtrT> phi_bb207_18;
  TNode<BoolT> phi_bb207_19;
  TNode<IntPtrT> phi_bb207_27;
  TNode<IntPtrT> phi_bb207_28;
  TNode<IntPtrT> phi_bb207_29;
  TNode<IntPtrT> phi_bb207_30;
  TNode<IntPtrT> phi_bb207_31;
  TNode<IntPtrT> phi_bb207_34;
  TNode<BoolT> phi_bb207_35;
  TNode<BoolT> phi_bb207_36;
  TNode<Object> phi_bb207_37;
  TNode<Int32T> phi_bb207_39;
  TNode<HeapObject> tmp365;
  if (block207.is_used()) {
    ca_.Bind(&block207, &phi_bb207_17, &phi_bb207_18, &phi_bb207_19, &phi_bb207_27, &phi_bb207_28, &phi_bb207_29, &phi_bb207_30, &phi_bb207_31, &phi_bb207_34, &phi_bb207_35, &phi_bb207_36, &phi_bb207_37, &phi_bb207_39);
    tmp365 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb207_17, phi_bb207_18, phi_bb207_19, tmp363, tmp365);
  }

  TNode<Object> phi_bb204_17;
  TNode<IntPtrT> phi_bb204_18;
  TNode<BoolT> phi_bb204_19;
  TNode<IntPtrT> phi_bb204_27;
  TNode<IntPtrT> phi_bb204_28;
  TNode<IntPtrT> phi_bb204_29;
  TNode<IntPtrT> phi_bb204_30;
  TNode<IntPtrT> phi_bb204_31;
  TNode<IntPtrT> phi_bb204_34;
  TNode<BoolT> phi_bb204_35;
  TNode<BoolT> phi_bb204_36;
  TNode<Object> phi_bb204_37;
  TNode<Int32T> phi_bb204_39;
  TNode<IntPtrT> tmp366;
  TNode<IntPtrT> tmp367;
  TNode<IntPtrT> tmp368;
  TNode<BoolT> tmp369;
  if (block204.is_used()) {
    ca_.Bind(&block204, &phi_bb204_17, &phi_bb204_18, &phi_bb204_19, &phi_bb204_27, &phi_bb204_28, &phi_bb204_29, &phi_bb204_30, &phi_bb204_31, &phi_bb204_34, &phi_bb204_35, &phi_bb204_36, &phi_bb204_37, &phi_bb204_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 332);
    tmp366 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp367 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb204_28}, TNode<IntPtrT>{tmp366});
    tmp368 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp369 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb204_28}, TNode<IntPtrT>{tmp368});
    ca_.Branch(tmp369, &block210, std::vector<compiler::Node*>{phi_bb204_17, phi_bb204_18, phi_bb204_19, phi_bb204_27, phi_bb204_29, phi_bb204_30, phi_bb204_31, phi_bb204_34, phi_bb204_35, phi_bb204_36, phi_bb204_37, phi_bb204_39}, &block211, std::vector<compiler::Node*>{phi_bb204_17, phi_bb204_18, phi_bb204_19, phi_bb204_27, phi_bb204_29, phi_bb204_30, phi_bb204_31, phi_bb204_34, phi_bb204_35, phi_bb204_36, phi_bb204_37, phi_bb204_39});
  }

  TNode<Object> phi_bb210_17;
  TNode<IntPtrT> phi_bb210_18;
  TNode<BoolT> phi_bb210_19;
  TNode<IntPtrT> phi_bb210_27;
  TNode<IntPtrT> phi_bb210_29;
  TNode<IntPtrT> phi_bb210_30;
  TNode<IntPtrT> phi_bb210_31;
  TNode<IntPtrT> phi_bb210_34;
  TNode<BoolT> phi_bb210_35;
  TNode<BoolT> phi_bb210_36;
  TNode<Object> phi_bb210_37;
  TNode<Int32T> phi_bb210_39;
  TNode<Object> tmp370;
  TNode<IntPtrT> tmp371;
  TNode<IntPtrT> tmp372;
  TNode<IntPtrT> tmp373;
  if (block210.is_used()) {
    ca_.Bind(&block210, &phi_bb210_17, &phi_bb210_18, &phi_bb210_19, &phi_bb210_27, &phi_bb210_29, &phi_bb210_30, &phi_bb210_31, &phi_bb210_34, &phi_bb210_35, &phi_bb210_36, &phi_bb210_37, &phi_bb210_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 333);
    std::tie(tmp370, tmp371) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb210_30}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 335);
    tmp372 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_float64_0(state_)));
    tmp373 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb210_30}, TNode<IntPtrT>{tmp372});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 336);
    ca_.Goto(&block209, phi_bb210_17, phi_bb210_18, phi_bb210_19, phi_bb210_27, phi_bb210_29, tmp373, phi_bb210_31, phi_bb210_34, phi_bb210_35, phi_bb210_36, phi_bb210_37, phi_bb210_39, tmp370, tmp371);
  }

  TNode<Object> phi_bb211_17;
  TNode<IntPtrT> phi_bb211_18;
  TNode<BoolT> phi_bb211_19;
  TNode<IntPtrT> phi_bb211_27;
  TNode<IntPtrT> phi_bb211_29;
  TNode<IntPtrT> phi_bb211_30;
  TNode<IntPtrT> phi_bb211_31;
  TNode<IntPtrT> phi_bb211_34;
  TNode<BoolT> phi_bb211_35;
  TNode<BoolT> phi_bb211_36;
  TNode<Object> phi_bb211_37;
  TNode<Int32T> phi_bb211_39;
  if (block211.is_used()) {
    ca_.Bind(&block211, &phi_bb211_17, &phi_bb211_18, &phi_bb211_19, &phi_bb211_27, &phi_bb211_29, &phi_bb211_30, &phi_bb211_31, &phi_bb211_34, &phi_bb211_35, &phi_bb211_36, &phi_bb211_37, &phi_bb211_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 338);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block212, phi_bb211_17, phi_bb211_18, phi_bb211_19, phi_bb211_27, phi_bb211_29, phi_bb211_30, phi_bb211_31, phi_bb211_34, phi_bb211_35, phi_bb211_36, phi_bb211_37, phi_bb211_39);
    } else {
      ca_.Goto(&block213, phi_bb211_17, phi_bb211_18, phi_bb211_19, phi_bb211_27, phi_bb211_29, phi_bb211_30, phi_bb211_31, phi_bb211_34, phi_bb211_35, phi_bb211_36, phi_bb211_37, phi_bb211_39);
    }
  }

  TNode<Object> phi_bb212_17;
  TNode<IntPtrT> phi_bb212_18;
  TNode<BoolT> phi_bb212_19;
  TNode<IntPtrT> phi_bb212_27;
  TNode<IntPtrT> phi_bb212_29;
  TNode<IntPtrT> phi_bb212_30;
  TNode<IntPtrT> phi_bb212_31;
  TNode<IntPtrT> phi_bb212_34;
  TNode<BoolT> phi_bb212_35;
  TNode<BoolT> phi_bb212_36;
  TNode<Object> phi_bb212_37;
  TNode<Int32T> phi_bb212_39;
  if (block212.is_used()) {
    ca_.Bind(&block212, &phi_bb212_17, &phi_bb212_18, &phi_bb212_19, &phi_bb212_27, &phi_bb212_29, &phi_bb212_30, &phi_bb212_31, &phi_bb212_34, &phi_bb212_35, &phi_bb212_36, &phi_bb212_37, &phi_bb212_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block216, phi_bb212_17, phi_bb212_18, phi_bb212_19, phi_bb212_27, phi_bb212_29, phi_bb212_30, phi_bb212_31, phi_bb212_34, phi_bb212_35, phi_bb212_36, phi_bb212_37, phi_bb212_39);
    } else {
      ca_.Goto(&block217, phi_bb212_17, phi_bb212_18, phi_bb212_19, phi_bb212_27, phi_bb212_29, phi_bb212_30, phi_bb212_31, phi_bb212_34, phi_bb212_35, phi_bb212_36, phi_bb212_37, phi_bb212_39);
    }
  }

  TNode<Object> phi_bb216_17;
  TNode<IntPtrT> phi_bb216_18;
  TNode<BoolT> phi_bb216_19;
  TNode<IntPtrT> phi_bb216_27;
  TNode<IntPtrT> phi_bb216_29;
  TNode<IntPtrT> phi_bb216_30;
  TNode<IntPtrT> phi_bb216_31;
  TNode<IntPtrT> phi_bb216_34;
  TNode<BoolT> phi_bb216_35;
  TNode<BoolT> phi_bb216_36;
  TNode<Object> phi_bb216_37;
  TNode<Int32T> phi_bb216_39;
  TNode<Object> tmp374;
  TNode<IntPtrT> tmp375;
  TNode<IntPtrT> tmp376;
  TNode<IntPtrT> tmp377;
  if (block216.is_used()) {
    ca_.Bind(&block216, &phi_bb216_17, &phi_bb216_18, &phi_bb216_19, &phi_bb216_27, &phi_bb216_29, &phi_bb216_30, &phi_bb216_31, &phi_bb216_34, &phi_bb216_35, &phi_bb216_36, &phi_bb216_37, &phi_bb216_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp374, tmp375) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb216_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp376 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp377 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb216_31}, TNode<IntPtrT>{tmp376});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block215, phi_bb216_17, phi_bb216_18, phi_bb216_19, phi_bb216_27, phi_bb216_29, phi_bb216_30, tmp377, phi_bb216_34, phi_bb216_35, phi_bb216_36, phi_bb216_37, phi_bb216_39, tmp374, tmp375);
  }

  TNode<Object> phi_bb217_17;
  TNode<IntPtrT> phi_bb217_18;
  TNode<BoolT> phi_bb217_19;
  TNode<IntPtrT> phi_bb217_27;
  TNode<IntPtrT> phi_bb217_29;
  TNode<IntPtrT> phi_bb217_30;
  TNode<IntPtrT> phi_bb217_31;
  TNode<IntPtrT> phi_bb217_34;
  TNode<BoolT> phi_bb217_35;
  TNode<BoolT> phi_bb217_36;
  TNode<Object> phi_bb217_37;
  TNode<Int32T> phi_bb217_39;
  TNode<IntPtrT> tmp378;
  TNode<BoolT> tmp379;
  if (block217.is_used()) {
    ca_.Bind(&block217, &phi_bb217_17, &phi_bb217_18, &phi_bb217_19, &phi_bb217_27, &phi_bb217_29, &phi_bb217_30, &phi_bb217_31, &phi_bb217_34, &phi_bb217_35, &phi_bb217_36, &phi_bb217_37, &phi_bb217_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp378 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp379 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb217_34}, TNode<IntPtrT>{tmp378});
    ca_.Branch(tmp379, &block219, std::vector<compiler::Node*>{phi_bb217_17, phi_bb217_18, phi_bb217_19, phi_bb217_27, phi_bb217_29, phi_bb217_30, phi_bb217_31, phi_bb217_34, phi_bb217_35, phi_bb217_36, phi_bb217_37, phi_bb217_39}, &block220, std::vector<compiler::Node*>{phi_bb217_17, phi_bb217_18, phi_bb217_19, phi_bb217_27, phi_bb217_29, phi_bb217_30, phi_bb217_31, phi_bb217_34, phi_bb217_35, phi_bb217_36, phi_bb217_37, phi_bb217_39});
  }

  TNode<Object> phi_bb219_17;
  TNode<IntPtrT> phi_bb219_18;
  TNode<BoolT> phi_bb219_19;
  TNode<IntPtrT> phi_bb219_27;
  TNode<IntPtrT> phi_bb219_29;
  TNode<IntPtrT> phi_bb219_30;
  TNode<IntPtrT> phi_bb219_31;
  TNode<IntPtrT> phi_bb219_34;
  TNode<BoolT> phi_bb219_35;
  TNode<BoolT> phi_bb219_36;
  TNode<Object> phi_bb219_37;
  TNode<Int32T> phi_bb219_39;
  TNode<Object> tmp380;
  TNode<IntPtrT> tmp381;
  TNode<IntPtrT> tmp382;
  TNode<BoolT> tmp383;
  if (block219.is_used()) {
    ca_.Bind(&block219, &phi_bb219_17, &phi_bb219_18, &phi_bb219_19, &phi_bb219_27, &phi_bb219_29, &phi_bb219_30, &phi_bb219_31, &phi_bb219_34, &phi_bb219_35, &phi_bb219_36, &phi_bb219_37, &phi_bb219_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp380, tmp381) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb219_34}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp382 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp383 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block215, phi_bb219_17, phi_bb219_18, phi_bb219_19, phi_bb219_27, phi_bb219_29, phi_bb219_30, phi_bb219_31, tmp382, tmp383, phi_bb219_36, phi_bb219_37, phi_bb219_39, tmp380, tmp381);
  }

  TNode<Object> phi_bb220_17;
  TNode<IntPtrT> phi_bb220_18;
  TNode<BoolT> phi_bb220_19;
  TNode<IntPtrT> phi_bb220_27;
  TNode<IntPtrT> phi_bb220_29;
  TNode<IntPtrT> phi_bb220_30;
  TNode<IntPtrT> phi_bb220_31;
  TNode<IntPtrT> phi_bb220_34;
  TNode<BoolT> phi_bb220_35;
  TNode<BoolT> phi_bb220_36;
  TNode<Object> phi_bb220_37;
  TNode<Int32T> phi_bb220_39;
  TNode<Object> tmp384;
  TNode<IntPtrT> tmp385;
  TNode<IntPtrT> tmp386;
  TNode<IntPtrT> tmp387;
  TNode<IntPtrT> tmp388;
  TNode<IntPtrT> tmp389;
  TNode<BoolT> tmp390;
  if (block220.is_used()) {
    ca_.Bind(&block220, &phi_bb220_17, &phi_bb220_18, &phi_bb220_19, &phi_bb220_27, &phi_bb220_29, &phi_bb220_30, &phi_bb220_31, &phi_bb220_34, &phi_bb220_35, &phi_bb220_36, &phi_bb220_37, &phi_bb220_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp384, tmp385) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb220_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp386 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp387 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb220_31}, TNode<IntPtrT>{tmp386});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp388 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp389 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp387}, TNode<IntPtrT>{tmp388});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp390 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block215, phi_bb220_17, phi_bb220_18, phi_bb220_19, phi_bb220_27, phi_bb220_29, phi_bb220_30, tmp389, tmp387, tmp390, phi_bb220_36, phi_bb220_37, phi_bb220_39, tmp384, tmp385);
  }

  TNode<Object> phi_bb215_17;
  TNode<IntPtrT> phi_bb215_18;
  TNode<BoolT> phi_bb215_19;
  TNode<IntPtrT> phi_bb215_27;
  TNode<IntPtrT> phi_bb215_29;
  TNode<IntPtrT> phi_bb215_30;
  TNode<IntPtrT> phi_bb215_31;
  TNode<IntPtrT> phi_bb215_34;
  TNode<BoolT> phi_bb215_35;
  TNode<BoolT> phi_bb215_36;
  TNode<Object> phi_bb215_37;
  TNode<Int32T> phi_bb215_39;
  TNode<Object> phi_bb215_42;
  TNode<IntPtrT> phi_bb215_43;
  if (block215.is_used()) {
    ca_.Bind(&block215, &phi_bb215_17, &phi_bb215_18, &phi_bb215_19, &phi_bb215_27, &phi_bb215_29, &phi_bb215_30, &phi_bb215_31, &phi_bb215_34, &phi_bb215_35, &phi_bb215_36, &phi_bb215_37, &phi_bb215_39, &phi_bb215_42, &phi_bb215_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 339);
    ca_.Goto(&block209, phi_bb215_17, phi_bb215_18, phi_bb215_19, phi_bb215_27, phi_bb215_29, phi_bb215_30, phi_bb215_31, phi_bb215_34, phi_bb215_35, phi_bb215_36, phi_bb215_37, phi_bb215_39, phi_bb215_42, phi_bb215_43);
  }

  TNode<Object> phi_bb213_17;
  TNode<IntPtrT> phi_bb213_18;
  TNode<BoolT> phi_bb213_19;
  TNode<IntPtrT> phi_bb213_27;
  TNode<IntPtrT> phi_bb213_29;
  TNode<IntPtrT> phi_bb213_30;
  TNode<IntPtrT> phi_bb213_31;
  TNode<IntPtrT> phi_bb213_34;
  TNode<BoolT> phi_bb213_35;
  TNode<BoolT> phi_bb213_36;
  TNode<Object> phi_bb213_37;
  TNode<Int32T> phi_bb213_39;
  TNode<Object> tmp391;
  TNode<IntPtrT> tmp392;
  TNode<IntPtrT> tmp393;
  TNode<IntPtrT> tmp394;
  TNode<BoolT> tmp395;
  if (block213.is_used()) {
    ca_.Bind(&block213, &phi_bb213_17, &phi_bb213_18, &phi_bb213_19, &phi_bb213_27, &phi_bb213_29, &phi_bb213_30, &phi_bb213_31, &phi_bb213_34, &phi_bb213_35, &phi_bb213_36, &phi_bb213_37, &phi_bb213_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 341);
    std::tie(tmp391, tmp392) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb213_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 343);
    tmp393 = FromConstexpr_intptr_constexpr_int31_0(state_, (CodeStubAssembler(state_).ConstexprInt31Mul((FromConstexpr_constexpr_int31_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x2ull))), (SizeOf_intptr_0(state_)))));
    tmp394 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb213_31}, TNode<IntPtrT>{tmp393});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 344);
    tmp395 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 345);
    ca_.Goto(&block209, phi_bb213_17, phi_bb213_18, phi_bb213_19, phi_bb213_27, phi_bb213_29, phi_bb213_30, tmp394, phi_bb213_34, tmp395, phi_bb213_36, phi_bb213_37, phi_bb213_39, tmp391, tmp392);
  }

  TNode<Object> phi_bb209_17;
  TNode<IntPtrT> phi_bb209_18;
  TNode<BoolT> phi_bb209_19;
  TNode<IntPtrT> phi_bb209_27;
  TNode<IntPtrT> phi_bb209_29;
  TNode<IntPtrT> phi_bb209_30;
  TNode<IntPtrT> phi_bb209_31;
  TNode<IntPtrT> phi_bb209_34;
  TNode<BoolT> phi_bb209_35;
  TNode<BoolT> phi_bb209_36;
  TNode<Object> phi_bb209_37;
  TNode<Int32T> phi_bb209_39;
  TNode<Object> phi_bb209_42;
  TNode<IntPtrT> phi_bb209_43;
  TNode<Object> tmp396;
  TNode<IntPtrT> tmp397;
      TNode<Object> tmp399;
  TNode<Float64T> tmp400;
      TNode<Object> tmp402;
  TNode<Float64T> tmp403;
      TNode<Object> tmp405;
  if (block209.is_used()) {
    ca_.Bind(&block209, &phi_bb209_17, &phi_bb209_18, &phi_bb209_19, &phi_bb209_27, &phi_bb209_29, &phi_bb209_30, &phi_bb209_31, &phi_bb209_34, &phi_bb209_35, &phi_bb209_36, &phi_bb209_37, &phi_bb209_39, &phi_bb209_42, &phi_bb209_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 700);
    compiler::CodeAssemblerExceptionHandlerLabel catch398__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch398__label);
    std::tie(tmp396, tmp397) = RefCast_float64_0(state_, TorqueStructReference_intptr_0{TNode<Object>{phi_bb209_42}, TNode<IntPtrT>{phi_bb209_43}, TorqueStructUnsafe_0{}}).Flatten();
    }
    if (catch398__label.is_used()) {
      compiler::CodeAssemblerLabel catch398_skip(&ca_);
      ca_.Goto(&catch398_skip);
      ca_.Bind(&catch398__label, &tmp399);
      ca_.Goto(&block221, phi_bb209_17, phi_bb209_18, phi_bb209_19, phi_bb209_27, phi_bb209_29, phi_bb209_30, phi_bb209_31, phi_bb209_34, phi_bb209_35, phi_bb209_36, phi_bb209_37, phi_bb209_39, phi_bb209_42, phi_bb209_43, phi_bb209_42, phi_bb209_43);
      ca_.Bind(&catch398_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch401__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch401__label);
    tmp400 = CodeStubAssembler(state_).ChangeTaggedToFloat64(TNode<Context>{p_context}, TNode<Object>{tmp234});
    }
    if (catch401__label.is_used()) {
      compiler::CodeAssemblerLabel catch401_skip(&ca_);
      ca_.Goto(&catch401_skip);
      ca_.Bind(&catch401__label, &tmp402);
      ca_.Goto(&block222, phi_bb209_17, phi_bb209_18, phi_bb209_19, phi_bb209_27, phi_bb209_29, phi_bb209_30, phi_bb209_31, phi_bb209_34, phi_bb209_35, phi_bb209_36, phi_bb209_37, phi_bb209_39, phi_bb209_42, phi_bb209_43);
      ca_.Bind(&catch401_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch404__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch404__label);
    tmp403 = CodeStubAssembler(state_).Float64SilenceNaN(TNode<Float64T>{tmp400});
    }
    if (catch404__label.is_used()) {
      compiler::CodeAssemblerLabel catch404_skip(&ca_);
      ca_.Goto(&catch404_skip);
      ca_.Bind(&catch404__label, &tmp405);
      ca_.Goto(&block223, phi_bb209_17, phi_bb209_18, phi_bb209_19, phi_bb209_27, phi_bb209_29, phi_bb209_30, phi_bb209_31, phi_bb209_34, phi_bb209_35, phi_bb209_36, phi_bb209_37, phi_bb209_39, phi_bb209_42, phi_bb209_43);
      ca_.Bind(&catch404_skip);
    }
    CodeStubAssembler(state_).StoreReference<Float64T>(CodeStubAssembler::Reference{tmp396, tmp397}, tmp403);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 698);
    ca_.Goto(&block208, phi_bb209_17, phi_bb209_18, phi_bb209_19, phi_bb209_27, tmp367, phi_bb209_29, phi_bb209_30, phi_bb209_31, phi_bb209_34, phi_bb209_35, phi_bb209_36, phi_bb209_37, phi_bb209_39);
  }

  TNode<Object> phi_bb221_17;
  TNode<IntPtrT> phi_bb221_18;
  TNode<BoolT> phi_bb221_19;
  TNode<IntPtrT> phi_bb221_27;
  TNode<IntPtrT> phi_bb221_29;
  TNode<IntPtrT> phi_bb221_30;
  TNode<IntPtrT> phi_bb221_31;
  TNode<IntPtrT> phi_bb221_34;
  TNode<BoolT> phi_bb221_35;
  TNode<BoolT> phi_bb221_36;
  TNode<Object> phi_bb221_37;
  TNode<Int32T> phi_bb221_39;
  TNode<Object> phi_bb221_42;
  TNode<IntPtrT> phi_bb221_43;
  TNode<Object> phi_bb221_44;
  TNode<IntPtrT> phi_bb221_45;
  TNode<HeapObject> tmp406;
  if (block221.is_used()) {
    ca_.Bind(&block221, &phi_bb221_17, &phi_bb221_18, &phi_bb221_19, &phi_bb221_27, &phi_bb221_29, &phi_bb221_30, &phi_bb221_31, &phi_bb221_34, &phi_bb221_35, &phi_bb221_36, &phi_bb221_37, &phi_bb221_39, &phi_bb221_42, &phi_bb221_43, &phi_bb221_44, &phi_bb221_45);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 700);
    tmp406 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb221_17, phi_bb221_18, phi_bb221_19, tmp399, tmp406);
  }

  TNode<Object> phi_bb222_17;
  TNode<IntPtrT> phi_bb222_18;
  TNode<BoolT> phi_bb222_19;
  TNode<IntPtrT> phi_bb222_27;
  TNode<IntPtrT> phi_bb222_29;
  TNode<IntPtrT> phi_bb222_30;
  TNode<IntPtrT> phi_bb222_31;
  TNode<IntPtrT> phi_bb222_34;
  TNode<BoolT> phi_bb222_35;
  TNode<BoolT> phi_bb222_36;
  TNode<Object> phi_bb222_37;
  TNode<Int32T> phi_bb222_39;
  TNode<Object> phi_bb222_42;
  TNode<IntPtrT> phi_bb222_43;
  TNode<HeapObject> tmp407;
  if (block222.is_used()) {
    ca_.Bind(&block222, &phi_bb222_17, &phi_bb222_18, &phi_bb222_19, &phi_bb222_27, &phi_bb222_29, &phi_bb222_30, &phi_bb222_31, &phi_bb222_34, &phi_bb222_35, &phi_bb222_36, &phi_bb222_37, &phi_bb222_39, &phi_bb222_42, &phi_bb222_43);
    tmp407 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb222_17, phi_bb222_18, phi_bb222_19, tmp402, tmp407);
  }

  TNode<Object> phi_bb223_17;
  TNode<IntPtrT> phi_bb223_18;
  TNode<BoolT> phi_bb223_19;
  TNode<IntPtrT> phi_bb223_27;
  TNode<IntPtrT> phi_bb223_29;
  TNode<IntPtrT> phi_bb223_30;
  TNode<IntPtrT> phi_bb223_31;
  TNode<IntPtrT> phi_bb223_34;
  TNode<BoolT> phi_bb223_35;
  TNode<BoolT> phi_bb223_36;
  TNode<Object> phi_bb223_37;
  TNode<Int32T> phi_bb223_39;
  TNode<Object> phi_bb223_42;
  TNode<IntPtrT> phi_bb223_43;
  TNode<HeapObject> tmp408;
  if (block223.is_used()) {
    ca_.Bind(&block223, &phi_bb223_17, &phi_bb223_18, &phi_bb223_19, &phi_bb223_27, &phi_bb223_29, &phi_bb223_30, &phi_bb223_31, &phi_bb223_34, &phi_bb223_35, &phi_bb223_36, &phi_bb223_37, &phi_bb223_39, &phi_bb223_42, &phi_bb223_43);
    tmp408 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb223_17, phi_bb223_18, phi_bb223_19, tmp405, tmp408);
  }

  TNode<Object> phi_bb205_17;
  TNode<IntPtrT> phi_bb205_18;
  TNode<BoolT> phi_bb205_19;
  TNode<IntPtrT> phi_bb205_27;
  TNode<IntPtrT> phi_bb205_28;
  TNode<IntPtrT> phi_bb205_29;
  TNode<IntPtrT> phi_bb205_30;
  TNode<IntPtrT> phi_bb205_31;
  TNode<IntPtrT> phi_bb205_34;
  TNode<BoolT> phi_bb205_35;
  TNode<BoolT> phi_bb205_36;
  TNode<Object> phi_bb205_37;
  TNode<Int32T> phi_bb205_39;
  TNode<Int32T> tmp409;
      TNode<Object> tmp411;
  TNode<BoolT> tmp412;
      TNode<Object> tmp414;
  if (block205.is_used()) {
    ca_.Bind(&block205, &phi_bb205_17, &phi_bb205_18, &phi_bb205_19, &phi_bb205_27, &phi_bb205_28, &phi_bb205_29, &phi_bb205_30, &phi_bb205_31, &phi_bb205_34, &phi_bb205_35, &phi_bb205_36, &phi_bb205_37, &phi_bb205_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 701);
    compiler::CodeAssemblerExceptionHandlerLabel catch410__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch410__label);
    tmp409 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmI64.raw_bit_field());
    }
    if (catch410__label.is_used()) {
      compiler::CodeAssemblerLabel catch410_skip(&ca_);
      ca_.Goto(&catch410_skip);
      ca_.Bind(&catch410__label, &tmp411);
      ca_.Goto(&block226, phi_bb205_17, phi_bb205_18, phi_bb205_19, phi_bb205_27, phi_bb205_28, phi_bb205_29, phi_bb205_30, phi_bb205_31, phi_bb205_34, phi_bb205_35, phi_bb205_36, phi_bb205_37, phi_bb205_39);
      ca_.Bind(&catch410_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch413__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch413__label);
    tmp412 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp250}, TNode<Int32T>{tmp409});
    }
    if (catch413__label.is_used()) {
      compiler::CodeAssemblerLabel catch413_skip(&ca_);
      ca_.Goto(&catch413_skip);
      ca_.Bind(&catch413__label, &tmp414);
      ca_.Goto(&block227, phi_bb205_17, phi_bb205_18, phi_bb205_19, phi_bb205_27, phi_bb205_28, phi_bb205_29, phi_bb205_30, phi_bb205_31, phi_bb205_34, phi_bb205_35, phi_bb205_36, phi_bb205_37, phi_bb205_39);
      ca_.Bind(&catch413_skip);
    }
    ca_.Branch(tmp412, &block224, std::vector<compiler::Node*>{phi_bb205_17, phi_bb205_18, phi_bb205_19, phi_bb205_27, phi_bb205_28, phi_bb205_29, phi_bb205_30, phi_bb205_31, phi_bb205_34, phi_bb205_35, phi_bb205_36, phi_bb205_37, phi_bb205_39}, &block225, std::vector<compiler::Node*>{phi_bb205_17, phi_bb205_18, phi_bb205_19, phi_bb205_27, phi_bb205_28, phi_bb205_29, phi_bb205_30, phi_bb205_31, phi_bb205_34, phi_bb205_35, phi_bb205_36, phi_bb205_37, phi_bb205_39});
  }

  TNode<Object> phi_bb226_17;
  TNode<IntPtrT> phi_bb226_18;
  TNode<BoolT> phi_bb226_19;
  TNode<IntPtrT> phi_bb226_27;
  TNode<IntPtrT> phi_bb226_28;
  TNode<IntPtrT> phi_bb226_29;
  TNode<IntPtrT> phi_bb226_30;
  TNode<IntPtrT> phi_bb226_31;
  TNode<IntPtrT> phi_bb226_34;
  TNode<BoolT> phi_bb226_35;
  TNode<BoolT> phi_bb226_36;
  TNode<Object> phi_bb226_37;
  TNode<Int32T> phi_bb226_39;
  TNode<HeapObject> tmp415;
  if (block226.is_used()) {
    ca_.Bind(&block226, &phi_bb226_17, &phi_bb226_18, &phi_bb226_19, &phi_bb226_27, &phi_bb226_28, &phi_bb226_29, &phi_bb226_30, &phi_bb226_31, &phi_bb226_34, &phi_bb226_35, &phi_bb226_36, &phi_bb226_37, &phi_bb226_39);
    tmp415 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb226_17, phi_bb226_18, phi_bb226_19, tmp411, tmp415);
  }

  TNode<Object> phi_bb227_17;
  TNode<IntPtrT> phi_bb227_18;
  TNode<BoolT> phi_bb227_19;
  TNode<IntPtrT> phi_bb227_27;
  TNode<IntPtrT> phi_bb227_28;
  TNode<IntPtrT> phi_bb227_29;
  TNode<IntPtrT> phi_bb227_30;
  TNode<IntPtrT> phi_bb227_31;
  TNode<IntPtrT> phi_bb227_34;
  TNode<BoolT> phi_bb227_35;
  TNode<BoolT> phi_bb227_36;
  TNode<Object> phi_bb227_37;
  TNode<Int32T> phi_bb227_39;
  TNode<HeapObject> tmp416;
  if (block227.is_used()) {
    ca_.Bind(&block227, &phi_bb227_17, &phi_bb227_18, &phi_bb227_19, &phi_bb227_27, &phi_bb227_28, &phi_bb227_29, &phi_bb227_30, &phi_bb227_31, &phi_bb227_34, &phi_bb227_35, &phi_bb227_36, &phi_bb227_37, &phi_bb227_39);
    tmp416 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb227_17, phi_bb227_18, phi_bb227_19, tmp414, tmp416);
  }

  TNode<Object> phi_bb224_17;
  TNode<IntPtrT> phi_bb224_18;
  TNode<BoolT> phi_bb224_19;
  TNode<IntPtrT> phi_bb224_27;
  TNode<IntPtrT> phi_bb224_28;
  TNode<IntPtrT> phi_bb224_29;
  TNode<IntPtrT> phi_bb224_30;
  TNode<IntPtrT> phi_bb224_31;
  TNode<IntPtrT> phi_bb224_34;
  TNode<BoolT> phi_bb224_35;
  TNode<BoolT> phi_bb224_36;
  TNode<Object> phi_bb224_37;
  TNode<Int32T> phi_bb224_39;
  if (block224.is_used()) {
    ca_.Bind(&block224, &phi_bb224_17, &phi_bb224_18, &phi_bb224_19, &phi_bb224_27, &phi_bb224_28, &phi_bb224_29, &phi_bb224_30, &phi_bb224_31, &phi_bb224_34, &phi_bb224_35, &phi_bb224_36, &phi_bb224_37, &phi_bb224_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 702);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block229, phi_bb224_17, phi_bb224_18, phi_bb224_19, phi_bb224_27, phi_bb224_28, phi_bb224_29, phi_bb224_30, phi_bb224_31, phi_bb224_34, phi_bb224_35, phi_bb224_36, phi_bb224_37, phi_bb224_39);
    } else {
      ca_.Goto(&block230, phi_bb224_17, phi_bb224_18, phi_bb224_19, phi_bb224_27, phi_bb224_28, phi_bb224_29, phi_bb224_30, phi_bb224_31, phi_bb224_34, phi_bb224_35, phi_bb224_36, phi_bb224_37, phi_bb224_39);
    }
  }

  TNode<Object> phi_bb229_17;
  TNode<IntPtrT> phi_bb229_18;
  TNode<BoolT> phi_bb229_19;
  TNode<IntPtrT> phi_bb229_27;
  TNode<IntPtrT> phi_bb229_28;
  TNode<IntPtrT> phi_bb229_29;
  TNode<IntPtrT> phi_bb229_30;
  TNode<IntPtrT> phi_bb229_31;
  TNode<IntPtrT> phi_bb229_34;
  TNode<BoolT> phi_bb229_35;
  TNode<BoolT> phi_bb229_36;
  TNode<Object> phi_bb229_37;
  TNode<Int32T> phi_bb229_39;
  TNode<IntPtrT> tmp417;
  TNode<IntPtrT> tmp418;
  TNode<IntPtrT> tmp419;
  TNode<BoolT> tmp420;
  if (block229.is_used()) {
    ca_.Bind(&block229, &phi_bb229_17, &phi_bb229_18, &phi_bb229_19, &phi_bb229_27, &phi_bb229_28, &phi_bb229_29, &phi_bb229_30, &phi_bb229_31, &phi_bb229_34, &phi_bb229_35, &phi_bb229_36, &phi_bb229_37, &phi_bb229_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 308);
    tmp417 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp418 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb229_27}, TNode<IntPtrT>{tmp417});
    tmp419 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp420 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb229_27}, TNode<IntPtrT>{tmp419});
    ca_.Branch(tmp420, &block233, std::vector<compiler::Node*>{phi_bb229_17, phi_bb229_18, phi_bb229_19, phi_bb229_28, phi_bb229_29, phi_bb229_30, phi_bb229_31, phi_bb229_34, phi_bb229_35, phi_bb229_36, phi_bb229_37, phi_bb229_39}, &block234, std::vector<compiler::Node*>{phi_bb229_17, phi_bb229_18, phi_bb229_19, phi_bb229_28, phi_bb229_29, phi_bb229_30, phi_bb229_31, phi_bb229_34, phi_bb229_35, phi_bb229_36, phi_bb229_37, phi_bb229_39});
  }

  TNode<Object> phi_bb233_17;
  TNode<IntPtrT> phi_bb233_18;
  TNode<BoolT> phi_bb233_19;
  TNode<IntPtrT> phi_bb233_28;
  TNode<IntPtrT> phi_bb233_29;
  TNode<IntPtrT> phi_bb233_30;
  TNode<IntPtrT> phi_bb233_31;
  TNode<IntPtrT> phi_bb233_34;
  TNode<BoolT> phi_bb233_35;
  TNode<BoolT> phi_bb233_36;
  TNode<Object> phi_bb233_37;
  TNode<Int32T> phi_bb233_39;
  TNode<Object> tmp421;
  TNode<IntPtrT> tmp422;
  TNode<IntPtrT> tmp423;
  TNode<IntPtrT> tmp424;
  if (block233.is_used()) {
    ca_.Bind(&block233, &phi_bb233_17, &phi_bb233_18, &phi_bb233_19, &phi_bb233_28, &phi_bb233_29, &phi_bb233_30, &phi_bb233_31, &phi_bb233_34, &phi_bb233_35, &phi_bb233_36, &phi_bb233_37, &phi_bb233_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    std::tie(tmp421, tmp422) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb233_29}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 311);
    tmp423 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp424 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb233_29}, TNode<IntPtrT>{tmp423});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 312);
    ca_.Goto(&block232, phi_bb233_17, phi_bb233_18, phi_bb233_19, phi_bb233_28, tmp424, phi_bb233_30, phi_bb233_31, phi_bb233_34, phi_bb233_35, phi_bb233_36, phi_bb233_37, phi_bb233_39, tmp421, tmp422);
  }

  TNode<Object> phi_bb234_17;
  TNode<IntPtrT> phi_bb234_18;
  TNode<BoolT> phi_bb234_19;
  TNode<IntPtrT> phi_bb234_28;
  TNode<IntPtrT> phi_bb234_29;
  TNode<IntPtrT> phi_bb234_30;
  TNode<IntPtrT> phi_bb234_31;
  TNode<IntPtrT> phi_bb234_34;
  TNode<BoolT> phi_bb234_35;
  TNode<BoolT> phi_bb234_36;
  TNode<Object> phi_bb234_37;
  TNode<Int32T> phi_bb234_39;
  if (block234.is_used()) {
    ca_.Bind(&block234, &phi_bb234_17, &phi_bb234_18, &phi_bb234_19, &phi_bb234_28, &phi_bb234_29, &phi_bb234_30, &phi_bb234_31, &phi_bb234_34, &phi_bb234_35, &phi_bb234_36, &phi_bb234_37, &phi_bb234_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block236, phi_bb234_17, phi_bb234_18, phi_bb234_19, phi_bb234_28, phi_bb234_29, phi_bb234_30, phi_bb234_31, phi_bb234_34, phi_bb234_35, phi_bb234_36, phi_bb234_37, phi_bb234_39);
    } else {
      ca_.Goto(&block237, phi_bb234_17, phi_bb234_18, phi_bb234_19, phi_bb234_28, phi_bb234_29, phi_bb234_30, phi_bb234_31, phi_bb234_34, phi_bb234_35, phi_bb234_36, phi_bb234_37, phi_bb234_39);
    }
  }

  TNode<Object> phi_bb236_17;
  TNode<IntPtrT> phi_bb236_18;
  TNode<BoolT> phi_bb236_19;
  TNode<IntPtrT> phi_bb236_28;
  TNode<IntPtrT> phi_bb236_29;
  TNode<IntPtrT> phi_bb236_30;
  TNode<IntPtrT> phi_bb236_31;
  TNode<IntPtrT> phi_bb236_34;
  TNode<BoolT> phi_bb236_35;
  TNode<BoolT> phi_bb236_36;
  TNode<Object> phi_bb236_37;
  TNode<Int32T> phi_bb236_39;
  TNode<Object> tmp425;
  TNode<IntPtrT> tmp426;
  TNode<IntPtrT> tmp427;
  TNode<IntPtrT> tmp428;
  if (block236.is_used()) {
    ca_.Bind(&block236, &phi_bb236_17, &phi_bb236_18, &phi_bb236_19, &phi_bb236_28, &phi_bb236_29, &phi_bb236_30, &phi_bb236_31, &phi_bb236_34, &phi_bb236_35, &phi_bb236_36, &phi_bb236_37, &phi_bb236_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp425, tmp426) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb236_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp427 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp428 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb236_31}, TNode<IntPtrT>{tmp427});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block235, phi_bb236_17, phi_bb236_18, phi_bb236_19, phi_bb236_28, phi_bb236_29, phi_bb236_30, tmp428, phi_bb236_34, phi_bb236_35, phi_bb236_36, phi_bb236_37, phi_bb236_39, tmp425, tmp426);
  }

  TNode<Object> phi_bb237_17;
  TNode<IntPtrT> phi_bb237_18;
  TNode<BoolT> phi_bb237_19;
  TNode<IntPtrT> phi_bb237_28;
  TNode<IntPtrT> phi_bb237_29;
  TNode<IntPtrT> phi_bb237_30;
  TNode<IntPtrT> phi_bb237_31;
  TNode<IntPtrT> phi_bb237_34;
  TNode<BoolT> phi_bb237_35;
  TNode<BoolT> phi_bb237_36;
  TNode<Object> phi_bb237_37;
  TNode<Int32T> phi_bb237_39;
  TNode<IntPtrT> tmp429;
  TNode<BoolT> tmp430;
  if (block237.is_used()) {
    ca_.Bind(&block237, &phi_bb237_17, &phi_bb237_18, &phi_bb237_19, &phi_bb237_28, &phi_bb237_29, &phi_bb237_30, &phi_bb237_31, &phi_bb237_34, &phi_bb237_35, &phi_bb237_36, &phi_bb237_37, &phi_bb237_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp429 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp430 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb237_34}, TNode<IntPtrT>{tmp429});
    ca_.Branch(tmp430, &block239, std::vector<compiler::Node*>{phi_bb237_17, phi_bb237_18, phi_bb237_19, phi_bb237_28, phi_bb237_29, phi_bb237_30, phi_bb237_31, phi_bb237_34, phi_bb237_35, phi_bb237_36, phi_bb237_37, phi_bb237_39}, &block240, std::vector<compiler::Node*>{phi_bb237_17, phi_bb237_18, phi_bb237_19, phi_bb237_28, phi_bb237_29, phi_bb237_30, phi_bb237_31, phi_bb237_34, phi_bb237_35, phi_bb237_36, phi_bb237_37, phi_bb237_39});
  }

  TNode<Object> phi_bb239_17;
  TNode<IntPtrT> phi_bb239_18;
  TNode<BoolT> phi_bb239_19;
  TNode<IntPtrT> phi_bb239_28;
  TNode<IntPtrT> phi_bb239_29;
  TNode<IntPtrT> phi_bb239_30;
  TNode<IntPtrT> phi_bb239_31;
  TNode<IntPtrT> phi_bb239_34;
  TNode<BoolT> phi_bb239_35;
  TNode<BoolT> phi_bb239_36;
  TNode<Object> phi_bb239_37;
  TNode<Int32T> phi_bb239_39;
  TNode<Object> tmp431;
  TNode<IntPtrT> tmp432;
  TNode<IntPtrT> tmp433;
  TNode<BoolT> tmp434;
  if (block239.is_used()) {
    ca_.Bind(&block239, &phi_bb239_17, &phi_bb239_18, &phi_bb239_19, &phi_bb239_28, &phi_bb239_29, &phi_bb239_30, &phi_bb239_31, &phi_bb239_34, &phi_bb239_35, &phi_bb239_36, &phi_bb239_37, &phi_bb239_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp431, tmp432) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb239_34}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp433 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp434 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block235, phi_bb239_17, phi_bb239_18, phi_bb239_19, phi_bb239_28, phi_bb239_29, phi_bb239_30, phi_bb239_31, tmp433, tmp434, phi_bb239_36, phi_bb239_37, phi_bb239_39, tmp431, tmp432);
  }

  TNode<Object> phi_bb240_17;
  TNode<IntPtrT> phi_bb240_18;
  TNode<BoolT> phi_bb240_19;
  TNode<IntPtrT> phi_bb240_28;
  TNode<IntPtrT> phi_bb240_29;
  TNode<IntPtrT> phi_bb240_30;
  TNode<IntPtrT> phi_bb240_31;
  TNode<IntPtrT> phi_bb240_34;
  TNode<BoolT> phi_bb240_35;
  TNode<BoolT> phi_bb240_36;
  TNode<Object> phi_bb240_37;
  TNode<Int32T> phi_bb240_39;
  TNode<Object> tmp435;
  TNode<IntPtrT> tmp436;
  TNode<IntPtrT> tmp437;
  TNode<IntPtrT> tmp438;
  TNode<IntPtrT> tmp439;
  TNode<IntPtrT> tmp440;
  TNode<BoolT> tmp441;
  if (block240.is_used()) {
    ca_.Bind(&block240, &phi_bb240_17, &phi_bb240_18, &phi_bb240_19, &phi_bb240_28, &phi_bb240_29, &phi_bb240_30, &phi_bb240_31, &phi_bb240_34, &phi_bb240_35, &phi_bb240_36, &phi_bb240_37, &phi_bb240_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp435, tmp436) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb240_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp437 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp438 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb240_31}, TNode<IntPtrT>{tmp437});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp439 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp440 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp438}, TNode<IntPtrT>{tmp439});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp441 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block235, phi_bb240_17, phi_bb240_18, phi_bb240_19, phi_bb240_28, phi_bb240_29, phi_bb240_30, tmp440, tmp438, tmp441, phi_bb240_36, phi_bb240_37, phi_bb240_39, tmp435, tmp436);
  }

  TNode<Object> phi_bb235_17;
  TNode<IntPtrT> phi_bb235_18;
  TNode<BoolT> phi_bb235_19;
  TNode<IntPtrT> phi_bb235_28;
  TNode<IntPtrT> phi_bb235_29;
  TNode<IntPtrT> phi_bb235_30;
  TNode<IntPtrT> phi_bb235_31;
  TNode<IntPtrT> phi_bb235_34;
  TNode<BoolT> phi_bb235_35;
  TNode<BoolT> phi_bb235_36;
  TNode<Object> phi_bb235_37;
  TNode<Int32T> phi_bb235_39;
  TNode<Object> phi_bb235_42;
  TNode<IntPtrT> phi_bb235_43;
  if (block235.is_used()) {
    ca_.Bind(&block235, &phi_bb235_17, &phi_bb235_18, &phi_bb235_19, &phi_bb235_28, &phi_bb235_29, &phi_bb235_30, &phi_bb235_31, &phi_bb235_34, &phi_bb235_35, &phi_bb235_36, &phi_bb235_37, &phi_bb235_39, &phi_bb235_42, &phi_bb235_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 314);
    ca_.Goto(&block232, phi_bb235_17, phi_bb235_18, phi_bb235_19, phi_bb235_28, phi_bb235_29, phi_bb235_30, phi_bb235_31, phi_bb235_34, phi_bb235_35, phi_bb235_36, phi_bb235_37, phi_bb235_39, phi_bb235_42, phi_bb235_43);
  }

  TNode<Object> phi_bb232_17;
  TNode<IntPtrT> phi_bb232_18;
  TNode<BoolT> phi_bb232_19;
  TNode<IntPtrT> phi_bb232_28;
  TNode<IntPtrT> phi_bb232_29;
  TNode<IntPtrT> phi_bb232_30;
  TNode<IntPtrT> phi_bb232_31;
  TNode<IntPtrT> phi_bb232_34;
  TNode<BoolT> phi_bb232_35;
  TNode<BoolT> phi_bb232_36;
  TNode<Object> phi_bb232_37;
  TNode<Int32T> phi_bb232_39;
  TNode<Object> phi_bb232_42;
  TNode<IntPtrT> phi_bb232_43;
  TNode<IntPtrT> tmp442;
      TNode<Object> tmp444;
  if (block232.is_used()) {
    ca_.Bind(&block232, &phi_bb232_17, &phi_bb232_18, &phi_bb232_19, &phi_bb232_28, &phi_bb232_29, &phi_bb232_30, &phi_bb232_31, &phi_bb232_34, &phi_bb232_35, &phi_bb232_36, &phi_bb232_37, &phi_bb232_39, &phi_bb232_42, &phi_bb232_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 704);
    compiler::CodeAssemblerExceptionHandlerLabel catch443__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch443__label);
    tmp442 = TruncateBigIntToI64_0(state_, TNode<Context>{p_context}, TNode<Object>{tmp234});
    }
    if (catch443__label.is_used()) {
      compiler::CodeAssemblerLabel catch443_skip(&ca_);
      ca_.Goto(&catch443_skip);
      ca_.Bind(&catch443__label, &tmp444);
      ca_.Goto(&block241, phi_bb232_17, phi_bb232_18, phi_bb232_19, phi_bb232_28, phi_bb232_29, phi_bb232_30, phi_bb232_31, phi_bb232_34, phi_bb232_35, phi_bb232_36, phi_bb232_37, phi_bb232_39, phi_bb232_42, phi_bb232_43);
      ca_.Bind(&catch443_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 705);
    CodeStubAssembler(state_).StoreReference<IntPtrT>(CodeStubAssembler::Reference{phi_bb232_42, phi_bb232_43}, tmp442);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 702);
    ca_.Goto(&block231, phi_bb232_17, phi_bb232_18, phi_bb232_19, tmp418, phi_bb232_28, phi_bb232_29, phi_bb232_30, phi_bb232_31, phi_bb232_34, phi_bb232_35, phi_bb232_36, phi_bb232_37, phi_bb232_39);
  }

  TNode<Object> phi_bb241_17;
  TNode<IntPtrT> phi_bb241_18;
  TNode<BoolT> phi_bb241_19;
  TNode<IntPtrT> phi_bb241_28;
  TNode<IntPtrT> phi_bb241_29;
  TNode<IntPtrT> phi_bb241_30;
  TNode<IntPtrT> phi_bb241_31;
  TNode<IntPtrT> phi_bb241_34;
  TNode<BoolT> phi_bb241_35;
  TNode<BoolT> phi_bb241_36;
  TNode<Object> phi_bb241_37;
  TNode<Int32T> phi_bb241_39;
  TNode<Object> phi_bb241_42;
  TNode<IntPtrT> phi_bb241_43;
  TNode<HeapObject> tmp445;
  if (block241.is_used()) {
    ca_.Bind(&block241, &phi_bb241_17, &phi_bb241_18, &phi_bb241_19, &phi_bb241_28, &phi_bb241_29, &phi_bb241_30, &phi_bb241_31, &phi_bb241_34, &phi_bb241_35, &phi_bb241_36, &phi_bb241_37, &phi_bb241_39, &phi_bb241_42, &phi_bb241_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 704);
    tmp445 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb241_17, phi_bb241_18, phi_bb241_19, tmp444, tmp445);
  }

  TNode<Object> phi_bb230_17;
  TNode<IntPtrT> phi_bb230_18;
  TNode<BoolT> phi_bb230_19;
  TNode<IntPtrT> phi_bb230_27;
  TNode<IntPtrT> phi_bb230_28;
  TNode<IntPtrT> phi_bb230_29;
  TNode<IntPtrT> phi_bb230_30;
  TNode<IntPtrT> phi_bb230_31;
  TNode<IntPtrT> phi_bb230_34;
  TNode<BoolT> phi_bb230_35;
  TNode<BoolT> phi_bb230_36;
  TNode<Object> phi_bb230_37;
  TNode<Int32T> phi_bb230_39;
  TNode<IntPtrT> tmp446;
  TNode<IntPtrT> tmp447;
  TNode<IntPtrT> tmp448;
  TNode<BoolT> tmp449;
  if (block230.is_used()) {
    ca_.Bind(&block230, &phi_bb230_17, &phi_bb230_18, &phi_bb230_19, &phi_bb230_27, &phi_bb230_28, &phi_bb230_29, &phi_bb230_30, &phi_bb230_31, &phi_bb230_34, &phi_bb230_35, &phi_bb230_36, &phi_bb230_37, &phi_bb230_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 308);
    tmp446 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp447 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb230_27}, TNode<IntPtrT>{tmp446});
    tmp448 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp449 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb230_27}, TNode<IntPtrT>{tmp448});
    ca_.Branch(tmp449, &block243, std::vector<compiler::Node*>{phi_bb230_17, phi_bb230_18, phi_bb230_19, phi_bb230_28, phi_bb230_29, phi_bb230_30, phi_bb230_31, phi_bb230_34, phi_bb230_35, phi_bb230_36, phi_bb230_37, phi_bb230_39}, &block244, std::vector<compiler::Node*>{phi_bb230_17, phi_bb230_18, phi_bb230_19, phi_bb230_28, phi_bb230_29, phi_bb230_30, phi_bb230_31, phi_bb230_34, phi_bb230_35, phi_bb230_36, phi_bb230_37, phi_bb230_39});
  }

  TNode<Object> phi_bb243_17;
  TNode<IntPtrT> phi_bb243_18;
  TNode<BoolT> phi_bb243_19;
  TNode<IntPtrT> phi_bb243_28;
  TNode<IntPtrT> phi_bb243_29;
  TNode<IntPtrT> phi_bb243_30;
  TNode<IntPtrT> phi_bb243_31;
  TNode<IntPtrT> phi_bb243_34;
  TNode<BoolT> phi_bb243_35;
  TNode<BoolT> phi_bb243_36;
  TNode<Object> phi_bb243_37;
  TNode<Int32T> phi_bb243_39;
  TNode<Object> tmp450;
  TNode<IntPtrT> tmp451;
  TNode<IntPtrT> tmp452;
  TNode<IntPtrT> tmp453;
  if (block243.is_used()) {
    ca_.Bind(&block243, &phi_bb243_17, &phi_bb243_18, &phi_bb243_19, &phi_bb243_28, &phi_bb243_29, &phi_bb243_30, &phi_bb243_31, &phi_bb243_34, &phi_bb243_35, &phi_bb243_36, &phi_bb243_37, &phi_bb243_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    std::tie(tmp450, tmp451) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb243_29}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 311);
    tmp452 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp453 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb243_29}, TNode<IntPtrT>{tmp452});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 312);
    ca_.Goto(&block242, phi_bb243_17, phi_bb243_18, phi_bb243_19, phi_bb243_28, tmp453, phi_bb243_30, phi_bb243_31, phi_bb243_34, phi_bb243_35, phi_bb243_36, phi_bb243_37, phi_bb243_39, tmp450, tmp451);
  }

  TNode<Object> phi_bb244_17;
  TNode<IntPtrT> phi_bb244_18;
  TNode<BoolT> phi_bb244_19;
  TNode<IntPtrT> phi_bb244_28;
  TNode<IntPtrT> phi_bb244_29;
  TNode<IntPtrT> phi_bb244_30;
  TNode<IntPtrT> phi_bb244_31;
  TNode<IntPtrT> phi_bb244_34;
  TNode<BoolT> phi_bb244_35;
  TNode<BoolT> phi_bb244_36;
  TNode<Object> phi_bb244_37;
  TNode<Int32T> phi_bb244_39;
  if (block244.is_used()) {
    ca_.Bind(&block244, &phi_bb244_17, &phi_bb244_18, &phi_bb244_19, &phi_bb244_28, &phi_bb244_29, &phi_bb244_30, &phi_bb244_31, &phi_bb244_34, &phi_bb244_35, &phi_bb244_36, &phi_bb244_37, &phi_bb244_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block246, phi_bb244_17, phi_bb244_18, phi_bb244_19, phi_bb244_28, phi_bb244_29, phi_bb244_30, phi_bb244_31, phi_bb244_34, phi_bb244_35, phi_bb244_36, phi_bb244_37, phi_bb244_39);
    } else {
      ca_.Goto(&block247, phi_bb244_17, phi_bb244_18, phi_bb244_19, phi_bb244_28, phi_bb244_29, phi_bb244_30, phi_bb244_31, phi_bb244_34, phi_bb244_35, phi_bb244_36, phi_bb244_37, phi_bb244_39);
    }
  }

  TNode<Object> phi_bb246_17;
  TNode<IntPtrT> phi_bb246_18;
  TNode<BoolT> phi_bb246_19;
  TNode<IntPtrT> phi_bb246_28;
  TNode<IntPtrT> phi_bb246_29;
  TNode<IntPtrT> phi_bb246_30;
  TNode<IntPtrT> phi_bb246_31;
  TNode<IntPtrT> phi_bb246_34;
  TNode<BoolT> phi_bb246_35;
  TNode<BoolT> phi_bb246_36;
  TNode<Object> phi_bb246_37;
  TNode<Int32T> phi_bb246_39;
  TNode<Object> tmp454;
  TNode<IntPtrT> tmp455;
  TNode<IntPtrT> tmp456;
  TNode<IntPtrT> tmp457;
  if (block246.is_used()) {
    ca_.Bind(&block246, &phi_bb246_17, &phi_bb246_18, &phi_bb246_19, &phi_bb246_28, &phi_bb246_29, &phi_bb246_30, &phi_bb246_31, &phi_bb246_34, &phi_bb246_35, &phi_bb246_36, &phi_bb246_37, &phi_bb246_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp454, tmp455) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb246_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp456 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp457 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb246_31}, TNode<IntPtrT>{tmp456});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block245, phi_bb246_17, phi_bb246_18, phi_bb246_19, phi_bb246_28, phi_bb246_29, phi_bb246_30, tmp457, phi_bb246_34, phi_bb246_35, phi_bb246_36, phi_bb246_37, phi_bb246_39, tmp454, tmp455);
  }

  TNode<Object> phi_bb247_17;
  TNode<IntPtrT> phi_bb247_18;
  TNode<BoolT> phi_bb247_19;
  TNode<IntPtrT> phi_bb247_28;
  TNode<IntPtrT> phi_bb247_29;
  TNode<IntPtrT> phi_bb247_30;
  TNode<IntPtrT> phi_bb247_31;
  TNode<IntPtrT> phi_bb247_34;
  TNode<BoolT> phi_bb247_35;
  TNode<BoolT> phi_bb247_36;
  TNode<Object> phi_bb247_37;
  TNode<Int32T> phi_bb247_39;
  TNode<IntPtrT> tmp458;
  TNode<BoolT> tmp459;
  if (block247.is_used()) {
    ca_.Bind(&block247, &phi_bb247_17, &phi_bb247_18, &phi_bb247_19, &phi_bb247_28, &phi_bb247_29, &phi_bb247_30, &phi_bb247_31, &phi_bb247_34, &phi_bb247_35, &phi_bb247_36, &phi_bb247_37, &phi_bb247_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp458 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp459 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb247_34}, TNode<IntPtrT>{tmp458});
    ca_.Branch(tmp459, &block249, std::vector<compiler::Node*>{phi_bb247_17, phi_bb247_18, phi_bb247_19, phi_bb247_28, phi_bb247_29, phi_bb247_30, phi_bb247_31, phi_bb247_34, phi_bb247_35, phi_bb247_36, phi_bb247_37, phi_bb247_39}, &block250, std::vector<compiler::Node*>{phi_bb247_17, phi_bb247_18, phi_bb247_19, phi_bb247_28, phi_bb247_29, phi_bb247_30, phi_bb247_31, phi_bb247_34, phi_bb247_35, phi_bb247_36, phi_bb247_37, phi_bb247_39});
  }

  TNode<Object> phi_bb249_17;
  TNode<IntPtrT> phi_bb249_18;
  TNode<BoolT> phi_bb249_19;
  TNode<IntPtrT> phi_bb249_28;
  TNode<IntPtrT> phi_bb249_29;
  TNode<IntPtrT> phi_bb249_30;
  TNode<IntPtrT> phi_bb249_31;
  TNode<IntPtrT> phi_bb249_34;
  TNode<BoolT> phi_bb249_35;
  TNode<BoolT> phi_bb249_36;
  TNode<Object> phi_bb249_37;
  TNode<Int32T> phi_bb249_39;
  TNode<Object> tmp460;
  TNode<IntPtrT> tmp461;
  TNode<IntPtrT> tmp462;
  TNode<BoolT> tmp463;
  if (block249.is_used()) {
    ca_.Bind(&block249, &phi_bb249_17, &phi_bb249_18, &phi_bb249_19, &phi_bb249_28, &phi_bb249_29, &phi_bb249_30, &phi_bb249_31, &phi_bb249_34, &phi_bb249_35, &phi_bb249_36, &phi_bb249_37, &phi_bb249_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp460, tmp461) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb249_34}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp462 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp463 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block245, phi_bb249_17, phi_bb249_18, phi_bb249_19, phi_bb249_28, phi_bb249_29, phi_bb249_30, phi_bb249_31, tmp462, tmp463, phi_bb249_36, phi_bb249_37, phi_bb249_39, tmp460, tmp461);
  }

  TNode<Object> phi_bb250_17;
  TNode<IntPtrT> phi_bb250_18;
  TNode<BoolT> phi_bb250_19;
  TNode<IntPtrT> phi_bb250_28;
  TNode<IntPtrT> phi_bb250_29;
  TNode<IntPtrT> phi_bb250_30;
  TNode<IntPtrT> phi_bb250_31;
  TNode<IntPtrT> phi_bb250_34;
  TNode<BoolT> phi_bb250_35;
  TNode<BoolT> phi_bb250_36;
  TNode<Object> phi_bb250_37;
  TNode<Int32T> phi_bb250_39;
  TNode<Object> tmp464;
  TNode<IntPtrT> tmp465;
  TNode<IntPtrT> tmp466;
  TNode<IntPtrT> tmp467;
  TNode<IntPtrT> tmp468;
  TNode<IntPtrT> tmp469;
  TNode<BoolT> tmp470;
  if (block250.is_used()) {
    ca_.Bind(&block250, &phi_bb250_17, &phi_bb250_18, &phi_bb250_19, &phi_bb250_28, &phi_bb250_29, &phi_bb250_30, &phi_bb250_31, &phi_bb250_34, &phi_bb250_35, &phi_bb250_36, &phi_bb250_37, &phi_bb250_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp464, tmp465) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb250_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp466 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp467 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb250_31}, TNode<IntPtrT>{tmp466});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp468 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp469 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp467}, TNode<IntPtrT>{tmp468});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp470 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block245, phi_bb250_17, phi_bb250_18, phi_bb250_19, phi_bb250_28, phi_bb250_29, phi_bb250_30, tmp469, tmp467, tmp470, phi_bb250_36, phi_bb250_37, phi_bb250_39, tmp464, tmp465);
  }

  TNode<Object> phi_bb245_17;
  TNode<IntPtrT> phi_bb245_18;
  TNode<BoolT> phi_bb245_19;
  TNode<IntPtrT> phi_bb245_28;
  TNode<IntPtrT> phi_bb245_29;
  TNode<IntPtrT> phi_bb245_30;
  TNode<IntPtrT> phi_bb245_31;
  TNode<IntPtrT> phi_bb245_34;
  TNode<BoolT> phi_bb245_35;
  TNode<BoolT> phi_bb245_36;
  TNode<Object> phi_bb245_37;
  TNode<Int32T> phi_bb245_39;
  TNode<Object> phi_bb245_42;
  TNode<IntPtrT> phi_bb245_43;
  if (block245.is_used()) {
    ca_.Bind(&block245, &phi_bb245_17, &phi_bb245_18, &phi_bb245_19, &phi_bb245_28, &phi_bb245_29, &phi_bb245_30, &phi_bb245_31, &phi_bb245_34, &phi_bb245_35, &phi_bb245_36, &phi_bb245_37, &phi_bb245_39, &phi_bb245_42, &phi_bb245_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 314);
    ca_.Goto(&block242, phi_bb245_17, phi_bb245_18, phi_bb245_19, phi_bb245_28, phi_bb245_29, phi_bb245_30, phi_bb245_31, phi_bb245_34, phi_bb245_35, phi_bb245_36, phi_bb245_37, phi_bb245_39, phi_bb245_42, phi_bb245_43);
  }

  TNode<Object> phi_bb242_17;
  TNode<IntPtrT> phi_bb242_18;
  TNode<BoolT> phi_bb242_19;
  TNode<IntPtrT> phi_bb242_28;
  TNode<IntPtrT> phi_bb242_29;
  TNode<IntPtrT> phi_bb242_30;
  TNode<IntPtrT> phi_bb242_31;
  TNode<IntPtrT> phi_bb242_34;
  TNode<BoolT> phi_bb242_35;
  TNode<BoolT> phi_bb242_36;
  TNode<Object> phi_bb242_37;
  TNode<Int32T> phi_bb242_39;
  TNode<Object> phi_bb242_42;
  TNode<IntPtrT> phi_bb242_43;
  TNode<IntPtrT> tmp471;
  TNode<IntPtrT> tmp472;
  TNode<IntPtrT> tmp473;
  TNode<BoolT> tmp474;
  if (block242.is_used()) {
    ca_.Bind(&block242, &phi_bb242_17, &phi_bb242_18, &phi_bb242_19, &phi_bb242_28, &phi_bb242_29, &phi_bb242_30, &phi_bb242_31, &phi_bb242_34, &phi_bb242_35, &phi_bb242_36, &phi_bb242_37, &phi_bb242_39, &phi_bb242_42, &phi_bb242_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 308);
    tmp471 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp472 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{tmp447}, TNode<IntPtrT>{tmp471});
    tmp473 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp474 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{tmp447}, TNode<IntPtrT>{tmp473});
    ca_.Branch(tmp474, &block252, std::vector<compiler::Node*>{phi_bb242_17, phi_bb242_18, phi_bb242_19, phi_bb242_28, phi_bb242_29, phi_bb242_30, phi_bb242_31, phi_bb242_34, phi_bb242_35, phi_bb242_36, phi_bb242_37, phi_bb242_39, phi_bb242_42, phi_bb242_43}, &block253, std::vector<compiler::Node*>{phi_bb242_17, phi_bb242_18, phi_bb242_19, phi_bb242_28, phi_bb242_29, phi_bb242_30, phi_bb242_31, phi_bb242_34, phi_bb242_35, phi_bb242_36, phi_bb242_37, phi_bb242_39, phi_bb242_42, phi_bb242_43});
  }

  TNode<Object> phi_bb252_17;
  TNode<IntPtrT> phi_bb252_18;
  TNode<BoolT> phi_bb252_19;
  TNode<IntPtrT> phi_bb252_28;
  TNode<IntPtrT> phi_bb252_29;
  TNode<IntPtrT> phi_bb252_30;
  TNode<IntPtrT> phi_bb252_31;
  TNode<IntPtrT> phi_bb252_34;
  TNode<BoolT> phi_bb252_35;
  TNode<BoolT> phi_bb252_36;
  TNode<Object> phi_bb252_37;
  TNode<Int32T> phi_bb252_39;
  TNode<Object> phi_bb252_42;
  TNode<IntPtrT> phi_bb252_43;
  TNode<Object> tmp475;
  TNode<IntPtrT> tmp476;
  TNode<IntPtrT> tmp477;
  TNode<IntPtrT> tmp478;
  if (block252.is_used()) {
    ca_.Bind(&block252, &phi_bb252_17, &phi_bb252_18, &phi_bb252_19, &phi_bb252_28, &phi_bb252_29, &phi_bb252_30, &phi_bb252_31, &phi_bb252_34, &phi_bb252_35, &phi_bb252_36, &phi_bb252_37, &phi_bb252_39, &phi_bb252_42, &phi_bb252_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    std::tie(tmp475, tmp476) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb252_29}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 311);
    tmp477 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp478 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb252_29}, TNode<IntPtrT>{tmp477});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 312);
    ca_.Goto(&block251, phi_bb252_17, phi_bb252_18, phi_bb252_19, phi_bb252_28, tmp478, phi_bb252_30, phi_bb252_31, phi_bb252_34, phi_bb252_35, phi_bb252_36, phi_bb252_37, phi_bb252_39, phi_bb252_42, phi_bb252_43, tmp475, tmp476);
  }

  TNode<Object> phi_bb253_17;
  TNode<IntPtrT> phi_bb253_18;
  TNode<BoolT> phi_bb253_19;
  TNode<IntPtrT> phi_bb253_28;
  TNode<IntPtrT> phi_bb253_29;
  TNode<IntPtrT> phi_bb253_30;
  TNode<IntPtrT> phi_bb253_31;
  TNode<IntPtrT> phi_bb253_34;
  TNode<BoolT> phi_bb253_35;
  TNode<BoolT> phi_bb253_36;
  TNode<Object> phi_bb253_37;
  TNode<Int32T> phi_bb253_39;
  TNode<Object> phi_bb253_42;
  TNode<IntPtrT> phi_bb253_43;
  if (block253.is_used()) {
    ca_.Bind(&block253, &phi_bb253_17, &phi_bb253_18, &phi_bb253_19, &phi_bb253_28, &phi_bb253_29, &phi_bb253_30, &phi_bb253_31, &phi_bb253_34, &phi_bb253_35, &phi_bb253_36, &phi_bb253_37, &phi_bb253_39, &phi_bb253_42, &phi_bb253_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block255, phi_bb253_17, phi_bb253_18, phi_bb253_19, phi_bb253_28, phi_bb253_29, phi_bb253_30, phi_bb253_31, phi_bb253_34, phi_bb253_35, phi_bb253_36, phi_bb253_37, phi_bb253_39, phi_bb253_42, phi_bb253_43);
    } else {
      ca_.Goto(&block256, phi_bb253_17, phi_bb253_18, phi_bb253_19, phi_bb253_28, phi_bb253_29, phi_bb253_30, phi_bb253_31, phi_bb253_34, phi_bb253_35, phi_bb253_36, phi_bb253_37, phi_bb253_39, phi_bb253_42, phi_bb253_43);
    }
  }

  TNode<Object> phi_bb255_17;
  TNode<IntPtrT> phi_bb255_18;
  TNode<BoolT> phi_bb255_19;
  TNode<IntPtrT> phi_bb255_28;
  TNode<IntPtrT> phi_bb255_29;
  TNode<IntPtrT> phi_bb255_30;
  TNode<IntPtrT> phi_bb255_31;
  TNode<IntPtrT> phi_bb255_34;
  TNode<BoolT> phi_bb255_35;
  TNode<BoolT> phi_bb255_36;
  TNode<Object> phi_bb255_37;
  TNode<Int32T> phi_bb255_39;
  TNode<Object> phi_bb255_42;
  TNode<IntPtrT> phi_bb255_43;
  TNode<Object> tmp479;
  TNode<IntPtrT> tmp480;
  TNode<IntPtrT> tmp481;
  TNode<IntPtrT> tmp482;
  if (block255.is_used()) {
    ca_.Bind(&block255, &phi_bb255_17, &phi_bb255_18, &phi_bb255_19, &phi_bb255_28, &phi_bb255_29, &phi_bb255_30, &phi_bb255_31, &phi_bb255_34, &phi_bb255_35, &phi_bb255_36, &phi_bb255_37, &phi_bb255_39, &phi_bb255_42, &phi_bb255_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp479, tmp480) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb255_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp481 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp482 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb255_31}, TNode<IntPtrT>{tmp481});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block254, phi_bb255_17, phi_bb255_18, phi_bb255_19, phi_bb255_28, phi_bb255_29, phi_bb255_30, tmp482, phi_bb255_34, phi_bb255_35, phi_bb255_36, phi_bb255_37, phi_bb255_39, phi_bb255_42, phi_bb255_43, tmp479, tmp480);
  }

  TNode<Object> phi_bb256_17;
  TNode<IntPtrT> phi_bb256_18;
  TNode<BoolT> phi_bb256_19;
  TNode<IntPtrT> phi_bb256_28;
  TNode<IntPtrT> phi_bb256_29;
  TNode<IntPtrT> phi_bb256_30;
  TNode<IntPtrT> phi_bb256_31;
  TNode<IntPtrT> phi_bb256_34;
  TNode<BoolT> phi_bb256_35;
  TNode<BoolT> phi_bb256_36;
  TNode<Object> phi_bb256_37;
  TNode<Int32T> phi_bb256_39;
  TNode<Object> phi_bb256_42;
  TNode<IntPtrT> phi_bb256_43;
  TNode<IntPtrT> tmp483;
  TNode<BoolT> tmp484;
  if (block256.is_used()) {
    ca_.Bind(&block256, &phi_bb256_17, &phi_bb256_18, &phi_bb256_19, &phi_bb256_28, &phi_bb256_29, &phi_bb256_30, &phi_bb256_31, &phi_bb256_34, &phi_bb256_35, &phi_bb256_36, &phi_bb256_37, &phi_bb256_39, &phi_bb256_42, &phi_bb256_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp483 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp484 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb256_34}, TNode<IntPtrT>{tmp483});
    ca_.Branch(tmp484, &block258, std::vector<compiler::Node*>{phi_bb256_17, phi_bb256_18, phi_bb256_19, phi_bb256_28, phi_bb256_29, phi_bb256_30, phi_bb256_31, phi_bb256_34, phi_bb256_35, phi_bb256_36, phi_bb256_37, phi_bb256_39, phi_bb256_42, phi_bb256_43}, &block259, std::vector<compiler::Node*>{phi_bb256_17, phi_bb256_18, phi_bb256_19, phi_bb256_28, phi_bb256_29, phi_bb256_30, phi_bb256_31, phi_bb256_34, phi_bb256_35, phi_bb256_36, phi_bb256_37, phi_bb256_39, phi_bb256_42, phi_bb256_43});
  }

  TNode<Object> phi_bb258_17;
  TNode<IntPtrT> phi_bb258_18;
  TNode<BoolT> phi_bb258_19;
  TNode<IntPtrT> phi_bb258_28;
  TNode<IntPtrT> phi_bb258_29;
  TNode<IntPtrT> phi_bb258_30;
  TNode<IntPtrT> phi_bb258_31;
  TNode<IntPtrT> phi_bb258_34;
  TNode<BoolT> phi_bb258_35;
  TNode<BoolT> phi_bb258_36;
  TNode<Object> phi_bb258_37;
  TNode<Int32T> phi_bb258_39;
  TNode<Object> phi_bb258_42;
  TNode<IntPtrT> phi_bb258_43;
  TNode<Object> tmp485;
  TNode<IntPtrT> tmp486;
  TNode<IntPtrT> tmp487;
  TNode<BoolT> tmp488;
  if (block258.is_used()) {
    ca_.Bind(&block258, &phi_bb258_17, &phi_bb258_18, &phi_bb258_19, &phi_bb258_28, &phi_bb258_29, &phi_bb258_30, &phi_bb258_31, &phi_bb258_34, &phi_bb258_35, &phi_bb258_36, &phi_bb258_37, &phi_bb258_39, &phi_bb258_42, &phi_bb258_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp485, tmp486) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb258_34}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp487 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp488 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block254, phi_bb258_17, phi_bb258_18, phi_bb258_19, phi_bb258_28, phi_bb258_29, phi_bb258_30, phi_bb258_31, tmp487, tmp488, phi_bb258_36, phi_bb258_37, phi_bb258_39, phi_bb258_42, phi_bb258_43, tmp485, tmp486);
  }

  TNode<Object> phi_bb259_17;
  TNode<IntPtrT> phi_bb259_18;
  TNode<BoolT> phi_bb259_19;
  TNode<IntPtrT> phi_bb259_28;
  TNode<IntPtrT> phi_bb259_29;
  TNode<IntPtrT> phi_bb259_30;
  TNode<IntPtrT> phi_bb259_31;
  TNode<IntPtrT> phi_bb259_34;
  TNode<BoolT> phi_bb259_35;
  TNode<BoolT> phi_bb259_36;
  TNode<Object> phi_bb259_37;
  TNode<Int32T> phi_bb259_39;
  TNode<Object> phi_bb259_42;
  TNode<IntPtrT> phi_bb259_43;
  TNode<Object> tmp489;
  TNode<IntPtrT> tmp490;
  TNode<IntPtrT> tmp491;
  TNode<IntPtrT> tmp492;
  TNode<IntPtrT> tmp493;
  TNode<IntPtrT> tmp494;
  TNode<BoolT> tmp495;
  if (block259.is_used()) {
    ca_.Bind(&block259, &phi_bb259_17, &phi_bb259_18, &phi_bb259_19, &phi_bb259_28, &phi_bb259_29, &phi_bb259_30, &phi_bb259_31, &phi_bb259_34, &phi_bb259_35, &phi_bb259_36, &phi_bb259_37, &phi_bb259_39, &phi_bb259_42, &phi_bb259_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp489, tmp490) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb259_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp491 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp492 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb259_31}, TNode<IntPtrT>{tmp491});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp493 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp494 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp492}, TNode<IntPtrT>{tmp493});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp495 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block254, phi_bb259_17, phi_bb259_18, phi_bb259_19, phi_bb259_28, phi_bb259_29, phi_bb259_30, tmp494, tmp492, tmp495, phi_bb259_36, phi_bb259_37, phi_bb259_39, phi_bb259_42, phi_bb259_43, tmp489, tmp490);
  }

  TNode<Object> phi_bb254_17;
  TNode<IntPtrT> phi_bb254_18;
  TNode<BoolT> phi_bb254_19;
  TNode<IntPtrT> phi_bb254_28;
  TNode<IntPtrT> phi_bb254_29;
  TNode<IntPtrT> phi_bb254_30;
  TNode<IntPtrT> phi_bb254_31;
  TNode<IntPtrT> phi_bb254_34;
  TNode<BoolT> phi_bb254_35;
  TNode<BoolT> phi_bb254_36;
  TNode<Object> phi_bb254_37;
  TNode<Int32T> phi_bb254_39;
  TNode<Object> phi_bb254_42;
  TNode<IntPtrT> phi_bb254_43;
  TNode<Object> phi_bb254_44;
  TNode<IntPtrT> phi_bb254_45;
  if (block254.is_used()) {
    ca_.Bind(&block254, &phi_bb254_17, &phi_bb254_18, &phi_bb254_19, &phi_bb254_28, &phi_bb254_29, &phi_bb254_30, &phi_bb254_31, &phi_bb254_34, &phi_bb254_35, &phi_bb254_36, &phi_bb254_37, &phi_bb254_39, &phi_bb254_42, &phi_bb254_43, &phi_bb254_44, &phi_bb254_45);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 314);
    ca_.Goto(&block251, phi_bb254_17, phi_bb254_18, phi_bb254_19, phi_bb254_28, phi_bb254_29, phi_bb254_30, phi_bb254_31, phi_bb254_34, phi_bb254_35, phi_bb254_36, phi_bb254_37, phi_bb254_39, phi_bb254_42, phi_bb254_43, phi_bb254_44, phi_bb254_45);
  }

  TNode<Object> phi_bb251_17;
  TNode<IntPtrT> phi_bb251_18;
  TNode<BoolT> phi_bb251_19;
  TNode<IntPtrT> phi_bb251_28;
  TNode<IntPtrT> phi_bb251_29;
  TNode<IntPtrT> phi_bb251_30;
  TNode<IntPtrT> phi_bb251_31;
  TNode<IntPtrT> phi_bb251_34;
  TNode<BoolT> phi_bb251_35;
  TNode<BoolT> phi_bb251_36;
  TNode<Object> phi_bb251_37;
  TNode<Int32T> phi_bb251_39;
  TNode<Object> phi_bb251_42;
  TNode<IntPtrT> phi_bb251_43;
  TNode<Object> phi_bb251_44;
  TNode<IntPtrT> phi_bb251_45;
  TNode<BigInt> tmp496;
      TNode<Object> tmp498;
  TNode<UintPtrT> tmp499;
  TNode<UintPtrT> tmp500;
      TNode<Object> tmp502;
  TNode<IntPtrT> tmp503;
      TNode<Object> tmp505;
  TNode<IntPtrT> tmp506;
      TNode<Object> tmp508;
  if (block251.is_used()) {
    ca_.Bind(&block251, &phi_bb251_17, &phi_bb251_18, &phi_bb251_19, &phi_bb251_28, &phi_bb251_29, &phi_bb251_30, &phi_bb251_31, &phi_bb251_34, &phi_bb251_35, &phi_bb251_36, &phi_bb251_37, &phi_bb251_39, &phi_bb251_42, &phi_bb251_43, &phi_bb251_44, &phi_bb251_45);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 709);
    compiler::CodeAssemblerExceptionHandlerLabel catch497__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch497__label);
    tmp496 = CodeStubAssembler(state_).ToBigInt(TNode<Context>{p_context}, TNode<Object>{tmp234});
    }
    if (catch497__label.is_used()) {
      compiler::CodeAssemblerLabel catch497_skip(&ca_);
      ca_.Goto(&catch497_skip);
      ca_.Bind(&catch497__label, &tmp498);
      ca_.Goto(&block260, phi_bb251_17, phi_bb251_18, phi_bb251_19, phi_bb251_28, phi_bb251_29, phi_bb251_30, phi_bb251_31, phi_bb251_34, phi_bb251_35, phi_bb251_36, phi_bb251_37, phi_bb251_39, phi_bb251_42, phi_bb251_43, phi_bb251_44, phi_bb251_45);
      ca_.Bind(&catch497_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 710);
    compiler::CodeAssemblerExceptionHandlerLabel catch501__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch501__label);
    std::tie(tmp499, tmp500) = CodeStubAssembler(state_).BigIntToRawBytes(TNode<BigInt>{tmp496}).Flatten();
    }
    if (catch501__label.is_used()) {
      compiler::CodeAssemblerLabel catch501_skip(&ca_);
      ca_.Goto(&catch501_skip);
      ca_.Bind(&catch501__label, &tmp502);
      ca_.Goto(&block261, phi_bb251_17, phi_bb251_18, phi_bb251_19, phi_bb251_28, phi_bb251_29, phi_bb251_30, phi_bb251_31, phi_bb251_34, phi_bb251_35, phi_bb251_36, phi_bb251_37, phi_bb251_39, phi_bb251_42, phi_bb251_43, phi_bb251_44, phi_bb251_45);
      ca_.Bind(&catch501_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 711);
    compiler::CodeAssemblerExceptionHandlerLabel catch504__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch504__label);
    tmp503 = CodeStubAssembler(state_).Signed(TNode<UintPtrT>{tmp499});
    }
    if (catch504__label.is_used()) {
      compiler::CodeAssemblerLabel catch504_skip(&ca_);
      ca_.Goto(&catch504_skip);
      ca_.Bind(&catch504__label, &tmp505);
      ca_.Goto(&block262, phi_bb251_17, phi_bb251_18, phi_bb251_19, phi_bb251_28, phi_bb251_29, phi_bb251_30, phi_bb251_31, phi_bb251_34, phi_bb251_35, phi_bb251_36, phi_bb251_37, phi_bb251_39, phi_bb251_42, phi_bb251_43, phi_bb251_44, phi_bb251_45, phi_bb251_42, phi_bb251_43);
      ca_.Bind(&catch504_skip);
    }
    CodeStubAssembler(state_).StoreReference<IntPtrT>(CodeStubAssembler::Reference{phi_bb251_42, phi_bb251_43}, tmp503);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 712);
    compiler::CodeAssemblerExceptionHandlerLabel catch507__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch507__label);
    tmp506 = CodeStubAssembler(state_).Signed(TNode<UintPtrT>{tmp500});
    }
    if (catch507__label.is_used()) {
      compiler::CodeAssemblerLabel catch507_skip(&ca_);
      ca_.Goto(&catch507_skip);
      ca_.Bind(&catch507__label, &tmp508);
      ca_.Goto(&block263, phi_bb251_17, phi_bb251_18, phi_bb251_19, phi_bb251_28, phi_bb251_29, phi_bb251_30, phi_bb251_31, phi_bb251_34, phi_bb251_35, phi_bb251_36, phi_bb251_37, phi_bb251_39, phi_bb251_42, phi_bb251_43, phi_bb251_44, phi_bb251_45, phi_bb251_44, phi_bb251_45);
      ca_.Bind(&catch507_skip);
    }
    CodeStubAssembler(state_).StoreReference<IntPtrT>(CodeStubAssembler::Reference{phi_bb251_44, phi_bb251_45}, tmp506);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 702);
    ca_.Goto(&block231, phi_bb251_17, phi_bb251_18, phi_bb251_19, tmp472, phi_bb251_28, phi_bb251_29, phi_bb251_30, phi_bb251_31, phi_bb251_34, phi_bb251_35, phi_bb251_36, phi_bb251_37, phi_bb251_39);
  }

  TNode<Object> phi_bb260_17;
  TNode<IntPtrT> phi_bb260_18;
  TNode<BoolT> phi_bb260_19;
  TNode<IntPtrT> phi_bb260_28;
  TNode<IntPtrT> phi_bb260_29;
  TNode<IntPtrT> phi_bb260_30;
  TNode<IntPtrT> phi_bb260_31;
  TNode<IntPtrT> phi_bb260_34;
  TNode<BoolT> phi_bb260_35;
  TNode<BoolT> phi_bb260_36;
  TNode<Object> phi_bb260_37;
  TNode<Int32T> phi_bb260_39;
  TNode<Object> phi_bb260_42;
  TNode<IntPtrT> phi_bb260_43;
  TNode<Object> phi_bb260_44;
  TNode<IntPtrT> phi_bb260_45;
  TNode<HeapObject> tmp509;
  if (block260.is_used()) {
    ca_.Bind(&block260, &phi_bb260_17, &phi_bb260_18, &phi_bb260_19, &phi_bb260_28, &phi_bb260_29, &phi_bb260_30, &phi_bb260_31, &phi_bb260_34, &phi_bb260_35, &phi_bb260_36, &phi_bb260_37, &phi_bb260_39, &phi_bb260_42, &phi_bb260_43, &phi_bb260_44, &phi_bb260_45);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 709);
    tmp509 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb260_17, phi_bb260_18, phi_bb260_19, tmp498, tmp509);
  }

  TNode<Object> phi_bb261_17;
  TNode<IntPtrT> phi_bb261_18;
  TNode<BoolT> phi_bb261_19;
  TNode<IntPtrT> phi_bb261_28;
  TNode<IntPtrT> phi_bb261_29;
  TNode<IntPtrT> phi_bb261_30;
  TNode<IntPtrT> phi_bb261_31;
  TNode<IntPtrT> phi_bb261_34;
  TNode<BoolT> phi_bb261_35;
  TNode<BoolT> phi_bb261_36;
  TNode<Object> phi_bb261_37;
  TNode<Int32T> phi_bb261_39;
  TNode<Object> phi_bb261_42;
  TNode<IntPtrT> phi_bb261_43;
  TNode<Object> phi_bb261_44;
  TNode<IntPtrT> phi_bb261_45;
  TNode<HeapObject> tmp510;
  if (block261.is_used()) {
    ca_.Bind(&block261, &phi_bb261_17, &phi_bb261_18, &phi_bb261_19, &phi_bb261_28, &phi_bb261_29, &phi_bb261_30, &phi_bb261_31, &phi_bb261_34, &phi_bb261_35, &phi_bb261_36, &phi_bb261_37, &phi_bb261_39, &phi_bb261_42, &phi_bb261_43, &phi_bb261_44, &phi_bb261_45);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 710);
    tmp510 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb261_17, phi_bb261_18, phi_bb261_19, tmp502, tmp510);
  }

  TNode<Object> phi_bb262_17;
  TNode<IntPtrT> phi_bb262_18;
  TNode<BoolT> phi_bb262_19;
  TNode<IntPtrT> phi_bb262_28;
  TNode<IntPtrT> phi_bb262_29;
  TNode<IntPtrT> phi_bb262_30;
  TNode<IntPtrT> phi_bb262_31;
  TNode<IntPtrT> phi_bb262_34;
  TNode<BoolT> phi_bb262_35;
  TNode<BoolT> phi_bb262_36;
  TNode<Object> phi_bb262_37;
  TNode<Int32T> phi_bb262_39;
  TNode<Object> phi_bb262_42;
  TNode<IntPtrT> phi_bb262_43;
  TNode<Object> phi_bb262_44;
  TNode<IntPtrT> phi_bb262_45;
  TNode<Object> phi_bb262_49;
  TNode<IntPtrT> phi_bb262_50;
  TNode<HeapObject> tmp511;
  if (block262.is_used()) {
    ca_.Bind(&block262, &phi_bb262_17, &phi_bb262_18, &phi_bb262_19, &phi_bb262_28, &phi_bb262_29, &phi_bb262_30, &phi_bb262_31, &phi_bb262_34, &phi_bb262_35, &phi_bb262_36, &phi_bb262_37, &phi_bb262_39, &phi_bb262_42, &phi_bb262_43, &phi_bb262_44, &phi_bb262_45, &phi_bb262_49, &phi_bb262_50);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 711);
    tmp511 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb262_17, phi_bb262_18, phi_bb262_19, tmp505, tmp511);
  }

  TNode<Object> phi_bb263_17;
  TNode<IntPtrT> phi_bb263_18;
  TNode<BoolT> phi_bb263_19;
  TNode<IntPtrT> phi_bb263_28;
  TNode<IntPtrT> phi_bb263_29;
  TNode<IntPtrT> phi_bb263_30;
  TNode<IntPtrT> phi_bb263_31;
  TNode<IntPtrT> phi_bb263_34;
  TNode<BoolT> phi_bb263_35;
  TNode<BoolT> phi_bb263_36;
  TNode<Object> phi_bb263_37;
  TNode<Int32T> phi_bb263_39;
  TNode<Object> phi_bb263_42;
  TNode<IntPtrT> phi_bb263_43;
  TNode<Object> phi_bb263_44;
  TNode<IntPtrT> phi_bb263_45;
  TNode<Object> phi_bb263_49;
  TNode<IntPtrT> phi_bb263_50;
  TNode<HeapObject> tmp512;
  if (block263.is_used()) {
    ca_.Bind(&block263, &phi_bb263_17, &phi_bb263_18, &phi_bb263_19, &phi_bb263_28, &phi_bb263_29, &phi_bb263_30, &phi_bb263_31, &phi_bb263_34, &phi_bb263_35, &phi_bb263_36, &phi_bb263_37, &phi_bb263_39, &phi_bb263_42, &phi_bb263_43, &phi_bb263_44, &phi_bb263_45, &phi_bb263_49, &phi_bb263_50);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 712);
    tmp512 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb263_17, phi_bb263_18, phi_bb263_19, tmp508, tmp512);
  }

  TNode<Object> phi_bb231_17;
  TNode<IntPtrT> phi_bb231_18;
  TNode<BoolT> phi_bb231_19;
  TNode<IntPtrT> phi_bb231_27;
  TNode<IntPtrT> phi_bb231_28;
  TNode<IntPtrT> phi_bb231_29;
  TNode<IntPtrT> phi_bb231_30;
  TNode<IntPtrT> phi_bb231_31;
  TNode<IntPtrT> phi_bb231_34;
  TNode<BoolT> phi_bb231_35;
  TNode<BoolT> phi_bb231_36;
  TNode<Object> phi_bb231_37;
  TNode<Int32T> phi_bb231_39;
  if (block231.is_used()) {
    ca_.Bind(&block231, &phi_bb231_17, &phi_bb231_18, &phi_bb231_19, &phi_bb231_27, &phi_bb231_28, &phi_bb231_29, &phi_bb231_30, &phi_bb231_31, &phi_bb231_34, &phi_bb231_35, &phi_bb231_36, &phi_bb231_37, &phi_bb231_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 701);
    ca_.Goto(&block228, phi_bb231_17, phi_bb231_18, phi_bb231_19, phi_bb231_27, phi_bb231_28, phi_bb231_29, phi_bb231_30, phi_bb231_31, phi_bb231_34, phi_bb231_35, phi_bb231_36, phi_bb231_37, phi_bb231_39);
  }

  TNode<Object> phi_bb225_17;
  TNode<IntPtrT> phi_bb225_18;
  TNode<BoolT> phi_bb225_19;
  TNode<IntPtrT> phi_bb225_27;
  TNode<IntPtrT> phi_bb225_28;
  TNode<IntPtrT> phi_bb225_29;
  TNode<IntPtrT> phi_bb225_30;
  TNode<IntPtrT> phi_bb225_31;
  TNode<IntPtrT> phi_bb225_34;
  TNode<BoolT> phi_bb225_35;
  TNode<BoolT> phi_bb225_36;
  TNode<Object> phi_bb225_37;
  TNode<Int32T> phi_bb225_39;
  TNode<Int32T> tmp513;
      TNode<Object> tmp515;
  TNode<Int32T> tmp516;
      TNode<Object> tmp518;
  TNode<Int32T> tmp519;
      TNode<Object> tmp521;
  TNode<BoolT> tmp522;
      TNode<Object> tmp524;
  if (block225.is_used()) {
    ca_.Bind(&block225, &phi_bb225_17, &phi_bb225_18, &phi_bb225_19, &phi_bb225_27, &phi_bb225_28, &phi_bb225_29, &phi_bb225_30, &phi_bb225_31, &phi_bb225_34, &phi_bb225_35, &phi_bb225_36, &phi_bb225_37, &phi_bb225_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 715);
    compiler::CodeAssemblerExceptionHandlerLabel catch514__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch514__label);
    tmp513 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmValueKindBitsMask);
    }
    if (catch514__label.is_used()) {
      compiler::CodeAssemblerLabel catch514_skip(&ca_);
      ca_.Goto(&catch514_skip);
      ca_.Bind(&catch514__label, &tmp515);
      ca_.Goto(&block264, phi_bb225_17, phi_bb225_18, phi_bb225_19, phi_bb225_27, phi_bb225_28, phi_bb225_29, phi_bb225_30, phi_bb225_31, phi_bb225_34, phi_bb225_35, phi_bb225_36, phi_bb225_37, phi_bb225_39);
      ca_.Bind(&catch514_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch517__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch517__label);
    tmp516 = CodeStubAssembler(state_).Word32And(TNode<Int32T>{tmp250}, TNode<Int32T>{tmp513});
    }
    if (catch517__label.is_used()) {
      compiler::CodeAssemblerLabel catch517_skip(&ca_);
      ca_.Goto(&catch517_skip);
      ca_.Bind(&catch517__label, &tmp518);
      ca_.Goto(&block265, phi_bb225_17, phi_bb225_18, phi_bb225_19, phi_bb225_27, phi_bb225_28, phi_bb225_29, phi_bb225_30, phi_bb225_31, phi_bb225_34, phi_bb225_35, phi_bb225_36, phi_bb225_37, phi_bb225_39);
      ca_.Bind(&catch517_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 716);
    compiler::CodeAssemblerExceptionHandlerLabel catch520__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch520__label);
    tmp519 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRef);
    }
    if (catch520__label.is_used()) {
      compiler::CodeAssemblerLabel catch520_skip(&ca_);
      ca_.Goto(&catch520_skip);
      ca_.Bind(&catch520__label, &tmp521);
      ca_.Goto(&block268, phi_bb225_17, phi_bb225_18, phi_bb225_19, phi_bb225_27, phi_bb225_28, phi_bb225_29, phi_bb225_30, phi_bb225_31, phi_bb225_34, phi_bb225_35, phi_bb225_36, phi_bb225_37, phi_bb225_39);
      ca_.Bind(&catch520_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch523__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch523__label);
    tmp522 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp516}, TNode<Int32T>{tmp519});
    }
    if (catch523__label.is_used()) {
      compiler::CodeAssemblerLabel catch523_skip(&ca_);
      ca_.Goto(&catch523_skip);
      ca_.Bind(&catch523__label, &tmp524);
      ca_.Goto(&block269, phi_bb225_17, phi_bb225_18, phi_bb225_19, phi_bb225_27, phi_bb225_28, phi_bb225_29, phi_bb225_30, phi_bb225_31, phi_bb225_34, phi_bb225_35, phi_bb225_36, phi_bb225_37, phi_bb225_39);
      ca_.Bind(&catch523_skip);
    }
    ca_.Branch(tmp522, &block270, std::vector<compiler::Node*>{phi_bb225_17, phi_bb225_18, phi_bb225_19, phi_bb225_27, phi_bb225_28, phi_bb225_29, phi_bb225_30, phi_bb225_31, phi_bb225_34, phi_bb225_35, phi_bb225_36, phi_bb225_37, phi_bb225_39}, &block271, std::vector<compiler::Node*>{phi_bb225_17, phi_bb225_18, phi_bb225_19, phi_bb225_27, phi_bb225_28, phi_bb225_29, phi_bb225_30, phi_bb225_31, phi_bb225_34, phi_bb225_35, phi_bb225_36, phi_bb225_37, phi_bb225_39});
  }

  TNode<Object> phi_bb264_17;
  TNode<IntPtrT> phi_bb264_18;
  TNode<BoolT> phi_bb264_19;
  TNode<IntPtrT> phi_bb264_27;
  TNode<IntPtrT> phi_bb264_28;
  TNode<IntPtrT> phi_bb264_29;
  TNode<IntPtrT> phi_bb264_30;
  TNode<IntPtrT> phi_bb264_31;
  TNode<IntPtrT> phi_bb264_34;
  TNode<BoolT> phi_bb264_35;
  TNode<BoolT> phi_bb264_36;
  TNode<Object> phi_bb264_37;
  TNode<Int32T> phi_bb264_39;
  TNode<HeapObject> tmp525;
  if (block264.is_used()) {
    ca_.Bind(&block264, &phi_bb264_17, &phi_bb264_18, &phi_bb264_19, &phi_bb264_27, &phi_bb264_28, &phi_bb264_29, &phi_bb264_30, &phi_bb264_31, &phi_bb264_34, &phi_bb264_35, &phi_bb264_36, &phi_bb264_37, &phi_bb264_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 715);
    tmp525 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb264_17, phi_bb264_18, phi_bb264_19, tmp515, tmp525);
  }

  TNode<Object> phi_bb265_17;
  TNode<IntPtrT> phi_bb265_18;
  TNode<BoolT> phi_bb265_19;
  TNode<IntPtrT> phi_bb265_27;
  TNode<IntPtrT> phi_bb265_28;
  TNode<IntPtrT> phi_bb265_29;
  TNode<IntPtrT> phi_bb265_30;
  TNode<IntPtrT> phi_bb265_31;
  TNode<IntPtrT> phi_bb265_34;
  TNode<BoolT> phi_bb265_35;
  TNode<BoolT> phi_bb265_36;
  TNode<Object> phi_bb265_37;
  TNode<Int32T> phi_bb265_39;
  TNode<HeapObject> tmp526;
  if (block265.is_used()) {
    ca_.Bind(&block265, &phi_bb265_17, &phi_bb265_18, &phi_bb265_19, &phi_bb265_27, &phi_bb265_28, &phi_bb265_29, &phi_bb265_30, &phi_bb265_31, &phi_bb265_34, &phi_bb265_35, &phi_bb265_36, &phi_bb265_37, &phi_bb265_39);
    tmp526 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb265_17, phi_bb265_18, phi_bb265_19, tmp518, tmp526);
  }

  TNode<Object> phi_bb268_17;
  TNode<IntPtrT> phi_bb268_18;
  TNode<BoolT> phi_bb268_19;
  TNode<IntPtrT> phi_bb268_27;
  TNode<IntPtrT> phi_bb268_28;
  TNode<IntPtrT> phi_bb268_29;
  TNode<IntPtrT> phi_bb268_30;
  TNode<IntPtrT> phi_bb268_31;
  TNode<IntPtrT> phi_bb268_34;
  TNode<BoolT> phi_bb268_35;
  TNode<BoolT> phi_bb268_36;
  TNode<Object> phi_bb268_37;
  TNode<Int32T> phi_bb268_39;
  TNode<HeapObject> tmp527;
  if (block268.is_used()) {
    ca_.Bind(&block268, &phi_bb268_17, &phi_bb268_18, &phi_bb268_19, &phi_bb268_27, &phi_bb268_28, &phi_bb268_29, &phi_bb268_30, &phi_bb268_31, &phi_bb268_34, &phi_bb268_35, &phi_bb268_36, &phi_bb268_37, &phi_bb268_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 716);
    tmp527 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb268_17, phi_bb268_18, phi_bb268_19, tmp521, tmp527);
  }

  TNode<Object> phi_bb269_17;
  TNode<IntPtrT> phi_bb269_18;
  TNode<BoolT> phi_bb269_19;
  TNode<IntPtrT> phi_bb269_27;
  TNode<IntPtrT> phi_bb269_28;
  TNode<IntPtrT> phi_bb269_29;
  TNode<IntPtrT> phi_bb269_30;
  TNode<IntPtrT> phi_bb269_31;
  TNode<IntPtrT> phi_bb269_34;
  TNode<BoolT> phi_bb269_35;
  TNode<BoolT> phi_bb269_36;
  TNode<Object> phi_bb269_37;
  TNode<Int32T> phi_bb269_39;
  TNode<HeapObject> tmp528;
  if (block269.is_used()) {
    ca_.Bind(&block269, &phi_bb269_17, &phi_bb269_18, &phi_bb269_19, &phi_bb269_27, &phi_bb269_28, &phi_bb269_29, &phi_bb269_30, &phi_bb269_31, &phi_bb269_34, &phi_bb269_35, &phi_bb269_36, &phi_bb269_37, &phi_bb269_39);
    tmp528 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb269_17, phi_bb269_18, phi_bb269_19, tmp524, tmp528);
  }

  TNode<Object> phi_bb270_17;
  TNode<IntPtrT> phi_bb270_18;
  TNode<BoolT> phi_bb270_19;
  TNode<IntPtrT> phi_bb270_27;
  TNode<IntPtrT> phi_bb270_28;
  TNode<IntPtrT> phi_bb270_29;
  TNode<IntPtrT> phi_bb270_30;
  TNode<IntPtrT> phi_bb270_31;
  TNode<IntPtrT> phi_bb270_34;
  TNode<BoolT> phi_bb270_35;
  TNode<BoolT> phi_bb270_36;
  TNode<Object> phi_bb270_37;
  TNode<Int32T> phi_bb270_39;
  TNode<BoolT> tmp529;
      TNode<Object> tmp531;
  if (block270.is_used()) {
    ca_.Bind(&block270, &phi_bb270_17, &phi_bb270_18, &phi_bb270_19, &phi_bb270_27, &phi_bb270_28, &phi_bb270_29, &phi_bb270_30, &phi_bb270_31, &phi_bb270_34, &phi_bb270_35, &phi_bb270_36, &phi_bb270_37, &phi_bb270_39);
    compiler::CodeAssemblerExceptionHandlerLabel catch530__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch530__label);
    tmp529 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    }
    if (catch530__label.is_used()) {
      compiler::CodeAssemblerLabel catch530_skip(&ca_);
      ca_.Goto(&catch530_skip);
      ca_.Bind(&catch530__label, &tmp531);
      ca_.Goto(&block273, phi_bb270_17, phi_bb270_18, phi_bb270_19, phi_bb270_27, phi_bb270_28, phi_bb270_29, phi_bb270_30, phi_bb270_31, phi_bb270_34, phi_bb270_35, phi_bb270_36, phi_bb270_37, phi_bb270_39);
      ca_.Bind(&catch530_skip);
    }
    ca_.Goto(&block272, phi_bb270_17, phi_bb270_18, phi_bb270_19, phi_bb270_27, phi_bb270_28, phi_bb270_29, phi_bb270_30, phi_bb270_31, phi_bb270_34, phi_bb270_35, phi_bb270_36, phi_bb270_37, phi_bb270_39, tmp529);
  }

  TNode<Object> phi_bb273_17;
  TNode<IntPtrT> phi_bb273_18;
  TNode<BoolT> phi_bb273_19;
  TNode<IntPtrT> phi_bb273_27;
  TNode<IntPtrT> phi_bb273_28;
  TNode<IntPtrT> phi_bb273_29;
  TNode<IntPtrT> phi_bb273_30;
  TNode<IntPtrT> phi_bb273_31;
  TNode<IntPtrT> phi_bb273_34;
  TNode<BoolT> phi_bb273_35;
  TNode<BoolT> phi_bb273_36;
  TNode<Object> phi_bb273_37;
  TNode<Int32T> phi_bb273_39;
  TNode<HeapObject> tmp532;
  if (block273.is_used()) {
    ca_.Bind(&block273, &phi_bb273_17, &phi_bb273_18, &phi_bb273_19, &phi_bb273_27, &phi_bb273_28, &phi_bb273_29, &phi_bb273_30, &phi_bb273_31, &phi_bb273_34, &phi_bb273_35, &phi_bb273_36, &phi_bb273_37, &phi_bb273_39);
    tmp532 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb273_17, phi_bb273_18, phi_bb273_19, tmp531, tmp532);
  }

  TNode<Object> phi_bb271_17;
  TNode<IntPtrT> phi_bb271_18;
  TNode<BoolT> phi_bb271_19;
  TNode<IntPtrT> phi_bb271_27;
  TNode<IntPtrT> phi_bb271_28;
  TNode<IntPtrT> phi_bb271_29;
  TNode<IntPtrT> phi_bb271_30;
  TNode<IntPtrT> phi_bb271_31;
  TNode<IntPtrT> phi_bb271_34;
  TNode<BoolT> phi_bb271_35;
  TNode<BoolT> phi_bb271_36;
  TNode<Object> phi_bb271_37;
  TNode<Int32T> phi_bb271_39;
  TNode<Int32T> tmp533;
      TNode<Object> tmp535;
  TNode<BoolT> tmp536;
      TNode<Object> tmp538;
  if (block271.is_used()) {
    ca_.Bind(&block271, &phi_bb271_17, &phi_bb271_18, &phi_bb271_19, &phi_bb271_27, &phi_bb271_28, &phi_bb271_29, &phi_bb271_30, &phi_bb271_31, &phi_bb271_34, &phi_bb271_35, &phi_bb271_36, &phi_bb271_37, &phi_bb271_39);
    compiler::CodeAssemblerExceptionHandlerLabel catch534__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch534__label);
    tmp533 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRefNull);
    }
    if (catch534__label.is_used()) {
      compiler::CodeAssemblerLabel catch534_skip(&ca_);
      ca_.Goto(&catch534_skip);
      ca_.Bind(&catch534__label, &tmp535);
      ca_.Goto(&block274, phi_bb271_17, phi_bb271_18, phi_bb271_19, phi_bb271_27, phi_bb271_28, phi_bb271_29, phi_bb271_30, phi_bb271_31, phi_bb271_34, phi_bb271_35, phi_bb271_36, phi_bb271_37, phi_bb271_39);
      ca_.Bind(&catch534_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch537__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch537__label);
    tmp536 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp516}, TNode<Int32T>{tmp533});
    }
    if (catch537__label.is_used()) {
      compiler::CodeAssemblerLabel catch537_skip(&ca_);
      ca_.Goto(&catch537_skip);
      ca_.Bind(&catch537__label, &tmp538);
      ca_.Goto(&block275, phi_bb271_17, phi_bb271_18, phi_bb271_19, phi_bb271_27, phi_bb271_28, phi_bb271_29, phi_bb271_30, phi_bb271_31, phi_bb271_34, phi_bb271_35, phi_bb271_36, phi_bb271_37, phi_bb271_39);
      ca_.Bind(&catch537_skip);
    }
    ca_.Goto(&block272, phi_bb271_17, phi_bb271_18, phi_bb271_19, phi_bb271_27, phi_bb271_28, phi_bb271_29, phi_bb271_30, phi_bb271_31, phi_bb271_34, phi_bb271_35, phi_bb271_36, phi_bb271_37, phi_bb271_39, tmp536);
  }

  TNode<Object> phi_bb274_17;
  TNode<IntPtrT> phi_bb274_18;
  TNode<BoolT> phi_bb274_19;
  TNode<IntPtrT> phi_bb274_27;
  TNode<IntPtrT> phi_bb274_28;
  TNode<IntPtrT> phi_bb274_29;
  TNode<IntPtrT> phi_bb274_30;
  TNode<IntPtrT> phi_bb274_31;
  TNode<IntPtrT> phi_bb274_34;
  TNode<BoolT> phi_bb274_35;
  TNode<BoolT> phi_bb274_36;
  TNode<Object> phi_bb274_37;
  TNode<Int32T> phi_bb274_39;
  TNode<HeapObject> tmp539;
  if (block274.is_used()) {
    ca_.Bind(&block274, &phi_bb274_17, &phi_bb274_18, &phi_bb274_19, &phi_bb274_27, &phi_bb274_28, &phi_bb274_29, &phi_bb274_30, &phi_bb274_31, &phi_bb274_34, &phi_bb274_35, &phi_bb274_36, &phi_bb274_37, &phi_bb274_39);
    tmp539 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb274_17, phi_bb274_18, phi_bb274_19, tmp535, tmp539);
  }

  TNode<Object> phi_bb275_17;
  TNode<IntPtrT> phi_bb275_18;
  TNode<BoolT> phi_bb275_19;
  TNode<IntPtrT> phi_bb275_27;
  TNode<IntPtrT> phi_bb275_28;
  TNode<IntPtrT> phi_bb275_29;
  TNode<IntPtrT> phi_bb275_30;
  TNode<IntPtrT> phi_bb275_31;
  TNode<IntPtrT> phi_bb275_34;
  TNode<BoolT> phi_bb275_35;
  TNode<BoolT> phi_bb275_36;
  TNode<Object> phi_bb275_37;
  TNode<Int32T> phi_bb275_39;
  TNode<HeapObject> tmp540;
  if (block275.is_used()) {
    ca_.Bind(&block275, &phi_bb275_17, &phi_bb275_18, &phi_bb275_19, &phi_bb275_27, &phi_bb275_28, &phi_bb275_29, &phi_bb275_30, &phi_bb275_31, &phi_bb275_34, &phi_bb275_35, &phi_bb275_36, &phi_bb275_37, &phi_bb275_39);
    tmp540 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb275_17, phi_bb275_18, phi_bb275_19, tmp538, tmp540);
  }

  TNode<Object> phi_bb272_17;
  TNode<IntPtrT> phi_bb272_18;
  TNode<BoolT> phi_bb272_19;
  TNode<IntPtrT> phi_bb272_27;
  TNode<IntPtrT> phi_bb272_28;
  TNode<IntPtrT> phi_bb272_29;
  TNode<IntPtrT> phi_bb272_30;
  TNode<IntPtrT> phi_bb272_31;
  TNode<IntPtrT> phi_bb272_34;
  TNode<BoolT> phi_bb272_35;
  TNode<BoolT> phi_bb272_36;
  TNode<Object> phi_bb272_37;
  TNode<Int32T> phi_bb272_39;
  TNode<BoolT> phi_bb272_44;
  if (block272.is_used()) {
    ca_.Bind(&block272, &phi_bb272_17, &phi_bb272_18, &phi_bb272_19, &phi_bb272_27, &phi_bb272_28, &phi_bb272_29, &phi_bb272_30, &phi_bb272_31, &phi_bb272_34, &phi_bb272_35, &phi_bb272_36, &phi_bb272_37, &phi_bb272_39, &phi_bb272_44);
    ca_.Branch(phi_bb272_44, &block266, std::vector<compiler::Node*>{phi_bb272_17, phi_bb272_18, phi_bb272_19, phi_bb272_27, phi_bb272_28, phi_bb272_29, phi_bb272_30, phi_bb272_31, phi_bb272_34, phi_bb272_35, phi_bb272_36, phi_bb272_37, phi_bb272_39}, &block267, std::vector<compiler::Node*>{phi_bb272_17, phi_bb272_18, phi_bb272_19, phi_bb272_27, phi_bb272_28, phi_bb272_29, phi_bb272_30, phi_bb272_31, phi_bb272_34, phi_bb272_35, phi_bb272_36, phi_bb272_37, phi_bb272_39});
  }

  TNode<Object> phi_bb267_17;
  TNode<IntPtrT> phi_bb267_18;
  TNode<BoolT> phi_bb267_19;
  TNode<IntPtrT> phi_bb267_27;
  TNode<IntPtrT> phi_bb267_28;
  TNode<IntPtrT> phi_bb267_29;
  TNode<IntPtrT> phi_bb267_30;
  TNode<IntPtrT> phi_bb267_31;
  TNode<IntPtrT> phi_bb267_34;
  TNode<BoolT> phi_bb267_35;
  TNode<BoolT> phi_bb267_36;
  TNode<Object> phi_bb267_37;
  TNode<Int32T> phi_bb267_39;
  if (block267.is_used()) {
    ca_.Bind(&block267, &phi_bb267_17, &phi_bb267_18, &phi_bb267_19, &phi_bb267_27, &phi_bb267_28, &phi_bb267_29, &phi_bb267_30, &phi_bb267_31, &phi_bb267_34, &phi_bb267_35, &phi_bb267_36, &phi_bb267_37, &phi_bb267_39);
    {
      auto pos_stack = ca_.GetMacroSourcePositionStack();
      pos_stack.push_back({"src/builtins/js-to-wasm.tq", 716});
      CodeStubAssembler(state_).FailAssert("Torque assert 'paramKind == ValueKind::kRef || paramKind == ValueKind::kRefNull' failed", pos_stack);
    }
  }

  TNode<Object> phi_bb266_17;
  TNode<IntPtrT> phi_bb266_18;
  TNode<BoolT> phi_bb266_19;
  TNode<IntPtrT> phi_bb266_27;
  TNode<IntPtrT> phi_bb266_28;
  TNode<IntPtrT> phi_bb266_29;
  TNode<IntPtrT> phi_bb266_30;
  TNode<IntPtrT> phi_bb266_31;
  TNode<IntPtrT> phi_bb266_34;
  TNode<BoolT> phi_bb266_35;
  TNode<BoolT> phi_bb266_36;
  TNode<Object> phi_bb266_37;
  TNode<Int32T> phi_bb266_39;
  TNode<BoolT> tmp541;
      TNode<Object> tmp543;
  TNode<BoolT> tmp544;
      TNode<Object> tmp546;
  if (block266.is_used()) {
    ca_.Bind(&block266, &phi_bb266_17, &phi_bb266_18, &phi_bb266_19, &phi_bb266_27, &phi_bb266_28, &phi_bb266_29, &phi_bb266_30, &phi_bb266_31, &phi_bb266_34, &phi_bb266_35, &phi_bb266_36, &phi_bb266_37, &phi_bb266_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 724);
    compiler::CodeAssemblerExceptionHandlerLabel catch542__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch542__label);
    tmp541 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    }
    if (catch542__label.is_used()) {
      compiler::CodeAssemblerLabel catch542_skip(&ca_);
      ca_.Goto(&catch542_skip);
      ca_.Bind(&catch542__label, &tmp543);
      ca_.Goto(&block276, phi_bb266_17, phi_bb266_18, phi_bb266_19, phi_bb266_27, phi_bb266_28, phi_bb266_29, phi_bb266_30, phi_bb266_31, phi_bb266_34, phi_bb266_35, phi_bb266_36, phi_bb266_37, phi_bb266_39);
      ca_.Bind(&catch542_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 725);
    compiler::CodeAssemblerExceptionHandlerLabel catch545__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch545__label);
    tmp544 = CodeStubAssembler(state_).TaggedIsSmi(TNode<Object>{phi_bb266_37});
    }
    if (catch545__label.is_used()) {
      compiler::CodeAssemblerLabel catch545_skip(&ca_);
      ca_.Goto(&catch545_skip);
      ca_.Bind(&catch545__label, &tmp546);
      ca_.Goto(&block279, phi_bb266_17, phi_bb266_18, phi_bb266_19, phi_bb266_27, phi_bb266_28, phi_bb266_29, phi_bb266_30, phi_bb266_31, phi_bb266_34, phi_bb266_35, phi_bb266_37, phi_bb266_39, phi_bb266_37);
      ca_.Bind(&catch545_skip);
    }
    ca_.Branch(tmp544, &block277, std::vector<compiler::Node*>{phi_bb266_17, phi_bb266_18, phi_bb266_19, phi_bb266_27, phi_bb266_28, phi_bb266_29, phi_bb266_30, phi_bb266_31, phi_bb266_34, phi_bb266_35, phi_bb266_37, phi_bb266_39}, &block278, std::vector<compiler::Node*>{phi_bb266_17, phi_bb266_18, phi_bb266_19, phi_bb266_27, phi_bb266_28, phi_bb266_29, phi_bb266_30, phi_bb266_31, phi_bb266_34, phi_bb266_35, phi_bb266_37, phi_bb266_39});
  }

  TNode<Object> phi_bb276_17;
  TNode<IntPtrT> phi_bb276_18;
  TNode<BoolT> phi_bb276_19;
  TNode<IntPtrT> phi_bb276_27;
  TNode<IntPtrT> phi_bb276_28;
  TNode<IntPtrT> phi_bb276_29;
  TNode<IntPtrT> phi_bb276_30;
  TNode<IntPtrT> phi_bb276_31;
  TNode<IntPtrT> phi_bb276_34;
  TNode<BoolT> phi_bb276_35;
  TNode<BoolT> phi_bb276_36;
  TNode<Object> phi_bb276_37;
  TNode<Int32T> phi_bb276_39;
  TNode<HeapObject> tmp547;
  if (block276.is_used()) {
    ca_.Bind(&block276, &phi_bb276_17, &phi_bb276_18, &phi_bb276_19, &phi_bb276_27, &phi_bb276_28, &phi_bb276_29, &phi_bb276_30, &phi_bb276_31, &phi_bb276_34, &phi_bb276_35, &phi_bb276_36, &phi_bb276_37, &phi_bb276_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 724);
    tmp547 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb276_17, phi_bb276_18, phi_bb276_19, tmp543, tmp547);
  }

  TNode<Object> phi_bb279_17;
  TNode<IntPtrT> phi_bb279_18;
  TNode<BoolT> phi_bb279_19;
  TNode<IntPtrT> phi_bb279_27;
  TNode<IntPtrT> phi_bb279_28;
  TNode<IntPtrT> phi_bb279_29;
  TNode<IntPtrT> phi_bb279_30;
  TNode<IntPtrT> phi_bb279_31;
  TNode<IntPtrT> phi_bb279_34;
  TNode<BoolT> phi_bb279_35;
  TNode<Object> phi_bb279_37;
  TNode<Int32T> phi_bb279_39;
  TNode<Object> phi_bb279_43;
  TNode<HeapObject> tmp548;
  if (block279.is_used()) {
    ca_.Bind(&block279, &phi_bb279_17, &phi_bb279_18, &phi_bb279_19, &phi_bb279_27, &phi_bb279_28, &phi_bb279_29, &phi_bb279_30, &phi_bb279_31, &phi_bb279_34, &phi_bb279_35, &phi_bb279_37, &phi_bb279_39, &phi_bb279_43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 725);
    tmp548 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb279_17, phi_bb279_18, phi_bb279_19, tmp546, tmp548);
  }

  TNode<Object> phi_bb277_17;
  TNode<IntPtrT> phi_bb277_18;
  TNode<BoolT> phi_bb277_19;
  TNode<IntPtrT> phi_bb277_27;
  TNode<IntPtrT> phi_bb277_28;
  TNode<IntPtrT> phi_bb277_29;
  TNode<IntPtrT> phi_bb277_30;
  TNode<IntPtrT> phi_bb277_31;
  TNode<IntPtrT> phi_bb277_34;
  TNode<BoolT> phi_bb277_35;
  TNode<Object> phi_bb277_37;
  TNode<Int32T> phi_bb277_39;
  TNode<IntPtrT> tmp549;
      TNode<Object> tmp551;
  TNode<FixedArray> tmp552;
      TNode<Object> tmp554;
  if (block277.is_used()) {
    ca_.Bind(&block277, &phi_bb277_17, &phi_bb277_18, &phi_bb277_19, &phi_bb277_27, &phi_bb277_28, &phi_bb277_29, &phi_bb277_30, &phi_bb277_31, &phi_bb277_34, &phi_bb277_35, &phi_bb277_37, &phi_bb277_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 727);
    compiler::CodeAssemblerExceptionHandlerLabel catch550__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch550__label);
    tmp549 = Convert_intptr_int32_0(state_, TNode<Int32T>{tmp26});
    }
    if (catch550__label.is_used()) {
      compiler::CodeAssemblerLabel catch550_skip(&ca_);
      ca_.Goto(&catch550_skip);
      ca_.Bind(&catch550__label, &tmp551);
      ca_.Goto(&block280, phi_bb277_17, phi_bb277_18, phi_bb277_19, phi_bb277_27, phi_bb277_28, phi_bb277_29, phi_bb277_30, phi_bb277_31, phi_bb277_34, phi_bb277_35, phi_bb277_37, phi_bb277_39);
      ca_.Bind(&catch550_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch553__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch553__label);
    tmp552 = ca_.CallBuiltin<FixedArray>(Builtin::kWasmAllocateZeroedFixedArray, TNode<Object>(), tmp549);
    }
    if (catch553__label.is_used()) {
      compiler::CodeAssemblerLabel catch553_skip(&ca_);
      ca_.Goto(&catch553_skip);
      ca_.Bind(&catch553__label, &tmp554);
      ca_.Goto(&block281, phi_bb277_17, phi_bb277_18, phi_bb277_19, phi_bb277_27, phi_bb277_28, phi_bb277_29, phi_bb277_30, phi_bb277_31, phi_bb277_34, phi_bb277_35, phi_bb277_37, phi_bb277_39);
      ca_.Bind(&catch553_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 725);
    ca_.Goto(&block278, phi_bb277_17, phi_bb277_18, phi_bb277_19, phi_bb277_27, phi_bb277_28, phi_bb277_29, phi_bb277_30, phi_bb277_31, phi_bb277_34, phi_bb277_35, tmp552, phi_bb277_39);
  }

  TNode<Object> phi_bb280_17;
  TNode<IntPtrT> phi_bb280_18;
  TNode<BoolT> phi_bb280_19;
  TNode<IntPtrT> phi_bb280_27;
  TNode<IntPtrT> phi_bb280_28;
  TNode<IntPtrT> phi_bb280_29;
  TNode<IntPtrT> phi_bb280_30;
  TNode<IntPtrT> phi_bb280_31;
  TNode<IntPtrT> phi_bb280_34;
  TNode<BoolT> phi_bb280_35;
  TNode<Object> phi_bb280_37;
  TNode<Int32T> phi_bb280_39;
  TNode<HeapObject> tmp555;
  if (block280.is_used()) {
    ca_.Bind(&block280, &phi_bb280_17, &phi_bb280_18, &phi_bb280_19, &phi_bb280_27, &phi_bb280_28, &phi_bb280_29, &phi_bb280_30, &phi_bb280_31, &phi_bb280_34, &phi_bb280_35, &phi_bb280_37, &phi_bb280_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 727);
    tmp555 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb280_17, phi_bb280_18, phi_bb280_19, tmp551, tmp555);
  }

  TNode<Object> phi_bb281_17;
  TNode<IntPtrT> phi_bb281_18;
  TNode<BoolT> phi_bb281_19;
  TNode<IntPtrT> phi_bb281_27;
  TNode<IntPtrT> phi_bb281_28;
  TNode<IntPtrT> phi_bb281_29;
  TNode<IntPtrT> phi_bb281_30;
  TNode<IntPtrT> phi_bb281_31;
  TNode<IntPtrT> phi_bb281_34;
  TNode<BoolT> phi_bb281_35;
  TNode<Object> phi_bb281_37;
  TNode<Int32T> phi_bb281_39;
  TNode<HeapObject> tmp556;
  if (block281.is_used()) {
    ca_.Bind(&block281, &phi_bb281_17, &phi_bb281_18, &phi_bb281_19, &phi_bb281_27, &phi_bb281_28, &phi_bb281_29, &phi_bb281_30, &phi_bb281_31, &phi_bb281_34, &phi_bb281_35, &phi_bb281_37, &phi_bb281_39);
    tmp556 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb281_17, phi_bb281_18, phi_bb281_19, tmp554, tmp556);
  }

  TNode<Object> phi_bb278_17;
  TNode<IntPtrT> phi_bb278_18;
  TNode<BoolT> phi_bb278_19;
  TNode<IntPtrT> phi_bb278_27;
  TNode<IntPtrT> phi_bb278_28;
  TNode<IntPtrT> phi_bb278_29;
  TNode<IntPtrT> phi_bb278_30;
  TNode<IntPtrT> phi_bb278_31;
  TNode<IntPtrT> phi_bb278_34;
  TNode<BoolT> phi_bb278_35;
  TNode<Object> phi_bb278_37;
  TNode<Int32T> phi_bb278_39;
  TNode<FixedArray> tmp557;
      TNode<Object> tmp559;
  TNode<Object> tmp560;
  TNode<IntPtrT> tmp561;
  TNode<IntPtrT> tmp562;
      TNode<Object> tmp564;
  TNode<IntPtrT> tmp565;
      TNode<Object> tmp567;
  TNode<UintPtrT> tmp568;
  TNode<UintPtrT> tmp569;
  TNode<BoolT> tmp570;
  if (block278.is_used()) {
    ca_.Bind(&block278, &phi_bb278_17, &phi_bb278_18, &phi_bb278_19, &phi_bb278_27, &phi_bb278_28, &phi_bb278_29, &phi_bb278_30, &phi_bb278_31, &phi_bb278_34, &phi_bb278_35, &phi_bb278_37, &phi_bb278_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 729);
    compiler::CodeAssemblerExceptionHandlerLabel catch558__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch558__label);
    tmp557 = UnsafeCast_FixedArray_0(state_, TNode<Context>{p_context}, TNode<Object>{phi_bb278_37});
    }
    if (catch558__label.is_used()) {
      compiler::CodeAssemblerLabel catch558_skip(&ca_);
      ca_.Goto(&catch558_skip);
      ca_.Bind(&catch558__label, &tmp559);
      ca_.Goto(&block282, phi_bb278_17, phi_bb278_18, phi_bb278_19, phi_bb278_27, phi_bb278_28, phi_bb278_29, phi_bb278_30, phi_bb278_31, phi_bb278_34, phi_bb278_35, phi_bb278_39);
      ca_.Bind(&catch558_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch563__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch563__label);
    std::tie(tmp560, tmp561, tmp562) = FieldSliceFixedArrayObjects_0(state_, TNode<FixedArray>{tmp557}).Flatten();
    }
    if (catch563__label.is_used()) {
      compiler::CodeAssemblerLabel catch563_skip(&ca_);
      ca_.Goto(&catch563_skip);
      ca_.Bind(&catch563__label, &tmp564);
      ca_.Goto(&block283, phi_bb278_17, phi_bb278_18, phi_bb278_19, phi_bb278_27, phi_bb278_28, phi_bb278_29, phi_bb278_30, phi_bb278_31, phi_bb278_34, phi_bb278_35, phi_bb278_39);
      ca_.Bind(&catch563_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 730);
    compiler::CodeAssemblerExceptionHandlerLabel catch566__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch566__label);
    tmp565 = Convert_intptr_int32_0(state_, TNode<Int32T>{phi_bb278_39});
    }
    if (catch566__label.is_used()) {
      compiler::CodeAssemblerLabel catch566_skip(&ca_);
      ca_.Goto(&catch566_skip);
      ca_.Bind(&catch566__label, &tmp567);
      ca_.Goto(&block284, phi_bb278_17, phi_bb278_18, phi_bb278_19, phi_bb278_27, phi_bb278_28, phi_bb278_29, phi_bb278_30, phi_bb278_31, phi_bb278_34, phi_bb278_35, phi_bb278_39, phi_bb278_39);
      ca_.Bind(&catch566_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 101);
    tmp568 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{tmp565});
    tmp569 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{tmp562});
    tmp570 = CodeStubAssembler(state_).UintPtrLessThan(TNode<UintPtrT>{tmp568}, TNode<UintPtrT>{tmp569});
    ca_.Branch(tmp570, &block289, std::vector<compiler::Node*>{phi_bb278_17, phi_bb278_18, phi_bb278_19, phi_bb278_27, phi_bb278_28, phi_bb278_29, phi_bb278_30, phi_bb278_31, phi_bb278_34, phi_bb278_35, phi_bb278_39}, &block290, std::vector<compiler::Node*>{phi_bb278_17, phi_bb278_18, phi_bb278_19, phi_bb278_27, phi_bb278_28, phi_bb278_29, phi_bb278_30, phi_bb278_31, phi_bb278_34, phi_bb278_35, phi_bb278_39});
  }

  TNode<Object> phi_bb282_17;
  TNode<IntPtrT> phi_bb282_18;
  TNode<BoolT> phi_bb282_19;
  TNode<IntPtrT> phi_bb282_27;
  TNode<IntPtrT> phi_bb282_28;
  TNode<IntPtrT> phi_bb282_29;
  TNode<IntPtrT> phi_bb282_30;
  TNode<IntPtrT> phi_bb282_31;
  TNode<IntPtrT> phi_bb282_34;
  TNode<BoolT> phi_bb282_35;
  TNode<Int32T> phi_bb282_39;
  TNode<HeapObject> tmp571;
  if (block282.is_used()) {
    ca_.Bind(&block282, &phi_bb282_17, &phi_bb282_18, &phi_bb282_19, &phi_bb282_27, &phi_bb282_28, &phi_bb282_29, &phi_bb282_30, &phi_bb282_31, &phi_bb282_34, &phi_bb282_35, &phi_bb282_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 729);
    tmp571 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb282_17, phi_bb282_18, phi_bb282_19, tmp559, tmp571);
  }

  TNode<Object> phi_bb283_17;
  TNode<IntPtrT> phi_bb283_18;
  TNode<BoolT> phi_bb283_19;
  TNode<IntPtrT> phi_bb283_27;
  TNode<IntPtrT> phi_bb283_28;
  TNode<IntPtrT> phi_bb283_29;
  TNode<IntPtrT> phi_bb283_30;
  TNode<IntPtrT> phi_bb283_31;
  TNode<IntPtrT> phi_bb283_34;
  TNode<BoolT> phi_bb283_35;
  TNode<Int32T> phi_bb283_39;
  TNode<HeapObject> tmp572;
  if (block283.is_used()) {
    ca_.Bind(&block283, &phi_bb283_17, &phi_bb283_18, &phi_bb283_19, &phi_bb283_27, &phi_bb283_28, &phi_bb283_29, &phi_bb283_30, &phi_bb283_31, &phi_bb283_34, &phi_bb283_35, &phi_bb283_39);
    tmp572 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb283_17, phi_bb283_18, phi_bb283_19, tmp564, tmp572);
  }

  TNode<Object> phi_bb284_17;
  TNode<IntPtrT> phi_bb284_18;
  TNode<BoolT> phi_bb284_19;
  TNode<IntPtrT> phi_bb284_27;
  TNode<IntPtrT> phi_bb284_28;
  TNode<IntPtrT> phi_bb284_29;
  TNode<IntPtrT> phi_bb284_30;
  TNode<IntPtrT> phi_bb284_31;
  TNode<IntPtrT> phi_bb284_34;
  TNode<BoolT> phi_bb284_35;
  TNode<Int32T> phi_bb284_39;
  TNode<Int32T> phi_bb284_48;
  TNode<HeapObject> tmp573;
  if (block284.is_used()) {
    ca_.Bind(&block284, &phi_bb284_17, &phi_bb284_18, &phi_bb284_19, &phi_bb284_27, &phi_bb284_28, &phi_bb284_29, &phi_bb284_30, &phi_bb284_31, &phi_bb284_34, &phi_bb284_35, &phi_bb284_39, &phi_bb284_48);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 730);
    tmp573 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb284_17, phi_bb284_18, phi_bb284_19, tmp567, tmp573);
  }

  TNode<Object> phi_bb289_17;
  TNode<IntPtrT> phi_bb289_18;
  TNode<BoolT> phi_bb289_19;
  TNode<IntPtrT> phi_bb289_27;
  TNode<IntPtrT> phi_bb289_28;
  TNode<IntPtrT> phi_bb289_29;
  TNode<IntPtrT> phi_bb289_30;
  TNode<IntPtrT> phi_bb289_31;
  TNode<IntPtrT> phi_bb289_34;
  TNode<BoolT> phi_bb289_35;
  TNode<Int32T> phi_bb289_39;
  TNode<IntPtrT> tmp574;
  TNode<IntPtrT> tmp575;
  TNode<Object> tmp576;
  TNode<IntPtrT> tmp577;
  TNode<Object> tmp578;
      TNode<Object> tmp580;
  if (block289.is_used()) {
    ca_.Bind(&block289, &phi_bb289_17, &phi_bb289_18, &phi_bb289_19, &phi_bb289_27, &phi_bb289_28, &phi_bb289_29, &phi_bb289_30, &phi_bb289_31, &phi_bb289_34, &phi_bb289_35, &phi_bb289_39);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp574 = TimesSizeOf_Object_0(state_, TNode<IntPtrT>{tmp565});
    tmp575 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp561}, TNode<IntPtrT>{tmp574});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp576, tmp577) = NewReference_Object_0(state_, TNode<Object>{tmp560}, TNode<IntPtrT>{tmp575}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 731);
    compiler::CodeAssemblerExceptionHandlerLabel catch579__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch579__label);
    tmp578 = JSToWasmObject_0(state_, TNode<NativeContext>{p_context}, TNode<HeapObject>{tmp21}, TNode<Int32T>{tmp250}, TNode<Object>{tmp234});
    }
    if (catch579__label.is_used()) {
      compiler::CodeAssemblerLabel catch579_skip(&ca_);
      ca_.Goto(&catch579_skip);
      ca_.Bind(&catch579__label, &tmp580);
      ca_.Goto(&block293, phi_bb289_17, phi_bb289_18, phi_bb289_19, phi_bb289_27, phi_bb289_28, phi_bb289_29, phi_bb289_30, phi_bb289_31, phi_bb289_34, phi_bb289_35, phi_bb289_39);
      ca_.Bind(&catch579_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 729);
    CodeStubAssembler(state_).StoreReference<Object>(CodeStubAssembler::Reference{tmp576, tmp577}, tmp578);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 701);
    ca_.Goto(&block228, phi_bb289_17, phi_bb289_18, phi_bb289_19, phi_bb289_27, phi_bb289_28, phi_bb289_29, phi_bb289_30, phi_bb289_31, phi_bb289_34, phi_bb289_35, tmp541, phi_bb278_37, phi_bb289_39);
  }

  TNode<Object> phi_bb290_17;
  TNode<IntPtrT> phi_bb290_18;
  TNode<BoolT> phi_bb290_19;
  TNode<IntPtrT> phi_bb290_27;
  TNode<IntPtrT> phi_bb290_28;
  TNode<IntPtrT> phi_bb290_29;
  TNode<IntPtrT> phi_bb290_30;
  TNode<IntPtrT> phi_bb290_31;
  TNode<IntPtrT> phi_bb290_34;
  TNode<BoolT> phi_bb290_35;
  TNode<Int32T> phi_bb290_39;
  if (block290.is_used()) {
    ca_.Bind(&block290, &phi_bb290_17, &phi_bb290_18, &phi_bb290_19, &phi_bb290_27, &phi_bb290_28, &phi_bb290_29, &phi_bb290_30, &phi_bb290_31, &phi_bb290_34, &phi_bb290_35, &phi_bb290_39);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 113);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<Object> phi_bb293_17;
  TNode<IntPtrT> phi_bb293_18;
  TNode<BoolT> phi_bb293_19;
  TNode<IntPtrT> phi_bb293_27;
  TNode<IntPtrT> phi_bb293_28;
  TNode<IntPtrT> phi_bb293_29;
  TNode<IntPtrT> phi_bb293_30;
  TNode<IntPtrT> phi_bb293_31;
  TNode<IntPtrT> phi_bb293_34;
  TNode<BoolT> phi_bb293_35;
  TNode<Int32T> phi_bb293_39;
  TNode<HeapObject> tmp581;
  if (block293.is_used()) {
    ca_.Bind(&block293, &phi_bb293_17, &phi_bb293_18, &phi_bb293_19, &phi_bb293_27, &phi_bb293_28, &phi_bb293_29, &phi_bb293_30, &phi_bb293_31, &phi_bb293_34, &phi_bb293_35, &phi_bb293_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 731);
    tmp581 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb293_17, phi_bb293_18, phi_bb293_19, tmp580, tmp581);
  }

  TNode<Object> phi_bb228_17;
  TNode<IntPtrT> phi_bb228_18;
  TNode<BoolT> phi_bb228_19;
  TNode<IntPtrT> phi_bb228_27;
  TNode<IntPtrT> phi_bb228_28;
  TNode<IntPtrT> phi_bb228_29;
  TNode<IntPtrT> phi_bb228_30;
  TNode<IntPtrT> phi_bb228_31;
  TNode<IntPtrT> phi_bb228_34;
  TNode<BoolT> phi_bb228_35;
  TNode<BoolT> phi_bb228_36;
  TNode<Object> phi_bb228_37;
  TNode<Int32T> phi_bb228_39;
  if (block228.is_used()) {
    ca_.Bind(&block228, &phi_bb228_17, &phi_bb228_18, &phi_bb228_19, &phi_bb228_27, &phi_bb228_28, &phi_bb228_29, &phi_bb228_30, &phi_bb228_31, &phi_bb228_34, &phi_bb228_35, &phi_bb228_36, &phi_bb228_37, &phi_bb228_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 698);
    ca_.Goto(&block208, phi_bb228_17, phi_bb228_18, phi_bb228_19, phi_bb228_27, phi_bb228_28, phi_bb228_29, phi_bb228_30, phi_bb228_31, phi_bb228_34, phi_bb228_35, phi_bb228_36, phi_bb228_37, phi_bb228_39);
  }

  TNode<Object> phi_bb208_17;
  TNode<IntPtrT> phi_bb208_18;
  TNode<BoolT> phi_bb208_19;
  TNode<IntPtrT> phi_bb208_27;
  TNode<IntPtrT> phi_bb208_28;
  TNode<IntPtrT> phi_bb208_29;
  TNode<IntPtrT> phi_bb208_30;
  TNode<IntPtrT> phi_bb208_31;
  TNode<IntPtrT> phi_bb208_34;
  TNode<BoolT> phi_bb208_35;
  TNode<BoolT> phi_bb208_36;
  TNode<Object> phi_bb208_37;
  TNode<Int32T> phi_bb208_39;
  if (block208.is_used()) {
    ca_.Bind(&block208, &phi_bb208_17, &phi_bb208_18, &phi_bb208_19, &phi_bb208_27, &phi_bb208_28, &phi_bb208_29, &phi_bb208_30, &phi_bb208_31, &phi_bb208_34, &phi_bb208_35, &phi_bb208_36, &phi_bb208_37, &phi_bb208_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 691);
    ca_.Goto(&block187, phi_bb208_17, phi_bb208_18, phi_bb208_19, phi_bb208_27, phi_bb208_28, phi_bb208_29, phi_bb208_30, phi_bb208_31, phi_bb208_34, phi_bb208_35, phi_bb208_36, phi_bb208_37, phi_bb208_39);
  }

  TNode<Object> phi_bb187_17;
  TNode<IntPtrT> phi_bb187_18;
  TNode<BoolT> phi_bb187_19;
  TNode<IntPtrT> phi_bb187_27;
  TNode<IntPtrT> phi_bb187_28;
  TNode<IntPtrT> phi_bb187_29;
  TNode<IntPtrT> phi_bb187_30;
  TNode<IntPtrT> phi_bb187_31;
  TNode<IntPtrT> phi_bb187_34;
  TNode<BoolT> phi_bb187_35;
  TNode<BoolT> phi_bb187_36;
  TNode<Object> phi_bb187_37;
  TNode<Int32T> phi_bb187_39;
  if (block187.is_used()) {
    ca_.Bind(&block187, &phi_bb187_17, &phi_bb187_18, &phi_bb187_19, &phi_bb187_27, &phi_bb187_28, &phi_bb187_29, &phi_bb187_30, &phi_bb187_31, &phi_bb187_34, &phi_bb187_35, &phi_bb187_36, &phi_bb187_37, &phi_bb187_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 681);
    ca_.Goto(&block164, phi_bb187_17, phi_bb187_18, phi_bb187_19, phi_bb187_27, phi_bb187_28, phi_bb187_29, phi_bb187_30, phi_bb187_31, phi_bb187_34, phi_bb187_35, phi_bb187_36, phi_bb187_37, phi_bb187_39);
  }

  TNode<Object> phi_bb164_17;
  TNode<IntPtrT> phi_bb164_18;
  TNode<BoolT> phi_bb164_19;
  TNode<IntPtrT> phi_bb164_27;
  TNode<IntPtrT> phi_bb164_28;
  TNode<IntPtrT> phi_bb164_29;
  TNode<IntPtrT> phi_bb164_30;
  TNode<IntPtrT> phi_bb164_31;
  TNode<IntPtrT> phi_bb164_34;
  TNode<BoolT> phi_bb164_35;
  TNode<BoolT> phi_bb164_36;
  TNode<Object> phi_bb164_37;
  TNode<Int32T> phi_bb164_39;
  TNode<Int32T> tmp582;
      TNode<Object> tmp584;
  TNode<Int32T> tmp585;
      TNode<Object> tmp587;
  if (block164.is_used()) {
    ca_.Bind(&block164, &phi_bb164_17, &phi_bb164_18, &phi_bb164_19, &phi_bb164_27, &phi_bb164_28, &phi_bb164_29, &phi_bb164_30, &phi_bb164_31, &phi_bb164_34, &phi_bb164_35, &phi_bb164_36, &phi_bb164_37, &phi_bb164_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 677);
    compiler::CodeAssemblerExceptionHandlerLabel catch583__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch583__label);
    tmp582 = FromConstexpr_int32_constexpr_int31_0(state_, 1);
    }
    if (catch583__label.is_used()) {
      compiler::CodeAssemblerLabel catch583_skip(&ca_);
      ca_.Goto(&catch583_skip);
      ca_.Bind(&catch583__label, &tmp584);
      ca_.Goto(&block294, phi_bb164_17, phi_bb164_18, phi_bb164_19, phi_bb164_27, phi_bb164_28, phi_bb164_29, phi_bb164_30, phi_bb164_31, phi_bb164_34, phi_bb164_35, phi_bb164_36, phi_bb164_37, phi_bb164_39, phi_bb164_39, phi_bb164_39);
      ca_.Bind(&catch583_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch586__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch586__label);
    tmp585 = CodeStubAssembler(state_).Int32Add(TNode<Int32T>{phi_bb164_39}, TNode<Int32T>{tmp582});
    }
    if (catch586__label.is_used()) {
      compiler::CodeAssemblerLabel catch586_skip(&ca_);
      ca_.Goto(&catch586_skip);
      ca_.Bind(&catch586__label, &tmp587);
      ca_.Goto(&block295, phi_bb164_17, phi_bb164_18, phi_bb164_19, phi_bb164_27, phi_bb164_28, phi_bb164_29, phi_bb164_30, phi_bb164_31, phi_bb164_34, phi_bb164_35, phi_bb164_36, phi_bb164_37, phi_bb164_39, phi_bb164_39);
      ca_.Bind(&catch586_skip);
    }
    ca_.Goto(&block151, phi_bb164_17, phi_bb164_18, phi_bb164_19, phi_bb164_27, phi_bb164_28, phi_bb164_29, phi_bb164_30, phi_bb164_31, phi_bb164_34, phi_bb164_35, phi_bb164_36, phi_bb164_37, tmp240, tmp585);
  }

  TNode<Object> phi_bb294_17;
  TNode<IntPtrT> phi_bb294_18;
  TNode<BoolT> phi_bb294_19;
  TNode<IntPtrT> phi_bb294_27;
  TNode<IntPtrT> phi_bb294_28;
  TNode<IntPtrT> phi_bb294_29;
  TNode<IntPtrT> phi_bb294_30;
  TNode<IntPtrT> phi_bb294_31;
  TNode<IntPtrT> phi_bb294_34;
  TNode<BoolT> phi_bb294_35;
  TNode<BoolT> phi_bb294_36;
  TNode<Object> phi_bb294_37;
  TNode<Int32T> phi_bb294_39;
  TNode<Int32T> phi_bb294_40;
  TNode<Int32T> phi_bb294_41;
  TNode<HeapObject> tmp588;
  if (block294.is_used()) {
    ca_.Bind(&block294, &phi_bb294_17, &phi_bb294_18, &phi_bb294_19, &phi_bb294_27, &phi_bb294_28, &phi_bb294_29, &phi_bb294_30, &phi_bb294_31, &phi_bb294_34, &phi_bb294_35, &phi_bb294_36, &phi_bb294_37, &phi_bb294_39, &phi_bb294_40, &phi_bb294_41);
    tmp588 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb294_17, phi_bb294_18, phi_bb294_19, tmp584, tmp588);
  }

  TNode<Object> phi_bb295_17;
  TNode<IntPtrT> phi_bb295_18;
  TNode<BoolT> phi_bb295_19;
  TNode<IntPtrT> phi_bb295_27;
  TNode<IntPtrT> phi_bb295_28;
  TNode<IntPtrT> phi_bb295_29;
  TNode<IntPtrT> phi_bb295_30;
  TNode<IntPtrT> phi_bb295_31;
  TNode<IntPtrT> phi_bb295_34;
  TNode<BoolT> phi_bb295_35;
  TNode<BoolT> phi_bb295_36;
  TNode<Object> phi_bb295_37;
  TNode<Int32T> phi_bb295_39;
  TNode<Int32T> phi_bb295_40;
  TNode<HeapObject> tmp589;
  if (block295.is_used()) {
    ca_.Bind(&block295, &phi_bb295_17, &phi_bb295_18, &phi_bb295_19, &phi_bb295_27, &phi_bb295_28, &phi_bb295_29, &phi_bb295_30, &phi_bb295_31, &phi_bb295_34, &phi_bb295_35, &phi_bb295_36, &phi_bb295_37, &phi_bb295_39, &phi_bb295_40);
    tmp589 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb295_17, phi_bb295_18, phi_bb295_19, tmp587, tmp589);
  }

  TNode<Object> phi_bb150_17;
  TNode<IntPtrT> phi_bb150_18;
  TNode<BoolT> phi_bb150_19;
  TNode<IntPtrT> phi_bb150_27;
  TNode<IntPtrT> phi_bb150_28;
  TNode<IntPtrT> phi_bb150_29;
  TNode<IntPtrT> phi_bb150_30;
  TNode<IntPtrT> phi_bb150_31;
  TNode<IntPtrT> phi_bb150_34;
  TNode<BoolT> phi_bb150_35;
  TNode<BoolT> phi_bb150_36;
  TNode<Object> phi_bb150_37;
  TNode<Int32T> phi_bb150_38;
  TNode<Int32T> phi_bb150_39;
  TNode<Undefined> tmp590;
  if (block150.is_used()) {
    ca_.Bind(&block150, &phi_bb150_17, &phi_bb150_18, &phi_bb150_19, &phi_bb150_27, &phi_bb150_28, &phi_bb150_29, &phi_bb150_30, &phi_bb150_31, &phi_bb150_34, &phi_bb150_35, &phi_bb150_36, &phi_bb150_37, &phi_bb150_38, &phi_bb150_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 734);
    tmp590 = Undefined_0(state_);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 735);
    if ((((CodeStubAssembler(state_).ConstexprInt32Equal(p_promise, wasm::Promise::kPromise)) || (CodeStubAssembler(state_).ConstexprInt32Equal(p_promise, wasm::Promise::kStressSwitch))))) {
      ca_.Goto(&block296, phi_bb150_17, phi_bb150_18, phi_bb150_19, phi_bb150_27, phi_bb150_28, phi_bb150_29, phi_bb150_30, phi_bb150_31, phi_bb150_34, phi_bb150_35, phi_bb150_36, phi_bb150_37, phi_bb150_38);
    } else {
      ca_.Goto(&block297, phi_bb150_17, phi_bb150_18, phi_bb150_19, phi_bb150_27, phi_bb150_28, phi_bb150_29, phi_bb150_30, phi_bb150_31, phi_bb150_34, phi_bb150_35, phi_bb150_36, phi_bb150_37, phi_bb150_38);
    }
  }

  TNode<Object> phi_bb296_17;
  TNode<IntPtrT> phi_bb296_18;
  TNode<BoolT> phi_bb296_19;
  TNode<IntPtrT> phi_bb296_27;
  TNode<IntPtrT> phi_bb296_28;
  TNode<IntPtrT> phi_bb296_29;
  TNode<IntPtrT> phi_bb296_30;
  TNode<IntPtrT> phi_bb296_31;
  TNode<IntPtrT> phi_bb296_34;
  TNode<BoolT> phi_bb296_35;
  TNode<BoolT> phi_bb296_36;
  TNode<Object> phi_bb296_37;
  TNode<Int32T> phi_bb296_38;
  TNode<Object> tmp591;
      TNode<Object> tmp593;
  TNode<BoolT> tmp594;
      TNode<Object> tmp596;
  if (block296.is_used()) {
    ca_.Bind(&block296, &phi_bb296_17, &phi_bb296_18, &phi_bb296_19, &phi_bb296_27, &phi_bb296_28, &phi_bb296_29, &phi_bb296_30, &phi_bb296_31, &phi_bb296_34, &phi_bb296_35, &phi_bb296_36, &phi_bb296_37, &phi_bb296_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 736);
    compiler::CodeAssemblerExceptionHandlerLabel catch592__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch592__label);
    tmp591 = CodeStubAssembler(state_).CallRuntime(Runtime::kWasmAllocateSuspender, p_context); 
    }
    if (catch592__label.is_used()) {
      compiler::CodeAssemblerLabel catch592_skip(&ca_);
      ca_.Goto(&catch592_skip);
      ca_.Bind(&catch592__label, &tmp593);
      ca_.Goto(&block299, phi_bb296_17, phi_bb296_18, phi_bb296_19, phi_bb296_27, phi_bb296_28, phi_bb296_29, phi_bb296_30, phi_bb296_31, phi_bb296_34, phi_bb296_35, phi_bb296_36, phi_bb296_37, phi_bb296_38);
      ca_.Bind(&catch592_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 737);
    compiler::CodeAssemblerExceptionHandlerLabel catch595__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch595__label);
    tmp594 = FromConstexpr_bool_constexpr_bool_0(state_, (CodeStubAssembler(state_).ConstexprInt32Equal(p_promise, wasm::Promise::kStressSwitch)));
    }
    if (catch595__label.is_used()) {
      compiler::CodeAssemblerLabel catch595_skip(&ca_);
      ca_.Goto(&catch595_skip);
      ca_.Bind(&catch595__label, &tmp596);
      ca_.Goto(&block302, phi_bb296_17, phi_bb296_18, phi_bb296_19, phi_bb296_27, phi_bb296_28, phi_bb296_29, phi_bb296_30, phi_bb296_31, phi_bb296_34, phi_bb296_35, phi_bb296_36, phi_bb296_37, phi_bb296_38);
      ca_.Bind(&catch595_skip);
    }
    ca_.Branch(tmp594, &block300, std::vector<compiler::Node*>{phi_bb296_17, phi_bb296_18, phi_bb296_19, phi_bb296_27, phi_bb296_28, phi_bb296_29, phi_bb296_30, phi_bb296_31, phi_bb296_34, phi_bb296_35, phi_bb296_36, phi_bb296_37, phi_bb296_38}, &block301, std::vector<compiler::Node*>{phi_bb296_17, phi_bb296_18, phi_bb296_19, phi_bb296_27, phi_bb296_28, phi_bb296_29, phi_bb296_30, phi_bb296_31, phi_bb296_34, phi_bb296_35, phi_bb296_36, phi_bb296_37, phi_bb296_38});
  }

  TNode<Object> phi_bb299_17;
  TNode<IntPtrT> phi_bb299_18;
  TNode<BoolT> phi_bb299_19;
  TNode<IntPtrT> phi_bb299_27;
  TNode<IntPtrT> phi_bb299_28;
  TNode<IntPtrT> phi_bb299_29;
  TNode<IntPtrT> phi_bb299_30;
  TNode<IntPtrT> phi_bb299_31;
  TNode<IntPtrT> phi_bb299_34;
  TNode<BoolT> phi_bb299_35;
  TNode<BoolT> phi_bb299_36;
  TNode<Object> phi_bb299_37;
  TNode<Int32T> phi_bb299_38;
  TNode<HeapObject> tmp597;
  if (block299.is_used()) {
    ca_.Bind(&block299, &phi_bb299_17, &phi_bb299_18, &phi_bb299_19, &phi_bb299_27, &phi_bb299_28, &phi_bb299_29, &phi_bb299_30, &phi_bb299_31, &phi_bb299_34, &phi_bb299_35, &phi_bb299_36, &phi_bb299_37, &phi_bb299_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 736);
    tmp597 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb299_17, phi_bb299_18, phi_bb299_19, tmp593, tmp597);
  }

  TNode<Object> phi_bb302_17;
  TNode<IntPtrT> phi_bb302_18;
  TNode<BoolT> phi_bb302_19;
  TNode<IntPtrT> phi_bb302_27;
  TNode<IntPtrT> phi_bb302_28;
  TNode<IntPtrT> phi_bb302_29;
  TNode<IntPtrT> phi_bb302_30;
  TNode<IntPtrT> phi_bb302_31;
  TNode<IntPtrT> phi_bb302_34;
  TNode<BoolT> phi_bb302_35;
  TNode<BoolT> phi_bb302_36;
  TNode<Object> phi_bb302_37;
  TNode<Int32T> phi_bb302_38;
  TNode<HeapObject> tmp598;
  if (block302.is_used()) {
    ca_.Bind(&block302, &phi_bb302_17, &phi_bb302_18, &phi_bb302_19, &phi_bb302_27, &phi_bb302_28, &phi_bb302_29, &phi_bb302_30, &phi_bb302_31, &phi_bb302_34, &phi_bb302_35, &phi_bb302_36, &phi_bb302_37, &phi_bb302_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 737);
    tmp598 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb302_17, phi_bb302_18, phi_bb302_19, tmp596, tmp598);
  }

  TNode<Object> phi_bb300_17;
  TNode<IntPtrT> phi_bb300_18;
  TNode<BoolT> phi_bb300_19;
  TNode<IntPtrT> phi_bb300_27;
  TNode<IntPtrT> phi_bb300_28;
  TNode<IntPtrT> phi_bb300_29;
  TNode<IntPtrT> phi_bb300_30;
  TNode<IntPtrT> phi_bb300_31;
  TNode<IntPtrT> phi_bb300_34;
  TNode<BoolT> phi_bb300_35;
  TNode<BoolT> phi_bb300_36;
  TNode<Object> phi_bb300_37;
  TNode<Int32T> phi_bb300_38;
  TNode<WasmSuspenderObject> tmp599;
      TNode<Object> tmp601;
  TNode<IntPtrT> tmp602;
      TNode<Object> tmp604;
  TNode<Undefined> tmp605;
  if (block300.is_used()) {
    ca_.Bind(&block300, &phi_bb300_17, &phi_bb300_18, &phi_bb300_19, &phi_bb300_27, &phi_bb300_28, &phi_bb300_29, &phi_bb300_30, &phi_bb300_31, &phi_bb300_34, &phi_bb300_35, &phi_bb300_36, &phi_bb300_37, &phi_bb300_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 741);
    compiler::CodeAssemblerExceptionHandlerLabel catch600__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch600__label);
    tmp599 = UnsafeCast_WasmSuspenderObject_0(state_, TNode<Context>{p_context}, TNode<Object>{tmp591});
    }
    if (catch600__label.is_used()) {
      compiler::CodeAssemblerLabel catch600_skip(&ca_);
      ca_.Goto(&catch600_skip);
      ca_.Bind(&catch600__label, &tmp601);
      ca_.Goto(&block303, phi_bb300_17, phi_bb300_18, phi_bb300_19, phi_bb300_27, phi_bb300_28, phi_bb300_29, phi_bb300_30, phi_bb300_31, phi_bb300_34, phi_bb300_35, phi_bb300_36, phi_bb300_37, phi_bb300_38);
      ca_.Bind(&catch600_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch603__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch603__label);
    tmp602 = FromConstexpr_intptr_constexpr_int31_0(state_, 16);
    }
    if (catch603__label.is_used()) {
      compiler::CodeAssemblerLabel catch603_skip(&ca_);
      ca_.Goto(&catch603_skip);
      ca_.Bind(&catch603__label, &tmp604);
      ca_.Goto(&block304, phi_bb300_17, phi_bb300_18, phi_bb300_19, phi_bb300_27, phi_bb300_28, phi_bb300_29, phi_bb300_30, phi_bb300_31, phi_bb300_34, phi_bb300_35, phi_bb300_36, phi_bb300_37, phi_bb300_38);
      ca_.Bind(&catch603_skip);
    }
    tmp605 = Undefined_0(state_);
    CodeStubAssembler(state_).StoreReference<HeapObject>(CodeStubAssembler::Reference{tmp599, tmp602}, tmp605);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 737);
    ca_.Goto(&block301, phi_bb300_17, phi_bb300_18, phi_bb300_19, phi_bb300_27, phi_bb300_28, phi_bb300_29, phi_bb300_30, phi_bb300_31, phi_bb300_34, phi_bb300_35, phi_bb300_36, phi_bb300_37, phi_bb300_38);
  }

  TNode<Object> phi_bb303_17;
  TNode<IntPtrT> phi_bb303_18;
  TNode<BoolT> phi_bb303_19;
  TNode<IntPtrT> phi_bb303_27;
  TNode<IntPtrT> phi_bb303_28;
  TNode<IntPtrT> phi_bb303_29;
  TNode<IntPtrT> phi_bb303_30;
  TNode<IntPtrT> phi_bb303_31;
  TNode<IntPtrT> phi_bb303_34;
  TNode<BoolT> phi_bb303_35;
  TNode<BoolT> phi_bb303_36;
  TNode<Object> phi_bb303_37;
  TNode<Int32T> phi_bb303_38;
  TNode<HeapObject> tmp606;
  if (block303.is_used()) {
    ca_.Bind(&block303, &phi_bb303_17, &phi_bb303_18, &phi_bb303_19, &phi_bb303_27, &phi_bb303_28, &phi_bb303_29, &phi_bb303_30, &phi_bb303_31, &phi_bb303_34, &phi_bb303_35, &phi_bb303_36, &phi_bb303_37, &phi_bb303_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 741);
    tmp606 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb303_17, phi_bb303_18, phi_bb303_19, tmp601, tmp606);
  }

  TNode<Object> phi_bb304_17;
  TNode<IntPtrT> phi_bb304_18;
  TNode<BoolT> phi_bb304_19;
  TNode<IntPtrT> phi_bb304_27;
  TNode<IntPtrT> phi_bb304_28;
  TNode<IntPtrT> phi_bb304_29;
  TNode<IntPtrT> phi_bb304_30;
  TNode<IntPtrT> phi_bb304_31;
  TNode<IntPtrT> phi_bb304_34;
  TNode<BoolT> phi_bb304_35;
  TNode<BoolT> phi_bb304_36;
  TNode<Object> phi_bb304_37;
  TNode<Int32T> phi_bb304_38;
  TNode<HeapObject> tmp607;
  if (block304.is_used()) {
    ca_.Bind(&block304, &phi_bb304_17, &phi_bb304_18, &phi_bb304_19, &phi_bb304_27, &phi_bb304_28, &phi_bb304_29, &phi_bb304_30, &phi_bb304_31, &phi_bb304_34, &phi_bb304_35, &phi_bb304_36, &phi_bb304_37, &phi_bb304_38);
    tmp607 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb304_17, phi_bb304_18, phi_bb304_19, tmp604, tmp607);
  }

  TNode<Object> phi_bb301_17;
  TNode<IntPtrT> phi_bb301_18;
  TNode<BoolT> phi_bb301_19;
  TNode<IntPtrT> phi_bb301_27;
  TNode<IntPtrT> phi_bb301_28;
  TNode<IntPtrT> phi_bb301_29;
  TNode<IntPtrT> phi_bb301_30;
  TNode<IntPtrT> phi_bb301_31;
  TNode<IntPtrT> phi_bb301_34;
  TNode<BoolT> phi_bb301_35;
  TNode<BoolT> phi_bb301_36;
  TNode<Object> phi_bb301_37;
  TNode<Int32T> phi_bb301_38;
  if (block301.is_used()) {
    ca_.Bind(&block301, &phi_bb301_17, &phi_bb301_18, &phi_bb301_19, &phi_bb301_27, &phi_bb301_28, &phi_bb301_29, &phi_bb301_30, &phi_bb301_31, &phi_bb301_34, &phi_bb301_35, &phi_bb301_36, &phi_bb301_37, &phi_bb301_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 735);
    ca_.Goto(&block298, phi_bb301_17, phi_bb301_18, phi_bb301_19, phi_bb301_27, phi_bb301_28, phi_bb301_29, phi_bb301_30, phi_bb301_31, phi_bb301_34, phi_bb301_35, phi_bb301_36, phi_bb301_37, phi_bb301_38, tmp591);
  }

  TNode<Object> phi_bb297_17;
  TNode<IntPtrT> phi_bb297_18;
  TNode<BoolT> phi_bb297_19;
  TNode<IntPtrT> phi_bb297_27;
  TNode<IntPtrT> phi_bb297_28;
  TNode<IntPtrT> phi_bb297_29;
  TNode<IntPtrT> phi_bb297_30;
  TNode<IntPtrT> phi_bb297_31;
  TNode<IntPtrT> phi_bb297_34;
  TNode<BoolT> phi_bb297_35;
  TNode<BoolT> phi_bb297_36;
  TNode<Object> phi_bb297_37;
  TNode<Int32T> phi_bb297_38;
  if (block297.is_used()) {
    ca_.Bind(&block297, &phi_bb297_17, &phi_bb297_18, &phi_bb297_19, &phi_bb297_27, &phi_bb297_28, &phi_bb297_29, &phi_bb297_30, &phi_bb297_31, &phi_bb297_34, &phi_bb297_35, &phi_bb297_36, &phi_bb297_37, &phi_bb297_38);
    ca_.Goto(&block298, phi_bb297_17, phi_bb297_18, phi_bb297_19, phi_bb297_27, phi_bb297_28, phi_bb297_29, phi_bb297_30, phi_bb297_31, phi_bb297_34, phi_bb297_35, phi_bb297_36, phi_bb297_37, phi_bb297_38, tmp590);
  }

  TNode<Object> phi_bb298_17;
  TNode<IntPtrT> phi_bb298_18;
  TNode<BoolT> phi_bb298_19;
  TNode<IntPtrT> phi_bb298_27;
  TNode<IntPtrT> phi_bb298_28;
  TNode<IntPtrT> phi_bb298_29;
  TNode<IntPtrT> phi_bb298_30;
  TNode<IntPtrT> phi_bb298_31;
  TNode<IntPtrT> phi_bb298_34;
  TNode<BoolT> phi_bb298_35;
  TNode<BoolT> phi_bb298_36;
  TNode<Object> phi_bb298_37;
  TNode<Int32T> phi_bb298_38;
  TNode<Object> phi_bb298_39;
  if (block298.is_used()) {
    ca_.Bind(&block298, &phi_bb298_17, &phi_bb298_18, &phi_bb298_19, &phi_bb298_27, &phi_bb298_28, &phi_bb298_29, &phi_bb298_30, &phi_bb298_31, &phi_bb298_34, &phi_bb298_35, &phi_bb298_36, &phi_bb298_37, &phi_bb298_38, &phi_bb298_39);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 744);
    ca_.Branch(phi_bb298_36, &block305, std::vector<compiler::Node*>{phi_bb298_17, phi_bb298_18, phi_bb298_19, phi_bb298_27, phi_bb298_28, phi_bb298_29, phi_bb298_30, phi_bb298_31, phi_bb298_34, phi_bb298_35, phi_bb298_36, phi_bb298_37, phi_bb298_38}, &block306, std::vector<compiler::Node*>{phi_bb298_17, phi_bb298_18, phi_bb298_19, phi_bb298_27, phi_bb298_28, phi_bb298_29, phi_bb298_30, phi_bb298_31, phi_bb298_34, phi_bb298_35, phi_bb298_36, phi_bb298_37, phi_bb298_38});
  }

  TNode<Object> phi_bb305_17;
  TNode<IntPtrT> phi_bb305_18;
  TNode<BoolT> phi_bb305_19;
  TNode<IntPtrT> phi_bb305_27;
  TNode<IntPtrT> phi_bb305_28;
  TNode<IntPtrT> phi_bb305_29;
  TNode<IntPtrT> phi_bb305_30;
  TNode<IntPtrT> phi_bb305_31;
  TNode<IntPtrT> phi_bb305_34;
  TNode<BoolT> phi_bb305_35;
  TNode<BoolT> phi_bb305_36;
  TNode<Object> phi_bb305_37;
  TNode<Int32T> phi_bb305_38;
  TNode<Int32T> tmp608;
      TNode<Object> tmp610;
  TNode<Int32T> tmp611;
      TNode<Object> tmp613;
  TNode<BoolT> tmp614;
  if (block305.is_used()) {
    ca_.Bind(&block305, &phi_bb305_17, &phi_bb305_18, &phi_bb305_19, &phi_bb305_27, &phi_bb305_28, &phi_bb305_29, &phi_bb305_30, &phi_bb305_31, &phi_bb305_34, &phi_bb305_35, &phi_bb305_36, &phi_bb305_37, &phi_bb305_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 746);
    compiler::CodeAssemblerExceptionHandlerLabel catch609__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch609__label);
    tmp608 = FromConstexpr_int32_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    }
    if (catch609__label.is_used()) {
      compiler::CodeAssemblerLabel catch609_skip(&ca_);
      ca_.Goto(&catch609_skip);
      ca_.Bind(&catch609__label, &tmp610);
      ca_.Goto(&block307, phi_bb305_17, phi_bb305_18, phi_bb305_19, phi_bb305_27, phi_bb305_28, phi_bb305_29, phi_bb305_30, phi_bb305_31, phi_bb305_34, phi_bb305_35, phi_bb305_36, phi_bb305_37, phi_bb305_38);
      ca_.Bind(&catch609_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 749);
    compiler::CodeAssemblerExceptionHandlerLabel catch612__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch612__label);
    tmp611 = FromConstexpr_int32_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    }
    if (catch612__label.is_used()) {
      compiler::CodeAssemblerLabel catch612_skip(&ca_);
      ca_.Goto(&catch612_skip);
      ca_.Bind(&catch612__label, &tmp613);
      ca_.Goto(&block308, phi_bb305_17, phi_bb305_18, phi_bb305_19, phi_bb305_27, phi_bb305_28, phi_bb305_29, phi_bb305_30, phi_bb305_31, phi_bb305_34, phi_bb305_35, phi_bb305_36, phi_bb305_37, phi_bb305_38);
      ca_.Bind(&catch612_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 352);
    tmp614 = CodeStubAssembler(state_).Word32BinaryNot(TNode<BoolT>{phi_bb305_35});
    ca_.Branch(tmp614, &block310, std::vector<compiler::Node*>{phi_bb305_17, phi_bb305_18, phi_bb305_19, phi_bb305_27, phi_bb305_28, phi_bb305_29, phi_bb305_30, phi_bb305_31, phi_bb305_34, phi_bb305_35, phi_bb305_36, phi_bb305_37, phi_bb305_38}, &block311, std::vector<compiler::Node*>{phi_bb305_17, phi_bb305_18, phi_bb305_19, phi_bb305_27, phi_bb305_28, phi_bb305_29, phi_bb305_30, phi_bb305_31, phi_bb305_34, phi_bb305_35, phi_bb305_36, phi_bb305_37, phi_bb305_38});
  }

  TNode<Object> phi_bb307_17;
  TNode<IntPtrT> phi_bb307_18;
  TNode<BoolT> phi_bb307_19;
  TNode<IntPtrT> phi_bb307_27;
  TNode<IntPtrT> phi_bb307_28;
  TNode<IntPtrT> phi_bb307_29;
  TNode<IntPtrT> phi_bb307_30;
  TNode<IntPtrT> phi_bb307_31;
  TNode<IntPtrT> phi_bb307_34;
  TNode<BoolT> phi_bb307_35;
  TNode<BoolT> phi_bb307_36;
  TNode<Object> phi_bb307_37;
  TNode<Int32T> phi_bb307_38;
  TNode<HeapObject> tmp615;
  if (block307.is_used()) {
    ca_.Bind(&block307, &phi_bb307_17, &phi_bb307_18, &phi_bb307_19, &phi_bb307_27, &phi_bb307_28, &phi_bb307_29, &phi_bb307_30, &phi_bb307_31, &phi_bb307_34, &phi_bb307_35, &phi_bb307_36, &phi_bb307_37, &phi_bb307_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 746);
    tmp615 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb307_17, phi_bb307_18, phi_bb307_19, tmp610, tmp615);
  }

  TNode<Object> phi_bb308_17;
  TNode<IntPtrT> phi_bb308_18;
  TNode<BoolT> phi_bb308_19;
  TNode<IntPtrT> phi_bb308_27;
  TNode<IntPtrT> phi_bb308_28;
  TNode<IntPtrT> phi_bb308_29;
  TNode<IntPtrT> phi_bb308_30;
  TNode<IntPtrT> phi_bb308_31;
  TNode<IntPtrT> phi_bb308_34;
  TNode<BoolT> phi_bb308_35;
  TNode<BoolT> phi_bb308_36;
  TNode<Object> phi_bb308_37;
  TNode<Int32T> phi_bb308_38;
  TNode<HeapObject> tmp616;
  if (block308.is_used()) {
    ca_.Bind(&block308, &phi_bb308_17, &phi_bb308_18, &phi_bb308_19, &phi_bb308_27, &phi_bb308_28, &phi_bb308_29, &phi_bb308_30, &phi_bb308_31, &phi_bb308_34, &phi_bb308_35, &phi_bb308_36, &phi_bb308_37, &phi_bb308_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 749);
    tmp616 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb308_17, phi_bb308_18, phi_bb308_19, tmp613, tmp616);
  }

  TNode<Object> phi_bb310_17;
  TNode<IntPtrT> phi_bb310_18;
  TNode<BoolT> phi_bb310_19;
  TNode<IntPtrT> phi_bb310_27;
  TNode<IntPtrT> phi_bb310_28;
  TNode<IntPtrT> phi_bb310_29;
  TNode<IntPtrT> phi_bb310_30;
  TNode<IntPtrT> phi_bb310_31;
  TNode<IntPtrT> phi_bb310_34;
  TNode<BoolT> phi_bb310_35;
  TNode<BoolT> phi_bb310_36;
  TNode<Object> phi_bb310_37;
  TNode<Int32T> phi_bb310_38;
  TNode<IntPtrT> tmp617;
  if (block310.is_used()) {
    ca_.Bind(&block310, &phi_bb310_17, &phi_bb310_18, &phi_bb310_19, &phi_bb310_27, &phi_bb310_28, &phi_bb310_29, &phi_bb310_30, &phi_bb310_31, &phi_bb310_34, &phi_bb310_35, &phi_bb310_36, &phi_bb310_37, &phi_bb310_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 353);
    tmp617 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 352);
    ca_.Goto(&block311, phi_bb310_17, phi_bb310_18, phi_bb310_19, phi_bb310_27, phi_bb310_28, phi_bb310_29, phi_bb310_30, phi_bb310_31, tmp617, phi_bb310_35, phi_bb310_36, phi_bb310_37, phi_bb310_38);
  }

  TNode<Object> phi_bb311_17;
  TNode<IntPtrT> phi_bb311_18;
  TNode<BoolT> phi_bb311_19;
  TNode<IntPtrT> phi_bb311_27;
  TNode<IntPtrT> phi_bb311_28;
  TNode<IntPtrT> phi_bb311_29;
  TNode<IntPtrT> phi_bb311_30;
  TNode<IntPtrT> phi_bb311_31;
  TNode<IntPtrT> phi_bb311_34;
  TNode<BoolT> phi_bb311_35;
  TNode<BoolT> phi_bb311_36;
  TNode<Object> phi_bb311_37;
  TNode<Int32T> phi_bb311_38;
  if (block311.is_used()) {
    ca_.Bind(&block311, &phi_bb311_17, &phi_bb311_18, &phi_bb311_19, &phi_bb311_27, &phi_bb311_28, &phi_bb311_29, &phi_bb311_30, &phi_bb311_31, &phi_bb311_34, &phi_bb311_35, &phi_bb311_36, &phi_bb311_37, &phi_bb311_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 754);
    ca_.Goto(&block314, phi_bb311_17, phi_bb311_18, phi_bb311_19, phi_bb311_27, phi_bb311_28, phi_bb311_29, phi_bb311_30, phi_bb311_31, phi_bb311_34, phi_bb311_35, phi_bb311_36, phi_bb311_37, phi_bb311_38, tmp608, tmp611);
  }

  TNode<Object> phi_bb314_17;
  TNode<IntPtrT> phi_bb314_18;
  TNode<BoolT> phi_bb314_19;
  TNode<IntPtrT> phi_bb314_27;
  TNode<IntPtrT> phi_bb314_28;
  TNode<IntPtrT> phi_bb314_29;
  TNode<IntPtrT> phi_bb314_30;
  TNode<IntPtrT> phi_bb314_31;
  TNode<IntPtrT> phi_bb314_34;
  TNode<BoolT> phi_bb314_35;
  TNode<BoolT> phi_bb314_36;
  TNode<Object> phi_bb314_37;
  TNode<Int32T> phi_bb314_38;
  TNode<Int32T> phi_bb314_40;
  TNode<Int32T> phi_bb314_41;
  TNode<BoolT> tmp618;
      TNode<Object> tmp620;
  if (block314.is_used()) {
    ca_.Bind(&block314, &phi_bb314_17, &phi_bb314_18, &phi_bb314_19, &phi_bb314_27, &phi_bb314_28, &phi_bb314_29, &phi_bb314_30, &phi_bb314_31, &phi_bb314_34, &phi_bb314_35, &phi_bb314_36, &phi_bb314_37, &phi_bb314_38, &phi_bb314_40, &phi_bb314_41);
    compiler::CodeAssemblerExceptionHandlerLabel catch619__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch619__label);
    tmp618 = CodeStubAssembler(state_).Int32LessThan(TNode<Int32T>{phi_bb314_40}, TNode<Int32T>{tmp26});
    }
    if (catch619__label.is_used()) {
      compiler::CodeAssemblerLabel catch619_skip(&ca_);
      ca_.Goto(&catch619_skip);
      ca_.Bind(&catch619__label, &tmp620);
      ca_.Goto(&block315, phi_bb314_17, phi_bb314_18, phi_bb314_19, phi_bb314_27, phi_bb314_28, phi_bb314_29, phi_bb314_30, phi_bb314_31, phi_bb314_34, phi_bb314_35, phi_bb314_36, phi_bb314_37, phi_bb314_38, phi_bb314_40, phi_bb314_41, phi_bb314_40);
      ca_.Bind(&catch619_skip);
    }
    ca_.Branch(tmp618, &block312, std::vector<compiler::Node*>{phi_bb314_17, phi_bb314_18, phi_bb314_19, phi_bb314_27, phi_bb314_28, phi_bb314_29, phi_bb314_30, phi_bb314_31, phi_bb314_34, phi_bb314_35, phi_bb314_36, phi_bb314_37, phi_bb314_38, phi_bb314_40, phi_bb314_41}, &block313, std::vector<compiler::Node*>{phi_bb314_17, phi_bb314_18, phi_bb314_19, phi_bb314_27, phi_bb314_28, phi_bb314_29, phi_bb314_30, phi_bb314_31, phi_bb314_34, phi_bb314_35, phi_bb314_36, phi_bb314_37, phi_bb314_38, phi_bb314_40, phi_bb314_41});
  }

  TNode<Object> phi_bb315_17;
  TNode<IntPtrT> phi_bb315_18;
  TNode<BoolT> phi_bb315_19;
  TNode<IntPtrT> phi_bb315_27;
  TNode<IntPtrT> phi_bb315_28;
  TNode<IntPtrT> phi_bb315_29;
  TNode<IntPtrT> phi_bb315_30;
  TNode<IntPtrT> phi_bb315_31;
  TNode<IntPtrT> phi_bb315_34;
  TNode<BoolT> phi_bb315_35;
  TNode<BoolT> phi_bb315_36;
  TNode<Object> phi_bb315_37;
  TNode<Int32T> phi_bb315_38;
  TNode<Int32T> phi_bb315_40;
  TNode<Int32T> phi_bb315_41;
  TNode<Int32T> phi_bb315_42;
  TNode<HeapObject> tmp621;
  if (block315.is_used()) {
    ca_.Bind(&block315, &phi_bb315_17, &phi_bb315_18, &phi_bb315_19, &phi_bb315_27, &phi_bb315_28, &phi_bb315_29, &phi_bb315_30, &phi_bb315_31, &phi_bb315_34, &phi_bb315_35, &phi_bb315_36, &phi_bb315_37, &phi_bb315_38, &phi_bb315_40, &phi_bb315_41, &phi_bb315_42);
    tmp621 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb315_17, phi_bb315_18, phi_bb315_19, tmp620, tmp621);
  }

  TNode<Object> phi_bb312_17;
  TNode<IntPtrT> phi_bb312_18;
  TNode<BoolT> phi_bb312_19;
  TNode<IntPtrT> phi_bb312_27;
  TNode<IntPtrT> phi_bb312_28;
  TNode<IntPtrT> phi_bb312_29;
  TNode<IntPtrT> phi_bb312_30;
  TNode<IntPtrT> phi_bb312_31;
  TNode<IntPtrT> phi_bb312_34;
  TNode<BoolT> phi_bb312_35;
  TNode<BoolT> phi_bb312_36;
  TNode<Object> phi_bb312_37;
  TNode<Int32T> phi_bb312_38;
  TNode<Int32T> phi_bb312_40;
  TNode<Int32T> phi_bb312_41;
  TNode<IntPtrT> tmp622;
      TNode<Object> tmp624;
  TNode<IntPtrT> tmp625;
  TNode<IntPtrT> tmp626;
  TNode<Object> tmp627;
  TNode<IntPtrT> tmp628;
  TNode<Int32T> tmp629;
  TNode<Int32T> tmp630;
      TNode<Object> tmp632;
  TNode<Int32T> tmp633;
      TNode<Object> tmp635;
  TNode<Int32T> tmp636;
      TNode<Object> tmp638;
  TNode<BoolT> tmp639;
      TNode<Object> tmp641;
  if (block312.is_used()) {
    ca_.Bind(&block312, &phi_bb312_17, &phi_bb312_18, &phi_bb312_19, &phi_bb312_27, &phi_bb312_28, &phi_bb312_29, &phi_bb312_30, &phi_bb312_31, &phi_bb312_34, &phi_bb312_35, &phi_bb312_36, &phi_bb312_37, &phi_bb312_38, &phi_bb312_40, &phi_bb312_41);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 755);
    compiler::CodeAssemblerExceptionHandlerLabel catch623__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch623__label);
    tmp622 = Convert_intptr_int32_0(state_, TNode<Int32T>{phi_bb312_40});
    }
    if (catch623__label.is_used()) {
      compiler::CodeAssemblerLabel catch623_skip(&ca_);
      ca_.Goto(&catch623_skip);
      ca_.Bind(&catch623__label, &tmp624);
      ca_.Goto(&block316, phi_bb312_17, phi_bb312_18, phi_bb312_19, phi_bb312_27, phi_bb312_28, phi_bb312_29, phi_bb312_30, phi_bb312_31, phi_bb312_34, phi_bb312_35, phi_bb312_36, phi_bb312_37, phi_bb312_38, phi_bb312_40, phi_bb312_41, phi_bb312_40);
      ca_.Bind(&catch623_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp625 = TimesSizeOf_int32_0(state_, TNode<IntPtrT>{tmp622});
    tmp626 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp177}, TNode<IntPtrT>{tmp625});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp627, tmp628) = NewReference_int32_0(state_, TNode<Object>{tmp176}, TNode<IntPtrT>{tmp626}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 755);
    tmp629 = CodeStubAssembler(state_).LoadReference<Int32T>(CodeStubAssembler::Reference{tmp627, tmp628});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 756);
    compiler::CodeAssemblerExceptionHandlerLabel catch631__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch631__label);
    tmp630 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmValueKindBitsMask);
    }
    if (catch631__label.is_used()) {
      compiler::CodeAssemblerLabel catch631_skip(&ca_);
      ca_.Goto(&catch631_skip);
      ca_.Bind(&catch631__label, &tmp632);
      ca_.Goto(&block318, phi_bb312_17, phi_bb312_18, phi_bb312_19, phi_bb312_27, phi_bb312_28, phi_bb312_29, phi_bb312_30, phi_bb312_31, phi_bb312_34, phi_bb312_35, phi_bb312_36, phi_bb312_37, phi_bb312_38, phi_bb312_40, phi_bb312_41);
      ca_.Bind(&catch631_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch634__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch634__label);
    tmp633 = CodeStubAssembler(state_).Word32And(TNode<Int32T>{tmp629}, TNode<Int32T>{tmp630});
    }
    if (catch634__label.is_used()) {
      compiler::CodeAssemblerLabel catch634_skip(&ca_);
      ca_.Goto(&catch634_skip);
      ca_.Bind(&catch634__label, &tmp635);
      ca_.Goto(&block319, phi_bb312_17, phi_bb312_18, phi_bb312_19, phi_bb312_27, phi_bb312_28, phi_bb312_29, phi_bb312_30, phi_bb312_31, phi_bb312_34, phi_bb312_35, phi_bb312_36, phi_bb312_37, phi_bb312_38, phi_bb312_40, phi_bb312_41);
      ca_.Bind(&catch634_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 757);
    compiler::CodeAssemblerExceptionHandlerLabel catch637__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch637__label);
    tmp636 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRef);
    }
    if (catch637__label.is_used()) {
      compiler::CodeAssemblerLabel catch637_skip(&ca_);
      ca_.Goto(&catch637_skip);
      ca_.Bind(&catch637__label, &tmp638);
      ca_.Goto(&block322, phi_bb312_17, phi_bb312_18, phi_bb312_19, phi_bb312_27, phi_bb312_28, phi_bb312_29, phi_bb312_30, phi_bb312_31, phi_bb312_34, phi_bb312_35, phi_bb312_36, phi_bb312_37, phi_bb312_38, phi_bb312_40, phi_bb312_41);
      ca_.Bind(&catch637_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch640__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch640__label);
    tmp639 = CodeStubAssembler(state_).Word32NotEqual(TNode<Int32T>{tmp633}, TNode<Int32T>{tmp636});
    }
    if (catch640__label.is_used()) {
      compiler::CodeAssemblerLabel catch640_skip(&ca_);
      ca_.Goto(&catch640_skip);
      ca_.Bind(&catch640__label, &tmp641);
      ca_.Goto(&block323, phi_bb312_17, phi_bb312_18, phi_bb312_19, phi_bb312_27, phi_bb312_28, phi_bb312_29, phi_bb312_30, phi_bb312_31, phi_bb312_34, phi_bb312_35, phi_bb312_36, phi_bb312_37, phi_bb312_38, phi_bb312_40, phi_bb312_41);
      ca_.Bind(&catch640_skip);
    }
    ca_.Branch(tmp639, &block324, std::vector<compiler::Node*>{phi_bb312_17, phi_bb312_18, phi_bb312_19, phi_bb312_27, phi_bb312_28, phi_bb312_29, phi_bb312_30, phi_bb312_31, phi_bb312_34, phi_bb312_35, phi_bb312_36, phi_bb312_37, phi_bb312_38, phi_bb312_40, phi_bb312_41}, &block325, std::vector<compiler::Node*>{phi_bb312_17, phi_bb312_18, phi_bb312_19, phi_bb312_27, phi_bb312_28, phi_bb312_29, phi_bb312_30, phi_bb312_31, phi_bb312_34, phi_bb312_35, phi_bb312_36, phi_bb312_37, phi_bb312_38, phi_bb312_40, phi_bb312_41});
  }

  TNode<Object> phi_bb316_17;
  TNode<IntPtrT> phi_bb316_18;
  TNode<BoolT> phi_bb316_19;
  TNode<IntPtrT> phi_bb316_27;
  TNode<IntPtrT> phi_bb316_28;
  TNode<IntPtrT> phi_bb316_29;
  TNode<IntPtrT> phi_bb316_30;
  TNode<IntPtrT> phi_bb316_31;
  TNode<IntPtrT> phi_bb316_34;
  TNode<BoolT> phi_bb316_35;
  TNode<BoolT> phi_bb316_36;
  TNode<Object> phi_bb316_37;
  TNode<Int32T> phi_bb316_38;
  TNode<Int32T> phi_bb316_40;
  TNode<Int32T> phi_bb316_41;
  TNode<Int32T> phi_bb316_45;
  TNode<HeapObject> tmp642;
  if (block316.is_used()) {
    ca_.Bind(&block316, &phi_bb316_17, &phi_bb316_18, &phi_bb316_19, &phi_bb316_27, &phi_bb316_28, &phi_bb316_29, &phi_bb316_30, &phi_bb316_31, &phi_bb316_34, &phi_bb316_35, &phi_bb316_36, &phi_bb316_37, &phi_bb316_38, &phi_bb316_40, &phi_bb316_41, &phi_bb316_45);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 755);
    tmp642 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb316_17, phi_bb316_18, phi_bb316_19, tmp624, tmp642);
  }

  TNode<Object> phi_bb318_17;
  TNode<IntPtrT> phi_bb318_18;
  TNode<BoolT> phi_bb318_19;
  TNode<IntPtrT> phi_bb318_27;
  TNode<IntPtrT> phi_bb318_28;
  TNode<IntPtrT> phi_bb318_29;
  TNode<IntPtrT> phi_bb318_30;
  TNode<IntPtrT> phi_bb318_31;
  TNode<IntPtrT> phi_bb318_34;
  TNode<BoolT> phi_bb318_35;
  TNode<BoolT> phi_bb318_36;
  TNode<Object> phi_bb318_37;
  TNode<Int32T> phi_bb318_38;
  TNode<Int32T> phi_bb318_40;
  TNode<Int32T> phi_bb318_41;
  TNode<HeapObject> tmp643;
  if (block318.is_used()) {
    ca_.Bind(&block318, &phi_bb318_17, &phi_bb318_18, &phi_bb318_19, &phi_bb318_27, &phi_bb318_28, &phi_bb318_29, &phi_bb318_30, &phi_bb318_31, &phi_bb318_34, &phi_bb318_35, &phi_bb318_36, &phi_bb318_37, &phi_bb318_38, &phi_bb318_40, &phi_bb318_41);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 756);
    tmp643 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb318_17, phi_bb318_18, phi_bb318_19, tmp632, tmp643);
  }

  TNode<Object> phi_bb319_17;
  TNode<IntPtrT> phi_bb319_18;
  TNode<BoolT> phi_bb319_19;
  TNode<IntPtrT> phi_bb319_27;
  TNode<IntPtrT> phi_bb319_28;
  TNode<IntPtrT> phi_bb319_29;
  TNode<IntPtrT> phi_bb319_30;
  TNode<IntPtrT> phi_bb319_31;
  TNode<IntPtrT> phi_bb319_34;
  TNode<BoolT> phi_bb319_35;
  TNode<BoolT> phi_bb319_36;
  TNode<Object> phi_bb319_37;
  TNode<Int32T> phi_bb319_38;
  TNode<Int32T> phi_bb319_40;
  TNode<Int32T> phi_bb319_41;
  TNode<HeapObject> tmp644;
  if (block319.is_used()) {
    ca_.Bind(&block319, &phi_bb319_17, &phi_bb319_18, &phi_bb319_19, &phi_bb319_27, &phi_bb319_28, &phi_bb319_29, &phi_bb319_30, &phi_bb319_31, &phi_bb319_34, &phi_bb319_35, &phi_bb319_36, &phi_bb319_37, &phi_bb319_38, &phi_bb319_40, &phi_bb319_41);
    tmp644 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb319_17, phi_bb319_18, phi_bb319_19, tmp635, tmp644);
  }

  TNode<Object> phi_bb322_17;
  TNode<IntPtrT> phi_bb322_18;
  TNode<BoolT> phi_bb322_19;
  TNode<IntPtrT> phi_bb322_27;
  TNode<IntPtrT> phi_bb322_28;
  TNode<IntPtrT> phi_bb322_29;
  TNode<IntPtrT> phi_bb322_30;
  TNode<IntPtrT> phi_bb322_31;
  TNode<IntPtrT> phi_bb322_34;
  TNode<BoolT> phi_bb322_35;
  TNode<BoolT> phi_bb322_36;
  TNode<Object> phi_bb322_37;
  TNode<Int32T> phi_bb322_38;
  TNode<Int32T> phi_bb322_40;
  TNode<Int32T> phi_bb322_41;
  TNode<HeapObject> tmp645;
  if (block322.is_used()) {
    ca_.Bind(&block322, &phi_bb322_17, &phi_bb322_18, &phi_bb322_19, &phi_bb322_27, &phi_bb322_28, &phi_bb322_29, &phi_bb322_30, &phi_bb322_31, &phi_bb322_34, &phi_bb322_35, &phi_bb322_36, &phi_bb322_37, &phi_bb322_38, &phi_bb322_40, &phi_bb322_41);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 757);
    tmp645 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb322_17, phi_bb322_18, phi_bb322_19, tmp638, tmp645);
  }

  TNode<Object> phi_bb323_17;
  TNode<IntPtrT> phi_bb323_18;
  TNode<BoolT> phi_bb323_19;
  TNode<IntPtrT> phi_bb323_27;
  TNode<IntPtrT> phi_bb323_28;
  TNode<IntPtrT> phi_bb323_29;
  TNode<IntPtrT> phi_bb323_30;
  TNode<IntPtrT> phi_bb323_31;
  TNode<IntPtrT> phi_bb323_34;
  TNode<BoolT> phi_bb323_35;
  TNode<BoolT> phi_bb323_36;
  TNode<Object> phi_bb323_37;
  TNode<Int32T> phi_bb323_38;
  TNode<Int32T> phi_bb323_40;
  TNode<Int32T> phi_bb323_41;
  TNode<HeapObject> tmp646;
  if (block323.is_used()) {
    ca_.Bind(&block323, &phi_bb323_17, &phi_bb323_18, &phi_bb323_19, &phi_bb323_27, &phi_bb323_28, &phi_bb323_29, &phi_bb323_30, &phi_bb323_31, &phi_bb323_34, &phi_bb323_35, &phi_bb323_36, &phi_bb323_37, &phi_bb323_38, &phi_bb323_40, &phi_bb323_41);
    tmp646 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb323_17, phi_bb323_18, phi_bb323_19, tmp641, tmp646);
  }

  TNode<Object> phi_bb324_17;
  TNode<IntPtrT> phi_bb324_18;
  TNode<BoolT> phi_bb324_19;
  TNode<IntPtrT> phi_bb324_27;
  TNode<IntPtrT> phi_bb324_28;
  TNode<IntPtrT> phi_bb324_29;
  TNode<IntPtrT> phi_bb324_30;
  TNode<IntPtrT> phi_bb324_31;
  TNode<IntPtrT> phi_bb324_34;
  TNode<BoolT> phi_bb324_35;
  TNode<BoolT> phi_bb324_36;
  TNode<Object> phi_bb324_37;
  TNode<Int32T> phi_bb324_38;
  TNode<Int32T> phi_bb324_40;
  TNode<Int32T> phi_bb324_41;
  TNode<Int32T> tmp647;
      TNode<Object> tmp649;
  TNode<BoolT> tmp650;
      TNode<Object> tmp652;
  if (block324.is_used()) {
    ca_.Bind(&block324, &phi_bb324_17, &phi_bb324_18, &phi_bb324_19, &phi_bb324_27, &phi_bb324_28, &phi_bb324_29, &phi_bb324_30, &phi_bb324_31, &phi_bb324_34, &phi_bb324_35, &phi_bb324_36, &phi_bb324_37, &phi_bb324_38, &phi_bb324_40, &phi_bb324_41);
    compiler::CodeAssemblerExceptionHandlerLabel catch648__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch648__label);
    tmp647 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRefNull);
    }
    if (catch648__label.is_used()) {
      compiler::CodeAssemblerLabel catch648_skip(&ca_);
      ca_.Goto(&catch648_skip);
      ca_.Bind(&catch648__label, &tmp649);
      ca_.Goto(&block327, phi_bb324_17, phi_bb324_18, phi_bb324_19, phi_bb324_27, phi_bb324_28, phi_bb324_29, phi_bb324_30, phi_bb324_31, phi_bb324_34, phi_bb324_35, phi_bb324_36, phi_bb324_37, phi_bb324_38, phi_bb324_40, phi_bb324_41);
      ca_.Bind(&catch648_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch651__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch651__label);
    tmp650 = CodeStubAssembler(state_).Word32NotEqual(TNode<Int32T>{tmp633}, TNode<Int32T>{tmp647});
    }
    if (catch651__label.is_used()) {
      compiler::CodeAssemblerLabel catch651_skip(&ca_);
      ca_.Goto(&catch651_skip);
      ca_.Bind(&catch651__label, &tmp652);
      ca_.Goto(&block328, phi_bb324_17, phi_bb324_18, phi_bb324_19, phi_bb324_27, phi_bb324_28, phi_bb324_29, phi_bb324_30, phi_bb324_31, phi_bb324_34, phi_bb324_35, phi_bb324_36, phi_bb324_37, phi_bb324_38, phi_bb324_40, phi_bb324_41);
      ca_.Bind(&catch651_skip);
    }
    ca_.Goto(&block326, phi_bb324_17, phi_bb324_18, phi_bb324_19, phi_bb324_27, phi_bb324_28, phi_bb324_29, phi_bb324_30, phi_bb324_31, phi_bb324_34, phi_bb324_35, phi_bb324_36, phi_bb324_37, phi_bb324_38, phi_bb324_40, phi_bb324_41, tmp650);
  }

  TNode<Object> phi_bb327_17;
  TNode<IntPtrT> phi_bb327_18;
  TNode<BoolT> phi_bb327_19;
  TNode<IntPtrT> phi_bb327_27;
  TNode<IntPtrT> phi_bb327_28;
  TNode<IntPtrT> phi_bb327_29;
  TNode<IntPtrT> phi_bb327_30;
  TNode<IntPtrT> phi_bb327_31;
  TNode<IntPtrT> phi_bb327_34;
  TNode<BoolT> phi_bb327_35;
  TNode<BoolT> phi_bb327_36;
  TNode<Object> phi_bb327_37;
  TNode<Int32T> phi_bb327_38;
  TNode<Int32T> phi_bb327_40;
  TNode<Int32T> phi_bb327_41;
  TNode<HeapObject> tmp653;
  if (block327.is_used()) {
    ca_.Bind(&block327, &phi_bb327_17, &phi_bb327_18, &phi_bb327_19, &phi_bb327_27, &phi_bb327_28, &phi_bb327_29, &phi_bb327_30, &phi_bb327_31, &phi_bb327_34, &phi_bb327_35, &phi_bb327_36, &phi_bb327_37, &phi_bb327_38, &phi_bb327_40, &phi_bb327_41);
    tmp653 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb327_17, phi_bb327_18, phi_bb327_19, tmp649, tmp653);
  }

  TNode<Object> phi_bb328_17;
  TNode<IntPtrT> phi_bb328_18;
  TNode<BoolT> phi_bb328_19;
  TNode<IntPtrT> phi_bb328_27;
  TNode<IntPtrT> phi_bb328_28;
  TNode<IntPtrT> phi_bb328_29;
  TNode<IntPtrT> phi_bb328_30;
  TNode<IntPtrT> phi_bb328_31;
  TNode<IntPtrT> phi_bb328_34;
  TNode<BoolT> phi_bb328_35;
  TNode<BoolT> phi_bb328_36;
  TNode<Object> phi_bb328_37;
  TNode<Int32T> phi_bb328_38;
  TNode<Int32T> phi_bb328_40;
  TNode<Int32T> phi_bb328_41;
  TNode<HeapObject> tmp654;
  if (block328.is_used()) {
    ca_.Bind(&block328, &phi_bb328_17, &phi_bb328_18, &phi_bb328_19, &phi_bb328_27, &phi_bb328_28, &phi_bb328_29, &phi_bb328_30, &phi_bb328_31, &phi_bb328_34, &phi_bb328_35, &phi_bb328_36, &phi_bb328_37, &phi_bb328_38, &phi_bb328_40, &phi_bb328_41);
    tmp654 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb328_17, phi_bb328_18, phi_bb328_19, tmp652, tmp654);
  }

  TNode<Object> phi_bb325_17;
  TNode<IntPtrT> phi_bb325_18;
  TNode<BoolT> phi_bb325_19;
  TNode<IntPtrT> phi_bb325_27;
  TNode<IntPtrT> phi_bb325_28;
  TNode<IntPtrT> phi_bb325_29;
  TNode<IntPtrT> phi_bb325_30;
  TNode<IntPtrT> phi_bb325_31;
  TNode<IntPtrT> phi_bb325_34;
  TNode<BoolT> phi_bb325_35;
  TNode<BoolT> phi_bb325_36;
  TNode<Object> phi_bb325_37;
  TNode<Int32T> phi_bb325_38;
  TNode<Int32T> phi_bb325_40;
  TNode<Int32T> phi_bb325_41;
  TNode<BoolT> tmp655;
      TNode<Object> tmp657;
  if (block325.is_used()) {
    ca_.Bind(&block325, &phi_bb325_17, &phi_bb325_18, &phi_bb325_19, &phi_bb325_27, &phi_bb325_28, &phi_bb325_29, &phi_bb325_30, &phi_bb325_31, &phi_bb325_34, &phi_bb325_35, &phi_bb325_36, &phi_bb325_37, &phi_bb325_38, &phi_bb325_40, &phi_bb325_41);
    compiler::CodeAssemblerExceptionHandlerLabel catch656__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch656__label);
    tmp655 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    }
    if (catch656__label.is_used()) {
      compiler::CodeAssemblerLabel catch656_skip(&ca_);
      ca_.Goto(&catch656_skip);
      ca_.Bind(&catch656__label, &tmp657);
      ca_.Goto(&block329, phi_bb325_17, phi_bb325_18, phi_bb325_19, phi_bb325_27, phi_bb325_28, phi_bb325_29, phi_bb325_30, phi_bb325_31, phi_bb325_34, phi_bb325_35, phi_bb325_36, phi_bb325_37, phi_bb325_38, phi_bb325_40, phi_bb325_41);
      ca_.Bind(&catch656_skip);
    }
    ca_.Goto(&block326, phi_bb325_17, phi_bb325_18, phi_bb325_19, phi_bb325_27, phi_bb325_28, phi_bb325_29, phi_bb325_30, phi_bb325_31, phi_bb325_34, phi_bb325_35, phi_bb325_36, phi_bb325_37, phi_bb325_38, phi_bb325_40, phi_bb325_41, tmp655);
  }

  TNode<Object> phi_bb329_17;
  TNode<IntPtrT> phi_bb329_18;
  TNode<BoolT> phi_bb329_19;
  TNode<IntPtrT> phi_bb329_27;
  TNode<IntPtrT> phi_bb329_28;
  TNode<IntPtrT> phi_bb329_29;
  TNode<IntPtrT> phi_bb329_30;
  TNode<IntPtrT> phi_bb329_31;
  TNode<IntPtrT> phi_bb329_34;
  TNode<BoolT> phi_bb329_35;
  TNode<BoolT> phi_bb329_36;
  TNode<Object> phi_bb329_37;
  TNode<Int32T> phi_bb329_38;
  TNode<Int32T> phi_bb329_40;
  TNode<Int32T> phi_bb329_41;
  TNode<HeapObject> tmp658;
  if (block329.is_used()) {
    ca_.Bind(&block329, &phi_bb329_17, &phi_bb329_18, &phi_bb329_19, &phi_bb329_27, &phi_bb329_28, &phi_bb329_29, &phi_bb329_30, &phi_bb329_31, &phi_bb329_34, &phi_bb329_35, &phi_bb329_36, &phi_bb329_37, &phi_bb329_38, &phi_bb329_40, &phi_bb329_41);
    tmp658 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb329_17, phi_bb329_18, phi_bb329_19, tmp657, tmp658);
  }

  TNode<Object> phi_bb326_17;
  TNode<IntPtrT> phi_bb326_18;
  TNode<BoolT> phi_bb326_19;
  TNode<IntPtrT> phi_bb326_27;
  TNode<IntPtrT> phi_bb326_28;
  TNode<IntPtrT> phi_bb326_29;
  TNode<IntPtrT> phi_bb326_30;
  TNode<IntPtrT> phi_bb326_31;
  TNode<IntPtrT> phi_bb326_34;
  TNode<BoolT> phi_bb326_35;
  TNode<BoolT> phi_bb326_36;
  TNode<Object> phi_bb326_37;
  TNode<Int32T> phi_bb326_38;
  TNode<Int32T> phi_bb326_40;
  TNode<Int32T> phi_bb326_41;
  TNode<BoolT> phi_bb326_45;
  if (block326.is_used()) {
    ca_.Bind(&block326, &phi_bb326_17, &phi_bb326_18, &phi_bb326_19, &phi_bb326_27, &phi_bb326_28, &phi_bb326_29, &phi_bb326_30, &phi_bb326_31, &phi_bb326_34, &phi_bb326_35, &phi_bb326_36, &phi_bb326_37, &phi_bb326_38, &phi_bb326_40, &phi_bb326_41, &phi_bb326_45);
    ca_.Branch(phi_bb326_45, &block320, std::vector<compiler::Node*>{phi_bb326_17, phi_bb326_18, phi_bb326_19, phi_bb326_27, phi_bb326_28, phi_bb326_29, phi_bb326_30, phi_bb326_31, phi_bb326_34, phi_bb326_35, phi_bb326_36, phi_bb326_37, phi_bb326_38, phi_bb326_40, phi_bb326_41}, &block321, std::vector<compiler::Node*>{phi_bb326_17, phi_bb326_18, phi_bb326_19, phi_bb326_27, phi_bb326_28, phi_bb326_29, phi_bb326_30, phi_bb326_31, phi_bb326_34, phi_bb326_35, phi_bb326_36, phi_bb326_37, phi_bb326_38, phi_bb326_40, phi_bb326_41});
  }

  TNode<Object> phi_bb320_17;
  TNode<IntPtrT> phi_bb320_18;
  TNode<BoolT> phi_bb320_19;
  TNode<IntPtrT> phi_bb320_27;
  TNode<IntPtrT> phi_bb320_28;
  TNode<IntPtrT> phi_bb320_29;
  TNode<IntPtrT> phi_bb320_30;
  TNode<IntPtrT> phi_bb320_31;
  TNode<IntPtrT> phi_bb320_34;
  TNode<BoolT> phi_bb320_35;
  TNode<BoolT> phi_bb320_36;
  TNode<Object> phi_bb320_37;
  TNode<Int32T> phi_bb320_38;
  TNode<Int32T> phi_bb320_40;
  TNode<Int32T> phi_bb320_41;
  TNode<Int32T> tmp659;
      TNode<Object> tmp661;
  TNode<Int32T> tmp662;
      TNode<Object> tmp664;
  TNode<Int32T> tmp665;
      TNode<Object> tmp667;
  TNode<Int32T> tmp668;
      TNode<Object> tmp670;
  if (block320.is_used()) {
    ca_.Bind(&block320, &phi_bb320_17, &phi_bb320_18, &phi_bb320_19, &phi_bb320_27, &phi_bb320_28, &phi_bb320_29, &phi_bb320_30, &phi_bb320_31, &phi_bb320_34, &phi_bb320_35, &phi_bb320_36, &phi_bb320_37, &phi_bb320_38, &phi_bb320_40, &phi_bb320_41);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 758);
    compiler::CodeAssemblerExceptionHandlerLabel catch660__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch660__label);
    tmp659 = FromConstexpr_int32_constexpr_int31_0(state_, 1);
    }
    if (catch660__label.is_used()) {
      compiler::CodeAssemblerLabel catch660_skip(&ca_);
      ca_.Goto(&catch660_skip);
      ca_.Bind(&catch660__label, &tmp661);
      ca_.Goto(&block330, phi_bb320_17, phi_bb320_18, phi_bb320_19, phi_bb320_27, phi_bb320_28, phi_bb320_29, phi_bb320_30, phi_bb320_31, phi_bb320_34, phi_bb320_35, phi_bb320_36, phi_bb320_37, phi_bb320_38, phi_bb320_40, phi_bb320_41, phi_bb320_40, phi_bb320_40);
      ca_.Bind(&catch660_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch663__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch663__label);
    tmp662 = CodeStubAssembler(state_).Int32Add(TNode<Int32T>{phi_bb320_40}, TNode<Int32T>{tmp659});
    }
    if (catch663__label.is_used()) {
      compiler::CodeAssemblerLabel catch663_skip(&ca_);
      ca_.Goto(&catch663_skip);
      ca_.Bind(&catch663__label, &tmp664);
      ca_.Goto(&block331, phi_bb320_17, phi_bb320_18, phi_bb320_19, phi_bb320_27, phi_bb320_28, phi_bb320_29, phi_bb320_30, phi_bb320_31, phi_bb320_34, phi_bb320_35, phi_bb320_36, phi_bb320_37, phi_bb320_38, phi_bb320_40, phi_bb320_41, phi_bb320_40);
      ca_.Bind(&catch663_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 759);
    compiler::CodeAssemblerExceptionHandlerLabel catch666__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch666__label);
    tmp665 = FromConstexpr_int32_constexpr_int31_0(state_, 1);
    }
    if (catch666__label.is_used()) {
      compiler::CodeAssemblerLabel catch666_skip(&ca_);
      ca_.Goto(&catch666_skip);
      ca_.Bind(&catch666__label, &tmp667);
      ca_.Goto(&block332, phi_bb320_17, phi_bb320_18, phi_bb320_19, phi_bb320_27, phi_bb320_28, phi_bb320_29, phi_bb320_30, phi_bb320_31, phi_bb320_34, phi_bb320_35, phi_bb320_36, phi_bb320_37, phi_bb320_38, phi_bb320_41, phi_bb320_41, phi_bb320_41);
      ca_.Bind(&catch666_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch669__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch669__label);
    tmp668 = CodeStubAssembler(state_).Int32Add(TNode<Int32T>{phi_bb320_41}, TNode<Int32T>{tmp665});
    }
    if (catch669__label.is_used()) {
      compiler::CodeAssemblerLabel catch669_skip(&ca_);
      ca_.Goto(&catch669_skip);
      ca_.Bind(&catch669__label, &tmp670);
      ca_.Goto(&block333, phi_bb320_17, phi_bb320_18, phi_bb320_19, phi_bb320_27, phi_bb320_28, phi_bb320_29, phi_bb320_30, phi_bb320_31, phi_bb320_34, phi_bb320_35, phi_bb320_36, phi_bb320_37, phi_bb320_38, phi_bb320_41, phi_bb320_41);
      ca_.Bind(&catch669_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 760);
    ca_.Goto(&block314, phi_bb320_17, phi_bb320_18, phi_bb320_19, phi_bb320_27, phi_bb320_28, phi_bb320_29, phi_bb320_30, phi_bb320_31, phi_bb320_34, phi_bb320_35, phi_bb320_36, phi_bb320_37, phi_bb320_38, tmp662, tmp668);
  }

  TNode<Object> phi_bb330_17;
  TNode<IntPtrT> phi_bb330_18;
  TNode<BoolT> phi_bb330_19;
  TNode<IntPtrT> phi_bb330_27;
  TNode<IntPtrT> phi_bb330_28;
  TNode<IntPtrT> phi_bb330_29;
  TNode<IntPtrT> phi_bb330_30;
  TNode<IntPtrT> phi_bb330_31;
  TNode<IntPtrT> phi_bb330_34;
  TNode<BoolT> phi_bb330_35;
  TNode<BoolT> phi_bb330_36;
  TNode<Object> phi_bb330_37;
  TNode<Int32T> phi_bb330_38;
  TNode<Int32T> phi_bb330_40;
  TNode<Int32T> phi_bb330_41;
  TNode<Int32T> phi_bb330_44;
  TNode<Int32T> phi_bb330_45;
  TNode<HeapObject> tmp671;
  if (block330.is_used()) {
    ca_.Bind(&block330, &phi_bb330_17, &phi_bb330_18, &phi_bb330_19, &phi_bb330_27, &phi_bb330_28, &phi_bb330_29, &phi_bb330_30, &phi_bb330_31, &phi_bb330_34, &phi_bb330_35, &phi_bb330_36, &phi_bb330_37, &phi_bb330_38, &phi_bb330_40, &phi_bb330_41, &phi_bb330_44, &phi_bb330_45);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 758);
    tmp671 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb330_17, phi_bb330_18, phi_bb330_19, tmp661, tmp671);
  }

  TNode<Object> phi_bb331_17;
  TNode<IntPtrT> phi_bb331_18;
  TNode<BoolT> phi_bb331_19;
  TNode<IntPtrT> phi_bb331_27;
  TNode<IntPtrT> phi_bb331_28;
  TNode<IntPtrT> phi_bb331_29;
  TNode<IntPtrT> phi_bb331_30;
  TNode<IntPtrT> phi_bb331_31;
  TNode<IntPtrT> phi_bb331_34;
  TNode<BoolT> phi_bb331_35;
  TNode<BoolT> phi_bb331_36;
  TNode<Object> phi_bb331_37;
  TNode<Int32T> phi_bb331_38;
  TNode<Int32T> phi_bb331_40;
  TNode<Int32T> phi_bb331_41;
  TNode<Int32T> phi_bb331_44;
  TNode<HeapObject> tmp672;
  if (block331.is_used()) {
    ca_.Bind(&block331, &phi_bb331_17, &phi_bb331_18, &phi_bb331_19, &phi_bb331_27, &phi_bb331_28, &phi_bb331_29, &phi_bb331_30, &phi_bb331_31, &phi_bb331_34, &phi_bb331_35, &phi_bb331_36, &phi_bb331_37, &phi_bb331_38, &phi_bb331_40, &phi_bb331_41, &phi_bb331_44);
    tmp672 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb331_17, phi_bb331_18, phi_bb331_19, tmp664, tmp672);
  }

  TNode<Object> phi_bb332_17;
  TNode<IntPtrT> phi_bb332_18;
  TNode<BoolT> phi_bb332_19;
  TNode<IntPtrT> phi_bb332_27;
  TNode<IntPtrT> phi_bb332_28;
  TNode<IntPtrT> phi_bb332_29;
  TNode<IntPtrT> phi_bb332_30;
  TNode<IntPtrT> phi_bb332_31;
  TNode<IntPtrT> phi_bb332_34;
  TNode<BoolT> phi_bb332_35;
  TNode<BoolT> phi_bb332_36;
  TNode<Object> phi_bb332_37;
  TNode<Int32T> phi_bb332_38;
  TNode<Int32T> phi_bb332_41;
  TNode<Int32T> phi_bb332_44;
  TNode<Int32T> phi_bb332_45;
  TNode<HeapObject> tmp673;
  if (block332.is_used()) {
    ca_.Bind(&block332, &phi_bb332_17, &phi_bb332_18, &phi_bb332_19, &phi_bb332_27, &phi_bb332_28, &phi_bb332_29, &phi_bb332_30, &phi_bb332_31, &phi_bb332_34, &phi_bb332_35, &phi_bb332_36, &phi_bb332_37, &phi_bb332_38, &phi_bb332_41, &phi_bb332_44, &phi_bb332_45);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 759);
    tmp673 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb332_17, phi_bb332_18, phi_bb332_19, tmp667, tmp673);
  }

  TNode<Object> phi_bb333_17;
  TNode<IntPtrT> phi_bb333_18;
  TNode<BoolT> phi_bb333_19;
  TNode<IntPtrT> phi_bb333_27;
  TNode<IntPtrT> phi_bb333_28;
  TNode<IntPtrT> phi_bb333_29;
  TNode<IntPtrT> phi_bb333_30;
  TNode<IntPtrT> phi_bb333_31;
  TNode<IntPtrT> phi_bb333_34;
  TNode<BoolT> phi_bb333_35;
  TNode<BoolT> phi_bb333_36;
  TNode<Object> phi_bb333_37;
  TNode<Int32T> phi_bb333_38;
  TNode<Int32T> phi_bb333_41;
  TNode<Int32T> phi_bb333_44;
  TNode<HeapObject> tmp674;
  if (block333.is_used()) {
    ca_.Bind(&block333, &phi_bb333_17, &phi_bb333_18, &phi_bb333_19, &phi_bb333_27, &phi_bb333_28, &phi_bb333_29, &phi_bb333_30, &phi_bb333_31, &phi_bb333_34, &phi_bb333_35, &phi_bb333_36, &phi_bb333_37, &phi_bb333_38, &phi_bb333_41, &phi_bb333_44);
    tmp674 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb333_17, phi_bb333_18, phi_bb333_19, tmp670, tmp674);
  }

  TNode<Object> phi_bb321_17;
  TNode<IntPtrT> phi_bb321_18;
  TNode<BoolT> phi_bb321_19;
  TNode<IntPtrT> phi_bb321_27;
  TNode<IntPtrT> phi_bb321_28;
  TNode<IntPtrT> phi_bb321_29;
  TNode<IntPtrT> phi_bb321_30;
  TNode<IntPtrT> phi_bb321_31;
  TNode<IntPtrT> phi_bb321_34;
  TNode<BoolT> phi_bb321_35;
  TNode<BoolT> phi_bb321_36;
  TNode<Object> phi_bb321_37;
  TNode<Int32T> phi_bb321_38;
  TNode<Int32T> phi_bb321_40;
  TNode<Int32T> phi_bb321_41;
  TNode<FixedArray> tmp675;
      TNode<Object> tmp677;
  TNode<Object> tmp678;
  TNode<IntPtrT> tmp679;
  TNode<IntPtrT> tmp680;
      TNode<Object> tmp682;
  TNode<Int32T> tmp683;
      TNode<Object> tmp685;
  TNode<Int32T> tmp686;
      TNode<Object> tmp688;
  TNode<IntPtrT> tmp689;
      TNode<Object> tmp691;
  TNode<UintPtrT> tmp692;
  TNode<UintPtrT> tmp693;
  TNode<BoolT> tmp694;
  if (block321.is_used()) {
    ca_.Bind(&block321, &phi_bb321_17, &phi_bb321_18, &phi_bb321_19, &phi_bb321_27, &phi_bb321_28, &phi_bb321_29, &phi_bb321_30, &phi_bb321_31, &phi_bb321_34, &phi_bb321_35, &phi_bb321_36, &phi_bb321_37, &phi_bb321_38, &phi_bb321_40, &phi_bb321_41);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 762);
    compiler::CodeAssemblerExceptionHandlerLabel catch676__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch676__label);
    tmp675 = UnsafeCast_FixedArray_0(state_, TNode<Context>{p_context}, TNode<Object>{phi_bb321_37});
    }
    if (catch676__label.is_used()) {
      compiler::CodeAssemblerLabel catch676_skip(&ca_);
      ca_.Goto(&catch676_skip);
      ca_.Bind(&catch676__label, &tmp677);
      ca_.Goto(&block334, phi_bb321_17, phi_bb321_18, phi_bb321_19, phi_bb321_27, phi_bb321_28, phi_bb321_29, phi_bb321_30, phi_bb321_31, phi_bb321_34, phi_bb321_35, phi_bb321_36, phi_bb321_37, phi_bb321_38, phi_bb321_40, phi_bb321_41, phi_bb321_37);
      ca_.Bind(&catch676_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch681__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch681__label);
    std::tie(tmp678, tmp679, tmp680) = FieldSliceFixedArrayObjects_0(state_, TNode<FixedArray>{tmp675}).Flatten();
    }
    if (catch681__label.is_used()) {
      compiler::CodeAssemblerLabel catch681_skip(&ca_);
      ca_.Goto(&catch681_skip);
      ca_.Bind(&catch681__label, &tmp682);
      ca_.Goto(&block335, phi_bb321_17, phi_bb321_18, phi_bb321_19, phi_bb321_27, phi_bb321_28, phi_bb321_29, phi_bb321_30, phi_bb321_31, phi_bb321_34, phi_bb321_35, phi_bb321_36, phi_bb321_37, phi_bb321_38, phi_bb321_40, phi_bb321_41);
      ca_.Bind(&catch681_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 763);
    compiler::CodeAssemblerExceptionHandlerLabel catch684__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch684__label);
    tmp683 = FromConstexpr_int32_constexpr_int31_0(state_, 1);
    }
    if (catch684__label.is_used()) {
      compiler::CodeAssemblerLabel catch684_skip(&ca_);
      ca_.Goto(&catch684_skip);
      ca_.Bind(&catch684__label, &tmp685);
      ca_.Goto(&block336, phi_bb321_17, phi_bb321_18, phi_bb321_19, phi_bb321_27, phi_bb321_28, phi_bb321_29, phi_bb321_30, phi_bb321_31, phi_bb321_34, phi_bb321_35, phi_bb321_36, phi_bb321_37, phi_bb321_38, phi_bb321_40, phi_bb321_41, phi_bb321_41, phi_bb321_41);
      ca_.Bind(&catch684_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch687__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch687__label);
    tmp686 = CodeStubAssembler(state_).Int32Add(TNode<Int32T>{phi_bb321_41}, TNode<Int32T>{tmp683});
    }
    if (catch687__label.is_used()) {
      compiler::CodeAssemblerLabel catch687_skip(&ca_);
      ca_.Goto(&catch687_skip);
      ca_.Bind(&catch687__label, &tmp688);
      ca_.Goto(&block337, phi_bb321_17, phi_bb321_18, phi_bb321_19, phi_bb321_27, phi_bb321_28, phi_bb321_29, phi_bb321_30, phi_bb321_31, phi_bb321_34, phi_bb321_35, phi_bb321_36, phi_bb321_37, phi_bb321_38, phi_bb321_40, phi_bb321_41, phi_bb321_41);
      ca_.Bind(&catch687_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch690__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch690__label);
    tmp689 = Convert_intptr_int32_0(state_, TNode<Int32T>{phi_bb321_41});
    }
    if (catch690__label.is_used()) {
      compiler::CodeAssemblerLabel catch690_skip(&ca_);
      ca_.Goto(&catch690_skip);
      ca_.Bind(&catch690__label, &tmp691);
      ca_.Goto(&block338, phi_bb321_17, phi_bb321_18, phi_bb321_19, phi_bb321_27, phi_bb321_28, phi_bb321_29, phi_bb321_30, phi_bb321_31, phi_bb321_34, phi_bb321_35, phi_bb321_36, phi_bb321_37, phi_bb321_38, phi_bb321_40, phi_bb321_41);
      ca_.Bind(&catch690_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 101);
    tmp692 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{tmp689});
    tmp693 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{tmp680});
    tmp694 = CodeStubAssembler(state_).UintPtrLessThan(TNode<UintPtrT>{tmp692}, TNode<UintPtrT>{tmp693});
    ca_.Branch(tmp694, &block343, std::vector<compiler::Node*>{phi_bb321_17, phi_bb321_18, phi_bb321_19, phi_bb321_27, phi_bb321_28, phi_bb321_29, phi_bb321_30, phi_bb321_31, phi_bb321_34, phi_bb321_35, phi_bb321_36, phi_bb321_37, phi_bb321_38, phi_bb321_40}, &block344, std::vector<compiler::Node*>{phi_bb321_17, phi_bb321_18, phi_bb321_19, phi_bb321_27, phi_bb321_28, phi_bb321_29, phi_bb321_30, phi_bb321_31, phi_bb321_34, phi_bb321_35, phi_bb321_36, phi_bb321_37, phi_bb321_38, phi_bb321_40});
  }

  TNode<Object> phi_bb334_17;
  TNode<IntPtrT> phi_bb334_18;
  TNode<BoolT> phi_bb334_19;
  TNode<IntPtrT> phi_bb334_27;
  TNode<IntPtrT> phi_bb334_28;
  TNode<IntPtrT> phi_bb334_29;
  TNode<IntPtrT> phi_bb334_30;
  TNode<IntPtrT> phi_bb334_31;
  TNode<IntPtrT> phi_bb334_34;
  TNode<BoolT> phi_bb334_35;
  TNode<BoolT> phi_bb334_36;
  TNode<Object> phi_bb334_37;
  TNode<Int32T> phi_bb334_38;
  TNode<Int32T> phi_bb334_40;
  TNode<Int32T> phi_bb334_41;
  TNode<Object> phi_bb334_44;
  TNode<HeapObject> tmp695;
  if (block334.is_used()) {
    ca_.Bind(&block334, &phi_bb334_17, &phi_bb334_18, &phi_bb334_19, &phi_bb334_27, &phi_bb334_28, &phi_bb334_29, &phi_bb334_30, &phi_bb334_31, &phi_bb334_34, &phi_bb334_35, &phi_bb334_36, &phi_bb334_37, &phi_bb334_38, &phi_bb334_40, &phi_bb334_41, &phi_bb334_44);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 762);
    tmp695 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb334_17, phi_bb334_18, phi_bb334_19, tmp677, tmp695);
  }

  TNode<Object> phi_bb335_17;
  TNode<IntPtrT> phi_bb335_18;
  TNode<BoolT> phi_bb335_19;
  TNode<IntPtrT> phi_bb335_27;
  TNode<IntPtrT> phi_bb335_28;
  TNode<IntPtrT> phi_bb335_29;
  TNode<IntPtrT> phi_bb335_30;
  TNode<IntPtrT> phi_bb335_31;
  TNode<IntPtrT> phi_bb335_34;
  TNode<BoolT> phi_bb335_35;
  TNode<BoolT> phi_bb335_36;
  TNode<Object> phi_bb335_37;
  TNode<Int32T> phi_bb335_38;
  TNode<Int32T> phi_bb335_40;
  TNode<Int32T> phi_bb335_41;
  TNode<HeapObject> tmp696;
  if (block335.is_used()) {
    ca_.Bind(&block335, &phi_bb335_17, &phi_bb335_18, &phi_bb335_19, &phi_bb335_27, &phi_bb335_28, &phi_bb335_29, &phi_bb335_30, &phi_bb335_31, &phi_bb335_34, &phi_bb335_35, &phi_bb335_36, &phi_bb335_37, &phi_bb335_38, &phi_bb335_40, &phi_bb335_41);
    tmp696 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb335_17, phi_bb335_18, phi_bb335_19, tmp682, tmp696);
  }

  TNode<Object> phi_bb336_17;
  TNode<IntPtrT> phi_bb336_18;
  TNode<BoolT> phi_bb336_19;
  TNode<IntPtrT> phi_bb336_27;
  TNode<IntPtrT> phi_bb336_28;
  TNode<IntPtrT> phi_bb336_29;
  TNode<IntPtrT> phi_bb336_30;
  TNode<IntPtrT> phi_bb336_31;
  TNode<IntPtrT> phi_bb336_34;
  TNode<BoolT> phi_bb336_35;
  TNode<BoolT> phi_bb336_36;
  TNode<Object> phi_bb336_37;
  TNode<Int32T> phi_bb336_38;
  TNode<Int32T> phi_bb336_40;
  TNode<Int32T> phi_bb336_41;
  TNode<Int32T> phi_bb336_49;
  TNode<Int32T> phi_bb336_50;
  TNode<HeapObject> tmp697;
  if (block336.is_used()) {
    ca_.Bind(&block336, &phi_bb336_17, &phi_bb336_18, &phi_bb336_19, &phi_bb336_27, &phi_bb336_28, &phi_bb336_29, &phi_bb336_30, &phi_bb336_31, &phi_bb336_34, &phi_bb336_35, &phi_bb336_36, &phi_bb336_37, &phi_bb336_38, &phi_bb336_40, &phi_bb336_41, &phi_bb336_49, &phi_bb336_50);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 763);
    tmp697 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb336_17, phi_bb336_18, phi_bb336_19, tmp685, tmp697);
  }

  TNode<Object> phi_bb337_17;
  TNode<IntPtrT> phi_bb337_18;
  TNode<BoolT> phi_bb337_19;
  TNode<IntPtrT> phi_bb337_27;
  TNode<IntPtrT> phi_bb337_28;
  TNode<IntPtrT> phi_bb337_29;
  TNode<IntPtrT> phi_bb337_30;
  TNode<IntPtrT> phi_bb337_31;
  TNode<IntPtrT> phi_bb337_34;
  TNode<BoolT> phi_bb337_35;
  TNode<BoolT> phi_bb337_36;
  TNode<Object> phi_bb337_37;
  TNode<Int32T> phi_bb337_38;
  TNode<Int32T> phi_bb337_40;
  TNode<Int32T> phi_bb337_41;
  TNode<Int32T> phi_bb337_49;
  TNode<HeapObject> tmp698;
  if (block337.is_used()) {
    ca_.Bind(&block337, &phi_bb337_17, &phi_bb337_18, &phi_bb337_19, &phi_bb337_27, &phi_bb337_28, &phi_bb337_29, &phi_bb337_30, &phi_bb337_31, &phi_bb337_34, &phi_bb337_35, &phi_bb337_36, &phi_bb337_37, &phi_bb337_38, &phi_bb337_40, &phi_bb337_41, &phi_bb337_49);
    tmp698 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb337_17, phi_bb337_18, phi_bb337_19, tmp688, tmp698);
  }

  TNode<Object> phi_bb338_17;
  TNode<IntPtrT> phi_bb338_18;
  TNode<BoolT> phi_bb338_19;
  TNode<IntPtrT> phi_bb338_27;
  TNode<IntPtrT> phi_bb338_28;
  TNode<IntPtrT> phi_bb338_29;
  TNode<IntPtrT> phi_bb338_30;
  TNode<IntPtrT> phi_bb338_31;
  TNode<IntPtrT> phi_bb338_34;
  TNode<BoolT> phi_bb338_35;
  TNode<BoolT> phi_bb338_36;
  TNode<Object> phi_bb338_37;
  TNode<Int32T> phi_bb338_38;
  TNode<Int32T> phi_bb338_40;
  TNode<Int32T> phi_bb338_49;
  TNode<HeapObject> tmp699;
  if (block338.is_used()) {
    ca_.Bind(&block338, &phi_bb338_17, &phi_bb338_18, &phi_bb338_19, &phi_bb338_27, &phi_bb338_28, &phi_bb338_29, &phi_bb338_30, &phi_bb338_31, &phi_bb338_34, &phi_bb338_35, &phi_bb338_36, &phi_bb338_37, &phi_bb338_38, &phi_bb338_40, &phi_bb338_49);
    tmp699 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb338_17, phi_bb338_18, phi_bb338_19, tmp691, tmp699);
  }

  TNode<Object> phi_bb343_17;
  TNode<IntPtrT> phi_bb343_18;
  TNode<BoolT> phi_bb343_19;
  TNode<IntPtrT> phi_bb343_27;
  TNode<IntPtrT> phi_bb343_28;
  TNode<IntPtrT> phi_bb343_29;
  TNode<IntPtrT> phi_bb343_30;
  TNode<IntPtrT> phi_bb343_31;
  TNode<IntPtrT> phi_bb343_34;
  TNode<BoolT> phi_bb343_35;
  TNode<BoolT> phi_bb343_36;
  TNode<Object> phi_bb343_37;
  TNode<Int32T> phi_bb343_38;
  TNode<Int32T> phi_bb343_40;
  TNode<IntPtrT> tmp700;
  TNode<IntPtrT> tmp701;
  TNode<Object> tmp702;
  TNode<IntPtrT> tmp703;
  TNode<Object> tmp704;
  TNode<IntPtrT> tmp705;
  TNode<IntPtrT> tmp706;
  TNode<IntPtrT> tmp707;
  TNode<BoolT> tmp708;
  if (block343.is_used()) {
    ca_.Bind(&block343, &phi_bb343_17, &phi_bb343_18, &phi_bb343_19, &phi_bb343_27, &phi_bb343_28, &phi_bb343_29, &phi_bb343_30, &phi_bb343_31, &phi_bb343_34, &phi_bb343_35, &phi_bb343_36, &phi_bb343_37, &phi_bb343_38, &phi_bb343_40);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp700 = TimesSizeOf_Object_0(state_, TNode<IntPtrT>{tmp689});
    tmp701 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp679}, TNode<IntPtrT>{tmp700});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp702, tmp703) = NewReference_Object_0(state_, TNode<Object>{tmp678}, TNode<IntPtrT>{tmp701}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 762);
    tmp704 = CodeStubAssembler(state_).LoadReference<Object>(CodeStubAssembler::Reference{tmp702, tmp703});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 308);
    tmp705 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp706 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb343_27}, TNode<IntPtrT>{tmp705});
    tmp707 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp708 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb343_27}, TNode<IntPtrT>{tmp707});
    ca_.Branch(tmp708, &block348, std::vector<compiler::Node*>{phi_bb343_17, phi_bb343_18, phi_bb343_19, phi_bb343_28, phi_bb343_29, phi_bb343_30, phi_bb343_31, phi_bb343_34, phi_bb343_35, phi_bb343_36, phi_bb343_37, phi_bb343_38, phi_bb343_40}, &block349, std::vector<compiler::Node*>{phi_bb343_17, phi_bb343_18, phi_bb343_19, phi_bb343_28, phi_bb343_29, phi_bb343_30, phi_bb343_31, phi_bb343_34, phi_bb343_35, phi_bb343_36, phi_bb343_37, phi_bb343_38, phi_bb343_40});
  }

  TNode<Object> phi_bb344_17;
  TNode<IntPtrT> phi_bb344_18;
  TNode<BoolT> phi_bb344_19;
  TNode<IntPtrT> phi_bb344_27;
  TNode<IntPtrT> phi_bb344_28;
  TNode<IntPtrT> phi_bb344_29;
  TNode<IntPtrT> phi_bb344_30;
  TNode<IntPtrT> phi_bb344_31;
  TNode<IntPtrT> phi_bb344_34;
  TNode<BoolT> phi_bb344_35;
  TNode<BoolT> phi_bb344_36;
  TNode<Object> phi_bb344_37;
  TNode<Int32T> phi_bb344_38;
  TNode<Int32T> phi_bb344_40;
  if (block344.is_used()) {
    ca_.Bind(&block344, &phi_bb344_17, &phi_bb344_18, &phi_bb344_19, &phi_bb344_27, &phi_bb344_28, &phi_bb344_29, &phi_bb344_30, &phi_bb344_31, &phi_bb344_34, &phi_bb344_35, &phi_bb344_36, &phi_bb344_37, &phi_bb344_38, &phi_bb344_40);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 113);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<Object> phi_bb348_17;
  TNode<IntPtrT> phi_bb348_18;
  TNode<BoolT> phi_bb348_19;
  TNode<IntPtrT> phi_bb348_28;
  TNode<IntPtrT> phi_bb348_29;
  TNode<IntPtrT> phi_bb348_30;
  TNode<IntPtrT> phi_bb348_31;
  TNode<IntPtrT> phi_bb348_34;
  TNode<BoolT> phi_bb348_35;
  TNode<BoolT> phi_bb348_36;
  TNode<Object> phi_bb348_37;
  TNode<Int32T> phi_bb348_38;
  TNode<Int32T> phi_bb348_40;
  TNode<Object> tmp709;
  TNode<IntPtrT> tmp710;
  TNode<IntPtrT> tmp711;
  TNode<IntPtrT> tmp712;
  if (block348.is_used()) {
    ca_.Bind(&block348, &phi_bb348_17, &phi_bb348_18, &phi_bb348_19, &phi_bb348_28, &phi_bb348_29, &phi_bb348_30, &phi_bb348_31, &phi_bb348_34, &phi_bb348_35, &phi_bb348_36, &phi_bb348_37, &phi_bb348_38, &phi_bb348_40);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    std::tie(tmp709, tmp710) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb348_29}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 311);
    tmp711 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp712 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb348_29}, TNode<IntPtrT>{tmp711});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 312);
    ca_.Goto(&block347, phi_bb348_17, phi_bb348_18, phi_bb348_19, phi_bb348_28, tmp712, phi_bb348_30, phi_bb348_31, phi_bb348_34, phi_bb348_35, phi_bb348_36, phi_bb348_37, phi_bb348_38, phi_bb348_40, tmp709, tmp710);
  }

  TNode<Object> phi_bb349_17;
  TNode<IntPtrT> phi_bb349_18;
  TNode<BoolT> phi_bb349_19;
  TNode<IntPtrT> phi_bb349_28;
  TNode<IntPtrT> phi_bb349_29;
  TNode<IntPtrT> phi_bb349_30;
  TNode<IntPtrT> phi_bb349_31;
  TNode<IntPtrT> phi_bb349_34;
  TNode<BoolT> phi_bb349_35;
  TNode<BoolT> phi_bb349_36;
  TNode<Object> phi_bb349_37;
  TNode<Int32T> phi_bb349_38;
  TNode<Int32T> phi_bb349_40;
  if (block349.is_used()) {
    ca_.Bind(&block349, &phi_bb349_17, &phi_bb349_18, &phi_bb349_19, &phi_bb349_28, &phi_bb349_29, &phi_bb349_30, &phi_bb349_31, &phi_bb349_34, &phi_bb349_35, &phi_bb349_36, &phi_bb349_37, &phi_bb349_38, &phi_bb349_40);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block351, phi_bb349_17, phi_bb349_18, phi_bb349_19, phi_bb349_28, phi_bb349_29, phi_bb349_30, phi_bb349_31, phi_bb349_34, phi_bb349_35, phi_bb349_36, phi_bb349_37, phi_bb349_38, phi_bb349_40);
    } else {
      ca_.Goto(&block352, phi_bb349_17, phi_bb349_18, phi_bb349_19, phi_bb349_28, phi_bb349_29, phi_bb349_30, phi_bb349_31, phi_bb349_34, phi_bb349_35, phi_bb349_36, phi_bb349_37, phi_bb349_38, phi_bb349_40);
    }
  }

  TNode<Object> phi_bb351_17;
  TNode<IntPtrT> phi_bb351_18;
  TNode<BoolT> phi_bb351_19;
  TNode<IntPtrT> phi_bb351_28;
  TNode<IntPtrT> phi_bb351_29;
  TNode<IntPtrT> phi_bb351_30;
  TNode<IntPtrT> phi_bb351_31;
  TNode<IntPtrT> phi_bb351_34;
  TNode<BoolT> phi_bb351_35;
  TNode<BoolT> phi_bb351_36;
  TNode<Object> phi_bb351_37;
  TNode<Int32T> phi_bb351_38;
  TNode<Int32T> phi_bb351_40;
  TNode<Object> tmp713;
  TNode<IntPtrT> tmp714;
  TNode<IntPtrT> tmp715;
  TNode<IntPtrT> tmp716;
  if (block351.is_used()) {
    ca_.Bind(&block351, &phi_bb351_17, &phi_bb351_18, &phi_bb351_19, &phi_bb351_28, &phi_bb351_29, &phi_bb351_30, &phi_bb351_31, &phi_bb351_34, &phi_bb351_35, &phi_bb351_36, &phi_bb351_37, &phi_bb351_38, &phi_bb351_40);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp713, tmp714) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb351_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp715 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp716 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb351_31}, TNode<IntPtrT>{tmp715});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block350, phi_bb351_17, phi_bb351_18, phi_bb351_19, phi_bb351_28, phi_bb351_29, phi_bb351_30, tmp716, phi_bb351_34, phi_bb351_35, phi_bb351_36, phi_bb351_37, phi_bb351_38, phi_bb351_40, tmp713, tmp714);
  }

  TNode<Object> phi_bb352_17;
  TNode<IntPtrT> phi_bb352_18;
  TNode<BoolT> phi_bb352_19;
  TNode<IntPtrT> phi_bb352_28;
  TNode<IntPtrT> phi_bb352_29;
  TNode<IntPtrT> phi_bb352_30;
  TNode<IntPtrT> phi_bb352_31;
  TNode<IntPtrT> phi_bb352_34;
  TNode<BoolT> phi_bb352_35;
  TNode<BoolT> phi_bb352_36;
  TNode<Object> phi_bb352_37;
  TNode<Int32T> phi_bb352_38;
  TNode<Int32T> phi_bb352_40;
  TNode<IntPtrT> tmp717;
  TNode<BoolT> tmp718;
  if (block352.is_used()) {
    ca_.Bind(&block352, &phi_bb352_17, &phi_bb352_18, &phi_bb352_19, &phi_bb352_28, &phi_bb352_29, &phi_bb352_30, &phi_bb352_31, &phi_bb352_34, &phi_bb352_35, &phi_bb352_36, &phi_bb352_37, &phi_bb352_38, &phi_bb352_40);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp717 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp718 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb352_34}, TNode<IntPtrT>{tmp717});
    ca_.Branch(tmp718, &block354, std::vector<compiler::Node*>{phi_bb352_17, phi_bb352_18, phi_bb352_19, phi_bb352_28, phi_bb352_29, phi_bb352_30, phi_bb352_31, phi_bb352_34, phi_bb352_35, phi_bb352_36, phi_bb352_37, phi_bb352_38, phi_bb352_40}, &block355, std::vector<compiler::Node*>{phi_bb352_17, phi_bb352_18, phi_bb352_19, phi_bb352_28, phi_bb352_29, phi_bb352_30, phi_bb352_31, phi_bb352_34, phi_bb352_35, phi_bb352_36, phi_bb352_37, phi_bb352_38, phi_bb352_40});
  }

  TNode<Object> phi_bb354_17;
  TNode<IntPtrT> phi_bb354_18;
  TNode<BoolT> phi_bb354_19;
  TNode<IntPtrT> phi_bb354_28;
  TNode<IntPtrT> phi_bb354_29;
  TNode<IntPtrT> phi_bb354_30;
  TNode<IntPtrT> phi_bb354_31;
  TNode<IntPtrT> phi_bb354_34;
  TNode<BoolT> phi_bb354_35;
  TNode<BoolT> phi_bb354_36;
  TNode<Object> phi_bb354_37;
  TNode<Int32T> phi_bb354_38;
  TNode<Int32T> phi_bb354_40;
  TNode<Object> tmp719;
  TNode<IntPtrT> tmp720;
  TNode<IntPtrT> tmp721;
  TNode<BoolT> tmp722;
  if (block354.is_used()) {
    ca_.Bind(&block354, &phi_bb354_17, &phi_bb354_18, &phi_bb354_19, &phi_bb354_28, &phi_bb354_29, &phi_bb354_30, &phi_bb354_31, &phi_bb354_34, &phi_bb354_35, &phi_bb354_36, &phi_bb354_37, &phi_bb354_38, &phi_bb354_40);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp719, tmp720) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb354_34}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp721 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp722 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block350, phi_bb354_17, phi_bb354_18, phi_bb354_19, phi_bb354_28, phi_bb354_29, phi_bb354_30, phi_bb354_31, tmp721, tmp722, phi_bb354_36, phi_bb354_37, phi_bb354_38, phi_bb354_40, tmp719, tmp720);
  }

  TNode<Object> phi_bb355_17;
  TNode<IntPtrT> phi_bb355_18;
  TNode<BoolT> phi_bb355_19;
  TNode<IntPtrT> phi_bb355_28;
  TNode<IntPtrT> phi_bb355_29;
  TNode<IntPtrT> phi_bb355_30;
  TNode<IntPtrT> phi_bb355_31;
  TNode<IntPtrT> phi_bb355_34;
  TNode<BoolT> phi_bb355_35;
  TNode<BoolT> phi_bb355_36;
  TNode<Object> phi_bb355_37;
  TNode<Int32T> phi_bb355_38;
  TNode<Int32T> phi_bb355_40;
  TNode<Object> tmp723;
  TNode<IntPtrT> tmp724;
  TNode<IntPtrT> tmp725;
  TNode<IntPtrT> tmp726;
  TNode<IntPtrT> tmp727;
  TNode<IntPtrT> tmp728;
  TNode<BoolT> tmp729;
  if (block355.is_used()) {
    ca_.Bind(&block355, &phi_bb355_17, &phi_bb355_18, &phi_bb355_19, &phi_bb355_28, &phi_bb355_29, &phi_bb355_30, &phi_bb355_31, &phi_bb355_34, &phi_bb355_35, &phi_bb355_36, &phi_bb355_37, &phi_bb355_38, &phi_bb355_40);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp723, tmp724) = NewReference_intptr_0(state_, TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb355_31}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp725 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp726 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb355_31}, TNode<IntPtrT>{tmp725});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp727 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp728 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp726}, TNode<IntPtrT>{tmp727});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp729 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block350, phi_bb355_17, phi_bb355_18, phi_bb355_19, phi_bb355_28, phi_bb355_29, phi_bb355_30, tmp728, tmp726, tmp729, phi_bb355_36, phi_bb355_37, phi_bb355_38, phi_bb355_40, tmp723, tmp724);
  }

  TNode<Object> phi_bb350_17;
  TNode<IntPtrT> phi_bb350_18;
  TNode<BoolT> phi_bb350_19;
  TNode<IntPtrT> phi_bb350_28;
  TNode<IntPtrT> phi_bb350_29;
  TNode<IntPtrT> phi_bb350_30;
  TNode<IntPtrT> phi_bb350_31;
  TNode<IntPtrT> phi_bb350_34;
  TNode<BoolT> phi_bb350_35;
  TNode<BoolT> phi_bb350_36;
  TNode<Object> phi_bb350_37;
  TNode<Int32T> phi_bb350_38;
  TNode<Int32T> phi_bb350_40;
  TNode<Object> phi_bb350_45;
  TNode<IntPtrT> phi_bb350_46;
  if (block350.is_used()) {
    ca_.Bind(&block350, &phi_bb350_17, &phi_bb350_18, &phi_bb350_19, &phi_bb350_28, &phi_bb350_29, &phi_bb350_30, &phi_bb350_31, &phi_bb350_34, &phi_bb350_35, &phi_bb350_36, &phi_bb350_37, &phi_bb350_38, &phi_bb350_40, &phi_bb350_45, &phi_bb350_46);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 314);
    ca_.Goto(&block347, phi_bb350_17, phi_bb350_18, phi_bb350_19, phi_bb350_28, phi_bb350_29, phi_bb350_30, phi_bb350_31, phi_bb350_34, phi_bb350_35, phi_bb350_36, phi_bb350_37, phi_bb350_38, phi_bb350_40, phi_bb350_45, phi_bb350_46);
  }

  TNode<Object> phi_bb347_17;
  TNode<IntPtrT> phi_bb347_18;
  TNode<BoolT> phi_bb347_19;
  TNode<IntPtrT> phi_bb347_28;
  TNode<IntPtrT> phi_bb347_29;
  TNode<IntPtrT> phi_bb347_30;
  TNode<IntPtrT> phi_bb347_31;
  TNode<IntPtrT> phi_bb347_34;
  TNode<BoolT> phi_bb347_35;
  TNode<BoolT> phi_bb347_36;
  TNode<Object> phi_bb347_37;
  TNode<Int32T> phi_bb347_38;
  TNode<Int32T> phi_bb347_40;
  TNode<Object> phi_bb347_45;
  TNode<IntPtrT> phi_bb347_46;
  TNode<IntPtrT> tmp730;
      TNode<Object> tmp732;
  TNode<Int32T> tmp733;
      TNode<Object> tmp735;
  TNode<Int32T> tmp736;
      TNode<Object> tmp738;
  if (block347.is_used()) {
    ca_.Bind(&block347, &phi_bb347_17, &phi_bb347_18, &phi_bb347_19, &phi_bb347_28, &phi_bb347_29, &phi_bb347_30, &phi_bb347_31, &phi_bb347_34, &phi_bb347_35, &phi_bb347_36, &phi_bb347_37, &phi_bb347_38, &phi_bb347_40, &phi_bb347_45, &phi_bb347_46);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 765);
    compiler::CodeAssemblerExceptionHandlerLabel catch731__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch731__label);
    tmp730 = CodeStubAssembler(state_).BitcastTaggedToWord(TNode<Object>{tmp704});
    }
    if (catch731__label.is_used()) {
      compiler::CodeAssemblerLabel catch731_skip(&ca_);
      ca_.Goto(&catch731_skip);
      ca_.Bind(&catch731__label, &tmp732);
      ca_.Goto(&block356, phi_bb347_17, phi_bb347_18, phi_bb347_19, phi_bb347_28, phi_bb347_29, phi_bb347_30, phi_bb347_31, phi_bb347_34, phi_bb347_35, phi_bb347_36, phi_bb347_37, phi_bb347_38, phi_bb347_40, phi_bb347_45, phi_bb347_46, phi_bb347_45, phi_bb347_46);
      ca_.Bind(&catch731_skip);
    }
    CodeStubAssembler(state_).StoreReference<IntPtrT>(CodeStubAssembler::Reference{phi_bb347_45, phi_bb347_46}, tmp730);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 766);
    compiler::CodeAssemblerExceptionHandlerLabel catch734__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch734__label);
    tmp733 = FromConstexpr_int32_constexpr_int31_0(state_, 1);
    }
    if (catch734__label.is_used()) {
      compiler::CodeAssemblerLabel catch734_skip(&ca_);
      ca_.Goto(&catch734_skip);
      ca_.Bind(&catch734__label, &tmp735);
      ca_.Goto(&block357, phi_bb347_17, phi_bb347_18, phi_bb347_19, phi_bb347_28, phi_bb347_29, phi_bb347_30, phi_bb347_31, phi_bb347_34, phi_bb347_35, phi_bb347_36, phi_bb347_37, phi_bb347_38, phi_bb347_40, phi_bb347_45, phi_bb347_46, phi_bb347_40, phi_bb347_40);
      ca_.Bind(&catch734_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch737__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch737__label);
    tmp736 = CodeStubAssembler(state_).Int32Add(TNode<Int32T>{phi_bb347_40}, TNode<Int32T>{tmp733});
    }
    if (catch737__label.is_used()) {
      compiler::CodeAssemblerLabel catch737_skip(&ca_);
      ca_.Goto(&catch737_skip);
      ca_.Bind(&catch737__label, &tmp738);
      ca_.Goto(&block358, phi_bb347_17, phi_bb347_18, phi_bb347_19, phi_bb347_28, phi_bb347_29, phi_bb347_30, phi_bb347_31, phi_bb347_34, phi_bb347_35, phi_bb347_36, phi_bb347_37, phi_bb347_38, phi_bb347_40, phi_bb347_45, phi_bb347_46, phi_bb347_40);
      ca_.Bind(&catch737_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 754);
    ca_.Goto(&block314, phi_bb347_17, phi_bb347_18, phi_bb347_19, tmp706, phi_bb347_28, phi_bb347_29, phi_bb347_30, phi_bb347_31, phi_bb347_34, phi_bb347_35, phi_bb347_36, phi_bb347_37, phi_bb347_38, tmp736, tmp686);
  }

  TNode<Object> phi_bb356_17;
  TNode<IntPtrT> phi_bb356_18;
  TNode<BoolT> phi_bb356_19;
  TNode<IntPtrT> phi_bb356_28;
  TNode<IntPtrT> phi_bb356_29;
  TNode<IntPtrT> phi_bb356_30;
  TNode<IntPtrT> phi_bb356_31;
  TNode<IntPtrT> phi_bb356_34;
  TNode<BoolT> phi_bb356_35;
  TNode<BoolT> phi_bb356_36;
  TNode<Object> phi_bb356_37;
  TNode<Int32T> phi_bb356_38;
  TNode<Int32T> phi_bb356_40;
  TNode<Object> phi_bb356_45;
  TNode<IntPtrT> phi_bb356_46;
  TNode<Object> phi_bb356_47;
  TNode<IntPtrT> phi_bb356_48;
  TNode<HeapObject> tmp739;
  if (block356.is_used()) {
    ca_.Bind(&block356, &phi_bb356_17, &phi_bb356_18, &phi_bb356_19, &phi_bb356_28, &phi_bb356_29, &phi_bb356_30, &phi_bb356_31, &phi_bb356_34, &phi_bb356_35, &phi_bb356_36, &phi_bb356_37, &phi_bb356_38, &phi_bb356_40, &phi_bb356_45, &phi_bb356_46, &phi_bb356_47, &phi_bb356_48);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 765);
    tmp739 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb356_17, phi_bb356_18, phi_bb356_19, tmp732, tmp739);
  }

  TNode<Object> phi_bb357_17;
  TNode<IntPtrT> phi_bb357_18;
  TNode<BoolT> phi_bb357_19;
  TNode<IntPtrT> phi_bb357_28;
  TNode<IntPtrT> phi_bb357_29;
  TNode<IntPtrT> phi_bb357_30;
  TNode<IntPtrT> phi_bb357_31;
  TNode<IntPtrT> phi_bb357_34;
  TNode<BoolT> phi_bb357_35;
  TNode<BoolT> phi_bb357_36;
  TNode<Object> phi_bb357_37;
  TNode<Int32T> phi_bb357_38;
  TNode<Int32T> phi_bb357_40;
  TNode<Object> phi_bb357_45;
  TNode<IntPtrT> phi_bb357_46;
  TNode<Int32T> phi_bb357_47;
  TNode<Int32T> phi_bb357_48;
  TNode<HeapObject> tmp740;
  if (block357.is_used()) {
    ca_.Bind(&block357, &phi_bb357_17, &phi_bb357_18, &phi_bb357_19, &phi_bb357_28, &phi_bb357_29, &phi_bb357_30, &phi_bb357_31, &phi_bb357_34, &phi_bb357_35, &phi_bb357_36, &phi_bb357_37, &phi_bb357_38, &phi_bb357_40, &phi_bb357_45, &phi_bb357_46, &phi_bb357_47, &phi_bb357_48);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 766);
    tmp740 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb357_17, phi_bb357_18, phi_bb357_19, tmp735, tmp740);
  }

  TNode<Object> phi_bb358_17;
  TNode<IntPtrT> phi_bb358_18;
  TNode<BoolT> phi_bb358_19;
  TNode<IntPtrT> phi_bb358_28;
  TNode<IntPtrT> phi_bb358_29;
  TNode<IntPtrT> phi_bb358_30;
  TNode<IntPtrT> phi_bb358_31;
  TNode<IntPtrT> phi_bb358_34;
  TNode<BoolT> phi_bb358_35;
  TNode<BoolT> phi_bb358_36;
  TNode<Object> phi_bb358_37;
  TNode<Int32T> phi_bb358_38;
  TNode<Int32T> phi_bb358_40;
  TNode<Object> phi_bb358_45;
  TNode<IntPtrT> phi_bb358_46;
  TNode<Int32T> phi_bb358_47;
  TNode<HeapObject> tmp741;
  if (block358.is_used()) {
    ca_.Bind(&block358, &phi_bb358_17, &phi_bb358_18, &phi_bb358_19, &phi_bb358_28, &phi_bb358_29, &phi_bb358_30, &phi_bb358_31, &phi_bb358_34, &phi_bb358_35, &phi_bb358_36, &phi_bb358_37, &phi_bb358_38, &phi_bb358_40, &phi_bb358_45, &phi_bb358_46, &phi_bb358_47);
    tmp741 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb358_17, phi_bb358_18, phi_bb358_19, tmp738, tmp741);
  }

  TNode<Object> phi_bb313_17;
  TNode<IntPtrT> phi_bb313_18;
  TNode<BoolT> phi_bb313_19;
  TNode<IntPtrT> phi_bb313_27;
  TNode<IntPtrT> phi_bb313_28;
  TNode<IntPtrT> phi_bb313_29;
  TNode<IntPtrT> phi_bb313_30;
  TNode<IntPtrT> phi_bb313_31;
  TNode<IntPtrT> phi_bb313_34;
  TNode<BoolT> phi_bb313_35;
  TNode<BoolT> phi_bb313_36;
  TNode<Object> phi_bb313_37;
  TNode<Int32T> phi_bb313_38;
  TNode<Int32T> phi_bb313_40;
  TNode<Int32T> phi_bb313_41;
  if (block313.is_used()) {
    ca_.Bind(&block313, &phi_bb313_17, &phi_bb313_18, &phi_bb313_19, &phi_bb313_27, &phi_bb313_28, &phi_bb313_29, &phi_bb313_30, &phi_bb313_31, &phi_bb313_34, &phi_bb313_35, &phi_bb313_36, &phi_bb313_37, &phi_bb313_38, &phi_bb313_40, &phi_bb313_41);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 744);
    ca_.Goto(&block306, phi_bb313_17, phi_bb313_18, phi_bb313_19, phi_bb313_27, phi_bb313_28, phi_bb313_29, phi_bb313_30, phi_bb313_31, phi_bb313_34, phi_bb313_35, phi_bb313_36, phi_bb313_37, phi_bb313_38);
  }

  TNode<Object> phi_bb306_17;
  TNode<IntPtrT> phi_bb306_18;
  TNode<BoolT> phi_bb306_19;
  TNode<IntPtrT> phi_bb306_27;
  TNode<IntPtrT> phi_bb306_28;
  TNode<IntPtrT> phi_bb306_29;
  TNode<IntPtrT> phi_bb306_30;
  TNode<IntPtrT> phi_bb306_31;
  TNode<IntPtrT> phi_bb306_34;
  TNode<BoolT> phi_bb306_35;
  TNode<BoolT> phi_bb306_36;
  TNode<Object> phi_bb306_37;
  TNode<Int32T> phi_bb306_38;
  TNode<RawPtrT> tmp742;
  TNode<RawPtrT> tmp743;
  if (block306.is_used()) {
    ca_.Bind(&block306, &phi_bb306_17, &phi_bb306_18, &phi_bb306_19, &phi_bb306_27, &phi_bb306_28, &phi_bb306_29, &phi_bb306_30, &phi_bb306_31, &phi_bb306_34, &phi_bb306_35, &phi_bb306_36, &phi_bb306_37, &phi_bb306_38);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 65);
    tmp742 = CodeStubAssembler(state_).GCUnsafeReferenceToRawPtr(TNode<Object>{phi_bb133_23}, TNode<IntPtrT>{phi_bb133_24});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 64);
    tmp743 = (TNode<RawPtrT>{tmp742});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 359);
    ca_.Branch(phi_bb306_35, &block361, std::vector<compiler::Node*>{phi_bb306_17, phi_bb306_18, phi_bb306_19, phi_bb306_27, phi_bb306_28, phi_bb306_29, phi_bb306_30, phi_bb306_31, phi_bb306_34, phi_bb306_35, phi_bb306_36, phi_bb306_37, phi_bb306_38, phi_bb306_31}, &block362, std::vector<compiler::Node*>{phi_bb306_17, phi_bb306_18, phi_bb306_19, phi_bb306_27, phi_bb306_28, phi_bb306_29, phi_bb306_30, phi_bb306_31, phi_bb306_34, phi_bb306_35, phi_bb306_36, phi_bb306_37, phi_bb306_38, phi_bb306_31});
  }

  TNode<Object> phi_bb361_17;
  TNode<IntPtrT> phi_bb361_18;
  TNode<BoolT> phi_bb361_19;
  TNode<IntPtrT> phi_bb361_27;
  TNode<IntPtrT> phi_bb361_28;
  TNode<IntPtrT> phi_bb361_29;
  TNode<IntPtrT> phi_bb361_30;
  TNode<IntPtrT> phi_bb361_31;
  TNode<IntPtrT> phi_bb361_34;
  TNode<BoolT> phi_bb361_35;
  TNode<BoolT> phi_bb361_36;
  TNode<Object> phi_bb361_37;
  TNode<Int32T> phi_bb361_38;
  TNode<IntPtrT> phi_bb361_41;
  TNode<IntPtrT> tmp744;
  TNode<IntPtrT> tmp745;
  if (block361.is_used()) {
    ca_.Bind(&block361, &phi_bb361_17, &phi_bb361_18, &phi_bb361_19, &phi_bb361_27, &phi_bb361_28, &phi_bb361_29, &phi_bb361_30, &phi_bb361_31, &phi_bb361_34, &phi_bb361_35, &phi_bb361_36, &phi_bb361_37, &phi_bb361_38, &phi_bb361_41);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 360);
    tmp744 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp745 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb361_41}, TNode<IntPtrT>{tmp744});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 359);
    ca_.Goto(&block362, phi_bb361_17, phi_bb361_18, phi_bb361_19, phi_bb361_27, phi_bb361_28, phi_bb361_29, phi_bb361_30, phi_bb361_31, phi_bb361_34, phi_bb361_35, phi_bb361_36, phi_bb361_37, phi_bb361_38, tmp745);
  }

  TNode<Object> phi_bb362_17;
  TNode<IntPtrT> phi_bb362_18;
  TNode<BoolT> phi_bb362_19;
  TNode<IntPtrT> phi_bb362_27;
  TNode<IntPtrT> phi_bb362_28;
  TNode<IntPtrT> phi_bb362_29;
  TNode<IntPtrT> phi_bb362_30;
  TNode<IntPtrT> phi_bb362_31;
  TNode<IntPtrT> phi_bb362_34;
  TNode<BoolT> phi_bb362_35;
  TNode<BoolT> phi_bb362_36;
  TNode<Object> phi_bb362_37;
  TNode<Int32T> phi_bb362_38;
  TNode<IntPtrT> phi_bb362_41;
  TNode<RawPtrT> tmp746;
  TNode<IntPtrT> tmp747;
  TNode<BoolT> tmp748;
  if (block362.is_used()) {
    ca_.Bind(&block362, &phi_bb362_17, &phi_bb362_18, &phi_bb362_19, &phi_bb362_27, &phi_bb362_28, &phi_bb362_29, &phi_bb362_30, &phi_bb362_31, &phi_bb362_34, &phi_bb362_35, &phi_bb362_36, &phi_bb362_37, &phi_bb362_38, &phi_bb362_41);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 362);
    tmp746 = CodeStubAssembler(state_).GCUnsafeReferenceToRawPtr(TNode<Object>{tmp198}, TNode<IntPtrT>{phi_bb362_41});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 380);
    tmp747 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp748 = CodeStubAssembler(state_).WordEqual(TNode<IntPtrT>{tmp205}, TNode<IntPtrT>{tmp747});
    ca_.Branch(tmp748, &block366, std::vector<compiler::Node*>{phi_bb362_17, phi_bb362_18, phi_bb362_19, phi_bb362_27, phi_bb362_28, phi_bb362_29, phi_bb362_30, phi_bb362_31, phi_bb362_34, phi_bb362_35, phi_bb362_36, phi_bb362_37, phi_bb362_38}, &block367, std::vector<compiler::Node*>{phi_bb362_17, phi_bb362_18, phi_bb362_19, phi_bb362_27, phi_bb362_28, phi_bb362_29, phi_bb362_30, phi_bb362_31, phi_bb362_34, phi_bb362_35, phi_bb362_36, phi_bb362_37, phi_bb362_38});
  }

  TNode<Object> phi_bb366_17;
  TNode<IntPtrT> phi_bb366_18;
  TNode<BoolT> phi_bb366_19;
  TNode<IntPtrT> phi_bb366_27;
  TNode<IntPtrT> phi_bb366_28;
  TNode<IntPtrT> phi_bb366_29;
  TNode<IntPtrT> phi_bb366_30;
  TNode<IntPtrT> phi_bb366_31;
  TNode<IntPtrT> phi_bb366_34;
  TNode<BoolT> phi_bb366_35;
  TNode<BoolT> phi_bb366_36;
  TNode<Object> phi_bb366_37;
  TNode<Int32T> phi_bb366_38;
  TNode<BoolT> tmp749;
  if (block366.is_used()) {
    ca_.Bind(&block366, &phi_bb366_17, &phi_bb366_18, &phi_bb366_19, &phi_bb366_27, &phi_bb366_28, &phi_bb366_29, &phi_bb366_30, &phi_bb366_31, &phi_bb366_34, &phi_bb366_35, &phi_bb366_36, &phi_bb366_37, &phi_bb366_38);
    tmp749 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block368, phi_bb366_17, phi_bb366_18, phi_bb366_19, phi_bb366_27, phi_bb366_28, phi_bb366_29, phi_bb366_30, phi_bb366_31, phi_bb366_34, phi_bb366_35, phi_bb366_36, phi_bb366_37, phi_bb366_38, tmp749);
  }

  TNode<Object> phi_bb367_17;
  TNode<IntPtrT> phi_bb367_18;
  TNode<BoolT> phi_bb367_19;
  TNode<IntPtrT> phi_bb367_27;
  TNode<IntPtrT> phi_bb367_28;
  TNode<IntPtrT> phi_bb367_29;
  TNode<IntPtrT> phi_bb367_30;
  TNode<IntPtrT> phi_bb367_31;
  TNode<IntPtrT> phi_bb367_34;
  TNode<BoolT> phi_bb367_35;
  TNode<BoolT> phi_bb367_36;
  TNode<Object> phi_bb367_37;
  TNode<Int32T> phi_bb367_38;
  TNode<BoolT> tmp750;
  if (block367.is_used()) {
    ca_.Bind(&block367, &phi_bb367_17, &phi_bb367_18, &phi_bb367_19, &phi_bb367_27, &phi_bb367_28, &phi_bb367_29, &phi_bb367_30, &phi_bb367_31, &phi_bb367_34, &phi_bb367_35, &phi_bb367_36, &phi_bb367_37, &phi_bb367_38);
    tmp750 = CodeStubAssembler(state_).IntPtrLessThanOrEqual(TNode<IntPtrT>{phi_bb367_31}, TNode<IntPtrT>{tmp205});
    ca_.Goto(&block368, phi_bb367_17, phi_bb367_18, phi_bb367_19, phi_bb367_27, phi_bb367_28, phi_bb367_29, phi_bb367_30, phi_bb367_31, phi_bb367_34, phi_bb367_35, phi_bb367_36, phi_bb367_37, phi_bb367_38, tmp750);
  }

  TNode<Object> phi_bb368_17;
  TNode<IntPtrT> phi_bb368_18;
  TNode<BoolT> phi_bb368_19;
  TNode<IntPtrT> phi_bb368_27;
  TNode<IntPtrT> phi_bb368_28;
  TNode<IntPtrT> phi_bb368_29;
  TNode<IntPtrT> phi_bb368_30;
  TNode<IntPtrT> phi_bb368_31;
  TNode<IntPtrT> phi_bb368_34;
  TNode<BoolT> phi_bb368_35;
  TNode<BoolT> phi_bb368_36;
  TNode<Object> phi_bb368_37;
  TNode<Int32T> phi_bb368_38;
  TNode<BoolT> phi_bb368_43;
  if (block368.is_used()) {
    ca_.Bind(&block368, &phi_bb368_17, &phi_bb368_18, &phi_bb368_19, &phi_bb368_27, &phi_bb368_28, &phi_bb368_29, &phi_bb368_30, &phi_bb368_31, &phi_bb368_34, &phi_bb368_35, &phi_bb368_36, &phi_bb368_37, &phi_bb368_38, &phi_bb368_43);
    ca_.Branch(phi_bb368_43, &block364, std::vector<compiler::Node*>{phi_bb368_17, phi_bb368_18, phi_bb368_19, phi_bb368_27, phi_bb368_28, phi_bb368_29, phi_bb368_30, phi_bb368_31, phi_bb368_34, phi_bb368_35, phi_bb368_36, phi_bb368_37, phi_bb368_38}, &block365, std::vector<compiler::Node*>{phi_bb368_17, phi_bb368_18, phi_bb368_19, phi_bb368_27, phi_bb368_28, phi_bb368_29, phi_bb368_30, phi_bb368_31, phi_bb368_34, phi_bb368_35, phi_bb368_36, phi_bb368_37, phi_bb368_38});
  }

  TNode<Object> phi_bb365_17;
  TNode<IntPtrT> phi_bb365_18;
  TNode<BoolT> phi_bb365_19;
  TNode<IntPtrT> phi_bb365_27;
  TNode<IntPtrT> phi_bb365_28;
  TNode<IntPtrT> phi_bb365_29;
  TNode<IntPtrT> phi_bb365_30;
  TNode<IntPtrT> phi_bb365_31;
  TNode<IntPtrT> phi_bb365_34;
  TNode<BoolT> phi_bb365_35;
  TNode<BoolT> phi_bb365_36;
  TNode<Object> phi_bb365_37;
  TNode<Int32T> phi_bb365_38;
  if (block365.is_used()) {
    ca_.Bind(&block365, &phi_bb365_17, &phi_bb365_18, &phi_bb365_19, &phi_bb365_27, &phi_bb365_28, &phi_bb365_29, &phi_bb365_30, &phi_bb365_31, &phi_bb365_34, &phi_bb365_35, &phi_bb365_36, &phi_bb365_37, &phi_bb365_38);
    {
      auto pos_stack = ca_.GetMacroSourcePositionStack();
      pos_stack.push_back({"src/builtins/js-to-wasm.tq", 380});
      CodeStubAssembler(state_).FailAssert("Torque assert 'this.paramBufferEnd == 0 || this.nextStack <= this.paramBufferEnd' failed", pos_stack);
    }
  }

  TNode<Object> phi_bb364_17;
  TNode<IntPtrT> phi_bb364_18;
  TNode<BoolT> phi_bb364_19;
  TNode<IntPtrT> phi_bb364_27;
  TNode<IntPtrT> phi_bb364_28;
  TNode<IntPtrT> phi_bb364_29;
  TNode<IntPtrT> phi_bb364_30;
  TNode<IntPtrT> phi_bb364_31;
  TNode<IntPtrT> phi_bb364_34;
  TNode<BoolT> phi_bb364_35;
  TNode<BoolT> phi_bb364_36;
  TNode<Object> phi_bb364_37;
  TNode<Int32T> phi_bb364_38;
  TNode<WasmInternalFunction> tmp751;
      TNode<Object> tmp753;
  TNode<IntPtrT> tmp754;
      TNode<Object> tmp756;
  TNode<Uint32T> tmp757;
  TNode<RawPtrT> tmp758;
      TNode<Object> tmp760;
  TNode<RawPtrT> tmp761;
  TNode<IntPtrT> tmp762;
      TNode<Object> tmp764;
  TNode<Object> tmp765;
  TNode<IntPtrT> tmp766;
      TNode<Object> tmp768;
  TNode<IntPtrT> tmp769;
      TNode<Object> tmp771;
  TNode<Object> tmp772;
  TNode<IntPtrT> tmp773;
      TNode<Object> tmp775;
  TNode<IntPtrT> tmp776;
      TNode<Object> tmp778;
  TNode<Object> tmp779;
  TNode<IntPtrT> tmp780;
      TNode<Object> tmp782;
  TNode<IntPtrT> tmp783;
      TNode<Object> tmp785;
  TNode<Object> tmp786;
  TNode<IntPtrT> tmp787;
      TNode<Object> tmp789;
  TNode<IntPtrT> tmp790;
      TNode<Object> tmp792;
  TNode<Object> tmp793;
  TNode<IntPtrT> tmp794;
      TNode<Object> tmp796;
  TNode<IntPtrT> tmp797;
      TNode<Object> tmp799;
  TNode<Object> tmp800;
  TNode<IntPtrT> tmp801;
      TNode<Object> tmp803;
  TNode<IntPtrT> tmp804;
      TNode<Object> tmp806;
  TNode<Object> tmp807;
  TNode<IntPtrT> tmp808;
      TNode<Object> tmp810;
  if (block364.is_used()) {
    ca_.Bind(&block364, &phi_bb364_17, &phi_bb364_18, &phi_bb364_19, &phi_bb364_27, &phi_bb364_28, &phi_bb364_29, &phi_bb364_30, &phi_bb364_31, &phi_bb364_34, &phi_bb364_35, &phi_bb364_36, &phi_bb364_37, &phi_bb364_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 773);
    compiler::CodeAssemblerExceptionHandlerLabel catch752__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch752__label);
    tmp751 = CodeStubAssembler(state_).LoadWasmInternalFunctionFromFunctionData(TNode<WasmFunctionData>{tmp2});
    }
    if (catch752__label.is_used()) {
      compiler::CodeAssemblerLabel catch752_skip(&ca_);
      ca_.Goto(&catch752_skip);
      ca_.Bind(&catch752__label, &tmp753);
      ca_.Goto(&block369, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch752_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch755__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch755__label);
    tmp754 = FromConstexpr_intptr_constexpr_int31_0(state_, 20);
    }
    if (catch755__label.is_used()) {
      compiler::CodeAssemblerLabel catch755_skip(&ca_);
      ca_.Goto(&catch755_skip);
      ca_.Bind(&catch755__label, &tmp756);
      ca_.Goto(&block370, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch755_skip);
    }
    tmp757 = CodeStubAssembler(state_).LoadReference<Uint32T>(CodeStubAssembler::Reference{tmp751, tmp754});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 781);
    compiler::CodeAssemblerExceptionHandlerLabel catch759__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch759__label);
    tmp758 = CodeStubAssembler(state_).StackSlotPtr(JSToWasmWrapperFrameConstants::kWrapperBufferSize, (SizeOf_intptr_0(state_)));
    }
    if (catch759__label.is_used()) {
      compiler::CodeAssemblerLabel catch759_skip(&ca_);
      ca_.Goto(&catch759_skip);
      ca_.Bind(&catch759__label, &tmp760);
      ca_.Goto(&block371, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch759_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 780);
    tmp761 = (TNode<RawPtrT>{tmp758});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 783);
    compiler::CodeAssemblerExceptionHandlerLabel catch763__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch763__label);
    tmp762 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferReturnCount);
    }
    if (catch763__label.is_used()) {
      compiler::CodeAssemblerLabel catch763_skip(&ca_);
      ca_.Goto(&catch763_skip);
      ca_.Bind(&catch763__label, &tmp764);
      ca_.Goto(&block372, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch763_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch767__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch767__label);
    std::tie(tmp765, tmp766) = GetRefAt_int32_RawPtr_intptr_0(state_, TNode<RawPtrT>{tmp761}, TNode<IntPtrT>{tmp762}).Flatten();
    }
    if (catch767__label.is_used()) {
      compiler::CodeAssemblerLabel catch767_skip(&ca_);
      ca_.Goto(&catch767_skip);
      ca_.Bind(&catch767__label, &tmp768);
      ca_.Goto(&block373, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch767_skip);
    }
    CodeStubAssembler(state_).StoreReference<Int32T>(CodeStubAssembler::Reference{tmp765, tmp766}, tmp31);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 784);
    compiler::CodeAssemblerExceptionHandlerLabel catch770__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch770__label);
    tmp769 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferRefReturnCount);
    }
    if (catch770__label.is_used()) {
      compiler::CodeAssemblerLabel catch770_skip(&ca_);
      ca_.Goto(&catch770_skip);
      ca_.Bind(&catch770__label, &tmp771);
      ca_.Goto(&block374, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch770_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch774__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch774__label);
    std::tie(tmp772, tmp773) = GetRefAt_bool_RawPtr_intptr_0(state_, TNode<RawPtrT>{tmp761}, TNode<IntPtrT>{tmp769}).Flatten();
    }
    if (catch774__label.is_used()) {
      compiler::CodeAssemblerLabel catch774_skip(&ca_);
      ca_.Goto(&catch774_skip);
      ca_.Bind(&catch774__label, &tmp775);
      ca_.Goto(&block375, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch774_skip);
    }
    CodeStubAssembler(state_).StoreReference<BoolT>(CodeStubAssembler::Reference{tmp772, tmp773}, phi_bb364_19);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 786);
    compiler::CodeAssemblerExceptionHandlerLabel catch777__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch777__label);
    tmp776 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferSigRepresentationArray);
    }
    if (catch777__label.is_used()) {
      compiler::CodeAssemblerLabel catch777_skip(&ca_);
      ca_.Goto(&catch777_skip);
      ca_.Bind(&catch777__label, &tmp778);
      ca_.Goto(&block376, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch777_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch781__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch781__label);
    std::tie(tmp779, tmp780) = GetRefAt_RawPtr_RawPtr_intptr_0(state_, TNode<RawPtrT>{tmp761}, TNode<IntPtrT>{tmp776}).Flatten();
    }
    if (catch781__label.is_used()) {
      compiler::CodeAssemblerLabel catch781_skip(&ca_);
      ca_.Goto(&catch781_skip);
      ca_.Bind(&catch781__label, &tmp782);
      ca_.Goto(&block377, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch781_skip);
    }
    CodeStubAssembler(state_).StoreReference<RawPtrT>(CodeStubAssembler::Reference{tmp779, tmp780}, tmp35);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 788);
    compiler::CodeAssemblerExceptionHandlerLabel catch784__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch784__label);
    tmp783 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferStackReturnBufferSize);
    }
    if (catch784__label.is_used()) {
      compiler::CodeAssemblerLabel catch784_skip(&ca_);
      ca_.Goto(&catch784_skip);
      ca_.Bind(&catch784__label, &tmp785);
      ca_.Goto(&block378, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch784_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch788__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch788__label);
    std::tie(tmp786, tmp787) = GetRefAt_intptr_RawPtr_intptr_0(state_, TNode<RawPtrT>{tmp761}, TNode<IntPtrT>{tmp783}).Flatten();
    }
    if (catch788__label.is_used()) {
      compiler::CodeAssemblerLabel catch788_skip(&ca_);
      ca_.Goto(&catch788_skip);
      ca_.Bind(&catch788__label, &tmp789);
      ca_.Goto(&block379, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch788_skip);
    }
    CodeStubAssembler(state_).StoreReference<IntPtrT>(CodeStubAssembler::Reference{tmp786, tmp787}, phi_bb364_18);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 790);
    compiler::CodeAssemblerExceptionHandlerLabel catch791__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch791__label);
    tmp790 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferCallTarget);
    }
    if (catch791__label.is_used()) {
      compiler::CodeAssemblerLabel catch791_skip(&ca_);
      ca_.Goto(&catch791_skip);
      ca_.Bind(&catch791__label, &tmp792);
      ca_.Goto(&block380, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch791_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch795__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch795__label);
    std::tie(tmp793, tmp794) = GetRefAt_WasmCodePointer_RawPtr_intptr_0(state_, TNode<RawPtrT>{tmp761}, TNode<IntPtrT>{tmp790}).Flatten();
    }
    if (catch795__label.is_used()) {
      compiler::CodeAssemblerLabel catch795_skip(&ca_);
      ca_.Goto(&catch795_skip);
      ca_.Bind(&catch795__label, &tmp796);
      ca_.Goto(&block381, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch795_skip);
    }
    CodeStubAssembler(state_).StoreReference<Uint32T>(CodeStubAssembler::Reference{tmp793, tmp794}, tmp757);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 792);
    compiler::CodeAssemblerExceptionHandlerLabel catch798__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch798__label);
    tmp797 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferParamStart);
    }
    if (catch798__label.is_used()) {
      compiler::CodeAssemblerLabel catch798_skip(&ca_);
      ca_.Goto(&catch798_skip);
      ca_.Bind(&catch798__label, &tmp799);
      ca_.Goto(&block382, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch798_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch802__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch802__label);
    std::tie(tmp800, tmp801) = GetRefAt_RawPtr_intptr_RawPtr_intptr_0(state_, TNode<RawPtrT>{tmp761}, TNode<IntPtrT>{tmp797}).Flatten();
    }
    if (catch802__label.is_used()) {
      compiler::CodeAssemblerLabel catch802_skip(&ca_);
      ca_.Goto(&catch802_skip);
      ca_.Bind(&catch802__label, &tmp803);
      ca_.Goto(&block383, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch802_skip);
    }
    CodeStubAssembler(state_).StoreReference<RawPtrT>(CodeStubAssembler::Reference{tmp800, tmp801}, tmp743);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 794);
    compiler::CodeAssemblerExceptionHandlerLabel catch805__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch805__label);
    tmp804 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferParamEnd);
    }
    if (catch805__label.is_used()) {
      compiler::CodeAssemblerLabel catch805_skip(&ca_);
      ca_.Goto(&catch805_skip);
      ca_.Bind(&catch805__label, &tmp806);
      ca_.Goto(&block384, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch805_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch809__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch809__label);
    std::tie(tmp807, tmp808) = GetRefAt_RawPtr_RawPtr_intptr_0(state_, TNode<RawPtrT>{tmp761}, TNode<IntPtrT>{tmp804}).Flatten();
    }
    if (catch809__label.is_used()) {
      compiler::CodeAssemblerLabel catch809_skip(&ca_);
      ca_.Goto(&catch809_skip);
      ca_.Bind(&catch809__label, &tmp810);
      ca_.Goto(&block385, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
      ca_.Bind(&catch809_skip);
    }
    CodeStubAssembler(state_).StoreReference<RawPtrT>(CodeStubAssembler::Reference{tmp807, tmp808}, tmp746);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 801);
    if (((CodeStubAssembler(state_).ConstexprInt32Equal(p_promise, wasm::Promise::kPromise)))) {
      ca_.Goto(&block386, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
    } else {
      ca_.Goto(&block387, phi_bb364_17, phi_bb364_18, phi_bb364_19, phi_bb364_27, phi_bb364_28, phi_bb364_29, phi_bb364_30, phi_bb364_31, phi_bb364_34, phi_bb364_35, phi_bb364_36, phi_bb364_37, phi_bb364_38);
    }
  }

  TNode<Object> phi_bb369_17;
  TNode<IntPtrT> phi_bb369_18;
  TNode<BoolT> phi_bb369_19;
  TNode<IntPtrT> phi_bb369_27;
  TNode<IntPtrT> phi_bb369_28;
  TNode<IntPtrT> phi_bb369_29;
  TNode<IntPtrT> phi_bb369_30;
  TNode<IntPtrT> phi_bb369_31;
  TNode<IntPtrT> phi_bb369_34;
  TNode<BoolT> phi_bb369_35;
  TNode<BoolT> phi_bb369_36;
  TNode<Object> phi_bb369_37;
  TNode<Int32T> phi_bb369_38;
  TNode<HeapObject> tmp811;
  if (block369.is_used()) {
    ca_.Bind(&block369, &phi_bb369_17, &phi_bb369_18, &phi_bb369_19, &phi_bb369_27, &phi_bb369_28, &phi_bb369_29, &phi_bb369_30, &phi_bb369_31, &phi_bb369_34, &phi_bb369_35, &phi_bb369_36, &phi_bb369_37, &phi_bb369_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 773);
    tmp811 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb369_17, phi_bb369_18, phi_bb369_19, tmp753, tmp811);
  }

  TNode<Object> phi_bb370_17;
  TNode<IntPtrT> phi_bb370_18;
  TNode<BoolT> phi_bb370_19;
  TNode<IntPtrT> phi_bb370_27;
  TNode<IntPtrT> phi_bb370_28;
  TNode<IntPtrT> phi_bb370_29;
  TNode<IntPtrT> phi_bb370_30;
  TNode<IntPtrT> phi_bb370_31;
  TNode<IntPtrT> phi_bb370_34;
  TNode<BoolT> phi_bb370_35;
  TNode<BoolT> phi_bb370_36;
  TNode<Object> phi_bb370_37;
  TNode<Int32T> phi_bb370_38;
  TNode<HeapObject> tmp812;
  if (block370.is_used()) {
    ca_.Bind(&block370, &phi_bb370_17, &phi_bb370_18, &phi_bb370_19, &phi_bb370_27, &phi_bb370_28, &phi_bb370_29, &phi_bb370_30, &phi_bb370_31, &phi_bb370_34, &phi_bb370_35, &phi_bb370_36, &phi_bb370_37, &phi_bb370_38);
    tmp812 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb370_17, phi_bb370_18, phi_bb370_19, tmp756, tmp812);
  }

  TNode<Object> phi_bb371_17;
  TNode<IntPtrT> phi_bb371_18;
  TNode<BoolT> phi_bb371_19;
  TNode<IntPtrT> phi_bb371_27;
  TNode<IntPtrT> phi_bb371_28;
  TNode<IntPtrT> phi_bb371_29;
  TNode<IntPtrT> phi_bb371_30;
  TNode<IntPtrT> phi_bb371_31;
  TNode<IntPtrT> phi_bb371_34;
  TNode<BoolT> phi_bb371_35;
  TNode<BoolT> phi_bb371_36;
  TNode<Object> phi_bb371_37;
  TNode<Int32T> phi_bb371_38;
  TNode<HeapObject> tmp813;
  if (block371.is_used()) {
    ca_.Bind(&block371, &phi_bb371_17, &phi_bb371_18, &phi_bb371_19, &phi_bb371_27, &phi_bb371_28, &phi_bb371_29, &phi_bb371_30, &phi_bb371_31, &phi_bb371_34, &phi_bb371_35, &phi_bb371_36, &phi_bb371_37, &phi_bb371_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 781);
    tmp813 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb371_17, phi_bb371_18, phi_bb371_19, tmp760, tmp813);
  }

  TNode<Object> phi_bb372_17;
  TNode<IntPtrT> phi_bb372_18;
  TNode<BoolT> phi_bb372_19;
  TNode<IntPtrT> phi_bb372_27;
  TNode<IntPtrT> phi_bb372_28;
  TNode<IntPtrT> phi_bb372_29;
  TNode<IntPtrT> phi_bb372_30;
  TNode<IntPtrT> phi_bb372_31;
  TNode<IntPtrT> phi_bb372_34;
  TNode<BoolT> phi_bb372_35;
  TNode<BoolT> phi_bb372_36;
  TNode<Object> phi_bb372_37;
  TNode<Int32T> phi_bb372_38;
  TNode<HeapObject> tmp814;
  if (block372.is_used()) {
    ca_.Bind(&block372, &phi_bb372_17, &phi_bb372_18, &phi_bb372_19, &phi_bb372_27, &phi_bb372_28, &phi_bb372_29, &phi_bb372_30, &phi_bb372_31, &phi_bb372_34, &phi_bb372_35, &phi_bb372_36, &phi_bb372_37, &phi_bb372_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 783);
    tmp814 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb372_17, phi_bb372_18, phi_bb372_19, tmp764, tmp814);
  }

  TNode<Object> phi_bb373_17;
  TNode<IntPtrT> phi_bb373_18;
  TNode<BoolT> phi_bb373_19;
  TNode<IntPtrT> phi_bb373_27;
  TNode<IntPtrT> phi_bb373_28;
  TNode<IntPtrT> phi_bb373_29;
  TNode<IntPtrT> phi_bb373_30;
  TNode<IntPtrT> phi_bb373_31;
  TNode<IntPtrT> phi_bb373_34;
  TNode<BoolT> phi_bb373_35;
  TNode<BoolT> phi_bb373_36;
  TNode<Object> phi_bb373_37;
  TNode<Int32T> phi_bb373_38;
  TNode<HeapObject> tmp815;
  if (block373.is_used()) {
    ca_.Bind(&block373, &phi_bb373_17, &phi_bb373_18, &phi_bb373_19, &phi_bb373_27, &phi_bb373_28, &phi_bb373_29, &phi_bb373_30, &phi_bb373_31, &phi_bb373_34, &phi_bb373_35, &phi_bb373_36, &phi_bb373_37, &phi_bb373_38);
    tmp815 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb373_17, phi_bb373_18, phi_bb373_19, tmp768, tmp815);
  }

  TNode<Object> phi_bb374_17;
  TNode<IntPtrT> phi_bb374_18;
  TNode<BoolT> phi_bb374_19;
  TNode<IntPtrT> phi_bb374_27;
  TNode<IntPtrT> phi_bb374_28;
  TNode<IntPtrT> phi_bb374_29;
  TNode<IntPtrT> phi_bb374_30;
  TNode<IntPtrT> phi_bb374_31;
  TNode<IntPtrT> phi_bb374_34;
  TNode<BoolT> phi_bb374_35;
  TNode<BoolT> phi_bb374_36;
  TNode<Object> phi_bb374_37;
  TNode<Int32T> phi_bb374_38;
  TNode<HeapObject> tmp816;
  if (block374.is_used()) {
    ca_.Bind(&block374, &phi_bb374_17, &phi_bb374_18, &phi_bb374_19, &phi_bb374_27, &phi_bb374_28, &phi_bb374_29, &phi_bb374_30, &phi_bb374_31, &phi_bb374_34, &phi_bb374_35, &phi_bb374_36, &phi_bb374_37, &phi_bb374_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 784);
    tmp816 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb374_17, phi_bb374_18, phi_bb374_19, tmp771, tmp816);
  }

  TNode<Object> phi_bb375_17;
  TNode<IntPtrT> phi_bb375_18;
  TNode<BoolT> phi_bb375_19;
  TNode<IntPtrT> phi_bb375_27;
  TNode<IntPtrT> phi_bb375_28;
  TNode<IntPtrT> phi_bb375_29;
  TNode<IntPtrT> phi_bb375_30;
  TNode<IntPtrT> phi_bb375_31;
  TNode<IntPtrT> phi_bb375_34;
  TNode<BoolT> phi_bb375_35;
  TNode<BoolT> phi_bb375_36;
  TNode<Object> phi_bb375_37;
  TNode<Int32T> phi_bb375_38;
  TNode<HeapObject> tmp817;
  if (block375.is_used()) {
    ca_.Bind(&block375, &phi_bb375_17, &phi_bb375_18, &phi_bb375_19, &phi_bb375_27, &phi_bb375_28, &phi_bb375_29, &phi_bb375_30, &phi_bb375_31, &phi_bb375_34, &phi_bb375_35, &phi_bb375_36, &phi_bb375_37, &phi_bb375_38);
    tmp817 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb375_17, phi_bb375_18, phi_bb375_19, tmp775, tmp817);
  }

  TNode<Object> phi_bb376_17;
  TNode<IntPtrT> phi_bb376_18;
  TNode<BoolT> phi_bb376_19;
  TNode<IntPtrT> phi_bb376_27;
  TNode<IntPtrT> phi_bb376_28;
  TNode<IntPtrT> phi_bb376_29;
  TNode<IntPtrT> phi_bb376_30;
  TNode<IntPtrT> phi_bb376_31;
  TNode<IntPtrT> phi_bb376_34;
  TNode<BoolT> phi_bb376_35;
  TNode<BoolT> phi_bb376_36;
  TNode<Object> phi_bb376_37;
  TNode<Int32T> phi_bb376_38;
  TNode<HeapObject> tmp818;
  if (block376.is_used()) {
    ca_.Bind(&block376, &phi_bb376_17, &phi_bb376_18, &phi_bb376_19, &phi_bb376_27, &phi_bb376_28, &phi_bb376_29, &phi_bb376_30, &phi_bb376_31, &phi_bb376_34, &phi_bb376_35, &phi_bb376_36, &phi_bb376_37, &phi_bb376_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 786);
    tmp818 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb376_17, phi_bb376_18, phi_bb376_19, tmp778, tmp818);
  }

  TNode<Object> phi_bb377_17;
  TNode<IntPtrT> phi_bb377_18;
  TNode<BoolT> phi_bb377_19;
  TNode<IntPtrT> phi_bb377_27;
  TNode<IntPtrT> phi_bb377_28;
  TNode<IntPtrT> phi_bb377_29;
  TNode<IntPtrT> phi_bb377_30;
  TNode<IntPtrT> phi_bb377_31;
  TNode<IntPtrT> phi_bb377_34;
  TNode<BoolT> phi_bb377_35;
  TNode<BoolT> phi_bb377_36;
  TNode<Object> phi_bb377_37;
  TNode<Int32T> phi_bb377_38;
  TNode<HeapObject> tmp819;
  if (block377.is_used()) {
    ca_.Bind(&block377, &phi_bb377_17, &phi_bb377_18, &phi_bb377_19, &phi_bb377_27, &phi_bb377_28, &phi_bb377_29, &phi_bb377_30, &phi_bb377_31, &phi_bb377_34, &phi_bb377_35, &phi_bb377_36, &phi_bb377_37, &phi_bb377_38);
    tmp819 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb377_17, phi_bb377_18, phi_bb377_19, tmp782, tmp819);
  }

  TNode<Object> phi_bb378_17;
  TNode<IntPtrT> phi_bb378_18;
  TNode<BoolT> phi_bb378_19;
  TNode<IntPtrT> phi_bb378_27;
  TNode<IntPtrT> phi_bb378_28;
  TNode<IntPtrT> phi_bb378_29;
  TNode<IntPtrT> phi_bb378_30;
  TNode<IntPtrT> phi_bb378_31;
  TNode<IntPtrT> phi_bb378_34;
  TNode<BoolT> phi_bb378_35;
  TNode<BoolT> phi_bb378_36;
  TNode<Object> phi_bb378_37;
  TNode<Int32T> phi_bb378_38;
  TNode<HeapObject> tmp820;
  if (block378.is_used()) {
    ca_.Bind(&block378, &phi_bb378_17, &phi_bb378_18, &phi_bb378_19, &phi_bb378_27, &phi_bb378_28, &phi_bb378_29, &phi_bb378_30, &phi_bb378_31, &phi_bb378_34, &phi_bb378_35, &phi_bb378_36, &phi_bb378_37, &phi_bb378_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 788);
    tmp820 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb378_17, phi_bb378_18, phi_bb378_19, tmp785, tmp820);
  }

  TNode<Object> phi_bb379_17;
  TNode<IntPtrT> phi_bb379_18;
  TNode<BoolT> phi_bb379_19;
  TNode<IntPtrT> phi_bb379_27;
  TNode<IntPtrT> phi_bb379_28;
  TNode<IntPtrT> phi_bb379_29;
  TNode<IntPtrT> phi_bb379_30;
  TNode<IntPtrT> phi_bb379_31;
  TNode<IntPtrT> phi_bb379_34;
  TNode<BoolT> phi_bb379_35;
  TNode<BoolT> phi_bb379_36;
  TNode<Object> phi_bb379_37;
  TNode<Int32T> phi_bb379_38;
  TNode<HeapObject> tmp821;
  if (block379.is_used()) {
    ca_.Bind(&block379, &phi_bb379_17, &phi_bb379_18, &phi_bb379_19, &phi_bb379_27, &phi_bb379_28, &phi_bb379_29, &phi_bb379_30, &phi_bb379_31, &phi_bb379_34, &phi_bb379_35, &phi_bb379_36, &phi_bb379_37, &phi_bb379_38);
    tmp821 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb379_17, phi_bb379_18, phi_bb379_19, tmp789, tmp821);
  }

  TNode<Object> phi_bb380_17;
  TNode<IntPtrT> phi_bb380_18;
  TNode<BoolT> phi_bb380_19;
  TNode<IntPtrT> phi_bb380_27;
  TNode<IntPtrT> phi_bb380_28;
  TNode<IntPtrT> phi_bb380_29;
  TNode<IntPtrT> phi_bb380_30;
  TNode<IntPtrT> phi_bb380_31;
  TNode<IntPtrT> phi_bb380_34;
  TNode<BoolT> phi_bb380_35;
  TNode<BoolT> phi_bb380_36;
  TNode<Object> phi_bb380_37;
  TNode<Int32T> phi_bb380_38;
  TNode<HeapObject> tmp822;
  if (block380.is_used()) {
    ca_.Bind(&block380, &phi_bb380_17, &phi_bb380_18, &phi_bb380_19, &phi_bb380_27, &phi_bb380_28, &phi_bb380_29, &phi_bb380_30, &phi_bb380_31, &phi_bb380_34, &phi_bb380_35, &phi_bb380_36, &phi_bb380_37, &phi_bb380_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 790);
    tmp822 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb380_17, phi_bb380_18, phi_bb380_19, tmp792, tmp822);
  }

  TNode<Object> phi_bb381_17;
  TNode<IntPtrT> phi_bb381_18;
  TNode<BoolT> phi_bb381_19;
  TNode<IntPtrT> phi_bb381_27;
  TNode<IntPtrT> phi_bb381_28;
  TNode<IntPtrT> phi_bb381_29;
  TNode<IntPtrT> phi_bb381_30;
  TNode<IntPtrT> phi_bb381_31;
  TNode<IntPtrT> phi_bb381_34;
  TNode<BoolT> phi_bb381_35;
  TNode<BoolT> phi_bb381_36;
  TNode<Object> phi_bb381_37;
  TNode<Int32T> phi_bb381_38;
  TNode<HeapObject> tmp823;
  if (block381.is_used()) {
    ca_.Bind(&block381, &phi_bb381_17, &phi_bb381_18, &phi_bb381_19, &phi_bb381_27, &phi_bb381_28, &phi_bb381_29, &phi_bb381_30, &phi_bb381_31, &phi_bb381_34, &phi_bb381_35, &phi_bb381_36, &phi_bb381_37, &phi_bb381_38);
    tmp823 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb381_17, phi_bb381_18, phi_bb381_19, tmp796, tmp823);
  }

  TNode<Object> phi_bb382_17;
  TNode<IntPtrT> phi_bb382_18;
  TNode<BoolT> phi_bb382_19;
  TNode<IntPtrT> phi_bb382_27;
  TNode<IntPtrT> phi_bb382_28;
  TNode<IntPtrT> phi_bb382_29;
  TNode<IntPtrT> phi_bb382_30;
  TNode<IntPtrT> phi_bb382_31;
  TNode<IntPtrT> phi_bb382_34;
  TNode<BoolT> phi_bb382_35;
  TNode<BoolT> phi_bb382_36;
  TNode<Object> phi_bb382_37;
  TNode<Int32T> phi_bb382_38;
  TNode<HeapObject> tmp824;
  if (block382.is_used()) {
    ca_.Bind(&block382, &phi_bb382_17, &phi_bb382_18, &phi_bb382_19, &phi_bb382_27, &phi_bb382_28, &phi_bb382_29, &phi_bb382_30, &phi_bb382_31, &phi_bb382_34, &phi_bb382_35, &phi_bb382_36, &phi_bb382_37, &phi_bb382_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 792);
    tmp824 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb382_17, phi_bb382_18, phi_bb382_19, tmp799, tmp824);
  }

  TNode<Object> phi_bb383_17;
  TNode<IntPtrT> phi_bb383_18;
  TNode<BoolT> phi_bb383_19;
  TNode<IntPtrT> phi_bb383_27;
  TNode<IntPtrT> phi_bb383_28;
  TNode<IntPtrT> phi_bb383_29;
  TNode<IntPtrT> phi_bb383_30;
  TNode<IntPtrT> phi_bb383_31;
  TNode<IntPtrT> phi_bb383_34;
  TNode<BoolT> phi_bb383_35;
  TNode<BoolT> phi_bb383_36;
  TNode<Object> phi_bb383_37;
  TNode<Int32T> phi_bb383_38;
  TNode<HeapObject> tmp825;
  if (block383.is_used()) {
    ca_.Bind(&block383, &phi_bb383_17, &phi_bb383_18, &phi_bb383_19, &phi_bb383_27, &phi_bb383_28, &phi_bb383_29, &phi_bb383_30, &phi_bb383_31, &phi_bb383_34, &phi_bb383_35, &phi_bb383_36, &phi_bb383_37, &phi_bb383_38);
    tmp825 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb383_17, phi_bb383_18, phi_bb383_19, tmp803, tmp825);
  }

  TNode<Object> phi_bb384_17;
  TNode<IntPtrT> phi_bb384_18;
  TNode<BoolT> phi_bb384_19;
  TNode<IntPtrT> phi_bb384_27;
  TNode<IntPtrT> phi_bb384_28;
  TNode<IntPtrT> phi_bb384_29;
  TNode<IntPtrT> phi_bb384_30;
  TNode<IntPtrT> phi_bb384_31;
  TNode<IntPtrT> phi_bb384_34;
  TNode<BoolT> phi_bb384_35;
  TNode<BoolT> phi_bb384_36;
  TNode<Object> phi_bb384_37;
  TNode<Int32T> phi_bb384_38;
  TNode<HeapObject> tmp826;
  if (block384.is_used()) {
    ca_.Bind(&block384, &phi_bb384_17, &phi_bb384_18, &phi_bb384_19, &phi_bb384_27, &phi_bb384_28, &phi_bb384_29, &phi_bb384_30, &phi_bb384_31, &phi_bb384_34, &phi_bb384_35, &phi_bb384_36, &phi_bb384_37, &phi_bb384_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 794);
    tmp826 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb384_17, phi_bb384_18, phi_bb384_19, tmp806, tmp826);
  }

  TNode<Object> phi_bb385_17;
  TNode<IntPtrT> phi_bb385_18;
  TNode<BoolT> phi_bb385_19;
  TNode<IntPtrT> phi_bb385_27;
  TNode<IntPtrT> phi_bb385_28;
  TNode<IntPtrT> phi_bb385_29;
  TNode<IntPtrT> phi_bb385_30;
  TNode<IntPtrT> phi_bb385_31;
  TNode<IntPtrT> phi_bb385_34;
  TNode<BoolT> phi_bb385_35;
  TNode<BoolT> phi_bb385_36;
  TNode<Object> phi_bb385_37;
  TNode<Int32T> phi_bb385_38;
  TNode<HeapObject> tmp827;
  if (block385.is_used()) {
    ca_.Bind(&block385, &phi_bb385_17, &phi_bb385_18, &phi_bb385_19, &phi_bb385_27, &phi_bb385_28, &phi_bb385_29, &phi_bb385_30, &phi_bb385_31, &phi_bb385_34, &phi_bb385_35, &phi_bb385_36, &phi_bb385_37, &phi_bb385_38);
    tmp827 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb385_17, phi_bb385_18, phi_bb385_19, tmp810, tmp827);
  }

  TNode<Object> phi_bb386_17;
  TNode<IntPtrT> phi_bb386_18;
  TNode<BoolT> phi_bb386_19;
  TNode<IntPtrT> phi_bb386_27;
  TNode<IntPtrT> phi_bb386_28;
  TNode<IntPtrT> phi_bb386_29;
  TNode<IntPtrT> phi_bb386_30;
  TNode<IntPtrT> phi_bb386_31;
  TNode<IntPtrT> phi_bb386_34;
  TNode<BoolT> phi_bb386_35;
  TNode<BoolT> phi_bb386_36;
  TNode<Object> phi_bb386_37;
  TNode<Int32T> phi_bb386_38;
  TNode<Object> tmp828;
      TNode<Object> tmp830;
  if (block386.is_used()) {
    ca_.Bind(&block386, &phi_bb386_17, &phi_bb386_18, &phi_bb386_19, &phi_bb386_27, &phi_bb386_28, &phi_bb386_29, &phi_bb386_30, &phi_bb386_31, &phi_bb386_34, &phi_bb386_35, &phi_bb386_36, &phi_bb386_37, &phi_bb386_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 802);
    compiler::CodeAssemblerExceptionHandlerLabel catch829__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch829__label);
    tmp828 = ca_.CallBuiltin<Object>(Builtin::kWasmReturnPromiseOnSuspendAsm, TNode<Object>(), tmp761, tmp20, phi_bb386_17);
    }
    if (catch829__label.is_used()) {
      compiler::CodeAssemblerLabel catch829_skip(&ca_);
      ca_.Goto(&catch829_skip);
      ca_.Bind(&catch829__label, &tmp830);
      ca_.Goto(&block389, phi_bb386_17, phi_bb386_18, phi_bb386_19, phi_bb386_27, phi_bb386_28, phi_bb386_29, phi_bb386_30, phi_bb386_31, phi_bb386_34, phi_bb386_35, phi_bb386_36, phi_bb386_37, phi_bb386_38, phi_bb386_17);
      ca_.Bind(&catch829_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 801);
    ca_.Goto(&block388, phi_bb386_17, phi_bb386_18, phi_bb386_19, phi_bb386_27, phi_bb386_28, phi_bb386_29, phi_bb386_30, phi_bb386_31, phi_bb386_34, phi_bb386_35, phi_bb386_36, phi_bb386_37, phi_bb386_38, tmp828);
  }

  TNode<Object> phi_bb389_17;
  TNode<IntPtrT> phi_bb389_18;
  TNode<BoolT> phi_bb389_19;
  TNode<IntPtrT> phi_bb389_27;
  TNode<IntPtrT> phi_bb389_28;
  TNode<IntPtrT> phi_bb389_29;
  TNode<IntPtrT> phi_bb389_30;
  TNode<IntPtrT> phi_bb389_31;
  TNode<IntPtrT> phi_bb389_34;
  TNode<BoolT> phi_bb389_35;
  TNode<BoolT> phi_bb389_36;
  TNode<Object> phi_bb389_37;
  TNode<Int32T> phi_bb389_38;
  TNode<Object> phi_bb389_47;
  TNode<HeapObject> tmp831;
  if (block389.is_used()) {
    ca_.Bind(&block389, &phi_bb389_17, &phi_bb389_18, &phi_bb389_19, &phi_bb389_27, &phi_bb389_28, &phi_bb389_29, &phi_bb389_30, &phi_bb389_31, &phi_bb389_34, &phi_bb389_35, &phi_bb389_36, &phi_bb389_37, &phi_bb389_38, &phi_bb389_47);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 802);
    tmp831 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb389_17, phi_bb389_18, phi_bb389_19, tmp830, tmp831);
  }

  TNode<Object> phi_bb387_17;
  TNode<IntPtrT> phi_bb387_18;
  TNode<BoolT> phi_bb387_19;
  TNode<IntPtrT> phi_bb387_27;
  TNode<IntPtrT> phi_bb387_28;
  TNode<IntPtrT> phi_bb387_29;
  TNode<IntPtrT> phi_bb387_30;
  TNode<IntPtrT> phi_bb387_31;
  TNode<IntPtrT> phi_bb387_34;
  TNode<BoolT> phi_bb387_35;
  TNode<BoolT> phi_bb387_36;
  TNode<Object> phi_bb387_37;
  TNode<Int32T> phi_bb387_38;
  TNode<BoolT> tmp832;
      TNode<Object> tmp834;
  if (block387.is_used()) {
    ca_.Bind(&block387, &phi_bb387_17, &phi_bb387_18, &phi_bb387_19, &phi_bb387_27, &phi_bb387_28, &phi_bb387_29, &phi_bb387_30, &phi_bb387_31, &phi_bb387_34, &phi_bb387_35, &phi_bb387_36, &phi_bb387_37, &phi_bb387_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 804);
    compiler::CodeAssemblerExceptionHandlerLabel catch833__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch833__label);
    tmp832 = FromConstexpr_bool_constexpr_bool_0(state_, (CodeStubAssembler(state_).ConstexprInt32Equal(p_promise, wasm::Promise::kNoPromise)));
    }
    if (catch833__label.is_used()) {
      compiler::CodeAssemblerLabel catch833_skip(&ca_);
      ca_.Goto(&catch833_skip);
      ca_.Bind(&catch833__label, &tmp834);
      ca_.Goto(&block392, phi_bb387_17, phi_bb387_18, phi_bb387_19, phi_bb387_27, phi_bb387_28, phi_bb387_29, phi_bb387_30, phi_bb387_31, phi_bb387_34, phi_bb387_35, phi_bb387_36, phi_bb387_37, phi_bb387_38);
      ca_.Bind(&catch833_skip);
    }
    ca_.Branch(tmp832, &block390, std::vector<compiler::Node*>{phi_bb387_17, phi_bb387_18, phi_bb387_19, phi_bb387_27, phi_bb387_28, phi_bb387_29, phi_bb387_30, phi_bb387_31, phi_bb387_34, phi_bb387_35, phi_bb387_36, phi_bb387_37, phi_bb387_38}, &block391, std::vector<compiler::Node*>{phi_bb387_17, phi_bb387_18, phi_bb387_19, phi_bb387_27, phi_bb387_28, phi_bb387_29, phi_bb387_30, phi_bb387_31, phi_bb387_34, phi_bb387_35, phi_bb387_36, phi_bb387_37, phi_bb387_38});
  }

  TNode<Object> phi_bb392_17;
  TNode<IntPtrT> phi_bb392_18;
  TNode<BoolT> phi_bb392_19;
  TNode<IntPtrT> phi_bb392_27;
  TNode<IntPtrT> phi_bb392_28;
  TNode<IntPtrT> phi_bb392_29;
  TNode<IntPtrT> phi_bb392_30;
  TNode<IntPtrT> phi_bb392_31;
  TNode<IntPtrT> phi_bb392_34;
  TNode<BoolT> phi_bb392_35;
  TNode<BoolT> phi_bb392_36;
  TNode<Object> phi_bb392_37;
  TNode<Int32T> phi_bb392_38;
  TNode<HeapObject> tmp835;
  if (block392.is_used()) {
    ca_.Bind(&block392, &phi_bb392_17, &phi_bb392_18, &phi_bb392_19, &phi_bb392_27, &phi_bb392_28, &phi_bb392_29, &phi_bb392_30, &phi_bb392_31, &phi_bb392_34, &phi_bb392_35, &phi_bb392_36, &phi_bb392_37, &phi_bb392_38);
    tmp835 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb392_17, phi_bb392_18, phi_bb392_19, tmp834, tmp835);
  }

  TNode<Object> phi_bb390_17;
  TNode<IntPtrT> phi_bb390_18;
  TNode<BoolT> phi_bb390_19;
  TNode<IntPtrT> phi_bb390_27;
  TNode<IntPtrT> phi_bb390_28;
  TNode<IntPtrT> phi_bb390_29;
  TNode<IntPtrT> phi_bb390_30;
  TNode<IntPtrT> phi_bb390_31;
  TNode<IntPtrT> phi_bb390_34;
  TNode<BoolT> phi_bb390_35;
  TNode<BoolT> phi_bb390_36;
  TNode<Object> phi_bb390_37;
  TNode<Int32T> phi_bb390_38;
  TNode<Object> tmp836;
      TNode<Object> tmp838;
  if (block390.is_used()) {
    ca_.Bind(&block390, &phi_bb390_17, &phi_bb390_18, &phi_bb390_19, &phi_bb390_27, &phi_bb390_28, &phi_bb390_29, &phi_bb390_30, &phi_bb390_31, &phi_bb390_34, &phi_bb390_35, &phi_bb390_36, &phi_bb390_37, &phi_bb390_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 805);
    compiler::CodeAssemblerExceptionHandlerLabel catch837__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch837__label);
    tmp836 = ca_.CallBuiltin<Object>(Builtin::kJSToWasmWrapperAsm, TNode<Object>(), tmp761, tmp20, phi_bb390_17);
    }
    if (catch837__label.is_used()) {
      compiler::CodeAssemblerLabel catch837_skip(&ca_);
      ca_.Goto(&catch837_skip);
      ca_.Bind(&catch837__label, &tmp838);
      ca_.Goto(&block394, phi_bb390_17, phi_bb390_18, phi_bb390_19, phi_bb390_27, phi_bb390_28, phi_bb390_29, phi_bb390_30, phi_bb390_31, phi_bb390_34, phi_bb390_35, phi_bb390_36, phi_bb390_37, phi_bb390_38, phi_bb390_17);
      ca_.Bind(&catch837_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 804);
    ca_.Goto(&block393, phi_bb390_17, phi_bb390_18, phi_bb390_19, phi_bb390_27, phi_bb390_28, phi_bb390_29, phi_bb390_30, phi_bb390_31, phi_bb390_34, phi_bb390_35, phi_bb390_36, phi_bb390_37, phi_bb390_38, tmp836);
  }

  TNode<Object> phi_bb394_17;
  TNode<IntPtrT> phi_bb394_18;
  TNode<BoolT> phi_bb394_19;
  TNode<IntPtrT> phi_bb394_27;
  TNode<IntPtrT> phi_bb394_28;
  TNode<IntPtrT> phi_bb394_29;
  TNode<IntPtrT> phi_bb394_30;
  TNode<IntPtrT> phi_bb394_31;
  TNode<IntPtrT> phi_bb394_34;
  TNode<BoolT> phi_bb394_35;
  TNode<BoolT> phi_bb394_36;
  TNode<Object> phi_bb394_37;
  TNode<Int32T> phi_bb394_38;
  TNode<Object> phi_bb394_47;
  TNode<HeapObject> tmp839;
  if (block394.is_used()) {
    ca_.Bind(&block394, &phi_bb394_17, &phi_bb394_18, &phi_bb394_19, &phi_bb394_27, &phi_bb394_28, &phi_bb394_29, &phi_bb394_30, &phi_bb394_31, &phi_bb394_34, &phi_bb394_35, &phi_bb394_36, &phi_bb394_37, &phi_bb394_38, &phi_bb394_47);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 805);
    tmp839 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb394_17, phi_bb394_18, phi_bb394_19, tmp838, tmp839);
  }

  TNode<Object> phi_bb391_17;
  TNode<IntPtrT> phi_bb391_18;
  TNode<BoolT> phi_bb391_19;
  TNode<IntPtrT> phi_bb391_27;
  TNode<IntPtrT> phi_bb391_28;
  TNode<IntPtrT> phi_bb391_29;
  TNode<IntPtrT> phi_bb391_30;
  TNode<IntPtrT> phi_bb391_31;
  TNode<IntPtrT> phi_bb391_34;
  TNode<BoolT> phi_bb391_35;
  TNode<BoolT> phi_bb391_36;
  TNode<Object> phi_bb391_37;
  TNode<Int32T> phi_bb391_38;
  TNode<BoolT> tmp840;
      TNode<Object> tmp842;
  if (block391.is_used()) {
    ca_.Bind(&block391, &phi_bb391_17, &phi_bb391_18, &phi_bb391_19, &phi_bb391_27, &phi_bb391_28, &phi_bb391_29, &phi_bb391_30, &phi_bb391_31, &phi_bb391_34, &phi_bb391_35, &phi_bb391_36, &phi_bb391_37, &phi_bb391_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 806);
    compiler::CodeAssemblerExceptionHandlerLabel catch841__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch841__label);
    tmp840 = FromConstexpr_bool_constexpr_bool_0(state_, (CodeStubAssembler(state_).ConstexprInt32Equal(p_promise, wasm::Promise::kStressSwitch)));
    }
    if (catch841__label.is_used()) {
      compiler::CodeAssemblerLabel catch841_skip(&ca_);
      ca_.Goto(&catch841_skip);
      ca_.Bind(&catch841__label, &tmp842);
      ca_.Goto(&block397, phi_bb391_17, phi_bb391_18, phi_bb391_19, phi_bb391_27, phi_bb391_28, phi_bb391_29, phi_bb391_30, phi_bb391_31, phi_bb391_34, phi_bb391_35, phi_bb391_36, phi_bb391_37, phi_bb391_38);
      ca_.Bind(&catch841_skip);
    }
    ca_.Branch(tmp840, &block395, std::vector<compiler::Node*>{phi_bb391_17, phi_bb391_18, phi_bb391_19, phi_bb391_27, phi_bb391_28, phi_bb391_29, phi_bb391_30, phi_bb391_31, phi_bb391_34, phi_bb391_35, phi_bb391_36, phi_bb391_37, phi_bb391_38}, &block396, std::vector<compiler::Node*>{phi_bb391_17, phi_bb391_18, phi_bb391_19, phi_bb391_27, phi_bb391_28, phi_bb391_29, phi_bb391_30, phi_bb391_31, phi_bb391_34, phi_bb391_35, phi_bb391_36, phi_bb391_37, phi_bb391_38});
  }

  TNode<Object> phi_bb397_17;
  TNode<IntPtrT> phi_bb397_18;
  TNode<BoolT> phi_bb397_19;
  TNode<IntPtrT> phi_bb397_27;
  TNode<IntPtrT> phi_bb397_28;
  TNode<IntPtrT> phi_bb397_29;
  TNode<IntPtrT> phi_bb397_30;
  TNode<IntPtrT> phi_bb397_31;
  TNode<IntPtrT> phi_bb397_34;
  TNode<BoolT> phi_bb397_35;
  TNode<BoolT> phi_bb397_36;
  TNode<Object> phi_bb397_37;
  TNode<Int32T> phi_bb397_38;
  TNode<HeapObject> tmp843;
  if (block397.is_used()) {
    ca_.Bind(&block397, &phi_bb397_17, &phi_bb397_18, &phi_bb397_19, &phi_bb397_27, &phi_bb397_28, &phi_bb397_29, &phi_bb397_30, &phi_bb397_31, &phi_bb397_34, &phi_bb397_35, &phi_bb397_36, &phi_bb397_37, &phi_bb397_38);
    tmp843 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb397_17, phi_bb397_18, phi_bb397_19, tmp842, tmp843);
  }

  TNode<Object> phi_bb395_17;
  TNode<IntPtrT> phi_bb395_18;
  TNode<BoolT> phi_bb395_19;
  TNode<IntPtrT> phi_bb395_27;
  TNode<IntPtrT> phi_bb395_28;
  TNode<IntPtrT> phi_bb395_29;
  TNode<IntPtrT> phi_bb395_30;
  TNode<IntPtrT> phi_bb395_31;
  TNode<IntPtrT> phi_bb395_34;
  TNode<BoolT> phi_bb395_35;
  TNode<BoolT> phi_bb395_36;
  TNode<Object> phi_bb395_37;
  TNode<Int32T> phi_bb395_38;
  TNode<Object> tmp844;
      TNode<Object> tmp846;
  if (block395.is_used()) {
    ca_.Bind(&block395, &phi_bb395_17, &phi_bb395_18, &phi_bb395_19, &phi_bb395_27, &phi_bb395_28, &phi_bb395_29, &phi_bb395_30, &phi_bb395_31, &phi_bb395_34, &phi_bb395_35, &phi_bb395_36, &phi_bb395_37, &phi_bb395_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 807);
    compiler::CodeAssemblerExceptionHandlerLabel catch845__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch845__label);
    tmp844 = ca_.CallBuiltin<Object>(Builtin::kJSToWasmStressSwitchStacksAsm, TNode<Object>(), tmp761, tmp20, phi_bb395_17);
    }
    if (catch845__label.is_used()) {
      compiler::CodeAssemblerLabel catch845_skip(&ca_);
      ca_.Goto(&catch845_skip);
      ca_.Bind(&catch845__label, &tmp846);
      ca_.Goto(&block399, phi_bb395_17, phi_bb395_18, phi_bb395_19, phi_bb395_27, phi_bb395_28, phi_bb395_29, phi_bb395_30, phi_bb395_31, phi_bb395_34, phi_bb395_35, phi_bb395_36, phi_bb395_37, phi_bb395_38, phi_bb395_17);
      ca_.Bind(&catch845_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 804);
    ca_.Goto(&block393, phi_bb395_17, phi_bb395_18, phi_bb395_19, phi_bb395_27, phi_bb395_28, phi_bb395_29, phi_bb395_30, phi_bb395_31, phi_bb395_34, phi_bb395_35, phi_bb395_36, phi_bb395_37, phi_bb395_38, tmp844);
  }

  TNode<Object> phi_bb399_17;
  TNode<IntPtrT> phi_bb399_18;
  TNode<BoolT> phi_bb399_19;
  TNode<IntPtrT> phi_bb399_27;
  TNode<IntPtrT> phi_bb399_28;
  TNode<IntPtrT> phi_bb399_29;
  TNode<IntPtrT> phi_bb399_30;
  TNode<IntPtrT> phi_bb399_31;
  TNode<IntPtrT> phi_bb399_34;
  TNode<BoolT> phi_bb399_35;
  TNode<BoolT> phi_bb399_36;
  TNode<Object> phi_bb399_37;
  TNode<Int32T> phi_bb399_38;
  TNode<Object> phi_bb399_47;
  TNode<HeapObject> tmp847;
  if (block399.is_used()) {
    ca_.Bind(&block399, &phi_bb399_17, &phi_bb399_18, &phi_bb399_19, &phi_bb399_27, &phi_bb399_28, &phi_bb399_29, &phi_bb399_30, &phi_bb399_31, &phi_bb399_34, &phi_bb399_35, &phi_bb399_36, &phi_bb399_37, &phi_bb399_38, &phi_bb399_47);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 807);
    tmp847 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb399_17, phi_bb399_18, phi_bb399_19, tmp846, tmp847);
  }

  TNode<Object> phi_bb396_17;
  TNode<IntPtrT> phi_bb396_18;
  TNode<BoolT> phi_bb396_19;
  TNode<IntPtrT> phi_bb396_27;
  TNode<IntPtrT> phi_bb396_28;
  TNode<IntPtrT> phi_bb396_29;
  TNode<IntPtrT> phi_bb396_30;
  TNode<IntPtrT> phi_bb396_31;
  TNode<IntPtrT> phi_bb396_34;
  TNode<BoolT> phi_bb396_35;
  TNode<BoolT> phi_bb396_36;
  TNode<Object> phi_bb396_37;
  TNode<Int32T> phi_bb396_38;
  if (block396.is_used()) {
    ca_.Bind(&block396, &phi_bb396_17, &phi_bb396_18, &phi_bb396_19, &phi_bb396_27, &phi_bb396_28, &phi_bb396_29, &phi_bb396_30, &phi_bb396_31, &phi_bb396_34, &phi_bb396_35, &phi_bb396_36, &phi_bb396_37, &phi_bb396_38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 810);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<Object> phi_bb393_17;
  TNode<IntPtrT> phi_bb393_18;
  TNode<BoolT> phi_bb393_19;
  TNode<IntPtrT> phi_bb393_27;
  TNode<IntPtrT> phi_bb393_28;
  TNode<IntPtrT> phi_bb393_29;
  TNode<IntPtrT> phi_bb393_30;
  TNode<IntPtrT> phi_bb393_31;
  TNode<IntPtrT> phi_bb393_34;
  TNode<BoolT> phi_bb393_35;
  TNode<BoolT> phi_bb393_36;
  TNode<Object> phi_bb393_37;
  TNode<Int32T> phi_bb393_38;
  TNode<Object> phi_bb393_44;
  if (block393.is_used()) {
    ca_.Bind(&block393, &phi_bb393_17, &phi_bb393_18, &phi_bb393_19, &phi_bb393_27, &phi_bb393_28, &phi_bb393_29, &phi_bb393_30, &phi_bb393_31, &phi_bb393_34, &phi_bb393_35, &phi_bb393_36, &phi_bb393_37, &phi_bb393_38, &phi_bb393_44);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 801);
    ca_.Goto(&block388, phi_bb393_17, phi_bb393_18, phi_bb393_19, phi_bb393_27, phi_bb393_28, phi_bb393_29, phi_bb393_30, phi_bb393_31, phi_bb393_34, phi_bb393_35, phi_bb393_36, phi_bb393_37, phi_bb393_38, phi_bb393_44);
  }

  TNode<Object> phi_bb388_17;
  TNode<IntPtrT> phi_bb388_18;
  TNode<BoolT> phi_bb388_19;
  TNode<IntPtrT> phi_bb388_27;
  TNode<IntPtrT> phi_bb388_28;
  TNode<IntPtrT> phi_bb388_29;
  TNode<IntPtrT> phi_bb388_30;
  TNode<IntPtrT> phi_bb388_31;
  TNode<IntPtrT> phi_bb388_34;
  TNode<BoolT> phi_bb388_35;
  TNode<BoolT> phi_bb388_36;
  TNode<Object> phi_bb388_37;
  TNode<Int32T> phi_bb388_38;
  TNode<Object> phi_bb388_44;
  TNode<Int32T> tmp848;
      TNode<Object> tmp850;
  TNode<BoolT> tmp851;
      TNode<Object> tmp853;
  if (block388.is_used()) {
    ca_.Bind(&block388, &phi_bb388_17, &phi_bb388_18, &phi_bb388_19, &phi_bb388_27, &phi_bb388_28, &phi_bb388_29, &phi_bb388_30, &phi_bb388_31, &phi_bb388_34, &phi_bb388_35, &phi_bb388_36, &phi_bb388_37, &phi_bb388_38, &phi_bb388_44);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 812);
    compiler::CodeAssemblerExceptionHandlerLabel catch849__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch849__label);
    tmp848 = FromConstexpr_int32_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0xaull));
    }
    if (catch849__label.is_used()) {
      compiler::CodeAssemblerLabel catch849_skip(&ca_);
      ca_.Goto(&catch849_skip);
      ca_.Bind(&catch849__label, &tmp850);
      ca_.Goto(&block402, phi_bb388_17, phi_bb388_18, phi_bb388_19, phi_bb388_27, phi_bb388_28, phi_bb388_29, phi_bb388_30, phi_bb388_31, phi_bb388_34, phi_bb388_35, phi_bb388_36, phi_bb388_37, phi_bb388_38, phi_bb388_44);
      ca_.Bind(&catch849_skip);
    }
    compiler::CodeAssemblerExceptionHandlerLabel catch852__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch852__label);
    tmp851 = CodeStubAssembler(state_).Int32GreaterThan(TNode<Int32T>{tmp26}, TNode<Int32T>{tmp848});
    }
    if (catch852__label.is_used()) {
      compiler::CodeAssemblerLabel catch852_skip(&ca_);
      ca_.Goto(&catch852_skip);
      ca_.Bind(&catch852__label, &tmp853);
      ca_.Goto(&block403, phi_bb388_17, phi_bb388_18, phi_bb388_19, phi_bb388_27, phi_bb388_28, phi_bb388_29, phi_bb388_30, phi_bb388_31, phi_bb388_34, phi_bb388_35, phi_bb388_36, phi_bb388_37, phi_bb388_38, phi_bb388_44);
      ca_.Bind(&catch852_skip);
    }
    ca_.Branch(tmp851, &block400, std::vector<compiler::Node*>{phi_bb388_17, phi_bb388_18, phi_bb388_19, phi_bb388_27, phi_bb388_28, phi_bb388_29, phi_bb388_30, phi_bb388_31, phi_bb388_34, phi_bb388_35, phi_bb388_36, phi_bb388_37, phi_bb388_38, phi_bb388_44}, &block401, std::vector<compiler::Node*>{phi_bb388_17, phi_bb388_18, phi_bb388_19, phi_bb388_27, phi_bb388_28, phi_bb388_29, phi_bb388_30, phi_bb388_31, phi_bb388_34, phi_bb388_35, phi_bb388_36, phi_bb388_37, phi_bb388_38, phi_bb388_44});
  }

  TNode<Object> phi_bb402_17;
  TNode<IntPtrT> phi_bb402_18;
  TNode<BoolT> phi_bb402_19;
  TNode<IntPtrT> phi_bb402_27;
  TNode<IntPtrT> phi_bb402_28;
  TNode<IntPtrT> phi_bb402_29;
  TNode<IntPtrT> phi_bb402_30;
  TNode<IntPtrT> phi_bb402_31;
  TNode<IntPtrT> phi_bb402_34;
  TNode<BoolT> phi_bb402_35;
  TNode<BoolT> phi_bb402_36;
  TNode<Object> phi_bb402_37;
  TNode<Int32T> phi_bb402_38;
  TNode<Object> phi_bb402_44;
  TNode<HeapObject> tmp854;
  if (block402.is_used()) {
    ca_.Bind(&block402, &phi_bb402_17, &phi_bb402_18, &phi_bb402_19, &phi_bb402_27, &phi_bb402_28, &phi_bb402_29, &phi_bb402_30, &phi_bb402_31, &phi_bb402_34, &phi_bb402_35, &phi_bb402_36, &phi_bb402_37, &phi_bb402_38, &phi_bb402_44);
    tmp854 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb402_17, phi_bb402_18, phi_bb402_19, tmp850, tmp854);
  }

  TNode<Object> phi_bb403_17;
  TNode<IntPtrT> phi_bb403_18;
  TNode<BoolT> phi_bb403_19;
  TNode<IntPtrT> phi_bb403_27;
  TNode<IntPtrT> phi_bb403_28;
  TNode<IntPtrT> phi_bb403_29;
  TNode<IntPtrT> phi_bb403_30;
  TNode<IntPtrT> phi_bb403_31;
  TNode<IntPtrT> phi_bb403_34;
  TNode<BoolT> phi_bb403_35;
  TNode<BoolT> phi_bb403_36;
  TNode<Object> phi_bb403_37;
  TNode<Int32T> phi_bb403_38;
  TNode<Object> phi_bb403_44;
  TNode<HeapObject> tmp855;
  if (block403.is_used()) {
    ca_.Bind(&block403, &phi_bb403_17, &phi_bb403_18, &phi_bb403_19, &phi_bb403_27, &phi_bb403_28, &phi_bb403_29, &phi_bb403_30, &phi_bb403_31, &phi_bb403_34, &phi_bb403_35, &phi_bb403_36, &phi_bb403_37, &phi_bb403_38, &phi_bb403_44);
    tmp855 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb403_17, phi_bb403_18, phi_bb403_19, tmp853, tmp855);
  }

  TNode<Object> phi_bb400_17;
  TNode<IntPtrT> phi_bb400_18;
  TNode<BoolT> phi_bb400_19;
  TNode<IntPtrT> phi_bb400_27;
  TNode<IntPtrT> phi_bb400_28;
  TNode<IntPtrT> phi_bb400_29;
  TNode<IntPtrT> phi_bb400_30;
  TNode<IntPtrT> phi_bb400_31;
  TNode<IntPtrT> phi_bb400_34;
  TNode<BoolT> phi_bb400_35;
  TNode<BoolT> phi_bb400_36;
  TNode<Object> phi_bb400_37;
  TNode<Int32T> phi_bb400_38;
  TNode<Object> phi_bb400_44;
  TNode<RawPtrT> tmp856;
  TNode<RawPtrT> tmp857;
      TNode<Object> tmp859;
  if (block400.is_used()) {
    ca_.Bind(&block400, &phi_bb400_17, &phi_bb400_18, &phi_bb400_19, &phi_bb400_27, &phi_bb400_28, &phi_bb400_29, &phi_bb400_30, &phi_bb400_31, &phi_bb400_34, &phi_bb400_35, &phi_bb400_36, &phi_bb400_37, &phi_bb400_38, &phi_bb400_44);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 65);
    tmp856 = CodeStubAssembler(state_).GCUnsafeReferenceToRawPtr(TNode<Object>{phi_bb133_23}, TNode<IntPtrT>{phi_bb133_24});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 64);
    tmp857 = (TNode<RawPtrT>{tmp856});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 813);
    compiler::CodeAssemblerExceptionHandlerLabel catch858__label(&ca_, compiler::CodeAssemblerLabel::kDeferred);
    { compiler::ScopedExceptionHandler s(&ca_, &catch858__label);
    CodeStubAssembler(state_).DeallocateBuffer(TNode<RawPtrT>{tmp857});
    }
    if (catch858__label.is_used()) {
      compiler::CodeAssemblerLabel catch858_skip(&ca_);
      ca_.Goto(&catch858_skip);
      ca_.Bind(&catch858__label, &tmp859);
      ca_.Goto(&block405, phi_bb400_17, phi_bb400_18, phi_bb400_19, phi_bb400_27, phi_bb400_28, phi_bb400_29, phi_bb400_30, phi_bb400_31, phi_bb400_34, phi_bb400_35, phi_bb400_36, phi_bb400_37, phi_bb400_38, phi_bb400_44);
      ca_.Bind(&catch858_skip);
    }
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 812);
    ca_.Goto(&block401, phi_bb400_17, phi_bb400_18, phi_bb400_19, phi_bb400_27, phi_bb400_28, phi_bb400_29, phi_bb400_30, phi_bb400_31, phi_bb400_34, phi_bb400_35, phi_bb400_36, phi_bb400_37, phi_bb400_38, phi_bb400_44);
  }

  TNode<Object> phi_bb405_17;
  TNode<IntPtrT> phi_bb405_18;
  TNode<BoolT> phi_bb405_19;
  TNode<IntPtrT> phi_bb405_27;
  TNode<IntPtrT> phi_bb405_28;
  TNode<IntPtrT> phi_bb405_29;
  TNode<IntPtrT> phi_bb405_30;
  TNode<IntPtrT> phi_bb405_31;
  TNode<IntPtrT> phi_bb405_34;
  TNode<BoolT> phi_bb405_35;
  TNode<BoolT> phi_bb405_36;
  TNode<Object> phi_bb405_37;
  TNode<Int32T> phi_bb405_38;
  TNode<Object> phi_bb405_44;
  TNode<HeapObject> tmp860;
  if (block405.is_used()) {
    ca_.Bind(&block405, &phi_bb405_17, &phi_bb405_18, &phi_bb405_19, &phi_bb405_27, &phi_bb405_28, &phi_bb405_29, &phi_bb405_30, &phi_bb405_31, &phi_bb405_34, &phi_bb405_35, &phi_bb405_36, &phi_bb405_37, &phi_bb405_38, &phi_bb405_44);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 813);
    tmp860 = GetAndResetPendingMessage_0(state_);
    ca_.Goto(&block143, phi_bb405_17, phi_bb405_18, phi_bb405_19, tmp859, tmp860);
  }

  TNode<Object> phi_bb401_17;
  TNode<IntPtrT> phi_bb401_18;
  TNode<BoolT> phi_bb401_19;
  TNode<IntPtrT> phi_bb401_27;
  TNode<IntPtrT> phi_bb401_28;
  TNode<IntPtrT> phi_bb401_29;
  TNode<IntPtrT> phi_bb401_30;
  TNode<IntPtrT> phi_bb401_31;
  TNode<IntPtrT> phi_bb401_34;
  TNode<BoolT> phi_bb401_35;
  TNode<BoolT> phi_bb401_36;
  TNode<Object> phi_bb401_37;
  TNode<Int32T> phi_bb401_38;
  TNode<Object> phi_bb401_44;
  if (block401.is_used()) {
    ca_.Bind(&block401, &phi_bb401_17, &phi_bb401_18, &phi_bb401_19, &phi_bb401_27, &phi_bb401_28, &phi_bb401_29, &phi_bb401_30, &phi_bb401_31, &phi_bb401_34, &phi_bb401_35, &phi_bb401_36, &phi_bb401_37, &phi_bb401_38, &phi_bb401_44);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 507);
    ca_.Goto(&block409, phi_bb401_44);
  }

  TNode<Object> phi_bb143_17;
  TNode<IntPtrT> phi_bb143_18;
  TNode<BoolT> phi_bb143_19;
  TNode<Object> phi_bb143_26;
  TNode<HeapObject> phi_bb143_27;
  TNode<Int32T> tmp861;
  TNode<BoolT> tmp862;
  if (block143.is_used()) {
    ca_.Bind(&block143, &phi_bb143_17, &phi_bb143_18, &phi_bb143_19, &phi_bb143_26, &phi_bb143_27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 817);
    tmp861 = FromConstexpr_int32_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0xaull));
    tmp862 = CodeStubAssembler(state_).Int32GreaterThan(TNode<Int32T>{tmp26}, TNode<Int32T>{tmp861});
    ca_.Branch(tmp862, &block406, std::vector<compiler::Node*>{phi_bb143_17, phi_bb143_18, phi_bb143_19}, &block407, std::vector<compiler::Node*>{phi_bb143_17, phi_bb143_18, phi_bb143_19});
  }

  TNode<Object> phi_bb406_17;
  TNode<IntPtrT> phi_bb406_18;
  TNode<BoolT> phi_bb406_19;
  TNode<RawPtrT> tmp863;
  TNode<RawPtrT> tmp864;
  if (block406.is_used()) {
    ca_.Bind(&block406, &phi_bb406_17, &phi_bb406_18, &phi_bb406_19);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 65);
    tmp863 = CodeStubAssembler(state_).GCUnsafeReferenceToRawPtr(TNode<Object>{phi_bb133_23}, TNode<IntPtrT>{phi_bb133_24});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 64);
    tmp864 = (TNode<RawPtrT>{tmp863});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 819);
    CodeStubAssembler(state_).DeallocateBuffer(TNode<RawPtrT>{tmp864});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 817);
    ca_.Goto(&block407, phi_bb406_17, phi_bb406_18, phi_bb406_19);
  }

  TNode<Object> phi_bb407_17;
  TNode<IntPtrT> phi_bb407_18;
  TNode<BoolT> phi_bb407_19;
  if (block407.is_used()) {
    ca_.Bind(&block407, &phi_bb407_17, &phi_bb407_18, &phi_bb407_19);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 821);
    CodeStubAssembler(state_).CallRuntime(Runtime::kReThrowWithMessage, p_context, phi_bb143_26, phi_bb143_27);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<Object> phi_bb409_7;
    ca_.Bind(&block409, &phi_bb409_7);
  return TNode<Object>{phi_bb409_7};
}

TF_BUILTIN(JSToWasmWrapper, CodeStubAssembler) {
  compiler::CodeAssemblerState* state_ = state();  compiler::CodeAssembler ca_(state());
  TNode<Word32T> argc = UncheckedParameter<Word32T>(Descriptor::kJSActualArgumentsCount);
  TNode<IntPtrT> arguments_length(ChangeInt32ToIntPtr(UncheckedCast<Int32T>(argc)));
  TNode<RawPtrT> arguments_frame = UncheckedCast<RawPtrT>(LoadFramePointer());
  TorqueStructArguments torque_arguments(GetFrameArguments(arguments_frame, arguments_length, FrameArgumentsArgcType::kCountIncludesReceiver));
  CodeStubArguments arguments(this, torque_arguments);
  TNode<NativeContext> parameter0 = UncheckedParameter<NativeContext>(Descriptor::kContext);
  USE(parameter0);
  TNode<Object> parameter1 = arguments.GetReceiver();
  USE(parameter1);
  TNode<JSFunction> parameter2 = UncheckedParameter<JSFunction>(Descriptor::kJSTarget);
  USE(parameter2);
  TNode<JSDispatchHandleT> parameter3 = UncheckedParameter<JSDispatchHandleT>(Descriptor::kJSDispatchHandle);
  USE(parameter3);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Object> tmp0;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 830);
    CodeStubAssembler(state_).SetSupportsDynamicParameterCount(TNode<JSFunction>{parameter2}, TNode<JSDispatchHandleT>{parameter3});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 832);
    tmp0 = JSToWasmWrapperHelper_0(state_, TNode<NativeContext>{parameter0}, TNode<Object>{parameter1}, TNode<JSFunction>{parameter2}, TorqueStructArguments{TNode<RawPtrT>{torque_arguments.frame}, TNode<RawPtrT>{torque_arguments.base}, TNode<IntPtrT>{torque_arguments.length}, TNode<IntPtrT>{torque_arguments.actual_count}}, wasm::Promise::kNoPromise);
    arguments.PopAndReturn(tmp0);
  }
}

TF_BUILTIN(WasmPromising, CodeStubAssembler) {
  compiler::CodeAssemblerState* state_ = state();  compiler::CodeAssembler ca_(state());
  TNode<Word32T> argc = UncheckedParameter<Word32T>(Descriptor::kJSActualArgumentsCount);
  TNode<IntPtrT> arguments_length(ChangeInt32ToIntPtr(UncheckedCast<Int32T>(argc)));
  TNode<RawPtrT> arguments_frame = UncheckedCast<RawPtrT>(LoadFramePointer());
  TorqueStructArguments torque_arguments(GetFrameArguments(arguments_frame, arguments_length, FrameArgumentsArgcType::kCountIncludesReceiver));
  CodeStubArguments arguments(this, torque_arguments);
  TNode<NativeContext> parameter0 = UncheckedParameter<NativeContext>(Descriptor::kContext);
  USE(parameter0);
  TNode<Object> parameter1 = arguments.GetReceiver();
  USE(parameter1);
  TNode<JSFunction> parameter2 = UncheckedParameter<JSFunction>(Descriptor::kJSTarget);
  USE(parameter2);
  TNode<JSDispatchHandleT> parameter3 = UncheckedParameter<JSDispatchHandleT>(Descriptor::kJSDispatchHandle);
  USE(parameter3);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Object> tmp0;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 841);
    CodeStubAssembler(state_).SetSupportsDynamicParameterCount(TNode<JSFunction>{parameter2}, TNode<JSDispatchHandleT>{parameter3});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 843);
    tmp0 = JSToWasmWrapperHelper_0(state_, TNode<NativeContext>{parameter0}, TNode<Object>{parameter1}, TNode<JSFunction>{parameter2}, TorqueStructArguments{TNode<RawPtrT>{torque_arguments.frame}, TNode<RawPtrT>{torque_arguments.base}, TNode<IntPtrT>{torque_arguments.length}, TNode<IntPtrT>{torque_arguments.actual_count}}, wasm::Promise::kPromise);
    arguments.PopAndReturn(tmp0);
  }
}

TF_BUILTIN(WasmStressSwitch, CodeStubAssembler) {
  compiler::CodeAssemblerState* state_ = state();  compiler::CodeAssembler ca_(state());
  TNode<Word32T> argc = UncheckedParameter<Word32T>(Descriptor::kJSActualArgumentsCount);
  TNode<IntPtrT> arguments_length(ChangeInt32ToIntPtr(UncheckedCast<Int32T>(argc)));
  TNode<RawPtrT> arguments_frame = UncheckedCast<RawPtrT>(LoadFramePointer());
  TorqueStructArguments torque_arguments(GetFrameArguments(arguments_frame, arguments_length, FrameArgumentsArgcType::kCountIncludesReceiver));
  CodeStubArguments arguments(this, torque_arguments);
  TNode<NativeContext> parameter0 = UncheckedParameter<NativeContext>(Descriptor::kContext);
  USE(parameter0);
  TNode<Object> parameter1 = arguments.GetReceiver();
  USE(parameter1);
  TNode<JSFunction> parameter2 = UncheckedParameter<JSFunction>(Descriptor::kJSTarget);
  USE(parameter2);
  TNode<JSDispatchHandleT> parameter3 = UncheckedParameter<JSDispatchHandleT>(Descriptor::kJSDispatchHandle);
  USE(parameter3);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Object> tmp0;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 852);
    CodeStubAssembler(state_).SetSupportsDynamicParameterCount(TNode<JSFunction>{parameter2}, TNode<JSDispatchHandleT>{parameter3});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 854);
    tmp0 = JSToWasmWrapperHelper_0(state_, TNode<NativeContext>{parameter0}, TNode<Object>{parameter1}, TNode<JSFunction>{parameter2}, TorqueStructArguments{TNode<RawPtrT>{torque_arguments.frame}, TNode<RawPtrT>{torque_arguments.base}, TNode<IntPtrT>{torque_arguments.length}, TNode<IntPtrT>{torque_arguments.actual_count}}, wasm::Promise::kStressSwitch);
    arguments.PopAndReturn(tmp0);
  }
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=858&c=1
TNode<Object> WasmToJSObject_0(compiler::CodeAssemblerState* state_, TNode<NativeContext> p_context, TNode<Object> p_value, TNode<Int32T> p_retType) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block7(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block8(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block9(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block10(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block11(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block12(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block13(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block14(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block15(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block16(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block17(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block18(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block19(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block20(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block21(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block22(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block23(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block24(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block25(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block26(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block27(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block28(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block29(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block30(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block31(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block32(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block33(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block34(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block35(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block36(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block37(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block38(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block39(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block5(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block6(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block3(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block41(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block40(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block44(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block45(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block46(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block47(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block48(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block49(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block50(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block51(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block52(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block42(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block43(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block53(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block54(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block57(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block58(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block59(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block60(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block61(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block62(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block63(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block64(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block65(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block66(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block67(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block68(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block69(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block70(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block71(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block55(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block56(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object> block1(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Object> block72(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Int32T> tmp0;
  TNode<Int32T> tmp1;
  TNode<Int32T> tmp2;
  TNode<Int32T> tmp3;
  TNode<Int32T> tmp4;
  TNode<Int32T> tmp5;
  TNode<Int32T> tmp6;
  TNode<BoolT> tmp7;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 860);
    tmp0 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmValueKindBitsMask);
    tmp1 = CodeStubAssembler(state_).Word32And(TNode<Int32T>{p_retType}, TNode<Int32T>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 861);
    tmp2 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueType::kKindBits);
    tmp3 = CodeStubAssembler(state_).Word32Sar(TNode<Int32T>{p_retType}, TNode<Int32T>{tmp2});
    tmp4 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmHeapTypeBitsMask);
    tmp5 = CodeStubAssembler(state_).Word32And(TNode<Int32T>{tmp3}, TNode<Int32T>{tmp4});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 862);
    tmp6 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRef);
    tmp7 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp1}, TNode<Int32T>{tmp6});
    ca_.Branch(tmp7, &block2, std::vector<compiler::Node*>{}, &block3, std::vector<compiler::Node*>{});
  }

  TNode<Int32T> tmp8;
  TNode<BoolT> tmp9;
  if (block2.is_used()) {
    ca_.Bind(&block2);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 863);
    tmp8 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kEq);
    tmp9 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp8});
    ca_.Branch(tmp9, &block7, std::vector<compiler::Node*>{}, &block8, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp10;
  if (block7.is_used()) {
    ca_.Bind(&block7);
    tmp10 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block9, tmp10);
  }

  TNode<Int32T> tmp11;
  TNode<BoolT> tmp12;
  if (block8.is_used()) {
    ca_.Bind(&block8);
    tmp11 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kI31);
    tmp12 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp11});
    ca_.Goto(&block9, tmp12);
  }

  TNode<BoolT> phi_bb9_6;
  if (block9.is_used()) {
    ca_.Bind(&block9, &phi_bb9_6);
    ca_.Branch(phi_bb9_6, &block10, std::vector<compiler::Node*>{}, &block11, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp13;
  if (block10.is_used()) {
    ca_.Bind(&block10);
    tmp13 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block12, tmp13);
  }

  TNode<Int32T> tmp14;
  TNode<BoolT> tmp15;
  if (block11.is_used()) {
    ca_.Bind(&block11);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 864);
    tmp14 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kStruct);
    tmp15 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp14});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 863);
    ca_.Goto(&block12, tmp15);
  }

  TNode<BoolT> phi_bb12_6;
  if (block12.is_used()) {
    ca_.Bind(&block12, &phi_bb12_6);
    ca_.Branch(phi_bb12_6, &block13, std::vector<compiler::Node*>{}, &block14, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp16;
  if (block13.is_used()) {
    ca_.Bind(&block13);
    tmp16 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block15, tmp16);
  }

  TNode<Int32T> tmp17;
  TNode<BoolT> tmp18;
  if (block14.is_used()) {
    ca_.Bind(&block14);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 864);
    tmp17 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kArray);
    tmp18 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp17});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 863);
    ca_.Goto(&block15, tmp18);
  }

  TNode<BoolT> phi_bb15_6;
  if (block15.is_used()) {
    ca_.Bind(&block15, &phi_bb15_6);
    ca_.Branch(phi_bb15_6, &block16, std::vector<compiler::Node*>{}, &block17, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp19;
  if (block16.is_used()) {
    ca_.Bind(&block16);
    tmp19 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block18, tmp19);
  }

  TNode<Int32T> tmp20;
  TNode<BoolT> tmp21;
  if (block17.is_used()) {
    ca_.Bind(&block17);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 865);
    tmp20 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kAny);
    tmp21 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp20});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 863);
    ca_.Goto(&block18, tmp21);
  }

  TNode<BoolT> phi_bb18_6;
  if (block18.is_used()) {
    ca_.Bind(&block18, &phi_bb18_6);
    ca_.Branch(phi_bb18_6, &block19, std::vector<compiler::Node*>{}, &block20, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp22;
  if (block19.is_used()) {
    ca_.Bind(&block19);
    tmp22 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block21, tmp22);
  }

  TNode<Int32T> tmp23;
  TNode<BoolT> tmp24;
  if (block20.is_used()) {
    ca_.Bind(&block20);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 865);
    tmp23 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kExtern);
    tmp24 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp23});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 863);
    ca_.Goto(&block21, tmp24);
  }

  TNode<BoolT> phi_bb21_6;
  if (block21.is_used()) {
    ca_.Bind(&block21, &phi_bb21_6);
    ca_.Branch(phi_bb21_6, &block22, std::vector<compiler::Node*>{}, &block23, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp25;
  if (block22.is_used()) {
    ca_.Bind(&block22);
    tmp25 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block24, tmp25);
  }

  TNode<Int32T> tmp26;
  TNode<BoolT> tmp27;
  if (block23.is_used()) {
    ca_.Bind(&block23);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 866);
    tmp26 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kString);
    tmp27 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp26});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 863);
    ca_.Goto(&block24, tmp27);
  }

  TNode<BoolT> phi_bb24_6;
  if (block24.is_used()) {
    ca_.Bind(&block24, &phi_bb24_6);
    ca_.Branch(phi_bb24_6, &block25, std::vector<compiler::Node*>{}, &block26, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp28;
  if (block25.is_used()) {
    ca_.Bind(&block25);
    tmp28 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block27, tmp28);
  }

  TNode<Int32T> tmp29;
  TNode<BoolT> tmp30;
  if (block26.is_used()) {
    ca_.Bind(&block26);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 866);
    tmp29 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kNone);
    tmp30 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp29});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 863);
    ca_.Goto(&block27, tmp30);
  }

  TNode<BoolT> phi_bb27_6;
  if (block27.is_used()) {
    ca_.Bind(&block27, &phi_bb27_6);
    ca_.Branch(phi_bb27_6, &block28, std::vector<compiler::Node*>{}, &block29, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp31;
  if (block28.is_used()) {
    ca_.Bind(&block28);
    tmp31 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block30, tmp31);
  }

  TNode<Int32T> tmp32;
  TNode<BoolT> tmp33;
  if (block29.is_used()) {
    ca_.Bind(&block29);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 867);
    tmp32 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kNoFunc);
    tmp33 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp32});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 863);
    ca_.Goto(&block30, tmp33);
  }

  TNode<BoolT> phi_bb30_6;
  if (block30.is_used()) {
    ca_.Bind(&block30, &phi_bb30_6);
    ca_.Branch(phi_bb30_6, &block31, std::vector<compiler::Node*>{}, &block32, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp34;
  if (block31.is_used()) {
    ca_.Bind(&block31);
    tmp34 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block33, tmp34);
  }

  TNode<Int32T> tmp35;
  TNode<BoolT> tmp36;
  if (block32.is_used()) {
    ca_.Bind(&block32);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 867);
    tmp35 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kNoExtern);
    tmp36 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp35});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 863);
    ca_.Goto(&block33, tmp36);
  }

  TNode<BoolT> phi_bb33_6;
  if (block33.is_used()) {
    ca_.Bind(&block33, &phi_bb33_6);
    ca_.Branch(phi_bb33_6, &block34, std::vector<compiler::Node*>{}, &block35, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp37;
  if (block34.is_used()) {
    ca_.Bind(&block34);
    tmp37 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block36, tmp37);
  }

  TNode<Int32T> tmp38;
  TNode<BoolT> tmp39;
  if (block35.is_used()) {
    ca_.Bind(&block35);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 868);
    tmp38 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kExn);
    tmp39 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp38});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 863);
    ca_.Goto(&block36, tmp39);
  }

  TNode<BoolT> phi_bb36_6;
  if (block36.is_used()) {
    ca_.Bind(&block36, &phi_bb36_6);
    ca_.Branch(phi_bb36_6, &block37, std::vector<compiler::Node*>{}, &block38, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp40;
  if (block37.is_used()) {
    ca_.Bind(&block37);
    tmp40 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block39, tmp40);
  }

  TNode<Int32T> tmp41;
  TNode<BoolT> tmp42;
  if (block38.is_used()) {
    ca_.Bind(&block38);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 868);
    tmp41 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kNoExn);
    tmp42 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp41});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 863);
    ca_.Goto(&block39, tmp42);
  }

  TNode<BoolT> phi_bb39_6;
  if (block39.is_used()) {
    ca_.Bind(&block39, &phi_bb39_6);
    ca_.Branch(phi_bb39_6, &block5, std::vector<compiler::Node*>{}, &block6, std::vector<compiler::Node*>{});
  }

  TNode<Object> tmp43;
  if (block5.is_used()) {
    ca_.Bind(&block5);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 869);
    tmp43 = UnsafeCast_JSAny_0(state_, TNode<Context>{p_context}, TNode<Object>{p_value});
    ca_.Goto(&block1, tmp43);
  }

  TNode<Object> tmp44;
  if (block6.is_used()) {
    ca_.Bind(&block6);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 875);
    tmp44 = CodeStubAssembler(state_).CallRuntime(Runtime::kWasmGenericWasmToJSObject, p_context, p_value); 
    ca_.Goto(&block1, tmp44);
  }

  TNode<Int32T> tmp45;
  TNode<BoolT> tmp46;
  if (block3.is_used()) {
    ca_.Bind(&block3);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 877);
    tmp45 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRefNull);
    tmp46 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp1}, TNode<Int32T>{tmp45});
    ca_.Branch(tmp46, &block40, std::vector<compiler::Node*>{}, &block41, std::vector<compiler::Node*>{});
  }

  if (block41.is_used()) {
    ca_.Bind(&block41);
    {
      auto pos_stack = ca_.GetMacroSourcePositionStack();
      pos_stack.push_back({"src/builtins/js-to-wasm.tq", 877});
      CodeStubAssembler(state_).FailAssert("Torque assert 'paramKind == ValueKind::kRefNull' failed", pos_stack);
    }
  }

  TNode<Int32T> tmp47;
  TNode<BoolT> tmp48;
  if (block40.is_used()) {
    ca_.Bind(&block40);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 878);
    tmp47 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kExtern);
    tmp48 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp47});
    ca_.Branch(tmp48, &block44, std::vector<compiler::Node*>{}, &block45, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp49;
  if (block44.is_used()) {
    ca_.Bind(&block44);
    tmp49 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block46, tmp49);
  }

  TNode<Int32T> tmp50;
  TNode<BoolT> tmp51;
  if (block45.is_used()) {
    ca_.Bind(&block45);
    tmp50 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kNoExtern);
    tmp51 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp50});
    ca_.Goto(&block46, tmp51);
  }

  TNode<BoolT> phi_bb46_6;
  if (block46.is_used()) {
    ca_.Bind(&block46, &phi_bb46_6);
    ca_.Branch(phi_bb46_6, &block47, std::vector<compiler::Node*>{}, &block48, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp52;
  if (block47.is_used()) {
    ca_.Bind(&block47);
    tmp52 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block49, tmp52);
  }

  TNode<Int32T> tmp53;
  TNode<BoolT> tmp54;
  if (block48.is_used()) {
    ca_.Bind(&block48);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 879);
    tmp53 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kExn);
    tmp54 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp53});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 878);
    ca_.Goto(&block49, tmp54);
  }

  TNode<BoolT> phi_bb49_6;
  if (block49.is_used()) {
    ca_.Bind(&block49, &phi_bb49_6);
    ca_.Branch(phi_bb49_6, &block50, std::vector<compiler::Node*>{}, &block51, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp55;
  if (block50.is_used()) {
    ca_.Bind(&block50);
    tmp55 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block52, tmp55);
  }

  TNode<Int32T> tmp56;
  TNode<BoolT> tmp57;
  if (block51.is_used()) {
    ca_.Bind(&block51);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 879);
    tmp56 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kNoExn);
    tmp57 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp56});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 878);
    ca_.Goto(&block52, tmp57);
  }

  TNode<BoolT> phi_bb52_6;
  if (block52.is_used()) {
    ca_.Bind(&block52, &phi_bb52_6);
    ca_.Branch(phi_bb52_6, &block42, std::vector<compiler::Node*>{}, &block43, std::vector<compiler::Node*>{});
  }

  TNode<Object> tmp58;
  if (block42.is_used()) {
    ca_.Bind(&block42);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 880);
    tmp58 = UnsafeCast_JSAny_0(state_, TNode<Context>{p_context}, TNode<Object>{p_value});
    ca_.Goto(&block1, tmp58);
  }

  TNode<WasmNull> tmp59;
  TNode<BoolT> tmp60;
  if (block43.is_used()) {
    ca_.Bind(&block43);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 882);
    tmp59 = kWasmNull_0(state_);
    tmp60 = CodeStubAssembler(state_).TaggedEqual(TNode<Object>{p_value}, TNode<HeapObject>{tmp59});
    ca_.Branch(tmp60, &block53, std::vector<compiler::Node*>{}, &block54, std::vector<compiler::Node*>{});
  }

  TNode<Null> tmp61;
  if (block53.is_used()) {
    ca_.Bind(&block53);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 883);
    tmp61 = Null_0(state_);
    ca_.Goto(&block1, tmp61);
  }

  TNode<Int32T> tmp62;
  TNode<BoolT> tmp63;
  if (block54.is_used()) {
    ca_.Bind(&block54);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 885);
    tmp62 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kEq);
    tmp63 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp62});
    ca_.Branch(tmp63, &block57, std::vector<compiler::Node*>{}, &block58, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp64;
  if (block57.is_used()) {
    ca_.Bind(&block57);
    tmp64 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block59, tmp64);
  }

  TNode<Int32T> tmp65;
  TNode<BoolT> tmp66;
  if (block58.is_used()) {
    ca_.Bind(&block58);
    tmp65 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kStruct);
    tmp66 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp65});
    ca_.Goto(&block59, tmp66);
  }

  TNode<BoolT> phi_bb59_6;
  if (block59.is_used()) {
    ca_.Bind(&block59, &phi_bb59_6);
    ca_.Branch(phi_bb59_6, &block60, std::vector<compiler::Node*>{}, &block61, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp67;
  if (block60.is_used()) {
    ca_.Bind(&block60);
    tmp67 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block62, tmp67);
  }

  TNode<Int32T> tmp68;
  TNode<BoolT> tmp69;
  if (block61.is_used()) {
    ca_.Bind(&block61);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 886);
    tmp68 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kArray);
    tmp69 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp68});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 885);
    ca_.Goto(&block62, tmp69);
  }

  TNode<BoolT> phi_bb62_6;
  if (block62.is_used()) {
    ca_.Bind(&block62, &phi_bb62_6);
    ca_.Branch(phi_bb62_6, &block63, std::vector<compiler::Node*>{}, &block64, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp70;
  if (block63.is_used()) {
    ca_.Bind(&block63);
    tmp70 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block65, tmp70);
  }

  TNode<Int32T> tmp71;
  TNode<BoolT> tmp72;
  if (block64.is_used()) {
    ca_.Bind(&block64);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 886);
    tmp71 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kString);
    tmp72 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp71});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 885);
    ca_.Goto(&block65, tmp72);
  }

  TNode<BoolT> phi_bb65_6;
  if (block65.is_used()) {
    ca_.Bind(&block65, &phi_bb65_6);
    ca_.Branch(phi_bb65_6, &block66, std::vector<compiler::Node*>{}, &block67, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp73;
  if (block66.is_used()) {
    ca_.Bind(&block66);
    tmp73 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block68, tmp73);
  }

  TNode<Int32T> tmp74;
  TNode<BoolT> tmp75;
  if (block67.is_used()) {
    ca_.Bind(&block67);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 887);
    tmp74 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kI31);
    tmp75 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp74});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 885);
    ca_.Goto(&block68, tmp75);
  }

  TNode<BoolT> phi_bb68_6;
  if (block68.is_used()) {
    ca_.Bind(&block68, &phi_bb68_6);
    ca_.Branch(phi_bb68_6, &block69, std::vector<compiler::Node*>{}, &block70, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp76;
  if (block69.is_used()) {
    ca_.Bind(&block69);
    tmp76 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block71, tmp76);
  }

  TNode<Int32T> tmp77;
  TNode<BoolT> tmp78;
  if (block70.is_used()) {
    ca_.Bind(&block70);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 887);
    tmp77 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::HeapType::Representation::kAny);
    tmp78 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp5}, TNode<Int32T>{tmp77});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 885);
    ca_.Goto(&block71, tmp78);
  }

  TNode<BoolT> phi_bb71_6;
  if (block71.is_used()) {
    ca_.Bind(&block71, &phi_bb71_6);
    ca_.Branch(phi_bb71_6, &block55, std::vector<compiler::Node*>{}, &block56, std::vector<compiler::Node*>{});
  }

  TNode<Object> tmp79;
  if (block55.is_used()) {
    ca_.Bind(&block55);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 888);
    tmp79 = UnsafeCast_JSAny_0(state_, TNode<Context>{p_context}, TNode<Object>{p_value});
    ca_.Goto(&block1, tmp79);
  }

  TNode<Object> tmp80;
  if (block56.is_used()) {
    ca_.Bind(&block56);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 894);
    tmp80 = CodeStubAssembler(state_).CallRuntime(Runtime::kWasmGenericWasmToJSObject, p_context, p_value); 
    ca_.Goto(&block1, tmp80);
  }

  TNode<Object> phi_bb1_3;
  if (block1.is_used()) {
    ca_.Bind(&block1, &phi_bb1_3);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 858);
    ca_.Goto(&block72, phi_bb1_3);
  }

  TNode<Object> phi_bb72_3;
    ca_.Bind(&block72, &phi_bb72_3);
  return TNode<Object>{phi_bb72_3};
}

TF_BUILTIN(JSToWasmHandleReturns, CodeStubAssembler) {
  compiler::CodeAssemblerState* state_ = state();  compiler::CodeAssembler ca_(state());
  TNode<NativeContext> parameter0 = UncheckedParameter<NativeContext>(Descriptor::kJsContext);
  USE(parameter0);
  TNode<JSArray> parameter1 = UncheckedParameter<JSArray>(Descriptor::kResultArray);
  USE(parameter1);
  TNode<RawPtrT> parameter2 = UncheckedParameter<RawPtrT>(Descriptor::kWrapperBuffer);
  USE(parameter2);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block1(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block3(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block5(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block8(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block9(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<Int32T> block10(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block6(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block11(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block14(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block15(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block17(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block18(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block12(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block20(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block21(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block23(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block26(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block27(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block24(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block31(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block32(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<BoolT> block33(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block30(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block29(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block4(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block47(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block51(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block49(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block54(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block58(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block59(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block61(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block62(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block64(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block65(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block60(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block57(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block55(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block66(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block70(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block71(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block73(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block74(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block76(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block77(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block72(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block69(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block67(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block78(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block82(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block83(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block85(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block86(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block88(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block89(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block84(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block81(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block90(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block94(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block95(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block97(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block98(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block100(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block101(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block96(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block93(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block91(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block92(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block79(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block102(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block105(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block106(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block107(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block111(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block112(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block114(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block115(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block110(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block108(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block104(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block103(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block80(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block68(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block56(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block50(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT> block117(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT> block118(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block121(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block119(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block126(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block127(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, BoolT> block128(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block124(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block130(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block131(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block133(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block134(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block136(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block137(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block132(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block129(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block142(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block143(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block125(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block120(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT> block48(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT> block149(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT> block150(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT> block151(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT> block148(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT> block147(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block154(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block152(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block157(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block161(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block162(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block164(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block165(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block167(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block168(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block163(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block160(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block169(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block170(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, Int32T> block171(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block176(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block177(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block158(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block180(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block184(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block185(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block187(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block188(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block190(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block191(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block186(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block183(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block192(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block195(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block196(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, Float32T> block198(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block193(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block199(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block202(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block203(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, Float32T> block205(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block200(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, Float32T> block201(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, Float32T> block194(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block210(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block211(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block181(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block214(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block217(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block221(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block222(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block224(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block225(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block227(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block228(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block223(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block220(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block233(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block234(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block218(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block238(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block239(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block241(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block242(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block244(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block245(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block240(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block237(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block247(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block248(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block250(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block251(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block253(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block254(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, Object, IntPtrT> block249(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, Object, IntPtrT> block246(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, Object, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block259(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, Object, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block260(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block219(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block215(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block263(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block267(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block268(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block269(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block273(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block274(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block276(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block277(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block272(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block270(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT> block266(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block282(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, Object, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block283(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block264(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block288(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block289(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, BoolT> block290(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block287(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block286(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block295(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block296(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block303(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT> block304(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block265(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block216(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block182(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block159(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, IntPtrT> block153(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT> block310(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT> block311(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT, BoolT> block312(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT> block309(&ca_, compiler::CodeAssemblerLabel::kDeferred);
  compiler::CodeAssemblerParameterizedLabel<IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, IntPtrT, BoolT> block308(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<IntPtrT> tmp0;
  TNode<Object> tmp1;
  TNode<IntPtrT> tmp2;
  TNode<Int32T> tmp3;
  TNode<Int32T> tmp4;
  TNode<BoolT> tmp5;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 901);
    tmp0 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferReturnCount);
    std::tie(tmp1, tmp2) = GetRefAt_int32_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp0}).Flatten();
    tmp3 = CodeStubAssembler(state_).LoadReference<Int32T>(CodeStubAssembler::Reference{tmp1, tmp2});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 903);
    tmp4 = FromConstexpr_int32_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp5 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp3}, TNode<Int32T>{tmp4});
    ca_.Branch(tmp5, &block1, std::vector<compiler::Node*>{}, &block2, std::vector<compiler::Node*>{});
  }

  TNode<Undefined> tmp6;
  if (block1.is_used()) {
    ca_.Bind(&block1);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 904);
    tmp6 = Undefined_0(state_);
    CodeStubAssembler(state_).Return(tmp6);
  }

  TNode<Int32T> tmp7;
  TNode<BoolT> tmp8;
  if (block2.is_used()) {
    ca_.Bind(&block2);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 906);
    tmp7 = FromConstexpr_int32_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x1ull));
    tmp8 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp3}, TNode<Int32T>{tmp7});
    ca_.Branch(tmp8, &block3, std::vector<compiler::Node*>{}, &block4, std::vector<compiler::Node*>{});
  }

  TNode<IntPtrT> tmp9;
  TNode<Object> tmp10;
  TNode<IntPtrT> tmp11;
  TNode<RawPtrT> tmp12;
  TNode<IntPtrT> tmp13;
  TNode<Object> tmp14;
  TNode<IntPtrT> tmp15;
  TNode<Int32T> tmp16;
  TNode<Int32T> tmp17;
  TNode<BoolT> tmp18;
  if (block3.is_used()) {
    ca_.Bind(&block3);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 907);
    tmp9 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferSigRepresentationArray);
    std::tie(tmp10, tmp11) = GetRefAt_RawPtr_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp9}).Flatten();
    tmp12 = CodeStubAssembler(state_).LoadReference<RawPtrT>(CodeStubAssembler::Reference{tmp10, tmp11});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 909);
    tmp13 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    std::tie(tmp14, tmp15) = GetRefAt_int32_RawPtr_0(state_, TNode<RawPtrT>{tmp12}, TNode<IntPtrT>{tmp13}).Flatten();
    tmp16 = CodeStubAssembler(state_).LoadReference<Int32T>(CodeStubAssembler::Reference{tmp14, tmp15});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 910);
    tmp17 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmI32.raw_bit_field());
    tmp18 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp16}, TNode<Int32T>{tmp17});
    ca_.Branch(tmp18, &block5, std::vector<compiler::Node*>{}, &block6, std::vector<compiler::Node*>{});
  }

  if (block5.is_used()) {
    ca_.Bind(&block5);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 912);
    if ((wasm::kIsBigEndian)) {
      ca_.Goto(&block8);
    } else {
      ca_.Goto(&block9);
    }
  }

  TNode<IntPtrT> tmp19;
  TNode<Object> tmp20;
  TNode<IntPtrT> tmp21;
  TNode<Int64T> tmp22;
  TNode<Int32T> tmp23;
  if (block8.is_used()) {
    ca_.Bind(&block8);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 913);
    tmp19 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferGPReturnRegister1);
    std::tie(tmp20, tmp21) = GetRefAt_int64_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp19}).Flatten();
    tmp22 = CodeStubAssembler(state_).LoadReference<Int64T>(CodeStubAssembler::Reference{tmp20, tmp21});
    tmp23 = CodeStubAssembler(state_).TruncateInt64ToInt32(TNode<Int64T>{tmp22});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 912);
    ca_.Goto(&block10, tmp23);
  }

  TNode<IntPtrT> tmp24;
  TNode<Object> tmp25;
  TNode<IntPtrT> tmp26;
  TNode<Int32T> tmp27;
  if (block9.is_used()) {
    ca_.Bind(&block9);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 916);
    tmp24 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferGPReturnRegister1);
    std::tie(tmp25, tmp26) = GetRefAt_int32_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp24}).Flatten();
    tmp27 = CodeStubAssembler(state_).LoadReference<Int32T>(CodeStubAssembler::Reference{tmp25, tmp26});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 912);
    ca_.Goto(&block10, tmp27);
  }

  TNode<Int32T> phi_bb10_6;
  TNode<Number> tmp28;
  if (block10.is_used()) {
    ca_.Bind(&block10, &phi_bb10_6);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 918);
    tmp28 = Convert_Number_int32_0(state_, TNode<Int32T>{phi_bb10_6});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 919);
    CodeStubAssembler(state_).Return(tmp28);
  }

  TNode<Int32T> tmp29;
  TNode<BoolT> tmp30;
  if (block6.is_used()) {
    ca_.Bind(&block6);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 920);
    tmp29 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmF32.raw_bit_field());
    tmp30 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp16}, TNode<Int32T>{tmp29});
    ca_.Branch(tmp30, &block11, std::vector<compiler::Node*>{}, &block12, std::vector<compiler::Node*>{});
  }

  if (block11.is_used()) {
    ca_.Bind(&block11);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 921);
    if ((wasm::kIsFpAlwaysDouble)) {
      ca_.Goto(&block14);
    } else {
      ca_.Goto(&block15);
    }
  }

  TNode<IntPtrT> tmp31;
  TNode<Object> tmp32;
  TNode<IntPtrT> tmp33;
  TNode<Float64T> tmp34;
  TNode<Float32T> tmp35;
  TNode<Number> tmp36;
  if (block14.is_used()) {
    ca_.Bind(&block14);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 922);
    tmp31 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferFPReturnRegister1);
    std::tie(tmp32, tmp33) = GetRefAt_float64_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp31}).Flatten();
    tmp34 = CodeStubAssembler(state_).LoadReference<Float64T>(CodeStubAssembler::Reference{tmp32, tmp33});
    tmp35 = CodeStubAssembler(state_).TruncateFloat64ToFloat32(TNode<Float64T>{tmp34});
    tmp36 = Convert_Number_float32_0(state_, TNode<Float32T>{tmp35});
    CodeStubAssembler(state_).Return(tmp36);
  }

  if (block15.is_used()) {
    ca_.Bind(&block15);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 924);
    if ((wasm::kIsBigEndianOnSim)) {
      ca_.Goto(&block17);
    } else {
      ca_.Goto(&block18);
    }
  }

  TNode<IntPtrT> tmp37;
  TNode<Object> tmp38;
  TNode<IntPtrT> tmp39;
  TNode<Int64T> tmp40;
  TNode<Int64T> tmp41;
  TNode<Int64T> tmp42;
  TNode<Int32T> tmp43;
  TNode<Float32T> tmp44;
  TNode<Number> tmp45;
  if (block17.is_used()) {
    ca_.Bind(&block17);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 926);
    tmp37 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferFPReturnRegister1);
    std::tie(tmp38, tmp39) = GetRefAt_int64_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp37}).Flatten();
    tmp40 = CodeStubAssembler(state_).LoadReference<Int64T>(CodeStubAssembler::Reference{tmp38, tmp39});
    tmp41 = FromConstexpr_int64_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x20ull));
    tmp42 = CodeStubAssembler(state_).Word64Sar(TNode<Int64T>{tmp40}, TNode<Int64T>{tmp41});
    tmp43 = CodeStubAssembler(state_).TruncateInt64ToInt32(TNode<Int64T>{tmp42});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 925);
    tmp44 = CodeStubAssembler(state_).BitcastInt32ToFloat32(TNode<Int32T>{tmp43});
    tmp45 = Convert_Number_float32_0(state_, TNode<Float32T>{tmp44});
    CodeStubAssembler(state_).Return(tmp45);
  }

  TNode<IntPtrT> tmp46;
  TNode<Object> tmp47;
  TNode<IntPtrT> tmp48;
  TNode<Float32T> tmp49;
  TNode<Number> tmp50;
  if (block18.is_used()) {
    ca_.Bind(&block18);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 932);
    tmp46 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferFPReturnRegister1);
    std::tie(tmp47, tmp48) = GetRefAt_float32_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp46}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 933);
    tmp49 = CodeStubAssembler(state_).LoadReference<Float32T>(CodeStubAssembler::Reference{tmp47, tmp48});
    tmp50 = Convert_Number_float32_0(state_, TNode<Float32T>{tmp49});
    CodeStubAssembler(state_).Return(tmp50);
  }

  TNode<Int32T> tmp51;
  TNode<BoolT> tmp52;
  if (block12.is_used()) {
    ca_.Bind(&block12);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 935);
    tmp51 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmF64.raw_bit_field());
    tmp52 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp16}, TNode<Int32T>{tmp51});
    ca_.Branch(tmp52, &block20, std::vector<compiler::Node*>{}, &block21, std::vector<compiler::Node*>{});
  }

  TNode<IntPtrT> tmp53;
  TNode<Object> tmp54;
  TNode<IntPtrT> tmp55;
  TNode<Float64T> tmp56;
  TNode<Number> tmp57;
  if (block20.is_used()) {
    ca_.Bind(&block20);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 937);
    tmp53 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferFPReturnRegister1);
    std::tie(tmp54, tmp55) = GetRefAt_float64_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp53}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 938);
    tmp56 = CodeStubAssembler(state_).LoadReference<Float64T>(CodeStubAssembler::Reference{tmp54, tmp55});
    tmp57 = Convert_Number_float64_0(state_, TNode<Float64T>{tmp56});
    CodeStubAssembler(state_).Return(tmp57);
  }

  TNode<Int32T> tmp58;
  TNode<BoolT> tmp59;
  if (block21.is_used()) {
    ca_.Bind(&block21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 939);
    tmp58 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmI64.raw_bit_field());
    tmp59 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp16}, TNode<Int32T>{tmp58});
    ca_.Branch(tmp59, &block23, std::vector<compiler::Node*>{}, &block24, std::vector<compiler::Node*>{});
  }

  if (block23.is_used()) {
    ca_.Bind(&block23);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 940);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block26);
    } else {
      ca_.Goto(&block27);
    }
  }

  TNode<IntPtrT> tmp60;
  TNode<Object> tmp61;
  TNode<IntPtrT> tmp62;
  TNode<IntPtrT> tmp63;
  TNode<BigInt> tmp64;
  if (block26.is_used()) {
    ca_.Bind(&block26);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 941);
    tmp60 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferGPReturnRegister1);
    std::tie(tmp61, tmp62) = GetRefAt_intptr_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp60}).Flatten();
    tmp63 = CodeStubAssembler(state_).LoadReference<IntPtrT>(CodeStubAssembler::Reference{tmp61, tmp62});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 943);
    tmp64 = ca_.CallBuiltin<BigInt>(Builtin::kI64ToBigInt, TNode<Object>(), tmp63);
    CodeStubAssembler(state_).Return(tmp64);
  }

  TNode<IntPtrT> tmp65;
  TNode<Object> tmp66;
  TNode<IntPtrT> tmp67;
  TNode<IntPtrT> tmp68;
  TNode<IntPtrT> tmp69;
  TNode<Object> tmp70;
  TNode<IntPtrT> tmp71;
  TNode<IntPtrT> tmp72;
  TNode<BigInt> tmp73;
  if (block27.is_used()) {
    ca_.Bind(&block27);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 945);
    tmp65 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferGPReturnRegister1);
    std::tie(tmp66, tmp67) = GetRefAt_intptr_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp65}).Flatten();
    tmp68 = CodeStubAssembler(state_).LoadReference<IntPtrT>(CodeStubAssembler::Reference{tmp66, tmp67});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 947);
    tmp69 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferGPReturnRegister2);
    std::tie(tmp70, tmp71) = GetRefAt_intptr_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp69}).Flatten();
    tmp72 = CodeStubAssembler(state_).LoadReference<IntPtrT>(CodeStubAssembler::Reference{tmp70, tmp71});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 949);
    tmp73 = ca_.CallBuiltin<BigInt>(Builtin::kI32PairToBigInt, TNode<Object>(), tmp68, tmp72);
    CodeStubAssembler(state_).Return(tmp73);
  }

  TNode<Int32T> tmp74;
  TNode<Int32T> tmp75;
  TNode<Int32T> tmp76;
  TNode<BoolT> tmp77;
  if (block24.is_used()) {
    ca_.Bind(&block24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 952);
    tmp74 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmValueKindBitsMask);
    tmp75 = CodeStubAssembler(state_).Word32And(TNode<Int32T>{tmp16}, TNode<Int32T>{tmp74});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 953);
    tmp76 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRef);
    tmp77 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp75}, TNode<Int32T>{tmp76});
    ca_.Branch(tmp77, &block31, std::vector<compiler::Node*>{}, &block32, std::vector<compiler::Node*>{});
  }

  TNode<BoolT> tmp78;
  if (block31.is_used()) {
    ca_.Bind(&block31);
    tmp78 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block33, tmp78);
  }

  TNode<Int32T> tmp79;
  TNode<BoolT> tmp80;
  if (block32.is_used()) {
    ca_.Bind(&block32);
    tmp79 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRefNull);
    tmp80 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp75}, TNode<Int32T>{tmp79});
    ca_.Goto(&block33, tmp80);
  }

  TNode<BoolT> phi_bb33_8;
  if (block33.is_used()) {
    ca_.Bind(&block33, &phi_bb33_8);
    ca_.Branch(phi_bb33_8, &block29, std::vector<compiler::Node*>{}, &block30, std::vector<compiler::Node*>{});
  }

  if (block30.is_used()) {
    ca_.Bind(&block30);
    {
      auto pos_stack = ca_.GetMacroSourcePositionStack();
      pos_stack.push_back({"src/builtins/js-to-wasm.tq", 953});
      CodeStubAssembler(state_).FailAssert("Torque assert 'retKind == ValueKind::kRef || retKind == ValueKind::kRefNull' failed", pos_stack);
    }
  }

  TNode<IntPtrT> tmp81;
  TNode<RawPtrT> tmp82;
  TNode<RawPtrT> tmp83;
  TNode<IntPtrT> tmp84;
  TNode<Object> tmp85;
  TNode<IntPtrT> tmp86;
  TNode<UintPtrT> tmp87;
  TNode<Object> tmp88;
  TNode<Object> tmp89;
  if (block29.is_used()) {
    ca_.Bind(&block29);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 955);
    tmp81 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferGPReturnRegister1);
    tmp82 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp81});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 954);
    tmp83 = (TNode<RawPtrT>{tmp82});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 956);
    tmp84 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    std::tie(tmp85, tmp86) = GetRefAt_uintptr_RawPtr_uintptr_0(state_, TNode<RawPtrT>{tmp83}, TNode<IntPtrT>{tmp84}).Flatten();
    tmp87 = CodeStubAssembler(state_).LoadReference<UintPtrT>(CodeStubAssembler::Reference{tmp85, tmp86});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 957);
    tmp88 = CodeStubAssembler(state_).BitcastWordToTagged(TNode<UintPtrT>{tmp87});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 958);
    tmp89 = WasmToJSObject_0(state_, TNode<NativeContext>{parameter0}, TNode<Object>{tmp88}, TNode<Int32T>{tmp16});
    CodeStubAssembler(state_).Return(tmp89);
  }

  TNode<IntPtrT> tmp90;
  TNode<FixedArrayBase> tmp91;
  TNode<FixedArray> tmp92;
  TNode<IntPtrT> tmp93;
  TNode<Object> tmp94;
  TNode<IntPtrT> tmp95;
  TNode<RawPtrT> tmp96;
  TNode<IntPtrT> tmp97;
  TNode<RawPtrT> tmp98;
  TNode<IntPtrT> tmp99;
  TNode<RawPtrT> tmp100;
  TNode<Object> tmp101;
  TNode<IntPtrT> tmp102;
  TNode<IntPtrT> tmp103;
  TNode<IntPtrT> tmp104;
  TNode<IntPtrT> tmp105;
  TNode<IntPtrT> tmp106;
  TNode<IntPtrT> tmp107;
  TNode<IntPtrT> tmp108;
  TNode<IntPtrT> tmp109;
  TNode<BoolT> tmp110;
  TNode<IntPtrT> tmp111;
  TNode<Object> tmp112;
  TNode<IntPtrT> tmp113;
  TNode<RawPtrT> tmp114;
  TNode<RawPtrT> tmp115;
  TNode<IntPtrT> tmp116;
  TNode<Object> tmp117;
  TNode<IntPtrT> tmp118;
  TNode<IntPtrT> tmp119;
  TNode<IntPtrT> tmp120;
  TNode<Object> tmp121;
  TNode<IntPtrT> tmp122;
  TNode<BoolT> tmp123;
  if (block4.is_used()) {
    ca_.Bind(&block4);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 963);
    tmp90 = FromConstexpr_intptr_constexpr_int31_0(state_, 8);
    tmp91 = CodeStubAssembler(state_).LoadReference<FixedArrayBase>(CodeStubAssembler::Reference{parameter1, tmp90});
    ca_.SetSourcePosition("../../v8/src/builtins/cast.tq", 866);
    tmp92 = TORQUE_CAST(TNode<Object>{tmp91});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 964);
    tmp93 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferStackReturnBufferStart);
    std::tie(tmp94, tmp95) = GetRefAt_RawPtr_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp93}).Flatten();
    tmp96 = CodeStubAssembler(state_).LoadReference<RawPtrT>(CodeStubAssembler::Reference{tmp94, tmp95});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 967);
    tmp97 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferGPReturnRegister1);
    tmp98 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp97});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 968);
    tmp99 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferFPReturnRegister1);
    tmp100 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp99});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 966);
    std::tie(tmp101, tmp102, tmp103, tmp104, tmp105, tmp106, tmp107, tmp108, tmp109, tmp110) = LocationAllocatorForReturns_0(state_, TNode<RawPtrT>{tmp98}, TNode<RawPtrT>{tmp100}, TNode<RawPtrT>{tmp96}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 970);
    tmp111 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferSigRepresentationArray);
    std::tie(tmp112, tmp113) = GetRefAt_RawPtr_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp111}).Flatten();
    tmp114 = CodeStubAssembler(state_).LoadReference<RawPtrT>(CodeStubAssembler::Reference{tmp112, tmp113});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 974);
    tmp115 = (TNode<RawPtrT>{tmp114});
    tmp116 = Convert_intptr_int32_0(state_, TNode<Int32T>{tmp3});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 973);
    std::tie(tmp117, tmp118, tmp119) = NewOffHeapConstSlice_int32_0(state_, TNode<RawPtrT>{tmp115}, TNode<IntPtrT>{tmp116}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 976);
    tmp120 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferRefReturnCount);
    std::tie(tmp121, tmp122) = GetRefAt_bool_RawPtr_intptr_0(state_, TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp120}).Flatten();
    tmp123 = CodeStubAssembler(state_).LoadReference<BoolT>(CodeStubAssembler::Reference{tmp121, tmp122});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 979);
    ca_.Branch(tmp123, &block47, std::vector<compiler::Node*>{}, &block48, std::vector<compiler::Node*>{tmp102, tmp103, tmp104, tmp105, tmp106, tmp109, tmp110});
  }

  TNode<IntPtrT> tmp124;
  if (block47.is_used()) {
    ca_.Bind(&block47);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 986);
    tmp124 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.Goto(&block51, tmp102, tmp103, tmp104, tmp105, tmp106, tmp109, tmp110, tmp124);
  }

  TNode<IntPtrT> phi_bb51_7;
  TNode<IntPtrT> phi_bb51_8;
  TNode<IntPtrT> phi_bb51_9;
  TNode<IntPtrT> phi_bb51_10;
  TNode<IntPtrT> phi_bb51_11;
  TNode<IntPtrT> phi_bb51_14;
  TNode<BoolT> phi_bb51_15;
  TNode<IntPtrT> phi_bb51_21;
  TNode<IntPtrT> tmp125;
  TNode<BoolT> tmp126;
  if (block51.is_used()) {
    ca_.Bind(&block51, &phi_bb51_7, &phi_bb51_8, &phi_bb51_9, &phi_bb51_10, &phi_bb51_11, &phi_bb51_14, &phi_bb51_15, &phi_bb51_21);
    tmp125 = Convert_intptr_int32_0(state_, TNode<Int32T>{tmp3});
    tmp126 = CodeStubAssembler(state_).IntPtrLessThan(TNode<IntPtrT>{phi_bb51_21}, TNode<IntPtrT>{tmp125});
    ca_.Branch(tmp126, &block49, std::vector<compiler::Node*>{phi_bb51_7, phi_bb51_8, phi_bb51_9, phi_bb51_10, phi_bb51_11, phi_bb51_14, phi_bb51_15, phi_bb51_21}, &block50, std::vector<compiler::Node*>{phi_bb51_7, phi_bb51_8, phi_bb51_9, phi_bb51_10, phi_bb51_11, phi_bb51_14, phi_bb51_15, phi_bb51_21});
  }

  TNode<IntPtrT> phi_bb49_7;
  TNode<IntPtrT> phi_bb49_8;
  TNode<IntPtrT> phi_bb49_9;
  TNode<IntPtrT> phi_bb49_10;
  TNode<IntPtrT> phi_bb49_11;
  TNode<IntPtrT> phi_bb49_14;
  TNode<BoolT> phi_bb49_15;
  TNode<IntPtrT> phi_bb49_21;
  TNode<IntPtrT> tmp127;
  TNode<IntPtrT> tmp128;
  TNode<Object> tmp129;
  TNode<IntPtrT> tmp130;
  TNode<Int32T> tmp131;
  TNode<Int32T> tmp132;
  TNode<BoolT> tmp133;
  if (block49.is_used()) {
    ca_.Bind(&block49, &phi_bb49_7, &phi_bb49_8, &phi_bb49_9, &phi_bb49_10, &phi_bb49_11, &phi_bb49_14, &phi_bb49_15, &phi_bb49_21);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp127 = TimesSizeOf_int32_0(state_, TNode<IntPtrT>{phi_bb49_21});
    tmp128 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp118}, TNode<IntPtrT>{tmp127});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp129, tmp130) = NewReference_int32_0(state_, TNode<Object>{tmp117}, TNode<IntPtrT>{tmp128}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 987);
    tmp131 = CodeStubAssembler(state_).LoadReference<Int32T>(CodeStubAssembler::Reference{tmp129, tmp130});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 988);
    tmp132 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmI32.raw_bit_field());
    tmp133 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp131}, TNode<Int32T>{tmp132});
    ca_.Branch(tmp133, &block54, std::vector<compiler::Node*>{phi_bb49_7, phi_bb49_8, phi_bb49_9, phi_bb49_10, phi_bb49_11, phi_bb49_14, phi_bb49_15, phi_bb49_21}, &block55, std::vector<compiler::Node*>{phi_bb49_7, phi_bb49_8, phi_bb49_9, phi_bb49_10, phi_bb49_11, phi_bb49_14, phi_bb49_15, phi_bb49_21});
  }

  TNode<IntPtrT> phi_bb54_7;
  TNode<IntPtrT> phi_bb54_8;
  TNode<IntPtrT> phi_bb54_9;
  TNode<IntPtrT> phi_bb54_10;
  TNode<IntPtrT> phi_bb54_11;
  TNode<IntPtrT> phi_bb54_14;
  TNode<BoolT> phi_bb54_15;
  TNode<IntPtrT> phi_bb54_21;
  TNode<IntPtrT> tmp134;
  TNode<IntPtrT> tmp135;
  TNode<IntPtrT> tmp136;
  TNode<BoolT> tmp137;
  if (block54.is_used()) {
    ca_.Bind(&block54, &phi_bb54_7, &phi_bb54_8, &phi_bb54_9, &phi_bb54_10, &phi_bb54_11, &phi_bb54_14, &phi_bb54_15, &phi_bb54_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 308);
    tmp134 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp135 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb54_7}, TNode<IntPtrT>{tmp134});
    tmp136 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp137 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb54_7}, TNode<IntPtrT>{tmp136});
    ca_.Branch(tmp137, &block58, std::vector<compiler::Node*>{phi_bb54_8, phi_bb54_9, phi_bb54_10, phi_bb54_11, phi_bb54_14, phi_bb54_15, phi_bb54_21}, &block59, std::vector<compiler::Node*>{phi_bb54_8, phi_bb54_9, phi_bb54_10, phi_bb54_11, phi_bb54_14, phi_bb54_15, phi_bb54_21});
  }

  TNode<IntPtrT> phi_bb58_8;
  TNode<IntPtrT> phi_bb58_9;
  TNode<IntPtrT> phi_bb58_10;
  TNode<IntPtrT> phi_bb58_11;
  TNode<IntPtrT> phi_bb58_14;
  TNode<BoolT> phi_bb58_15;
  TNode<IntPtrT> phi_bb58_21;
  TNode<Object> tmp138;
  TNode<IntPtrT> tmp139;
  TNode<IntPtrT> tmp140;
  TNode<IntPtrT> tmp141;
  if (block58.is_used()) {
    ca_.Bind(&block58, &phi_bb58_8, &phi_bb58_9, &phi_bb58_10, &phi_bb58_11, &phi_bb58_14, &phi_bb58_15, &phi_bb58_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    std::tie(tmp138, tmp139) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb58_9}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 311);
    tmp140 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp141 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb58_9}, TNode<IntPtrT>{tmp140});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 312);
    ca_.Goto(&block57, phi_bb58_8, tmp141, phi_bb58_10, phi_bb58_11, phi_bb58_14, phi_bb58_15, phi_bb58_21, tmp138, tmp139);
  }

  TNode<IntPtrT> phi_bb59_8;
  TNode<IntPtrT> phi_bb59_9;
  TNode<IntPtrT> phi_bb59_10;
  TNode<IntPtrT> phi_bb59_11;
  TNode<IntPtrT> phi_bb59_14;
  TNode<BoolT> phi_bb59_15;
  TNode<IntPtrT> phi_bb59_21;
  if (block59.is_used()) {
    ca_.Bind(&block59, &phi_bb59_8, &phi_bb59_9, &phi_bb59_10, &phi_bb59_11, &phi_bb59_14, &phi_bb59_15, &phi_bb59_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block61, phi_bb59_8, phi_bb59_9, phi_bb59_10, phi_bb59_11, phi_bb59_14, phi_bb59_15, phi_bb59_21);
    } else {
      ca_.Goto(&block62, phi_bb59_8, phi_bb59_9, phi_bb59_10, phi_bb59_11, phi_bb59_14, phi_bb59_15, phi_bb59_21);
    }
  }

  TNode<IntPtrT> phi_bb61_8;
  TNode<IntPtrT> phi_bb61_9;
  TNode<IntPtrT> phi_bb61_10;
  TNode<IntPtrT> phi_bb61_11;
  TNode<IntPtrT> phi_bb61_14;
  TNode<BoolT> phi_bb61_15;
  TNode<IntPtrT> phi_bb61_21;
  TNode<Object> tmp142;
  TNode<IntPtrT> tmp143;
  TNode<IntPtrT> tmp144;
  TNode<IntPtrT> tmp145;
  if (block61.is_used()) {
    ca_.Bind(&block61, &phi_bb61_8, &phi_bb61_9, &phi_bb61_10, &phi_bb61_11, &phi_bb61_14, &phi_bb61_15, &phi_bb61_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp142, tmp143) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb61_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp144 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp145 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb61_11}, TNode<IntPtrT>{tmp144});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block60, phi_bb61_8, phi_bb61_9, phi_bb61_10, tmp145, phi_bb61_14, phi_bb61_15, phi_bb61_21, tmp142, tmp143);
  }

  TNode<IntPtrT> phi_bb62_8;
  TNode<IntPtrT> phi_bb62_9;
  TNode<IntPtrT> phi_bb62_10;
  TNode<IntPtrT> phi_bb62_11;
  TNode<IntPtrT> phi_bb62_14;
  TNode<BoolT> phi_bb62_15;
  TNode<IntPtrT> phi_bb62_21;
  TNode<IntPtrT> tmp146;
  TNode<BoolT> tmp147;
  if (block62.is_used()) {
    ca_.Bind(&block62, &phi_bb62_8, &phi_bb62_9, &phi_bb62_10, &phi_bb62_11, &phi_bb62_14, &phi_bb62_15, &phi_bb62_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp146 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp147 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb62_14}, TNode<IntPtrT>{tmp146});
    ca_.Branch(tmp147, &block64, std::vector<compiler::Node*>{phi_bb62_8, phi_bb62_9, phi_bb62_10, phi_bb62_11, phi_bb62_14, phi_bb62_15, phi_bb62_21}, &block65, std::vector<compiler::Node*>{phi_bb62_8, phi_bb62_9, phi_bb62_10, phi_bb62_11, phi_bb62_14, phi_bb62_15, phi_bb62_21});
  }

  TNode<IntPtrT> phi_bb64_8;
  TNode<IntPtrT> phi_bb64_9;
  TNode<IntPtrT> phi_bb64_10;
  TNode<IntPtrT> phi_bb64_11;
  TNode<IntPtrT> phi_bb64_14;
  TNode<BoolT> phi_bb64_15;
  TNode<IntPtrT> phi_bb64_21;
  TNode<Object> tmp148;
  TNode<IntPtrT> tmp149;
  TNode<IntPtrT> tmp150;
  TNode<BoolT> tmp151;
  if (block64.is_used()) {
    ca_.Bind(&block64, &phi_bb64_8, &phi_bb64_9, &phi_bb64_10, &phi_bb64_11, &phi_bb64_14, &phi_bb64_15, &phi_bb64_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp148, tmp149) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb64_14}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp150 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp151 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block60, phi_bb64_8, phi_bb64_9, phi_bb64_10, phi_bb64_11, tmp150, tmp151, phi_bb64_21, tmp148, tmp149);
  }

  TNode<IntPtrT> phi_bb65_8;
  TNode<IntPtrT> phi_bb65_9;
  TNode<IntPtrT> phi_bb65_10;
  TNode<IntPtrT> phi_bb65_11;
  TNode<IntPtrT> phi_bb65_14;
  TNode<BoolT> phi_bb65_15;
  TNode<IntPtrT> phi_bb65_21;
  TNode<Object> tmp152;
  TNode<IntPtrT> tmp153;
  TNode<IntPtrT> tmp154;
  TNode<IntPtrT> tmp155;
  TNode<IntPtrT> tmp156;
  TNode<IntPtrT> tmp157;
  TNode<BoolT> tmp158;
  if (block65.is_used()) {
    ca_.Bind(&block65, &phi_bb65_8, &phi_bb65_9, &phi_bb65_10, &phi_bb65_11, &phi_bb65_14, &phi_bb65_15, &phi_bb65_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp152, tmp153) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb65_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp154 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp155 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb65_11}, TNode<IntPtrT>{tmp154});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp156 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp157 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp155}, TNode<IntPtrT>{tmp156});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp158 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block60, phi_bb65_8, phi_bb65_9, phi_bb65_10, tmp157, tmp155, tmp158, phi_bb65_21, tmp152, tmp153);
  }

  TNode<IntPtrT> phi_bb60_8;
  TNode<IntPtrT> phi_bb60_9;
  TNode<IntPtrT> phi_bb60_10;
  TNode<IntPtrT> phi_bb60_11;
  TNode<IntPtrT> phi_bb60_14;
  TNode<BoolT> phi_bb60_15;
  TNode<IntPtrT> phi_bb60_21;
  TNode<Object> phi_bb60_23;
  TNode<IntPtrT> phi_bb60_24;
  if (block60.is_used()) {
    ca_.Bind(&block60, &phi_bb60_8, &phi_bb60_9, &phi_bb60_10, &phi_bb60_11, &phi_bb60_14, &phi_bb60_15, &phi_bb60_21, &phi_bb60_23, &phi_bb60_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 314);
    ca_.Goto(&block57, phi_bb60_8, phi_bb60_9, phi_bb60_10, phi_bb60_11, phi_bb60_14, phi_bb60_15, phi_bb60_21, phi_bb60_23, phi_bb60_24);
  }

  TNode<IntPtrT> phi_bb57_8;
  TNode<IntPtrT> phi_bb57_9;
  TNode<IntPtrT> phi_bb57_10;
  TNode<IntPtrT> phi_bb57_11;
  TNode<IntPtrT> phi_bb57_14;
  TNode<BoolT> phi_bb57_15;
  TNode<IntPtrT> phi_bb57_21;
  TNode<Object> phi_bb57_23;
  TNode<IntPtrT> phi_bb57_24;
  if (block57.is_used()) {
    ca_.Bind(&block57, &phi_bb57_8, &phi_bb57_9, &phi_bb57_10, &phi_bb57_11, &phi_bb57_14, &phi_bb57_15, &phi_bb57_21, &phi_bb57_23, &phi_bb57_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 988);
    ca_.Goto(&block56, tmp135, phi_bb57_8, phi_bb57_9, phi_bb57_10, phi_bb57_11, phi_bb57_14, phi_bb57_15, phi_bb57_21);
  }

  TNode<IntPtrT> phi_bb55_7;
  TNode<IntPtrT> phi_bb55_8;
  TNode<IntPtrT> phi_bb55_9;
  TNode<IntPtrT> phi_bb55_10;
  TNode<IntPtrT> phi_bb55_11;
  TNode<IntPtrT> phi_bb55_14;
  TNode<BoolT> phi_bb55_15;
  TNode<IntPtrT> phi_bb55_21;
  TNode<Int32T> tmp159;
  TNode<BoolT> tmp160;
  if (block55.is_used()) {
    ca_.Bind(&block55, &phi_bb55_7, &phi_bb55_8, &phi_bb55_9, &phi_bb55_10, &phi_bb55_11, &phi_bb55_14, &phi_bb55_15, &phi_bb55_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 990);
    tmp159 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmF32.raw_bit_field());
    tmp160 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp131}, TNode<Int32T>{tmp159});
    ca_.Branch(tmp160, &block66, std::vector<compiler::Node*>{phi_bb55_7, phi_bb55_8, phi_bb55_9, phi_bb55_10, phi_bb55_11, phi_bb55_14, phi_bb55_15, phi_bb55_21}, &block67, std::vector<compiler::Node*>{phi_bb55_7, phi_bb55_8, phi_bb55_9, phi_bb55_10, phi_bb55_11, phi_bb55_14, phi_bb55_15, phi_bb55_21});
  }

  TNode<IntPtrT> phi_bb66_7;
  TNode<IntPtrT> phi_bb66_8;
  TNode<IntPtrT> phi_bb66_9;
  TNode<IntPtrT> phi_bb66_10;
  TNode<IntPtrT> phi_bb66_11;
  TNode<IntPtrT> phi_bb66_14;
  TNode<BoolT> phi_bb66_15;
  TNode<IntPtrT> phi_bb66_21;
  TNode<IntPtrT> tmp161;
  TNode<IntPtrT> tmp162;
  TNode<IntPtrT> tmp163;
  TNode<BoolT> tmp164;
  if (block66.is_used()) {
    ca_.Bind(&block66, &phi_bb66_7, &phi_bb66_8, &phi_bb66_9, &phi_bb66_10, &phi_bb66_11, &phi_bb66_14, &phi_bb66_15, &phi_bb66_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 318);
    tmp161 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp162 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb66_8}, TNode<IntPtrT>{tmp161});
    tmp163 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp164 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb66_8}, TNode<IntPtrT>{tmp163});
    ca_.Branch(tmp164, &block70, std::vector<compiler::Node*>{phi_bb66_7, phi_bb66_9, phi_bb66_10, phi_bb66_11, phi_bb66_14, phi_bb66_15, phi_bb66_21}, &block71, std::vector<compiler::Node*>{phi_bb66_7, phi_bb66_9, phi_bb66_10, phi_bb66_11, phi_bb66_14, phi_bb66_15, phi_bb66_21});
  }

  TNode<IntPtrT> phi_bb70_7;
  TNode<IntPtrT> phi_bb70_9;
  TNode<IntPtrT> phi_bb70_10;
  TNode<IntPtrT> phi_bb70_11;
  TNode<IntPtrT> phi_bb70_14;
  TNode<BoolT> phi_bb70_15;
  TNode<IntPtrT> phi_bb70_21;
  TNode<Object> tmp165;
  TNode<IntPtrT> tmp166;
  TNode<IntPtrT> tmp167;
  TNode<IntPtrT> tmp168;
  if (block70.is_used()) {
    ca_.Bind(&block70, &phi_bb70_7, &phi_bb70_9, &phi_bb70_10, &phi_bb70_11, &phi_bb70_14, &phi_bb70_15, &phi_bb70_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 319);
    std::tie(tmp165, tmp166) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb70_10}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 321);
    tmp167 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_float64_0(state_)));
    tmp168 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb70_10}, TNode<IntPtrT>{tmp167});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 322);
    ca_.Goto(&block69, phi_bb70_7, phi_bb70_9, tmp168, phi_bb70_11, phi_bb70_14, phi_bb70_15, phi_bb70_21, tmp165, tmp166);
  }

  TNode<IntPtrT> phi_bb71_7;
  TNode<IntPtrT> phi_bb71_9;
  TNode<IntPtrT> phi_bb71_10;
  TNode<IntPtrT> phi_bb71_11;
  TNode<IntPtrT> phi_bb71_14;
  TNode<BoolT> phi_bb71_15;
  TNode<IntPtrT> phi_bb71_21;
  if (block71.is_used()) {
    ca_.Bind(&block71, &phi_bb71_7, &phi_bb71_9, &phi_bb71_10, &phi_bb71_11, &phi_bb71_14, &phi_bb71_15, &phi_bb71_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block73, phi_bb71_7, phi_bb71_9, phi_bb71_10, phi_bb71_11, phi_bb71_14, phi_bb71_15, phi_bb71_21);
    } else {
      ca_.Goto(&block74, phi_bb71_7, phi_bb71_9, phi_bb71_10, phi_bb71_11, phi_bb71_14, phi_bb71_15, phi_bb71_21);
    }
  }

  TNode<IntPtrT> phi_bb73_7;
  TNode<IntPtrT> phi_bb73_9;
  TNode<IntPtrT> phi_bb73_10;
  TNode<IntPtrT> phi_bb73_11;
  TNode<IntPtrT> phi_bb73_14;
  TNode<BoolT> phi_bb73_15;
  TNode<IntPtrT> phi_bb73_21;
  TNode<Object> tmp169;
  TNode<IntPtrT> tmp170;
  TNode<IntPtrT> tmp171;
  TNode<IntPtrT> tmp172;
  if (block73.is_used()) {
    ca_.Bind(&block73, &phi_bb73_7, &phi_bb73_9, &phi_bb73_10, &phi_bb73_11, &phi_bb73_14, &phi_bb73_15, &phi_bb73_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp169, tmp170) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb73_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp171 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp172 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb73_11}, TNode<IntPtrT>{tmp171});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block72, phi_bb73_7, phi_bb73_9, phi_bb73_10, tmp172, phi_bb73_14, phi_bb73_15, phi_bb73_21, tmp169, tmp170);
  }

  TNode<IntPtrT> phi_bb74_7;
  TNode<IntPtrT> phi_bb74_9;
  TNode<IntPtrT> phi_bb74_10;
  TNode<IntPtrT> phi_bb74_11;
  TNode<IntPtrT> phi_bb74_14;
  TNode<BoolT> phi_bb74_15;
  TNode<IntPtrT> phi_bb74_21;
  TNode<IntPtrT> tmp173;
  TNode<BoolT> tmp174;
  if (block74.is_used()) {
    ca_.Bind(&block74, &phi_bb74_7, &phi_bb74_9, &phi_bb74_10, &phi_bb74_11, &phi_bb74_14, &phi_bb74_15, &phi_bb74_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp173 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp174 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb74_14}, TNode<IntPtrT>{tmp173});
    ca_.Branch(tmp174, &block76, std::vector<compiler::Node*>{phi_bb74_7, phi_bb74_9, phi_bb74_10, phi_bb74_11, phi_bb74_14, phi_bb74_15, phi_bb74_21}, &block77, std::vector<compiler::Node*>{phi_bb74_7, phi_bb74_9, phi_bb74_10, phi_bb74_11, phi_bb74_14, phi_bb74_15, phi_bb74_21});
  }

  TNode<IntPtrT> phi_bb76_7;
  TNode<IntPtrT> phi_bb76_9;
  TNode<IntPtrT> phi_bb76_10;
  TNode<IntPtrT> phi_bb76_11;
  TNode<IntPtrT> phi_bb76_14;
  TNode<BoolT> phi_bb76_15;
  TNode<IntPtrT> phi_bb76_21;
  TNode<Object> tmp175;
  TNode<IntPtrT> tmp176;
  TNode<IntPtrT> tmp177;
  TNode<BoolT> tmp178;
  if (block76.is_used()) {
    ca_.Bind(&block76, &phi_bb76_7, &phi_bb76_9, &phi_bb76_10, &phi_bb76_11, &phi_bb76_14, &phi_bb76_15, &phi_bb76_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp175, tmp176) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb76_14}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp177 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp178 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block72, phi_bb76_7, phi_bb76_9, phi_bb76_10, phi_bb76_11, tmp177, tmp178, phi_bb76_21, tmp175, tmp176);
  }

  TNode<IntPtrT> phi_bb77_7;
  TNode<IntPtrT> phi_bb77_9;
  TNode<IntPtrT> phi_bb77_10;
  TNode<IntPtrT> phi_bb77_11;
  TNode<IntPtrT> phi_bb77_14;
  TNode<BoolT> phi_bb77_15;
  TNode<IntPtrT> phi_bb77_21;
  TNode<Object> tmp179;
  TNode<IntPtrT> tmp180;
  TNode<IntPtrT> tmp181;
  TNode<IntPtrT> tmp182;
  TNode<IntPtrT> tmp183;
  TNode<IntPtrT> tmp184;
  TNode<BoolT> tmp185;
  if (block77.is_used()) {
    ca_.Bind(&block77, &phi_bb77_7, &phi_bb77_9, &phi_bb77_10, &phi_bb77_11, &phi_bb77_14, &phi_bb77_15, &phi_bb77_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp179, tmp180) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb77_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp181 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp182 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb77_11}, TNode<IntPtrT>{tmp181});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp183 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp184 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp182}, TNode<IntPtrT>{tmp183});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp185 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block72, phi_bb77_7, phi_bb77_9, phi_bb77_10, tmp184, tmp182, tmp185, phi_bb77_21, tmp179, tmp180);
  }

  TNode<IntPtrT> phi_bb72_7;
  TNode<IntPtrT> phi_bb72_9;
  TNode<IntPtrT> phi_bb72_10;
  TNode<IntPtrT> phi_bb72_11;
  TNode<IntPtrT> phi_bb72_14;
  TNode<BoolT> phi_bb72_15;
  TNode<IntPtrT> phi_bb72_21;
  TNode<Object> phi_bb72_23;
  TNode<IntPtrT> phi_bb72_24;
  if (block72.is_used()) {
    ca_.Bind(&block72, &phi_bb72_7, &phi_bb72_9, &phi_bb72_10, &phi_bb72_11, &phi_bb72_14, &phi_bb72_15, &phi_bb72_21, &phi_bb72_23, &phi_bb72_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 324);
    ca_.Goto(&block69, phi_bb72_7, phi_bb72_9, phi_bb72_10, phi_bb72_11, phi_bb72_14, phi_bb72_15, phi_bb72_21, phi_bb72_23, phi_bb72_24);
  }

  TNode<IntPtrT> phi_bb69_7;
  TNode<IntPtrT> phi_bb69_9;
  TNode<IntPtrT> phi_bb69_10;
  TNode<IntPtrT> phi_bb69_11;
  TNode<IntPtrT> phi_bb69_14;
  TNode<BoolT> phi_bb69_15;
  TNode<IntPtrT> phi_bb69_21;
  TNode<Object> phi_bb69_23;
  TNode<IntPtrT> phi_bb69_24;
  if (block69.is_used()) {
    ca_.Bind(&block69, &phi_bb69_7, &phi_bb69_9, &phi_bb69_10, &phi_bb69_11, &phi_bb69_14, &phi_bb69_15, &phi_bb69_21, &phi_bb69_23, &phi_bb69_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 990);
    ca_.Goto(&block68, phi_bb69_7, tmp162, phi_bb69_9, phi_bb69_10, phi_bb69_11, phi_bb69_14, phi_bb69_15, phi_bb69_21);
  }

  TNode<IntPtrT> phi_bb67_7;
  TNode<IntPtrT> phi_bb67_8;
  TNode<IntPtrT> phi_bb67_9;
  TNode<IntPtrT> phi_bb67_10;
  TNode<IntPtrT> phi_bb67_11;
  TNode<IntPtrT> phi_bb67_14;
  TNode<BoolT> phi_bb67_15;
  TNode<IntPtrT> phi_bb67_21;
  TNode<Int32T> tmp186;
  TNode<BoolT> tmp187;
  if (block67.is_used()) {
    ca_.Bind(&block67, &phi_bb67_7, &phi_bb67_8, &phi_bb67_9, &phi_bb67_10, &phi_bb67_11, &phi_bb67_14, &phi_bb67_15, &phi_bb67_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 992);
    tmp186 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmI64.raw_bit_field());
    tmp187 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp131}, TNode<Int32T>{tmp186});
    ca_.Branch(tmp187, &block78, std::vector<compiler::Node*>{phi_bb67_7, phi_bb67_8, phi_bb67_9, phi_bb67_10, phi_bb67_11, phi_bb67_14, phi_bb67_15, phi_bb67_21}, &block79, std::vector<compiler::Node*>{phi_bb67_7, phi_bb67_8, phi_bb67_9, phi_bb67_10, phi_bb67_11, phi_bb67_14, phi_bb67_15, phi_bb67_21});
  }

  TNode<IntPtrT> phi_bb78_7;
  TNode<IntPtrT> phi_bb78_8;
  TNode<IntPtrT> phi_bb78_9;
  TNode<IntPtrT> phi_bb78_10;
  TNode<IntPtrT> phi_bb78_11;
  TNode<IntPtrT> phi_bb78_14;
  TNode<BoolT> phi_bb78_15;
  TNode<IntPtrT> phi_bb78_21;
  TNode<IntPtrT> tmp188;
  TNode<IntPtrT> tmp189;
  TNode<IntPtrT> tmp190;
  TNode<BoolT> tmp191;
  if (block78.is_used()) {
    ca_.Bind(&block78, &phi_bb78_7, &phi_bb78_8, &phi_bb78_9, &phi_bb78_10, &phi_bb78_11, &phi_bb78_14, &phi_bb78_15, &phi_bb78_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 308);
    tmp188 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp189 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb78_7}, TNode<IntPtrT>{tmp188});
    tmp190 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp191 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb78_7}, TNode<IntPtrT>{tmp190});
    ca_.Branch(tmp191, &block82, std::vector<compiler::Node*>{phi_bb78_8, phi_bb78_9, phi_bb78_10, phi_bb78_11, phi_bb78_14, phi_bb78_15, phi_bb78_21}, &block83, std::vector<compiler::Node*>{phi_bb78_8, phi_bb78_9, phi_bb78_10, phi_bb78_11, phi_bb78_14, phi_bb78_15, phi_bb78_21});
  }

  TNode<IntPtrT> phi_bb82_8;
  TNode<IntPtrT> phi_bb82_9;
  TNode<IntPtrT> phi_bb82_10;
  TNode<IntPtrT> phi_bb82_11;
  TNode<IntPtrT> phi_bb82_14;
  TNode<BoolT> phi_bb82_15;
  TNode<IntPtrT> phi_bb82_21;
  TNode<Object> tmp192;
  TNode<IntPtrT> tmp193;
  TNode<IntPtrT> tmp194;
  TNode<IntPtrT> tmp195;
  if (block82.is_used()) {
    ca_.Bind(&block82, &phi_bb82_8, &phi_bb82_9, &phi_bb82_10, &phi_bb82_11, &phi_bb82_14, &phi_bb82_15, &phi_bb82_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    std::tie(tmp192, tmp193) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb82_9}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 311);
    tmp194 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp195 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb82_9}, TNode<IntPtrT>{tmp194});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 312);
    ca_.Goto(&block81, phi_bb82_8, tmp195, phi_bb82_10, phi_bb82_11, phi_bb82_14, phi_bb82_15, phi_bb82_21, tmp192, tmp193);
  }

  TNode<IntPtrT> phi_bb83_8;
  TNode<IntPtrT> phi_bb83_9;
  TNode<IntPtrT> phi_bb83_10;
  TNode<IntPtrT> phi_bb83_11;
  TNode<IntPtrT> phi_bb83_14;
  TNode<BoolT> phi_bb83_15;
  TNode<IntPtrT> phi_bb83_21;
  if (block83.is_used()) {
    ca_.Bind(&block83, &phi_bb83_8, &phi_bb83_9, &phi_bb83_10, &phi_bb83_11, &phi_bb83_14, &phi_bb83_15, &phi_bb83_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block85, phi_bb83_8, phi_bb83_9, phi_bb83_10, phi_bb83_11, phi_bb83_14, phi_bb83_15, phi_bb83_21);
    } else {
      ca_.Goto(&block86, phi_bb83_8, phi_bb83_9, phi_bb83_10, phi_bb83_11, phi_bb83_14, phi_bb83_15, phi_bb83_21);
    }
  }

  TNode<IntPtrT> phi_bb85_8;
  TNode<IntPtrT> phi_bb85_9;
  TNode<IntPtrT> phi_bb85_10;
  TNode<IntPtrT> phi_bb85_11;
  TNode<IntPtrT> phi_bb85_14;
  TNode<BoolT> phi_bb85_15;
  TNode<IntPtrT> phi_bb85_21;
  TNode<Object> tmp196;
  TNode<IntPtrT> tmp197;
  TNode<IntPtrT> tmp198;
  TNode<IntPtrT> tmp199;
  if (block85.is_used()) {
    ca_.Bind(&block85, &phi_bb85_8, &phi_bb85_9, &phi_bb85_10, &phi_bb85_11, &phi_bb85_14, &phi_bb85_15, &phi_bb85_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp196, tmp197) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb85_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp198 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp199 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb85_11}, TNode<IntPtrT>{tmp198});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block84, phi_bb85_8, phi_bb85_9, phi_bb85_10, tmp199, phi_bb85_14, phi_bb85_15, phi_bb85_21, tmp196, tmp197);
  }

  TNode<IntPtrT> phi_bb86_8;
  TNode<IntPtrT> phi_bb86_9;
  TNode<IntPtrT> phi_bb86_10;
  TNode<IntPtrT> phi_bb86_11;
  TNode<IntPtrT> phi_bb86_14;
  TNode<BoolT> phi_bb86_15;
  TNode<IntPtrT> phi_bb86_21;
  TNode<IntPtrT> tmp200;
  TNode<BoolT> tmp201;
  if (block86.is_used()) {
    ca_.Bind(&block86, &phi_bb86_8, &phi_bb86_9, &phi_bb86_10, &phi_bb86_11, &phi_bb86_14, &phi_bb86_15, &phi_bb86_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp200 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp201 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb86_14}, TNode<IntPtrT>{tmp200});
    ca_.Branch(tmp201, &block88, std::vector<compiler::Node*>{phi_bb86_8, phi_bb86_9, phi_bb86_10, phi_bb86_11, phi_bb86_14, phi_bb86_15, phi_bb86_21}, &block89, std::vector<compiler::Node*>{phi_bb86_8, phi_bb86_9, phi_bb86_10, phi_bb86_11, phi_bb86_14, phi_bb86_15, phi_bb86_21});
  }

  TNode<IntPtrT> phi_bb88_8;
  TNode<IntPtrT> phi_bb88_9;
  TNode<IntPtrT> phi_bb88_10;
  TNode<IntPtrT> phi_bb88_11;
  TNode<IntPtrT> phi_bb88_14;
  TNode<BoolT> phi_bb88_15;
  TNode<IntPtrT> phi_bb88_21;
  TNode<Object> tmp202;
  TNode<IntPtrT> tmp203;
  TNode<IntPtrT> tmp204;
  TNode<BoolT> tmp205;
  if (block88.is_used()) {
    ca_.Bind(&block88, &phi_bb88_8, &phi_bb88_9, &phi_bb88_10, &phi_bb88_11, &phi_bb88_14, &phi_bb88_15, &phi_bb88_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp202, tmp203) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb88_14}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp204 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp205 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block84, phi_bb88_8, phi_bb88_9, phi_bb88_10, phi_bb88_11, tmp204, tmp205, phi_bb88_21, tmp202, tmp203);
  }

  TNode<IntPtrT> phi_bb89_8;
  TNode<IntPtrT> phi_bb89_9;
  TNode<IntPtrT> phi_bb89_10;
  TNode<IntPtrT> phi_bb89_11;
  TNode<IntPtrT> phi_bb89_14;
  TNode<BoolT> phi_bb89_15;
  TNode<IntPtrT> phi_bb89_21;
  TNode<Object> tmp206;
  TNode<IntPtrT> tmp207;
  TNode<IntPtrT> tmp208;
  TNode<IntPtrT> tmp209;
  TNode<IntPtrT> tmp210;
  TNode<IntPtrT> tmp211;
  TNode<BoolT> tmp212;
  if (block89.is_used()) {
    ca_.Bind(&block89, &phi_bb89_8, &phi_bb89_9, &phi_bb89_10, &phi_bb89_11, &phi_bb89_14, &phi_bb89_15, &phi_bb89_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp206, tmp207) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb89_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp208 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp209 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb89_11}, TNode<IntPtrT>{tmp208});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp210 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp211 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp209}, TNode<IntPtrT>{tmp210});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp212 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block84, phi_bb89_8, phi_bb89_9, phi_bb89_10, tmp211, tmp209, tmp212, phi_bb89_21, tmp206, tmp207);
  }

  TNode<IntPtrT> phi_bb84_8;
  TNode<IntPtrT> phi_bb84_9;
  TNode<IntPtrT> phi_bb84_10;
  TNode<IntPtrT> phi_bb84_11;
  TNode<IntPtrT> phi_bb84_14;
  TNode<BoolT> phi_bb84_15;
  TNode<IntPtrT> phi_bb84_21;
  TNode<Object> phi_bb84_23;
  TNode<IntPtrT> phi_bb84_24;
  if (block84.is_used()) {
    ca_.Bind(&block84, &phi_bb84_8, &phi_bb84_9, &phi_bb84_10, &phi_bb84_11, &phi_bb84_14, &phi_bb84_15, &phi_bb84_21, &phi_bb84_23, &phi_bb84_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 314);
    ca_.Goto(&block81, phi_bb84_8, phi_bb84_9, phi_bb84_10, phi_bb84_11, phi_bb84_14, phi_bb84_15, phi_bb84_21, phi_bb84_23, phi_bb84_24);
  }

  TNode<IntPtrT> phi_bb81_8;
  TNode<IntPtrT> phi_bb81_9;
  TNode<IntPtrT> phi_bb81_10;
  TNode<IntPtrT> phi_bb81_11;
  TNode<IntPtrT> phi_bb81_14;
  TNode<BoolT> phi_bb81_15;
  TNode<IntPtrT> phi_bb81_21;
  TNode<Object> phi_bb81_23;
  TNode<IntPtrT> phi_bb81_24;
  if (block81.is_used()) {
    ca_.Bind(&block81, &phi_bb81_8, &phi_bb81_9, &phi_bb81_10, &phi_bb81_11, &phi_bb81_14, &phi_bb81_15, &phi_bb81_21, &phi_bb81_23, &phi_bb81_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 994);
    if (((CodeStubAssembler(state_).ConstexprBoolNot((CodeStubAssembler(state_).Is64()))))) {
      ca_.Goto(&block90, phi_bb81_8, phi_bb81_9, phi_bb81_10, phi_bb81_11, phi_bb81_14, phi_bb81_15, phi_bb81_21);
    } else {
      ca_.Goto(&block91, phi_bb81_8, phi_bb81_9, phi_bb81_10, phi_bb81_11, phi_bb81_14, phi_bb81_15, phi_bb81_21);
    }
  }

  TNode<IntPtrT> phi_bb90_8;
  TNode<IntPtrT> phi_bb90_9;
  TNode<IntPtrT> phi_bb90_10;
  TNode<IntPtrT> phi_bb90_11;
  TNode<IntPtrT> phi_bb90_14;
  TNode<BoolT> phi_bb90_15;
  TNode<IntPtrT> phi_bb90_21;
  TNode<IntPtrT> tmp213;
  TNode<IntPtrT> tmp214;
  TNode<IntPtrT> tmp215;
  TNode<BoolT> tmp216;
  if (block90.is_used()) {
    ca_.Bind(&block90, &phi_bb90_8, &phi_bb90_9, &phi_bb90_10, &phi_bb90_11, &phi_bb90_14, &phi_bb90_15, &phi_bb90_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 308);
    tmp213 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp214 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{tmp189}, TNode<IntPtrT>{tmp213});
    tmp215 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp216 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{tmp189}, TNode<IntPtrT>{tmp215});
    ca_.Branch(tmp216, &block94, std::vector<compiler::Node*>{phi_bb90_8, phi_bb90_9, phi_bb90_10, phi_bb90_11, phi_bb90_14, phi_bb90_15, phi_bb90_21}, &block95, std::vector<compiler::Node*>{phi_bb90_8, phi_bb90_9, phi_bb90_10, phi_bb90_11, phi_bb90_14, phi_bb90_15, phi_bb90_21});
  }

  TNode<IntPtrT> phi_bb94_8;
  TNode<IntPtrT> phi_bb94_9;
  TNode<IntPtrT> phi_bb94_10;
  TNode<IntPtrT> phi_bb94_11;
  TNode<IntPtrT> phi_bb94_14;
  TNode<BoolT> phi_bb94_15;
  TNode<IntPtrT> phi_bb94_21;
  TNode<Object> tmp217;
  TNode<IntPtrT> tmp218;
  TNode<IntPtrT> tmp219;
  TNode<IntPtrT> tmp220;
  if (block94.is_used()) {
    ca_.Bind(&block94, &phi_bb94_8, &phi_bb94_9, &phi_bb94_10, &phi_bb94_11, &phi_bb94_14, &phi_bb94_15, &phi_bb94_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    std::tie(tmp217, tmp218) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb94_9}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 311);
    tmp219 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp220 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb94_9}, TNode<IntPtrT>{tmp219});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 312);
    ca_.Goto(&block93, phi_bb94_8, tmp220, phi_bb94_10, phi_bb94_11, phi_bb94_14, phi_bb94_15, phi_bb94_21, tmp217, tmp218);
  }

  TNode<IntPtrT> phi_bb95_8;
  TNode<IntPtrT> phi_bb95_9;
  TNode<IntPtrT> phi_bb95_10;
  TNode<IntPtrT> phi_bb95_11;
  TNode<IntPtrT> phi_bb95_14;
  TNode<BoolT> phi_bb95_15;
  TNode<IntPtrT> phi_bb95_21;
  if (block95.is_used()) {
    ca_.Bind(&block95, &phi_bb95_8, &phi_bb95_9, &phi_bb95_10, &phi_bb95_11, &phi_bb95_14, &phi_bb95_15, &phi_bb95_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block97, phi_bb95_8, phi_bb95_9, phi_bb95_10, phi_bb95_11, phi_bb95_14, phi_bb95_15, phi_bb95_21);
    } else {
      ca_.Goto(&block98, phi_bb95_8, phi_bb95_9, phi_bb95_10, phi_bb95_11, phi_bb95_14, phi_bb95_15, phi_bb95_21);
    }
  }

  TNode<IntPtrT> phi_bb97_8;
  TNode<IntPtrT> phi_bb97_9;
  TNode<IntPtrT> phi_bb97_10;
  TNode<IntPtrT> phi_bb97_11;
  TNode<IntPtrT> phi_bb97_14;
  TNode<BoolT> phi_bb97_15;
  TNode<IntPtrT> phi_bb97_21;
  TNode<Object> tmp221;
  TNode<IntPtrT> tmp222;
  TNode<IntPtrT> tmp223;
  TNode<IntPtrT> tmp224;
  if (block97.is_used()) {
    ca_.Bind(&block97, &phi_bb97_8, &phi_bb97_9, &phi_bb97_10, &phi_bb97_11, &phi_bb97_14, &phi_bb97_15, &phi_bb97_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp221, tmp222) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb97_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp223 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp224 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb97_11}, TNode<IntPtrT>{tmp223});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block96, phi_bb97_8, phi_bb97_9, phi_bb97_10, tmp224, phi_bb97_14, phi_bb97_15, phi_bb97_21, tmp221, tmp222);
  }

  TNode<IntPtrT> phi_bb98_8;
  TNode<IntPtrT> phi_bb98_9;
  TNode<IntPtrT> phi_bb98_10;
  TNode<IntPtrT> phi_bb98_11;
  TNode<IntPtrT> phi_bb98_14;
  TNode<BoolT> phi_bb98_15;
  TNode<IntPtrT> phi_bb98_21;
  TNode<IntPtrT> tmp225;
  TNode<BoolT> tmp226;
  if (block98.is_used()) {
    ca_.Bind(&block98, &phi_bb98_8, &phi_bb98_9, &phi_bb98_10, &phi_bb98_11, &phi_bb98_14, &phi_bb98_15, &phi_bb98_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp225 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp226 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb98_14}, TNode<IntPtrT>{tmp225});
    ca_.Branch(tmp226, &block100, std::vector<compiler::Node*>{phi_bb98_8, phi_bb98_9, phi_bb98_10, phi_bb98_11, phi_bb98_14, phi_bb98_15, phi_bb98_21}, &block101, std::vector<compiler::Node*>{phi_bb98_8, phi_bb98_9, phi_bb98_10, phi_bb98_11, phi_bb98_14, phi_bb98_15, phi_bb98_21});
  }

  TNode<IntPtrT> phi_bb100_8;
  TNode<IntPtrT> phi_bb100_9;
  TNode<IntPtrT> phi_bb100_10;
  TNode<IntPtrT> phi_bb100_11;
  TNode<IntPtrT> phi_bb100_14;
  TNode<BoolT> phi_bb100_15;
  TNode<IntPtrT> phi_bb100_21;
  TNode<Object> tmp227;
  TNode<IntPtrT> tmp228;
  TNode<IntPtrT> tmp229;
  TNode<BoolT> tmp230;
  if (block100.is_used()) {
    ca_.Bind(&block100, &phi_bb100_8, &phi_bb100_9, &phi_bb100_10, &phi_bb100_11, &phi_bb100_14, &phi_bb100_15, &phi_bb100_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp227, tmp228) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb100_14}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp229 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp230 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block96, phi_bb100_8, phi_bb100_9, phi_bb100_10, phi_bb100_11, tmp229, tmp230, phi_bb100_21, tmp227, tmp228);
  }

  TNode<IntPtrT> phi_bb101_8;
  TNode<IntPtrT> phi_bb101_9;
  TNode<IntPtrT> phi_bb101_10;
  TNode<IntPtrT> phi_bb101_11;
  TNode<IntPtrT> phi_bb101_14;
  TNode<BoolT> phi_bb101_15;
  TNode<IntPtrT> phi_bb101_21;
  TNode<Object> tmp231;
  TNode<IntPtrT> tmp232;
  TNode<IntPtrT> tmp233;
  TNode<IntPtrT> tmp234;
  TNode<IntPtrT> tmp235;
  TNode<IntPtrT> tmp236;
  TNode<BoolT> tmp237;
  if (block101.is_used()) {
    ca_.Bind(&block101, &phi_bb101_8, &phi_bb101_9, &phi_bb101_10, &phi_bb101_11, &phi_bb101_14, &phi_bb101_15, &phi_bb101_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp231, tmp232) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb101_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp233 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp234 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb101_11}, TNode<IntPtrT>{tmp233});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp235 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp236 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp234}, TNode<IntPtrT>{tmp235});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp237 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block96, phi_bb101_8, phi_bb101_9, phi_bb101_10, tmp236, tmp234, tmp237, phi_bb101_21, tmp231, tmp232);
  }

  TNode<IntPtrT> phi_bb96_8;
  TNode<IntPtrT> phi_bb96_9;
  TNode<IntPtrT> phi_bb96_10;
  TNode<IntPtrT> phi_bb96_11;
  TNode<IntPtrT> phi_bb96_14;
  TNode<BoolT> phi_bb96_15;
  TNode<IntPtrT> phi_bb96_21;
  TNode<Object> phi_bb96_23;
  TNode<IntPtrT> phi_bb96_24;
  if (block96.is_used()) {
    ca_.Bind(&block96, &phi_bb96_8, &phi_bb96_9, &phi_bb96_10, &phi_bb96_11, &phi_bb96_14, &phi_bb96_15, &phi_bb96_21, &phi_bb96_23, &phi_bb96_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 314);
    ca_.Goto(&block93, phi_bb96_8, phi_bb96_9, phi_bb96_10, phi_bb96_11, phi_bb96_14, phi_bb96_15, phi_bb96_21, phi_bb96_23, phi_bb96_24);
  }

  TNode<IntPtrT> phi_bb93_8;
  TNode<IntPtrT> phi_bb93_9;
  TNode<IntPtrT> phi_bb93_10;
  TNode<IntPtrT> phi_bb93_11;
  TNode<IntPtrT> phi_bb93_14;
  TNode<BoolT> phi_bb93_15;
  TNode<IntPtrT> phi_bb93_21;
  TNode<Object> phi_bb93_23;
  TNode<IntPtrT> phi_bb93_24;
  if (block93.is_used()) {
    ca_.Bind(&block93, &phi_bb93_8, &phi_bb93_9, &phi_bb93_10, &phi_bb93_11, &phi_bb93_14, &phi_bb93_15, &phi_bb93_21, &phi_bb93_23, &phi_bb93_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 994);
    ca_.Goto(&block92, tmp214, phi_bb93_8, phi_bb93_9, phi_bb93_10, phi_bb93_11, phi_bb93_14, phi_bb93_15, phi_bb93_21);
  }

  TNode<IntPtrT> phi_bb91_8;
  TNode<IntPtrT> phi_bb91_9;
  TNode<IntPtrT> phi_bb91_10;
  TNode<IntPtrT> phi_bb91_11;
  TNode<IntPtrT> phi_bb91_14;
  TNode<BoolT> phi_bb91_15;
  TNode<IntPtrT> phi_bb91_21;
  if (block91.is_used()) {
    ca_.Bind(&block91, &phi_bb91_8, &phi_bb91_9, &phi_bb91_10, &phi_bb91_11, &phi_bb91_14, &phi_bb91_15, &phi_bb91_21);
    ca_.Goto(&block92, tmp189, phi_bb91_8, phi_bb91_9, phi_bb91_10, phi_bb91_11, phi_bb91_14, phi_bb91_15, phi_bb91_21);
  }

  TNode<IntPtrT> phi_bb92_7;
  TNode<IntPtrT> phi_bb92_8;
  TNode<IntPtrT> phi_bb92_9;
  TNode<IntPtrT> phi_bb92_10;
  TNode<IntPtrT> phi_bb92_11;
  TNode<IntPtrT> phi_bb92_14;
  TNode<BoolT> phi_bb92_15;
  TNode<IntPtrT> phi_bb92_21;
  if (block92.is_used()) {
    ca_.Bind(&block92, &phi_bb92_7, &phi_bb92_8, &phi_bb92_9, &phi_bb92_10, &phi_bb92_11, &phi_bb92_14, &phi_bb92_15, &phi_bb92_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 992);
    ca_.Goto(&block80, phi_bb92_7, phi_bb92_8, phi_bb92_9, phi_bb92_10, phi_bb92_11, phi_bb92_14, phi_bb92_15, phi_bb92_21);
  }

  TNode<IntPtrT> phi_bb79_7;
  TNode<IntPtrT> phi_bb79_8;
  TNode<IntPtrT> phi_bb79_9;
  TNode<IntPtrT> phi_bb79_10;
  TNode<IntPtrT> phi_bb79_11;
  TNode<IntPtrT> phi_bb79_14;
  TNode<BoolT> phi_bb79_15;
  TNode<IntPtrT> phi_bb79_21;
  TNode<Int32T> tmp238;
  TNode<BoolT> tmp239;
  if (block79.is_used()) {
    ca_.Bind(&block79, &phi_bb79_7, &phi_bb79_8, &phi_bb79_9, &phi_bb79_10, &phi_bb79_11, &phi_bb79_14, &phi_bb79_15, &phi_bb79_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 997);
    tmp238 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmF64.raw_bit_field());
    tmp239 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp131}, TNode<Int32T>{tmp238});
    ca_.Branch(tmp239, &block102, std::vector<compiler::Node*>{phi_bb79_7, phi_bb79_8, phi_bb79_9, phi_bb79_10, phi_bb79_11, phi_bb79_14, phi_bb79_15, phi_bb79_21}, &block103, std::vector<compiler::Node*>{phi_bb79_7, phi_bb79_8, phi_bb79_9, phi_bb79_10, phi_bb79_11, phi_bb79_14, phi_bb79_15, phi_bb79_21});
  }

  TNode<IntPtrT> phi_bb102_7;
  TNode<IntPtrT> phi_bb102_8;
  TNode<IntPtrT> phi_bb102_9;
  TNode<IntPtrT> phi_bb102_10;
  TNode<IntPtrT> phi_bb102_11;
  TNode<IntPtrT> phi_bb102_14;
  TNode<BoolT> phi_bb102_15;
  TNode<IntPtrT> phi_bb102_21;
  TNode<IntPtrT> tmp240;
  TNode<IntPtrT> tmp241;
  TNode<IntPtrT> tmp242;
  TNode<BoolT> tmp243;
  if (block102.is_used()) {
    ca_.Bind(&block102, &phi_bb102_7, &phi_bb102_8, &phi_bb102_9, &phi_bb102_10, &phi_bb102_11, &phi_bb102_14, &phi_bb102_15, &phi_bb102_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 332);
    tmp240 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp241 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb102_8}, TNode<IntPtrT>{tmp240});
    tmp242 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp243 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb102_8}, TNode<IntPtrT>{tmp242});
    ca_.Branch(tmp243, &block105, std::vector<compiler::Node*>{phi_bb102_7, phi_bb102_9, phi_bb102_10, phi_bb102_11, phi_bb102_14, phi_bb102_15, phi_bb102_21}, &block106, std::vector<compiler::Node*>{phi_bb102_7, phi_bb102_9, phi_bb102_10, phi_bb102_11, phi_bb102_14, phi_bb102_15, phi_bb102_21});
  }

  TNode<IntPtrT> phi_bb105_7;
  TNode<IntPtrT> phi_bb105_9;
  TNode<IntPtrT> phi_bb105_10;
  TNode<IntPtrT> phi_bb105_11;
  TNode<IntPtrT> phi_bb105_14;
  TNode<BoolT> phi_bb105_15;
  TNode<IntPtrT> phi_bb105_21;
  TNode<Object> tmp244;
  TNode<IntPtrT> tmp245;
  TNode<IntPtrT> tmp246;
  TNode<IntPtrT> tmp247;
  if (block105.is_used()) {
    ca_.Bind(&block105, &phi_bb105_7, &phi_bb105_9, &phi_bb105_10, &phi_bb105_11, &phi_bb105_14, &phi_bb105_15, &phi_bb105_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 333);
    std::tie(tmp244, tmp245) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb105_10}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 335);
    tmp246 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_float64_0(state_)));
    tmp247 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb105_10}, TNode<IntPtrT>{tmp246});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 336);
    ca_.Goto(&block104, phi_bb105_7, phi_bb105_9, tmp247, phi_bb105_11, phi_bb105_14, phi_bb105_15, phi_bb105_21, tmp244, tmp245);
  }

  TNode<IntPtrT> phi_bb106_7;
  TNode<IntPtrT> phi_bb106_9;
  TNode<IntPtrT> phi_bb106_10;
  TNode<IntPtrT> phi_bb106_11;
  TNode<IntPtrT> phi_bb106_14;
  TNode<BoolT> phi_bb106_15;
  TNode<IntPtrT> phi_bb106_21;
  if (block106.is_used()) {
    ca_.Bind(&block106, &phi_bb106_7, &phi_bb106_9, &phi_bb106_10, &phi_bb106_11, &phi_bb106_14, &phi_bb106_15, &phi_bb106_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 338);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block107, phi_bb106_7, phi_bb106_9, phi_bb106_10, phi_bb106_11, phi_bb106_14, phi_bb106_15, phi_bb106_21);
    } else {
      ca_.Goto(&block108, phi_bb106_7, phi_bb106_9, phi_bb106_10, phi_bb106_11, phi_bb106_14, phi_bb106_15, phi_bb106_21);
    }
  }

  TNode<IntPtrT> phi_bb107_7;
  TNode<IntPtrT> phi_bb107_9;
  TNode<IntPtrT> phi_bb107_10;
  TNode<IntPtrT> phi_bb107_11;
  TNode<IntPtrT> phi_bb107_14;
  TNode<BoolT> phi_bb107_15;
  TNode<IntPtrT> phi_bb107_21;
  if (block107.is_used()) {
    ca_.Bind(&block107, &phi_bb107_7, &phi_bb107_9, &phi_bb107_10, &phi_bb107_11, &phi_bb107_14, &phi_bb107_15, &phi_bb107_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block111, phi_bb107_7, phi_bb107_9, phi_bb107_10, phi_bb107_11, phi_bb107_14, phi_bb107_15, phi_bb107_21);
    } else {
      ca_.Goto(&block112, phi_bb107_7, phi_bb107_9, phi_bb107_10, phi_bb107_11, phi_bb107_14, phi_bb107_15, phi_bb107_21);
    }
  }

  TNode<IntPtrT> phi_bb111_7;
  TNode<IntPtrT> phi_bb111_9;
  TNode<IntPtrT> phi_bb111_10;
  TNode<IntPtrT> phi_bb111_11;
  TNode<IntPtrT> phi_bb111_14;
  TNode<BoolT> phi_bb111_15;
  TNode<IntPtrT> phi_bb111_21;
  TNode<Object> tmp248;
  TNode<IntPtrT> tmp249;
  TNode<IntPtrT> tmp250;
  TNode<IntPtrT> tmp251;
  if (block111.is_used()) {
    ca_.Bind(&block111, &phi_bb111_7, &phi_bb111_9, &phi_bb111_10, &phi_bb111_11, &phi_bb111_14, &phi_bb111_15, &phi_bb111_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp248, tmp249) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb111_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp250 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp251 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb111_11}, TNode<IntPtrT>{tmp250});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block110, phi_bb111_7, phi_bb111_9, phi_bb111_10, tmp251, phi_bb111_14, phi_bb111_15, phi_bb111_21, tmp248, tmp249);
  }

  TNode<IntPtrT> phi_bb112_7;
  TNode<IntPtrT> phi_bb112_9;
  TNode<IntPtrT> phi_bb112_10;
  TNode<IntPtrT> phi_bb112_11;
  TNode<IntPtrT> phi_bb112_14;
  TNode<BoolT> phi_bb112_15;
  TNode<IntPtrT> phi_bb112_21;
  TNode<IntPtrT> tmp252;
  TNode<BoolT> tmp253;
  if (block112.is_used()) {
    ca_.Bind(&block112, &phi_bb112_7, &phi_bb112_9, &phi_bb112_10, &phi_bb112_11, &phi_bb112_14, &phi_bb112_15, &phi_bb112_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp252 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp253 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb112_14}, TNode<IntPtrT>{tmp252});
    ca_.Branch(tmp253, &block114, std::vector<compiler::Node*>{phi_bb112_7, phi_bb112_9, phi_bb112_10, phi_bb112_11, phi_bb112_14, phi_bb112_15, phi_bb112_21}, &block115, std::vector<compiler::Node*>{phi_bb112_7, phi_bb112_9, phi_bb112_10, phi_bb112_11, phi_bb112_14, phi_bb112_15, phi_bb112_21});
  }

  TNode<IntPtrT> phi_bb114_7;
  TNode<IntPtrT> phi_bb114_9;
  TNode<IntPtrT> phi_bb114_10;
  TNode<IntPtrT> phi_bb114_11;
  TNode<IntPtrT> phi_bb114_14;
  TNode<BoolT> phi_bb114_15;
  TNode<IntPtrT> phi_bb114_21;
  TNode<Object> tmp254;
  TNode<IntPtrT> tmp255;
  TNode<IntPtrT> tmp256;
  TNode<BoolT> tmp257;
  if (block114.is_used()) {
    ca_.Bind(&block114, &phi_bb114_7, &phi_bb114_9, &phi_bb114_10, &phi_bb114_11, &phi_bb114_14, &phi_bb114_15, &phi_bb114_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp254, tmp255) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb114_14}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp256 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp257 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block110, phi_bb114_7, phi_bb114_9, phi_bb114_10, phi_bb114_11, tmp256, tmp257, phi_bb114_21, tmp254, tmp255);
  }

  TNode<IntPtrT> phi_bb115_7;
  TNode<IntPtrT> phi_bb115_9;
  TNode<IntPtrT> phi_bb115_10;
  TNode<IntPtrT> phi_bb115_11;
  TNode<IntPtrT> phi_bb115_14;
  TNode<BoolT> phi_bb115_15;
  TNode<IntPtrT> phi_bb115_21;
  TNode<Object> tmp258;
  TNode<IntPtrT> tmp259;
  TNode<IntPtrT> tmp260;
  TNode<IntPtrT> tmp261;
  TNode<IntPtrT> tmp262;
  TNode<IntPtrT> tmp263;
  TNode<BoolT> tmp264;
  if (block115.is_used()) {
    ca_.Bind(&block115, &phi_bb115_7, &phi_bb115_9, &phi_bb115_10, &phi_bb115_11, &phi_bb115_14, &phi_bb115_15, &phi_bb115_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp258, tmp259) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb115_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp260 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp261 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb115_11}, TNode<IntPtrT>{tmp260});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp262 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp263 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp261}, TNode<IntPtrT>{tmp262});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp264 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block110, phi_bb115_7, phi_bb115_9, phi_bb115_10, tmp263, tmp261, tmp264, phi_bb115_21, tmp258, tmp259);
  }

  TNode<IntPtrT> phi_bb110_7;
  TNode<IntPtrT> phi_bb110_9;
  TNode<IntPtrT> phi_bb110_10;
  TNode<IntPtrT> phi_bb110_11;
  TNode<IntPtrT> phi_bb110_14;
  TNode<BoolT> phi_bb110_15;
  TNode<IntPtrT> phi_bb110_21;
  TNode<Object> phi_bb110_23;
  TNode<IntPtrT> phi_bb110_24;
  if (block110.is_used()) {
    ca_.Bind(&block110, &phi_bb110_7, &phi_bb110_9, &phi_bb110_10, &phi_bb110_11, &phi_bb110_14, &phi_bb110_15, &phi_bb110_21, &phi_bb110_23, &phi_bb110_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 339);
    ca_.Goto(&block104, phi_bb110_7, phi_bb110_9, phi_bb110_10, phi_bb110_11, phi_bb110_14, phi_bb110_15, phi_bb110_21, phi_bb110_23, phi_bb110_24);
  }

  TNode<IntPtrT> phi_bb108_7;
  TNode<IntPtrT> phi_bb108_9;
  TNode<IntPtrT> phi_bb108_10;
  TNode<IntPtrT> phi_bb108_11;
  TNode<IntPtrT> phi_bb108_14;
  TNode<BoolT> phi_bb108_15;
  TNode<IntPtrT> phi_bb108_21;
  TNode<Object> tmp265;
  TNode<IntPtrT> tmp266;
  TNode<IntPtrT> tmp267;
  TNode<IntPtrT> tmp268;
  TNode<BoolT> tmp269;
  if (block108.is_used()) {
    ca_.Bind(&block108, &phi_bb108_7, &phi_bb108_9, &phi_bb108_10, &phi_bb108_11, &phi_bb108_14, &phi_bb108_15, &phi_bb108_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 341);
    std::tie(tmp265, tmp266) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb108_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 343);
    tmp267 = FromConstexpr_intptr_constexpr_int31_0(state_, (CodeStubAssembler(state_).ConstexprInt31Mul((FromConstexpr_constexpr_int31_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x2ull))), (SizeOf_intptr_0(state_)))));
    tmp268 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb108_11}, TNode<IntPtrT>{tmp267});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 344);
    tmp269 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 345);
    ca_.Goto(&block104, phi_bb108_7, phi_bb108_9, phi_bb108_10, tmp268, phi_bb108_14, tmp269, phi_bb108_21, tmp265, tmp266);
  }

  TNode<IntPtrT> phi_bb104_7;
  TNode<IntPtrT> phi_bb104_9;
  TNode<IntPtrT> phi_bb104_10;
  TNode<IntPtrT> phi_bb104_11;
  TNode<IntPtrT> phi_bb104_14;
  TNode<BoolT> phi_bb104_15;
  TNode<IntPtrT> phi_bb104_21;
  TNode<Object> phi_bb104_23;
  TNode<IntPtrT> phi_bb104_24;
  if (block104.is_used()) {
    ca_.Bind(&block104, &phi_bb104_7, &phi_bb104_9, &phi_bb104_10, &phi_bb104_11, &phi_bb104_14, &phi_bb104_15, &phi_bb104_21, &phi_bb104_23, &phi_bb104_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 997);
    ca_.Goto(&block103, phi_bb104_7, tmp241, phi_bb104_9, phi_bb104_10, phi_bb104_11, phi_bb104_14, phi_bb104_15, phi_bb104_21);
  }

  TNode<IntPtrT> phi_bb103_7;
  TNode<IntPtrT> phi_bb103_8;
  TNode<IntPtrT> phi_bb103_9;
  TNode<IntPtrT> phi_bb103_10;
  TNode<IntPtrT> phi_bb103_11;
  TNode<IntPtrT> phi_bb103_14;
  TNode<BoolT> phi_bb103_15;
  TNode<IntPtrT> phi_bb103_21;
  if (block103.is_used()) {
    ca_.Bind(&block103, &phi_bb103_7, &phi_bb103_8, &phi_bb103_9, &phi_bb103_10, &phi_bb103_11, &phi_bb103_14, &phi_bb103_15, &phi_bb103_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 992);
    ca_.Goto(&block80, phi_bb103_7, phi_bb103_8, phi_bb103_9, phi_bb103_10, phi_bb103_11, phi_bb103_14, phi_bb103_15, phi_bb103_21);
  }

  TNode<IntPtrT> phi_bb80_7;
  TNode<IntPtrT> phi_bb80_8;
  TNode<IntPtrT> phi_bb80_9;
  TNode<IntPtrT> phi_bb80_10;
  TNode<IntPtrT> phi_bb80_11;
  TNode<IntPtrT> phi_bb80_14;
  TNode<BoolT> phi_bb80_15;
  TNode<IntPtrT> phi_bb80_21;
  if (block80.is_used()) {
    ca_.Bind(&block80, &phi_bb80_7, &phi_bb80_8, &phi_bb80_9, &phi_bb80_10, &phi_bb80_11, &phi_bb80_14, &phi_bb80_15, &phi_bb80_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 990);
    ca_.Goto(&block68, phi_bb80_7, phi_bb80_8, phi_bb80_9, phi_bb80_10, phi_bb80_11, phi_bb80_14, phi_bb80_15, phi_bb80_21);
  }

  TNode<IntPtrT> phi_bb68_7;
  TNode<IntPtrT> phi_bb68_8;
  TNode<IntPtrT> phi_bb68_9;
  TNode<IntPtrT> phi_bb68_10;
  TNode<IntPtrT> phi_bb68_11;
  TNode<IntPtrT> phi_bb68_14;
  TNode<BoolT> phi_bb68_15;
  TNode<IntPtrT> phi_bb68_21;
  if (block68.is_used()) {
    ca_.Bind(&block68, &phi_bb68_7, &phi_bb68_8, &phi_bb68_9, &phi_bb68_10, &phi_bb68_11, &phi_bb68_14, &phi_bb68_15, &phi_bb68_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 988);
    ca_.Goto(&block56, phi_bb68_7, phi_bb68_8, phi_bb68_9, phi_bb68_10, phi_bb68_11, phi_bb68_14, phi_bb68_15, phi_bb68_21);
  }

  TNode<IntPtrT> phi_bb56_7;
  TNode<IntPtrT> phi_bb56_8;
  TNode<IntPtrT> phi_bb56_9;
  TNode<IntPtrT> phi_bb56_10;
  TNode<IntPtrT> phi_bb56_11;
  TNode<IntPtrT> phi_bb56_14;
  TNode<BoolT> phi_bb56_15;
  TNode<IntPtrT> phi_bb56_21;
  TNode<IntPtrT> tmp270;
  TNode<IntPtrT> tmp271;
  if (block56.is_used()) {
    ca_.Bind(&block56, &phi_bb56_7, &phi_bb56_8, &phi_bb56_9, &phi_bb56_10, &phi_bb56_11, &phi_bb56_14, &phi_bb56_15, &phi_bb56_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 986);
    tmp270 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp271 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb56_21}, TNode<IntPtrT>{tmp270});
    ca_.Goto(&block51, phi_bb56_7, phi_bb56_8, phi_bb56_9, phi_bb56_10, phi_bb56_11, phi_bb56_14, phi_bb56_15, tmp271);
  }

  TNode<IntPtrT> phi_bb50_7;
  TNode<IntPtrT> phi_bb50_8;
  TNode<IntPtrT> phi_bb50_9;
  TNode<IntPtrT> phi_bb50_10;
  TNode<IntPtrT> phi_bb50_11;
  TNode<IntPtrT> phi_bb50_14;
  TNode<BoolT> phi_bb50_15;
  TNode<IntPtrT> phi_bb50_21;
  TNode<BoolT> tmp272;
  if (block50.is_used()) {
    ca_.Bind(&block50, &phi_bb50_7, &phi_bb50_8, &phi_bb50_9, &phi_bb50_10, &phi_bb50_11, &phi_bb50_14, &phi_bb50_15, &phi_bb50_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 352);
    tmp272 = CodeStubAssembler(state_).Word32BinaryNot(TNode<BoolT>{phi_bb50_15});
    ca_.Branch(tmp272, &block117, std::vector<compiler::Node*>{phi_bb50_7, phi_bb50_8, phi_bb50_9, phi_bb50_10, phi_bb50_11, phi_bb50_14, phi_bb50_15}, &block118, std::vector<compiler::Node*>{phi_bb50_7, phi_bb50_8, phi_bb50_9, phi_bb50_10, phi_bb50_11, phi_bb50_14, phi_bb50_15});
  }

  TNode<IntPtrT> phi_bb117_7;
  TNode<IntPtrT> phi_bb117_8;
  TNode<IntPtrT> phi_bb117_9;
  TNode<IntPtrT> phi_bb117_10;
  TNode<IntPtrT> phi_bb117_11;
  TNode<IntPtrT> phi_bb117_14;
  TNode<BoolT> phi_bb117_15;
  TNode<IntPtrT> tmp273;
  if (block117.is_used()) {
    ca_.Bind(&block117, &phi_bb117_7, &phi_bb117_8, &phi_bb117_9, &phi_bb117_10, &phi_bb117_11, &phi_bb117_14, &phi_bb117_15);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 353);
    tmp273 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 352);
    ca_.Goto(&block118, phi_bb117_7, phi_bb117_8, phi_bb117_9, phi_bb117_10, phi_bb117_11, tmp273, phi_bb117_15);
  }

  TNode<IntPtrT> phi_bb118_7;
  TNode<IntPtrT> phi_bb118_8;
  TNode<IntPtrT> phi_bb118_9;
  TNode<IntPtrT> phi_bb118_10;
  TNode<IntPtrT> phi_bb118_11;
  TNode<IntPtrT> phi_bb118_14;
  TNode<BoolT> phi_bb118_15;
  TNode<IntPtrT> tmp274;
  if (block118.is_used()) {
    ca_.Bind(&block118, &phi_bb118_7, &phi_bb118_8, &phi_bb118_9, &phi_bb118_10, &phi_bb118_11, &phi_bb118_14, &phi_bb118_15);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1003);
    tmp274 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.Goto(&block121, phi_bb118_7, phi_bb118_8, phi_bb118_9, phi_bb118_10, phi_bb118_11, phi_bb118_14, phi_bb118_15, tmp274);
  }

  TNode<IntPtrT> phi_bb121_7;
  TNode<IntPtrT> phi_bb121_8;
  TNode<IntPtrT> phi_bb121_9;
  TNode<IntPtrT> phi_bb121_10;
  TNode<IntPtrT> phi_bb121_11;
  TNode<IntPtrT> phi_bb121_14;
  TNode<BoolT> phi_bb121_15;
  TNode<IntPtrT> phi_bb121_21;
  TNode<IntPtrT> tmp275;
  TNode<BoolT> tmp276;
  if (block121.is_used()) {
    ca_.Bind(&block121, &phi_bb121_7, &phi_bb121_8, &phi_bb121_9, &phi_bb121_10, &phi_bb121_11, &phi_bb121_14, &phi_bb121_15, &phi_bb121_21);
    tmp275 = Convert_intptr_int32_0(state_, TNode<Int32T>{tmp3});
    tmp276 = CodeStubAssembler(state_).IntPtrLessThan(TNode<IntPtrT>{phi_bb121_21}, TNode<IntPtrT>{tmp275});
    ca_.Branch(tmp276, &block119, std::vector<compiler::Node*>{phi_bb121_7, phi_bb121_8, phi_bb121_9, phi_bb121_10, phi_bb121_11, phi_bb121_14, phi_bb121_15, phi_bb121_21}, &block120, std::vector<compiler::Node*>{phi_bb121_7, phi_bb121_8, phi_bb121_9, phi_bb121_10, phi_bb121_11, phi_bb121_14, phi_bb121_15, phi_bb121_21});
  }

  TNode<IntPtrT> phi_bb119_7;
  TNode<IntPtrT> phi_bb119_8;
  TNode<IntPtrT> phi_bb119_9;
  TNode<IntPtrT> phi_bb119_10;
  TNode<IntPtrT> phi_bb119_11;
  TNode<IntPtrT> phi_bb119_14;
  TNode<BoolT> phi_bb119_15;
  TNode<IntPtrT> phi_bb119_21;
  TNode<IntPtrT> tmp277;
  TNode<IntPtrT> tmp278;
  TNode<Object> tmp279;
  TNode<IntPtrT> tmp280;
  TNode<Int32T> tmp281;
  TNode<Int32T> tmp282;
  TNode<Int32T> tmp283;
  TNode<Int32T> tmp284;
  TNode<BoolT> tmp285;
  if (block119.is_used()) {
    ca_.Bind(&block119, &phi_bb119_7, &phi_bb119_8, &phi_bb119_9, &phi_bb119_10, &phi_bb119_11, &phi_bb119_14, &phi_bb119_15, &phi_bb119_21);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp277 = TimesSizeOf_int32_0(state_, TNode<IntPtrT>{phi_bb119_21});
    tmp278 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp118}, TNode<IntPtrT>{tmp277});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp279, tmp280) = NewReference_int32_0(state_, TNode<Object>{tmp117}, TNode<IntPtrT>{tmp278}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1004);
    tmp281 = CodeStubAssembler(state_).LoadReference<Int32T>(CodeStubAssembler::Reference{tmp279, tmp280});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1005);
    tmp282 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmValueKindBitsMask);
    tmp283 = CodeStubAssembler(state_).Word32And(TNode<Int32T>{tmp281}, TNode<Int32T>{tmp282});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1006);
    tmp284 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRef);
    tmp285 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp283}, TNode<Int32T>{tmp284});
    ca_.Branch(tmp285, &block126, std::vector<compiler::Node*>{phi_bb119_7, phi_bb119_8, phi_bb119_9, phi_bb119_10, phi_bb119_11, phi_bb119_14, phi_bb119_15, phi_bb119_21}, &block127, std::vector<compiler::Node*>{phi_bb119_7, phi_bb119_8, phi_bb119_9, phi_bb119_10, phi_bb119_11, phi_bb119_14, phi_bb119_15, phi_bb119_21});
  }

  TNode<IntPtrT> phi_bb126_7;
  TNode<IntPtrT> phi_bb126_8;
  TNode<IntPtrT> phi_bb126_9;
  TNode<IntPtrT> phi_bb126_10;
  TNode<IntPtrT> phi_bb126_11;
  TNode<IntPtrT> phi_bb126_14;
  TNode<BoolT> phi_bb126_15;
  TNode<IntPtrT> phi_bb126_21;
  TNode<BoolT> tmp286;
  if (block126.is_used()) {
    ca_.Bind(&block126, &phi_bb126_7, &phi_bb126_8, &phi_bb126_9, &phi_bb126_10, &phi_bb126_11, &phi_bb126_14, &phi_bb126_15, &phi_bb126_21);
    tmp286 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block128, phi_bb126_7, phi_bb126_8, phi_bb126_9, phi_bb126_10, phi_bb126_11, phi_bb126_14, phi_bb126_15, phi_bb126_21, tmp286);
  }

  TNode<IntPtrT> phi_bb127_7;
  TNode<IntPtrT> phi_bb127_8;
  TNode<IntPtrT> phi_bb127_9;
  TNode<IntPtrT> phi_bb127_10;
  TNode<IntPtrT> phi_bb127_11;
  TNode<IntPtrT> phi_bb127_14;
  TNode<BoolT> phi_bb127_15;
  TNode<IntPtrT> phi_bb127_21;
  TNode<Int32T> tmp287;
  TNode<BoolT> tmp288;
  if (block127.is_used()) {
    ca_.Bind(&block127, &phi_bb127_7, &phi_bb127_8, &phi_bb127_9, &phi_bb127_10, &phi_bb127_11, &phi_bb127_14, &phi_bb127_15, &phi_bb127_21);
    tmp287 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRefNull);
    tmp288 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp283}, TNode<Int32T>{tmp287});
    ca_.Goto(&block128, phi_bb127_7, phi_bb127_8, phi_bb127_9, phi_bb127_10, phi_bb127_11, phi_bb127_14, phi_bb127_15, phi_bb127_21, tmp288);
  }

  TNode<IntPtrT> phi_bb128_7;
  TNode<IntPtrT> phi_bb128_8;
  TNode<IntPtrT> phi_bb128_9;
  TNode<IntPtrT> phi_bb128_10;
  TNode<IntPtrT> phi_bb128_11;
  TNode<IntPtrT> phi_bb128_14;
  TNode<BoolT> phi_bb128_15;
  TNode<IntPtrT> phi_bb128_21;
  TNode<BoolT> phi_bb128_25;
  if (block128.is_used()) {
    ca_.Bind(&block128, &phi_bb128_7, &phi_bb128_8, &phi_bb128_9, &phi_bb128_10, &phi_bb128_11, &phi_bb128_14, &phi_bb128_15, &phi_bb128_21, &phi_bb128_25);
    ca_.Branch(phi_bb128_25, &block124, std::vector<compiler::Node*>{phi_bb128_7, phi_bb128_8, phi_bb128_9, phi_bb128_10, phi_bb128_11, phi_bb128_14, phi_bb128_15, phi_bb128_21}, &block125, std::vector<compiler::Node*>{phi_bb128_7, phi_bb128_8, phi_bb128_9, phi_bb128_10, phi_bb128_11, phi_bb128_14, phi_bb128_15, phi_bb128_21});
  }

  TNode<IntPtrT> phi_bb124_7;
  TNode<IntPtrT> phi_bb124_8;
  TNode<IntPtrT> phi_bb124_9;
  TNode<IntPtrT> phi_bb124_10;
  TNode<IntPtrT> phi_bb124_11;
  TNode<IntPtrT> phi_bb124_14;
  TNode<BoolT> phi_bb124_15;
  TNode<IntPtrT> phi_bb124_21;
  TNode<IntPtrT> tmp289;
  TNode<IntPtrT> tmp290;
  TNode<IntPtrT> tmp291;
  TNode<BoolT> tmp292;
  if (block124.is_used()) {
    ca_.Bind(&block124, &phi_bb124_7, &phi_bb124_8, &phi_bb124_9, &phi_bb124_10, &phi_bb124_11, &phi_bb124_14, &phi_bb124_15, &phi_bb124_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 308);
    tmp289 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp290 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb124_7}, TNode<IntPtrT>{tmp289});
    tmp291 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp292 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb124_7}, TNode<IntPtrT>{tmp291});
    ca_.Branch(tmp292, &block130, std::vector<compiler::Node*>{phi_bb124_8, phi_bb124_9, phi_bb124_10, phi_bb124_11, phi_bb124_14, phi_bb124_15, phi_bb124_21}, &block131, std::vector<compiler::Node*>{phi_bb124_8, phi_bb124_9, phi_bb124_10, phi_bb124_11, phi_bb124_14, phi_bb124_15, phi_bb124_21});
  }

  TNode<IntPtrT> phi_bb130_8;
  TNode<IntPtrT> phi_bb130_9;
  TNode<IntPtrT> phi_bb130_10;
  TNode<IntPtrT> phi_bb130_11;
  TNode<IntPtrT> phi_bb130_14;
  TNode<BoolT> phi_bb130_15;
  TNode<IntPtrT> phi_bb130_21;
  TNode<Object> tmp293;
  TNode<IntPtrT> tmp294;
  TNode<IntPtrT> tmp295;
  TNode<IntPtrT> tmp296;
  if (block130.is_used()) {
    ca_.Bind(&block130, &phi_bb130_8, &phi_bb130_9, &phi_bb130_10, &phi_bb130_11, &phi_bb130_14, &phi_bb130_15, &phi_bb130_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    std::tie(tmp293, tmp294) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb130_9}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 311);
    tmp295 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp296 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb130_9}, TNode<IntPtrT>{tmp295});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 312);
    ca_.Goto(&block129, phi_bb130_8, tmp296, phi_bb130_10, phi_bb130_11, phi_bb130_14, phi_bb130_15, phi_bb130_21, tmp293, tmp294);
  }

  TNode<IntPtrT> phi_bb131_8;
  TNode<IntPtrT> phi_bb131_9;
  TNode<IntPtrT> phi_bb131_10;
  TNode<IntPtrT> phi_bb131_11;
  TNode<IntPtrT> phi_bb131_14;
  TNode<BoolT> phi_bb131_15;
  TNode<IntPtrT> phi_bb131_21;
  if (block131.is_used()) {
    ca_.Bind(&block131, &phi_bb131_8, &phi_bb131_9, &phi_bb131_10, &phi_bb131_11, &phi_bb131_14, &phi_bb131_15, &phi_bb131_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block133, phi_bb131_8, phi_bb131_9, phi_bb131_10, phi_bb131_11, phi_bb131_14, phi_bb131_15, phi_bb131_21);
    } else {
      ca_.Goto(&block134, phi_bb131_8, phi_bb131_9, phi_bb131_10, phi_bb131_11, phi_bb131_14, phi_bb131_15, phi_bb131_21);
    }
  }

  TNode<IntPtrT> phi_bb133_8;
  TNode<IntPtrT> phi_bb133_9;
  TNode<IntPtrT> phi_bb133_10;
  TNode<IntPtrT> phi_bb133_11;
  TNode<IntPtrT> phi_bb133_14;
  TNode<BoolT> phi_bb133_15;
  TNode<IntPtrT> phi_bb133_21;
  TNode<Object> tmp297;
  TNode<IntPtrT> tmp298;
  TNode<IntPtrT> tmp299;
  TNode<IntPtrT> tmp300;
  if (block133.is_used()) {
    ca_.Bind(&block133, &phi_bb133_8, &phi_bb133_9, &phi_bb133_10, &phi_bb133_11, &phi_bb133_14, &phi_bb133_15, &phi_bb133_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp297, tmp298) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb133_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp299 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp300 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb133_11}, TNode<IntPtrT>{tmp299});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block132, phi_bb133_8, phi_bb133_9, phi_bb133_10, tmp300, phi_bb133_14, phi_bb133_15, phi_bb133_21, tmp297, tmp298);
  }

  TNode<IntPtrT> phi_bb134_8;
  TNode<IntPtrT> phi_bb134_9;
  TNode<IntPtrT> phi_bb134_10;
  TNode<IntPtrT> phi_bb134_11;
  TNode<IntPtrT> phi_bb134_14;
  TNode<BoolT> phi_bb134_15;
  TNode<IntPtrT> phi_bb134_21;
  TNode<IntPtrT> tmp301;
  TNode<BoolT> tmp302;
  if (block134.is_used()) {
    ca_.Bind(&block134, &phi_bb134_8, &phi_bb134_9, &phi_bb134_10, &phi_bb134_11, &phi_bb134_14, &phi_bb134_15, &phi_bb134_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp301 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp302 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb134_14}, TNode<IntPtrT>{tmp301});
    ca_.Branch(tmp302, &block136, std::vector<compiler::Node*>{phi_bb134_8, phi_bb134_9, phi_bb134_10, phi_bb134_11, phi_bb134_14, phi_bb134_15, phi_bb134_21}, &block137, std::vector<compiler::Node*>{phi_bb134_8, phi_bb134_9, phi_bb134_10, phi_bb134_11, phi_bb134_14, phi_bb134_15, phi_bb134_21});
  }

  TNode<IntPtrT> phi_bb136_8;
  TNode<IntPtrT> phi_bb136_9;
  TNode<IntPtrT> phi_bb136_10;
  TNode<IntPtrT> phi_bb136_11;
  TNode<IntPtrT> phi_bb136_14;
  TNode<BoolT> phi_bb136_15;
  TNode<IntPtrT> phi_bb136_21;
  TNode<Object> tmp303;
  TNode<IntPtrT> tmp304;
  TNode<IntPtrT> tmp305;
  TNode<BoolT> tmp306;
  if (block136.is_used()) {
    ca_.Bind(&block136, &phi_bb136_8, &phi_bb136_9, &phi_bb136_10, &phi_bb136_11, &phi_bb136_14, &phi_bb136_15, &phi_bb136_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp303, tmp304) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb136_14}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp305 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp306 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block132, phi_bb136_8, phi_bb136_9, phi_bb136_10, phi_bb136_11, tmp305, tmp306, phi_bb136_21, tmp303, tmp304);
  }

  TNode<IntPtrT> phi_bb137_8;
  TNode<IntPtrT> phi_bb137_9;
  TNode<IntPtrT> phi_bb137_10;
  TNode<IntPtrT> phi_bb137_11;
  TNode<IntPtrT> phi_bb137_14;
  TNode<BoolT> phi_bb137_15;
  TNode<IntPtrT> phi_bb137_21;
  TNode<Object> tmp307;
  TNode<IntPtrT> tmp308;
  TNode<IntPtrT> tmp309;
  TNode<IntPtrT> tmp310;
  TNode<IntPtrT> tmp311;
  TNode<IntPtrT> tmp312;
  TNode<BoolT> tmp313;
  if (block137.is_used()) {
    ca_.Bind(&block137, &phi_bb137_8, &phi_bb137_9, &phi_bb137_10, &phi_bb137_11, &phi_bb137_14, &phi_bb137_15, &phi_bb137_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp307, tmp308) = NewReference_intptr_0(state_, TNode<Object>{tmp101}, TNode<IntPtrT>{phi_bb137_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp309 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp310 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb137_11}, TNode<IntPtrT>{tmp309});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp311 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp312 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp310}, TNode<IntPtrT>{tmp311});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp313 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block132, phi_bb137_8, phi_bb137_9, phi_bb137_10, tmp312, tmp310, tmp313, phi_bb137_21, tmp307, tmp308);
  }

  TNode<IntPtrT> phi_bb132_8;
  TNode<IntPtrT> phi_bb132_9;
  TNode<IntPtrT> phi_bb132_10;
  TNode<IntPtrT> phi_bb132_11;
  TNode<IntPtrT> phi_bb132_14;
  TNode<BoolT> phi_bb132_15;
  TNode<IntPtrT> phi_bb132_21;
  TNode<Object> phi_bb132_24;
  TNode<IntPtrT> phi_bb132_25;
  if (block132.is_used()) {
    ca_.Bind(&block132, &phi_bb132_8, &phi_bb132_9, &phi_bb132_10, &phi_bb132_11, &phi_bb132_14, &phi_bb132_15, &phi_bb132_21, &phi_bb132_24, &phi_bb132_25);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 314);
    ca_.Goto(&block129, phi_bb132_8, phi_bb132_9, phi_bb132_10, phi_bb132_11, phi_bb132_14, phi_bb132_15, phi_bb132_21, phi_bb132_24, phi_bb132_25);
  }

  TNode<IntPtrT> phi_bb129_8;
  TNode<IntPtrT> phi_bb129_9;
  TNode<IntPtrT> phi_bb129_10;
  TNode<IntPtrT> phi_bb129_11;
  TNode<IntPtrT> phi_bb129_14;
  TNode<BoolT> phi_bb129_15;
  TNode<IntPtrT> phi_bb129_21;
  TNode<Object> phi_bb129_24;
  TNode<IntPtrT> phi_bb129_25;
  TNode<IntPtrT> tmp314;
  TNode<Object> tmp315;
  TNode<Object> tmp316;
  TNode<IntPtrT> tmp317;
  TNode<IntPtrT> tmp318;
  TNode<UintPtrT> tmp319;
  TNode<UintPtrT> tmp320;
  TNode<BoolT> tmp321;
  if (block129.is_used()) {
    ca_.Bind(&block129, &phi_bb129_8, &phi_bb129_9, &phi_bb129_10, &phi_bb129_11, &phi_bb129_14, &phi_bb129_15, &phi_bb129_21, &phi_bb129_24, &phi_bb129_25);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1008);
    tmp314 = CodeStubAssembler(state_).LoadReference<IntPtrT>(CodeStubAssembler::Reference{phi_bb129_24, phi_bb129_25});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1009);
    tmp315 = CodeStubAssembler(state_).BitcastWordToTagged(TNode<IntPtrT>{tmp314});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1012);
    std::tie(tmp316, tmp317, tmp318) = FieldSliceFixedArrayObjects_0(state_, TNode<FixedArray>{tmp92}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 101);
    tmp319 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{phi_bb129_21});
    tmp320 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{tmp318});
    tmp321 = CodeStubAssembler(state_).UintPtrLessThan(TNode<UintPtrT>{tmp319}, TNode<UintPtrT>{tmp320});
    ca_.Branch(tmp321, &block142, std::vector<compiler::Node*>{phi_bb129_8, phi_bb129_9, phi_bb129_10, phi_bb129_11, phi_bb129_14, phi_bb129_15, phi_bb129_21, phi_bb129_24, phi_bb129_25, phi_bb129_21, phi_bb129_21, phi_bb129_21, phi_bb129_21}, &block143, std::vector<compiler::Node*>{phi_bb129_8, phi_bb129_9, phi_bb129_10, phi_bb129_11, phi_bb129_14, phi_bb129_15, phi_bb129_21, phi_bb129_24, phi_bb129_25, phi_bb129_21, phi_bb129_21, phi_bb129_21, phi_bb129_21});
  }

  TNode<IntPtrT> phi_bb142_8;
  TNode<IntPtrT> phi_bb142_9;
  TNode<IntPtrT> phi_bb142_10;
  TNode<IntPtrT> phi_bb142_11;
  TNode<IntPtrT> phi_bb142_14;
  TNode<BoolT> phi_bb142_15;
  TNode<IntPtrT> phi_bb142_21;
  TNode<Object> phi_bb142_24;
  TNode<IntPtrT> phi_bb142_25;
  TNode<IntPtrT> phi_bb142_32;
  TNode<IntPtrT> phi_bb142_33;
  TNode<IntPtrT> phi_bb142_37;
  TNode<IntPtrT> phi_bb142_38;
  TNode<IntPtrT> tmp322;
  TNode<IntPtrT> tmp323;
  TNode<Object> tmp324;
  TNode<IntPtrT> tmp325;
  if (block142.is_used()) {
    ca_.Bind(&block142, &phi_bb142_8, &phi_bb142_9, &phi_bb142_10, &phi_bb142_11, &phi_bb142_14, &phi_bb142_15, &phi_bb142_21, &phi_bb142_24, &phi_bb142_25, &phi_bb142_32, &phi_bb142_33, &phi_bb142_37, &phi_bb142_38);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp322 = TimesSizeOf_Object_0(state_, TNode<IntPtrT>{phi_bb142_38});
    tmp323 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp317}, TNode<IntPtrT>{tmp322});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp324, tmp325) = NewReference_Object_0(state_, TNode<Object>{tmp316}, TNode<IntPtrT>{tmp323}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1012);
    CodeStubAssembler(state_).StoreReference<Object>(CodeStubAssembler::Reference{tmp324, tmp325}, tmp315);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1006);
    ca_.Goto(&block125, tmp290, phi_bb142_8, phi_bb142_9, phi_bb142_10, phi_bb142_11, phi_bb142_14, phi_bb142_15, phi_bb142_21);
  }

  TNode<IntPtrT> phi_bb143_8;
  TNode<IntPtrT> phi_bb143_9;
  TNode<IntPtrT> phi_bb143_10;
  TNode<IntPtrT> phi_bb143_11;
  TNode<IntPtrT> phi_bb143_14;
  TNode<BoolT> phi_bb143_15;
  TNode<IntPtrT> phi_bb143_21;
  TNode<Object> phi_bb143_24;
  TNode<IntPtrT> phi_bb143_25;
  TNode<IntPtrT> phi_bb143_32;
  TNode<IntPtrT> phi_bb143_33;
  TNode<IntPtrT> phi_bb143_37;
  TNode<IntPtrT> phi_bb143_38;
  if (block143.is_used()) {
    ca_.Bind(&block143, &phi_bb143_8, &phi_bb143_9, &phi_bb143_10, &phi_bb143_11, &phi_bb143_14, &phi_bb143_15, &phi_bb143_21, &phi_bb143_24, &phi_bb143_25, &phi_bb143_32, &phi_bb143_33, &phi_bb143_37, &phi_bb143_38);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 113);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<IntPtrT> phi_bb125_7;
  TNode<IntPtrT> phi_bb125_8;
  TNode<IntPtrT> phi_bb125_9;
  TNode<IntPtrT> phi_bb125_10;
  TNode<IntPtrT> phi_bb125_11;
  TNode<IntPtrT> phi_bb125_14;
  TNode<BoolT> phi_bb125_15;
  TNode<IntPtrT> phi_bb125_21;
  TNode<IntPtrT> tmp326;
  TNode<IntPtrT> tmp327;
  if (block125.is_used()) {
    ca_.Bind(&block125, &phi_bb125_7, &phi_bb125_8, &phi_bb125_9, &phi_bb125_10, &phi_bb125_11, &phi_bb125_14, &phi_bb125_15, &phi_bb125_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1003);
    tmp326 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp327 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb125_21}, TNode<IntPtrT>{tmp326});
    ca_.Goto(&block121, phi_bb125_7, phi_bb125_8, phi_bb125_9, phi_bb125_10, phi_bb125_11, phi_bb125_14, phi_bb125_15, tmp327);
  }

  TNode<IntPtrT> phi_bb120_7;
  TNode<IntPtrT> phi_bb120_8;
  TNode<IntPtrT> phi_bb120_9;
  TNode<IntPtrT> phi_bb120_10;
  TNode<IntPtrT> phi_bb120_11;
  TNode<IntPtrT> phi_bb120_14;
  TNode<BoolT> phi_bb120_15;
  TNode<IntPtrT> phi_bb120_21;
  if (block120.is_used()) {
    ca_.Bind(&block120, &phi_bb120_7, &phi_bb120_8, &phi_bb120_9, &phi_bb120_10, &phi_bb120_11, &phi_bb120_14, &phi_bb120_15, &phi_bb120_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 979);
    ca_.Goto(&block48, phi_bb120_7, phi_bb120_8, phi_bb120_9, phi_bb120_10, phi_bb120_11, phi_bb120_14, phi_bb120_15);
  }

  TNode<IntPtrT> phi_bb48_7;
  TNode<IntPtrT> phi_bb48_8;
  TNode<IntPtrT> phi_bb48_9;
  TNode<IntPtrT> phi_bb48_10;
  TNode<IntPtrT> phi_bb48_11;
  TNode<IntPtrT> phi_bb48_14;
  TNode<BoolT> phi_bb48_15;
  TNode<IntPtrT> tmp328;
  TNode<BoolT> tmp329;
  if (block48.is_used()) {
    ca_.Bind(&block48, &phi_bb48_7, &phi_bb48_8, &phi_bb48_9, &phi_bb48_10, &phi_bb48_11, &phi_bb48_14, &phi_bb48_15);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 380);
    tmp328 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp329 = CodeStubAssembler(state_).WordEqual(TNode<IntPtrT>{tmp108}, TNode<IntPtrT>{tmp328});
    ca_.Branch(tmp329, &block149, std::vector<compiler::Node*>{phi_bb48_7, phi_bb48_8, phi_bb48_9, phi_bb48_10, phi_bb48_11, phi_bb48_14, phi_bb48_15}, &block150, std::vector<compiler::Node*>{phi_bb48_7, phi_bb48_8, phi_bb48_9, phi_bb48_10, phi_bb48_11, phi_bb48_14, phi_bb48_15});
  }

  TNode<IntPtrT> phi_bb149_7;
  TNode<IntPtrT> phi_bb149_8;
  TNode<IntPtrT> phi_bb149_9;
  TNode<IntPtrT> phi_bb149_10;
  TNode<IntPtrT> phi_bb149_11;
  TNode<IntPtrT> phi_bb149_14;
  TNode<BoolT> phi_bb149_15;
  TNode<BoolT> tmp330;
  if (block149.is_used()) {
    ca_.Bind(&block149, &phi_bb149_7, &phi_bb149_8, &phi_bb149_9, &phi_bb149_10, &phi_bb149_11, &phi_bb149_14, &phi_bb149_15);
    tmp330 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block151, phi_bb149_7, phi_bb149_8, phi_bb149_9, phi_bb149_10, phi_bb149_11, phi_bb149_14, phi_bb149_15, tmp330);
  }

  TNode<IntPtrT> phi_bb150_7;
  TNode<IntPtrT> phi_bb150_8;
  TNode<IntPtrT> phi_bb150_9;
  TNode<IntPtrT> phi_bb150_10;
  TNode<IntPtrT> phi_bb150_11;
  TNode<IntPtrT> phi_bb150_14;
  TNode<BoolT> phi_bb150_15;
  TNode<BoolT> tmp331;
  if (block150.is_used()) {
    ca_.Bind(&block150, &phi_bb150_7, &phi_bb150_8, &phi_bb150_9, &phi_bb150_10, &phi_bb150_11, &phi_bb150_14, &phi_bb150_15);
    tmp331 = CodeStubAssembler(state_).IntPtrLessThanOrEqual(TNode<IntPtrT>{phi_bb150_11}, TNode<IntPtrT>{tmp108});
    ca_.Goto(&block151, phi_bb150_7, phi_bb150_8, phi_bb150_9, phi_bb150_10, phi_bb150_11, phi_bb150_14, phi_bb150_15, tmp331);
  }

  TNode<IntPtrT> phi_bb151_7;
  TNode<IntPtrT> phi_bb151_8;
  TNode<IntPtrT> phi_bb151_9;
  TNode<IntPtrT> phi_bb151_10;
  TNode<IntPtrT> phi_bb151_11;
  TNode<IntPtrT> phi_bb151_14;
  TNode<BoolT> phi_bb151_15;
  TNode<BoolT> phi_bb151_22;
  if (block151.is_used()) {
    ca_.Bind(&block151, &phi_bb151_7, &phi_bb151_8, &phi_bb151_9, &phi_bb151_10, &phi_bb151_11, &phi_bb151_14, &phi_bb151_15, &phi_bb151_22);
    ca_.Branch(phi_bb151_22, &block147, std::vector<compiler::Node*>{phi_bb151_7, phi_bb151_8, phi_bb151_9, phi_bb151_10, phi_bb151_11, phi_bb151_14, phi_bb151_15}, &block148, std::vector<compiler::Node*>{phi_bb151_7, phi_bb151_8, phi_bb151_9, phi_bb151_10, phi_bb151_11, phi_bb151_14, phi_bb151_15});
  }

  TNode<IntPtrT> phi_bb148_7;
  TNode<IntPtrT> phi_bb148_8;
  TNode<IntPtrT> phi_bb148_9;
  TNode<IntPtrT> phi_bb148_10;
  TNode<IntPtrT> phi_bb148_11;
  TNode<IntPtrT> phi_bb148_14;
  TNode<BoolT> phi_bb148_15;
  if (block148.is_used()) {
    ca_.Bind(&block148, &phi_bb148_7, &phi_bb148_8, &phi_bb148_9, &phi_bb148_10, &phi_bb148_11, &phi_bb148_14, &phi_bb148_15);
    {
      auto pos_stack = ca_.GetMacroSourcePositionStack();
      pos_stack.push_back({"src/builtins/js-to-wasm.tq", 380});
      CodeStubAssembler(state_).FailAssert("Torque assert 'this.paramBufferEnd == 0 || this.nextStack <= this.paramBufferEnd' failed", pos_stack);
    }
  }

  TNode<IntPtrT> phi_bb147_7;
  TNode<IntPtrT> phi_bb147_8;
  TNode<IntPtrT> phi_bb147_9;
  TNode<IntPtrT> phi_bb147_10;
  TNode<IntPtrT> phi_bb147_11;
  TNode<IntPtrT> phi_bb147_14;
  TNode<BoolT> phi_bb147_15;
  TNode<IntPtrT> tmp332;
  TNode<RawPtrT> tmp333;
  TNode<IntPtrT> tmp334;
  TNode<RawPtrT> tmp335;
  TNode<Object> tmp336;
  TNode<IntPtrT> tmp337;
  TNode<IntPtrT> tmp338;
  TNode<IntPtrT> tmp339;
  TNode<IntPtrT> tmp340;
  TNode<IntPtrT> tmp341;
  TNode<IntPtrT> tmp342;
  TNode<IntPtrT> tmp343;
  TNode<IntPtrT> tmp344;
  TNode<BoolT> tmp345;
  TNode<IntPtrT> tmp346;
  if (block147.is_used()) {
    ca_.Bind(&block147, &phi_bb147_7, &phi_bb147_8, &phi_bb147_9, &phi_bb147_10, &phi_bb147_11, &phi_bb147_14, &phi_bb147_15);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1019);
    tmp332 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferGPReturnRegister1);
    tmp333 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp332});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1020);
    tmp334 = FromConstexpr_intptr_constexpr_intptr_0(state_, JSToWasmWrapperFrameConstants::kWrapperBufferFPReturnRegister1);
    tmp335 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{parameter2}, TNode<IntPtrT>{tmp334});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1018);
    std::tie(tmp336, tmp337, tmp338, tmp339, tmp340, tmp341, tmp342, tmp343, tmp344, tmp345) = LocationAllocatorForReturns_0(state_, TNode<RawPtrT>{tmp333}, TNode<RawPtrT>{tmp335}, TNode<RawPtrT>{tmp96}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1022);
    tmp346 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.Goto(&block154, tmp337, tmp338, tmp339, tmp340, tmp341, tmp344, tmp345, tmp346);
  }

  TNode<IntPtrT> phi_bb154_7;
  TNode<IntPtrT> phi_bb154_8;
  TNode<IntPtrT> phi_bb154_9;
  TNode<IntPtrT> phi_bb154_10;
  TNode<IntPtrT> phi_bb154_11;
  TNode<IntPtrT> phi_bb154_14;
  TNode<BoolT> phi_bb154_15;
  TNode<IntPtrT> phi_bb154_21;
  TNode<IntPtrT> tmp347;
  TNode<BoolT> tmp348;
  if (block154.is_used()) {
    ca_.Bind(&block154, &phi_bb154_7, &phi_bb154_8, &phi_bb154_9, &phi_bb154_10, &phi_bb154_11, &phi_bb154_14, &phi_bb154_15, &phi_bb154_21);
    tmp347 = Convert_intptr_int32_0(state_, TNode<Int32T>{tmp3});
    tmp348 = CodeStubAssembler(state_).IntPtrLessThan(TNode<IntPtrT>{phi_bb154_21}, TNode<IntPtrT>{tmp347});
    ca_.Branch(tmp348, &block152, std::vector<compiler::Node*>{phi_bb154_7, phi_bb154_8, phi_bb154_9, phi_bb154_10, phi_bb154_11, phi_bb154_14, phi_bb154_15, phi_bb154_21}, &block153, std::vector<compiler::Node*>{phi_bb154_7, phi_bb154_8, phi_bb154_9, phi_bb154_10, phi_bb154_11, phi_bb154_14, phi_bb154_15, phi_bb154_21});
  }

  TNode<IntPtrT> phi_bb152_7;
  TNode<IntPtrT> phi_bb152_8;
  TNode<IntPtrT> phi_bb152_9;
  TNode<IntPtrT> phi_bb152_10;
  TNode<IntPtrT> phi_bb152_11;
  TNode<IntPtrT> phi_bb152_14;
  TNode<BoolT> phi_bb152_15;
  TNode<IntPtrT> phi_bb152_21;
  TNode<IntPtrT> tmp349;
  TNode<IntPtrT> tmp350;
  TNode<Object> tmp351;
  TNode<IntPtrT> tmp352;
  TNode<Int32T> tmp353;
  TNode<Int32T> tmp354;
  TNode<BoolT> tmp355;
  if (block152.is_used()) {
    ca_.Bind(&block152, &phi_bb152_7, &phi_bb152_8, &phi_bb152_9, &phi_bb152_10, &phi_bb152_11, &phi_bb152_14, &phi_bb152_15, &phi_bb152_21);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp349 = TimesSizeOf_int32_0(state_, TNode<IntPtrT>{phi_bb152_21});
    tmp350 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp118}, TNode<IntPtrT>{tmp349});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp351, tmp352) = NewReference_int32_0(state_, TNode<Object>{tmp117}, TNode<IntPtrT>{tmp350}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1023);
    tmp353 = CodeStubAssembler(state_).LoadReference<Int32T>(CodeStubAssembler::Reference{tmp351, tmp352});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1024);
    tmp354 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmI32.raw_bit_field());
    tmp355 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp353}, TNode<Int32T>{tmp354});
    ca_.Branch(tmp355, &block157, std::vector<compiler::Node*>{phi_bb152_7, phi_bb152_8, phi_bb152_9, phi_bb152_10, phi_bb152_11, phi_bb152_14, phi_bb152_15, phi_bb152_21}, &block158, std::vector<compiler::Node*>{phi_bb152_7, phi_bb152_8, phi_bb152_9, phi_bb152_10, phi_bb152_11, phi_bb152_14, phi_bb152_15, phi_bb152_21});
  }

  TNode<IntPtrT> phi_bb157_7;
  TNode<IntPtrT> phi_bb157_8;
  TNode<IntPtrT> phi_bb157_9;
  TNode<IntPtrT> phi_bb157_10;
  TNode<IntPtrT> phi_bb157_11;
  TNode<IntPtrT> phi_bb157_14;
  TNode<BoolT> phi_bb157_15;
  TNode<IntPtrT> phi_bb157_21;
  TNode<IntPtrT> tmp356;
  TNode<IntPtrT> tmp357;
  TNode<IntPtrT> tmp358;
  TNode<BoolT> tmp359;
  if (block157.is_used()) {
    ca_.Bind(&block157, &phi_bb157_7, &phi_bb157_8, &phi_bb157_9, &phi_bb157_10, &phi_bb157_11, &phi_bb157_14, &phi_bb157_15, &phi_bb157_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 308);
    tmp356 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp357 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb157_7}, TNode<IntPtrT>{tmp356});
    tmp358 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp359 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb157_7}, TNode<IntPtrT>{tmp358});
    ca_.Branch(tmp359, &block161, std::vector<compiler::Node*>{phi_bb157_8, phi_bb157_9, phi_bb157_10, phi_bb157_11, phi_bb157_14, phi_bb157_15, phi_bb157_21}, &block162, std::vector<compiler::Node*>{phi_bb157_8, phi_bb157_9, phi_bb157_10, phi_bb157_11, phi_bb157_14, phi_bb157_15, phi_bb157_21});
  }

  TNode<IntPtrT> phi_bb161_8;
  TNode<IntPtrT> phi_bb161_9;
  TNode<IntPtrT> phi_bb161_10;
  TNode<IntPtrT> phi_bb161_11;
  TNode<IntPtrT> phi_bb161_14;
  TNode<BoolT> phi_bb161_15;
  TNode<IntPtrT> phi_bb161_21;
  TNode<Object> tmp360;
  TNode<IntPtrT> tmp361;
  TNode<IntPtrT> tmp362;
  TNode<IntPtrT> tmp363;
  if (block161.is_used()) {
    ca_.Bind(&block161, &phi_bb161_8, &phi_bb161_9, &phi_bb161_10, &phi_bb161_11, &phi_bb161_14, &phi_bb161_15, &phi_bb161_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    std::tie(tmp360, tmp361) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb161_9}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 311);
    tmp362 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp363 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb161_9}, TNode<IntPtrT>{tmp362});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 312);
    ca_.Goto(&block160, phi_bb161_8, tmp363, phi_bb161_10, phi_bb161_11, phi_bb161_14, phi_bb161_15, phi_bb161_21, tmp360, tmp361);
  }

  TNode<IntPtrT> phi_bb162_8;
  TNode<IntPtrT> phi_bb162_9;
  TNode<IntPtrT> phi_bb162_10;
  TNode<IntPtrT> phi_bb162_11;
  TNode<IntPtrT> phi_bb162_14;
  TNode<BoolT> phi_bb162_15;
  TNode<IntPtrT> phi_bb162_21;
  if (block162.is_used()) {
    ca_.Bind(&block162, &phi_bb162_8, &phi_bb162_9, &phi_bb162_10, &phi_bb162_11, &phi_bb162_14, &phi_bb162_15, &phi_bb162_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block164, phi_bb162_8, phi_bb162_9, phi_bb162_10, phi_bb162_11, phi_bb162_14, phi_bb162_15, phi_bb162_21);
    } else {
      ca_.Goto(&block165, phi_bb162_8, phi_bb162_9, phi_bb162_10, phi_bb162_11, phi_bb162_14, phi_bb162_15, phi_bb162_21);
    }
  }

  TNode<IntPtrT> phi_bb164_8;
  TNode<IntPtrT> phi_bb164_9;
  TNode<IntPtrT> phi_bb164_10;
  TNode<IntPtrT> phi_bb164_11;
  TNode<IntPtrT> phi_bb164_14;
  TNode<BoolT> phi_bb164_15;
  TNode<IntPtrT> phi_bb164_21;
  TNode<Object> tmp364;
  TNode<IntPtrT> tmp365;
  TNode<IntPtrT> tmp366;
  TNode<IntPtrT> tmp367;
  if (block164.is_used()) {
    ca_.Bind(&block164, &phi_bb164_8, &phi_bb164_9, &phi_bb164_10, &phi_bb164_11, &phi_bb164_14, &phi_bb164_15, &phi_bb164_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp364, tmp365) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb164_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp366 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp367 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb164_11}, TNode<IntPtrT>{tmp366});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block163, phi_bb164_8, phi_bb164_9, phi_bb164_10, tmp367, phi_bb164_14, phi_bb164_15, phi_bb164_21, tmp364, tmp365);
  }

  TNode<IntPtrT> phi_bb165_8;
  TNode<IntPtrT> phi_bb165_9;
  TNode<IntPtrT> phi_bb165_10;
  TNode<IntPtrT> phi_bb165_11;
  TNode<IntPtrT> phi_bb165_14;
  TNode<BoolT> phi_bb165_15;
  TNode<IntPtrT> phi_bb165_21;
  TNode<IntPtrT> tmp368;
  TNode<BoolT> tmp369;
  if (block165.is_used()) {
    ca_.Bind(&block165, &phi_bb165_8, &phi_bb165_9, &phi_bb165_10, &phi_bb165_11, &phi_bb165_14, &phi_bb165_15, &phi_bb165_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp368 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp369 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb165_14}, TNode<IntPtrT>{tmp368});
    ca_.Branch(tmp369, &block167, std::vector<compiler::Node*>{phi_bb165_8, phi_bb165_9, phi_bb165_10, phi_bb165_11, phi_bb165_14, phi_bb165_15, phi_bb165_21}, &block168, std::vector<compiler::Node*>{phi_bb165_8, phi_bb165_9, phi_bb165_10, phi_bb165_11, phi_bb165_14, phi_bb165_15, phi_bb165_21});
  }

  TNode<IntPtrT> phi_bb167_8;
  TNode<IntPtrT> phi_bb167_9;
  TNode<IntPtrT> phi_bb167_10;
  TNode<IntPtrT> phi_bb167_11;
  TNode<IntPtrT> phi_bb167_14;
  TNode<BoolT> phi_bb167_15;
  TNode<IntPtrT> phi_bb167_21;
  TNode<Object> tmp370;
  TNode<IntPtrT> tmp371;
  TNode<IntPtrT> tmp372;
  TNode<BoolT> tmp373;
  if (block167.is_used()) {
    ca_.Bind(&block167, &phi_bb167_8, &phi_bb167_9, &phi_bb167_10, &phi_bb167_11, &phi_bb167_14, &phi_bb167_15, &phi_bb167_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp370, tmp371) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb167_14}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp372 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp373 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block163, phi_bb167_8, phi_bb167_9, phi_bb167_10, phi_bb167_11, tmp372, tmp373, phi_bb167_21, tmp370, tmp371);
  }

  TNode<IntPtrT> phi_bb168_8;
  TNode<IntPtrT> phi_bb168_9;
  TNode<IntPtrT> phi_bb168_10;
  TNode<IntPtrT> phi_bb168_11;
  TNode<IntPtrT> phi_bb168_14;
  TNode<BoolT> phi_bb168_15;
  TNode<IntPtrT> phi_bb168_21;
  TNode<Object> tmp374;
  TNode<IntPtrT> tmp375;
  TNode<IntPtrT> tmp376;
  TNode<IntPtrT> tmp377;
  TNode<IntPtrT> tmp378;
  TNode<IntPtrT> tmp379;
  TNode<BoolT> tmp380;
  if (block168.is_used()) {
    ca_.Bind(&block168, &phi_bb168_8, &phi_bb168_9, &phi_bb168_10, &phi_bb168_11, &phi_bb168_14, &phi_bb168_15, &phi_bb168_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp374, tmp375) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb168_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp376 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp377 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb168_11}, TNode<IntPtrT>{tmp376});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp378 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp379 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp377}, TNode<IntPtrT>{tmp378});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp380 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block163, phi_bb168_8, phi_bb168_9, phi_bb168_10, tmp379, tmp377, tmp380, phi_bb168_21, tmp374, tmp375);
  }

  TNode<IntPtrT> phi_bb163_8;
  TNode<IntPtrT> phi_bb163_9;
  TNode<IntPtrT> phi_bb163_10;
  TNode<IntPtrT> phi_bb163_11;
  TNode<IntPtrT> phi_bb163_14;
  TNode<BoolT> phi_bb163_15;
  TNode<IntPtrT> phi_bb163_21;
  TNode<Object> phi_bb163_23;
  TNode<IntPtrT> phi_bb163_24;
  if (block163.is_used()) {
    ca_.Bind(&block163, &phi_bb163_8, &phi_bb163_9, &phi_bb163_10, &phi_bb163_11, &phi_bb163_14, &phi_bb163_15, &phi_bb163_21, &phi_bb163_23, &phi_bb163_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 314);
    ca_.Goto(&block160, phi_bb163_8, phi_bb163_9, phi_bb163_10, phi_bb163_11, phi_bb163_14, phi_bb163_15, phi_bb163_21, phi_bb163_23, phi_bb163_24);
  }

  TNode<IntPtrT> phi_bb160_8;
  TNode<IntPtrT> phi_bb160_9;
  TNode<IntPtrT> phi_bb160_10;
  TNode<IntPtrT> phi_bb160_11;
  TNode<IntPtrT> phi_bb160_14;
  TNode<BoolT> phi_bb160_15;
  TNode<IntPtrT> phi_bb160_21;
  TNode<Object> phi_bb160_23;
  TNode<IntPtrT> phi_bb160_24;
  if (block160.is_used()) {
    ca_.Bind(&block160, &phi_bb160_8, &phi_bb160_9, &phi_bb160_10, &phi_bb160_11, &phi_bb160_14, &phi_bb160_15, &phi_bb160_21, &phi_bb160_23, &phi_bb160_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1027);
    if ((wasm::kIsBigEndian)) {
      ca_.Goto(&block169, phi_bb160_8, phi_bb160_9, phi_bb160_10, phi_bb160_11, phi_bb160_14, phi_bb160_15, phi_bb160_21, phi_bb160_23, phi_bb160_24);
    } else {
      ca_.Goto(&block170, phi_bb160_8, phi_bb160_9, phi_bb160_10, phi_bb160_11, phi_bb160_14, phi_bb160_15, phi_bb160_21, phi_bb160_23, phi_bb160_24);
    }
  }

  TNode<IntPtrT> phi_bb169_8;
  TNode<IntPtrT> phi_bb169_9;
  TNode<IntPtrT> phi_bb169_10;
  TNode<IntPtrT> phi_bb169_11;
  TNode<IntPtrT> phi_bb169_14;
  TNode<BoolT> phi_bb169_15;
  TNode<IntPtrT> phi_bb169_21;
  TNode<Object> phi_bb169_23;
  TNode<IntPtrT> phi_bb169_24;
  TNode<Object> tmp381;
  TNode<IntPtrT> tmp382;
  TNode<Int64T> tmp383;
  TNode<Int32T> tmp384;
  if (block169.is_used()) {
    ca_.Bind(&block169, &phi_bb169_8, &phi_bb169_9, &phi_bb169_10, &phi_bb169_11, &phi_bb169_14, &phi_bb169_15, &phi_bb169_21, &phi_bb169_23, &phi_bb169_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1028);
    std::tie(tmp381, tmp382) = RefCast_int64_0(state_, TorqueStructReference_intptr_0{TNode<Object>{phi_bb169_23}, TNode<IntPtrT>{phi_bb169_24}, TorqueStructUnsafe_0{}}).Flatten();
    tmp383 = CodeStubAssembler(state_).LoadReference<Int64T>(CodeStubAssembler::Reference{tmp381, tmp382});
    tmp384 = CodeStubAssembler(state_).TruncateInt64ToInt32(TNode<Int64T>{tmp383});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1027);
    ca_.Goto(&block171, phi_bb169_8, phi_bb169_9, phi_bb169_10, phi_bb169_11, phi_bb169_14, phi_bb169_15, phi_bb169_21, phi_bb169_23, phi_bb169_24, tmp384);
  }

  TNode<IntPtrT> phi_bb170_8;
  TNode<IntPtrT> phi_bb170_9;
  TNode<IntPtrT> phi_bb170_10;
  TNode<IntPtrT> phi_bb170_11;
  TNode<IntPtrT> phi_bb170_14;
  TNode<BoolT> phi_bb170_15;
  TNode<IntPtrT> phi_bb170_21;
  TNode<Object> phi_bb170_23;
  TNode<IntPtrT> phi_bb170_24;
  TNode<Object> tmp385;
  TNode<IntPtrT> tmp386;
  TNode<Int32T> tmp387;
  if (block170.is_used()) {
    ca_.Bind(&block170, &phi_bb170_8, &phi_bb170_9, &phi_bb170_10, &phi_bb170_11, &phi_bb170_14, &phi_bb170_15, &phi_bb170_21, &phi_bb170_23, &phi_bb170_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1030);
    std::tie(tmp385, tmp386) = RefCast_int32_0(state_, TorqueStructReference_intptr_0{TNode<Object>{phi_bb170_23}, TNode<IntPtrT>{phi_bb170_24}, TorqueStructUnsafe_0{}}).Flatten();
    tmp387 = CodeStubAssembler(state_).LoadReference<Int32T>(CodeStubAssembler::Reference{tmp385, tmp386});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1027);
    ca_.Goto(&block171, phi_bb170_8, phi_bb170_9, phi_bb170_10, phi_bb170_11, phi_bb170_14, phi_bb170_15, phi_bb170_21, phi_bb170_23, phi_bb170_24, tmp387);
  }

  TNode<IntPtrT> phi_bb171_8;
  TNode<IntPtrT> phi_bb171_9;
  TNode<IntPtrT> phi_bb171_10;
  TNode<IntPtrT> phi_bb171_11;
  TNode<IntPtrT> phi_bb171_14;
  TNode<BoolT> phi_bb171_15;
  TNode<IntPtrT> phi_bb171_21;
  TNode<Object> phi_bb171_23;
  TNode<IntPtrT> phi_bb171_24;
  TNode<Int32T> phi_bb171_25;
  TNode<Object> tmp388;
  TNode<IntPtrT> tmp389;
  TNode<IntPtrT> tmp390;
  TNode<UintPtrT> tmp391;
  TNode<UintPtrT> tmp392;
  TNode<BoolT> tmp393;
  if (block171.is_used()) {
    ca_.Bind(&block171, &phi_bb171_8, &phi_bb171_9, &phi_bb171_10, &phi_bb171_11, &phi_bb171_14, &phi_bb171_15, &phi_bb171_21, &phi_bb171_23, &phi_bb171_24, &phi_bb171_25);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1032);
    std::tie(tmp388, tmp389, tmp390) = FieldSliceFixedArrayObjects_0(state_, TNode<FixedArray>{tmp92}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 101);
    tmp391 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{phi_bb171_21});
    tmp392 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{tmp390});
    tmp393 = CodeStubAssembler(state_).UintPtrLessThan(TNode<UintPtrT>{tmp391}, TNode<UintPtrT>{tmp392});
    ca_.Branch(tmp393, &block176, std::vector<compiler::Node*>{phi_bb171_8, phi_bb171_9, phi_bb171_10, phi_bb171_11, phi_bb171_14, phi_bb171_15, phi_bb171_21, phi_bb171_23, phi_bb171_24, phi_bb171_21, phi_bb171_21, phi_bb171_21, phi_bb171_21}, &block177, std::vector<compiler::Node*>{phi_bb171_8, phi_bb171_9, phi_bb171_10, phi_bb171_11, phi_bb171_14, phi_bb171_15, phi_bb171_21, phi_bb171_23, phi_bb171_24, phi_bb171_21, phi_bb171_21, phi_bb171_21, phi_bb171_21});
  }

  TNode<IntPtrT> phi_bb176_8;
  TNode<IntPtrT> phi_bb176_9;
  TNode<IntPtrT> phi_bb176_10;
  TNode<IntPtrT> phi_bb176_11;
  TNode<IntPtrT> phi_bb176_14;
  TNode<BoolT> phi_bb176_15;
  TNode<IntPtrT> phi_bb176_21;
  TNode<Object> phi_bb176_23;
  TNode<IntPtrT> phi_bb176_24;
  TNode<IntPtrT> phi_bb176_30;
  TNode<IntPtrT> phi_bb176_31;
  TNode<IntPtrT> phi_bb176_35;
  TNode<IntPtrT> phi_bb176_36;
  TNode<IntPtrT> tmp394;
  TNode<IntPtrT> tmp395;
  TNode<Object> tmp396;
  TNode<IntPtrT> tmp397;
  TNode<Number> tmp398;
  if (block176.is_used()) {
    ca_.Bind(&block176, &phi_bb176_8, &phi_bb176_9, &phi_bb176_10, &phi_bb176_11, &phi_bb176_14, &phi_bb176_15, &phi_bb176_21, &phi_bb176_23, &phi_bb176_24, &phi_bb176_30, &phi_bb176_31, &phi_bb176_35, &phi_bb176_36);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp394 = TimesSizeOf_Object_0(state_, TNode<IntPtrT>{phi_bb176_36});
    tmp395 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp389}, TNode<IntPtrT>{tmp394});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp396, tmp397) = NewReference_Object_0(state_, TNode<Object>{tmp388}, TNode<IntPtrT>{tmp395}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1032);
    tmp398 = Convert_Number_int32_0(state_, TNode<Int32T>{phi_bb171_25});
    CodeStubAssembler(state_).StoreReference<Object>(CodeStubAssembler::Reference{tmp396, tmp397}, tmp398);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1024);
    ca_.Goto(&block159, tmp357, phi_bb176_8, phi_bb176_9, phi_bb176_10, phi_bb176_11, phi_bb176_14, phi_bb176_15, phi_bb176_21);
  }

  TNode<IntPtrT> phi_bb177_8;
  TNode<IntPtrT> phi_bb177_9;
  TNode<IntPtrT> phi_bb177_10;
  TNode<IntPtrT> phi_bb177_11;
  TNode<IntPtrT> phi_bb177_14;
  TNode<BoolT> phi_bb177_15;
  TNode<IntPtrT> phi_bb177_21;
  TNode<Object> phi_bb177_23;
  TNode<IntPtrT> phi_bb177_24;
  TNode<IntPtrT> phi_bb177_30;
  TNode<IntPtrT> phi_bb177_31;
  TNode<IntPtrT> phi_bb177_35;
  TNode<IntPtrT> phi_bb177_36;
  if (block177.is_used()) {
    ca_.Bind(&block177, &phi_bb177_8, &phi_bb177_9, &phi_bb177_10, &phi_bb177_11, &phi_bb177_14, &phi_bb177_15, &phi_bb177_21, &phi_bb177_23, &phi_bb177_24, &phi_bb177_30, &phi_bb177_31, &phi_bb177_35, &phi_bb177_36);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 113);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<IntPtrT> phi_bb158_7;
  TNode<IntPtrT> phi_bb158_8;
  TNode<IntPtrT> phi_bb158_9;
  TNode<IntPtrT> phi_bb158_10;
  TNode<IntPtrT> phi_bb158_11;
  TNode<IntPtrT> phi_bb158_14;
  TNode<BoolT> phi_bb158_15;
  TNode<IntPtrT> phi_bb158_21;
  TNode<Int32T> tmp399;
  TNode<BoolT> tmp400;
  if (block158.is_used()) {
    ca_.Bind(&block158, &phi_bb158_7, &phi_bb158_8, &phi_bb158_9, &phi_bb158_10, &phi_bb158_11, &phi_bb158_14, &phi_bb158_15, &phi_bb158_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1033);
    tmp399 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmF32.raw_bit_field());
    tmp400 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp353}, TNode<Int32T>{tmp399});
    ca_.Branch(tmp400, &block180, std::vector<compiler::Node*>{phi_bb158_7, phi_bb158_8, phi_bb158_9, phi_bb158_10, phi_bb158_11, phi_bb158_14, phi_bb158_15, phi_bb158_21}, &block181, std::vector<compiler::Node*>{phi_bb158_7, phi_bb158_8, phi_bb158_9, phi_bb158_10, phi_bb158_11, phi_bb158_14, phi_bb158_15, phi_bb158_21});
  }

  TNode<IntPtrT> phi_bb180_7;
  TNode<IntPtrT> phi_bb180_8;
  TNode<IntPtrT> phi_bb180_9;
  TNode<IntPtrT> phi_bb180_10;
  TNode<IntPtrT> phi_bb180_11;
  TNode<IntPtrT> phi_bb180_14;
  TNode<BoolT> phi_bb180_15;
  TNode<IntPtrT> phi_bb180_21;
  TNode<IntPtrT> tmp401;
  TNode<IntPtrT> tmp402;
  TNode<IntPtrT> tmp403;
  TNode<BoolT> tmp404;
  if (block180.is_used()) {
    ca_.Bind(&block180, &phi_bb180_7, &phi_bb180_8, &phi_bb180_9, &phi_bb180_10, &phi_bb180_11, &phi_bb180_14, &phi_bb180_15, &phi_bb180_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 318);
    tmp401 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp402 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb180_8}, TNode<IntPtrT>{tmp401});
    tmp403 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp404 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb180_8}, TNode<IntPtrT>{tmp403});
    ca_.Branch(tmp404, &block184, std::vector<compiler::Node*>{phi_bb180_7, phi_bb180_9, phi_bb180_10, phi_bb180_11, phi_bb180_14, phi_bb180_15, phi_bb180_21}, &block185, std::vector<compiler::Node*>{phi_bb180_7, phi_bb180_9, phi_bb180_10, phi_bb180_11, phi_bb180_14, phi_bb180_15, phi_bb180_21});
  }

  TNode<IntPtrT> phi_bb184_7;
  TNode<IntPtrT> phi_bb184_9;
  TNode<IntPtrT> phi_bb184_10;
  TNode<IntPtrT> phi_bb184_11;
  TNode<IntPtrT> phi_bb184_14;
  TNode<BoolT> phi_bb184_15;
  TNode<IntPtrT> phi_bb184_21;
  TNode<Object> tmp405;
  TNode<IntPtrT> tmp406;
  TNode<IntPtrT> tmp407;
  TNode<IntPtrT> tmp408;
  if (block184.is_used()) {
    ca_.Bind(&block184, &phi_bb184_7, &phi_bb184_9, &phi_bb184_10, &phi_bb184_11, &phi_bb184_14, &phi_bb184_15, &phi_bb184_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 319);
    std::tie(tmp405, tmp406) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb184_10}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 321);
    tmp407 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_float64_0(state_)));
    tmp408 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb184_10}, TNode<IntPtrT>{tmp407});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 322);
    ca_.Goto(&block183, phi_bb184_7, phi_bb184_9, tmp408, phi_bb184_11, phi_bb184_14, phi_bb184_15, phi_bb184_21, tmp405, tmp406);
  }

  TNode<IntPtrT> phi_bb185_7;
  TNode<IntPtrT> phi_bb185_9;
  TNode<IntPtrT> phi_bb185_10;
  TNode<IntPtrT> phi_bb185_11;
  TNode<IntPtrT> phi_bb185_14;
  TNode<BoolT> phi_bb185_15;
  TNode<IntPtrT> phi_bb185_21;
  if (block185.is_used()) {
    ca_.Bind(&block185, &phi_bb185_7, &phi_bb185_9, &phi_bb185_10, &phi_bb185_11, &phi_bb185_14, &phi_bb185_15, &phi_bb185_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block187, phi_bb185_7, phi_bb185_9, phi_bb185_10, phi_bb185_11, phi_bb185_14, phi_bb185_15, phi_bb185_21);
    } else {
      ca_.Goto(&block188, phi_bb185_7, phi_bb185_9, phi_bb185_10, phi_bb185_11, phi_bb185_14, phi_bb185_15, phi_bb185_21);
    }
  }

  TNode<IntPtrT> phi_bb187_7;
  TNode<IntPtrT> phi_bb187_9;
  TNode<IntPtrT> phi_bb187_10;
  TNode<IntPtrT> phi_bb187_11;
  TNode<IntPtrT> phi_bb187_14;
  TNode<BoolT> phi_bb187_15;
  TNode<IntPtrT> phi_bb187_21;
  TNode<Object> tmp409;
  TNode<IntPtrT> tmp410;
  TNode<IntPtrT> tmp411;
  TNode<IntPtrT> tmp412;
  if (block187.is_used()) {
    ca_.Bind(&block187, &phi_bb187_7, &phi_bb187_9, &phi_bb187_10, &phi_bb187_11, &phi_bb187_14, &phi_bb187_15, &phi_bb187_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp409, tmp410) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb187_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp411 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp412 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb187_11}, TNode<IntPtrT>{tmp411});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block186, phi_bb187_7, phi_bb187_9, phi_bb187_10, tmp412, phi_bb187_14, phi_bb187_15, phi_bb187_21, tmp409, tmp410);
  }

  TNode<IntPtrT> phi_bb188_7;
  TNode<IntPtrT> phi_bb188_9;
  TNode<IntPtrT> phi_bb188_10;
  TNode<IntPtrT> phi_bb188_11;
  TNode<IntPtrT> phi_bb188_14;
  TNode<BoolT> phi_bb188_15;
  TNode<IntPtrT> phi_bb188_21;
  TNode<IntPtrT> tmp413;
  TNode<BoolT> tmp414;
  if (block188.is_used()) {
    ca_.Bind(&block188, &phi_bb188_7, &phi_bb188_9, &phi_bb188_10, &phi_bb188_11, &phi_bb188_14, &phi_bb188_15, &phi_bb188_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp413 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp414 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb188_14}, TNode<IntPtrT>{tmp413});
    ca_.Branch(tmp414, &block190, std::vector<compiler::Node*>{phi_bb188_7, phi_bb188_9, phi_bb188_10, phi_bb188_11, phi_bb188_14, phi_bb188_15, phi_bb188_21}, &block191, std::vector<compiler::Node*>{phi_bb188_7, phi_bb188_9, phi_bb188_10, phi_bb188_11, phi_bb188_14, phi_bb188_15, phi_bb188_21});
  }

  TNode<IntPtrT> phi_bb190_7;
  TNode<IntPtrT> phi_bb190_9;
  TNode<IntPtrT> phi_bb190_10;
  TNode<IntPtrT> phi_bb190_11;
  TNode<IntPtrT> phi_bb190_14;
  TNode<BoolT> phi_bb190_15;
  TNode<IntPtrT> phi_bb190_21;
  TNode<Object> tmp415;
  TNode<IntPtrT> tmp416;
  TNode<IntPtrT> tmp417;
  TNode<BoolT> tmp418;
  if (block190.is_used()) {
    ca_.Bind(&block190, &phi_bb190_7, &phi_bb190_9, &phi_bb190_10, &phi_bb190_11, &phi_bb190_14, &phi_bb190_15, &phi_bb190_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp415, tmp416) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb190_14}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp417 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp418 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block186, phi_bb190_7, phi_bb190_9, phi_bb190_10, phi_bb190_11, tmp417, tmp418, phi_bb190_21, tmp415, tmp416);
  }

  TNode<IntPtrT> phi_bb191_7;
  TNode<IntPtrT> phi_bb191_9;
  TNode<IntPtrT> phi_bb191_10;
  TNode<IntPtrT> phi_bb191_11;
  TNode<IntPtrT> phi_bb191_14;
  TNode<BoolT> phi_bb191_15;
  TNode<IntPtrT> phi_bb191_21;
  TNode<Object> tmp419;
  TNode<IntPtrT> tmp420;
  TNode<IntPtrT> tmp421;
  TNode<IntPtrT> tmp422;
  TNode<IntPtrT> tmp423;
  TNode<IntPtrT> tmp424;
  TNode<BoolT> tmp425;
  if (block191.is_used()) {
    ca_.Bind(&block191, &phi_bb191_7, &phi_bb191_9, &phi_bb191_10, &phi_bb191_11, &phi_bb191_14, &phi_bb191_15, &phi_bb191_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp419, tmp420) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb191_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp421 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp422 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb191_11}, TNode<IntPtrT>{tmp421});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp423 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp424 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp422}, TNode<IntPtrT>{tmp423});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp425 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block186, phi_bb191_7, phi_bb191_9, phi_bb191_10, tmp424, tmp422, tmp425, phi_bb191_21, tmp419, tmp420);
  }

  TNode<IntPtrT> phi_bb186_7;
  TNode<IntPtrT> phi_bb186_9;
  TNode<IntPtrT> phi_bb186_10;
  TNode<IntPtrT> phi_bb186_11;
  TNode<IntPtrT> phi_bb186_14;
  TNode<BoolT> phi_bb186_15;
  TNode<IntPtrT> phi_bb186_21;
  TNode<Object> phi_bb186_23;
  TNode<IntPtrT> phi_bb186_24;
  if (block186.is_used()) {
    ca_.Bind(&block186, &phi_bb186_7, &phi_bb186_9, &phi_bb186_10, &phi_bb186_11, &phi_bb186_14, &phi_bb186_15, &phi_bb186_21, &phi_bb186_23, &phi_bb186_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 324);
    ca_.Goto(&block183, phi_bb186_7, phi_bb186_9, phi_bb186_10, phi_bb186_11, phi_bb186_14, phi_bb186_15, phi_bb186_21, phi_bb186_23, phi_bb186_24);
  }

  TNode<IntPtrT> phi_bb183_7;
  TNode<IntPtrT> phi_bb183_9;
  TNode<IntPtrT> phi_bb183_10;
  TNode<IntPtrT> phi_bb183_11;
  TNode<IntPtrT> phi_bb183_14;
  TNode<BoolT> phi_bb183_15;
  TNode<IntPtrT> phi_bb183_21;
  TNode<Object> phi_bb183_23;
  TNode<IntPtrT> phi_bb183_24;
  if (block183.is_used()) {
    ca_.Bind(&block183, &phi_bb183_7, &phi_bb183_9, &phi_bb183_10, &phi_bb183_11, &phi_bb183_14, &phi_bb183_15, &phi_bb183_21, &phi_bb183_23, &phi_bb183_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1036);
    if ((wasm::kIsFpAlwaysDouble)) {
      ca_.Goto(&block192, phi_bb183_7, phi_bb183_9, phi_bb183_10, phi_bb183_11, phi_bb183_14, phi_bb183_15, phi_bb183_21, phi_bb183_23, phi_bb183_24);
    } else {
      ca_.Goto(&block193, phi_bb183_7, phi_bb183_9, phi_bb183_10, phi_bb183_11, phi_bb183_14, phi_bb183_15, phi_bb183_21, phi_bb183_23, phi_bb183_24);
    }
  }

  TNode<IntPtrT> phi_bb192_7;
  TNode<IntPtrT> phi_bb192_9;
  TNode<IntPtrT> phi_bb192_10;
  TNode<IntPtrT> phi_bb192_11;
  TNode<IntPtrT> phi_bb192_14;
  TNode<BoolT> phi_bb192_15;
  TNode<IntPtrT> phi_bb192_21;
  TNode<Object> phi_bb192_23;
  TNode<IntPtrT> phi_bb192_24;
  TNode<IntPtrT> tmp426;
  TNode<BoolT> tmp427;
  if (block192.is_used()) {
    ca_.Bind(&block192, &phi_bb192_7, &phi_bb192_9, &phi_bb192_10, &phi_bb192_11, &phi_bb192_14, &phi_bb192_15, &phi_bb192_21, &phi_bb192_23, &phi_bb192_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1037);
    tmp426 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp427 = CodeStubAssembler(state_).IntPtrGreaterThanOrEqual(TNode<IntPtrT>{tmp402}, TNode<IntPtrT>{tmp426});
    ca_.Branch(tmp427, &block195, std::vector<compiler::Node*>{phi_bb192_7, phi_bb192_9, phi_bb192_10, phi_bb192_11, phi_bb192_14, phi_bb192_15, phi_bb192_21, phi_bb192_23, phi_bb192_24}, &block196, std::vector<compiler::Node*>{phi_bb192_7, phi_bb192_9, phi_bb192_10, phi_bb192_11, phi_bb192_14, phi_bb192_15, phi_bb192_21, phi_bb192_23, phi_bb192_24});
  }

  TNode<IntPtrT> phi_bb195_7;
  TNode<IntPtrT> phi_bb195_9;
  TNode<IntPtrT> phi_bb195_10;
  TNode<IntPtrT> phi_bb195_11;
  TNode<IntPtrT> phi_bb195_14;
  TNode<BoolT> phi_bb195_15;
  TNode<IntPtrT> phi_bb195_21;
  TNode<Object> phi_bb195_23;
  TNode<IntPtrT> phi_bb195_24;
  TNode<Object> tmp428;
  TNode<IntPtrT> tmp429;
  TNode<Float64T> tmp430;
  TNode<Float32T> tmp431;
  if (block195.is_used()) {
    ca_.Bind(&block195, &phi_bb195_7, &phi_bb195_9, &phi_bb195_10, &phi_bb195_11, &phi_bb195_14, &phi_bb195_15, &phi_bb195_21, &phi_bb195_23, &phi_bb195_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1038);
    std::tie(tmp428, tmp429) = RefCast_float64_0(state_, TorqueStructReference_intptr_0{TNode<Object>{phi_bb195_23}, TNode<IntPtrT>{phi_bb195_24}, TorqueStructUnsafe_0{}}).Flatten();
    tmp430 = CodeStubAssembler(state_).LoadReference<Float64T>(CodeStubAssembler::Reference{tmp428, tmp429});
    tmp431 = CodeStubAssembler(state_).TruncateFloat64ToFloat32(TNode<Float64T>{tmp430});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1037);
    ca_.Goto(&block198, phi_bb195_7, phi_bb195_9, phi_bb195_10, phi_bb195_11, phi_bb195_14, phi_bb195_15, phi_bb195_21, phi_bb195_23, phi_bb195_24, tmp431);
  }

  TNode<IntPtrT> phi_bb196_7;
  TNode<IntPtrT> phi_bb196_9;
  TNode<IntPtrT> phi_bb196_10;
  TNode<IntPtrT> phi_bb196_11;
  TNode<IntPtrT> phi_bb196_14;
  TNode<BoolT> phi_bb196_15;
  TNode<IntPtrT> phi_bb196_21;
  TNode<Object> phi_bb196_23;
  TNode<IntPtrT> phi_bb196_24;
  TNode<Object> tmp432;
  TNode<IntPtrT> tmp433;
  TNode<Float32T> tmp434;
  if (block196.is_used()) {
    ca_.Bind(&block196, &phi_bb196_7, &phi_bb196_9, &phi_bb196_10, &phi_bb196_11, &phi_bb196_14, &phi_bb196_15, &phi_bb196_21, &phi_bb196_23, &phi_bb196_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1040);
    std::tie(tmp432, tmp433) = RefCast_float32_0(state_, TorqueStructReference_intptr_0{TNode<Object>{phi_bb196_23}, TNode<IntPtrT>{phi_bb196_24}, TorqueStructUnsafe_0{}}).Flatten();
    tmp434 = CodeStubAssembler(state_).LoadReference<Float32T>(CodeStubAssembler::Reference{tmp432, tmp433});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1037);
    ca_.Goto(&block198, phi_bb196_7, phi_bb196_9, phi_bb196_10, phi_bb196_11, phi_bb196_14, phi_bb196_15, phi_bb196_21, phi_bb196_23, phi_bb196_24, tmp434);
  }

  TNode<IntPtrT> phi_bb198_7;
  TNode<IntPtrT> phi_bb198_9;
  TNode<IntPtrT> phi_bb198_10;
  TNode<IntPtrT> phi_bb198_11;
  TNode<IntPtrT> phi_bb198_14;
  TNode<BoolT> phi_bb198_15;
  TNode<IntPtrT> phi_bb198_21;
  TNode<Object> phi_bb198_23;
  TNode<IntPtrT> phi_bb198_24;
  TNode<Float32T> phi_bb198_25;
  if (block198.is_used()) {
    ca_.Bind(&block198, &phi_bb198_7, &phi_bb198_9, &phi_bb198_10, &phi_bb198_11, &phi_bb198_14, &phi_bb198_15, &phi_bb198_21, &phi_bb198_23, &phi_bb198_24, &phi_bb198_25);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1036);
    ca_.Goto(&block194, phi_bb198_7, phi_bb198_9, phi_bb198_10, phi_bb198_11, phi_bb198_14, phi_bb198_15, phi_bb198_21, phi_bb198_23, phi_bb198_24, phi_bb198_25);
  }

  TNode<IntPtrT> phi_bb193_7;
  TNode<IntPtrT> phi_bb193_9;
  TNode<IntPtrT> phi_bb193_10;
  TNode<IntPtrT> phi_bb193_11;
  TNode<IntPtrT> phi_bb193_14;
  TNode<BoolT> phi_bb193_15;
  TNode<IntPtrT> phi_bb193_21;
  TNode<Object> phi_bb193_23;
  TNode<IntPtrT> phi_bb193_24;
  if (block193.is_used()) {
    ca_.Bind(&block193, &phi_bb193_7, &phi_bb193_9, &phi_bb193_10, &phi_bb193_11, &phi_bb193_14, &phi_bb193_15, &phi_bb193_21, &phi_bb193_23, &phi_bb193_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1042);
    if ((wasm::kIsBigEndianOnSim)) {
      ca_.Goto(&block199, phi_bb193_7, phi_bb193_9, phi_bb193_10, phi_bb193_11, phi_bb193_14, phi_bb193_15, phi_bb193_21, phi_bb193_23, phi_bb193_24);
    } else {
      ca_.Goto(&block200, phi_bb193_7, phi_bb193_9, phi_bb193_10, phi_bb193_11, phi_bb193_14, phi_bb193_15, phi_bb193_21, phi_bb193_23, phi_bb193_24);
    }
  }

  TNode<IntPtrT> phi_bb199_7;
  TNode<IntPtrT> phi_bb199_9;
  TNode<IntPtrT> phi_bb199_10;
  TNode<IntPtrT> phi_bb199_11;
  TNode<IntPtrT> phi_bb199_14;
  TNode<BoolT> phi_bb199_15;
  TNode<IntPtrT> phi_bb199_21;
  TNode<Object> phi_bb199_23;
  TNode<IntPtrT> phi_bb199_24;
  TNode<IntPtrT> tmp435;
  TNode<BoolT> tmp436;
  if (block199.is_used()) {
    ca_.Bind(&block199, &phi_bb199_7, &phi_bb199_9, &phi_bb199_10, &phi_bb199_11, &phi_bb199_14, &phi_bb199_15, &phi_bb199_21, &phi_bb199_23, &phi_bb199_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1043);
    tmp435 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp436 = CodeStubAssembler(state_).IntPtrGreaterThanOrEqual(TNode<IntPtrT>{tmp402}, TNode<IntPtrT>{tmp435});
    ca_.Branch(tmp436, &block202, std::vector<compiler::Node*>{phi_bb199_7, phi_bb199_9, phi_bb199_10, phi_bb199_11, phi_bb199_14, phi_bb199_15, phi_bb199_21, phi_bb199_23, phi_bb199_24}, &block203, std::vector<compiler::Node*>{phi_bb199_7, phi_bb199_9, phi_bb199_10, phi_bb199_11, phi_bb199_14, phi_bb199_15, phi_bb199_21, phi_bb199_23, phi_bb199_24});
  }

  TNode<IntPtrT> phi_bb202_7;
  TNode<IntPtrT> phi_bb202_9;
  TNode<IntPtrT> phi_bb202_10;
  TNode<IntPtrT> phi_bb202_11;
  TNode<IntPtrT> phi_bb202_14;
  TNode<BoolT> phi_bb202_15;
  TNode<IntPtrT> phi_bb202_21;
  TNode<Object> phi_bb202_23;
  TNode<IntPtrT> phi_bb202_24;
  TNode<Object> tmp437;
  TNode<IntPtrT> tmp438;
  TNode<Int64T> tmp439;
  TNode<Int64T> tmp440;
  TNode<Int64T> tmp441;
  TNode<Int32T> tmp442;
  TNode<Float32T> tmp443;
  if (block202.is_used()) {
    ca_.Bind(&block202, &phi_bb202_7, &phi_bb202_9, &phi_bb202_10, &phi_bb202_11, &phi_bb202_14, &phi_bb202_15, &phi_bb202_21, &phi_bb202_23, &phi_bb202_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1045);
    std::tie(tmp437, tmp438) = RefCast_int64_0(state_, TorqueStructReference_intptr_0{TNode<Object>{phi_bb202_23}, TNode<IntPtrT>{phi_bb202_24}, TorqueStructUnsafe_0{}}).Flatten();
    tmp439 = CodeStubAssembler(state_).LoadReference<Int64T>(CodeStubAssembler::Reference{tmp437, tmp438});
    tmp440 = FromConstexpr_int64_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x20ull));
    tmp441 = CodeStubAssembler(state_).Word64Sar(TNode<Int64T>{tmp439}, TNode<Int64T>{tmp440});
    tmp442 = CodeStubAssembler(state_).TruncateInt64ToInt32(TNode<Int64T>{tmp441});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1044);
    tmp443 = CodeStubAssembler(state_).BitcastInt32ToFloat32(TNode<Int32T>{tmp442});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1043);
    ca_.Goto(&block205, phi_bb202_7, phi_bb202_9, phi_bb202_10, phi_bb202_11, phi_bb202_14, phi_bb202_15, phi_bb202_21, phi_bb202_23, phi_bb202_24, tmp443);
  }

  TNode<IntPtrT> phi_bb203_7;
  TNode<IntPtrT> phi_bb203_9;
  TNode<IntPtrT> phi_bb203_10;
  TNode<IntPtrT> phi_bb203_11;
  TNode<IntPtrT> phi_bb203_14;
  TNode<BoolT> phi_bb203_15;
  TNode<IntPtrT> phi_bb203_21;
  TNode<Object> phi_bb203_23;
  TNode<IntPtrT> phi_bb203_24;
  TNode<Object> tmp444;
  TNode<IntPtrT> tmp445;
  TNode<Float32T> tmp446;
  if (block203.is_used()) {
    ca_.Bind(&block203, &phi_bb203_7, &phi_bb203_9, &phi_bb203_10, &phi_bb203_11, &phi_bb203_14, &phi_bb203_15, &phi_bb203_21, &phi_bb203_23, &phi_bb203_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1047);
    std::tie(tmp444, tmp445) = RefCast_float32_0(state_, TorqueStructReference_intptr_0{TNode<Object>{phi_bb203_23}, TNode<IntPtrT>{phi_bb203_24}, TorqueStructUnsafe_0{}}).Flatten();
    tmp446 = CodeStubAssembler(state_).LoadReference<Float32T>(CodeStubAssembler::Reference{tmp444, tmp445});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1043);
    ca_.Goto(&block205, phi_bb203_7, phi_bb203_9, phi_bb203_10, phi_bb203_11, phi_bb203_14, phi_bb203_15, phi_bb203_21, phi_bb203_23, phi_bb203_24, tmp446);
  }

  TNode<IntPtrT> phi_bb205_7;
  TNode<IntPtrT> phi_bb205_9;
  TNode<IntPtrT> phi_bb205_10;
  TNode<IntPtrT> phi_bb205_11;
  TNode<IntPtrT> phi_bb205_14;
  TNode<BoolT> phi_bb205_15;
  TNode<IntPtrT> phi_bb205_21;
  TNode<Object> phi_bb205_23;
  TNode<IntPtrT> phi_bb205_24;
  TNode<Float32T> phi_bb205_25;
  if (block205.is_used()) {
    ca_.Bind(&block205, &phi_bb205_7, &phi_bb205_9, &phi_bb205_10, &phi_bb205_11, &phi_bb205_14, &phi_bb205_15, &phi_bb205_21, &phi_bb205_23, &phi_bb205_24, &phi_bb205_25);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1042);
    ca_.Goto(&block201, phi_bb205_7, phi_bb205_9, phi_bb205_10, phi_bb205_11, phi_bb205_14, phi_bb205_15, phi_bb205_21, phi_bb205_23, phi_bb205_24, phi_bb205_25);
  }

  TNode<IntPtrT> phi_bb200_7;
  TNode<IntPtrT> phi_bb200_9;
  TNode<IntPtrT> phi_bb200_10;
  TNode<IntPtrT> phi_bb200_11;
  TNode<IntPtrT> phi_bb200_14;
  TNode<BoolT> phi_bb200_15;
  TNode<IntPtrT> phi_bb200_21;
  TNode<Object> phi_bb200_23;
  TNode<IntPtrT> phi_bb200_24;
  TNode<Object> tmp447;
  TNode<IntPtrT> tmp448;
  TNode<Float32T> tmp449;
  if (block200.is_used()) {
    ca_.Bind(&block200, &phi_bb200_7, &phi_bb200_9, &phi_bb200_10, &phi_bb200_11, &phi_bb200_14, &phi_bb200_15, &phi_bb200_21, &phi_bb200_23, &phi_bb200_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1050);
    std::tie(tmp447, tmp448) = RefCast_float32_0(state_, TorqueStructReference_intptr_0{TNode<Object>{phi_bb200_23}, TNode<IntPtrT>{phi_bb200_24}, TorqueStructUnsafe_0{}}).Flatten();
    tmp449 = CodeStubAssembler(state_).LoadReference<Float32T>(CodeStubAssembler::Reference{tmp447, tmp448});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1042);
    ca_.Goto(&block201, phi_bb200_7, phi_bb200_9, phi_bb200_10, phi_bb200_11, phi_bb200_14, phi_bb200_15, phi_bb200_21, phi_bb200_23, phi_bb200_24, tmp449);
  }

  TNode<IntPtrT> phi_bb201_7;
  TNode<IntPtrT> phi_bb201_9;
  TNode<IntPtrT> phi_bb201_10;
  TNode<IntPtrT> phi_bb201_11;
  TNode<IntPtrT> phi_bb201_14;
  TNode<BoolT> phi_bb201_15;
  TNode<IntPtrT> phi_bb201_21;
  TNode<Object> phi_bb201_23;
  TNode<IntPtrT> phi_bb201_24;
  TNode<Float32T> phi_bb201_25;
  if (block201.is_used()) {
    ca_.Bind(&block201, &phi_bb201_7, &phi_bb201_9, &phi_bb201_10, &phi_bb201_11, &phi_bb201_14, &phi_bb201_15, &phi_bb201_21, &phi_bb201_23, &phi_bb201_24, &phi_bb201_25);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1036);
    ca_.Goto(&block194, phi_bb201_7, phi_bb201_9, phi_bb201_10, phi_bb201_11, phi_bb201_14, phi_bb201_15, phi_bb201_21, phi_bb201_23, phi_bb201_24, phi_bb201_25);
  }

  TNode<IntPtrT> phi_bb194_7;
  TNode<IntPtrT> phi_bb194_9;
  TNode<IntPtrT> phi_bb194_10;
  TNode<IntPtrT> phi_bb194_11;
  TNode<IntPtrT> phi_bb194_14;
  TNode<BoolT> phi_bb194_15;
  TNode<IntPtrT> phi_bb194_21;
  TNode<Object> phi_bb194_23;
  TNode<IntPtrT> phi_bb194_24;
  TNode<Float32T> phi_bb194_25;
  TNode<Object> tmp450;
  TNode<IntPtrT> tmp451;
  TNode<IntPtrT> tmp452;
  TNode<UintPtrT> tmp453;
  TNode<UintPtrT> tmp454;
  TNode<BoolT> tmp455;
  if (block194.is_used()) {
    ca_.Bind(&block194, &phi_bb194_7, &phi_bb194_9, &phi_bb194_10, &phi_bb194_11, &phi_bb194_14, &phi_bb194_15, &phi_bb194_21, &phi_bb194_23, &phi_bb194_24, &phi_bb194_25);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1052);
    std::tie(tmp450, tmp451, tmp452) = FieldSliceFixedArrayObjects_0(state_, TNode<FixedArray>{tmp92}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 101);
    tmp453 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{phi_bb194_21});
    tmp454 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{tmp452});
    tmp455 = CodeStubAssembler(state_).UintPtrLessThan(TNode<UintPtrT>{tmp453}, TNode<UintPtrT>{tmp454});
    ca_.Branch(tmp455, &block210, std::vector<compiler::Node*>{phi_bb194_7, phi_bb194_9, phi_bb194_10, phi_bb194_11, phi_bb194_14, phi_bb194_15, phi_bb194_21, phi_bb194_23, phi_bb194_24, phi_bb194_21, phi_bb194_21, phi_bb194_21, phi_bb194_21}, &block211, std::vector<compiler::Node*>{phi_bb194_7, phi_bb194_9, phi_bb194_10, phi_bb194_11, phi_bb194_14, phi_bb194_15, phi_bb194_21, phi_bb194_23, phi_bb194_24, phi_bb194_21, phi_bb194_21, phi_bb194_21, phi_bb194_21});
  }

  TNode<IntPtrT> phi_bb210_7;
  TNode<IntPtrT> phi_bb210_9;
  TNode<IntPtrT> phi_bb210_10;
  TNode<IntPtrT> phi_bb210_11;
  TNode<IntPtrT> phi_bb210_14;
  TNode<BoolT> phi_bb210_15;
  TNode<IntPtrT> phi_bb210_21;
  TNode<Object> phi_bb210_23;
  TNode<IntPtrT> phi_bb210_24;
  TNode<IntPtrT> phi_bb210_30;
  TNode<IntPtrT> phi_bb210_31;
  TNode<IntPtrT> phi_bb210_35;
  TNode<IntPtrT> phi_bb210_36;
  TNode<IntPtrT> tmp456;
  TNode<IntPtrT> tmp457;
  TNode<Object> tmp458;
  TNode<IntPtrT> tmp459;
  TNode<Number> tmp460;
  if (block210.is_used()) {
    ca_.Bind(&block210, &phi_bb210_7, &phi_bb210_9, &phi_bb210_10, &phi_bb210_11, &phi_bb210_14, &phi_bb210_15, &phi_bb210_21, &phi_bb210_23, &phi_bb210_24, &phi_bb210_30, &phi_bb210_31, &phi_bb210_35, &phi_bb210_36);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp456 = TimesSizeOf_Object_0(state_, TNode<IntPtrT>{phi_bb210_36});
    tmp457 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp451}, TNode<IntPtrT>{tmp456});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp458, tmp459) = NewReference_Object_0(state_, TNode<Object>{tmp450}, TNode<IntPtrT>{tmp457}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1052);
    tmp460 = Convert_Number_float32_0(state_, TNode<Float32T>{phi_bb194_25});
    CodeStubAssembler(state_).StoreReference<Object>(CodeStubAssembler::Reference{tmp458, tmp459}, tmp460);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1033);
    ca_.Goto(&block182, phi_bb210_7, tmp402, phi_bb210_9, phi_bb210_10, phi_bb210_11, phi_bb210_14, phi_bb210_15, phi_bb210_21);
  }

  TNode<IntPtrT> phi_bb211_7;
  TNode<IntPtrT> phi_bb211_9;
  TNode<IntPtrT> phi_bb211_10;
  TNode<IntPtrT> phi_bb211_11;
  TNode<IntPtrT> phi_bb211_14;
  TNode<BoolT> phi_bb211_15;
  TNode<IntPtrT> phi_bb211_21;
  TNode<Object> phi_bb211_23;
  TNode<IntPtrT> phi_bb211_24;
  TNode<IntPtrT> phi_bb211_30;
  TNode<IntPtrT> phi_bb211_31;
  TNode<IntPtrT> phi_bb211_35;
  TNode<IntPtrT> phi_bb211_36;
  if (block211.is_used()) {
    ca_.Bind(&block211, &phi_bb211_7, &phi_bb211_9, &phi_bb211_10, &phi_bb211_11, &phi_bb211_14, &phi_bb211_15, &phi_bb211_21, &phi_bb211_23, &phi_bb211_24, &phi_bb211_30, &phi_bb211_31, &phi_bb211_35, &phi_bb211_36);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 113);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<IntPtrT> phi_bb181_7;
  TNode<IntPtrT> phi_bb181_8;
  TNode<IntPtrT> phi_bb181_9;
  TNode<IntPtrT> phi_bb181_10;
  TNode<IntPtrT> phi_bb181_11;
  TNode<IntPtrT> phi_bb181_14;
  TNode<BoolT> phi_bb181_15;
  TNode<IntPtrT> phi_bb181_21;
  TNode<Int32T> tmp461;
  TNode<BoolT> tmp462;
  if (block181.is_used()) {
    ca_.Bind(&block181, &phi_bb181_7, &phi_bb181_8, &phi_bb181_9, &phi_bb181_10, &phi_bb181_11, &phi_bb181_14, &phi_bb181_15, &phi_bb181_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1053);
    tmp461 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmI64.raw_bit_field());
    tmp462 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp353}, TNode<Int32T>{tmp461});
    ca_.Branch(tmp462, &block214, std::vector<compiler::Node*>{phi_bb181_7, phi_bb181_8, phi_bb181_9, phi_bb181_10, phi_bb181_11, phi_bb181_14, phi_bb181_15, phi_bb181_21}, &block215, std::vector<compiler::Node*>{phi_bb181_7, phi_bb181_8, phi_bb181_9, phi_bb181_10, phi_bb181_11, phi_bb181_14, phi_bb181_15, phi_bb181_21});
  }

  TNode<IntPtrT> phi_bb214_7;
  TNode<IntPtrT> phi_bb214_8;
  TNode<IntPtrT> phi_bb214_9;
  TNode<IntPtrT> phi_bb214_10;
  TNode<IntPtrT> phi_bb214_11;
  TNode<IntPtrT> phi_bb214_14;
  TNode<BoolT> phi_bb214_15;
  TNode<IntPtrT> phi_bb214_21;
  if (block214.is_used()) {
    ca_.Bind(&block214, &phi_bb214_7, &phi_bb214_8, &phi_bb214_9, &phi_bb214_10, &phi_bb214_11, &phi_bb214_14, &phi_bb214_15, &phi_bb214_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1054);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block217, phi_bb214_7, phi_bb214_8, phi_bb214_9, phi_bb214_10, phi_bb214_11, phi_bb214_14, phi_bb214_15, phi_bb214_21);
    } else {
      ca_.Goto(&block218, phi_bb214_7, phi_bb214_8, phi_bb214_9, phi_bb214_10, phi_bb214_11, phi_bb214_14, phi_bb214_15, phi_bb214_21);
    }
  }

  TNode<IntPtrT> phi_bb217_7;
  TNode<IntPtrT> phi_bb217_8;
  TNode<IntPtrT> phi_bb217_9;
  TNode<IntPtrT> phi_bb217_10;
  TNode<IntPtrT> phi_bb217_11;
  TNode<IntPtrT> phi_bb217_14;
  TNode<BoolT> phi_bb217_15;
  TNode<IntPtrT> phi_bb217_21;
  TNode<IntPtrT> tmp463;
  TNode<IntPtrT> tmp464;
  TNode<IntPtrT> tmp465;
  TNode<BoolT> tmp466;
  if (block217.is_used()) {
    ca_.Bind(&block217, &phi_bb217_7, &phi_bb217_8, &phi_bb217_9, &phi_bb217_10, &phi_bb217_11, &phi_bb217_14, &phi_bb217_15, &phi_bb217_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 308);
    tmp463 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp464 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb217_7}, TNode<IntPtrT>{tmp463});
    tmp465 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp466 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb217_7}, TNode<IntPtrT>{tmp465});
    ca_.Branch(tmp466, &block221, std::vector<compiler::Node*>{phi_bb217_8, phi_bb217_9, phi_bb217_10, phi_bb217_11, phi_bb217_14, phi_bb217_15, phi_bb217_21}, &block222, std::vector<compiler::Node*>{phi_bb217_8, phi_bb217_9, phi_bb217_10, phi_bb217_11, phi_bb217_14, phi_bb217_15, phi_bb217_21});
  }

  TNode<IntPtrT> phi_bb221_8;
  TNode<IntPtrT> phi_bb221_9;
  TNode<IntPtrT> phi_bb221_10;
  TNode<IntPtrT> phi_bb221_11;
  TNode<IntPtrT> phi_bb221_14;
  TNode<BoolT> phi_bb221_15;
  TNode<IntPtrT> phi_bb221_21;
  TNode<Object> tmp467;
  TNode<IntPtrT> tmp468;
  TNode<IntPtrT> tmp469;
  TNode<IntPtrT> tmp470;
  if (block221.is_used()) {
    ca_.Bind(&block221, &phi_bb221_8, &phi_bb221_9, &phi_bb221_10, &phi_bb221_11, &phi_bb221_14, &phi_bb221_15, &phi_bb221_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    std::tie(tmp467, tmp468) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb221_9}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 311);
    tmp469 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp470 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb221_9}, TNode<IntPtrT>{tmp469});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 312);
    ca_.Goto(&block220, phi_bb221_8, tmp470, phi_bb221_10, phi_bb221_11, phi_bb221_14, phi_bb221_15, phi_bb221_21, tmp467, tmp468);
  }

  TNode<IntPtrT> phi_bb222_8;
  TNode<IntPtrT> phi_bb222_9;
  TNode<IntPtrT> phi_bb222_10;
  TNode<IntPtrT> phi_bb222_11;
  TNode<IntPtrT> phi_bb222_14;
  TNode<BoolT> phi_bb222_15;
  TNode<IntPtrT> phi_bb222_21;
  if (block222.is_used()) {
    ca_.Bind(&block222, &phi_bb222_8, &phi_bb222_9, &phi_bb222_10, &phi_bb222_11, &phi_bb222_14, &phi_bb222_15, &phi_bb222_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block224, phi_bb222_8, phi_bb222_9, phi_bb222_10, phi_bb222_11, phi_bb222_14, phi_bb222_15, phi_bb222_21);
    } else {
      ca_.Goto(&block225, phi_bb222_8, phi_bb222_9, phi_bb222_10, phi_bb222_11, phi_bb222_14, phi_bb222_15, phi_bb222_21);
    }
  }

  TNode<IntPtrT> phi_bb224_8;
  TNode<IntPtrT> phi_bb224_9;
  TNode<IntPtrT> phi_bb224_10;
  TNode<IntPtrT> phi_bb224_11;
  TNode<IntPtrT> phi_bb224_14;
  TNode<BoolT> phi_bb224_15;
  TNode<IntPtrT> phi_bb224_21;
  TNode<Object> tmp471;
  TNode<IntPtrT> tmp472;
  TNode<IntPtrT> tmp473;
  TNode<IntPtrT> tmp474;
  if (block224.is_used()) {
    ca_.Bind(&block224, &phi_bb224_8, &phi_bb224_9, &phi_bb224_10, &phi_bb224_11, &phi_bb224_14, &phi_bb224_15, &phi_bb224_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp471, tmp472) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb224_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp473 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp474 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb224_11}, TNode<IntPtrT>{tmp473});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block223, phi_bb224_8, phi_bb224_9, phi_bb224_10, tmp474, phi_bb224_14, phi_bb224_15, phi_bb224_21, tmp471, tmp472);
  }

  TNode<IntPtrT> phi_bb225_8;
  TNode<IntPtrT> phi_bb225_9;
  TNode<IntPtrT> phi_bb225_10;
  TNode<IntPtrT> phi_bb225_11;
  TNode<IntPtrT> phi_bb225_14;
  TNode<BoolT> phi_bb225_15;
  TNode<IntPtrT> phi_bb225_21;
  TNode<IntPtrT> tmp475;
  TNode<BoolT> tmp476;
  if (block225.is_used()) {
    ca_.Bind(&block225, &phi_bb225_8, &phi_bb225_9, &phi_bb225_10, &phi_bb225_11, &phi_bb225_14, &phi_bb225_15, &phi_bb225_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp475 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp476 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb225_14}, TNode<IntPtrT>{tmp475});
    ca_.Branch(tmp476, &block227, std::vector<compiler::Node*>{phi_bb225_8, phi_bb225_9, phi_bb225_10, phi_bb225_11, phi_bb225_14, phi_bb225_15, phi_bb225_21}, &block228, std::vector<compiler::Node*>{phi_bb225_8, phi_bb225_9, phi_bb225_10, phi_bb225_11, phi_bb225_14, phi_bb225_15, phi_bb225_21});
  }

  TNode<IntPtrT> phi_bb227_8;
  TNode<IntPtrT> phi_bb227_9;
  TNode<IntPtrT> phi_bb227_10;
  TNode<IntPtrT> phi_bb227_11;
  TNode<IntPtrT> phi_bb227_14;
  TNode<BoolT> phi_bb227_15;
  TNode<IntPtrT> phi_bb227_21;
  TNode<Object> tmp477;
  TNode<IntPtrT> tmp478;
  TNode<IntPtrT> tmp479;
  TNode<BoolT> tmp480;
  if (block227.is_used()) {
    ca_.Bind(&block227, &phi_bb227_8, &phi_bb227_9, &phi_bb227_10, &phi_bb227_11, &phi_bb227_14, &phi_bb227_15, &phi_bb227_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp477, tmp478) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb227_14}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp479 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp480 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block223, phi_bb227_8, phi_bb227_9, phi_bb227_10, phi_bb227_11, tmp479, tmp480, phi_bb227_21, tmp477, tmp478);
  }

  TNode<IntPtrT> phi_bb228_8;
  TNode<IntPtrT> phi_bb228_9;
  TNode<IntPtrT> phi_bb228_10;
  TNode<IntPtrT> phi_bb228_11;
  TNode<IntPtrT> phi_bb228_14;
  TNode<BoolT> phi_bb228_15;
  TNode<IntPtrT> phi_bb228_21;
  TNode<Object> tmp481;
  TNode<IntPtrT> tmp482;
  TNode<IntPtrT> tmp483;
  TNode<IntPtrT> tmp484;
  TNode<IntPtrT> tmp485;
  TNode<IntPtrT> tmp486;
  TNode<BoolT> tmp487;
  if (block228.is_used()) {
    ca_.Bind(&block228, &phi_bb228_8, &phi_bb228_9, &phi_bb228_10, &phi_bb228_11, &phi_bb228_14, &phi_bb228_15, &phi_bb228_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp481, tmp482) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb228_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp483 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp484 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb228_11}, TNode<IntPtrT>{tmp483});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp485 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp486 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp484}, TNode<IntPtrT>{tmp485});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp487 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block223, phi_bb228_8, phi_bb228_9, phi_bb228_10, tmp486, tmp484, tmp487, phi_bb228_21, tmp481, tmp482);
  }

  TNode<IntPtrT> phi_bb223_8;
  TNode<IntPtrT> phi_bb223_9;
  TNode<IntPtrT> phi_bb223_10;
  TNode<IntPtrT> phi_bb223_11;
  TNode<IntPtrT> phi_bb223_14;
  TNode<BoolT> phi_bb223_15;
  TNode<IntPtrT> phi_bb223_21;
  TNode<Object> phi_bb223_23;
  TNode<IntPtrT> phi_bb223_24;
  if (block223.is_used()) {
    ca_.Bind(&block223, &phi_bb223_8, &phi_bb223_9, &phi_bb223_10, &phi_bb223_11, &phi_bb223_14, &phi_bb223_15, &phi_bb223_21, &phi_bb223_23, &phi_bb223_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 314);
    ca_.Goto(&block220, phi_bb223_8, phi_bb223_9, phi_bb223_10, phi_bb223_11, phi_bb223_14, phi_bb223_15, phi_bb223_21, phi_bb223_23, phi_bb223_24);
  }

  TNode<IntPtrT> phi_bb220_8;
  TNode<IntPtrT> phi_bb220_9;
  TNode<IntPtrT> phi_bb220_10;
  TNode<IntPtrT> phi_bb220_11;
  TNode<IntPtrT> phi_bb220_14;
  TNode<BoolT> phi_bb220_15;
  TNode<IntPtrT> phi_bb220_21;
  TNode<Object> phi_bb220_23;
  TNode<IntPtrT> phi_bb220_24;
  TNode<IntPtrT> tmp488;
  TNode<Object> tmp489;
  TNode<IntPtrT> tmp490;
  TNode<IntPtrT> tmp491;
  TNode<UintPtrT> tmp492;
  TNode<UintPtrT> tmp493;
  TNode<BoolT> tmp494;
  if (block220.is_used()) {
    ca_.Bind(&block220, &phi_bb220_8, &phi_bb220_9, &phi_bb220_10, &phi_bb220_11, &phi_bb220_14, &phi_bb220_15, &phi_bb220_21, &phi_bb220_23, &phi_bb220_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1056);
    tmp488 = CodeStubAssembler(state_).LoadReference<IntPtrT>(CodeStubAssembler::Reference{phi_bb220_23, phi_bb220_24});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1057);
    std::tie(tmp489, tmp490, tmp491) = FieldSliceFixedArrayObjects_0(state_, TNode<FixedArray>{tmp92}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 101);
    tmp492 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{phi_bb220_21});
    tmp493 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{tmp491});
    tmp494 = CodeStubAssembler(state_).UintPtrLessThan(TNode<UintPtrT>{tmp492}, TNode<UintPtrT>{tmp493});
    ca_.Branch(tmp494, &block233, std::vector<compiler::Node*>{phi_bb220_8, phi_bb220_9, phi_bb220_10, phi_bb220_11, phi_bb220_14, phi_bb220_15, phi_bb220_21, phi_bb220_23, phi_bb220_24, phi_bb220_21, phi_bb220_21, phi_bb220_21, phi_bb220_21}, &block234, std::vector<compiler::Node*>{phi_bb220_8, phi_bb220_9, phi_bb220_10, phi_bb220_11, phi_bb220_14, phi_bb220_15, phi_bb220_21, phi_bb220_23, phi_bb220_24, phi_bb220_21, phi_bb220_21, phi_bb220_21, phi_bb220_21});
  }

  TNode<IntPtrT> phi_bb233_8;
  TNode<IntPtrT> phi_bb233_9;
  TNode<IntPtrT> phi_bb233_10;
  TNode<IntPtrT> phi_bb233_11;
  TNode<IntPtrT> phi_bb233_14;
  TNode<BoolT> phi_bb233_15;
  TNode<IntPtrT> phi_bb233_21;
  TNode<Object> phi_bb233_23;
  TNode<IntPtrT> phi_bb233_24;
  TNode<IntPtrT> phi_bb233_30;
  TNode<IntPtrT> phi_bb233_31;
  TNode<IntPtrT> phi_bb233_35;
  TNode<IntPtrT> phi_bb233_36;
  TNode<IntPtrT> tmp495;
  TNode<IntPtrT> tmp496;
  TNode<Object> tmp497;
  TNode<IntPtrT> tmp498;
  TNode<BigInt> tmp499;
  if (block233.is_used()) {
    ca_.Bind(&block233, &phi_bb233_8, &phi_bb233_9, &phi_bb233_10, &phi_bb233_11, &phi_bb233_14, &phi_bb233_15, &phi_bb233_21, &phi_bb233_23, &phi_bb233_24, &phi_bb233_30, &phi_bb233_31, &phi_bb233_35, &phi_bb233_36);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp495 = TimesSizeOf_Object_0(state_, TNode<IntPtrT>{phi_bb233_36});
    tmp496 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp490}, TNode<IntPtrT>{tmp495});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp497, tmp498) = NewReference_Object_0(state_, TNode<Object>{tmp489}, TNode<IntPtrT>{tmp496}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1057);
    tmp499 = ca_.CallBuiltin<BigInt>(Builtin::kI64ToBigInt, TNode<Object>(), tmp488);
    CodeStubAssembler(state_).StoreReference<Object>(CodeStubAssembler::Reference{tmp497, tmp498}, tmp499);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1054);
    ca_.Goto(&block219, tmp464, phi_bb233_8, phi_bb233_9, phi_bb233_10, phi_bb233_11, phi_bb233_14, phi_bb233_15, phi_bb233_21);
  }

  TNode<IntPtrT> phi_bb234_8;
  TNode<IntPtrT> phi_bb234_9;
  TNode<IntPtrT> phi_bb234_10;
  TNode<IntPtrT> phi_bb234_11;
  TNode<IntPtrT> phi_bb234_14;
  TNode<BoolT> phi_bb234_15;
  TNode<IntPtrT> phi_bb234_21;
  TNode<Object> phi_bb234_23;
  TNode<IntPtrT> phi_bb234_24;
  TNode<IntPtrT> phi_bb234_30;
  TNode<IntPtrT> phi_bb234_31;
  TNode<IntPtrT> phi_bb234_35;
  TNode<IntPtrT> phi_bb234_36;
  if (block234.is_used()) {
    ca_.Bind(&block234, &phi_bb234_8, &phi_bb234_9, &phi_bb234_10, &phi_bb234_11, &phi_bb234_14, &phi_bb234_15, &phi_bb234_21, &phi_bb234_23, &phi_bb234_24, &phi_bb234_30, &phi_bb234_31, &phi_bb234_35, &phi_bb234_36);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 113);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<IntPtrT> phi_bb218_7;
  TNode<IntPtrT> phi_bb218_8;
  TNode<IntPtrT> phi_bb218_9;
  TNode<IntPtrT> phi_bb218_10;
  TNode<IntPtrT> phi_bb218_11;
  TNode<IntPtrT> phi_bb218_14;
  TNode<BoolT> phi_bb218_15;
  TNode<IntPtrT> phi_bb218_21;
  TNode<IntPtrT> tmp500;
  TNode<IntPtrT> tmp501;
  TNode<IntPtrT> tmp502;
  TNode<BoolT> tmp503;
  if (block218.is_used()) {
    ca_.Bind(&block218, &phi_bb218_7, &phi_bb218_8, &phi_bb218_9, &phi_bb218_10, &phi_bb218_11, &phi_bb218_14, &phi_bb218_15, &phi_bb218_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 308);
    tmp500 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp501 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb218_7}, TNode<IntPtrT>{tmp500});
    tmp502 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp503 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb218_7}, TNode<IntPtrT>{tmp502});
    ca_.Branch(tmp503, &block238, std::vector<compiler::Node*>{phi_bb218_8, phi_bb218_9, phi_bb218_10, phi_bb218_11, phi_bb218_14, phi_bb218_15, phi_bb218_21}, &block239, std::vector<compiler::Node*>{phi_bb218_8, phi_bb218_9, phi_bb218_10, phi_bb218_11, phi_bb218_14, phi_bb218_15, phi_bb218_21});
  }

  TNode<IntPtrT> phi_bb238_8;
  TNode<IntPtrT> phi_bb238_9;
  TNode<IntPtrT> phi_bb238_10;
  TNode<IntPtrT> phi_bb238_11;
  TNode<IntPtrT> phi_bb238_14;
  TNode<BoolT> phi_bb238_15;
  TNode<IntPtrT> phi_bb238_21;
  TNode<Object> tmp504;
  TNode<IntPtrT> tmp505;
  TNode<IntPtrT> tmp506;
  TNode<IntPtrT> tmp507;
  if (block238.is_used()) {
    ca_.Bind(&block238, &phi_bb238_8, &phi_bb238_9, &phi_bb238_10, &phi_bb238_11, &phi_bb238_14, &phi_bb238_15, &phi_bb238_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    std::tie(tmp504, tmp505) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb238_9}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 311);
    tmp506 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp507 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb238_9}, TNode<IntPtrT>{tmp506});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 312);
    ca_.Goto(&block237, phi_bb238_8, tmp507, phi_bb238_10, phi_bb238_11, phi_bb238_14, phi_bb238_15, phi_bb238_21, tmp504, tmp505);
  }

  TNode<IntPtrT> phi_bb239_8;
  TNode<IntPtrT> phi_bb239_9;
  TNode<IntPtrT> phi_bb239_10;
  TNode<IntPtrT> phi_bb239_11;
  TNode<IntPtrT> phi_bb239_14;
  TNode<BoolT> phi_bb239_15;
  TNode<IntPtrT> phi_bb239_21;
  if (block239.is_used()) {
    ca_.Bind(&block239, &phi_bb239_8, &phi_bb239_9, &phi_bb239_10, &phi_bb239_11, &phi_bb239_14, &phi_bb239_15, &phi_bb239_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block241, phi_bb239_8, phi_bb239_9, phi_bb239_10, phi_bb239_11, phi_bb239_14, phi_bb239_15, phi_bb239_21);
    } else {
      ca_.Goto(&block242, phi_bb239_8, phi_bb239_9, phi_bb239_10, phi_bb239_11, phi_bb239_14, phi_bb239_15, phi_bb239_21);
    }
  }

  TNode<IntPtrT> phi_bb241_8;
  TNode<IntPtrT> phi_bb241_9;
  TNode<IntPtrT> phi_bb241_10;
  TNode<IntPtrT> phi_bb241_11;
  TNode<IntPtrT> phi_bb241_14;
  TNode<BoolT> phi_bb241_15;
  TNode<IntPtrT> phi_bb241_21;
  TNode<Object> tmp508;
  TNode<IntPtrT> tmp509;
  TNode<IntPtrT> tmp510;
  TNode<IntPtrT> tmp511;
  if (block241.is_used()) {
    ca_.Bind(&block241, &phi_bb241_8, &phi_bb241_9, &phi_bb241_10, &phi_bb241_11, &phi_bb241_14, &phi_bb241_15, &phi_bb241_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp508, tmp509) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb241_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp510 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp511 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb241_11}, TNode<IntPtrT>{tmp510});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block240, phi_bb241_8, phi_bb241_9, phi_bb241_10, tmp511, phi_bb241_14, phi_bb241_15, phi_bb241_21, tmp508, tmp509);
  }

  TNode<IntPtrT> phi_bb242_8;
  TNode<IntPtrT> phi_bb242_9;
  TNode<IntPtrT> phi_bb242_10;
  TNode<IntPtrT> phi_bb242_11;
  TNode<IntPtrT> phi_bb242_14;
  TNode<BoolT> phi_bb242_15;
  TNode<IntPtrT> phi_bb242_21;
  TNode<IntPtrT> tmp512;
  TNode<BoolT> tmp513;
  if (block242.is_used()) {
    ca_.Bind(&block242, &phi_bb242_8, &phi_bb242_9, &phi_bb242_10, &phi_bb242_11, &phi_bb242_14, &phi_bb242_15, &phi_bb242_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp512 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp513 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb242_14}, TNode<IntPtrT>{tmp512});
    ca_.Branch(tmp513, &block244, std::vector<compiler::Node*>{phi_bb242_8, phi_bb242_9, phi_bb242_10, phi_bb242_11, phi_bb242_14, phi_bb242_15, phi_bb242_21}, &block245, std::vector<compiler::Node*>{phi_bb242_8, phi_bb242_9, phi_bb242_10, phi_bb242_11, phi_bb242_14, phi_bb242_15, phi_bb242_21});
  }

  TNode<IntPtrT> phi_bb244_8;
  TNode<IntPtrT> phi_bb244_9;
  TNode<IntPtrT> phi_bb244_10;
  TNode<IntPtrT> phi_bb244_11;
  TNode<IntPtrT> phi_bb244_14;
  TNode<BoolT> phi_bb244_15;
  TNode<IntPtrT> phi_bb244_21;
  TNode<Object> tmp514;
  TNode<IntPtrT> tmp515;
  TNode<IntPtrT> tmp516;
  TNode<BoolT> tmp517;
  if (block244.is_used()) {
    ca_.Bind(&block244, &phi_bb244_8, &phi_bb244_9, &phi_bb244_10, &phi_bb244_11, &phi_bb244_14, &phi_bb244_15, &phi_bb244_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp514, tmp515) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb244_14}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp516 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp517 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block240, phi_bb244_8, phi_bb244_9, phi_bb244_10, phi_bb244_11, tmp516, tmp517, phi_bb244_21, tmp514, tmp515);
  }

  TNode<IntPtrT> phi_bb245_8;
  TNode<IntPtrT> phi_bb245_9;
  TNode<IntPtrT> phi_bb245_10;
  TNode<IntPtrT> phi_bb245_11;
  TNode<IntPtrT> phi_bb245_14;
  TNode<BoolT> phi_bb245_15;
  TNode<IntPtrT> phi_bb245_21;
  TNode<Object> tmp518;
  TNode<IntPtrT> tmp519;
  TNode<IntPtrT> tmp520;
  TNode<IntPtrT> tmp521;
  TNode<IntPtrT> tmp522;
  TNode<IntPtrT> tmp523;
  TNode<BoolT> tmp524;
  if (block245.is_used()) {
    ca_.Bind(&block245, &phi_bb245_8, &phi_bb245_9, &phi_bb245_10, &phi_bb245_11, &phi_bb245_14, &phi_bb245_15, &phi_bb245_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp518, tmp519) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb245_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp520 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp521 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb245_11}, TNode<IntPtrT>{tmp520});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp522 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp523 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp521}, TNode<IntPtrT>{tmp522});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp524 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block240, phi_bb245_8, phi_bb245_9, phi_bb245_10, tmp523, tmp521, tmp524, phi_bb245_21, tmp518, tmp519);
  }

  TNode<IntPtrT> phi_bb240_8;
  TNode<IntPtrT> phi_bb240_9;
  TNode<IntPtrT> phi_bb240_10;
  TNode<IntPtrT> phi_bb240_11;
  TNode<IntPtrT> phi_bb240_14;
  TNode<BoolT> phi_bb240_15;
  TNode<IntPtrT> phi_bb240_21;
  TNode<Object> phi_bb240_23;
  TNode<IntPtrT> phi_bb240_24;
  if (block240.is_used()) {
    ca_.Bind(&block240, &phi_bb240_8, &phi_bb240_9, &phi_bb240_10, &phi_bb240_11, &phi_bb240_14, &phi_bb240_15, &phi_bb240_21, &phi_bb240_23, &phi_bb240_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 314);
    ca_.Goto(&block237, phi_bb240_8, phi_bb240_9, phi_bb240_10, phi_bb240_11, phi_bb240_14, phi_bb240_15, phi_bb240_21, phi_bb240_23, phi_bb240_24);
  }

  TNode<IntPtrT> phi_bb237_8;
  TNode<IntPtrT> phi_bb237_9;
  TNode<IntPtrT> phi_bb237_10;
  TNode<IntPtrT> phi_bb237_11;
  TNode<IntPtrT> phi_bb237_14;
  TNode<BoolT> phi_bb237_15;
  TNode<IntPtrT> phi_bb237_21;
  TNode<Object> phi_bb237_23;
  TNode<IntPtrT> phi_bb237_24;
  TNode<IntPtrT> tmp525;
  TNode<IntPtrT> tmp526;
  TNode<IntPtrT> tmp527;
  TNode<BoolT> tmp528;
  if (block237.is_used()) {
    ca_.Bind(&block237, &phi_bb237_8, &phi_bb237_9, &phi_bb237_10, &phi_bb237_11, &phi_bb237_14, &phi_bb237_15, &phi_bb237_21, &phi_bb237_23, &phi_bb237_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 308);
    tmp525 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp526 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{tmp501}, TNode<IntPtrT>{tmp525});
    tmp527 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp528 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{tmp501}, TNode<IntPtrT>{tmp527});
    ca_.Branch(tmp528, &block247, std::vector<compiler::Node*>{phi_bb237_8, phi_bb237_9, phi_bb237_10, phi_bb237_11, phi_bb237_14, phi_bb237_15, phi_bb237_21, phi_bb237_23, phi_bb237_24}, &block248, std::vector<compiler::Node*>{phi_bb237_8, phi_bb237_9, phi_bb237_10, phi_bb237_11, phi_bb237_14, phi_bb237_15, phi_bb237_21, phi_bb237_23, phi_bb237_24});
  }

  TNode<IntPtrT> phi_bb247_8;
  TNode<IntPtrT> phi_bb247_9;
  TNode<IntPtrT> phi_bb247_10;
  TNode<IntPtrT> phi_bb247_11;
  TNode<IntPtrT> phi_bb247_14;
  TNode<BoolT> phi_bb247_15;
  TNode<IntPtrT> phi_bb247_21;
  TNode<Object> phi_bb247_23;
  TNode<IntPtrT> phi_bb247_24;
  TNode<Object> tmp529;
  TNode<IntPtrT> tmp530;
  TNode<IntPtrT> tmp531;
  TNode<IntPtrT> tmp532;
  if (block247.is_used()) {
    ca_.Bind(&block247, &phi_bb247_8, &phi_bb247_9, &phi_bb247_10, &phi_bb247_11, &phi_bb247_14, &phi_bb247_15, &phi_bb247_21, &phi_bb247_23, &phi_bb247_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    std::tie(tmp529, tmp530) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb247_9}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 311);
    tmp531 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp532 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb247_9}, TNode<IntPtrT>{tmp531});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 312);
    ca_.Goto(&block246, phi_bb247_8, tmp532, phi_bb247_10, phi_bb247_11, phi_bb247_14, phi_bb247_15, phi_bb247_21, phi_bb247_23, phi_bb247_24, tmp529, tmp530);
  }

  TNode<IntPtrT> phi_bb248_8;
  TNode<IntPtrT> phi_bb248_9;
  TNode<IntPtrT> phi_bb248_10;
  TNode<IntPtrT> phi_bb248_11;
  TNode<IntPtrT> phi_bb248_14;
  TNode<BoolT> phi_bb248_15;
  TNode<IntPtrT> phi_bb248_21;
  TNode<Object> phi_bb248_23;
  TNode<IntPtrT> phi_bb248_24;
  if (block248.is_used()) {
    ca_.Bind(&block248, &phi_bb248_8, &phi_bb248_9, &phi_bb248_10, &phi_bb248_11, &phi_bb248_14, &phi_bb248_15, &phi_bb248_21, &phi_bb248_23, &phi_bb248_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block250, phi_bb248_8, phi_bb248_9, phi_bb248_10, phi_bb248_11, phi_bb248_14, phi_bb248_15, phi_bb248_21, phi_bb248_23, phi_bb248_24);
    } else {
      ca_.Goto(&block251, phi_bb248_8, phi_bb248_9, phi_bb248_10, phi_bb248_11, phi_bb248_14, phi_bb248_15, phi_bb248_21, phi_bb248_23, phi_bb248_24);
    }
  }

  TNode<IntPtrT> phi_bb250_8;
  TNode<IntPtrT> phi_bb250_9;
  TNode<IntPtrT> phi_bb250_10;
  TNode<IntPtrT> phi_bb250_11;
  TNode<IntPtrT> phi_bb250_14;
  TNode<BoolT> phi_bb250_15;
  TNode<IntPtrT> phi_bb250_21;
  TNode<Object> phi_bb250_23;
  TNode<IntPtrT> phi_bb250_24;
  TNode<Object> tmp533;
  TNode<IntPtrT> tmp534;
  TNode<IntPtrT> tmp535;
  TNode<IntPtrT> tmp536;
  if (block250.is_used()) {
    ca_.Bind(&block250, &phi_bb250_8, &phi_bb250_9, &phi_bb250_10, &phi_bb250_11, &phi_bb250_14, &phi_bb250_15, &phi_bb250_21, &phi_bb250_23, &phi_bb250_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp533, tmp534) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb250_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp535 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp536 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb250_11}, TNode<IntPtrT>{tmp535});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block249, phi_bb250_8, phi_bb250_9, phi_bb250_10, tmp536, phi_bb250_14, phi_bb250_15, phi_bb250_21, phi_bb250_23, phi_bb250_24, tmp533, tmp534);
  }

  TNode<IntPtrT> phi_bb251_8;
  TNode<IntPtrT> phi_bb251_9;
  TNode<IntPtrT> phi_bb251_10;
  TNode<IntPtrT> phi_bb251_11;
  TNode<IntPtrT> phi_bb251_14;
  TNode<BoolT> phi_bb251_15;
  TNode<IntPtrT> phi_bb251_21;
  TNode<Object> phi_bb251_23;
  TNode<IntPtrT> phi_bb251_24;
  TNode<IntPtrT> tmp537;
  TNode<BoolT> tmp538;
  if (block251.is_used()) {
    ca_.Bind(&block251, &phi_bb251_8, &phi_bb251_9, &phi_bb251_10, &phi_bb251_11, &phi_bb251_14, &phi_bb251_15, &phi_bb251_21, &phi_bb251_23, &phi_bb251_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp537 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp538 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb251_14}, TNode<IntPtrT>{tmp537});
    ca_.Branch(tmp538, &block253, std::vector<compiler::Node*>{phi_bb251_8, phi_bb251_9, phi_bb251_10, phi_bb251_11, phi_bb251_14, phi_bb251_15, phi_bb251_21, phi_bb251_23, phi_bb251_24}, &block254, std::vector<compiler::Node*>{phi_bb251_8, phi_bb251_9, phi_bb251_10, phi_bb251_11, phi_bb251_14, phi_bb251_15, phi_bb251_21, phi_bb251_23, phi_bb251_24});
  }

  TNode<IntPtrT> phi_bb253_8;
  TNode<IntPtrT> phi_bb253_9;
  TNode<IntPtrT> phi_bb253_10;
  TNode<IntPtrT> phi_bb253_11;
  TNode<IntPtrT> phi_bb253_14;
  TNode<BoolT> phi_bb253_15;
  TNode<IntPtrT> phi_bb253_21;
  TNode<Object> phi_bb253_23;
  TNode<IntPtrT> phi_bb253_24;
  TNode<Object> tmp539;
  TNode<IntPtrT> tmp540;
  TNode<IntPtrT> tmp541;
  TNode<BoolT> tmp542;
  if (block253.is_used()) {
    ca_.Bind(&block253, &phi_bb253_8, &phi_bb253_9, &phi_bb253_10, &phi_bb253_11, &phi_bb253_14, &phi_bb253_15, &phi_bb253_21, &phi_bb253_23, &phi_bb253_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp539, tmp540) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb253_14}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp541 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp542 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block249, phi_bb253_8, phi_bb253_9, phi_bb253_10, phi_bb253_11, tmp541, tmp542, phi_bb253_21, phi_bb253_23, phi_bb253_24, tmp539, tmp540);
  }

  TNode<IntPtrT> phi_bb254_8;
  TNode<IntPtrT> phi_bb254_9;
  TNode<IntPtrT> phi_bb254_10;
  TNode<IntPtrT> phi_bb254_11;
  TNode<IntPtrT> phi_bb254_14;
  TNode<BoolT> phi_bb254_15;
  TNode<IntPtrT> phi_bb254_21;
  TNode<Object> phi_bb254_23;
  TNode<IntPtrT> phi_bb254_24;
  TNode<Object> tmp543;
  TNode<IntPtrT> tmp544;
  TNode<IntPtrT> tmp545;
  TNode<IntPtrT> tmp546;
  TNode<IntPtrT> tmp547;
  TNode<IntPtrT> tmp548;
  TNode<BoolT> tmp549;
  if (block254.is_used()) {
    ca_.Bind(&block254, &phi_bb254_8, &phi_bb254_9, &phi_bb254_10, &phi_bb254_11, &phi_bb254_14, &phi_bb254_15, &phi_bb254_21, &phi_bb254_23, &phi_bb254_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp543, tmp544) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb254_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp545 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp546 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb254_11}, TNode<IntPtrT>{tmp545});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp547 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp548 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp546}, TNode<IntPtrT>{tmp547});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp549 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block249, phi_bb254_8, phi_bb254_9, phi_bb254_10, tmp548, tmp546, tmp549, phi_bb254_21, phi_bb254_23, phi_bb254_24, tmp543, tmp544);
  }

  TNode<IntPtrT> phi_bb249_8;
  TNode<IntPtrT> phi_bb249_9;
  TNode<IntPtrT> phi_bb249_10;
  TNode<IntPtrT> phi_bb249_11;
  TNode<IntPtrT> phi_bb249_14;
  TNode<BoolT> phi_bb249_15;
  TNode<IntPtrT> phi_bb249_21;
  TNode<Object> phi_bb249_23;
  TNode<IntPtrT> phi_bb249_24;
  TNode<Object> phi_bb249_25;
  TNode<IntPtrT> phi_bb249_26;
  if (block249.is_used()) {
    ca_.Bind(&block249, &phi_bb249_8, &phi_bb249_9, &phi_bb249_10, &phi_bb249_11, &phi_bb249_14, &phi_bb249_15, &phi_bb249_21, &phi_bb249_23, &phi_bb249_24, &phi_bb249_25, &phi_bb249_26);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 314);
    ca_.Goto(&block246, phi_bb249_8, phi_bb249_9, phi_bb249_10, phi_bb249_11, phi_bb249_14, phi_bb249_15, phi_bb249_21, phi_bb249_23, phi_bb249_24, phi_bb249_25, phi_bb249_26);
  }

  TNode<IntPtrT> phi_bb246_8;
  TNode<IntPtrT> phi_bb246_9;
  TNode<IntPtrT> phi_bb246_10;
  TNode<IntPtrT> phi_bb246_11;
  TNode<IntPtrT> phi_bb246_14;
  TNode<BoolT> phi_bb246_15;
  TNode<IntPtrT> phi_bb246_21;
  TNode<Object> phi_bb246_23;
  TNode<IntPtrT> phi_bb246_24;
  TNode<Object> phi_bb246_25;
  TNode<IntPtrT> phi_bb246_26;
  TNode<IntPtrT> tmp550;
  TNode<IntPtrT> tmp551;
  TNode<Object> tmp552;
  TNode<IntPtrT> tmp553;
  TNode<IntPtrT> tmp554;
  TNode<UintPtrT> tmp555;
  TNode<UintPtrT> tmp556;
  TNode<BoolT> tmp557;
  if (block246.is_used()) {
    ca_.Bind(&block246, &phi_bb246_8, &phi_bb246_9, &phi_bb246_10, &phi_bb246_11, &phi_bb246_14, &phi_bb246_15, &phi_bb246_21, &phi_bb246_23, &phi_bb246_24, &phi_bb246_25, &phi_bb246_26);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1061);
    tmp550 = CodeStubAssembler(state_).LoadReference<IntPtrT>(CodeStubAssembler::Reference{phi_bb246_23, phi_bb246_24});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1062);
    tmp551 = CodeStubAssembler(state_).LoadReference<IntPtrT>(CodeStubAssembler::Reference{phi_bb246_25, phi_bb246_26});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1063);
    std::tie(tmp552, tmp553, tmp554) = FieldSliceFixedArrayObjects_0(state_, TNode<FixedArray>{tmp92}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 101);
    tmp555 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{phi_bb246_21});
    tmp556 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{tmp554});
    tmp557 = CodeStubAssembler(state_).UintPtrLessThan(TNode<UintPtrT>{tmp555}, TNode<UintPtrT>{tmp556});
    ca_.Branch(tmp557, &block259, std::vector<compiler::Node*>{phi_bb246_8, phi_bb246_9, phi_bb246_10, phi_bb246_11, phi_bb246_14, phi_bb246_15, phi_bb246_21, phi_bb246_23, phi_bb246_24, phi_bb246_25, phi_bb246_26, phi_bb246_21, phi_bb246_21, phi_bb246_21, phi_bb246_21}, &block260, std::vector<compiler::Node*>{phi_bb246_8, phi_bb246_9, phi_bb246_10, phi_bb246_11, phi_bb246_14, phi_bb246_15, phi_bb246_21, phi_bb246_23, phi_bb246_24, phi_bb246_25, phi_bb246_26, phi_bb246_21, phi_bb246_21, phi_bb246_21, phi_bb246_21});
  }

  TNode<IntPtrT> phi_bb259_8;
  TNode<IntPtrT> phi_bb259_9;
  TNode<IntPtrT> phi_bb259_10;
  TNode<IntPtrT> phi_bb259_11;
  TNode<IntPtrT> phi_bb259_14;
  TNode<BoolT> phi_bb259_15;
  TNode<IntPtrT> phi_bb259_21;
  TNode<Object> phi_bb259_23;
  TNode<IntPtrT> phi_bb259_24;
  TNode<Object> phi_bb259_25;
  TNode<IntPtrT> phi_bb259_26;
  TNode<IntPtrT> phi_bb259_33;
  TNode<IntPtrT> phi_bb259_34;
  TNode<IntPtrT> phi_bb259_38;
  TNode<IntPtrT> phi_bb259_39;
  TNode<IntPtrT> tmp558;
  TNode<IntPtrT> tmp559;
  TNode<Object> tmp560;
  TNode<IntPtrT> tmp561;
  TNode<BigInt> tmp562;
  if (block259.is_used()) {
    ca_.Bind(&block259, &phi_bb259_8, &phi_bb259_9, &phi_bb259_10, &phi_bb259_11, &phi_bb259_14, &phi_bb259_15, &phi_bb259_21, &phi_bb259_23, &phi_bb259_24, &phi_bb259_25, &phi_bb259_26, &phi_bb259_33, &phi_bb259_34, &phi_bb259_38, &phi_bb259_39);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp558 = TimesSizeOf_Object_0(state_, TNode<IntPtrT>{phi_bb259_39});
    tmp559 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp553}, TNode<IntPtrT>{tmp558});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp560, tmp561) = NewReference_Object_0(state_, TNode<Object>{tmp552}, TNode<IntPtrT>{tmp559}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1063);
    tmp562 = ca_.CallBuiltin<BigInt>(Builtin::kI32PairToBigInt, TNode<Object>(), tmp550, tmp551);
    CodeStubAssembler(state_).StoreReference<Object>(CodeStubAssembler::Reference{tmp560, tmp561}, tmp562);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1054);
    ca_.Goto(&block219, tmp526, phi_bb259_8, phi_bb259_9, phi_bb259_10, phi_bb259_11, phi_bb259_14, phi_bb259_15, phi_bb259_21);
  }

  TNode<IntPtrT> phi_bb260_8;
  TNode<IntPtrT> phi_bb260_9;
  TNode<IntPtrT> phi_bb260_10;
  TNode<IntPtrT> phi_bb260_11;
  TNode<IntPtrT> phi_bb260_14;
  TNode<BoolT> phi_bb260_15;
  TNode<IntPtrT> phi_bb260_21;
  TNode<Object> phi_bb260_23;
  TNode<IntPtrT> phi_bb260_24;
  TNode<Object> phi_bb260_25;
  TNode<IntPtrT> phi_bb260_26;
  TNode<IntPtrT> phi_bb260_33;
  TNode<IntPtrT> phi_bb260_34;
  TNode<IntPtrT> phi_bb260_38;
  TNode<IntPtrT> phi_bb260_39;
  if (block260.is_used()) {
    ca_.Bind(&block260, &phi_bb260_8, &phi_bb260_9, &phi_bb260_10, &phi_bb260_11, &phi_bb260_14, &phi_bb260_15, &phi_bb260_21, &phi_bb260_23, &phi_bb260_24, &phi_bb260_25, &phi_bb260_26, &phi_bb260_33, &phi_bb260_34, &phi_bb260_38, &phi_bb260_39);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 113);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<IntPtrT> phi_bb219_7;
  TNode<IntPtrT> phi_bb219_8;
  TNode<IntPtrT> phi_bb219_9;
  TNode<IntPtrT> phi_bb219_10;
  TNode<IntPtrT> phi_bb219_11;
  TNode<IntPtrT> phi_bb219_14;
  TNode<BoolT> phi_bb219_15;
  TNode<IntPtrT> phi_bb219_21;
  if (block219.is_used()) {
    ca_.Bind(&block219, &phi_bb219_7, &phi_bb219_8, &phi_bb219_9, &phi_bb219_10, &phi_bb219_11, &phi_bb219_14, &phi_bb219_15, &phi_bb219_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1053);
    ca_.Goto(&block216, phi_bb219_7, phi_bb219_8, phi_bb219_9, phi_bb219_10, phi_bb219_11, phi_bb219_14, phi_bb219_15, phi_bb219_21);
  }

  TNode<IntPtrT> phi_bb215_7;
  TNode<IntPtrT> phi_bb215_8;
  TNode<IntPtrT> phi_bb215_9;
  TNode<IntPtrT> phi_bb215_10;
  TNode<IntPtrT> phi_bb215_11;
  TNode<IntPtrT> phi_bb215_14;
  TNode<BoolT> phi_bb215_15;
  TNode<IntPtrT> phi_bb215_21;
  TNode<Int32T> tmp563;
  TNode<BoolT> tmp564;
  if (block215.is_used()) {
    ca_.Bind(&block215, &phi_bb215_7, &phi_bb215_8, &phi_bb215_9, &phi_bb215_10, &phi_bb215_11, &phi_bb215_14, &phi_bb215_15, &phi_bb215_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1065);
    tmp563 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmF64.raw_bit_field());
    tmp564 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp353}, TNode<Int32T>{tmp563});
    ca_.Branch(tmp564, &block263, std::vector<compiler::Node*>{phi_bb215_7, phi_bb215_8, phi_bb215_9, phi_bb215_10, phi_bb215_11, phi_bb215_14, phi_bb215_15, phi_bb215_21}, &block264, std::vector<compiler::Node*>{phi_bb215_7, phi_bb215_8, phi_bb215_9, phi_bb215_10, phi_bb215_11, phi_bb215_14, phi_bb215_15, phi_bb215_21});
  }

  TNode<IntPtrT> phi_bb263_7;
  TNode<IntPtrT> phi_bb263_8;
  TNode<IntPtrT> phi_bb263_9;
  TNode<IntPtrT> phi_bb263_10;
  TNode<IntPtrT> phi_bb263_11;
  TNode<IntPtrT> phi_bb263_14;
  TNode<BoolT> phi_bb263_15;
  TNode<IntPtrT> phi_bb263_21;
  TNode<IntPtrT> tmp565;
  TNode<IntPtrT> tmp566;
  TNode<IntPtrT> tmp567;
  TNode<BoolT> tmp568;
  if (block263.is_used()) {
    ca_.Bind(&block263, &phi_bb263_7, &phi_bb263_8, &phi_bb263_9, &phi_bb263_10, &phi_bb263_11, &phi_bb263_14, &phi_bb263_15, &phi_bb263_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 332);
    tmp565 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp566 = CodeStubAssembler(state_).IntPtrSub(TNode<IntPtrT>{phi_bb263_8}, TNode<IntPtrT>{tmp565});
    tmp567 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp568 = CodeStubAssembler(state_).IntPtrGreaterThan(TNode<IntPtrT>{phi_bb263_8}, TNode<IntPtrT>{tmp567});
    ca_.Branch(tmp568, &block267, std::vector<compiler::Node*>{phi_bb263_7, phi_bb263_9, phi_bb263_10, phi_bb263_11, phi_bb263_14, phi_bb263_15, phi_bb263_21}, &block268, std::vector<compiler::Node*>{phi_bb263_7, phi_bb263_9, phi_bb263_10, phi_bb263_11, phi_bb263_14, phi_bb263_15, phi_bb263_21});
  }

  TNode<IntPtrT> phi_bb267_7;
  TNode<IntPtrT> phi_bb267_9;
  TNode<IntPtrT> phi_bb267_10;
  TNode<IntPtrT> phi_bb267_11;
  TNode<IntPtrT> phi_bb267_14;
  TNode<BoolT> phi_bb267_15;
  TNode<IntPtrT> phi_bb267_21;
  TNode<Object> tmp569;
  TNode<IntPtrT> tmp570;
  TNode<IntPtrT> tmp571;
  TNode<IntPtrT> tmp572;
  if (block267.is_used()) {
    ca_.Bind(&block267, &phi_bb267_7, &phi_bb267_9, &phi_bb267_10, &phi_bb267_11, &phi_bb267_14, &phi_bb267_15, &phi_bb267_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 333);
    std::tie(tmp569, tmp570) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb267_10}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 335);
    tmp571 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_float64_0(state_)));
    tmp572 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb267_10}, TNode<IntPtrT>{tmp571});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 336);
    ca_.Goto(&block266, phi_bb267_7, phi_bb267_9, tmp572, phi_bb267_11, phi_bb267_14, phi_bb267_15, phi_bb267_21, tmp569, tmp570);
  }

  TNode<IntPtrT> phi_bb268_7;
  TNode<IntPtrT> phi_bb268_9;
  TNode<IntPtrT> phi_bb268_10;
  TNode<IntPtrT> phi_bb268_11;
  TNode<IntPtrT> phi_bb268_14;
  TNode<BoolT> phi_bb268_15;
  TNode<IntPtrT> phi_bb268_21;
  if (block268.is_used()) {
    ca_.Bind(&block268, &phi_bb268_7, &phi_bb268_9, &phi_bb268_10, &phi_bb268_11, &phi_bb268_14, &phi_bb268_15, &phi_bb268_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 338);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block269, phi_bb268_7, phi_bb268_9, phi_bb268_10, phi_bb268_11, phi_bb268_14, phi_bb268_15, phi_bb268_21);
    } else {
      ca_.Goto(&block270, phi_bb268_7, phi_bb268_9, phi_bb268_10, phi_bb268_11, phi_bb268_14, phi_bb268_15, phi_bb268_21);
    }
  }

  TNode<IntPtrT> phi_bb269_7;
  TNode<IntPtrT> phi_bb269_9;
  TNode<IntPtrT> phi_bb269_10;
  TNode<IntPtrT> phi_bb269_11;
  TNode<IntPtrT> phi_bb269_14;
  TNode<BoolT> phi_bb269_15;
  TNode<IntPtrT> phi_bb269_21;
  if (block269.is_used()) {
    ca_.Bind(&block269, &phi_bb269_7, &phi_bb269_9, &phi_bb269_10, &phi_bb269_11, &phi_bb269_14, &phi_bb269_15, &phi_bb269_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 285);
    if (((CodeStubAssembler(state_).Is64()))) {
      ca_.Goto(&block273, phi_bb269_7, phi_bb269_9, phi_bb269_10, phi_bb269_11, phi_bb269_14, phi_bb269_15, phi_bb269_21);
    } else {
      ca_.Goto(&block274, phi_bb269_7, phi_bb269_9, phi_bb269_10, phi_bb269_11, phi_bb269_14, phi_bb269_15, phi_bb269_21);
    }
  }

  TNode<IntPtrT> phi_bb273_7;
  TNode<IntPtrT> phi_bb273_9;
  TNode<IntPtrT> phi_bb273_10;
  TNode<IntPtrT> phi_bb273_11;
  TNode<IntPtrT> phi_bb273_14;
  TNode<BoolT> phi_bb273_15;
  TNode<IntPtrT> phi_bb273_21;
  TNode<Object> tmp573;
  TNode<IntPtrT> tmp574;
  TNode<IntPtrT> tmp575;
  TNode<IntPtrT> tmp576;
  if (block273.is_used()) {
    ca_.Bind(&block273, &phi_bb273_7, &phi_bb273_9, &phi_bb273_10, &phi_bb273_11, &phi_bb273_14, &phi_bb273_15, &phi_bb273_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 286);
    std::tie(tmp573, tmp574) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb273_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 288);
    tmp575 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp576 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb273_11}, TNode<IntPtrT>{tmp575});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 289);
    ca_.Goto(&block272, phi_bb273_7, phi_bb273_9, phi_bb273_10, tmp576, phi_bb273_14, phi_bb273_15, phi_bb273_21, tmp573, tmp574);
  }

  TNode<IntPtrT> phi_bb274_7;
  TNode<IntPtrT> phi_bb274_9;
  TNode<IntPtrT> phi_bb274_10;
  TNode<IntPtrT> phi_bb274_11;
  TNode<IntPtrT> phi_bb274_14;
  TNode<BoolT> phi_bb274_15;
  TNode<IntPtrT> phi_bb274_21;
  TNode<IntPtrT> tmp577;
  TNode<BoolT> tmp578;
  if (block274.is_used()) {
    ca_.Bind(&block274, &phi_bb274_7, &phi_bb274_9, &phi_bb274_10, &phi_bb274_11, &phi_bb274_14, &phi_bb274_15, &phi_bb274_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 291);
    tmp577 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp578 = CodeStubAssembler(state_).WordNotEqual(TNode<IntPtrT>{phi_bb274_14}, TNode<IntPtrT>{tmp577});
    ca_.Branch(tmp578, &block276, std::vector<compiler::Node*>{phi_bb274_7, phi_bb274_9, phi_bb274_10, phi_bb274_11, phi_bb274_14, phi_bb274_15, phi_bb274_21}, &block277, std::vector<compiler::Node*>{phi_bb274_7, phi_bb274_9, phi_bb274_10, phi_bb274_11, phi_bb274_14, phi_bb274_15, phi_bb274_21});
  }

  TNode<IntPtrT> phi_bb276_7;
  TNode<IntPtrT> phi_bb276_9;
  TNode<IntPtrT> phi_bb276_10;
  TNode<IntPtrT> phi_bb276_11;
  TNode<IntPtrT> phi_bb276_14;
  TNode<BoolT> phi_bb276_15;
  TNode<IntPtrT> phi_bb276_21;
  TNode<Object> tmp579;
  TNode<IntPtrT> tmp580;
  TNode<IntPtrT> tmp581;
  TNode<BoolT> tmp582;
  if (block276.is_used()) {
    ca_.Bind(&block276, &phi_bb276_7, &phi_bb276_9, &phi_bb276_10, &phi_bb276_11, &phi_bb276_14, &phi_bb276_15, &phi_bb276_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 292);
    std::tie(tmp579, tmp580) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb276_14}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 294);
    tmp581 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 295);
    tmp582 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 296);
    ca_.Goto(&block272, phi_bb276_7, phi_bb276_9, phi_bb276_10, phi_bb276_11, tmp581, tmp582, phi_bb276_21, tmp579, tmp580);
  }

  TNode<IntPtrT> phi_bb277_7;
  TNode<IntPtrT> phi_bb277_9;
  TNode<IntPtrT> phi_bb277_10;
  TNode<IntPtrT> phi_bb277_11;
  TNode<IntPtrT> phi_bb277_14;
  TNode<BoolT> phi_bb277_15;
  TNode<IntPtrT> phi_bb277_21;
  TNode<Object> tmp583;
  TNode<IntPtrT> tmp584;
  TNode<IntPtrT> tmp585;
  TNode<IntPtrT> tmp586;
  TNode<IntPtrT> tmp587;
  TNode<IntPtrT> tmp588;
  TNode<BoolT> tmp589;
  if (block277.is_used()) {
    ca_.Bind(&block277, &phi_bb277_7, &phi_bb277_9, &phi_bb277_10, &phi_bb277_11, &phi_bb277_14, &phi_bb277_15, &phi_bb277_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 298);
    std::tie(tmp583, tmp584) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb277_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 300);
    tmp585 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp586 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb277_11}, TNode<IntPtrT>{tmp585});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 301);
    tmp587 = FromConstexpr_intptr_constexpr_int31_0(state_, (SizeOf_intptr_0(state_)));
    tmp588 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp586}, TNode<IntPtrT>{tmp587});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 302);
    tmp589 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 303);
    ca_.Goto(&block272, phi_bb277_7, phi_bb277_9, phi_bb277_10, tmp588, tmp586, tmp589, phi_bb277_21, tmp583, tmp584);
  }

  TNode<IntPtrT> phi_bb272_7;
  TNode<IntPtrT> phi_bb272_9;
  TNode<IntPtrT> phi_bb272_10;
  TNode<IntPtrT> phi_bb272_11;
  TNode<IntPtrT> phi_bb272_14;
  TNode<BoolT> phi_bb272_15;
  TNode<IntPtrT> phi_bb272_21;
  TNode<Object> phi_bb272_23;
  TNode<IntPtrT> phi_bb272_24;
  if (block272.is_used()) {
    ca_.Bind(&block272, &phi_bb272_7, &phi_bb272_9, &phi_bb272_10, &phi_bb272_11, &phi_bb272_14, &phi_bb272_15, &phi_bb272_21, &phi_bb272_23, &phi_bb272_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 339);
    ca_.Goto(&block266, phi_bb272_7, phi_bb272_9, phi_bb272_10, phi_bb272_11, phi_bb272_14, phi_bb272_15, phi_bb272_21, phi_bb272_23, phi_bb272_24);
  }

  TNode<IntPtrT> phi_bb270_7;
  TNode<IntPtrT> phi_bb270_9;
  TNode<IntPtrT> phi_bb270_10;
  TNode<IntPtrT> phi_bb270_11;
  TNode<IntPtrT> phi_bb270_14;
  TNode<BoolT> phi_bb270_15;
  TNode<IntPtrT> phi_bb270_21;
  TNode<Object> tmp590;
  TNode<IntPtrT> tmp591;
  TNode<IntPtrT> tmp592;
  TNode<IntPtrT> tmp593;
  TNode<BoolT> tmp594;
  if (block270.is_used()) {
    ca_.Bind(&block270, &phi_bb270_7, &phi_bb270_9, &phi_bb270_10, &phi_bb270_11, &phi_bb270_14, &phi_bb270_15, &phi_bb270_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 341);
    std::tie(tmp590, tmp591) = NewReference_intptr_0(state_, TNode<Object>{tmp336}, TNode<IntPtrT>{phi_bb270_11}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 343);
    tmp592 = FromConstexpr_intptr_constexpr_int31_0(state_, (CodeStubAssembler(state_).ConstexprInt31Mul((FromConstexpr_constexpr_int31_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x2ull))), (SizeOf_intptr_0(state_)))));
    tmp593 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb270_11}, TNode<IntPtrT>{tmp592});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 344);
    tmp594 = FromConstexpr_bool_constexpr_bool_0(state_, false);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 345);
    ca_.Goto(&block266, phi_bb270_7, phi_bb270_9, phi_bb270_10, tmp593, phi_bb270_14, tmp594, phi_bb270_21, tmp590, tmp591);
  }

  TNode<IntPtrT> phi_bb266_7;
  TNode<IntPtrT> phi_bb266_9;
  TNode<IntPtrT> phi_bb266_10;
  TNode<IntPtrT> phi_bb266_11;
  TNode<IntPtrT> phi_bb266_14;
  TNode<BoolT> phi_bb266_15;
  TNode<IntPtrT> phi_bb266_21;
  TNode<Object> phi_bb266_23;
  TNode<IntPtrT> phi_bb266_24;
  TNode<Object> tmp595;
  TNode<IntPtrT> tmp596;
  TNode<Float64T> tmp597;
  TNode<Object> tmp598;
  TNode<IntPtrT> tmp599;
  TNode<IntPtrT> tmp600;
  TNode<UintPtrT> tmp601;
  TNode<UintPtrT> tmp602;
  TNode<BoolT> tmp603;
  if (block266.is_used()) {
    ca_.Bind(&block266, &phi_bb266_7, &phi_bb266_9, &phi_bb266_10, &phi_bb266_11, &phi_bb266_14, &phi_bb266_15, &phi_bb266_21, &phi_bb266_23, &phi_bb266_24);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1067);
    std::tie(tmp595, tmp596) = RefCast_float64_0(state_, TorqueStructReference_intptr_0{TNode<Object>{phi_bb266_23}, TNode<IntPtrT>{phi_bb266_24}, TorqueStructUnsafe_0{}}).Flatten();
    tmp597 = CodeStubAssembler(state_).LoadReference<Float64T>(CodeStubAssembler::Reference{tmp595, tmp596});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1068);
    std::tie(tmp598, tmp599, tmp600) = FieldSliceFixedArrayObjects_0(state_, TNode<FixedArray>{tmp92}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 101);
    tmp601 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{phi_bb266_21});
    tmp602 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{tmp600});
    tmp603 = CodeStubAssembler(state_).UintPtrLessThan(TNode<UintPtrT>{tmp601}, TNode<UintPtrT>{tmp602});
    ca_.Branch(tmp603, &block282, std::vector<compiler::Node*>{phi_bb266_7, phi_bb266_9, phi_bb266_10, phi_bb266_11, phi_bb266_14, phi_bb266_15, phi_bb266_21, phi_bb266_23, phi_bb266_24, phi_bb266_21, phi_bb266_21, phi_bb266_21, phi_bb266_21}, &block283, std::vector<compiler::Node*>{phi_bb266_7, phi_bb266_9, phi_bb266_10, phi_bb266_11, phi_bb266_14, phi_bb266_15, phi_bb266_21, phi_bb266_23, phi_bb266_24, phi_bb266_21, phi_bb266_21, phi_bb266_21, phi_bb266_21});
  }

  TNode<IntPtrT> phi_bb282_7;
  TNode<IntPtrT> phi_bb282_9;
  TNode<IntPtrT> phi_bb282_10;
  TNode<IntPtrT> phi_bb282_11;
  TNode<IntPtrT> phi_bb282_14;
  TNode<BoolT> phi_bb282_15;
  TNode<IntPtrT> phi_bb282_21;
  TNode<Object> phi_bb282_23;
  TNode<IntPtrT> phi_bb282_24;
  TNode<IntPtrT> phi_bb282_30;
  TNode<IntPtrT> phi_bb282_31;
  TNode<IntPtrT> phi_bb282_35;
  TNode<IntPtrT> phi_bb282_36;
  TNode<IntPtrT> tmp604;
  TNode<IntPtrT> tmp605;
  TNode<Object> tmp606;
  TNode<IntPtrT> tmp607;
  TNode<Number> tmp608;
  if (block282.is_used()) {
    ca_.Bind(&block282, &phi_bb282_7, &phi_bb282_9, &phi_bb282_10, &phi_bb282_11, &phi_bb282_14, &phi_bb282_15, &phi_bb282_21, &phi_bb282_23, &phi_bb282_24, &phi_bb282_30, &phi_bb282_31, &phi_bb282_35, &phi_bb282_36);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp604 = TimesSizeOf_Object_0(state_, TNode<IntPtrT>{phi_bb282_36});
    tmp605 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp599}, TNode<IntPtrT>{tmp604});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp606, tmp607) = NewReference_Object_0(state_, TNode<Object>{tmp598}, TNode<IntPtrT>{tmp605}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1068);
    tmp608 = Convert_Number_float64_0(state_, TNode<Float64T>{tmp597});
    CodeStubAssembler(state_).StoreReference<Object>(CodeStubAssembler::Reference{tmp606, tmp607}, tmp608);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1065);
    ca_.Goto(&block265, phi_bb282_7, tmp566, phi_bb282_9, phi_bb282_10, phi_bb282_11, phi_bb282_14, phi_bb282_15, phi_bb282_21);
  }

  TNode<IntPtrT> phi_bb283_7;
  TNode<IntPtrT> phi_bb283_9;
  TNode<IntPtrT> phi_bb283_10;
  TNode<IntPtrT> phi_bb283_11;
  TNode<IntPtrT> phi_bb283_14;
  TNode<BoolT> phi_bb283_15;
  TNode<IntPtrT> phi_bb283_21;
  TNode<Object> phi_bb283_23;
  TNode<IntPtrT> phi_bb283_24;
  TNode<IntPtrT> phi_bb283_30;
  TNode<IntPtrT> phi_bb283_31;
  TNode<IntPtrT> phi_bb283_35;
  TNode<IntPtrT> phi_bb283_36;
  if (block283.is_used()) {
    ca_.Bind(&block283, &phi_bb283_7, &phi_bb283_9, &phi_bb283_10, &phi_bb283_11, &phi_bb283_14, &phi_bb283_15, &phi_bb283_21, &phi_bb283_23, &phi_bb283_24, &phi_bb283_30, &phi_bb283_31, &phi_bb283_35, &phi_bb283_36);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 113);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<IntPtrT> phi_bb264_7;
  TNode<IntPtrT> phi_bb264_8;
  TNode<IntPtrT> phi_bb264_9;
  TNode<IntPtrT> phi_bb264_10;
  TNode<IntPtrT> phi_bb264_11;
  TNode<IntPtrT> phi_bb264_14;
  TNode<BoolT> phi_bb264_15;
  TNode<IntPtrT> phi_bb264_21;
  TNode<Int32T> tmp609;
  TNode<Int32T> tmp610;
  TNode<Int32T> tmp611;
  TNode<BoolT> tmp612;
  if (block264.is_used()) {
    ca_.Bind(&block264, &phi_bb264_7, &phi_bb264_8, &phi_bb264_9, &phi_bb264_10, &phi_bb264_11, &phi_bb264_14, &phi_bb264_15, &phi_bb264_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1070);
    tmp609 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::kWasmValueKindBitsMask);
    tmp610 = CodeStubAssembler(state_).Word32And(TNode<Int32T>{tmp353}, TNode<Int32T>{tmp609});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1071);
    tmp611 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRef);
    tmp612 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp610}, TNode<Int32T>{tmp611});
    ca_.Branch(tmp612, &block288, std::vector<compiler::Node*>{phi_bb264_7, phi_bb264_8, phi_bb264_9, phi_bb264_10, phi_bb264_11, phi_bb264_14, phi_bb264_15, phi_bb264_21}, &block289, std::vector<compiler::Node*>{phi_bb264_7, phi_bb264_8, phi_bb264_9, phi_bb264_10, phi_bb264_11, phi_bb264_14, phi_bb264_15, phi_bb264_21});
  }

  TNode<IntPtrT> phi_bb288_7;
  TNode<IntPtrT> phi_bb288_8;
  TNode<IntPtrT> phi_bb288_9;
  TNode<IntPtrT> phi_bb288_10;
  TNode<IntPtrT> phi_bb288_11;
  TNode<IntPtrT> phi_bb288_14;
  TNode<BoolT> phi_bb288_15;
  TNode<IntPtrT> phi_bb288_21;
  TNode<BoolT> tmp613;
  if (block288.is_used()) {
    ca_.Bind(&block288, &phi_bb288_7, &phi_bb288_8, &phi_bb288_9, &phi_bb288_10, &phi_bb288_11, &phi_bb288_14, &phi_bb288_15, &phi_bb288_21);
    tmp613 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block290, phi_bb288_7, phi_bb288_8, phi_bb288_9, phi_bb288_10, phi_bb288_11, phi_bb288_14, phi_bb288_15, phi_bb288_21, tmp613);
  }

  TNode<IntPtrT> phi_bb289_7;
  TNode<IntPtrT> phi_bb289_8;
  TNode<IntPtrT> phi_bb289_9;
  TNode<IntPtrT> phi_bb289_10;
  TNode<IntPtrT> phi_bb289_11;
  TNode<IntPtrT> phi_bb289_14;
  TNode<BoolT> phi_bb289_15;
  TNode<IntPtrT> phi_bb289_21;
  TNode<Int32T> tmp614;
  TNode<BoolT> tmp615;
  if (block289.is_used()) {
    ca_.Bind(&block289, &phi_bb289_7, &phi_bb289_8, &phi_bb289_9, &phi_bb289_10, &phi_bb289_11, &phi_bb289_14, &phi_bb289_15, &phi_bb289_21);
    tmp614 = FromConstexpr_int32_constexpr_int32_0(state_, wasm::ValueKind::kRefNull);
    tmp615 = CodeStubAssembler(state_).Word32Equal(TNode<Int32T>{tmp610}, TNode<Int32T>{tmp614});
    ca_.Goto(&block290, phi_bb289_7, phi_bb289_8, phi_bb289_9, phi_bb289_10, phi_bb289_11, phi_bb289_14, phi_bb289_15, phi_bb289_21, tmp615);
  }

  TNode<IntPtrT> phi_bb290_7;
  TNode<IntPtrT> phi_bb290_8;
  TNode<IntPtrT> phi_bb290_9;
  TNode<IntPtrT> phi_bb290_10;
  TNode<IntPtrT> phi_bb290_11;
  TNode<IntPtrT> phi_bb290_14;
  TNode<BoolT> phi_bb290_15;
  TNode<IntPtrT> phi_bb290_21;
  TNode<BoolT> phi_bb290_25;
  if (block290.is_used()) {
    ca_.Bind(&block290, &phi_bb290_7, &phi_bb290_8, &phi_bb290_9, &phi_bb290_10, &phi_bb290_11, &phi_bb290_14, &phi_bb290_15, &phi_bb290_21, &phi_bb290_25);
    ca_.Branch(phi_bb290_25, &block286, std::vector<compiler::Node*>{phi_bb290_7, phi_bb290_8, phi_bb290_9, phi_bb290_10, phi_bb290_11, phi_bb290_14, phi_bb290_15, phi_bb290_21}, &block287, std::vector<compiler::Node*>{phi_bb290_7, phi_bb290_8, phi_bb290_9, phi_bb290_10, phi_bb290_11, phi_bb290_14, phi_bb290_15, phi_bb290_21});
  }

  TNode<IntPtrT> phi_bb287_7;
  TNode<IntPtrT> phi_bb287_8;
  TNode<IntPtrT> phi_bb287_9;
  TNode<IntPtrT> phi_bb287_10;
  TNode<IntPtrT> phi_bb287_11;
  TNode<IntPtrT> phi_bb287_14;
  TNode<BoolT> phi_bb287_15;
  TNode<IntPtrT> phi_bb287_21;
  if (block287.is_used()) {
    ca_.Bind(&block287, &phi_bb287_7, &phi_bb287_8, &phi_bb287_9, &phi_bb287_10, &phi_bb287_11, &phi_bb287_14, &phi_bb287_15, &phi_bb287_21);
    {
      auto pos_stack = ca_.GetMacroSourcePositionStack();
      pos_stack.push_back({"src/builtins/js-to-wasm.tq", 1071});
      CodeStubAssembler(state_).FailAssert("Torque assert 'retKind == ValueKind::kRef || retKind == ValueKind::kRefNull' failed", pos_stack);
    }
  }

  TNode<IntPtrT> phi_bb286_7;
  TNode<IntPtrT> phi_bb286_8;
  TNode<IntPtrT> phi_bb286_9;
  TNode<IntPtrT> phi_bb286_10;
  TNode<IntPtrT> phi_bb286_11;
  TNode<IntPtrT> phi_bb286_14;
  TNode<BoolT> phi_bb286_15;
  TNode<IntPtrT> phi_bb286_21;
  TNode<Object> tmp616;
  TNode<IntPtrT> tmp617;
  TNode<IntPtrT> tmp618;
  TNode<UintPtrT> tmp619;
  TNode<UintPtrT> tmp620;
  TNode<BoolT> tmp621;
  if (block286.is_used()) {
    ca_.Bind(&block286, &phi_bb286_7, &phi_bb286_8, &phi_bb286_9, &phi_bb286_10, &phi_bb286_11, &phi_bb286_14, &phi_bb286_15, &phi_bb286_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1072);
    std::tie(tmp616, tmp617, tmp618) = FieldSliceFixedArrayObjects_0(state_, TNode<FixedArray>{tmp92}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 101);
    tmp619 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{phi_bb286_21});
    tmp620 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{tmp618});
    tmp621 = CodeStubAssembler(state_).UintPtrLessThan(TNode<UintPtrT>{tmp619}, TNode<UintPtrT>{tmp620});
    ca_.Branch(tmp621, &block295, std::vector<compiler::Node*>{phi_bb286_7, phi_bb286_8, phi_bb286_9, phi_bb286_10, phi_bb286_11, phi_bb286_14, phi_bb286_15, phi_bb286_21, phi_bb286_21, phi_bb286_21, phi_bb286_21, phi_bb286_21}, &block296, std::vector<compiler::Node*>{phi_bb286_7, phi_bb286_8, phi_bb286_9, phi_bb286_10, phi_bb286_11, phi_bb286_14, phi_bb286_15, phi_bb286_21, phi_bb286_21, phi_bb286_21, phi_bb286_21, phi_bb286_21});
  }

  TNode<IntPtrT> phi_bb295_7;
  TNode<IntPtrT> phi_bb295_8;
  TNode<IntPtrT> phi_bb295_9;
  TNode<IntPtrT> phi_bb295_10;
  TNode<IntPtrT> phi_bb295_11;
  TNode<IntPtrT> phi_bb295_14;
  TNode<BoolT> phi_bb295_15;
  TNode<IntPtrT> phi_bb295_21;
  TNode<IntPtrT> phi_bb295_28;
  TNode<IntPtrT> phi_bb295_29;
  TNode<IntPtrT> phi_bb295_33;
  TNode<IntPtrT> phi_bb295_34;
  TNode<IntPtrT> tmp622;
  TNode<IntPtrT> tmp623;
  TNode<Object> tmp624;
  TNode<IntPtrT> tmp625;
  TNode<Object> tmp626;
  TNode<Object> tmp627;
  TNode<IntPtrT> tmp628;
  TNode<IntPtrT> tmp629;
  TNode<UintPtrT> tmp630;
  TNode<UintPtrT> tmp631;
  TNode<BoolT> tmp632;
  if (block295.is_used()) {
    ca_.Bind(&block295, &phi_bb295_7, &phi_bb295_8, &phi_bb295_9, &phi_bb295_10, &phi_bb295_11, &phi_bb295_14, &phi_bb295_15, &phi_bb295_21, &phi_bb295_28, &phi_bb295_29, &phi_bb295_33, &phi_bb295_34);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp622 = TimesSizeOf_Object_0(state_, TNode<IntPtrT>{phi_bb295_34});
    tmp623 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp617}, TNode<IntPtrT>{tmp622});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp624, tmp625) = NewReference_Object_0(state_, TNode<Object>{tmp616}, TNode<IntPtrT>{tmp623}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1072);
    tmp626 = CodeStubAssembler(state_).LoadReference<Object>(CodeStubAssembler::Reference{tmp624, tmp625});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1073);
    std::tie(tmp627, tmp628, tmp629) = FieldSliceFixedArrayObjects_0(state_, TNode<FixedArray>{tmp92}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 101);
    tmp630 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{phi_bb295_21});
    tmp631 = Convert_uintptr_intptr_0(state_, TNode<IntPtrT>{tmp629});
    tmp632 = CodeStubAssembler(state_).UintPtrLessThan(TNode<UintPtrT>{tmp630}, TNode<UintPtrT>{tmp631});
    ca_.Branch(tmp632, &block303, std::vector<compiler::Node*>{phi_bb295_7, phi_bb295_8, phi_bb295_9, phi_bb295_10, phi_bb295_11, phi_bb295_14, phi_bb295_15, phi_bb295_21, phi_bb295_21, phi_bb295_21, phi_bb295_21, phi_bb295_21}, &block304, std::vector<compiler::Node*>{phi_bb295_7, phi_bb295_8, phi_bb295_9, phi_bb295_10, phi_bb295_11, phi_bb295_14, phi_bb295_15, phi_bb295_21, phi_bb295_21, phi_bb295_21, phi_bb295_21, phi_bb295_21});
  }

  TNode<IntPtrT> phi_bb296_7;
  TNode<IntPtrT> phi_bb296_8;
  TNode<IntPtrT> phi_bb296_9;
  TNode<IntPtrT> phi_bb296_10;
  TNode<IntPtrT> phi_bb296_11;
  TNode<IntPtrT> phi_bb296_14;
  TNode<BoolT> phi_bb296_15;
  TNode<IntPtrT> phi_bb296_21;
  TNode<IntPtrT> phi_bb296_28;
  TNode<IntPtrT> phi_bb296_29;
  TNode<IntPtrT> phi_bb296_33;
  TNode<IntPtrT> phi_bb296_34;
  if (block296.is_used()) {
    ca_.Bind(&block296, &phi_bb296_7, &phi_bb296_8, &phi_bb296_9, &phi_bb296_10, &phi_bb296_11, &phi_bb296_14, &phi_bb296_15, &phi_bb296_21, &phi_bb296_28, &phi_bb296_29, &phi_bb296_33, &phi_bb296_34);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 113);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<IntPtrT> phi_bb303_7;
  TNode<IntPtrT> phi_bb303_8;
  TNode<IntPtrT> phi_bb303_9;
  TNode<IntPtrT> phi_bb303_10;
  TNode<IntPtrT> phi_bb303_11;
  TNode<IntPtrT> phi_bb303_14;
  TNode<BoolT> phi_bb303_15;
  TNode<IntPtrT> phi_bb303_21;
  TNode<IntPtrT> phi_bb303_29;
  TNode<IntPtrT> phi_bb303_30;
  TNode<IntPtrT> phi_bb303_34;
  TNode<IntPtrT> phi_bb303_35;
  TNode<IntPtrT> tmp633;
  TNode<IntPtrT> tmp634;
  TNode<Object> tmp635;
  TNode<IntPtrT> tmp636;
  TNode<Object> tmp637;
  if (block303.is_used()) {
    ca_.Bind(&block303, &phi_bb303_7, &phi_bb303_8, &phi_bb303_9, &phi_bb303_10, &phi_bb303_11, &phi_bb303_14, &phi_bb303_15, &phi_bb303_21, &phi_bb303_29, &phi_bb303_30, &phi_bb303_34, &phi_bb303_35);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 109);
    tmp633 = TimesSizeOf_Object_0(state_, TNode<IntPtrT>{phi_bb303_35});
    tmp634 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{tmp628}, TNode<IntPtrT>{tmp633});
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 108);
    std::tie(tmp635, tmp636) = NewReference_Object_0(state_, TNode<Object>{tmp627}, TNode<IntPtrT>{tmp634}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1073);
    tmp637 = WasmToJSObject_0(state_, TNode<NativeContext>{parameter0}, TNode<Object>{tmp626}, TNode<Int32T>{tmp353});
    CodeStubAssembler(state_).StoreReference<Object>(CodeStubAssembler::Reference{tmp635, tmp636}, tmp637);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1065);
    ca_.Goto(&block265, phi_bb303_7, phi_bb303_8, phi_bb303_9, phi_bb303_10, phi_bb303_11, phi_bb303_14, phi_bb303_15, phi_bb303_21);
  }

  TNode<IntPtrT> phi_bb304_7;
  TNode<IntPtrT> phi_bb304_8;
  TNode<IntPtrT> phi_bb304_9;
  TNode<IntPtrT> phi_bb304_10;
  TNode<IntPtrT> phi_bb304_11;
  TNode<IntPtrT> phi_bb304_14;
  TNode<BoolT> phi_bb304_15;
  TNode<IntPtrT> phi_bb304_21;
  TNode<IntPtrT> phi_bb304_29;
  TNode<IntPtrT> phi_bb304_30;
  TNode<IntPtrT> phi_bb304_34;
  TNode<IntPtrT> phi_bb304_35;
  if (block304.is_used()) {
    ca_.Bind(&block304, &phi_bb304_7, &phi_bb304_8, &phi_bb304_9, &phi_bb304_10, &phi_bb304_11, &phi_bb304_14, &phi_bb304_15, &phi_bb304_21, &phi_bb304_29, &phi_bb304_30, &phi_bb304_34, &phi_bb304_35);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 113);
    CodeStubAssembler(state_).Unreachable();
  }

  TNode<IntPtrT> phi_bb265_7;
  TNode<IntPtrT> phi_bb265_8;
  TNode<IntPtrT> phi_bb265_9;
  TNode<IntPtrT> phi_bb265_10;
  TNode<IntPtrT> phi_bb265_11;
  TNode<IntPtrT> phi_bb265_14;
  TNode<BoolT> phi_bb265_15;
  TNode<IntPtrT> phi_bb265_21;
  if (block265.is_used()) {
    ca_.Bind(&block265, &phi_bb265_7, &phi_bb265_8, &phi_bb265_9, &phi_bb265_10, &phi_bb265_11, &phi_bb265_14, &phi_bb265_15, &phi_bb265_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1053);
    ca_.Goto(&block216, phi_bb265_7, phi_bb265_8, phi_bb265_9, phi_bb265_10, phi_bb265_11, phi_bb265_14, phi_bb265_15, phi_bb265_21);
  }

  TNode<IntPtrT> phi_bb216_7;
  TNode<IntPtrT> phi_bb216_8;
  TNode<IntPtrT> phi_bb216_9;
  TNode<IntPtrT> phi_bb216_10;
  TNode<IntPtrT> phi_bb216_11;
  TNode<IntPtrT> phi_bb216_14;
  TNode<BoolT> phi_bb216_15;
  TNode<IntPtrT> phi_bb216_21;
  if (block216.is_used()) {
    ca_.Bind(&block216, &phi_bb216_7, &phi_bb216_8, &phi_bb216_9, &phi_bb216_10, &phi_bb216_11, &phi_bb216_14, &phi_bb216_15, &phi_bb216_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1033);
    ca_.Goto(&block182, phi_bb216_7, phi_bb216_8, phi_bb216_9, phi_bb216_10, phi_bb216_11, phi_bb216_14, phi_bb216_15, phi_bb216_21);
  }

  TNode<IntPtrT> phi_bb182_7;
  TNode<IntPtrT> phi_bb182_8;
  TNode<IntPtrT> phi_bb182_9;
  TNode<IntPtrT> phi_bb182_10;
  TNode<IntPtrT> phi_bb182_11;
  TNode<IntPtrT> phi_bb182_14;
  TNode<BoolT> phi_bb182_15;
  TNode<IntPtrT> phi_bb182_21;
  if (block182.is_used()) {
    ca_.Bind(&block182, &phi_bb182_7, &phi_bb182_8, &phi_bb182_9, &phi_bb182_10, &phi_bb182_11, &phi_bb182_14, &phi_bb182_15, &phi_bb182_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1024);
    ca_.Goto(&block159, phi_bb182_7, phi_bb182_8, phi_bb182_9, phi_bb182_10, phi_bb182_11, phi_bb182_14, phi_bb182_15, phi_bb182_21);
  }

  TNode<IntPtrT> phi_bb159_7;
  TNode<IntPtrT> phi_bb159_8;
  TNode<IntPtrT> phi_bb159_9;
  TNode<IntPtrT> phi_bb159_10;
  TNode<IntPtrT> phi_bb159_11;
  TNode<IntPtrT> phi_bb159_14;
  TNode<BoolT> phi_bb159_15;
  TNode<IntPtrT> phi_bb159_21;
  TNode<IntPtrT> tmp638;
  TNode<IntPtrT> tmp639;
  if (block159.is_used()) {
    ca_.Bind(&block159, &phi_bb159_7, &phi_bb159_8, &phi_bb159_9, &phi_bb159_10, &phi_bb159_11, &phi_bb159_14, &phi_bb159_15, &phi_bb159_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1022);
    tmp638 = FromConstexpr_intptr_constexpr_int31_0(state_, 1);
    tmp639 = CodeStubAssembler(state_).IntPtrAdd(TNode<IntPtrT>{phi_bb159_21}, TNode<IntPtrT>{tmp638});
    ca_.Goto(&block154, phi_bb159_7, phi_bb159_8, phi_bb159_9, phi_bb159_10, phi_bb159_11, phi_bb159_14, phi_bb159_15, tmp639);
  }

  TNode<IntPtrT> phi_bb153_7;
  TNode<IntPtrT> phi_bb153_8;
  TNode<IntPtrT> phi_bb153_9;
  TNode<IntPtrT> phi_bb153_10;
  TNode<IntPtrT> phi_bb153_11;
  TNode<IntPtrT> phi_bb153_14;
  TNode<BoolT> phi_bb153_15;
  TNode<IntPtrT> phi_bb153_21;
  TNode<IntPtrT> tmp640;
  TNode<BoolT> tmp641;
  if (block153.is_used()) {
    ca_.Bind(&block153, &phi_bb153_7, &phi_bb153_8, &phi_bb153_9, &phi_bb153_10, &phi_bb153_11, &phi_bb153_14, &phi_bb153_15, &phi_bb153_21);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 380);
    tmp640 = FromConstexpr_intptr_constexpr_IntegerLiteral_0(state_, IntegerLiteral(false, 0x0ull));
    tmp641 = CodeStubAssembler(state_).WordEqual(TNode<IntPtrT>{tmp343}, TNode<IntPtrT>{tmp640});
    ca_.Branch(tmp641, &block310, std::vector<compiler::Node*>{phi_bb153_7, phi_bb153_8, phi_bb153_9, phi_bb153_10, phi_bb153_11, phi_bb153_14, phi_bb153_15}, &block311, std::vector<compiler::Node*>{phi_bb153_7, phi_bb153_8, phi_bb153_9, phi_bb153_10, phi_bb153_11, phi_bb153_14, phi_bb153_15});
  }

  TNode<IntPtrT> phi_bb310_7;
  TNode<IntPtrT> phi_bb310_8;
  TNode<IntPtrT> phi_bb310_9;
  TNode<IntPtrT> phi_bb310_10;
  TNode<IntPtrT> phi_bb310_11;
  TNode<IntPtrT> phi_bb310_14;
  TNode<BoolT> phi_bb310_15;
  TNode<BoolT> tmp642;
  if (block310.is_used()) {
    ca_.Bind(&block310, &phi_bb310_7, &phi_bb310_8, &phi_bb310_9, &phi_bb310_10, &phi_bb310_11, &phi_bb310_14, &phi_bb310_15);
    tmp642 = FromConstexpr_bool_constexpr_bool_0(state_, true);
    ca_.Goto(&block312, phi_bb310_7, phi_bb310_8, phi_bb310_9, phi_bb310_10, phi_bb310_11, phi_bb310_14, phi_bb310_15, tmp642);
  }

  TNode<IntPtrT> phi_bb311_7;
  TNode<IntPtrT> phi_bb311_8;
  TNode<IntPtrT> phi_bb311_9;
  TNode<IntPtrT> phi_bb311_10;
  TNode<IntPtrT> phi_bb311_11;
  TNode<IntPtrT> phi_bb311_14;
  TNode<BoolT> phi_bb311_15;
  TNode<BoolT> tmp643;
  if (block311.is_used()) {
    ca_.Bind(&block311, &phi_bb311_7, &phi_bb311_8, &phi_bb311_9, &phi_bb311_10, &phi_bb311_11, &phi_bb311_14, &phi_bb311_15);
    tmp643 = CodeStubAssembler(state_).IntPtrLessThanOrEqual(TNode<IntPtrT>{phi_bb311_11}, TNode<IntPtrT>{tmp343});
    ca_.Goto(&block312, phi_bb311_7, phi_bb311_8, phi_bb311_9, phi_bb311_10, phi_bb311_11, phi_bb311_14, phi_bb311_15, tmp643);
  }

  TNode<IntPtrT> phi_bb312_7;
  TNode<IntPtrT> phi_bb312_8;
  TNode<IntPtrT> phi_bb312_9;
  TNode<IntPtrT> phi_bb312_10;
  TNode<IntPtrT> phi_bb312_11;
  TNode<IntPtrT> phi_bb312_14;
  TNode<BoolT> phi_bb312_15;
  TNode<BoolT> phi_bb312_22;
  if (block312.is_used()) {
    ca_.Bind(&block312, &phi_bb312_7, &phi_bb312_8, &phi_bb312_9, &phi_bb312_10, &phi_bb312_11, &phi_bb312_14, &phi_bb312_15, &phi_bb312_22);
    ca_.Branch(phi_bb312_22, &block308, std::vector<compiler::Node*>{phi_bb312_7, phi_bb312_8, phi_bb312_9, phi_bb312_10, phi_bb312_11, phi_bb312_14, phi_bb312_15}, &block309, std::vector<compiler::Node*>{phi_bb312_7, phi_bb312_8, phi_bb312_9, phi_bb312_10, phi_bb312_11, phi_bb312_14, phi_bb312_15});
  }

  TNode<IntPtrT> phi_bb309_7;
  TNode<IntPtrT> phi_bb309_8;
  TNode<IntPtrT> phi_bb309_9;
  TNode<IntPtrT> phi_bb309_10;
  TNode<IntPtrT> phi_bb309_11;
  TNode<IntPtrT> phi_bb309_14;
  TNode<BoolT> phi_bb309_15;
  if (block309.is_used()) {
    ca_.Bind(&block309, &phi_bb309_7, &phi_bb309_8, &phi_bb309_9, &phi_bb309_10, &phi_bb309_11, &phi_bb309_14, &phi_bb309_15);
    {
      auto pos_stack = ca_.GetMacroSourcePositionStack();
      pos_stack.push_back({"src/builtins/js-to-wasm.tq", 380});
      CodeStubAssembler(state_).FailAssert("Torque assert 'this.paramBufferEnd == 0 || this.nextStack <= this.paramBufferEnd' failed", pos_stack);
    }
  }

  TNode<IntPtrT> phi_bb308_7;
  TNode<IntPtrT> phi_bb308_8;
  TNode<IntPtrT> phi_bb308_9;
  TNode<IntPtrT> phi_bb308_10;
  TNode<IntPtrT> phi_bb308_11;
  TNode<IntPtrT> phi_bb308_14;
  TNode<BoolT> phi_bb308_15;
  if (block308.is_used()) {
    ca_.Bind(&block308, &phi_bb308_7, &phi_bb308_8, &phi_bb308_9, &phi_bb308_10, &phi_bb308_11, &phi_bb308_14, &phi_bb308_15);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1078);
    CodeStubAssembler(state_).Return(parameter1);
  }
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=421&c=32
int31_t SizeOf_float64_0(compiler::CodeAssemblerState* state_) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 421);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return kDoubleSize;
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=490&c=8
TorqueStructReference_float64_0 RefCast_float64_0(compiler::CodeAssemblerState* state_, TorqueStructReference_intptr_0 p_i) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Object> tmp0;
  TNode<IntPtrT> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 156);
    std::tie(tmp0, tmp1) = NewReference_float64_0(state_, TNode<Object>{p_i.object}, TNode<IntPtrT>{p_i.offset}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 490);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_float64_0{TNode<Object>{tmp0}, TNode<IntPtrT>{tmp1}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=493&c=8
TorqueStructReference_float32_0 RefCast_float32_0(compiler::CodeAssemblerState* state_, TorqueStructReference_intptr_0 p_i) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Object> tmp0;
  TNode<IntPtrT> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 156);
    std::tie(tmp0, tmp1) = NewReference_float32_0(state_, TNode<Object>{p_i.object}, TNode<IntPtrT>{p_i.offset}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 493);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_float32_0{TNode<Object>{tmp0}, TNode<IntPtrT>{tmp1}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=531&c=30
TorqueStructReference_intptr_0 GetRefAt_intptr_RawFunctionSigPtr_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_base, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<RawPtrT> tmp0;
  TNode<RawPtrT> tmp1;
  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 875);
    tmp0 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{p_base}, TNode<IntPtrT>{p_offset});
    tmp1 = (TNode<RawPtrT>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 874);
    std::tie(tmp2, tmp3) = NewOffHeapReference_intptr_0(state_, TNode<RawPtrT>{tmp1}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 531);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_intptr_0{TNode<Object>{tmp2}, TNode<IntPtrT>{tmp3}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=536&c=17
TorqueStructReference_RawPtr_0 GetRefAt_RawPtr_RawFunctionSigPtr_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_base, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<RawPtrT> tmp0;
  TNode<RawPtrT> tmp1;
  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 875);
    tmp0 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{p_base}, TNode<IntPtrT>{p_offset});
    tmp1 = (TNode<RawPtrT>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 874);
    std::tie(tmp2, tmp3) = NewOffHeapReference_RawPtr_0(state_, TNode<RawPtrT>{tmp1}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 536);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_RawPtr_0{TNode<Object>{tmp2}, TNode<IntPtrT>{tmp3}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=309&c=22
TorqueStructReference_intptr_0 NewReference_intptr_0(compiler::CodeAssemblerState* state_, TNode<Object> p_object, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Object> tmp0;
  TNode<IntPtrT> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 78);
    std::tie(tmp0, tmp1) = (TorqueStructReference_intptr_0{TNode<Object>{p_object}, TNode<IntPtrT>{p_offset}, TorqueStructUnsafe_0{}}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 309);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_intptr_0{TNode<Object>{tmp0}, TNode<IntPtrT>{tmp1}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=741&c=9
TNode<WasmSuspenderObject> UnsafeCast_WasmSuspenderObject_0(compiler::CodeAssemblerState* state_, TNode<Context> p_context, TNode<Object> p_o) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block6(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<WasmSuspenderObject> tmp0;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/cast.tq", 866);
    tmp0 = TORQUE_CAST(TNode<Object>{p_o});
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 741);
    ca_.Goto(&block6);
  }

    ca_.Bind(&block6);
  return TNode<WasmSuspenderObject>{tmp0};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=783&c=6
TorqueStructReference_int32_0 GetRefAt_int32_RawPtr_intptr_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_base, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<RawPtrT> tmp0;
  TNode<RawPtrT> tmp1;
  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 875);
    tmp0 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{p_base}, TNode<IntPtrT>{p_offset});
    tmp1 = (TNode<RawPtrT>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 874);
    std::tie(tmp2, tmp3) = NewOffHeapReference_int32_0(state_, TNode<RawPtrT>{tmp1}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 783);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_int32_0{TNode<Object>{tmp2}, TNode<IntPtrT>{tmp3}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=784&c=6
TorqueStructReference_bool_0 GetRefAt_bool_RawPtr_intptr_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_base, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<RawPtrT> tmp0;
  TNode<RawPtrT> tmp1;
  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 875);
    tmp0 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{p_base}, TNode<IntPtrT>{p_offset});
    tmp1 = (TNode<RawPtrT>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 874);
    std::tie(tmp2, tmp3) = NewOffHeapReference_bool_0(state_, TNode<RawPtrT>{tmp1}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 784);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_bool_0{TNode<Object>{tmp2}, TNode<IntPtrT>{tmp3}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=786&c=6
TorqueStructReference_RawPtr_0 GetRefAt_RawPtr_RawPtr_intptr_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_base, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<RawPtrT> tmp0;
  TNode<RawPtrT> tmp1;
  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 875);
    tmp0 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{p_base}, TNode<IntPtrT>{p_offset});
    tmp1 = (TNode<RawPtrT>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 874);
    std::tie(tmp2, tmp3) = NewOffHeapReference_RawPtr_0(state_, TNode<RawPtrT>{tmp1}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 786);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_RawPtr_0{TNode<Object>{tmp2}, TNode<IntPtrT>{tmp3}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=788&c=6
TorqueStructReference_intptr_0 GetRefAt_intptr_RawPtr_intptr_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_base, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<RawPtrT> tmp0;
  TNode<RawPtrT> tmp1;
  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 875);
    tmp0 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{p_base}, TNode<IntPtrT>{p_offset});
    tmp1 = (TNode<RawPtrT>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 874);
    std::tie(tmp2, tmp3) = NewOffHeapReference_intptr_0(state_, TNode<RawPtrT>{tmp1}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 788);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_intptr_0{TNode<Object>{tmp2}, TNode<IntPtrT>{tmp3}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=790&c=6
TorqueStructReference_WasmCodePointer_0 GetRefAt_WasmCodePointer_RawPtr_intptr_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_base, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<RawPtrT> tmp0;
  TNode<RawPtrT> tmp1;
  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 875);
    tmp0 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{p_base}, TNode<IntPtrT>{p_offset});
    tmp1 = (TNode<RawPtrT>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 874);
    std::tie(tmp2, tmp3) = NewOffHeapReference_WasmCodePointer_0(state_, TNode<RawPtrT>{tmp1}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 790);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_WasmCodePointer_0{TNode<Object>{tmp2}, TNode<IntPtrT>{tmp3}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=792&c=6
TorqueStructReference_RawPtr_intptr_0 GetRefAt_RawPtr_intptr_RawPtr_intptr_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_base, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<RawPtrT> tmp0;
  TNode<RawPtrT> tmp1;
  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 875);
    tmp0 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{p_base}, TNode<IntPtrT>{p_offset});
    tmp1 = (TNode<RawPtrT>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 874);
    std::tie(tmp2, tmp3) = NewOffHeapReference_RawPtr_intptr_0(state_, TNode<RawPtrT>{tmp1}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 792);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_RawPtr_intptr_0{TNode<Object>{tmp2}, TNode<IntPtrT>{tmp3}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=909&c=22
TorqueStructReference_int32_0 GetRefAt_int32_RawPtr_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_base, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<RawPtrT> tmp0;
  TNode<RawPtrT> tmp1;
  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 875);
    tmp0 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{p_base}, TNode<IntPtrT>{p_offset});
    tmp1 = (TNode<RawPtrT>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 874);
    std::tie(tmp2, tmp3) = NewOffHeapReference_int32_0(state_, TNode<RawPtrT>{tmp1}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 909);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_int32_0{TNode<Object>{tmp2}, TNode<IntPtrT>{tmp3}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=913&c=37
TorqueStructReference_int64_0 GetRefAt_int64_RawPtr_intptr_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_base, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<RawPtrT> tmp0;
  TNode<RawPtrT> tmp1;
  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 875);
    tmp0 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{p_base}, TNode<IntPtrT>{p_offset});
    tmp1 = (TNode<RawPtrT>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 874);
    std::tie(tmp2, tmp3) = NewOffHeapReference_int64_0(state_, TNode<RawPtrT>{tmp1}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 913);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_int64_0{TNode<Object>{tmp2}, TNode<IntPtrT>{tmp3}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=922&c=58
TorqueStructReference_float64_0 GetRefAt_float64_RawPtr_intptr_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_base, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<RawPtrT> tmp0;
  TNode<RawPtrT> tmp1;
  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 875);
    tmp0 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{p_base}, TNode<IntPtrT>{p_offset});
    tmp1 = (TNode<RawPtrT>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 874);
    std::tie(tmp2, tmp3) = NewOffHeapReference_float64_0(state_, TNode<RawPtrT>{tmp1}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 922);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_float64_0{TNode<Object>{tmp2}, TNode<IntPtrT>{tmp3}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=932&c=13
TorqueStructReference_float32_0 GetRefAt_float32_RawPtr_intptr_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_base, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<RawPtrT> tmp0;
  TNode<RawPtrT> tmp1;
  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 875);
    tmp0 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{p_base}, TNode<IntPtrT>{p_offset});
    tmp1 = (TNode<RawPtrT>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 874);
    std::tie(tmp2, tmp3) = NewOffHeapReference_float32_0(state_, TNode<RawPtrT>{tmp1}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 932);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_float32_0{TNode<Object>{tmp2}, TNode<IntPtrT>{tmp3}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=956&c=23
TorqueStructReference_uintptr_0 GetRefAt_uintptr_RawPtr_uintptr_0(compiler::CodeAssemblerState* state_, TNode<RawPtrT> p_base, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<RawPtrT> tmp0;
  TNode<RawPtrT> tmp1;
  TNode<Object> tmp2;
  TNode<IntPtrT> tmp3;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 875);
    tmp0 = CodeStubAssembler(state_).RawPtrAdd(TNode<RawPtrT>{p_base}, TNode<IntPtrT>{p_offset});
    tmp1 = (TNode<RawPtrT>{tmp0});
    ca_.SetSourcePosition("../../v8/src/builtins/wasm.tq", 874);
    std::tie(tmp2, tmp3) = NewOffHeapReference_uintptr_0(state_, TNode<RawPtrT>{tmp1}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 956);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_uintptr_0{TNode<Object>{tmp2}, TNode<IntPtrT>{tmp3}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=1028&c=37
TorqueStructReference_int64_0 RefCast_int64_0(compiler::CodeAssemblerState* state_, TorqueStructReference_intptr_0 p_i) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Object> tmp0;
  TNode<IntPtrT> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 156);
    std::tie(tmp0, tmp1) = NewReference_int64_0(state_, TNode<Object>{p_i.object}, TNode<IntPtrT>{p_i.offset}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1028);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_int64_0{TNode<Object>{tmp0}, TNode<IntPtrT>{tmp1}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=1030&c=16
TorqueStructReference_int32_0 RefCast_int32_0(compiler::CodeAssemblerState* state_, TorqueStructReference_intptr_0 p_i) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Object> tmp0;
  TNode<IntPtrT> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 156);
    std::tie(tmp0, tmp1) = NewReference_int32_0(state_, TNode<Object>{p_i.object}, TNode<IntPtrT>{p_i.offset}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 1030);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_int32_0{TNode<Object>{tmp0}, TNode<IntPtrT>{tmp1}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=156&c=10
TorqueStructReference_float32_0 NewReference_float32_0(compiler::CodeAssemblerState* state_, TNode<Object> p_object, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Object> tmp0;
  TNode<IntPtrT> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 78);
    std::tie(tmp0, tmp1) = (TorqueStructReference_float32_0{TNode<Object>{p_object}, TNode<IntPtrT>{p_offset}, TorqueStructUnsafe_0{}}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 156);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_float32_0{TNode<Object>{tmp0}, TNode<IntPtrT>{tmp1}, TorqueStructUnsafe_0{}};
}

// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/builtins/js-to-wasm.tq?l=156&c=10
TorqueStructReference_int64_0 NewReference_int64_0(compiler::CodeAssemblerState* state_, TNode<Object> p_object, TNode<IntPtrT> p_offset) {
  compiler::CodeAssembler ca_(state_);
  compiler::CodeAssembler::SourcePositionScope pos_scope(&ca_);
  compiler::CodeAssemblerParameterizedLabel<> block0(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
  compiler::CodeAssemblerParameterizedLabel<> block2(&ca_, compiler::CodeAssemblerLabel::kNonDeferred);
    ca_.Goto(&block0);

  TNode<Object> tmp0;
  TNode<IntPtrT> tmp1;
  if (block0.is_used()) {
    ca_.Bind(&block0);
    ca_.SetSourcePosition("../../v8/src/builtins/torque-internal.tq", 78);
    std::tie(tmp0, tmp1) = (TorqueStructReference_int64_0{TNode<Object>{p_object}, TNode<IntPtrT>{p_offset}, TorqueStructUnsafe_0{}}).Flatten();
    ca_.SetSourcePosition("../../v8/src/builtins/js-to-wasm.tq", 156);
    ca_.Goto(&block2);
  }

    ca_.Bind(&block2);
  return TorqueStructReference_int64_0{TNode<Object>{tmp0}, TNode<IntPtrT>{tmp1}, TorqueStructUnsafe_0{}};
}

} // namespace internal
} // namespace v8
